@@ -1,20 +1,15 @@
 """High performance data structures
-
-Note that PyPy also contains a built-in module '_collections' which will hide
-this one if compiled in.
-
-THIS ONE IS BOGUS in the sense that it is NOT THREAD-SAFE!  It is provided
-only as documentation nowadays.  Please don't run in production a PyPy
-without the '_collections' built-in module.  The built-in module is
-correctly thread-safe, like it is on CPython.
 """
 #
 # Copied and completed from the sandbox of CPython
 #   (nondist/sandbox/collections/pydeque.py rev 1.1, Raymond Hettinger)
 #
 
+# Note that PyPy also contains a built-in module '_collections' which will hide
+# this one if compiled in.
+
 try:
-    from _thread import _get_ident as _thread_ident
+    from threading import _get_ident as _thread_ident
 except ImportError:
     def _thread_ident():
         return -1
@@ -380,7 +375,7 @@ class deque_iterator(object):
             raise RuntimeError("deque mutated during iteration")
         self._gen = itergen(deq.state, giveup)
 
-    def __next__(self):
+    def next(self):
         res = next(self._gen)
         self.counter -= 1
         return res
@@ -389,8 +384,7 @@ class deque_iterator(object):
         return self
 
 class defaultdict(dict):
-    __slots__ = ["default_factory"]
-
+    
     def __init__(self, *args, **kwds):
         if len(args) > 0:
             default_factory = args[0]
@@ -401,10 +395,10 @@ class defaultdict(dict):
             default_factory = None
         self.default_factory = default_factory
         super(defaultdict, self).__init__(*args, **kwds)
-
+ 
     def __missing__(self, key):
         # from defaultdict docs
-        if self.default_factory is None:
+        if self.default_factory is None: 
             raise KeyError(key)
         self[key] = value = self.default_factory()
         return value
@@ -420,7 +414,7 @@ class defaultdict(dict):
 
     def copy(self):
         return type(self)(self.default_factory, self)
-
+    
     def __copy__(self):
         return self.copy()
 
@@ -436,5 +430,5 @@ class defaultdict(dict):
 
            This API is used by pickle.py and copy.py.
         """
-        return (type(self), (self.default_factory,), None, None,
-                iter(self.items()))
+        return (type(self), (self.default_factory,), None, None, self.iteritems())
+
