@@ -1,24 +1,21 @@
-from _ctypes.basics import (
-    _CData, _CDataMeta, cdata_from_address, ArgumentError, keepalive_key,
-    is_struct_shape, sizeof)
+
+from _ctypes.basics import _CData, _CDataMeta, cdata_from_address
 from _ctypes.primitive import SimpleType, _SimpleCData
+from _ctypes.basics import ArgumentError, keepalive_key
+from _ctypes.basics import is_struct_shape
 from _ctypes.builtin import get_errno, set_errno, get_last_error, set_last_error
 import _rawffi
 from _rawffi import alt as _ffi
-from __pypy__ import newmemoryview
 import sys
 import traceback
 
-
-try:
-    from __pypy__ import builtinify
-except ImportError:
-    builtinify = lambda f: f
+try: from __pypy__ import builtinify
+except ImportError: builtinify = lambda f: f
 
 # XXX this file needs huge refactoring I fear
 
-PARAMFLAG_FIN = 0x1
-PARAMFLAG_FOUT = 0x2
+PARAMFLAG_FIN   = 0x1
+PARAMFLAG_FOUT  = 0x2
 PARAMFLAG_FLCID = 0x4
 PARAMFLAG_COMBINED = PARAMFLAG_FIN | PARAMFLAG_FOUT | PARAMFLAG_FLCID
 
@@ -27,11 +24,10 @@ VALID_PARAMFLAGS = (
     PARAMFLAG_FIN,
     PARAMFLAG_FIN | PARAMFLAG_FOUT,
     PARAMFLAG_FIN | PARAMFLAG_FLCID
-)
+    )
 
-WIN64 = sys.platform == 'win32' and sys.maxsize == 2**63 - 1
+WIN64 = sys.platform == 'win32' and sys.maxint == 2**63 - 1
 
-CTYPES_MAX_ARGCOUNT = 1024
 
 def get_com_error(errcode, riid, pIunk):
     "Win32 specific: build a COM Error exception"
@@ -39,7 +35,6 @@ def get_com_error(errcode, riid, pIunk):
     from _ctypes import COMError
     return COMError(errcode, None, None)
 
-
 @builtinify
 def call_function(func, args):
     "Only for debugging so far: So that we can call CFunction instances"
@@ -62,11 +57,9 @@ class CFuncPtrType(_CDataMeta):
 
     from_address = cdata_from_address
 
-    def _getformat(self):
-        return 'X{}'
 
-
-class CFuncPtr(_CData, metaclass=CFuncPtrType):
+class CFuncPtr(_CData):
+    __metaclass__ = CFuncPtrType
 
     _argtypes_ = None
     _restype_ = None
@@ -101,9 +94,14 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
                         "item %d in _argtypes_ has no from_param method" % (
                             i + 1,))
             self._argtypes_ = list(argtypes)
-
+            self._check_argtypes_for_fastpath()
     argtypes = property(_getargtypes, _setargtypes)
 
+    def _check_argtypes_for_fastpath(self):
+        if all([hasattr(argtype, '_ffiargshape_') for argtype in self._argtypes_]):
+            fastpath_cls = make_fastpath_subclass(self.__class__)
+            fastpath_cls.enable_fastpath_maybe(self)
+
     def _getparamflags(self):
         return self._paramflags
 
@@ -128,26 +126,27 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
                 raise TypeError(
                     "paramflags must be a sequence of (int [,string [,value]]) "
                     "tuples"
-                )
+                    )
             if not isinstance(flag, int):
                 raise TypeError(
                     "paramflags must be a sequence of (int [,string [,value]]) "
                     "tuples"
-                )
+                    )
             _flag = flag & PARAMFLAG_COMBINED
             if _flag == PARAMFLAG_FOUT:
                 typ = self._argtypes_[idx]
                 if getattr(typ, '_ffiargshape_', None) not in ('P', 'z', 'Z'):
                     raise TypeError(
                         "'out' parameter %d must be a pointer type, not %s"
-                        % (idx + 1, type(typ).__name__)
-                    )
+                        % (idx+1, type(typ).__name__)
+                        )
             elif _flag not in VALID_PARAMFLAGS:
                 raise TypeError("paramflag value %d not supported" % flag)
         self._paramflags = paramflags
 
     paramflags = property(_getparamflags, _setparamflags)
 
+
     def _getrestype(self):
         return self._restype_
 
@@ -157,7 +156,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             from ctypes import c_int
             restype = c_int
         if not (isinstance(restype, _CDataMeta) or restype is None or
-                    callable(restype)):
+                callable(restype)):
             raise TypeError("restype must be a type, a callable, or None")
         self._restype_ = restype
 
@@ -169,18 +168,15 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
 
     def _geterrcheck(self):
         return getattr(self, '_errcheck_', None)
-
     def _seterrcheck(self, errcheck):
         if not callable(errcheck):
             raise TypeError("The errcheck attribute must be callable")
         self._errcheck_ = errcheck
-
     def _delerrcheck(self):
         try:
             del self._errcheck_
         except AttributeError:
             pass
-
     errcheck = property(_geterrcheck, _seterrcheck, _delerrcheck)
 
     def _ffishapes(self, args, restype):
@@ -192,7 +188,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
                 raise TypeError("invalid result type for callback function")
             restype = restype._ffiargshape_
         else:
-            restype = 'O'  # void
+            restype = 'O' # void
         return argtypes, restype
 
     def _set_address(self, address):
@@ -205,7 +201,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
 
     def __init__(self, *args):
         self.name = None
-        self._objects = {keepalive_key(0): self}
+        self._objects = {keepalive_key(0):self}
         self._needs_free = True
 
         # Empty function object -- this is needed for casts
@@ -217,7 +213,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
         argument = argsl.pop(0)
 
         # Direct construction from raw address
-        if isinstance(argument, int) and not argsl:
+        if isinstance(argument, (int, long)) and not argsl:
             self._set_address(argument)
             restype = self._restype_
             if restype is None:
@@ -226,8 +222,10 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             if self._argtypes_ is None:
                 self._argtypes_ = []
             self._ptr = self._getfuncptr_fromaddress(self._argtypes_, restype)
+            self._check_argtypes_for_fastpath()
             return
 
+
         # A callback into python
         if callable(argument) and not argsl:
             self.callable = argument
@@ -258,10 +256,10 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             return
 
         # A COM function call, by index
-        if (sys.platform == 'win32' and isinstance(argument, int)
+        if (sys.platform == 'win32' and isinstance(argument, (int, long))
             and argsl):
             ffiargs, ffires = self._ffishapes(self._argtypes_, self._restype_)
-            self._com_index = argument + 0x1000
+            self._com_index =  argument + 0x1000
             self.name = argsl.pop(0)
             if argsl:
                 self.paramflags = argsl.pop(0)
@@ -272,7 +270,6 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             return
 
         raise TypeError("Unknown constructor %s" % (args,))
-    _init_no_arg_ = __init__
 
     def _wrap_callable(self, to_call, argtypes):
         def f(*args):
@@ -284,14 +281,11 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             except SystemExit as e:
                 handle_system_exit(e)
                 raise
-
         return f
 
     def __call__(self, *args, **kwargs):
         argtypes = self._argtypes_
         if self.callable is not None:
-            if len(args) > CTYPES_MAX_ARGCOUNT:
-                raise ArgumentError("too many arguments (%s), maximum is %s" % (len(args), CTYPES_MAX_ARGCOUNT))
             if len(args) == len(argtypes):
                 pass
             elif self._flags_ & _rawffi.FUNCFLAG_CDECL:
@@ -320,20 +314,13 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
                 except SystemExit as e:
                     handle_system_exit(e)
                     raise
-            except Exception as e:
-                from __pypy__ import write_unraisable
-                write_unraisable('in calling ctypes callback function', e, self.callable) 
+            except:
+                exc_info = sys.exc_info()
+                traceback.print_tb(exc_info[2], file=sys.stderr)
+                print >>sys.stderr, "%s: %s" % (exc_info[0].__name__, exc_info[1])
                 return 0
             if self._restype_ is not None:
-                if self._restype_._ffishape_ == 'O':
-                    return res
-                try:
-                    return self._restype_(res).value
-                except Exception as e:
-                    from __pypy__ import write_unraisable
-                    write_unraisable(
-                        "on converting result of ctypes callback function",
-                        e, self.callable)
+                return res
             return
 
         if argtypes is None:
@@ -341,7 +328,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             # really slow".  Now we don't worry that much about slowness
             # of ctypes, and it's strange to get warnings for perfectly-
             # legal code.
-            # warnings.warn('C function without declared arguments called',
+            #warnings.warn('C function without declared arguments called',
             #              RuntimeWarning, stacklevel=2)
             argtypes = []
 
@@ -350,7 +337,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             if not args:
                 raise ValueError(
                     "native COM method call without 'this' parameter"
-                )
+                    )
             thisvalue = args[0]
             thisarg = cast(thisvalue, POINTER(POINTER(c_void_p)))
             keepalives, newargs, argtypes, outargs, errcheckargs = (
@@ -379,6 +366,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
         return tuple(outargs)
 
     def _call_funcptr(self, funcptr, *newargs):
+
         if self._flags_ & _rawffi.FUNCFLAG_USE_ERRNO:
             tmp = _rawffi.get_errno()
             _rawffi.set_errno(get_errno())
@@ -407,8 +395,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
     def _do_errcheck(self, result, args):
         # The 'errcheck' protocol
         if self._errcheck_:
-            args = tuple(args)
-            v = self._errcheck_(result, self, args)
+            v = self._errcheck_(result, self, tuple(args))
             # If the errcheck funtion failed, let it throw
             # If the errcheck function returned newargs unchanged,
             # continue normal processing.
@@ -444,8 +431,8 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             ffiargs = [argtype.get_ffi_argtype() for argtype in argtypes]
             ffires = restype.get_ffi_argtype()
             return _ffi.FuncPtr.fromaddr(ptr, '', ffiargs, ffires, self._flags_)
-
-        cdll = self.dll.__pypy_dll__
+        
+        cdll = self.dll._handle
         try:
             ffi_argtypes = [argtype.get_ffi_argtype() for argtype in argtypes]
             ffi_restype = restype.get_ffi_argtype()
@@ -463,7 +450,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             # funcname -> _funcname@<n>
             # where n is 0, 4, 8, 12, ..., 128
             for i in range(33):
-                mangled_name = "_%s@%d" % (self.name, i * 4)
+                mangled_name = "_%s@%d" % (self.name, i*4)
                 try:
                     return cdll.getfunc(mangled_name,
                                         ffi_argtypes, ffi_restype,
@@ -486,13 +473,13 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
         # jit trace of the normal case
         from ctypes import c_char_p, c_wchar_p, c_void_p, c_int
         #
-        if isinstance(arg, bytes):
+        if isinstance(arg, str):
             cobj = c_char_p(arg)
-        elif isinstance(arg, str):
+        elif isinstance(arg, unicode):
             cobj = c_wchar_p(arg)
         elif arg is None:
             cobj = c_void_p()
-        elif isinstance(arg, int):
+        elif isinstance(arg, (int, long)):
             cobj = c_int(arg)
         else:
             raise TypeError("Don't know how to handle %s" % (arg,))
@@ -500,21 +487,15 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
         return cobj, cobj._to_ffi_param(), type(cobj)
 
     def _convert_args_for_callback(self, argtypes, args):
-        from _ctypes.structure import StructOrUnion
-        #
         assert len(argtypes) == len(args)
         newargs = []
         for argtype, arg in zip(argtypes, args):
             param = argtype.from_param(arg)
             _type_ = getattr(argtype, '_type_', None)
-            if _type_ == 'P':  # special-case for c_void_p
+            if _type_ == 'P': # special-case for c_void_p
                 param = param._get_buffer_value()
             elif self._is_primitive(argtype):
                 param = param.value
-            elif isinstance(param, StructOrUnion):   # not a *pointer* to struct
-                newparam = StructOrUnion.__new__(type(param))
-                param._copy_to(newparam._buffer.buffer)
-                param = newparam
             newargs.append(param)
         return newargs
 
@@ -578,7 +559,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
                         keepalive, newarg, newargtype = self._conv_param(argtype, defval)
                     else:
                         import ctypes
-                        val = argtype._type_._newowninstance_()
+                        val = argtype._type_()
                         keepalive = None
                         newarg = ctypes.byref(val)
                         newargtype = type(newarg)
@@ -623,8 +604,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
         """
         # hack for performance: if restype is a "simple" primitive type, don't
         # allocate the buffer because it's going to be thrown away immediately
-        if (self._is_primitive(restype) and restype._type_ != '?'
-            and not restype._is_pointer_like()):
+        if self._is_primitive(restype) and not restype._is_pointer_like():
             return result
         #
         shape = restype._ffishape_
@@ -671,7 +651,7 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
 
         return retval
 
-    def __bool__(self):
+    def __nonzero__(self):
         return self._com_index is not None or bool(self._buffer[0])
 
     def __del__(self):
@@ -687,11 +667,69 @@ class CFuncPtr(_CData, metaclass=CFuncPtrType):
             self._needs_free = False
 
 
+def make_fastpath_subclass(CFuncPtr):
+    if CFuncPtr._is_fastpath:
+        return CFuncPtr
+    #
+    try:
+        return make_fastpath_subclass.memo[CFuncPtr]
+    except KeyError:
+        pass
+
+    class CFuncPtrFast(CFuncPtr):
+
+        _is_fastpath = True
+        _slowpath_allowed = True # set to False by tests
+
+        @classmethod
+        def enable_fastpath_maybe(cls, obj):
+            if (obj.callable is None and
+                obj._com_index is None):
+                obj.__class__ = cls
+
+        def __rollback(self):
+            assert self._slowpath_allowed
+            self.__class__ = CFuncPtr
+
+        # disable the fast path if we reset argtypes
+        def _setargtypes(self, argtypes):
+            self.__rollback()
+            self._setargtypes(argtypes)
+        argtypes = property(CFuncPtr._getargtypes, _setargtypes)
+
+        def _setcallable(self, func):
+            self.__rollback()
+            self.callable = func
+        callable = property(lambda x: None, _setcallable)
+
+        def _setcom_index(self, idx):
+            self.__rollback()
+            self._com_index = idx
+        _com_index = property(lambda x: None, _setcom_index)
+
+        def __call__(self, *args):
+            thisarg = None
+            argtypes = self._argtypes_
+            restype = self._restype_
+            funcptr = self._getfuncptr(argtypes, restype, thisarg)
+            try:
+                result = self._call_funcptr(funcptr, *args)
+                result, _ = self._do_errcheck(result, args)
+            except (TypeError, ArgumentError, UnicodeDecodeError):
+                assert self._slowpath_allowed
+                return CFuncPtr.__call__(self, *args)
+            return result
+
+    make_fastpath_subclass.memo[CFuncPtr] = CFuncPtrFast
+    return CFuncPtrFast
+make_fastpath_subclass.memo = {}
+
+
 def handle_system_exit(e):
     # issue #1194: if we get SystemExit here, then exit the interpreter.
     # Highly obscure imho but some people seem to depend on it.
     if sys.flags.inspect:
-        return  # Don't exit if -i flag was given.
+        return   # Don't exit if -i flag was given.
     else:
         code = e.code
         if isinstance(code, int):
