@@ -2,9 +2,9 @@ import sys
 import _rawffi
 from _ctypes.basics import _CData, _CDataMeta, keepalive_key,\
      store_reference, ensure_objects, CArgObject
-from _ctypes.array import Array, swappedorder, byteorder
+from _ctypes.array import Array
 from _ctypes.pointer import _Pointer
-import __pypy__
+import inspect
 
 
 def names_and_fields(self, _fields_, superclass, anonymous_fields=None):
@@ -28,7 +28,7 @@ def names_and_fields(self, _fields_, superclass, anonymous_fields=None):
                 raise TypeError('bit fields not allowed for type ' + tp.__name__)
 
     all_fields = []
-    for cls in reversed(superclass.__mro__):
+    for cls in reversed(inspect.getmro(superclass)):
         # The first field comes from the most base class
         all_fields.extend(getattr(cls, '_fields_', []))
     all_fields.extend(_fields_)
@@ -40,22 +40,6 @@ def names_and_fields(self, _fields_, superclass, anonymous_fields=None):
         else:
             rawfields.append((f[0], f[1]._ffishape_))
 
-    # hack for duplicate field names
-    already_seen = set()
-    names1 = names
-    names = []
-    for f in names1:
-        if f not in already_seen:
-            names.append(f)
-            already_seen.add(f)
-    already_seen = set()
-    for i in reversed(range(len(rawfields))):
-        if rawfields[i][0] in already_seen:
-            rawfields[i] = (('$DUP%d$%s' % (i, rawfields[i][0]),)
-                            + rawfields[i][1:])
-        already_seen.add(rawfields[i][0])
-    # /hack
-
     _set_shape(self, rawfields, self._is_union)
 
     fields = {}
@@ -119,8 +103,6 @@ class Field(object):
         if self.is_bitfield:
             # bitfield member, use direct access
             return obj._buffer.__getattr__(self.name)
-        elif not isinstance(obj, _CData):
-            raise(TypeError, 'not a ctype instance') 
         else:
             fieldtype = self.ctype
             offset = self.num
@@ -144,13 +126,10 @@ class Field(object):
             from ctypes import memmove
             dest = obj._buffer.fieldaddress(self.name)
             memmove(dest, arg, fieldtype._fficompositesize_)
-        elif not isinstance(obj, _CData):
-            raise(TypeError, 'not a ctype instance') 
         else:
             obj._buffer.__setattr__(self.name, arg)
 
 
-
 def _set_shape(tp, rawfields, is_union=False):
     tp._ffistruct_ = _rawffi.Structure(rawfields, is_union,
                                       getattr(tp, '_pack_', 0))
@@ -164,10 +143,6 @@ def struct_setattr(self, name, value):
             raise AttributeError("_fields_ is final")
         if self in [f[1] for f in value]:
             raise AttributeError("Structure or union cannot contain itself")
-        if self._ffiargtype is not None:
-            raise NotImplementedError("Too late to set _fields_: we already "
-                        "said to libffi that the structure type %s is opaque"
-                        % (self,))
         names_and_fields(
             self,
             value, self.__bases__[0],
@@ -180,11 +155,6 @@ def struct_setattr(self, name, value):
 class StructOrUnionMeta(_CDataMeta):
     def __new__(self, name, cls, typedict):
         res = type.__new__(self, name, cls, typedict)
-        if hasattr(res, '_swappedbytes_') and '_fields_' in typedict:
-            # Activate the stdlib ctypes._swapped_meta.__setattr__ to convert fields
-            tmp = res._fields_
-            delattr(res, '_fields_')
-            setattr(res, '_fields_', tmp)
         if "_abstract_" in typedict:
             return res
         cls = cls or (object,)
@@ -213,15 +183,12 @@ class StructOrUnionMeta(_CDataMeta):
 
     __setattr__ = struct_setattr
 
-    def _is_abstract(self):
-        return False
-
     def from_address(self, address):
         instance = StructOrUnion.__new__(self)
         if isinstance(address, _rawffi.StructureInstance):
             address = address.buffer
         # fix the address: turn it into as unsigned, in case it is negative
-        address = address & (sys.maxsize * 2 + 1)
+        address = address & (sys.maxint * 2 + 1)
         instance.__dict__['_buffer'] = self._ffistruct_.fromaddress(address)
         return instance
 
@@ -237,7 +204,7 @@ class StructOrUnionMeta(_CDataMeta):
         if isinstance(value, tuple):
             try:
                 value = self(*value)
-            except Exception as e:
+            except Exception, e:
                 # XXX CPython does not even respect the exception type
                 raise RuntimeError("(%s) %s: %s" % (self.__name__, type(e), e))
         return _CDataMeta.from_param(self, value)
@@ -257,36 +224,14 @@ class StructOrUnionMeta(_CDataMeta):
         res.__dict__['_index'] = -1
         return res
 
-    def _getformat(self):
-        if self._is_union or hasattr(self, '_pack_'):
-            return "B"
-        if hasattr(self, '_swappedbytes_'):
-            bo = swappedorder[sys.byteorder]
-        else:
-            bo = byteorder[sys.byteorder]
-        flds = []
-        cum_size = 0
-        for name, obj in self._fields_:
-            padding = self._ffistruct_.fieldoffset(name) - cum_size
-            if padding:
-                flds.append('%dx' % padding)
-            flds.append(obj._getformat())
-            flds.append(':')
-            flds.append(name)
-            flds.append(':')
-            cum_size += self._ffistruct_.fieldsize(name)
-        return 'T{' + ''.join(flds) + '}'
 
-class StructOrUnion(_CData, metaclass=StructOrUnionMeta):
+class StructOrUnion(_CData):
+    __metaclass__ = StructOrUnionMeta
 
     def __new__(cls, *args, **kwds):
-        from _ctypes import union
-        if ('_abstract_' in cls.__dict__ or cls is Structure
-                                         or cls is union.Union):
-            raise TypeError("abstract class")
-        if hasattr(cls, '_swappedbytes_'):
-            names_and_fields(cls, cls._fields_, _CData, cls.__dict__.get('_anonymous_', None))
         self = super(_CData, cls).__new__(cls)
+        if '_abstract_' in cls.__dict__:
+            raise TypeError("abstract class")
         if hasattr(cls, '_ffistruct_'):
             self.__dict__['_buffer'] = self._ffistruct_(autofree=True)
         return self
@@ -302,7 +247,6 @@ class StructOrUnion(_CData, metaclass=StructOrUnionMeta):
             self.__setattr__(name, arg)
         for name, arg in kwds.items():
             self.__setattr__(name, arg)
-    _init_no_arg_ = __init__
 
     def _subarray(self, fieldtype, name):
         """Return a _rawffi array of length 1 whose address is the same as
@@ -317,23 +261,13 @@ class StructOrUnion(_CData, metaclass=StructOrUnionMeta):
     def _get_buffer_value(self):
         return self._buffer.buffer
 
-    def _copy_to(self, addr):
-        from ctypes import memmove
-        origin = self._get_buffer_value()
-        memmove(addr, origin, self._fficompositesize_)
-
     def _to_ffi_param(self):
-        # Do not copy, like CPython
         return self._buffer
 
-    def __buffer__(self, flags):
-        fmt = type(self)._getformat()
-        itemsize = type(self)._sizeofinstances()
-        return __pypy__.newmemoryview(memoryview(self._buffer), itemsize, fmt, ())
 
 class StructureMeta(StructOrUnionMeta):
     _is_union = False
 
 
-class Structure(StructOrUnion, metaclass=StructureMeta):
-    pass
+class Structure(StructOrUnion):
+    __metaclass__ = StructureMeta
