@@ -15,93 +15,71 @@ all_modules = [p.basename for p in modulepath.listdir()
                and not p.basename.startswith('test')]
 
 essential_modules = set([
-    "exceptions", "_io", "sys", "builtins", "posix", "_warnings",
-    "itertools", "_frozen_importlib", "operator", "_locale", "struct",
-    "thread", "__pypy__", "zipimport", "_opcode",
+    "exceptions", "_file", "sys", "__builtin__", "posix", "_warnings",
+    "itertools"
 ])
-if sys.platform == "win32":
-    essential_modules.add("_winreg")
 
 default_modules = essential_modules.copy()
 default_modules.update([
-    "_codecs", "atexit", "gc", "_weakref", "marshal", "errno", "imp",
-    "itertools", "math", "cmath", "_sre", "_pickle_support",
-    "parser", "symbol", "token", "_ast", "_random",
-    "_string", "_testing", "time", "_abc",
+    "_codecs", "gc", "_weakref", "marshal", "errno", "imp", "math", "cmath",
+    "_sre", "_pickle_support", "operator", "parser", "symbol", "token", "_ast",
+    "_io", "_random", "__pypy__", "_testing", "time"
 ])
 
 
 # --allworkingmodules
 working_modules = default_modules.copy()
 working_modules.update([
-    "_socket", "unicodedata", "mmap", "fcntl", "pwd",
-    "select", "_lsprof", "signal", "_rawffi", "termios",
-    "zlib", "bz2", "_md5", "_minimal_curses",
-    "itertools", "pyexpat", "cpyext", "array",
+    "_socket", "unicodedata", "mmap", "fcntl", "_locale", "pwd",
+    "select", "zipimport", "_lsprof", "crypt", "signal", "_rawffi", "termios",
+    "zlib", "bz2", "struct", "_hashlib", "_md5", "_sha", "_minimal_curses",
+    "cStringIO", "thread", "itertools", "pyexpat", "_ssl", "cpyext", "array",
     "binascii", "_multiprocessing", '_warnings', "_collections",
-    "_multibytecodec", "_continuation", "_cffi_backend",
-    "_csv", "_pypyjson", "_posixsubprocess", "_cppyy", # "micronumpy",
-    "_jitlog", "_hpy_universal"
-    # "_hashlib", "crypt"
+    "_multibytecodec", "micronumpy", "_continuation", "_cffi_backend",
+    "_csv", "cppyy", "_pypyjson", "_jitlog"
 ])
 
-import rpython.rlib.rvmprof.cintf
-if rpython.rlib.rvmprof.cintf.IS_SUPPORTED:
-    working_modules.add('_vmprof')
-    working_modules.add('faulthandler')
+from rpython.jit.backend import detect_cpu
+try:
+    if detect_cpu.autodetect().startswith('x86'):
+        working_modules.add('_vmprof')
+        working_modules.add('faulthandler')
+except detect_cpu.ProcessorAutodetectError:
+    pass
 
 translation_modules = default_modules.copy()
 translation_modules.update([
-    "fcntl", "time", "select", "signal", "_rawffi", "zlib", "struct",
-    "array", "binascii",
+    "fcntl", "time", "select", "signal", "_rawffi", "zlib", "struct", "_md5",
+    "cStringIO", "array", "binascii",
     # the following are needed for pyrepl (and hence for the
     # interactive prompt/pdb)
     "termios", "_minimal_curses",
 ])
 
-reverse_debugger_disable_modules = set([
-    "_continuation", "_vmprof", "_multiprocessing",
-    "micronumpy",
-    ])
-
 # XXX this should move somewhere else, maybe to platform ("is this posixish"
 #     check or something)
 if sys.platform == "win32":
+    working_modules.add("_winreg")
     # unix only modules
-    for name in ["crypt", "fcntl", "pwd", "termios", "_minimal_curses",
-                 "_posixsubprocess"]:
-        if name in working_modules:
-            working_modules.remove(name)
+    for name in ["crypt", "fcntl", "pwd", "termios", "_minimal_curses"]:
+        working_modules.remove(name)
         if name in translation_modules:
             translation_modules.remove(name)
 
+    if "cppyy" in working_modules:
+        working_modules.remove("cppyy")  # not tested on win32
     if "faulthandler" in working_modules:
         working_modules.remove("faulthandler")  # missing details
-    if "_vmprof" in working_modules:
-        working_modules.remove("_vmprof")  # FIXME: missing details
 
     # The _locale module is needed by site.py on Windows
     default_modules.add("_locale")
 
-    # needed to invoke MSVC
-    translation_modules.update(["thread", "_winreg", "_cffi_backend"])
-
-    # not ported yet
-    if IS_64_BITS:
-        for name in ["_cppyy", "micronumpy"]:
-            if name in working_modules:
-                working_modules.remove(name)
-
 if sys.platform == "sunos5":
     working_modules.remove('fcntl')  # LOCK_NB not defined
     working_modules.remove("_minimal_curses")
     working_modules.remove("termios")
-    if "_cppyy" in working_modules:
-        working_modules.remove("_cppyy")  # depends on ctypes
-
-if sys.platform.startswith('linux') and sys.maxsize <= 2**31:
-    # _hpy_universal needs tweaking to work on 32-bit linux
-    working_modules.remove('_hpy_universal')
+    if "cppyy" in working_modules:
+        working_modules.remove("cppyy")  # depends on ctypes
 
 #if sys.platform.startswith("linux"):
 #    _mach = os.popen('uname -m', 'r').read().strip()
@@ -112,9 +90,8 @@ if sys.platform.startswith('linux') and sys.maxsize <= 2**31:
 module_dependencies = {
     '_multiprocessing': [('objspace.usemodules.time', True),
                          ('objspace.usemodules.thread', True)],
-    '_cffi_backend': [('objspace.usemodules.thread', True)],
     'cpyext': [('objspace.usemodules.array', True)],
-    '_cppyy': [('objspace.usemodules.cpyext', True)],
+    'cppyy': [('objspace.usemodules.cpyext', True)],
     'faulthandler': [('objspace.usemodules._vmprof', True)],
     }
 module_suggests = {
@@ -128,6 +105,43 @@ if sys.platform == "win32":
     module_suggests["cpyext"].append(("translation.shared", True))
 
 
+# NOTE: this dictionary is not used any more
+module_import_dependencies = {
+    # no _rawffi if importing rpython.rlib.clibffi raises ImportError
+    # or CompilationError or py.test.skip.Exception
+    "_rawffi"   : ["rpython.rlib.clibffi"],
+
+    "zlib"      : ["rpython.rlib.rzlib"],
+    "bz2"       : ["pypy.module.bz2.interp_bz2"],
+    "pyexpat"   : ["pypy.module.pyexpat.interp_pyexpat"],
+    "_ssl"      : ["pypy.module._ssl.interp_ssl"],
+    "_hashlib"  : ["pypy.module._ssl.interp_ssl"],
+    "_minimal_curses": ["pypy.module._minimal_curses.fficurses"],
+    "_continuation": ["rpython.rlib.rstacklet"],
+    "_vmprof"      : ["pypy.module._vmprof.interp_vmprof"],
+    "faulthandler" : ["pypy.module._vmprof.interp_vmprof"],
+    }
+
+def get_module_validator(modname):
+    # NOTE: this function is not used any more
+    if modname in module_import_dependencies:
+        modlist = module_import_dependencies[modname]
+        def validator(config):
+            from rpython.rtyper.tool.rffi_platform import CompilationError
+            try:
+                for name in modlist:
+                    __import__(name)
+            except (ImportError, CompilationError, py.test.skip.Exception) as e:
+                errcls = e.__class__.__name__
+                raise Exception(
+                    "The module %r is disabled\n" % (modname,) +
+                    "because importing %s raised %s\n" % (name, errcls) +
+                    str(e))
+        return validator
+    else:
+        return None
+
+
 pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
     OptionDescription("usemodules", "Which Modules should be used", [
         BoolOption(modname, "use module %s" % (modname, ),
@@ -136,7 +150,7 @@ pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
                    requires=module_dependencies.get(modname, []),
                    suggests=module_suggests.get(modname, []),
                    negation=modname not in essential_modules,
-                   )
+                   ) #validator=get_module_validator(modname))
         for modname in all_modules]),
 
     BoolOption("allworkingmodules", "use as many working modules as possible",
@@ -160,6 +174,9 @@ pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
                cmdline="--translationmodules",
                suggests=[("objspace.allworkingmodules", False)]),
 
+    BoolOption("lonepycfiles", "Import pyc files with no matching py file",
+               default=False),
+
     StrOption("soabi",
               "Tag to differentiate extension modules built for different Python interpreters",
               cmdline="--soabi",
@@ -173,32 +190,6 @@ pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
                "make sure that all calls go through space.call_args",
                default=False),
 
-    BoolOption("disable_entrypoints",
-               "Disable external entry points, notably the"
-               " cpyext module and cffi's embedding mode.",
-               default=False,
-               requires=[("objspace.usemodules.cpyext", False)]),
-
-    BoolOption("disable_entrypoints_in_cffi",
-               "Disable only cffi's embedding mode.",
-               default=False),
-
-    ChoiceOption("hash",
-                 "The hash function to use for strings: fnv from CPython 2.7"
-                 " or siphash24 from CPython >= 3.4",
-                 ["fnv", "siphash24"],
-                 default="siphash24",
-                 cmdline="--hash"),
-
-    BoolOption("hpy_cpyext_API",
-               "Enable the HPy/cpyext API in the hpy_universal module",
-               default=True),
-
-    StrOption("platlibdir",
-              "Configure the platlibdir at translation time",
-              cmdline="--platlibdir",
-              default="lib"),
-
     OptionDescription("std", "Standard Object Space Options", [
         BoolOption("withtproxy", "support transparent proxies",
                    default=True),
@@ -215,10 +206,18 @@ pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
         BoolOption("withsmalllong", "use a version of 'long' in a C long long",
                    default=False),
 
+        BoolOption("withstrbuf", "use strings optimized for addition (ver 2)",
+                   default=False),
+
         BoolOption("withspecialisedtuple",
                    "use specialised tuples",
                    default=False),
 
+        BoolOption("withcelldict",
+                   "use dictionaries that are optimized for being used as module dicts",
+                   default=False,
+                   requires=[("objspace.honor__builtins__", False)]),
+
         BoolOption("withliststrategies",
                    "enable optimized ways to store lists of primitives ",
                    default=True),
@@ -240,6 +239,7 @@ pypy_optiondescription = OptionDescription("objspace", "Object Space Options", [
         BoolOption("newshortcut",
                    "cache and shortcut calling __new__ from builtin types",
                    default=False),
+
      ]),
 ])
 
@@ -277,19 +277,13 @@ def set_pypy_opt_level(config, level):
 
     # extra optimizations with the JIT
     if level == 'jit':
-        pass # none at the moment
-
-    if config.translation.sandbox or config.translation.reverse_debugger:
-        config.objspace.hash = "fnv"
+        config.objspace.std.suggest(withcelldict=True)
 
 
 def enable_allworkingmodules(config):
     modules = working_modules.copy()
     if config.translation.sandbox:
         modules = default_modules
-    if config.translation.reverse_debugger:
-        for mod in reverse_debugger_disable_modules:
-            setattr(config.objspace.usemodules, mod, False)
     # ignore names from 'essential_modules', notably 'exceptions', which
     # may not be present in config.objspace.usemodules at all
     modules = [name for name in modules if name not in essential_modules]
@@ -308,4 +302,3 @@ if __name__ == '__main__':
     parser = to_optparse(config) #, useoptions=["translation.*"])
     option, args = parser.parse_args()
     print config
-    print working_modules
