@@ -1,7 +1,6 @@
 import py
 
 import os, sys, subprocess
-from os.path import join, dirname
 
 import pypy
 from pypy.interpreter import gateway
@@ -9,19 +8,17 @@ from pypy.interpreter.error import OperationError
 from pypy.tool.ann_override import PyPyAnnotatorPolicy
 from rpython.config.config import to_optparse, make_dict, SUPPRESS_USAGE
 from rpython.config.config import ConflictConfigError
-from rpython.rlib import rlocale
 from pypy.tool.option import make_objspace
 from pypy import pypydir
 from rpython.rlib import rthread
 from pypy.module.thread import os_thread
-from pypy.module.sys.version import CPYTHON_VERSION
 
 thisdir = py.path.local(__file__).dirpath()
 
 try:
-    this_dir = dirname(__file__)
+    this_dir = os.path.dirname(__file__)
 except NameError:
-    this_dir = dirname(sys.argv[0])
+    this_dir = os.path.dirname(sys.argv[0])
 
 def debug(msg):
     try:
@@ -34,25 +31,15 @@ def debug(msg):
 
 def create_entry_point(space, w_dict):
     if w_dict is not None: # for tests
-        w_entry_point = space.getitem(w_dict, space.newtext('entry_point'))
-        w_run_toplevel = space.getitem(w_dict, space.newtext('run_toplevel'))
-        w_initstdio = space.getitem(w_dict, space.newtext('initstdio'))
+        w_entry_point = space.getitem(w_dict, space.wrap('entry_point'))
+        w_run_toplevel = space.getitem(w_dict, space.wrap('run_toplevel'))
         withjit = space.config.objspace.usemodules.pypyjit
-        hashfunc = space.config.objspace.hash
-    else:
-        w_initstdio = space.appexec([], """():
-            return lambda unbuffered: None
-        """)
 
     def entry_point(argv):
         if withjit:
             from rpython.jit.backend.hlinfo import highleveljitinfo
             highleveljitinfo.sys_executable = argv[0]
 
-        if hashfunc == "siphash24":
-            from rpython.rlib import rsiphash
-            rsiphash.enable_siphash24()
-
         #debug("entry point starting")
         #for arg in argv:
         #    debug(" argv -> " + arg)
@@ -68,49 +55,35 @@ def create_entry_point(space, w_dict):
         try:
             try:
                 space.startup()
-                if rlocale.HAVE_LANGINFO:
-                    try:
-                        rlocale.setlocale(rlocale.LC_CTYPE, '')
-                    except rlocale.LocaleError:
-                        pass
-                w_executable = space.newfilename(argv[0])
-                w_argv = space.newlist([space.newfilename(s)
-                                        for s in argv[1:]])
-                w_bargv = space.newlist([space.newbytes(s)
-                                        for s in argv[1:]])
-                w_exitcode = space.call_function(w_entry_point, w_executable, w_bargv, w_argv)
+                w_executable = space.wrap(argv[0])
+                w_argv = space.newlist([space.wrap(s) for s in argv[1:]])
+                w_exitcode = space.call_function(w_entry_point, w_executable, w_argv)
                 exitcode = space.int_w(w_exitcode)
+                # try to pull it all in
+            ##    from pypy.interpreter import main, interactive, error
+            ##    con = interactive.PyPyConsole(space)
+            ##    con.interact()
             except OperationError as e:
                 debug("OperationError:")
                 debug(" operror-type: " + e.w_type.getname(space))
-                debug(" operror-value: " + space.text_w(space.str(e.get_w_value(space))))
+                debug(" operror-value: " + space.str_w(space.str(e.get_w_value(space))))
                 return 1
         finally:
             try:
-                # the equivalent of Py_FinalizeEx
-                if space.finish() < 0:
-                    # Value unlikely to be confused with a non-error exit status
-                    # or other special meaning (from cpython/Modules/main.c)
-                    exitcode = 120
+                space.finish()
             except OperationError as e:
                 debug("OperationError:")
                 debug(" operror-type: " + e.w_type.getname(space))
-                debug(" operror-value: " + space.text_w(space.str(e.get_w_value(space))))
+                debug(" operror-value: " + space.str_w(space.str(e.get_w_value(space))))
                 return 1
         return exitcode
 
-    return entry_point, get_additional_entrypoints(space, w_initstdio)
-
-
-def get_additional_entrypoints(space, w_initstdio):
     # register the minimal equivalent of running a small piece of code. This
     # should be used as sparsely as possible, just to register callbacks
+
     from rpython.rlib.entrypoint import entrypoint_highlevel
     from rpython.rtyper.lltypesystem import rffi, lltype
 
-    if space.config.objspace.disable_entrypoints:
-        return {}
-
     @entrypoint_highlevel('main', [rffi.CCHARP, rffi.INT],
                           c_name='pypy_setup_home')
     def pypy_setup_home(ll_home, verbose):
@@ -118,7 +91,7 @@ def get_additional_entrypoints(space, w_initstdio):
         verbose = rffi.cast(lltype.Signed, verbose)
         if ll_home and ord(ll_home[0]):
             home1 = rffi.charp2str(ll_home)
-            home = join(home1, 'x') # <- so that 'll_home' can be
+            home = os.path.join(home1, 'x') # <- so that 'll_home' can be
                                             # directly the root directory
         else:
             home1 = "pypy's shared library location"
@@ -130,43 +103,25 @@ def get_additional_entrypoints(space, w_initstdio):
                       " not found in %s or in any parent directory" % home1)
             return rffi.cast(rffi.INT, 1)
         space.startup()
-        must_leave = space.threadlocals.try_enter_thread(space)
+        space.appexec([w_path], """(path):
+            import sys
+            sys.path[:] = path
+        """)
+        # import site
         try:
-            # initialize sys.{path,executable,stdin,stdout,stderr}
-            # (in unbuffered mode, to avoid troubles) and import site
-            space.appexec([w_path, space.newfilename(home), w_initstdio],
-            r"""(path, home, initstdio):
-                import sys 
-                # don't import anything more above this: sys.path is not set
-                sys.path[:] = path
-                sys.executable = home
-                initstdio(unbuffered=True)
-                import os   # don't move it to the first line of this function!
-                _MACOSX = sys.platform == 'darwin'
-                if _MACOSX:
-                    # __PYVENV_LAUNCHER__, used by CPython on macOS, should be ignored
-                    # since it (possibly) results in a wrong sys.prefix and
-                    # sys.exec_prefix (and consequently sys.path).
-                    old_pyvenv_launcher = os.environ.pop('__PYVENV_LAUNCHER__', None)
-                try:
-                    import site
-                except Exception as e:
-                    sys.stderr.write("'import site' failed:\n")
-                    import traceback
-                    traceback.print_exc()
-                if _MACOSX and old_pyvenv_launcher:
-                    os.environ['__PYVENV_LAUNCHER__'] = old_pyvenv_launcher
-            """)
+            space.setattr(space.getbuiltinmodule('sys'),
+                          space.wrap('executable'),
+                          space.wrap(home))
+            import_ = space.getattr(space.getbuiltinmodule('__builtin__'),
+                                    space.wrap('__import__'))
+            space.call_function(import_, space.wrap('site'))
             return rffi.cast(rffi.INT, 0)
         except OperationError as e:
             if verbose:
                 debug("OperationError:")
                 debug(" operror-type: " + e.w_type.getname(space))
-                debug(" operror-value: " + space.text_w(space.str(e.get_w_value(space))))
+                debug(" operror-value: " + space.str_w(space.str(e.get_w_value(space))))
             return rffi.cast(rffi.INT, -1)
-        finally:
-            if must_leave:
-                space.threadlocals.leave_thread(space)
 
     @entrypoint_highlevel('main', [rffi.CCHARP], c_name='pypy_execute_source')
     def pypy_execute_source(ll_source):
@@ -201,30 +156,30 @@ def get_additional_entrypoints(space, w_initstdio):
     def _pypy_execute_source(source, c_argument):
         try:
             w_globals = space.newdict(module=True)
-            space.setitem(w_globals, space.newtext('__builtins__'),
-                          space.builtin_modules['builtins'])
-            space.setitem(w_globals, space.newtext('c_argument'),
-                          space.newint(c_argument))
-            space.appexec([space.newtext(source), w_globals], """(src, glob):
+            space.setitem(w_globals, space.wrap('__builtins__'),
+                          space.builtin_modules['__builtin__'])
+            space.setitem(w_globals, space.wrap('c_argument'),
+                          space.wrap(c_argument))
+            space.appexec([space.wrap(source), w_globals], """(src, glob):
                 import sys
                 stmt = compile(src, 'c callback', 'exec')
                 if not hasattr(sys, '_pypy_execute_source'):
                     sys._pypy_execute_source = []
                 sys._pypy_execute_source.append(glob)
-                exec(stmt, glob)
+                exec stmt in glob
             """)
         except OperationError as e:
             debug("OperationError:")
             debug(" operror-type: " + e.w_type.getname(space))
-            debug(" operror-value: " + space.text_w(space.str(e.get_w_value(space))))
+            debug(" operror-value: " + space.str_w(space.str(e.get_w_value(space))))
             return -1
         return 0
 
-    return {'pypy_execute_source': pypy_execute_source,
-            'pypy_execute_source_ptr': pypy_execute_source_ptr,
-            'pypy_init_threads': pypy_init_threads,
-            'pypy_thread_attach': pypy_thread_attach,
-            'pypy_setup_home': pypy_setup_home}
+    return entry_point, {'pypy_execute_source': pypy_execute_source,
+                         'pypy_execute_source_ptr': pypy_execute_source_ptr,
+                         'pypy_init_threads': pypy_init_threads,
+                         'pypy_thread_attach': pypy_thread_attach,
+                         'pypy_setup_home': pypy_setup_home}
 
 
 # _____ Define and setup target ___
@@ -236,7 +191,6 @@ class PyPyTarget(object):
     usage = SUPPRESS_USAGE
 
     take_options = True
-    space = None
 
     def opt_parser(self, config):
         parser = to_optparse(config, useoptions=["objspace.*"],
@@ -249,12 +203,6 @@ class PyPyTarget(object):
             raise Exception("You have to specify the --opt level.\n"
                     "Try --opt=2 or --opt=jit, or equivalently -O2 or -Ojit .")
         self.translateconfig = translateconfig
-
-        # change the default for this option
-        # XXX disabled until we fix the real problem: a per-translation
-        # seed for siphash is bad
-        #config.translation.suggest(hash="siphash24")
-
         # set up the objspace optimizations based on the --opt argument
         from pypy.config.pypyoption import set_pypy_opt_level
         set_pypy_opt_level(config, translateconfig.opt)
@@ -267,7 +215,7 @@ class PyPyTarget(object):
         return pypy_optiondescription
 
     def target(self, driver, args):
-        driver.exe_name = 'pypy%d.%d-%%(backend)s' % CPYTHON_VERSION[:2]
+        driver.exe_name = 'pypy-%(backend)s'
 
         config = driver.config
         parser = self.opt_parser(config)
@@ -286,26 +234,20 @@ class PyPyTarget(object):
 
         config.translation.suggest(check_str_without_nul=True)
         config.translation.suggest(shared=True)
-        config.translation.suggest(icon=join(this_dir, 'pypy.ico'))
-        config.translation.suggest(manifest=join(this_dir, 'python.manifest'))
+        config.translation.suggest(icon=os.path.join(this_dir, 'pypy.ico'))
         if config.translation.shared:
             if config.translation.output is not None:
                 raise Exception("Cannot use the --output option with PyPy "
                                 "when --shared is on (it is by default). "
                                 "See issue #1971.")
-
-        # if both profopt and profoptpath are specified then we keep them as they are with no other changes
-        if config.translation.profopt:
-            if config.translation.profoptargs is None:
-                config.translation.profoptargs = "$(RPYDIR)/../lib-python/2.7/test/regrtest.py --pgo -x test_asyncore test_gdb test_multiprocessing test_subprocess || true"
-        elif config.translation.profoptargs is not None:
-            raise Exception("Cannot use --profoptargs without specifying --profopt as well")
-
+            if config.translation.profopt is not None:
+                raise Exception("Cannot use the --profopt option "
+                                "when --shared is on (it is by default). "
+                                "See issue #2398.")
         if sys.platform == 'win32':
             libdir = thisdir.join('..', '..', 'libs')
             libdir.ensure(dir=1)
-            pythonlib = "python{0[0]}{0[1]}.lib".format(CPYTHON_VERSION)
-            config.translation.libname = str(libdir.join(pythonlib))
+            config.translation.libname = str(libdir.join('python27.lib'))
 
         if config.translation.thread:
             config.objspace.usemodules.thread = True
@@ -340,15 +282,7 @@ class PyPyTarget(object):
             config.translation.jit = True
 
         if config.translation.sandbox:
-            assert 0, ("--sandbox is not tested nor maintained.  If you "
-                       "really want to try it anyway, remove this line in "
-                       "pypy/goal/targetpypystandalone.py.")
-
-        if config.objspace.usemodules.cpyext:
-            if config.translation.gc not in ('incminimark', 'boehm'):
-                raise Exception("The 'cpyext' module requires the 'incminimark'"
-                    " or 'boehm' GC.  You need either 'targetpypystandalone.py"
-                    " --withoutmod-cpyext', or use one of these two GCs.")
+            config.objspace.lonepycfiles = False
 
         config.translating = True
 
@@ -356,24 +290,19 @@ class PyPyTarget(object):
         translate.log_config(config.objspace, "PyPy config object")
 
         # obscure hack to stuff the translation options into the translated PyPy
-        from pypy.module.sys.moduledef import Module as SysModule
+        import pypy.module.sys
         options = make_dict(config)
-        wrapstr = 'space.wrap(%r)' % (options)  # import time
-        SysModule.interpleveldefs['pypy_translation_info'] = wrapstr
-        
-        if 'compile' in driver._disabled:
-            driver.default_goal = 'source'
-        elif config.objspace.usemodules._cffi_backend:
+        wrapstr = 'space.wrap(%r)' % (options)
+        pypy.module.sys.Module.interpleveldefs['pypy_translation_info'] = wrapstr
+        if config.objspace.usemodules._cffi_backend:
             self.hack_for_cffi_modules(driver)
-        else:
-            driver.default_goal = 'compile' 
+
         return self.get_entry_point(config)
 
     def hack_for_cffi_modules(self, driver):
         # HACKHACKHACK
-        # ugly hack to modify target goal from compile_* to build_cffi_imports,
-        # as done in package.py
-        # this is needed by the benchmark buildbot run, maybe do it as a seperate step there?
+        # ugly hack to modify target goal from compile_* to build_cffi_imports
+        # this should probably get cleaned up and merged with driver.create_exe
         from rpython.tool.runsubprocess import run_subprocess
         from rpython.translator.driver import taskdef
         import types
@@ -382,13 +311,12 @@ class PyPyTarget(object):
         @taskdef([compile_goal], "Create cffi bindings for modules")
         def task_build_cffi_imports(self):
             ''' Use cffi to compile cffi interfaces to modules'''
-            filename = join(pypydir, '..', 'lib_pypy', 'pypy_tools',
-                                   'build_cffi_imports.py')
-            argv = [filename,]
-            exe_name = py.path.local(driver.c_entryp)
-            status, out, err = run_subprocess(str(exe_name), argv)
+            filename = os.path.join(pypydir, 'tool', 'build_cffi_imports.py')
+            status, out, err = run_subprocess(str(driver.compute_exe_name()),
+                                              [filename])
             sys.stdout.write(out)
             sys.stderr.write(err)
+            # otherwise, ignore errors
         driver.task_build_cffi_imports = types.MethodType(task_build_cffi_imports, driver)
         driver.tasks['build_cffi_imports'] = driver.task_build_cffi_imports, [compile_goal]
         driver.default_goal = 'build_cffi_imports'
@@ -399,30 +327,24 @@ class PyPyTarget(object):
         from pypy.module.pypyjit.hooks import pypy_hooks
         return PyPyJitPolicy(pypy_hooks)
 
-    def get_gchooks(self):
-        from pypy.module.gc.hook import LowLevelGcHooks
-        if self.space is None:
-            raise Exception("get_gchooks must be called after get_entry_point")
-        return self.space.fromcache(LowLevelGcHooks)
-
     def get_entry_point(self, config):
-        self.space = make_objspace(config)
+        space = make_objspace(config)
 
         # manually imports app_main.py
-        filename = join(pypydir, 'interpreter', 'app_main.py')
+        filename = os.path.join(pypydir, 'interpreter', 'app_main.py')
         app = gateway.applevel(open(filename).read(), 'app_main.py', 'app_main')
         app.hidden_applevel = False
-        w_dict = app.getwdict(self.space)
-        entry_point, _ = create_entry_point(self.space, w_dict)
+        w_dict = app.getwdict(space)
+        entry_point, _ = create_entry_point(space, w_dict)
 
-        return entry_point, None, PyPyAnnotatorPolicy(self.space)
+        return entry_point, None, PyPyAnnotatorPolicy()
 
     def interface(self, ns):
         for name in ['take_options', 'handle_config', 'print_help', 'target',
                      'jitpolicy', 'get_entry_point',
                      'get_additional_config_options']:
             ns[name] = getattr(self, name)
-        ns['get_gchooks'] = self.get_gchooks
+
 
 PyPyTarget().interface(globals())
 
