@@ -1,15 +1,15 @@
 import sys
 from pypy.interpreter.error import OperationError, get_cleared_operation_error
 from rpython.rlib.unroll import unrolling_iterable
-from rpython.rlib.objectmodel import specialize, not_rpython
+from rpython.rlib.objectmodel import specialize
 from rpython.rlib import jit, rgc, objectmodel
 
 TICK_COUNTER_STEP = 100
 
 def app_profile_call(space, w_callable, frame, event, w_arg):
     space.call_function(w_callable,
-                        frame,
-                        space.newtext(event), w_arg)
+                        space.wrap(frame),
+                        space.wrap(event), w_arg)
 
 class ExecutionContext(object):
     """An ExecutionContext holds the state of an execution thread
@@ -23,27 +23,17 @@ class ExecutionContext(object):
     # XXX [fijal] but they're not. is_being_profiled is guarded a bit all
     #     over the place as well as w_tracefunc
 
-    _immutable_fields_ = [
-        'profilefunc?', 'w_tracefunc?',
-        'w_asyncgen_firstiter_fn?', 'w_asyncgen_finalizer_fn?']
+    _immutable_fields_ = ['profilefunc?', 'w_tracefunc?']
 
     def __init__(self, space):
         self.space = space
         self.topframeref = jit.vref_None
-        # this is exposed to app-level as 'sys.exc_info()'.  At any point in
-        # time it is the exception caught by the topmost 'except ... as e:'
-        # app-level block.
-        self.sys_exc_operror = None
         self.w_tracefunc = None
         self.is_tracing = 0
         self.compiler = space.createcompiler()
         self.profilefunc = None
         self.w_profilefuncarg = None
         self.thread_disappeared = False   # might be set to True after os.fork()
-        self.w_asyncgen_firstiter_fn = None
-        self.w_asyncgen_finalizer_fn = None
-        self.contextvar_context = None
-        self.coroutine_origin_tracking_depth = 0
 
     @staticmethod
     def _mark_thread_disappeared(space):
@@ -74,8 +64,6 @@ class ExecutionContext(object):
         return frame
 
     def enter(self, frame):
-        if self.space.reverse_debugging:
-            self._revdb_enter(frame)
         frame.f_backref = self.topframeref
         self.topframeref = jit.virtual_ref(frame)
 
@@ -96,8 +84,6 @@ class ExecutionContext(object):
                 # be accessed also later
                 frame_vref()
             jit.virtual_ref_finish(frame_vref, frame)
-            if self.space.reverse_debugging:
-                self._revdb_leave(got_exception)
 
     # ________________________________________________________________
 
@@ -167,8 +153,6 @@ class ExecutionContext(object):
         Like bytecode_trace() but doesn't invoke any other events besides the
         trace function.
         """
-        if self.space.reverse_debugging:
-            self._revdb_potential_stop_point(frame)
         if (frame.get_w_f_trace() is None or self.is_tracing or
             self.gettrace() is None):
             return
@@ -178,8 +162,11 @@ class ExecutionContext(object):
     def run_trace_func(self, frame):
         code = frame.pycode
         d = frame.getorcreatedebug()
-        line = d.f_lineno
-        if not (d.instr_lb <= frame.last_instr < d.instr_ub):
+        if d.instr_lb <= frame.last_instr < d.instr_ub:
+            if frame.last_instr < d.instr_prev_plus_one:
+                # We jumped backwards in the same line.
+                self._trace(frame, 'line', self.space.w_None)
+        else:
             size = len(code.co_lnotab) / 2
             addr = 0
             line = code.co_firstlineno
@@ -192,10 +179,8 @@ class ExecutionContext(object):
                 addr += c
                 if c:
                     d.instr_lb = addr
-                line_offset = ord(lineno[p + 1])
-                if line_offset >= 0x80:
-                    line_offset -= 0x100
-                line += line_offset
+
+                line += ord(lineno[p + 1])
                 p += 2
                 size -= 1
 
@@ -212,14 +197,9 @@ class ExecutionContext(object):
             else:
                 d.instr_ub = sys.maxint
 
-        # when we are at a start of a line, or executing a backwards jump,
-        # produce a line event
-        if d.instr_lb == frame.last_instr or frame.last_instr < d.instr_prev_plus_one:
-            d.f_lineno = line
-            if d.f_trace_lines:
+            if d.instr_lb == frame.last_instr: # At start of line!
+                d.f_lineno = line
                 self._trace(frame, 'line', self.space.w_None)
-        if d.f_trace_opcodes:
-            self._trace(frame, 'opcode', self.space.w_None)
 
         d.instr_prev_plus_one = frame.last_instr + 1
 
@@ -240,40 +220,39 @@ class ExecutionContext(object):
             self._trace(frame, 'exception', None, operationerr)
         #operationerr.print_detailed_traceback(self.space)
 
-    @jit.unroll_safe
-    def sys_exc_info(self):
+    @jit.dont_look_inside
+    @specialize.arg(1)
+    def sys_exc_info(self, for_hidden=False):
         """Implements sys.exc_info().
         Return an OperationError instance or None.
-        Returns the "top-most" exception in the stack.
+
+        Ignores exceptions within hidden frames unless for_hidden=True
+        is specified.
 
         # NOTE: the result is not the wrapped sys.exc_info() !!!
 
         """
-        return self.sys_exc_operror
+        return self.gettopframe()._exc_info_unroll(self.space, for_hidden)
 
     def set_sys_exc_info(self, operror):
-        self.sys_exc_operror = operror
+        frame = self.gettopframe_nohidden()
+        if frame:     # else, the exception goes nowhere and is lost
+            frame.last_exception = operror
 
-    def set_sys_exc_info3(self, w_type, w_value, w_traceback):
-        from pypy.interpreter import pytraceback
-
-        space = self.space
-        if space.is_none(w_value):
-            operror = None
-        else:
-            tb = None
-            if not space.is_none(w_traceback):
-                try:
-                    tb = pytraceback.check_traceback(space, w_traceback, '?')
-                except OperationError:    # catch and ignore bogus objects
-                    pass
-            operror = OperationError(w_type, w_value, tb)
-        self.set_sys_exc_info(operror)
+    def clear_sys_exc_info(self):
+        # Find the frame out of which sys_exc_info() would return its result,
+        # and hack this frame's last_exception to become the cleared
+        # OperationError (which is different from None!).
+        frame = self.gettopframe_nohidden()
+        while frame:
+            if frame.last_exception is not None:
+                frame.last_exception = get_cleared_operation_error(self.space)
+                break
+            frame = self.getnextframe_nohidden(frame)
 
     @jit.dont_look_inside
     def settrace(self, w_func):
         """Set the global trace function."""
-        # self.space.audit("sys.settrace", [])
         if self.space.is_w(w_func, self.space.w_None):
             self.w_tracefunc = None
         else:
@@ -288,7 +267,6 @@ class ExecutionContext(object):
 
     def setprofile(self, w_func):
         """Set the global trace function."""
-        # self.space.audit("sys.setprofile", [])
         if self.space.is_w(w_func, self.space.w_None):
             self.profilefunc = None
             self.w_profilefuncarg = None
@@ -343,10 +321,9 @@ class ExecutionContext(object):
 
         if w_callback is not None and event != "leaveframe":
             if operr is not None:
-                operr.normalize_exception(space)
                 w_value = operr.get_w_value(space)
                 w_arg = space.newtuple([operr.w_type, w_value,
-                                        operr.get_w_traceback(space)])
+                                     space.wrap(operr.get_traceback())])
 
             d = frame.getorcreatedebug()
             if d.w_locals is not None:
@@ -354,17 +331,12 @@ class ExecutionContext(object):
                 # if it does not exist yet and the tracer accesses it via
                 # frame.f_locals, it is filled by PyFrame.getdictscope
                 frame.fast2locals()
-            prev_line_tracing = d.is_in_line_tracing
             self.is_tracing += 1
             try:
-                if event == 'line':
-                    d.is_in_line_tracing = True
                 try:
-                    w_result = space.call_function(w_callback, frame, space.newtext(event), w_arg)
+                    w_result = space.call_function(w_callback, space.wrap(frame), space.wrap(event), w_arg)
                     if space.is_w(w_result, space.w_None):
-                        # bug-to-bug compatibility with CPython
-                        # http://bugs.python.org/issue11992
-                        pass   #d.w_f_trace = None
+                        d.w_f_trace = None
                     else:
                         d.w_f_trace = w_result
                 except:
@@ -373,7 +345,6 @@ class ExecutionContext(object):
                     raise
             finally:
                 self.is_tracing -= 1
-                d.is_in_line_tracing = prev_line_tracing
                 if d.w_locals is not None:
                     frame.locals2fast()
 
@@ -386,6 +357,7 @@ class ExecutionContext(object):
                     event == 'c_exception'):
                 return
 
+            last_exception = frame.last_exception
             if event == 'leaveframe':
                 event = 'return'
 
@@ -401,6 +373,7 @@ class ExecutionContext(object):
                     raise
 
             finally:
+                frame.last_exception = last_exception
                 self.is_tracing -= 1
 
     def checksignals(self):
@@ -410,21 +383,6 @@ class ExecutionContext(object):
         if self.space.check_signal_action is not None:
             self.space.check_signal_action.perform(self, None)
 
-    def _revdb_enter(self, frame):
-        # moved in its own function for the import statement
-        from pypy.interpreter.reverse_debugging import enter_call
-        enter_call(self.topframeref(), frame)
-
-    def _revdb_leave(self, got_exception):
-        # moved in its own function for the import statement
-        from pypy.interpreter.reverse_debugging import leave_call
-        leave_call(self.topframeref(), got_exception)
-
-    def _revdb_potential_stop_point(self, frame):
-        # moved in its own function for the import statement
-        from pypy.interpreter.reverse_debugging import potential_stop_point
-        potential_stop_point(frame)
-
     def _freeze_(self):
         raise Exception("ExecutionContext instances should not be seen during"
                         " translation.  Now is a good time to inspect the"
@@ -444,7 +402,7 @@ class AbstractActionFlag(object):
         self._periodic_actions = []
         self._nonperiodic_actions = []
         self.has_bytecode_counter = False
-        self._fired_actions_reset()
+        self.fired_actions = None
         # the default value is not 100, unlike CPython 2.7, but a much
         # larger value, because we use a technique that not only allows
         # but actually *forces* another thread to run whenever the counter
@@ -456,31 +414,15 @@ class AbstractActionFlag(object):
         """Request for the action to be run before the next opcode."""
         if not action._fired:
             action._fired = True
-            self._fired_actions_append(action)
+            if self.fired_actions is None:
+                self.fired_actions = []
+            self.fired_actions.append(action)
             # set the ticker to -1 in order to force action_dispatcher()
             # to run at the next possible bytecode
             self.reset_ticker(-1)
 
-    def _fired_actions_reset(self):
-        # linked list of actions. We cannot use a normal RPython list because
-        # we want AsyncAction.fire() to be marked as @rgc.collect: this way,
-        # we can call it from e.g. GcHooks or cpyext's dealloc_trigger.
-        self._fired_actions_first = None
-        self._fired_actions_last = None
-
-    @rgc.no_collect
-    def _fired_actions_append(self, action):
-        assert action._next is None
-        if self._fired_actions_first is None:
-            self._fired_actions_first = action
-            self._fired_actions_last = action
-        else:
-            self._fired_actions_last._next = action
-            self._fired_actions_last = action
-
-    @not_rpython
     def register_periodic_action(self, action, use_bytecode_counter):
-        """
+        """NOT_RPYTHON:
         Register the PeriodicAsyncAction action to be called whenever the
         tick counter becomes smaller than 0.  If 'use_bytecode_counter' is
         True, make sure that we decrease the tick counter at every bytecode.
@@ -522,26 +464,19 @@ class AbstractActionFlag(object):
                 action.perform(ec, frame)
 
             # nonperiodic actions
-            action = self._fired_actions_first
-            if action:
-                self._fired_actions_reset()
+            list = self.fired_actions
+            if list is not None:
+                self.fired_actions = None
                 # NB. in case there are several actions, we reset each
                 # 'action._fired' to false only when we're about to call
                 # 'action.perform()'.  This means that if
                 # 'action.fire()' happens to be called any time before
                 # the corresponding perform(), the fire() has no
                 # effect---which is the effect we want, because
-                # perform() will be called anyway.  All such pending
-                # actions with _fired == True are still inside the old
-                # chained list.  As soon as we reset _fired to False,
-                # we also reset _next to None and we are ready for
-                # another fire().
-                while action is not None:
-                    next_action = action._next
-                    action._next = None
+                # perform() will be called anyway.
+                for action in list:
                     action._fired = False
                     action.perform(ec, frame)
-                    action = next_action
 
         self.action_dispatcher = action_dispatcher
 
@@ -574,12 +509,10 @@ class AsyncAction(object):
     to occur between two opcodes, not at a completely random time.
     """
     _fired = False
-    _next = None
 
     def __init__(self, space):
         self.space = space
 
-    @rgc.no_collect
     def fire(self):
         """Request for the action to be run before the next opcode.
         The action must have been registered at space initalization time."""
@@ -614,8 +547,6 @@ class UserDelAction(AsyncAction):
 
     @jit.dont_look_inside
     def _run_finalizers(self):
-        # called by perform() when we have to "perform" this action,
-        # and also directly at the end of gc.collect).
         while True:
             w_obj = self.space.finalizer_queue.next_dead()
             if w_obj is None:
@@ -654,14 +585,9 @@ class UserDelAction(AsyncAction):
             if self.gc_disabled(w_obj):
                 return
             try:
-                w_impl = space.get(w_del, w_obj)
+                space.get_and_call_function(w_del, w_obj)
             except Exception as e:
                 report_error(space, e, "method __del__ of ", w_obj)
-            else:
-                try:
-                    space.call_function(w_impl)
-                except Exception as e:
-                    report_error(space, e, '', w_del)
 
         # Call the RPython-level _finalize_() method.
         try:
@@ -679,13 +605,13 @@ def report_error(space, e, where, w_obj):
         msg = ("RPython exception %s in %s<%s at 0x%s> ignored\n" % (
                    str(e), where, space.type(w_obj).name, addrstring))
         space.call_method(space.sys.get('stderr'), 'write',
-                          space.newtext(msg))
+                          space.wrap(msg))
 
 
 def make_finalizer_queue(W_Root, space):
     """Make a FinalizerQueue subclass which responds to GC finalizer
     events by 'firing' the UserDelAction class above.  It does not
-    directly fetches the objects to finalize at all; they stay in the
+    directly fetches the objects to finalize at all; they stay in the 
     GC-managed queue, and will only be fetched by UserDelAction
     (between bytecodes)."""
 
