@@ -1,4 +1,3 @@
-# coding: utf-8
 import sys
 
 from rpython.tool.udir import udir
@@ -28,77 +27,47 @@ class AppTestBuiltinApp:
         else:
             cls.w_safe_runtimerror = space.wrap(sys.version_info < (2, 6))
 
+        emptyfile = udir.join('emptyfile.py')
+        emptyfile.write('')
+        nullbytes = udir.join('nullbytes.py')
+        nullbytes.write('#abc\x00def\n')
+        nonexistent = udir.join('builtins-nonexistent')
+        cls.w_emptyfile = space.wrap(str(emptyfile))
+        cls.w_nullbytes = space.wrap(str(nullbytes))
+        cls.w_nonexistent = space.wrap(str(nonexistent))
+
     def test_builtin_names(self):
-        import builtins as __builtin__
-        assert __builtin__.bytes is bytes
+        import __builtin__
+        assert __builtin__.None is None
+        assert __builtin__.False is False
+        assert __builtin__.True is True
+
+        assert __builtin__.buffer is buffer
+        assert __builtin__.bytes is str
         assert __builtin__.dict is dict
         assert __builtin__.memoryview is memoryview
-        assert not hasattr(__builtin__, 'buffer')
 
     def test_bytes_alias(self):
-        assert bytes is not str
-        assert isinstance(eval("b'hi'"), bytes)
-
-    def test_eval_adds_builtins(self):
-        d = {}
-        eval('1', d)
-        assert "__builtins__" in d
+        assert bytes is str
+        assert isinstance(eval("b'hi'"), str)
 
     def test_import(self):
-        m = __import__('sys')
-        assert m.__name__ == "sys"
+        m = __import__('pprint')
+        assert m.pformat({}) == '{}'
+        assert m.__name__ == "pprint"
         raises(ImportError, __import__, 'spamspam')
         raises(TypeError, __import__, 1, 2, 3, 4)
-        print(__import__.__doc__)
-        assert __import__.__doc__.endswith('relative to the current module.')
 
-    def test_ascii(self):
-        assert ascii('') == '\'\''
-        assert ascii(0) == '0'
-        assert ascii(()) == '()'
-        assert ascii([]) == '[]'
-        assert ascii({}) == '{}'
-        a = []
-        a.append(a)
-        assert ascii(a) == '[[...]]'
-        a = {}
-        a[0] = a
-        assert ascii(a) == '{0: {...}}'
-        # Advanced checks for unicode strings
-        def _check_uni(s):
-            assert ascii(s) == repr(s)
-        _check_uni("'")
-        _check_uni('"')
-        _check_uni('"\'')
-        _check_uni('\0')
-        _check_uni('\r\n\t .')
-        # Unprintable non-ASCII characters
-        _check_uni('\x85')
-        _check_uni('\u1fff')
-        _check_uni('\U00012fff')
-        # Lone surrogates
-        _check_uni('\ud800')
-        _check_uni('\udfff')
-        # Issue #9804: surrogates should be joined even for printable
-        # wide characters (UCS-2 builds).
-        assert ascii('\U0001d121') == "'\\U0001d121'"
-        # another buggy case
-        x = ascii("\U00012fff")
-        assert x == r"'\U00012fff'"
-        # All together
-        s = "'\0\"\n\r\t abcd\x85é\U00012fff\uD800\U0001D121xxx."
-        assert ascii(s) == \
-            r"""'\'\x00"\n\r\t abcd\x85\xe9\U00012fff\ud800\U0001d121xxx.'"""
+    def test_chr(self):
+        assert chr(65) == 'A'
+        raises(ValueError, chr, -1)
+        raises(TypeError, chr, 'a')
 
     def test_bin(self):
-        class Foo:
-            def __index__(self):
-                return 4
         assert bin(0) == "0b0"
         assert bin(-1) == "-0b1"
-        assert bin(2) == "0b10"
-        assert bin(-2) == "-0b10"
-        assert bin(Foo()) == "0b100"
+        assert bin(2L) == "0b10"
+        assert bin(-2L) == "-0b10"
         raises(TypeError, bin, 0.)
         class C(object):
             def __index__(self):
@@ -108,46 +77,38 @@ class AppTestBuiltinApp:
             def __int__(self):
                 return 42
         exc = raises(TypeError, bin, D())
-        assert "integer" in str(exc.value)
+        assert "index" in exc.value.message
 
-    def test_oct(self):
-        class Foo:
-            def __index__(self):
-                return 4
-        assert oct(0) == "0o0"
-        assert oct(-1) == "-0o1"
-        assert oct(8) == "0o10"
-        assert oct(-8) == "-0o10"
-        assert oct(Foo()) == "0o4"
-        raises(TypeError, oct, 0.)
-
-    def test_hex(self):
-        class Foo:
-            def __index__(self):
-                return 4
-        assert hex(0) == "0x0"
-        assert hex(-1) == "-0x1"
-        assert hex(16) == "0x10"
-        assert hex(-16) == "-0x10"
-        assert hex(Foo()) == "0x4"
-        raises(TypeError, hex, 0.)
-
-    def test_chr(self):
+    def test_unichr(self):
         import sys
-        assert chr(65) == 'A'
-        assert type(str(65)) is str
-        assert chr(0x9876) == '\u9876'
+        assert unichr(65) == u'A'
+        assert type(unicode(65)) is unicode
+        assert unichr(0x9876) == u'\u9876'
         if sys.maxunicode > 0xFFFF:
-            assert chr(sys.maxunicode) == '\U0010FFFF'
+            assert unichr(sys.maxunicode) == u'\U0010FFFF'
+            assert unichr(0x10000) == u'\U00010000'
         else:
-            assert chr(sys.maxunicode) == '\uFFFF'
-        assert chr(0x00010000) == '\U00010000'
-        assert chr(0x0010ffff) == '\U0010FFFF'
-        raises(ValueError, chr, -1)
+            assert unichr(sys.maxunicode) == u'\uFFFF'
+        raises(ValueError, unichr, -1)
+        raises(ValueError, unichr, sys.maxunicode+1)
+
+    def test_intern(self):
+        raises(TypeError, intern)
+        raises(TypeError, intern, 1)
+        class S(str):
+            pass
+        raises(TypeError, intern, S("hello"))
+        s = "never interned before"
+        s2 = intern(s)
+        assert s == s2
+        s3 = s.swapcase()
+        assert s3 != s2
+        s4 = s3.swapcase()
+        assert intern(s4) is s2
 
     def test_globals(self):
         d = {"foo":"bar"}
-        exec("def f(): return globals()", d)
+        exec "def f(): return globals()" in d
         d2 = d["f"]()
         assert d2 is d
 
@@ -191,13 +152,12 @@ class AppTestBuiltinApp:
             def __getitem__(self, item):
                 raise KeyError(item)
             def keys(self):
-                return 'abcd'    # not a list!
-        names = eval("dir()", {}, C())
-        assert names == ['a', 'b', 'c', 'd']
+                return 'a'    # not a list!
+        raises(TypeError, eval, "dir()", {}, C())
 
     def test_dir_broken_module(self):
-        import sys
-        class Foo(type(sys)):
+        import types
+        class Foo(types.ModuleType):
             __dict__ = 8
         raises(TypeError, dir, Foo("foo"))
 
@@ -223,28 +183,27 @@ class AppTestBuiltinApp:
                 return 42
         f = Foo()
         raises(TypeError, dir, f)
-        import sys
-        class Foo(type(sys)):
+        import types
+        class Foo(types.ModuleType):
             def __dir__(self):
                 return ["blah"]
         assert dir(Foo("a_mod")) == ["blah"]
 
     def test_dir_custom_lookup(self):
-        """
         class M(type):
             def __dir__(self, *args): return ["14"]
-        class X(metaclass=M):
-            pass
+        class X(object):
+            __metaclass__ = M
         x = X()
         x.__dir__ = lambda x: ["14"]
         assert dir(x) != ["14"]
-        """
 
     def test_format(self):
         assert format(4) == "4"
         assert format(10, "o") == "12"
         assert format(10, "#o") == "0o12"
         assert format("hi") == "hi"
+        assert isinstance(format(4, u""), unicode)
 
     def test_vars(self):
         def f():
@@ -267,96 +226,16 @@ class AppTestBuiltinApp:
                 return 42
         assert sum([Foo()], None) == 42
 
-    def test_sum_fast_path(self):
-        # Fast paths for expected behaviour
-        start = []
-        assert sum([[1, 2], [3]], start) == [1, 2, 3]
-        assert start == []
-
-        start = [1, 2]
-        assert sum([[3]], start) == [1, 2, 3]
-        assert start == [1, 2]
-
-        assert sum([(1, 2), (3,)], ()) == (1, 2, 3)
-        assert sum([(3,)], (1, 2)) == (1, 2, 3)
-        assert sum(([x + 1] for x in range(3)), []) == [1, 2, 3]
-
-    def test_sum_empty_edge_cases(self):
-        assert sum([], []) == []
-        assert sum(iter([]), []) == []
-        assert sum([], ()) == ()
-        start = []
-        assert sum([], start) is start
-        assert sum([[]], start) is not start
-
-    def test_sum_type_errors(self):
-        with raises(TypeError):
-            sum([[]], ())
-        with raises(TypeError):
-            sum([()], [])
-        with raises(TypeError):
-            sum([[], [], ()], [])
-
-    def test_sum_strange_objects(self):
-        # All that follows should be rare, but needs care
-        class TupleTail(object):
-            def __radd__(self, other):
-                assert isinstance(other, tuple)
-                return other[1:]
-
-        strange_seq = [(1, 2), (3, 4), TupleTail(), (5,)]
-        assert sum(strange_seq, (2, 3, 4, 5))
-        assert sum(iter(strange_seq), (2, 3, 4, 5))
-
-        class NotAList(list):
-            def __add__(self, _):
-                return "!"
-
-            def __radd__(self, _):
-                return "?"
-
-            def __iadd__(self, _):
-                raise RuntimeError(
-                    "Calling __iadd__ breaks CPython compatability"
-                )
-
-        assert sum([[1]], NotAList()) == "!"
-        assert sum([[1], NotAList()], []) == "?"
-
-    def test_sum_first_object_edge_cases(self):
-        class X(list):
-            def __radd__(self, other):
-                return Y()
-
-        class Y(object):
-            calls = []
-
-            def __add__(self, other):
-                Y.calls.append("add")
-
-            def __iadd__(self, other):
-                Y.calls.append("iadd")
-
-        assert sum([X(), []], []) is None
-        assert Y.calls == ["add"]
-
-        class Z(tuple):
-            def __radd__(self, other):
-                return Y()
-
-        assert sum([Z(), []], []) is None
-        assert Y.calls == ["add", "add"]
-
     def test_type_selftest(self):
         assert type(type) is type
 
     def test_iter_sequence(self):
         raises(TypeError,iter,3)
         x = iter(['a','b','c'])
-        assert next(x) =='a'
-        assert next(x) =='b'
-        assert next(x) =='c'
-        raises(StopIteration, next, x)
+        assert x.next() =='a'
+        assert x.next() =='b'
+        assert x.next() =='c'
+        raises(StopIteration,x.next)
 
     def test_iter___iter__(self):
         # This test assumes that dict.keys() method returns keys in
@@ -365,7 +244,7 @@ class AppTestBuiltinApp:
         # it tests 4 calls to __iter__() in one assert.  It could
         # be modified if better granularity on the assert is required.
         mydict = {'a':1,'b':2,'c':3}
-        assert list(iter(mydict)) == list(mydict.keys())
+        assert list(iter(mydict)) ==mydict.keys()
 
     def test_iter_callable_sentinel(self):
         class count(object):
@@ -373,37 +252,34 @@ class AppTestBuiltinApp:
                 self.value = 0
             def __call__(self):
                 self.value += 1
-                if self.value > 10:
-                    raise StopIteration
                 return self.value
-        with raises(TypeError):
-            iter(3, 5)
-
+        # XXX Raising errors is quite slow --
+        #            uncomment these lines when fixed
+        #self.assertRaises(TypeError,iter,3,5)
+        #self.assertRaises(TypeError,iter,[],5)
+        #self.assertRaises(TypeError,iter,{},5)
         x = iter(count(),3)
-        assert next(x) ==1
-        assert next(x) ==2
-        raises(StopIteration, next, x)
-
-        # a case that runs till the end
-        assert list(iter(count(), 100)) == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
+        assert x.next() ==1
+        assert x.next() ==2
+        raises(StopIteration,x.next)
 
     def test_enumerate(self):
         import sys
         seq = range(2,4)
         enum = enumerate(seq)
-        assert next(enum) == (0, 2)
-        assert next(enum) == (1, 3)
-        raises(StopIteration, next, enum)
+        assert enum.next() == (0, 2)
+        assert enum.next() == (1, 3)
+        raises(StopIteration, enum.next)
         raises(TypeError, enumerate, 1)
         raises(TypeError, enumerate, None)
         enum = enumerate(range(5), 2)
-        assert list(enum) == list(zip(range(2, 7), range(5)))
+        assert list(enum) == zip(range(2, 7), range(5))
 
         enum = enumerate(range(2), 2**100)
         assert list(enum) == [(2**100, 0), (2**100+1, 1)]
 
-        enum = enumerate(range(2), sys.maxsize)
-        assert list(enum) == [(sys.maxsize, 0), (sys.maxsize+1, 1)]
+        enum = enumerate(range(2), sys.maxint)
+        assert list(enum) == [(sys.maxint, 0), (sys.maxint+1, 1)]
 
         raises(TypeError, enumerate, range(2), 5.5)
 
@@ -420,7 +296,7 @@ class AppTestBuiltinApp:
         class Counter:
             def __init__(self):
                 self.count = 0
-            def __next__(self):
+            def next(self):
                 self.count += 1
                 return self.count
         x = Counter()
@@ -428,75 +304,68 @@ class AppTestBuiltinApp:
         assert next(x) == 2
         assert next(x) == 3
 
-    def test_range_args(self):
-        raises(ValueError, range, 0, 1, 0)
+    def test_xrange_args(self):
+        raises(ValueError, xrange, 0, 1, 0)
 
-    def test_range_repr(self):
-        assert repr(range(1)) == 'range(0, 1)'
-        assert repr(range(1,2)) == 'range(1, 2)'
-        assert repr(range(1,2,3)) == 'range(1, 2, 3)'
+    def test_xrange_repr(self):
+        assert repr(xrange(1)) == 'xrange(1)'
+        assert repr(xrange(1,2)) == 'xrange(1, 2)'
+        assert repr(xrange(1,2,3)) == 'xrange(1, 4, 3)'
 
-    def test_range_up(self):
-        x = range(2)
+    def test_xrange_up(self):
+        x = xrange(2)
         iter_x = iter(x)
-        assert next(iter_x) == 0
-        assert next(iter_x) == 1
-        raises(StopIteration, next, iter_x)
+        assert iter_x.next() == 0
+        assert iter_x.next() == 1
+        raises(StopIteration, iter_x.next)
 
-    def test_range_down(self):
-        x = range(4,2,-1)
+    def test_xrange_down(self):
+        x = xrange(4,2,-1)
 
         iter_x = iter(x)
-        assert next(iter_x) == 4
-        assert next(iter_x) == 3
-        raises(StopIteration, next, iter_x)
+        assert iter_x.next() == 4
+        assert iter_x.next() == 3
+        raises(StopIteration, iter_x.next)
 
-    def test_range_has_type_identity(self):
-        assert type(range(1)) == type(range(1))
+    def test_xrange_has_type_identity(self):
+        assert type(xrange(1)) == type(xrange(1))
 
-    def test_range_len(self):
-        x = range(33)
+    def test_xrange_len(self):
+        x = xrange(33)
         assert len(x) == 33
-        exc = raises(TypeError, range, 33.2)
+        exc = raises(TypeError, xrange, 33.2)
         assert "integer" in str(exc.value)
-        x = range(33,0,-1)
+        x = xrange(33,0,-1)
         assert len(x) == 33
-        x = range(33,0)
+        x = xrange(33,0)
         assert len(x) == 0
-        exc = raises(TypeError, range, 33, 0.2)
+        exc = raises(TypeError, xrange, 33, 0.2)
         assert "integer" in str(exc.value)
-        x = range(0,33)
+        x = xrange(0,33)
         assert len(x) == 33
-        x = range(0,33,-1)
+        x = xrange(0,33,-1)
         assert len(x) == 0
-        x = range(0,33,2)
+        x = xrange(0,33,2)
         assert len(x) == 17
-        x = range(0,32,2)
+        x = xrange(0,32,2)
         assert len(x) == 16
 
-    def test_range_indexing(self):
-        x = range(0,33,2)
+    def test_xrange_indexing(self):
+        x = xrange(0,33,2)
         assert x[7] == 14
         assert x[-7] == 20
         raises(IndexError, x.__getitem__, 17)
         raises(IndexError, x.__getitem__, -18)
-        assert list(x.__getitem__(slice(0,3,1))) == [0, 2, 4]
+        raises(TypeError, x.__getitem__, slice(0,3,1))
 
-    def test_range_bad_args(self):
-        raises(TypeError, range, '1')
-        raises(TypeError, range, None)
-        raises(TypeError, range, 3+2j)
-        raises(TypeError, range, 1, '1')
-        raises(TypeError, range, 1, 3+2j)
-        raises(TypeError, range, 1, 2, '1')
-        raises(TypeError, range, 1, 2, 3+2j)
-
-    def test_range_bool(self):
-        import sys
-        a = range(-sys.maxsize, sys.maxsize)
-        assert bool(a) is True
-        b = range(10, 0)
-        assert bool(b) is False
+    def test_xrange_bad_args(self):
+        raises(TypeError, xrange, '1')
+        raises(TypeError, xrange, None)
+        raises(TypeError, xrange, 3+2j)
+        raises(TypeError, xrange, 1, '1')
+        raises(TypeError, xrange, 1, 3+2j)
+        raises(TypeError, xrange, 1, 2, '1')
+        raises(TypeError, xrange, 1, 2, 3+2j)
 
     def test_sorted(self):
         l = []
@@ -516,10 +385,6 @@ class AppTestBuiltinApp:
         assert sorted_l is not l
         assert sorted_l == ['C', 'b', 'a']
         raises(TypeError, sorted, [], reverse=None)
-        raises(TypeError, sorted, [], None)
-
-    def test_sorted_posonlyarg(self):
-        raises(TypeError, sorted, iterable=[])
 
     def test_reversed_simple_sequences(self):
         l = range(5)
@@ -537,11 +402,58 @@ class AppTestBuiltinApp:
         obj = SomeClass()
         assert reversed(obj) == 42
 
+
+    def test_cmp(self):
+        assert cmp(9,9) == 0
+        assert cmp(0,9) < 0
+        assert cmp(9,0) > 0
+        assert cmp("abc", 12) != 0
+        assert cmp(u"abc", 12) != 0
+
+    def test_cmp_more(self):
+        class C(object):
+            def __eq__(self, other):
+                return True
+            def __cmp__(self, other):
+                raise RuntimeError
+        c1 = C()
+        c2 = C()
+        raises(RuntimeError, cmp, c1, c2)
+
+    def test_cmp_cyclic(self):
+        if not self.sane_lookup:
+            skip("underlying Python implementation has insane dict lookup")
+        if not self.safe_runtimerror:
+            skip("underlying Python may raise random exceptions on stack ovf")
+        a = []; a.append(a)
+        b = []; b.append(b)
+        from UserList import UserList
+        c = UserList(); c.append(c)
+        raises(RuntimeError, cmp, a, b)
+        raises(RuntimeError, cmp, b, c)
+        raises(RuntimeError, cmp, c, a)
+        raises(RuntimeError, cmp, a, c)
+        # okay, now break the cycles
+        a.pop(); b.pop(); c.pop()
+
+    def test_coerce(self):
+        assert coerce(1, 2)    == (1, 2)
+        assert coerce(1L, 2L)  == (1L, 2L)
+        assert coerce(1, 2L)   == (1L, 2L)
+        assert coerce(1L, 2)   == (1L, 2L)
+        assert coerce(1, 2.0)  == (1.0, 2.0)
+        assert coerce(1.0, 2L) == (1.0, 2.0)
+        assert coerce(1L, 2.0) == (1.0, 2.0)
+        raises(TypeError,coerce, 1    , 'a')
+        raises(TypeError,coerce, u'a' , 'a')
+
     def test_return_None(self):
         class X(object): pass
         x = X()
         assert setattr(x, 'x', 11) == None
         assert delattr(x, 'x') == None
+        # To make this test, we need autopath to work in application space.
+        assert execfile(self.emptyfile) == None
 
     def test_divmod(self):
         assert divmod(15,10) ==(1,5)
@@ -653,30 +565,23 @@ class AppTestBuiltinApp:
         assert hasattr(x, '__class__') is True
         assert hasattr(x, 'foo') is True
         assert hasattr(x, 'bar') is False
-        raises(TypeError, "hasattr(x, 'abc')")
-        raises(TypeError, "hasattr(x, 'bac')")
+        assert hasattr(x, 'abc') is False    # CPython compliance
+        assert hasattr(x, 'bac') is False    # CPython compliance
         raises(TypeError, hasattr, x, None)
         raises(TypeError, hasattr, x, 42)
-        assert hasattr(x, '\u5678') is False
+        raises(UnicodeError, hasattr, x, u'\u5678')  # cannot encode attr name
 
-    def test_hasattr_exception(self):
-        class X(object):
-            def __getattr__(self, name):
-                if name == 'foo':
-                    raise AttributeError
-                else:
-                    raise KeyError
-        x = X()
-        assert hasattr(x, 'foo') is False
-        raises(KeyError, "hasattr(x, 'bar')")
+    def test_execfile_args(self):
+        execfile(self.nullbytes) # works
+        raises(TypeError, execfile, self.nonexistent, {}, ())
 
     def test_print_function(self):
-        import builtins
+        import __builtin__
         import sys
-        import _io
-        pr = getattr(builtins, "print")
+        import StringIO
+        pr = getattr(__builtin__, "print")
         save = sys.stdout
-        out = sys.stdout = _io.StringIO()
+        out = sys.stdout = StringIO.StringIO()
         try:
             pr("Hello,", "person!")
             pr("2nd line", file=None)
@@ -685,156 +590,52 @@ class AppTestBuiltinApp:
         finally:
             sys.stdout = save
         assert out.getvalue() == "Hello, person!\n2nd line\n"
-        out = _io.StringIO()
+        out = StringIO.StringIO()
         pr("Hello,", "person!", file=out)
         assert out.getvalue() == "Hello, person!\n"
-        out = _io.StringIO()
+        out = StringIO.StringIO()
         pr("Hello,", "person!", file=out, end="")
         assert out.getvalue() == "Hello, person!"
-        out = _io.StringIO()
+        out = StringIO.StringIO()
         pr("Hello,", "person!", file=out, sep="X")
         assert out.getvalue() == "Hello,Xperson!\n"
-        out = _io.StringIO()
-        pr(b"Hello,", b"person!", file=out)
+        out = StringIO.StringIO()
+        pr(u"Hello,", u"person!", file=out)
         result = out.getvalue()
-        assert isinstance(result, str)
-        assert result == "b'Hello,' b'person!'\n"
-        out = _io.StringIO()
+        assert isinstance(result, unicode)
+        assert result == u"Hello, person!\n"
+        out = StringIO.StringIO()
         pr(None, file=out)
         assert out.getvalue() == "None\n"
-        out = sys.stdout = _io.StringIO()
-        try:
-            pr("amaury", file=None)
-        finally:
-            sys.stdout = save
-        assert out.getvalue() == "amaury\n"
-
-    def test_print_function2(self):
-        import builtins
-        import _io
-        class MyStr(str):
-            def __str__(self):
-                return "sqlalchemy"
-        out = _io.StringIO()
-        s = MyStr('A')
-        pr = getattr(builtins, 'print')
-        pr(s, file=out)
-        pr(str(s), file=out)
-        assert out.getvalue() == "sqlalchemy\nsqlalchemy\n"
 
     def test_print_exceptions(self):
-        import builtins
-        pr = getattr(builtins, "print")
+        import __builtin__
+        pr = getattr(__builtin__, "print")
         raises(TypeError, pr, x=3)
         raises(TypeError, pr, end=3)
         raises(TypeError, pr, sep=42)
 
     def test_round(self):
-        assert round(11.234) == 11
-        assert type(round(11.234)) is int
-        assert round(11.234, -1) == 10
-        assert type(round(11.234, -1)) is float
-        assert round(11.234, 0) == 11
+        assert round(11.234) == 11.0
+        assert round(11.234, -1) == 10.0
+        assert round(11.234, 0) == 11.0
         assert round(11.234, 1) == 11.2
         #
         assert round(5e15-1) == 5e15-1
         assert round(5e15) == 5e15
         assert round(-(5e15-1)) == -(5e15-1)
         assert round(-5e15) == -5e15
-        assert round(5e15/2) == 5e15/2
-        assert round((5e15+1)/2) == 5e15/2
-        assert round((5e15-1)/2) == 5e15/2
         #
         inf = 1e200 * 1e200
-        raises(OverflowError, round, inf)
-        raises(OverflowError, round, -inf)
+        assert round(inf) == inf
+        assert round(-inf) == -inf
         nan = inf / inf
-        raises(ValueError, round, nan)
+        assert repr(round(nan)) == repr(nan)
+        #
         raises(OverflowError, round, 1.6e308, -308)
         #
         assert round(562949953421312.5, 1) == 562949953421312.5
         assert round(56294995342131.5, 3) == 56294995342131.5
-        #
-        for i in range(-10, 10):
-            expected = i + (i % 2)
-            assert round(i + 0.5) == round(i + 0.5, 0) == expected
-            x = i * 10 + 5
-            assert round(x, -1) == round(float(x), -1) == expected * 10
-
-        assert round(0.0) == 0.0
-        assert type(round(0.0)) == int
-        assert round(1.0) == 1.0
-        assert round(10.0) == 10.0
-        assert round(1000000000.0) == 1000000000.0
-        assert round(1e20) == 1e20
-
-        assert round(-1.0) == -1.0
-        assert round(-10.0) == -10.0
-        assert round(-1000000000.0) == -1000000000.0
-        assert round(-1e20) == -1e20
-
-        assert round(0.1) == 0.0
-        assert round(1.1) == 1.0
-        assert round(10.1) == 10.0
-        assert round(1000000000.1) == 1000000000.0
-
-        assert round(-1.1) == -1.0
-        assert round(-10.1) == -10.0
-        assert round(-1000000000.1) == -1000000000.0
-
-        assert round(0.9) == 1.0
-        assert round(9.9) == 10.0
-        assert round(999999999.9) == 1000000000.0
-
-        assert round(-0.9) == -1.0
-        assert round(-9.9) == -10.0
-        assert round(-999999999.9) == -1000000000.0
-
-        assert round(-8.0, -1) == -10.0
-        assert type(round(-8.0, -1)) == float
-
-        assert type(round(-8.0, 0)) == float
-        assert type(round(-8.0, 1)) == float
-
-        # Check even / odd rounding behaviour
-        assert round(5.5) == 6
-        assert round(6.5) == 6
-        assert round(-5.5) == -6
-        assert round(-6.5) == -6
-
-        # Check behavior on ints
-        assert round(0) == 0
-        assert round(8) == 8
-        assert round(-8) == -8
-        assert type(round(0)) == int
-        assert type(round(-8, -1)) == int
-        assert type(round(-8, 0)) == int
-        assert type(round(-8, 1)) == int
-
-        assert round(number=-8.0, ndigits=-1) == -10.0
-        raises(TypeError, round)
-
-        # test generic rounding delegation for reals
-        class TestRound:
-            def __round__(self):
-                return 23
-
-        class TestNoRound:
-            pass
-
-        assert round(TestRound()) == 23
-
-        raises(TypeError, round, 1, 2, 3)
-        raises(TypeError, round, TestNoRound())
-
-        t = TestNoRound()
-        t.__round__ = lambda *args: args
-        raises(TypeError, round, t)
-        raises(TypeError, round, t, 0)
-
-        assert round(3, ndigits=None) == 3
-        assert round(3.0, ndigits=None) == 3
-        assert type(round(3.0, ndigits=None)) is int
 
     def test_vars_obscure_case(self):
         class C_get_vars(object):
@@ -843,21 +644,6 @@ class AppTestBuiltinApp:
             __dict__ = property(fget=getDict)
         assert vars(C_get_vars()) == {'a':2}
 
-    def test_len_negative_overflow(self):
-        import sys
-        class NegativeLen:
-            def __len__(self):
-                return -10
-        raises(ValueError, len, NegativeLen())
-        class HugeLen:
-            def __len__(self):
-                return sys.maxsize + 1
-        raises(OverflowError, len, HugeLen())
-        class HugeNegativeLen:
-            def __len__(self):
-                return -sys.maxsize-10
-        raises(ValueError, len, HugeNegativeLen())
-
 
 class AppTestGetattr:
     spaceconfig = {}
@@ -868,8 +654,9 @@ class AppTestGetattr:
         assert getattr(a, 'i') == 5
         raises(AttributeError, getattr, a, 'k')
         assert getattr(a, 'k', 42) == 42
-        raises(TypeError, getattr, a, b'i')
-        raises(TypeError, getattr, a, b'k', 42)
+        assert getattr(a, u'i') == 5
+        raises(AttributeError, getattr, a, u'k')
+        assert getattr(a, u'k', 42) == 42
 
     def test_getattr_typecheck(self):
         class A(object):
@@ -883,26 +670,45 @@ class AppTestGetattr:
         raises(TypeError, setattr, A(), 42, 'x')
         raises(TypeError, delattr, A(), 42)
 
-    def test_getattr_None(self):
-        import sys
-        if '__pypy__' not in sys.modules:
-            skip('CPython uses wrapper types for this')
-        class C:
-            def _m(self): pass
-        assert isinstance(getattr(type(None), '__eq__'), type(lambda: None))
-        assert isinstance(getattr(None, '__eq__'), type(C()._m))
 
-    def test_getattr_userobject(self):
-        class C:
-            def _m(self): pass
-        class A(object):
-            def __eq__(self, other):
-                pass
-        a = A()
-        assert isinstance(getattr(A, '__eq__'), type(lambda: None))
-        assert isinstance(getattr(a, '__eq__'), type(C()._m))
-        a.__eq__ = 42
-        assert a.__eq__ == 42
+class TestInternal:
+    def test_execfile(self, space):
+        fn = str(udir.join('test_execfile'))
+        f = open(fn, 'w')
+        print >>f, "i=42"
+        f.close()
 
-    def test_pow_kwarg(self):
-        assert pow(base=5, exp=2, mod=14) == 11
+        w_execfile = space.builtin.get("execfile")
+        w_dict = space.newdict()
+        space.call_function(w_execfile,
+            space.wrap(fn), w_dict, space.w_None)
+        w_value = space.getitem(w_dict, space.wrap('i'))
+        assert space.eq_w(w_value, space.wrap(42))
+
+    def test_execfile_different_lineendings(self, space):
+        from rpython.tool.udir import udir
+        d = udir.ensure('lineending', dir=1)
+        dos = d.join('dos.py')
+        f = dos.open('wb')
+        f.write("x=3\r\n\r\ny=4\r\n")
+        f.close()
+        space.appexec([space.wrap(str(dos))], """
+            (filename):
+                d = {}
+                execfile(filename, d)
+                assert d['x'] == 3
+                assert d['y'] == 4
+        """)
+
+        unix = d.join('unix.py')
+        f = unix.open('wb')
+        f.write("x=5\n\ny=6\n")
+        f.close()
+
+        space.appexec([space.wrap(str(unix))], """
+            (filename):
+                d = {}
+                execfile(filename, d)
+                assert d['x'] == 5
+                assert d['y'] == 6
+        """)
