@@ -6,13 +6,11 @@ from rpython.rlib import rtime
 
 
 class BuildersModule(MixedModule):
-    """ Module containing string and unicode builders """
-
     appleveldefs = {}
 
     interpleveldefs = {
         "StringBuilder": "interp_builders.W_StringBuilder",
-        "BytesBuilder": "interp_builders.W_BytesBuilder",
+        "UnicodeBuilder": "interp_builders.W_UnicodeBuilder",
     }
 
 class TimeModule(MixedModule):
@@ -36,8 +34,6 @@ class ThreadModule(MixedModule):
 
 
 class IntOpModule(MixedModule):
-    """ Module for integer operations that have two-complement overflow
-    behaviour instead of overflowing to longs """
     appleveldefs = {}
     interpleveldefs = {
         'int_add':         'interp_intop.int_add',
@@ -48,7 +44,6 @@ class IntOpModule(MixedModule):
         'int_lshift':      'interp_intop.int_lshift',
         'int_rshift':      'interp_intop.int_rshift',
         'uint_rshift':     'interp_intop.uint_rshift',
-        'int_mulmod':      'interp_intop.int_mulmod',
     }
 
 
@@ -59,24 +54,13 @@ class OsModule(MixedModule):
     }
 
 
-class PyPyDateTime(MixedModule):
-    appleveldefs = {}
-    interpleveldefs = {
-        'dateinterop': 'interp_pypydatetime.W_DateTime_Date',
-        'timeinterop'    : 'interp_pypydatetime.W_DateTime_Time',
-        'deltainterop'   : 'interp_pypydatetime.W_DateTime_Delta',
-    }
-
 class Module(MixedModule):
-    """ PyPy specific "magic" functions. A lot of them are experimental and
-    subject to change, many are internal. """
     appleveldefs = {
     }
 
     interpleveldefs = {
         'attach_gdb'                : 'interp_magic.attach_gdb',
         'internal_repr'             : 'interp_magic.internal_repr',
-        'objects_in_repr'           : 'interp_magic.objects_in_repr',
         'bytebuffer'                : 'bytebuffer.bytebuffer',
         'identity_dict'             : 'interp_identitydict.W_IdentityDict',
         'debug_start'               : 'interp_debug.debug_start',
@@ -84,34 +68,29 @@ class Module(MixedModule):
         'debug_stop'                : 'interp_debug.debug_stop',
         'debug_print_once'          : 'interp_debug.debug_print_once',
         'debug_flush'               : 'interp_debug.debug_flush',
-        'debug_read_timestamp'      : 'interp_debug.debug_read_timestamp',
-        'debug_get_timestamp_unit'  : 'interp_debug.debug_get_timestamp_unit',
         'builtinify'                : 'interp_magic.builtinify',
         'hidden_applevel'           : 'interp_magic.hidden_applevel',
+        'get_hidden_tb'             : 'interp_magic.get_hidden_tb',
         'lookup_special'            : 'interp_magic.lookup_special',
         'do_what_I_mean'            : 'interp_magic.do_what_I_mean',
+        'validate_fd'               : 'interp_magic.validate_fd',
         'resizelist_hint'           : 'interp_magic.resizelist_hint',
         'newlist_hint'              : 'interp_magic.newlist_hint',
         'add_memory_pressure'       : 'interp_magic.add_memory_pressure',
         'newdict'                   : 'interp_dict.newdict',
         'reversed_dict'             : 'interp_dict.reversed_dict',
-        'dict_popitem_first'        : 'interp_dict.dict_popitem_first',
-        'delitem_if_value_is'       : 'interp_dict.delitem_if_value_is',
-        'move_to_end'               : 'interp_dict.move_to_end',
         'strategy'                  : 'interp_magic.strategy',  # dict,set,list
+        'specialized_zip_2_lists'   : 'interp_magic.specialized_zip_2_lists',
         'set_debug'                 : 'interp_magic.set_debug',
         'locals_to_fast'            : 'interp_magic.locals_to_fast',
         'set_code_callback'         : 'interp_magic.set_code_callback',
+        'save_module_content_for_future_reload':
+                          'interp_magic.save_module_content_for_future_reload',
         'decode_long'               : 'interp_magic.decode_long',
         '_promote'                   : 'interp_magic._promote',
-        'normalize_exc'             : 'interp_magic.normalize_exc',
-        'StdErrPrinter'             : 'interp_stderrprinter.W_StdErrPrinter',
-        'side_effects_ok'           : 'interp_magic.side_effects_ok',
-        'stack_almost_full'         : 'interp_magic.stack_almost_full',
-        'fsencode'                  : 'interp_magic.fsencode',
-        'fsdecode'                  : 'interp_magic.fsdecode',
-        'pyos_inputhook'            : 'interp_magic.pyos_inputhook',
     }
+    if sys.platform == 'win32':
+        interpleveldefs['get_console_cp'] = 'interp_magic.get_console_cp'
 
     submodules = {
         "builders": BuildersModule,
@@ -119,7 +98,6 @@ class Module(MixedModule):
         "thread": ThreadModule,
         "intop": IntOpModule,
         "os": OsModule,
-        '_pypydatetime': PyPyDateTime,
     }
 
     def setup_after_space_initialization(self):
@@ -142,12 +120,8 @@ class Module(MixedModule):
                 raise
             else:
                 pass   # ok fine to ignore in this case
-
+        
         if self.space.config.translation.jit:
             features = detect_cpu.getcpufeatures(model)
             self.extra_interpdef('jit_backend_features',
                                     'space.wrap(%r)' % features)
-        if self.space.config.translation.reverse_debugger:
-            self.extra_interpdef('revdb_stop', 'interp_magic.revdb_stop')
-        else:
-            self.extra_interpdef('revdb_stop', 'space.w_None')
