@@ -66,70 +66,60 @@ class W_CData(W_Root):
             from pypy.module._cffi_backend import ctypestruct
             if isinstance(self.ctype, ctypestruct.W_CTypeStructOrUnion):
                 extra1 = ' &'
-        return self.space.newtext("<cdata '%s%s' %s>" % (
+        return self.space.wrap("<cdata '%s%s' %s>" % (
             self.ctype.name, extra1, extra2))
 
-    def bool(self):
+    def nonzero(self):
         with self as ptr:
             nonzero = self.ctype.nonzero(ptr)
-        return self.space.newbool(nonzero)
+        return self.space.wrap(nonzero)
 
     def int(self, space):
         with self as ptr:
             w_result = self.ctype.cast_to_int(ptr)
         return w_result
 
+    def long(self, space):
+        w_result = self.int(space)
+        space = self.space
+        if space.is_w(space.type(w_result), space.w_int):
+            w_result = space.newlong(space.int_w(w_result))
+        return w_result
+
     def float(self):
         with self as ptr:
             w_result = self.ctype.float(ptr)
         return w_result
 
-    def complex(self):
-        with self as ptr:
-            w_result = self.ctype.complex(ptr)
-        return w_result
-
     def len(self):
         from pypy.module._cffi_backend import ctypearray
         space = self.space
         if isinstance(self.ctype, ctypearray.W_CTypeArray):
-            return space.newint(self.get_array_length())
+            return space.wrap(self.get_array_length())
         raise oefmt(space.w_TypeError,
                     "cdata of type '%s' has no len()", self.ctype.name)
 
-    def _compare_mode(self, w_other):
-        from pypy.module._cffi_backend.ctypeprim import W_CTypePrimitive
-        zero = rffi.cast(lltype.Unsigned, 0)
-        self_is_ptr = not isinstance(self.ctype, W_CTypePrimitive)
-        other_is_ptr = (isinstance(w_other, W_CData) and
-                           not isinstance(w_other.ctype, W_CTypePrimitive))
-        if other_is_ptr and self_is_ptr:
-            with self as ptr1, w_other as ptr2:
-                ptr1 = rffi.cast(lltype.Unsigned, ptr1)
-                ptr2 = rffi.cast(lltype.Unsigned, ptr2)
-            return (0, ptr1, ptr2, None, None)
-        elif other_is_ptr or self_is_ptr:
-            return (-1, zero, zero, None, None)
-        else:
-            w_ob1 = self.convert_to_object()
-            if isinstance(w_other, W_CData):
-                w_ob2 = w_other.convert_to_object()
-            else:
-                w_ob2 = w_other
-            return (1, zero, zero, w_ob1, w_ob2)
-
     def _make_comparison(name):
         op = getattr(operator, name)
+        requires_ordering = name not in ('eq', 'ne')
         #
         def _cmp(self, w_other):
+            from pypy.module._cffi_backend.ctypeprim import W_CTypePrimitive
             space = self.space
-            mode, adr1, adr2, w_ob1, w_ob2 = self._compare_mode(w_other)
-            if mode == 0:
-                return space.newbool(op(adr1, adr2))
-            elif mode == 1:
-                return getattr(space, name)(w_ob1, w_ob2)
-            else:
+            if not isinstance(w_other, W_CData):
                 return space.w_NotImplemented
+
+            with self as ptr1, w_other as ptr2:
+                if requires_ordering:
+                    if (isinstance(self.ctype, W_CTypePrimitive) or
+                        isinstance(w_other.ctype, W_CTypePrimitive)):
+                        raise oefmt(space.w_TypeError,
+                                    "cannot do comparison on a primitive "
+                                    "cdata")
+                    ptr1 = rffi.cast(lltype.Unsigned, ptr1)
+                    ptr2 = rffi.cast(lltype.Unsigned, ptr2)
+                result = op(ptr1, ptr2)
+            return space.newbool(result)
         #
         return func_with_new_name(_cmp, name)
 
@@ -141,18 +131,13 @@ class W_CData(W_Root):
     ge = _make_comparison('ge')
 
     def hash(self):
-        from pypy.module._cffi_backend.ctypeprim import W_CTypePrimitive
-        if isinstance(self.ctype, W_CTypePrimitive):
-            w_ob = self.convert_to_object()
-            if not isinstance(w_ob, W_CData):
-                return self.space.hash(w_ob)
         ptr = self.unsafe_escaping_ptr()
         h = rffi.cast(lltype.Signed, ptr)
         # To hash pointers in dictionaries.  Assumes that h shows some
         # alignment (to 4, 8, maybe 16 bytes), so we use the following
         # formula to avoid the trailing bits being always 0.
         h = h ^ (h >> 4)
-        return self.space.newint(h)
+        return self.space.wrap(h)
 
     def getitem(self, w_index):
         space = self.space
@@ -247,10 +232,10 @@ class W_CData(W_Root):
         from pypy.module._cffi_backend import ctypeprim
         space = self.space
         if isinstance(ctitem, ctypeprim.W_CTypePrimitive) and ctitem.size == 1:
-            if space.isinstance_w(w_value, space.w_bytes):
+            if space.isinstance_w(w_value, space.w_str):
                 from rpython.rtyper.annlowlevel import llstr
                 from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw
-                value = space.bytes_w(w_value)
+                value = space.str_w(w_value)
                 if len(value) != length:
                     raise oefmt(space.w_ValueError,
                                 "need a string of length %d, got %d",
@@ -334,32 +319,21 @@ class W_CData(W_Root):
                         "pointer subtraction: the distance between the two "
                         "pointers is not a multiple of the item size")
                 diff //= itemsize
-            return space.newint(diff)
+            return space.wrap(diff)
         #
         return self._add_or_sub(w_other, -1)
 
-    def getcfield(self, w_attr, mode):
-        space = self.space
-        attr = space.text_w(w_attr)
-        try:
-            cfield = self.ctype.getcfield(attr)
-        except KeyError:
-            raise oefmt(space.w_AttributeError, "cdata '%s' has no field '%s'",
-                        self.ctype.name, attr)
-        if cfield is None:
-            raise oefmt(space.w_AttributeError,
-                        "cdata '%s' points to an opaque type: cannot %s fields",
-                        self.ctype.name, mode)
-        return cfield
+    def getcfield(self, w_attr):
+        return self.ctype.getcfield(self.space.str_w(w_attr))
 
     def getattr(self, w_attr):
-        cfield = self.getcfield(w_attr, mode="read")
+        cfield = self.getcfield(w_attr)
         with self as ptr:
             w_res = cfield.read(ptr, self)
         return w_res
 
     def setattr(self, w_attr, w_value):
-        cfield = self.getcfield(w_attr, mode="write")
+        cfield = self.getcfield(w_attr)
         with self as ptr:
             cfield.write(ptr, w_value)
 
@@ -403,13 +377,6 @@ class W_CData(W_Root):
         with self as ptr:
             misc.write_raw_float_data(ptr, source, self.ctype.size)
 
-    def write_raw_complex_data(self, real, imag):
-        with self as ptr:
-            halfsize = self.ctype.size >> 1
-            ptr2 = rffi.ptradd(ptr, halfsize)
-            misc.write_raw_float_data(ptr, real, halfsize)
-            misc.write_raw_float_data(ptr2, imag, halfsize)
-
     def convert_to_object(self):
         with self as ptr:
             w_obj = self.ctype.convert_to_object(ptr)
@@ -426,25 +393,17 @@ class W_CData(W_Root):
     def _sizeof(self):
         return self.ctype.size
 
-    def with_gc(self, w_destructor, size=0):
+    def with_gc(self, w_destructor):
         space = self.space
         if space.is_none(w_destructor):
             if isinstance(self, W_CDataGCP):
-                self.detach_destructor()
-                w_res = space.w_None
-            else:
-                raise oefmt(space.w_TypeError,
-                            "Can remove destructor only on a object "
-                            "previously returned by ffi.gc()")
-        else:
-            with self as ptr:
-                w_res = W_CDataGCP(space, ptr, self.ctype, self, w_destructor)
-        if size != 0:
-            if isinstance(w_res, W_CDataGCP):
-                rgc.add_memory_pressure(size, w_res)
-            else:
-                rgc.add_memory_pressure(size, self)
-        return w_res
+                self.w_destructor = None
+                return space.w_None
+            raise oefmt(space.w_TypeError,
+                        "Can remove destructor only on a object "
+                        "previously returned by ffi.gc()")
+        with self as ptr:
+            return W_CDataGCP(space, ptr, self.ctype, self, w_destructor)
 
     def unpack(self, length):
         from pypy.module._cffi_backend.ctypeptr import W_CTypePtrOrArray
@@ -460,8 +419,8 @@ class W_CData(W_Root):
         with self as ptr:
             if not ptr:
                 raise oefmt(space.w_RuntimeError,
-                            "cannot use unpack() on %R",
-                            self)
+                            "cannot use unpack() on %s",
+                            space.str_w(self.repr()))
             w_result = ctype.ctitem.unpack_ptr(ctype, ptr, length)
         return w_result
 
@@ -471,24 +430,11 @@ class W_CData(W_Root):
         if isinstance(ct, W_CTypePointer):
             ct = ct.ctitem
         lst = ct.cdata_dir()
-        return space.newlist([space.newtext(s) for s in lst])
+        return space.newlist([space.wrap(s) for s in lst])
 
     def get_structobj(self):
         return None
 
-    def enter_exit(self, exit_now):
-        raise oefmt(self.space.w_ValueError,
-            "only 'cdata' object from ffi.new(), ffi.gc(), ffi.from_buffer() "
-            "or ffi.new_allocator()() can be used with the 'with' keyword or "
-            "ffi.release()")
-
-    def descr_enter(self):
-        self.enter_exit(False)
-        return self
-
-    def descr_exit(self, args_w):
-        self.enter_exit(True)
-
 
 class W_CDataMem(W_CData):
     """This is used only by the results of cffi.cast('int', x)
@@ -541,33 +487,14 @@ class W_CDataNewOwning(W_CData):
     def get_structobj(self):
         return self
 
-    def enter_exit(self, exit_now):
-        from pypy.module._cffi_backend.ctypeptr import W_CTypePtrOrArray
-        if not isinstance(self.ctype, W_CTypePtrOrArray):
-            W_CData.enter_exit(self, exit_now)
-        elif exit_now:
-            self._do_exit()
-
-    def _do_exit(self):
-        raise NotImplementedError
-
 
 class W_CDataNewStd(W_CDataNewOwning):
     """Subclass using the standard allocator, lltype.malloc()/lltype.free()"""
-    _attrs_ = ['explicitly_freed']
-    explicitly_freed = False
+    _attrs_ = []
 
     @rgc.must_be_light_finalizer
     def __del__(self):
-        if not self.explicitly_freed:
-            lltype.free(self._ptr, flavor='raw')
-
-    def _do_exit(self):
-        if not self.explicitly_freed:
-            rgc.add_memory_pressure(-self._sizeof(), self)
-            self.explicitly_freed = True
-            rgc.may_ignore_finalizer(self)
-            lltype.free(self._ptr, flavor='raw')
+        lltype.free(self._ptr, flavor='raw')
 
 
 class W_CDataNewNonStd(W_CDataNewOwning):
@@ -575,16 +502,7 @@ class W_CDataNewNonStd(W_CDataNewOwning):
     _attrs_ = ['w_raw_cdata', 'w_free']
 
     def _finalize_(self):
-        if self.w_free is not None:
-            self.space.call_function(self.w_free, self.w_raw_cdata)
-
-    def _do_exit(self):
-        w_free = self.w_free
-        if w_free is not None:
-            rgc.add_memory_pressure(-self._sizeof(), self)
-            self.w_free = None
-            self.may_unregister_rpython_finalizer(self.space)
-            self.space.call_function(w_free, self.w_raw_cdata)
+        self.space.call_function(self.w_free, self.w_raw_cdata)
 
 
 class W_CDataPtrToStructOrUnion(W_CData):
@@ -614,12 +532,6 @@ class W_CDataPtrToStructOrUnion(W_CData):
         else:
             return None
 
-    def enter_exit(self, exit_now):
-        if exit_now:
-            structobj = self.structobj
-            if isinstance(structobj, W_CDataNewOwning):
-                structobj._do_exit()
-
 
 class W_CDataSliced(W_CData):
     """Subclass with an explicit length, for slices."""
@@ -653,44 +565,26 @@ class W_CDataHandle(W_CData):
 
     def _repr_extra(self):
         w_repr = self.space.repr(self.w_keepalive)
-        return "handle to %s" % (self.space.text_w(w_repr),)
+        return "handle to %s" % (self.space.str_w(w_repr),)
 
 
 class W_CDataFromBuffer(W_CData):
     _attrs_ = ['buf', 'length', 'w_keepalive']
-    _immutable_fields_ = ['buf', 'length']
+    _immutable_fields_ = ['buf', 'length', 'w_keepalive']
 
-    def __init__(self, space, cdata, length, ctype, buf, w_object):
+    def __init__(self, space, cdata, ctype, buf, w_object):
         W_CData.__init__(self, space, cdata, ctype)
         self.buf = buf
-        self.length = length
+        self.length = buf.getlength()
         self.w_keepalive = w_object
 
     def get_array_length(self):
         return self.length
 
-    def _sizeof(self):
-        from pypy.module._cffi_backend import ctypearray
-        ctype = self.ctype
-        if isinstance(ctype, ctypearray.W_CTypeArray):
-            return self.length * ctype.ctitem.size
-        else:
-            return W_CData._sizeof(self)
-
     def _repr_extra(self):
-        from pypy.module._cffi_backend import ctypearray
-        if self.w_keepalive is None:
-            return "buffer RELEASED"
-        obj_tp_name = self.space.type(self.w_keepalive).name
-        if isinstance(self.ctype, ctypearray.W_CTypeArray):
-            return "buffer len %d from '%s' object" % (self.length, obj_tp_name)
-        else:
-            return "buffer from '%s' object" % (obj_tp_name,)
-
-    def enter_exit(self, exit_now):
-        # for now, limited effect on PyPy
-        if exit_now:
-            self.w_keepalive = None
+        w_repr = self.space.repr(self.w_keepalive)
+        return "buffer len %d from '%s' object" % (
+            self.length, self.space.type(self.w_keepalive).name)
 
 
 class W_CDataGCP(W_CData):
@@ -705,36 +599,21 @@ class W_CDataGCP(W_CData):
         self.register_finalizer(space)
 
     def _finalize_(self):
-        self.invoke_finalizer()
-
-    def invoke_finalizer(self):
         w_destructor = self.w_destructor
         if w_destructor is not None:
             self.w_destructor = None
             self.space.call_function(w_destructor, self.w_original_cdata)
 
-    def detach_destructor(self):
-        self.w_destructor = None
-        self.may_unregister_rpython_finalizer(self.space)
-
-    def enter_exit(self, exit_now):
-        if exit_now:
-            self.may_unregister_rpython_finalizer(self.space)
-            self.invoke_finalizer()
-
 
 W_CData.typedef = TypeDef(
-    '_cffi_backend._CDataBase',
-    __doc__ = "The internal base type for CData objects.  Use FFI.CData to "
-              "access it.  Always check with isinstance(): subtypes are "
-              "sometimes returned on CPython, for performance reasons.",
-    __module__ = '_cffi_backend',   # attribute also visible on instances
-    __name__ = '<cdata>',           # attribute also visible on instances
+    '_cffi_backend.CData',
+    __module__ = '_cffi_backend',
+    __name__ = '<cdata>',
     __repr__ = interp2app(W_CData.repr),
-    __bool__ = interp2app(W_CData.bool),
+    __nonzero__ = interp2app(W_CData.nonzero),
     __int__ = interp2app(W_CData.int),
+    __long__ = interp2app(W_CData.long),
     __float__ = interp2app(W_CData.float),
-    __complex__ = interp2app(W_CData.complex),
     __len__ = interp2app(W_CData.len),
     __lt__ = interp2app(W_CData.lt),
     __le__ = interp2app(W_CData.le),
@@ -754,7 +633,5 @@ W_CData.typedef = TypeDef(
     __iter__ = interp2app(W_CData.iter),
     __weakref__ = make_weakref_descr(W_CData),
     __dir__ = interp2app(W_CData.dir),
-    __enter__ = interp2app(W_CData.descr_enter),
-    __exit__ = interp2app(W_CData.descr_exit),
     )
 W_CData.typedef.acceptable_as_base_class = False
