@@ -10,19 +10,17 @@ from rpython.rlib.jit_libffi import (CIF_DESCRIPTION, CIF_DESCRIPTION_P,
 from rpython.rlib.objectmodel import we_are_translated, instantiate
 from rpython.rlib.objectmodel import keepalive_until_here
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
-from rpython.rtyper.annlowlevel import llstr
 
 from pypy.interpreter.error import OperationError, oefmt
-from pypy.module._cffi_backend.moduledef import (
-    FFI_DEFAULT_ABI, has_stdcall, FFI_STDCALL)
+from pypy.module import _cffi_backend
 from pypy.module._cffi_backend import ctypearray, cdataobj, cerrno
 from pypy.module._cffi_backend.ctypeobj import W_CType
 from pypy.module._cffi_backend.ctypeptr import W_CTypePtrBase, W_CTypePointer
 from pypy.module._cffi_backend.ctypevoid import W_CTypeVoid
-from pypy.module._cffi_backend.ctypestruct import W_CTypeStruct, W_CTypeUnion
+from pypy.module._cffi_backend.ctypestruct import W_CTypeStruct
 from pypy.module._cffi_backend.ctypeprim import (W_CTypePrimitiveSigned,
     W_CTypePrimitiveUnsigned, W_CTypePrimitiveCharOrUniChar,
-    W_CTypePrimitiveFloat, W_CTypePrimitiveLongDouble, W_CTypePrimitiveComplex)
+    W_CTypePrimitiveFloat, W_CTypePrimitiveLongDouble)
 
 
 class W_CTypeFunc(W_CTypePtrBase):
@@ -33,7 +31,7 @@ class W_CTypeFunc(W_CTypePtrBase):
     cif_descr = lltype.nullptr(CIF_DESCRIPTION)
 
     def __init__(self, space, fargs, fresult, ellipsis,
-                 abi=FFI_DEFAULT_ABI):
+                 abi=_cffi_backend.FFI_DEFAULT_ABI):
         assert isinstance(ellipsis, bool)
         extra, xpos = self._compute_extra_text(fargs, fresult, ellipsis, abi)
         size = rffi.sizeof(rffi.VOIDP)
@@ -100,9 +98,10 @@ class W_CTypeFunc(W_CTypePtrBase):
             lltype.free(self.cif_descr, flavor='raw')
 
     def _compute_extra_text(self, fargs, fresult, ellipsis, abi):
+        from pypy.module._cffi_backend import newtype
         argnames = ['(*)(']
         xpos = 2
-        if has_stdcall and abi == FFI_STDCALL:
+        if _cffi_backend.has_stdcall and abi == _cffi_backend.FFI_STDCALL:
             argnames[0] = '(__stdcall *)('
             xpos += len('__stdcall ')
         for i, farg in enumerate(fargs):
@@ -118,20 +117,17 @@ class W_CTypeFunc(W_CTypePtrBase):
 
     def _fget(self, attrchar):
         if attrchar == 'a':    # args
-            return self.space.newtuple([a for a in self.fargs])
+            return self.space.newtuple([self.space.wrap(a)
+                                        for a in self.fargs])
         if attrchar == 'r':    # result
-            return self.ctitem
+            return self.space.wrap(self.ctitem)
         if attrchar == 'E':    # ellipsis
-            return self.space.newbool(self.ellipsis)
+            return self.space.wrap(self.ellipsis)
         if attrchar == 'A':    # abi
-            return self.space.newint(self.abi)
+            return self.space.wrap(self.abi)
         return W_CTypePtrBase._fget(self, attrchar)
 
     def call(self, funcaddr, args_w):
-        if not funcaddr:
-            raise oefmt(self.space.w_RuntimeError,
-                        "cannot call null function pointer from cdata '%s'",
-                        self.name)
         if self.cif_descr:
             # regular case: this function does not take '...' arguments
             self = jit.promote(self)
@@ -168,9 +164,9 @@ class W_CTypeFunc(W_CTypePtrBase):
         cif_descr = self.cif_descr   # 'self' should have been promoted here
         size = cif_descr.exchange_size
         mustfree_max_plus_1 = 0
-        keepalives = [llstr(None)] * len(args_w)    # llstrings
         buffer = lltype.malloc(rffi.CCHARP.TO, size, flavor='raw')
         try:
+            keepalives = [None] * len(args_w)    # None or strings
             for i in range(len(args_w)):
                 data = rffi.ptradd(buffer, cif_descr.exchange_args[i])
                 w_obj = args_w[i]
@@ -195,15 +191,10 @@ class W_CTypeFunc(W_CTypePtrBase):
                     raw_cdata = rffi.cast(rffi.CCHARPP, data)[0]
                     if flag == 1:
                         lltype.free(raw_cdata, flavor='raw')
-                    elif flag == 3:
-                        rgc.unpin(keepalives[i])
-                        if not we_are_translated():
-                            lltype.free(keepalives[i])
                     elif flag >= 4:
-                        llobj = keepalives[i]
-                        assert llobj     # not NULL
-                        rffi.free_nonmovingbuffer_ll(raw_cdata,
-                                                     llobj, chr(flag))
+                        value = keepalives[i]
+                        assert value is not None
+                        rffi.free_nonmovingbuffer(value, raw_cdata, chr(flag))
             lltype.free(buffer, flavor='raw')
             keepalive_until_here(args_w)
         return w_res
@@ -212,48 +203,34 @@ def get_mustfree_flag(data):
     return ord(rffi.ptradd(data, -1)[0])
 
 def set_mustfree_flag(data, flag):
-    """ Set a flag for future handling of the pointer after the call,
-    possible values are:
-    0 - not set
-    1 - free the argument
-    2 - file argument
-    3 - unpin the keepalive
-    4, 5, 6 - free the keepalive slot, different values returned from
-              rffi.get_nonmovingbuffer_ll_final_null
-    """
     rffi.ptradd(data, -1)[0] = chr(flag)
 
 # ____________________________________________________________
 
 
+USE_C_LIBFFI_MSVC = getattr(clibffi, 'USE_C_LIBFFI_MSVC', False)
+
+
 # ----------
 # We attach to the classes small methods that return a 'ffi_type'
-
-def _notimplemented_ffi_type(self, is_result_type, extra=''):
+def _missing_ffi_type(self, cifbuilder, is_result_type):
+    space = self.space
+    if self.size < 0:
+        raise oefmt(space.w_TypeError,
+                    "ctype '%s' has incomplete type", self.name)
     if is_result_type:
         place = "return value"
     else:
         place = "argument"
-    raise oefmt(self.space.w_NotImplementedError,
-                "ctype '%s' (size %d) not supported as %s%s",
-                self.name, self.size, place, extra)
-
-def _missing_ffi_type(self, cifbuilder, is_result_type):
-    if self.size < 0:
-        raise oefmt(self.space.w_TypeError,
-                    "ctype '%s' has incomplete type", self.name)
-    raise _notimplemented_ffi_type(self, is_result_type)
+    raise oefmt(space.w_NotImplementedError,
+                "ctype '%s' (size %d) not supported as %s",
+                self.name, self.size, place)
 
 def _struct_ffi_type(self, cifbuilder, is_result_type):
     if self.size >= 0:
         return cifbuilder.fb_struct_ffi_type(self, is_result_type)
     return _missing_ffi_type(self, cifbuilder, is_result_type)
 
-def _union_ffi_type(self, cifbuilder, is_result_type):
-    if self.size >= 0:   # only for a better error message
-        return cifbuilder.fb_union_ffi_type(self, is_result_type)
-    return _missing_ffi_type(self, cifbuilder, is_result_type)
-
 def _primsigned_ffi_type(self, cifbuilder, is_result_type):
     size = self.size
     if   size == 1: return clibffi.ffi_type_sint8
@@ -279,13 +256,6 @@ def _primfloat_ffi_type(self, cifbuilder, is_result_type):
 def _primlongdouble_ffi_type(self, cifbuilder, is_result_type):
     return clibffi.ffi_type_longdouble
 
-def _primcomplex_ffi_type(self, cifbuilder, is_result_type):
-    raise _notimplemented_ffi_type(self, is_result_type,
-        extra = " (the support for complex types inside libffi "
-                "is mostly missing at this point, so CFFI only "
-                "supports complex types as arguments or return "
-                "value in API-mode functions)")
-
 def _ptr_ffi_type(self, cifbuilder, is_result_type):
     return clibffi.ffi_type_pointer
 
@@ -296,24 +266,16 @@ def _void_ffi_type(self, cifbuilder, is_result_type):
 
 W_CType._get_ffi_type                       = _missing_ffi_type
 W_CTypeStruct._get_ffi_type                 = _struct_ffi_type
-W_CTypeUnion._get_ffi_type                  = _union_ffi_type
 W_CTypePrimitiveSigned._get_ffi_type        = _primsigned_ffi_type
 W_CTypePrimitiveCharOrUniChar._get_ffi_type = _primunsigned_ffi_type
 W_CTypePrimitiveUnsigned._get_ffi_type      = _primunsigned_ffi_type
 W_CTypePrimitiveFloat._get_ffi_type         = _primfloat_ffi_type
 W_CTypePrimitiveLongDouble._get_ffi_type    = _primlongdouble_ffi_type
-W_CTypePrimitiveComplex._get_ffi_type       = _primcomplex_ffi_type
 W_CTypePtrBase._get_ffi_type                = _ptr_ffi_type
 W_CTypeVoid._get_ffi_type                   = _void_ffi_type
 # ----------
 
 
-_SUPPORTED_IN_API_MODE = (
-        " are only supported as %s if the function is "
-        "'API mode' and non-variadic (i.e. declared inside ffibuilder"
-        ".cdef()+ffibuilder.set_source() and not taking a final '...' "
-        "argument)")
-
 class CifDescrBuilder(object):
     rawmem = lltype.nullptr(rffi.CCHARP.TO)
 
@@ -335,20 +297,6 @@ class CifDescrBuilder(object):
     def fb_fill_type(self, ctype, is_result_type):
         return ctype._get_ffi_type(self, is_result_type)
 
-    def fb_unsupported(self, ctype, is_result_type, detail):
-        place = "return value" if is_result_type else "argument"
-        raise oefmt(self.space.w_NotImplementedError,
-            "ctype '%s' not supported as %s.  %s.  "
-            "Such structs" + _SUPPORTED_IN_API_MODE,
-            ctype.name, place, detail, place)
-
-    def fb_union_ffi_type(self, ctype, is_result_type=False):
-        place = "return value" if is_result_type else "argument"
-        raise oefmt(self.space.w_NotImplementedError,
-            "ctype '%s' not supported as %s by libffi.  "
-            "Unions" + _SUPPORTED_IN_API_MODE,
-            ctype.name, place, place)
-
     def fb_struct_ffi_type(self, ctype, is_result_type=False):
         # We can't pass a struct that was completed by verify().
         # Issue: assume verify() is given "struct { long b; ...; }".
@@ -361,42 +309,49 @@ class CifDescrBuilder(object):
         # Another reason for 'custom_field_pos' would be anonymous
         # nested structures: we lost the information about having it
         # here, so better safe (and forbid it) than sorry (and maybe
-        # crash).  Note: it seems we only get in this case with
-        # ffi.verify().
+        # crash).
         space = self.space
         ctype.force_lazy_struct()
         if ctype._custom_field_pos:
             # these NotImplementedErrors may be caught and ignored until
             # a real call is made to a function of this type
-            raise self.fb_unsupported(ctype, is_result_type,
-                "It is a struct declared with \"...;\", but the C "
-                "calling convention may depend on the missing fields; "
-                "or, it contains anonymous struct/unions")
-        # Another reason: __attribute__((packed)) is not supported by libffi.
-        if ctype._with_packed_change:
-            raise self.fb_unsupported(ctype, is_result_type,
-                "It is a 'packed' structure, with a different layout than "
-                "expected by libffi")
+            place = "return value" if is_result_type else "argument"
+            raise oefmt(space.w_NotImplementedError,
+                "ctype '%s' not supported as %s (it is a struct declared "
+                "with \"...;\", but the C calling convention may depend "
+                "on the missing fields)", ctype.name, place)
 
         # walk the fields, expanding arrays into repetitions; first,
         # only count how many flattened fields there are
         nflat = 0
         for i, cf in enumerate(ctype._fields_list):
             if cf.is_bitfield():
-                raise self.fb_unsupported(ctype, is_result_type,
-                    "It is a struct with bit fields, which libffi does not "
-                    "support")
+                place = "return value" if is_result_type else "argument"
+                raise oefmt(space.w_NotImplementedError,
+                    "ctype '%s' not supported as %s"
+                    " (it is a struct with bit fields)", ctype.name, place)
             flat = 1
             ct = cf.ctype
             while isinstance(ct, ctypearray.W_CTypeArray):
                 flat *= ct.length
                 ct = ct.ctitem
             if flat <= 0:
-                raise self.fb_unsupported(ctype, is_result_type,
-                    "It is a struct with a zero-length array, which libffi "
-                    "does not support")
+                place = "return value" if is_result_type else "argument"
+                raise oefmt(space.w_NotImplementedError,
+                    "ctype '%s' not supported as %s (it is a struct"
+                    " with a zero-length array)", ctype.name, place)
             nflat += flat
 
+        if USE_C_LIBFFI_MSVC and is_result_type:
+            # MSVC returns small structures in registers.  Pretend int32 or
+            # int64 return type.  This is needed as a workaround for what
+            # is really a bug of libffi_msvc seen as an independent library
+            # (ctypes has a similar workaround).
+            if ctype.size <= 4:
+                return clibffi.ffi_type_sint32
+            if ctype.size <= 8:
+                return clibffi.ffi_type_sint64
+
         # allocate an array of (nflat + 1) ffi_types
         elements = self.fb_alloc(rffi.sizeof(FFI_TYPE_P) * (nflat + 1))
         elements = rffi.cast(FFI_TYPE_PP, elements)
