@@ -5,12 +5,12 @@ Primitives.
 import sys
 
 from rpython.rlib.rarithmetic import r_uint, r_ulonglong, intmask
-from rpython.rlib import jit, rutf8
+from rpython.rlib import jit
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.tool import rfficache
 
 from pypy.interpreter.error import oefmt
-from pypy.module._cffi_backend import cdataobj, misc, wchar_helper
+from pypy.module._cffi_backend import cdataobj, misc
 from pypy.module._cffi_backend.ctypeobj import W_CType
 
 
@@ -25,14 +25,14 @@ class W_CTypePrimitive(W_CType):
 
     def extra_repr(self, cdata):
         w_ob = self.convert_to_object(cdata)
-        return self.space.text_w(self.space.repr(w_ob))
+        return self.space.str_w(self.space.repr(w_ob))
 
     def _alignof(self):
         return self.align
 
     def cast_str(self, w_ob):
         space = self.space
-        s = space.bytes_w(w_ob)
+        s = space.str_w(w_ob)
         if len(s) != 1:
             raise oefmt(space.w_TypeError,
                         "cannot cast string of length %d to ctype '%s'",
@@ -41,12 +41,12 @@ class W_CTypePrimitive(W_CType):
 
     def cast_unicode(self, w_ob):
         space = self.space
-        w_u = space.convert_arg_to_w_unicode(w_ob)
-        if w_u._len() != 1:
+        s = space.unicode_w(w_ob)
+        if len(s) != 1:
             raise oefmt(space.w_TypeError,
                         "cannot cast unicode string of length %d to ctype '%s'",
-                        w_u._len(), self.name)
-        return rutf8.codepoint_at_pos(w_u._utf8, 0)
+                        len(s), self.name)
+        return ord(s[0])
 
     def cast(self, w_ob):
         from pypy.module._cffi_backend import ctypeptr
@@ -56,7 +56,7 @@ class W_CTypePrimitive(W_CType):
             ptr = w_ob.unsafe_escaping_ptr()
             value = rffi.cast(lltype.Signed, ptr)
             value = self._cast_result(value)
-        elif space.isinstance_w(w_ob, space.w_bytes):
+        elif space.isinstance_w(w_ob, space.w_str):
             value = self.cast_str(w_ob)
             value = self._cast_result(value)
         elif space.isinstance_w(w_ob, space.w_unicode):
@@ -76,7 +76,7 @@ class W_CTypePrimitive(W_CType):
 
     def _overflow(self, w_ob):
         space = self.space
-        s = space.text_w(space.str(w_ob))
+        s = space.str_w(space.str(w_ob))
         raise oefmt(space.w_OverflowError,
                     "integer %s does not fit '%s'", s, self.name)
 
@@ -122,14 +122,14 @@ class W_CTypePrimitiveChar(W_CTypePrimitiveCharOrUniChar):
     _attrs_ = []
 
     def cast_to_int(self, cdata):
-        return self.space.newint(ord(cdata[0]))
+        return self.space.wrap(ord(cdata[0]))
 
     def convert_to_object(self, cdata):
         return self.space.newbytes(cdata[0])
 
     def _convert_to_char(self, w_ob):
         space = self.space
-        if space.isinstance_w(w_ob, space.w_bytes):
+        if space.isinstance_w(w_ob, space.w_str):
             s = space.bytes_w(w_ob)
             if len(s) == 1:
                 return s[0]
@@ -148,82 +148,54 @@ class W_CTypePrimitiveChar(W_CTypePrimitiveCharOrUniChar):
         return self.space.newbytes(s)
 
 
+# XXX explicitly use an integer type instead of lltype.UniChar here,
+# because for now the latter is defined as unsigned by RPython (even
+# though it may be signed when 'wchar_t' is written to C).
+WCHAR_INT = {(2, False): rffi.USHORT,
+             (4, False): rffi.UINT,
+             (4, True): rffi.INT}[rffi.sizeof(lltype.UniChar),
+                                  rfficache.signof_c_type('wchar_t')]
+WCHAR_INTP = rffi.CArrayPtr(WCHAR_INT)
+
 class W_CTypePrimitiveUniChar(W_CTypePrimitiveCharOrUniChar):
-    _attrs_            = ['is_signed_wchar']
-    _immutable_fields_ = ['is_signed_wchar']
+    _attrs_ = []
 
-    _wchar_is_signed = rfficache.signof_c_type('wchar_t')
-
-    def __init__(self, space, size, name, name_position, align):
-        W_CTypePrimitiveCharOrUniChar.__init__(self, space, size, name,
-                                               name_position, align)
-        self.is_signed_wchar = self._wchar_is_signed and (name == "wchar_t")
-        # "char16_t" and "char32_t" are always unsigned
+    if rffi.r_wchar_t.SIGN:
+        def write_raw_integer_data(self, w_cdata, value):
+            w_cdata.write_raw_signed_data(value)
 
     def cast_to_int(self, cdata):
-        if self.is_signed_wchar:
-            value = misc.read_raw_long_data(cdata, self.size)
-            return self.space.newint(value)
-        else:
-            value = misc.read_raw_ulong_data(cdata, self.size)
-            if self.size < rffi.sizeof(lltype.Signed):
-                return self.space.newint(intmask(value))
-            else:
-                return self.space.newint(value)    # r_uint => 'long' object
+        unichardata = rffi.cast(WCHAR_INTP, cdata)
+        return self.space.wrap(unichardata[0])
 
     def convert_to_object(self, cdata):
-        value = misc.read_raw_ulong_data(cdata, self.size)   # r_uint
-        try:
-            utf8 = rutf8.unichr_as_utf8(value, allow_surrogates=True)
-        except rutf8.OutOfRange:
-            if self.is_signed_wchar:
-                s = hex(intmask(value))
-            else:
-                s = hex(value)
-            raise oefmt(self.space.w_ValueError,
-                        "%s out of range for conversion to unicode: %s",
-                        self.name, s)
-        return self.space.newutf8(utf8, 1)
+        unichardata = rffi.cast(rffi.CWCHARP, cdata)
+        return self.space.wrap(unichardata[0])
 
     def string(self, cdataobj, maxlen):
         with cdataobj as ptr:
             w_res = self.convert_to_object(ptr)
         return w_res
 
-    def _convert_to_charN_t(self, w_ob):
-        # returns a r_uint.  If self.size == 2, it is smaller than 0x10000
+    def _convert_to_unichar(self, w_ob):
         space = self.space
         if space.isinstance_w(w_ob, space.w_unicode):
-            w_u = space.convert_arg_to_w_unicode(w_ob)
-            if w_u._len() != 1:
-                raise self._convert_error("single character", w_ob)
-            ordinal = rutf8.codepoint_at_pos(w_u._utf8, 0)
-            if self.size == 2 and ordinal > 0xFFFF:
-                raise self._convert_error("single character <= 0xFFFF", w_ob)
-            return r_uint(ordinal)
-        elif (isinstance(w_ob, cdataobj.W_CData) and
-               isinstance(w_ob.ctype, W_CTypePrimitiveUniChar) and
-               w_ob.ctype.size == self.size):
+            s = space.unicode_w(w_ob)
+            if len(s) == 1:
+                return s[0]
+        if (isinstance(w_ob, cdataobj.W_CData) and
+               isinstance(w_ob.ctype, W_CTypePrimitiveUniChar)):
             with w_ob as ptr:
-                return misc.read_raw_ulong_data(ptr, self.size)
+                return rffi.cast(rffi.CWCHARP, ptr)[0]
         raise self._convert_error("unicode string of length 1", w_ob)
 
     def convert_from_object(self, cdata, w_ob):
-        ordinal = self._convert_to_charN_t(w_ob)
-        misc.write_raw_unsigned_data(cdata, ordinal, self.size)
+        value = self._convert_to_unichar(w_ob)
+        rffi.cast(rffi.CWCHARP, cdata)[0] = value
 
     def unpack_ptr(self, w_ctypeptr, ptr, length):
-        if self.size == 2:
-            utf8, lgt = wchar_helper.utf8_from_char16(ptr, length)
-        else:
-            try:
-                utf8, lgt = wchar_helper.utf8_from_char32(ptr, length)
-            except wchar_helper.OutOfRange as e:
-                raise oefmt(self.space.w_ValueError,
-                            "%s out of range for conversion to unicode: %s",
-                            self.name, hex(e.ordinal))
-        assert lgt >= 0
-        return self.space.newutf8(utf8, lgt)
+        u = rffi.wcharpsize2unicode(rffi.cast(rffi.CWCHARP, ptr), length)
+        return self.space.wrap(u)
 
 
 class W_CTypePrimitiveSigned(W_CTypePrimitive):
@@ -242,14 +214,14 @@ class W_CTypePrimitiveSigned(W_CTypePrimitive):
     def convert_to_object(self, cdata):
         if self.value_fits_long:
             value = misc.read_raw_long_data(cdata, self.size)
-            return self.space.newint(value)
+            return self.space.wrap(value)
         else:
             return self._convert_to_object_longlong(cdata)
 
     def _convert_to_object_longlong(self, cdata):
         # in its own function: LONGLONG may make the whole function jit-opaque
         value = misc.read_raw_signed_data(cdata, self.size)
-        return self.space.newint(value)    # r_longlong => on 32-bit, 'long'
+        return self.space.wrap(value)    # r_longlong => on 32-bit, 'long'
 
     def convert_from_object(self, cdata, w_ob):
         if self.value_fits_long:
@@ -276,10 +248,10 @@ class W_CTypePrimitiveSigned(W_CTypePrimitive):
         w_cdata.write_raw_signed_data(value)
 
     def unpack_list_of_int_items(self, ptr, length):
-        if self.size == rffi.sizeof(rffi.SIGNED):
+        if self.size == rffi.sizeof(rffi.LONG):
             from rpython.rlib.rrawarray import populate_list_from_raw_array
             res = []
-            buf = rffi.cast(rffi.SIGNEDP, ptr)
+            buf = rffi.cast(rffi.LONGP, ptr)
             populate_list_from_raw_array(res, buf, length)
             return res
         elif self.value_smaller_than_long:
@@ -288,22 +260,20 @@ class W_CTypePrimitiveSigned(W_CTypePrimitive):
             return res
         return None
 
-    def pack_list_of_items(self, cdata, w_ob, expected_length):
+    def pack_list_of_items(self, cdata, w_ob):
         int_list = self.space.listview_int(w_ob)
-        if (int_list is not None and
-                self._within_bounds(len(int_list), expected_length)):
-            if self.size == rffi.sizeof(rffi.SIGNED): # fastest path
+        if int_list is not None:
+            if self.size == rffi.sizeof(rffi.LONG): # fastest path
                 from rpython.rlib.rrawarray import copy_list_to_raw_array
-                cdata = rffi.cast(rffi.SIGNEDP, cdata)
+                cdata = rffi.cast(rffi.LONGP, cdata)
                 copy_list_to_raw_array(int_list, cdata)
             else:
                 overflowed = misc.pack_list_to_raw_array_bounds_signed(
                     int_list, cdata, self.size)
                 if overflowed != 0:
-                    self._overflow(self.space.newint(overflowed))
+                    self._overflow(self.space.wrap(overflowed))
             return True
-        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob,
-                                                   expected_length)
+        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob)
 
 
 class W_CTypePrimitiveUnsigned(W_CTypePrimitive):
@@ -346,16 +316,16 @@ class W_CTypePrimitiveUnsigned(W_CTypePrimitive):
         if self.value_fits_ulong:
             value = misc.read_raw_ulong_data(cdata, self.size)
             if self.value_fits_long:
-                return self.space.newint(intmask(value))
+                return self.space.wrap(intmask(value))
             else:
-                return self.space.newint(value)    # r_uint => 'long' object
+                return self.space.wrap(value)    # r_uint => 'long' object
         else:
             return self._convert_to_object_longlong(cdata)
 
     def _convert_to_object_longlong(self, cdata):
         # in its own function: LONGLONG may make the whole function jit-opaque
         value = misc.read_raw_unsigned_data(cdata, self.size)
-        return self.space.newint(value)    # r_ulonglong => 'long' object
+        return self.space.wrap(value)    # r_ulonglong => 'long' object
 
     def get_vararg_type(self):
         if self.size < rffi.sizeof(rffi.INT):
@@ -373,17 +343,15 @@ class W_CTypePrimitiveUnsigned(W_CTypePrimitive):
             return res
         return None
 
-    def pack_list_of_items(self, cdata, w_ob, expected_length):
+    def pack_list_of_items(self, cdata, w_ob):
         int_list = self.space.listview_int(w_ob)
-        if (int_list is not None and
-                self._within_bounds(len(int_list), expected_length)):
+        if int_list is not None:
             overflowed = misc.pack_list_to_raw_array_bounds_unsigned(
                 int_list, cdata, self.size, self.vrangemax)
             if overflowed != 0:
-                self._overflow(self.space.newint(overflowed))
+                self._overflow(self.space.wrap(overflowed))
             return True
-        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob,
-                                                   expected_length)
+        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob)
 
 
 class W_CTypePrimitiveBool(W_CTypePrimitiveUnsigned):
@@ -402,26 +370,6 @@ class W_CTypePrimitiveBool(W_CTypePrimitiveUnsigned):
         # bypass the method 'string' implemented in W_CTypePrimitive
         return W_CType.string(self, cdataobj, maxlen)
 
-    def _read_bool_0_or_1(self, cdata):
-        """Read one byte, check it is 0 or 1, but return it as an integer"""
-        value = ord(cdata[0])
-        if value >= 2:
-            raise oefmt(self.space.w_ValueError,
-                        "got a _Bool of value %d, expected 0 or 1",
-                        value)
-        return value
-
-    def convert_to_object(self, cdata):
-        value = self._read_bool_0_or_1(cdata)
-        return self.space.newbool(value != 0)
-
-    def cast_to_int(self, cdata):
-        value = self._read_bool_0_or_1(cdata)
-        return self.space.newint(value)
-
-    def unpack_list_of_int_items(self, ptr, length):
-        return None
-
 
 class W_CTypePrimitiveFloat(W_CTypePrimitive):
     _attrs_ = []
@@ -435,7 +383,7 @@ class W_CTypePrimitiveFloat(W_CTypePrimitive):
                             w_ob.ctype.name, self.name)
             w_ob = w_ob.convert_to_object()
         #
-        if space.isinstance_w(w_ob, space.w_bytes):
+        if space.isinstance_w(w_ob, space.w_str):
             value = self.cast_str(w_ob)
         elif space.isinstance_w(w_ob, space.w_unicode):
             value = self.cast_unicode(w_ob)
@@ -458,7 +406,7 @@ class W_CTypePrimitiveFloat(W_CTypePrimitive):
 
     def convert_to_object(self, cdata):
         value = misc.read_raw_float_data(cdata, self.size)
-        return self.space.newfloat(value)
+        return self.space.wrap(value)
 
     def convert_from_object(self, cdata, w_ob):
         space = self.space
@@ -478,10 +426,9 @@ class W_CTypePrimitiveFloat(W_CTypePrimitive):
             return res
         return None
 
-    def pack_list_of_items(self, cdata, w_ob, expected_length):
+    def pack_list_of_items(self, cdata, w_ob):
         float_list = self.space.listview_float(w_ob)
-        if (float_list is not None and
-                self._within_bounds(len(float_list), expected_length)):
+        if float_list is not None:
             if self.size == rffi.sizeof(rffi.DOUBLE):   # fastest path
                 from rpython.rlib.rrawarray import copy_list_to_raw_array
                 cdata = rffi.cast(rffi.DOUBLEP, cdata)
@@ -491,8 +438,7 @@ class W_CTypePrimitiveFloat(W_CTypePrimitive):
                 misc.pack_float_list_to_raw_array(float_list, cdata,
                                                   rffi.FLOAT, rffi.FLOATP)
                 return True
-        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob,
-                                                   expected_length)
+        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob)
 
     def unpack_ptr(self, w_ctypeptr, ptr, length):
         result = self.unpack_list_of_float_items(ptr, length)
@@ -540,7 +486,7 @@ class W_CTypePrimitiveLongDouble(W_CTypePrimitiveFloat):
 
     def float(self, cdata):
         value = self._read_from_longdouble(cdata)
-        return self.space.newfloat(value)
+        return self.space.wrap(value)
 
     def convert_to_object(self, cdata):
         w_cdata = cdataobj.W_CDataMem(self.space, self)
@@ -562,64 +508,14 @@ class W_CTypePrimitiveLongDouble(W_CTypePrimitiveFloat):
     # 'list(array-of-longdouble)' returns a list of cdata objects,
     # not a list of floats.
 
-    def pack_list_of_items(self, cdata, w_ob, expected_length):
+    def pack_list_of_items(self, cdata, w_ob):
         float_list = self.space.listview_float(w_ob)
-        if (float_list is not None and
-                self._within_bounds(len(float_list), expected_length)):
+        if float_list is not None:
             misc.pack_float_list_to_raw_array(float_list, cdata,
                                              rffi.LONGDOUBLE, rffi.LONGDOUBLEP)
             return True
-        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob,
-                                                   expected_length)
+        return W_CTypePrimitive.pack_list_of_items(self, cdata, w_ob)
 
     @jit.dont_look_inside
     def nonzero(self, cdata):
         return misc.is_nonnull_longdouble(cdata)
-
-
-class W_CTypePrimitiveComplex(W_CTypePrimitive):
-    _attrs_ = []
-
-    def cast(self, w_ob):
-        space = self.space
-        if isinstance(w_ob, cdataobj.W_CData):
-            if not isinstance(w_ob.ctype, W_CTypePrimitive):
-                raise oefmt(space.w_TypeError,
-                            "cannot cast ctype '%s' to ctype '%s'",
-                            w_ob.ctype.name, self.name)
-            w_ob = w_ob.convert_to_object()
-        #
-        imag = 0.0
-        if space.isinstance_w(w_ob, space.w_bytes):
-            real = self.cast_str(w_ob)
-        elif space.isinstance_w(w_ob, space.w_unicode):
-            real = self.cast_unicode(w_ob)
-        else:
-            real, imag = space.unpackcomplex(w_ob)
-        w_cdata = cdataobj.W_CDataMem(space, self)
-        w_cdata.write_raw_complex_data(real, imag)
-        return w_cdata
-
-    def complex(self, cdata):
-        return self.convert_to_object(cdata)
-
-    def convert_to_object(self, cdata):
-        halfsize = self.size >> 1
-        cdata2 = rffi.ptradd(cdata, halfsize)
-        real = misc.read_raw_float_data(cdata, halfsize)
-        imag = misc.read_raw_float_data(cdata2, halfsize)
-        return self.space.newcomplex(real, imag)
-
-    def convert_from_object(self, cdata, w_ob):
-        space = self.space
-        real, imag = space.unpackcomplex(w_ob)
-        halfsize = self.size >> 1
-        cdata2 = rffi.ptradd(cdata, halfsize)
-        misc.write_raw_float_data(cdata,  real, halfsize)
-        misc.write_raw_float_data(cdata2, imag, halfsize)
-
-    def nonzero(self, cdata):
-        halfsize = self.size >> 1
-        cdata2 = rffi.ptradd(cdata, halfsize)
-        return (misc.is_nonnull_float(cdata, halfsize) |
-                misc.is_nonnull_float(cdata2, halfsize))
