@@ -2,14 +2,11 @@
 Pointers.
 """
 
-import os
-
-from rpython.rlib import rposix, jit, rgc
-from rpython.rlib.objectmodel import we_are_translated
+from rpython.rlib import rposix
 from rpython.rlib.rarithmetic import ovfcheck
-from rpython.rtyper.annlowlevel import llstr
+from rpython.rtyper.annlowlevel import llstr, llunicode
 from rpython.rtyper.lltypesystem import lltype, rffi
-from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw, STR
+from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw, copy_unicode_to_raw
 
 from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
 from pypy.module._cffi_backend import cdataobj, misc, ctypeprim, ctypevoid
@@ -59,7 +56,7 @@ class W_CTypePtrOrArray(W_CType):
 
     def _convert_array_from_listview(self, cdata, lst_w):
         space = self.space
-        if not self._within_bounds(len(lst_w), self.length):
+        if self.length >= 0 and len(lst_w) > self.length:
             raise oefmt(space.w_IndexError,
                         "too many initializers for '%s' (got %d)",
                         self.name, len(lst_w))
@@ -72,61 +69,46 @@ class W_CTypePtrOrArray(W_CType):
         space = self.space
         if (space.isinstance_w(w_ob, space.w_list) or
             space.isinstance_w(w_ob, space.w_tuple)):
-            if self.ctitem.pack_list_of_items(cdata, w_ob, self.length):
-                pass    # fast path
+            if self.ctitem.pack_list_of_items(cdata, w_ob):   # fast path
+                pass
             else:
                 self._convert_array_from_listview(cdata, space.listview(w_ob))
         elif self.accept_str:
-            if not space.isinstance_w(w_ob, space.w_bytes):
-                raise self._convert_error("bytes or list or tuple", w_ob)
-            s = space.bytes_w(w_ob)
+            if not space.isinstance_w(w_ob, space.w_str):
+                raise self._convert_error("str or list or tuple", w_ob)
+            s = space.str_w(w_ob)
             n = len(s)
             if self.length >= 0 and n > self.length:
                 raise oefmt(space.w_IndexError,
                             "initializer string is too long for '%s' (got %d "
                             "characters)", self.name, n)
-            if isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveBool):
-                self._must_be_string_of_zero_or_one(s)
             copy_string_to_raw(llstr(s), cdata, 0, n)
             if n != self.length:
                 cdata[n] = '\x00'
         elif isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveUniChar):
-            from pypy.module._cffi_backend import wchar_helper
             if not space.isinstance_w(w_ob, space.w_unicode):
                 raise self._convert_error("unicode or list or tuple", w_ob)
-            w_u = space.convert_arg_to_w_unicode(w_ob)
-            s = w_u._utf8
-            if self.ctitem.size == 2:
-                n = wchar_helper.utf8_size_as_char16(s)
-            else:
-                n = w_u._len()
+            s = space.unicode_w(w_ob)
+            n = len(s)
             if self.length >= 0 and n > self.length:
                 raise oefmt(space.w_IndexError,
                             "initializer unicode string is too long for '%s' "
                             "(got %d characters)", self.name, n)
-            add_final_zero = (n != self.length)
-            if self.ctitem.size == 2:
-                wchar_helper.utf8_to_char16(s, cdata, n, add_final_zero)
-            else:
-                wchar_helper.utf8_to_char32(s, cdata, n, add_final_zero)
+            unichardata = rffi.cast(rffi.CWCHARP, cdata)
+            copy_unicode_to_raw(llunicode(s), unichardata, 0, n)
+            if n != self.length:
+                unichardata[n] = u'\x00'
         else:
             raise self._convert_error("list or tuple", w_ob)
 
-    def _must_be_string_of_zero_or_one(self, s):
-        for c in s:
-            if ord(c) > 1:
-                raise oefmt(self.space.w_ValueError,
-                            "an array of _Bool can only contain \\x00 or \\x01")
-
     def string(self, cdataobj, maxlen):
         space = self.space
-        if (isinstance(self.ctitem, ctypeprim.W_CTypePrimitive) and
-               not isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveBool)):
+        if isinstance(self.ctitem, ctypeprim.W_CTypePrimitive):
             with cdataobj as ptr:
                 if not ptr:
                     raise oefmt(space.w_RuntimeError,
-                                "cannot use string() on %R",
-                                cdataobj)
+                                "cannot use string() on %s",
+                                space.str_w(cdataobj.repr()))
                 #
                 from pypy.module._cffi_backend import ctypearray
                 length = maxlen
@@ -143,23 +125,22 @@ class W_CTypePtrOrArray(W_CType):
                 #
                 # pointer to a wchar_t: builds and returns a unicode
                 if self.is_unichar_ptr_or_array():
-                    from pypy.module._cffi_backend import wchar_helper
-                    if self.ctitem.size == 2:
-                        length = wchar_helper.measure_length_16(ptr, length)
+                    cdata = rffi.cast(rffi.CWCHARP, ptr)
+                    if length < 0:
+                        u = rffi.wcharp2unicode(cdata)
                     else:
-                        length = wchar_helper.measure_length_32(ptr, length)
-                    return self.ctitem.unpack_ptr(self, ptr, length)
+                        u = rffi.wcharp2unicoden(cdata, length)
+                    return space.newunicode(u)
         #
         return W_CType.string(self, cdataobj, maxlen)
 
 
 class W_CTypePtrBase(W_CTypePtrOrArray):
     # base class for both pointers and pointers-to-functions
-    _attrs_ = ['is_void_ptr', 'is_voidchar_ptr', 'is_onebyte_ptr']
-    _immutable_fields_ = ['is_void_ptr', 'is_voidchar_ptr', 'is_onebyte_ptr']
+    _attrs_ = ['is_void_ptr', 'is_voidchar_ptr']
+    _immutable_fields_ = ['is_void_ptr', 'is_voidchar_ptr']
     is_void_ptr = False
     is_voidchar_ptr = False
-    is_onebyte_ptr = False
 
     def convert_to_object(self, cdata):
         ptrdata = rffi.cast(rffi.CCHARPP, cdata)[0]
@@ -179,20 +160,12 @@ class W_CTypePtrBase(W_CTypePtrOrArray):
             if self.is_void_ptr or other.is_void_ptr:
                 pass     # cast from or to 'void *'
             elif self.is_voidchar_ptr or other.is_voidchar_ptr:
-                # for backward compatibility, accept "char *" as either
-                # source of target.  This is not what C does, though,
-                # so emit a warning that will eventually turn into an
-                # error.  The warning is turned off if both types are
-                # pointers to single bytes.
-                if self.is_onebyte_ptr and other.is_onebyte_ptr:
-                    pass   # no warning
-                else:
-                    space = self.space
-                    msg = ("implicit cast from '%s' to '%s' "
-                        "will be forbidden in the future (check that the types "
-                        "are as you expect; use an explicit ffi.cast() if they "
-                        "are correct)" % (other.name, self.name))
-                    space.warn(space.newtext(msg), space.w_UserWarning)
+                space = self.space
+                msg = ("implicit cast from '%s' to '%s' "
+                    "will be forbidden in the future (check that the types "
+                    "are as you expect; use an explicit ffi.cast() if they "
+                    "are correct)" % (other.name, self.name))
+                space.warn(space.wrap(msg), space.w_UserWarning)
             else:
                 raise self._convert_error("compatible pointer", w_ob)
 
@@ -222,7 +195,6 @@ class W_CTypePointer(W_CTypePtrBase):
         self.is_void_ptr = isinstance(ctitem, ctypevoid.W_CTypeVoid)
         self.is_voidchar_ptr = (self.is_void_ptr or
                            isinstance(ctitem, ctypeprim.W_CTypePrimitiveChar))
-        self.is_onebyte_ptr = (ctitem.size == 1)
         W_CTypePtrBase.__init__(self, space, size, extra, 2, ctitem)
 
     def newp(self, w_init, allocator):
@@ -300,29 +272,28 @@ class W_CTypePointer(W_CTypePtrBase):
         return W_CTypePtrBase.cast(self, w_ob)
 
     def prepare_file(self, w_ob):
-        from pypy.module._io.interp_iobase import W_IOBase
-        if isinstance(w_ob, W_IOBase):
+        from pypy.module._file.interp_file import W_File
+        if isinstance(w_ob, W_File):
             return prepare_file_argument(self.space, w_ob)
         else:
             return lltype.nullptr(rffi.CCHARP.TO)
 
-    def _prepare_pointer_call_argument(self, w_init, cdata):
+    def _prepare_pointer_call_argument(self, w_init, cdata, keepalives, i):
         space = self.space
+        if self.accept_str and space.isinstance_w(w_init, space.w_str):
+            # special case to optimize strings passed to a "char *" argument
+            value = w_init.str_w(space)
+            keepalives[i] = value
+            buf, buf_flag = rffi.get_nonmovingbuffer_final_null(value)
+            rffi.cast(rffi.CCHARPP, cdata)[0] = buf
+            return ord(buf_flag)    # 4, 5 or 6
+        #
         if (space.isinstance_w(w_init, space.w_list) or
             space.isinstance_w(w_init, space.w_tuple)):
             length = space.int_w(space.len(w_init))
-        elif space.isinstance_w(w_init, space.w_bytes):
+        elif space.isinstance_w(w_init, space.w_basestring):
             # from a string, we add the null terminator
-            s = space.bytes_w(w_init)
-            length = len(s) + 1
-        elif space.isinstance_w(w_init, space.w_unicode):
-            from pypy.module._cffi_backend import wchar_helper
-            w_u = space.convert_arg_to_w_unicode(w_init)
-            if self.ctitem.size == 2:
-                length = wchar_helper.utf8_size_as_char16(w_u._utf8)
-            else:
-                length = w_u._len()
-            length += 1
+            length = space.int_w(space.len(w_init)) + 1
         elif self.is_file:
             result = self.prepare_file(w_init)
             if result:
@@ -353,74 +324,17 @@ class W_CTypePointer(W_CTypePtrBase):
         return 1
 
     def convert_argument_from_object(self, cdata, w_ob, keepalives, i):
-        # writes the pointer to cdata[0], writes the must-free flag in
-        # the byte just before cdata[0], and returns >0 if something
-        # must be done later to free. Description of flags in set_mustfree_flag
         from pypy.module._cffi_backend.ctypefunc import set_mustfree_flag
-        from pypy.module._cffi_backend.func import OffsetInBytes
-
-        if isinstance(w_ob, cdataobj.W_CData):
-            result = 0
-        else:
-            space = self.space
-            if self.accept_str and isinstance(w_ob, OffsetInBytes):
-                return self.process_str_from_offset_in_bytes(cdata, w_ob, keepalives, i)
-            if self.accept_str and space.isinstance_w(w_ob, space.w_bytes):
-                # special case to optimize strings passed to a "char *" argument
-                value = space.bytes_w(w_ob)
-                return self.accept_movable_str(cdata, value, keepalives, i)
-            result = self._prepare_pointer_call_argument(w_ob, cdata)
-
+        result = (not isinstance(w_ob, cdataobj.W_CData) and
+                  self._prepare_pointer_call_argument(w_ob, cdata,
+                                                      keepalives, i))
         if result == 0:
             self.convert_from_object(cdata, w_ob)
         set_mustfree_flag(cdata, result)
-        return result == 1      # 0 or 2 => False, nothing to do later
-
-    @jit.dont_look_inside
-    def process_str_from_offset_in_bytes(self, cdata, w_ob, keepalives, i):
-        if not rgc.must_split_gc_address_space():
-            lldata = llstr(w_ob.bytes)
-            if not rgc.can_move(lldata):
-                # easy case - it can't move, pass it directly
-                self.accept_str_from_offset_in_bytes(cdata, lldata,
-                                           keepalives, i, w_ob.offset)
-                if not we_are_translated():
-                    keepalives[i] = lldata
-                    return 1
-                return 0
-            elif rgc.pin(lldata):
-                return self.accept_str_from_offset_in_bytes(cdata, lldata,
-                                               keepalives, i, w_ob.offset)
-        # we failed to pin, need to make a copy
-        value = w_ob.bytes
-        return self.accept_movable_str(cdata, value, keepalives, i)
-
-
-    def accept_movable_str(self, cdata, value, keepalives, i):
-        if isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveBool):
-            self._must_be_string_of_zero_or_one(value)
-        keepalives[i] = misc.write_string_as_charp(cdata, value)
-        return 1
-
-    @jit.dont_look_inside
-    def accept_str_from_offset_in_bytes(self, cdata, lldata, keepalives, i, offset):
-        from rpython.rtyper.lltypesystem import llmemory
-        from rpython.rtyper.lltypesystem.rffi import offsetof, VOIDP, VOIDPP
-
-        addr = (llmemory.cast_ptr_to_adr(lldata) +
-              offsetof(STR, 'chars') +
-              llmemory.itemoffsetof(STR.chars, 0) + llmemory.sizeof(lltype.Char) * offset)
-        rffi.cast(VOIDPP, cdata)[0] = rffi.cast(VOIDP, addr)
-        if not we_are_translated():
-            keepalives[i] = lldata
-            return 3
-        return 0
+        return result
 
     def getcfield(self, attr):
-        from pypy.module._cffi_backend.ctypestruct import W_CTypeStructOrUnion
-        if isinstance(self.ctitem, W_CTypeStructOrUnion):
-            return self.ctitem.getcfield(attr)
-        return W_CType.getcfield(self, attr)
+        return self.ctitem.getcfield(attr)
 
     def typeoffsetof_field(self, fieldname, following):
         if following == 0:
@@ -454,7 +368,7 @@ class W_CTypePointer(W_CTypePtrBase):
 
     def _fget(self, attrchar):
         if attrchar == 'i':     # item
-            return self.ctitem
+            return self.space.wrap(self.ctitem)
         return W_CTypePtrBase._fget(self, attrchar)
 
 # ____________________________________________________________
@@ -480,17 +394,13 @@ class CffiFileObj(object):
 
 
 def prepare_file_argument(space, w_fileobj):
-    from pypy.module._io.interp_iobase import W_IOBase
-    assert isinstance(w_fileobj, W_IOBase)
-    space.call_method(w_fileobj, "flush")
+    w_fileobj.direct_flush()
     if w_fileobj.cffi_fileobj is None:
-        fd = space.int_w(space.call_method(w_fileobj, "fileno"))
+        fd = w_fileobj.direct_fileno()
         if fd < 0:
             raise oefmt(space.w_ValueError, "file has no OS file descriptor")
-        fd = os.dup(fd)
-        mode = space.text_w(space.getattr(w_fileobj, space.newtext("mode")))
         try:
-            w_fileobj.cffi_fileobj = CffiFileObj(fd, mode)
+            w_fileobj.cffi_fileobj = CffiFileObj(fd, w_fileobj.mode)
         except OSError as e:
             raise wrap_oserror(space, e)
     return rffi.cast(rffi.CCHARP, w_fileobj.cffi_fileobj.llf)
