@@ -4,7 +4,7 @@ Struct and unions.
 
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
-from pypy.interpreter.typedef import TypeDef, interp_attrproperty, interp_attrproperty_w
+from pypy.interpreter.typedef import TypeDef, interp_attrproperty
 
 from rpython.rlib import jit
 from rpython.rlib.rarithmetic import r_uint, r_ulonglong, r_longlong, intmask
@@ -34,7 +34,6 @@ class W_CTypeStructOrUnion(W_CType):
     _fields_dict = None
     _custom_field_pos = False
     _with_var_array = False
-    _with_packed_changed = False
 
     def __init__(self, space, name):
         W_CType.__init__(self, space, -1, name, len(name))
@@ -70,8 +69,8 @@ class W_CTypeStructOrUnion(W_CType):
             result = [None] * len(self._fields_list)
             for fname, field in self._fields_dict.iteritems():
                 i = self._fields_list.index(field)
-                result[i] = space.newtuple([space.newtext(fname),
-                                            field])
+                result[i] = space.newtuple([space.wrap(fname),
+                                            space.wrap(field)])
             return space.newlist(result)
         return W_CType._fget(self, attrchar)
 
@@ -85,7 +84,7 @@ class W_CTypeStructOrUnion(W_CType):
         self.check_complete()
         ptr = lltype.malloc(rffi.CCHARP.TO, self.size, flavor='raw', zero=False)
         misc._raw_memcopy(source, ptr, self.size)
-        return cdataobj.W_CDataNewStd(space, ptr, self, -1, self.size)
+        return cdataobj.W_CDataNewStd(space, ptr, self)
 
     def typeoffsetof_field(self, fieldname, following):
         self.force_lazy_struct()
@@ -93,7 +92,7 @@ class W_CTypeStructOrUnion(W_CType):
         try:
             cfield = self._getcfield_const(fieldname)
         except KeyError:
-            raise OperationError(space.w_KeyError, space.newtext(fieldname))
+            raise OperationError(space.w_KeyError, space.wrap(fieldname))
         if cfield.bitshift >= 0:
             raise oefmt(space.w_TypeError, "not supported for bitfields")
         return (cfield.ctype, cfield.offset)
@@ -110,41 +109,6 @@ class W_CTypeStructOrUnion(W_CType):
         if not self._copy_from_same(cdata, w_ob):
             self.convert_struct_from_object(cdata, w_ob, optvarsize=-1)
 
-    @jit.unroll_safe
-    def _max_initializers(self):
-        if self._with_var_array:
-            return -1
-        res = 0
-        for i in range(len(self._fields_list)):
-            j = 0
-            res += not (self._fields_list[j].flags &
-                               W_CField.BF_IGNORE_IN_CTOR)
-        return res
-
-    @jit.unroll_safe
-    def _unpack_initializer(self, w_ob):
-        from pypy.objspace.std.listobject import W_ListObject
-        from pypy.objspace.std.tupleobject import W_AbstractTupleObject
-        space = self.space
-        # this code is a bit weird. in theory, the look_inside_iff decorations
-        # on space.listview and its callees should be enough to get the correct
-        # unrolling effect. however, in a typicaly use case of ffi.new, there
-        # is too much app-level code running between the call to ffi.new and
-        # reaching convert_struct_from_object and the front-end loses the
-        # virtualness information. therefore we unroll a bit more aggressively
-        # here: we unroll, if the struct is not var-sized and the initializer
-        # length is up to the number of fields of the struct. in addition, the
-        # W_*Object must be virtual. see test_cffi_init_struct_with_list.
-        if jit.we_are_jitted() and not self._with_var_array:
-            max_num_initializer = self._max_initializers()
-            length = space.len_w(w_ob)
-            if max_num_initializer >= length:
-                if isinstance(w_ob, W_AbstractTupleObject) and space._uses_tuple_iter(w_ob):
-                    return w_ob.getitems_copy()
-                if type(w_ob) is W_ListObject:
-                    return w_ob.getitems_unroll()[:]
-        return space.listview(w_ob)
-
     @jit.look_inside_iff(
         lambda self, cdata, w_ob, optvarsize: jit.isvirtual(w_ob)
     )
@@ -154,7 +118,7 @@ class W_CTypeStructOrUnion(W_CType):
         space = self.space
         if (space.isinstance_w(w_ob, space.w_list) or
             space.isinstance_w(w_ob, space.w_tuple)):
-            lst_w = self._unpack_initializer(w_ob)
+            lst_w = space.listview(w_ob)
             j = 0
             for w_obj in lst_w:
                 try:
@@ -174,7 +138,7 @@ class W_CTypeStructOrUnion(W_CType):
             lst_w = space.fixedview(w_ob)
             for i in range(len(lst_w)):
                 w_key = lst_w[i]
-                key = space.text_w(w_key)
+                key = space.str_w(w_key)
                 try:
                     cf = self._fields_dict[key]
                 except KeyError:
@@ -196,18 +160,18 @@ class W_CTypeStructOrUnion(W_CType):
         return self._fields_dict[attr]
 
     def getcfield(self, attr):
-        # Returns a W_CField.  Error cases: returns None if we are an
-        # opaque struct; or raises KeyError if the particular field
-        # 'attr' does not exist.  The point of not directly building the
-        # error here is to get the exact ctype in the error message: it
-        # might be of the kind 'struct foo' or 'struct foo *'.
-        if self._fields_dict is None:
-            if self.size < 0:
-                return None
+        ready = self._fields_dict is not None
+        if not ready and self.size >= 0:
             self.force_lazy_struct()
-        self = jit.promote(self)
-        attr = jit.promote_string(attr)
-        return self._getcfield_const(attr)    # <= KeyError here
+            ready = True
+        if ready:
+            self = jit.promote(self)
+            attr = jit.promote_string(attr)
+            try:
+                return self._getcfield_const(attr)
+            except KeyError:
+                pass
+        return W_CType.getcfield(self, attr)
 
     def cdata_dir(self):
         if self.size < 0:
@@ -273,32 +237,26 @@ class W_CField(W_Root):
         else:
             self.ctype.convert_from_object(cdata, w_ob)
 
-    def add_varsize_length(self, space, itemsize, varsizelength, optvarsize):
-        # returns an updated 'optvarsize' to account for an array of
-        # 'varsizelength' elements, each of size 'itemsize', that starts
-        # at 'self.offset'.
-        try:
-            varsize = ovfcheck(itemsize * varsizelength)
-            size = ovfcheck(self.offset + varsize)
-        except OverflowError:
-            raise oefmt(space.w_OverflowError,
-                        "array size would overflow a ssize_t")
-        assert size >= 0
-        return max(size, optvarsize)
-
     def write_v(self, cdata, w_ob, optvarsize):
         # a special case for var-sized C99 arrays
         from pypy.module._cffi_backend import ctypearray
         ct = self.ctype
-        space = ct.space
         if isinstance(ct, ctypearray.W_CTypeArray) and ct.length < 0:
-            w_ob, varsizelength = ct.get_new_array_length(w_ob)
+            space = ct.space
+            w_ob, varsizelength = misc.get_new_array_length(space, w_ob)
             if optvarsize != -1:
                 # in this mode, the only purpose of this function is to compute
                 # the real size of the structure from a var-sized C99 array
                 assert cdata == lltype.nullptr(rffi.CCHARP.TO)
-                return self.add_varsize_length(space, ct.ctitem.size,
-                    varsizelength, optvarsize)
+                itemsize = ct.ctitem.size
+                try:
+                    varsize = ovfcheck(itemsize * varsizelength)
+                    size = ovfcheck(self.offset + varsize)
+                except OverflowError:
+                    raise oefmt(space.w_OverflowError,
+                                "array size would overflow a ssize_t")
+                assert size >= 0
+                return max(size, optvarsize)
             # if 'value' was only an integer, get_new_array_length() returns
             # w_ob = space.w_None.  Detect if this was the case,
             # and if so, stop here, leaving the content uninitialized
@@ -308,12 +266,6 @@ class W_CField(W_Root):
         #
         if optvarsize == -1:
             self.write(cdata, w_ob)
-        elif (isinstance(ct, W_CTypeStructOrUnion) and ct._with_var_array and
-              not isinstance(w_ob, cdataobj.W_CData)):
-            subsize = ct.size
-            subsize = ct.convert_struct_from_object(
-                lltype.nullptr(rffi.CCHARP.TO), w_ob, subsize)
-            optvarsize = self.add_varsize_length(space, 1, subsize, optvarsize)
         return optvarsize
 
     def convert_bitfield_to_object(self, cdata):
@@ -327,14 +279,14 @@ class W_CField(W_Root):
                 shiftforsign = r_uint(1) << (self.bitsize - 1)
                 value = ((value >> self.bitshift) + shiftforsign) & valuemask
                 result = intmask(value) - intmask(shiftforsign)
-                return space.newint(result)
+                return space.wrap(result)
             else:
                 value = misc.read_raw_unsigned_data(cdata, ctype.size)
                 valuemask = (r_ulonglong(1) << self.bitsize) - 1
                 shiftforsign = r_ulonglong(1) << (self.bitsize - 1)
                 value = ((value >> self.bitshift) + shiftforsign) & valuemask
                 result = r_longlong(value) - r_longlong(shiftforsign)
-                return space.newint(result)
+                return space.wrap(result)
         #
         if isinstance(ctype, ctypeprim.W_CTypePrimitiveUnsigned):
             value_fits_long = ctype.value_fits_long
@@ -350,14 +302,14 @@ class W_CField(W_Root):
             valuemask = (r_uint(1) << self.bitsize) - 1
             value = (value >> self.bitshift) & valuemask
             if value_fits_long:
-                return space.newint(intmask(value))
+                return space.wrap(intmask(value))
             else:
-                return space.newint(value)    # uint => wrapped long object
+                return space.wrap(value)    # uint => wrapped long object
         else:
             value = misc.read_raw_unsigned_data(cdata, ctype.size)
             valuemask = (r_ulonglong(1) << self.bitsize) - 1
             value = (value >> self.bitshift) & valuemask
-            return space.newint(value)      # ulonglong => wrapped long object
+            return space.wrap(value)      # ulonglong => wrapped long object
 
     def convert_bitfield_from_object(self, cdata, w_ob):
         ctype = self.ctype
@@ -390,10 +342,10 @@ class W_CField(W_Root):
 
 W_CField.typedef = TypeDef(
     '_cffi_backend.CField',
-    type = interp_attrproperty_w('ctype', W_CField),
-    offset = interp_attrproperty('offset', W_CField, wrapfn="newint"),
-    bitshift = interp_attrproperty('bitshift', W_CField, wrapfn="newint"),
-    bitsize = interp_attrproperty('bitsize', W_CField, wrapfn="newint"),
-    flags = interp_attrproperty('flags', W_CField, wrapfn="newint"),
+    type = interp_attrproperty('ctype', W_CField),
+    offset = interp_attrproperty('offset', W_CField),
+    bitshift = interp_attrproperty('bitshift', W_CField),
+    bitsize = interp_attrproperty('bitsize', W_CField),
+    flags = interp_attrproperty('flags', W_CField),
     )
 W_CField.typedef.acceptable_as_base_class = False
