@@ -1,41 +1,27 @@
-import os, pytest
-import sys
+import os, py
 
 from rpython.tool.udir import udir
 from pypy.interpreter.gateway import unwrap_spec, interp2app
 from pypy.module._cffi_backend.newtype import _clean_cache
 import pypy.module.cpyext.api     # side-effect of pre-importing it
-from sysconfig import get_config_var
 
 
-def get_ext_suffix():
-    # soabi is None on cpython < 3.7 (incl 2.7), 'pypy-73' on pypy2 v7.3.1
-    # and something like 'cpython-38-x86_64-linux-gnu' on cpython 3.8
-    soabi = get_config_var('SOABI') or ''
-    ret = soabi + get_config_var('SO')
-    # either '.so' or 'pypy-73.so'
-    if ret[0] == '.':
-        return ret[1:]
-    return ret
-
-
-@unwrap_spec(cdef='text', module_name='text', source='text', packed=int)
+@unwrap_spec(cdef=str, module_name=str, source=str)
 def prepare(space, cdef, module_name, source, w_includes=None,
-            w_extra_source=None, w_min_version=None, packed=False,
-            w_extra_compile_args=None):
+            w_extra_source=None, w_min_version=None):
     try:
         import cffi
         from cffi import FFI            # <== the system one, which
         from cffi import recompiler     # needs to be at least cffi 1.0.4
         from cffi import ffiplatform
     except ImportError:
-        pytest.skip("system cffi module not found or older than 1.0.0")
+        py.test.skip("system cffi module not found or older than 1.0.0")
     if w_min_version is None:
         min_version = (1, 4, 0)
     else:
         min_version = tuple(space.unwrap(w_min_version))
     if cffi.__version_info__ < min_version:
-        pytest.skip("system cffi module needs to be at least %s, got %s" % (
+        py.test.skip("system cffi module needs to be at least %s, got %s" % (
             min_version, cffi.__version_info__))
     space.appexec([], """():
         import _cffi_backend     # force it to be initialized
@@ -61,25 +47,21 @@ def prepare(space, cdef, module_name, source, w_includes=None,
     ffi = FFI()
     for include_ffi_object in includes:
         ffi.include(include_ffi_object._test_recompiler_source_ffi)
-    ffi.cdef(cdef, packed=packed)
+    ffi.cdef(cdef)
     ffi.set_source(module_name, source)
     ffi.emit_c_code(c_file)
 
     base_module_name = module_name.split('.')[-1]
     sources = []
     if w_extra_source is not None:
-        sources.append(space.text_w(w_extra_source))
-    kwargs = {}
-    if w_extra_compile_args is not None:
-        kwargs['extra_compile_args'] = space.unwrap(w_extra_compile_args)
+        sources.append(space.str_w(w_extra_source))
     ext = ffiplatform.get_extension(c_file, module_name,
             include_dirs=[str(rdir)],
             export_symbols=['_cffi_pypyinit_' + base_module_name],
-            sources=sources,
-            **kwargs)
+            sources=sources)
     ffiplatform.compile(str(rdir), ext)
 
-    for extension in [get_ext_suffix(), 'so', 'pyd', 'dylib']:
+    for extension in ['so', 'pyd', 'dylib']:
         so_file = str(rdir.join('%s.%s' % (path, extension)))
         if os.path.exists(so_file):
             break
@@ -88,12 +70,8 @@ def prepare(space, cdef, module_name, source, w_includes=None,
 
     args_w = [space.wrap(module_name), space.wrap(so_file)]
     w_res = space.appexec(args_w, """(modulename, filename):
-        import _imp
-        class Spec: pass
-        spec = Spec()
-        spec.name = modulename
-        spec.origin = filename
-        mod = _imp.create_dynamic(spec)
+        import imp
+        mod = imp.load_dynamic(modulename, filename)
         assert mod.__name__ == modulename
         return (mod.ffi, mod.lib)
     """)
@@ -106,20 +84,17 @@ def prepare(space, cdef, module_name, source, w_includes=None,
 
 
 class AppTestRecompiler:
-    spaceconfig = dict(usemodules=['_cffi_backend', 'imp', 'cpyext', 'struct'])
+    spaceconfig = dict(usemodules=['_cffi_backend', 'imp'])
 
     def setup_class(cls):
         if cls.runappdirect:
-            pytest.skip("not a test for -A")
-        elif sys.platform == 'darwin':
-            pytest.skip('fails untranslated on darwin')
+            py.test.skip("not a test for -A")
         cls.w_prepare = cls.space.wrap(interp2app(prepare))
         cls.w_udir = cls.space.wrap(str(udir))
         cls.w_os_sep = cls.space.wrap(os.sep)
 
     def setup_method(self, meth):
         self._w_modules = self.space.appexec([], """():
-            import cpyext      # ignore stuff there in the leakfinder
             import sys
             return set(sys.modules)
         """)
@@ -131,7 +106,7 @@ class AppTestRecompiler:
             del self.space._cleanup_ffi
         self.space.appexec([self._w_modules], """(old_modules):
             import sys
-            for key in list(sys.modules.keys()):
+            for key in sys.modules.keys():
                 if key not in old_modules:
                     del sys.modules[key]
         """)
@@ -363,9 +338,9 @@ class AppTestRecompiler:
             'test_verify_exact_field_offset',
             """struct foo_s { short a; int b; };""")
         e = raises(ffi.error, ffi.new, "struct foo_s *", [])    # lazily
-        assert str(e.value).startswith(
-            "struct foo_s: wrong offset for field 'b' (cdef "
-            'says 0, but C compiler says 4). fix it or use "...;" ')
+        assert str(e.value) == ("struct foo_s: wrong offset for field 'b' (cdef "
+                           'says 0, but C compiler says 4). fix it or use "...;" '
+                           "in the cdef for struct foo_s to make it flexible")
 
     def test_type_caching(self):
         ffi1, lib1 = self.prepare(
@@ -948,13 +923,13 @@ class AppTestRecompiler:
         # but we can get its address
         p = ffi.addressof(lib, 'globvar')
         assert ffi.typeof(p) == ffi.typeof('opaque_t *')
-        assert ffi.string(ffi.cast("char *", p), 8) == b"hello"
+        assert ffi.string(ffi.cast("char *", p), 8) == "hello"
 
     def test_constant_of_value_unknown_to_the_compiler(self):
         extra_c_source = self.udir + self.os_sep + (
             'extra_test_constant_of_value_unknown_to_the_compiler.c')
-        with open(extra_c_source, 'wb') as f:
-            f.write(b'const int external_foo = 42;\n')
+        with open(extra_c_source, 'w') as f:
+            f.write('const int external_foo = 42;\n')
         ffi, lib = self.prepare(
             "const int external_foo;",
             'test_constant_of_value_unknown_to_the_compiler',
@@ -1105,7 +1080,7 @@ class AppTestRecompiler:
         #
         @ffi.callback("int *(*)(void)")
         def get_my_value():
-            return values + next(it)
+            return values + it.next()
         lib.get_my_value = get_my_value
         #
         values[0] = 41
@@ -1449,7 +1424,7 @@ class AppTestRecompiler:
                 def getvalue(self):
                     if self._result is None:
                         os.close(self._wr)
-                        self._result = os.read(self._rd, 4096).decode()
+                        self._result = os.read(self._rd, 4096)
                         os.close(self._rd)
                         # xxx hack away these lines
                         while self._result.startswith('[platform:execute]'):
@@ -1496,13 +1471,8 @@ class AppTestRecompiler:
         with self.StdErrCapture(fd=True) as f:
             res = lib.bar(4, 5)
         assert res == 0
-        assert f.getvalue() in (
-            # If the underlying cffi is <= 1.9
+        assert f.getvalue() == (
             "extern \"Python\": function bar() called, but no code was attached "
-            "to it yet with @ffi.def_extern().  Returning 0.\n",
-            # If the underlying cffi is >= 1.10
-            "extern \"Python\": function _CFFI_test_extern_python_1.bar() "
-            "called, but no code was attached "
             "to it yet with @ffi.def_extern().  Returning 0.\n")
 
         @ffi.def_extern("bar")
@@ -1520,11 +1490,11 @@ class AppTestRecompiler:
         baz1 = ffi.def_extern()(baz)
         assert baz1 is baz
         seen = []
-        baz(40, 4)
-        res = lib.baz(50, 8)
+        baz(40L, 4L)
+        res = lib.baz(50L, 8L)
         assert res is None
-        assert seen == [("Baz", 40, 4), ("Baz", 50, 8)]
-        assert type(seen[0][1]) is type(seen[0][2]) is int
+        assert seen == [("Baz", 40L, 4L), ("Baz", 50, 8)]
+        assert type(seen[0][1]) is type(seen[0][2]) is long
         assert type(seen[1][1]) is type(seen[1][2]) is int
 
         @ffi.def_extern(name="bok")
@@ -1575,11 +1545,10 @@ class AppTestRecompiler:
             return n * 10
         with self.StdErrCapture() as f:
             res = lib.bar(321)
-        msg = f.getvalue()
         assert res is None
-        assert "rom cffi callback %r" % (bar,) in msg
-        assert "rying to convert the result back to C:\n" in msg
-        assert msg.endswith(
+        assert f.getvalue() == (
+            "From cffi callback %r:\n" % (bar,) +
+            "Trying to convert the result back to C:\n"
             "TypeError: callback with the return type 'void' must return None\n")
 
     def test_extern_python_redefine(self):
@@ -1692,7 +1661,6 @@ class AppTestRecompiler:
         raises(TypeError, ffi.def_extern(name='bar', onerror=42),
                        lambda x: x)
 
-    @pytest.mark.skipif("not config.option.runappdirect")
     def test_extern_python_stdcall(self):
         ffi, lib = self.prepare("""
             extern "Python" int __stdcall foo(int);
@@ -1846,68 +1814,6 @@ class AppTestRecompiler:
         assert lib.f.__get__(42) is lib.f
         assert lib.f.__get__(42, int) is lib.f
 
-    def test_function_returns_float_complex(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("MSVC may not support _Complex")
-        ffi, lib = self.prepare(
-            "float _Complex f1(float a, float b);",
-            "test_function_returns_float_complex", """
-            #include <complex.h>
-            static float _Complex f1(float a, float b) { return a + I*2.0*b; }
-        """, min_version=(1, 11, 0))
-        result = lib.f1(1.25, 5.1)
-        assert type(result) == complex
-        assert result.real == 1.25   # exact
-        assert (result.imag != 2*5.1) and (abs(result.imag - 2*5.1) < 1e-5) # inexact
-
-    def test_function_returns_double_complex(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("MSVC may not support _Complex")
-        ffi, lib = self.prepare(
-            "double _Complex f1(double a, double b);",
-            "test_function_returns_double_complex", """
-            #include <complex.h>
-            static double _Complex f1(double a, double b) { return a + I*2.0*b; }
-        """, min_version=(1, 11, 0))
-        result = lib.f1(1.25, 5.1)
-        assert type(result) == complex
-        assert result.real == 1.25   # exact
-        assert result.imag == 2*5.1  # exact
-
-    def test_function_argument_float_complex(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("MSVC may not support _Complex")
-        ffi, lib = self.prepare(
-            "float f1(float _Complex x);",
-            "test_function_argument_float_complex", """
-            #include <complex.h>
-            static float f1(float _Complex x) { return cabsf(x); }
-        """, min_version=(1, 11, 0))
-        x = complex(12.34, 56.78)
-        result = lib.f1(x)
-        assert abs(result - abs(x)) < 1e-5
-        result2 = lib.f1(ffi.cast("float _Complex", x))
-        assert result2 == result
-
-    def test_function_argument_double_complex(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("MSVC may not support _Complex")
-        ffi, lib = self.prepare(
-            "double f1(double _Complex);",
-            "test_function_argument_double_complex", """
-            #include <complex.h>
-            static double f1(double _Complex x) { return cabs(x); }
-        """, min_version=(1, 11, 0))
-        x = complex(12.34, 56.78)
-        result = lib.f1(x)
-        assert abs(result - abs(x)) < 1e-11
-        result2 = lib.f1(ffi.cast("double _Complex", x))
-        assert result2 == result
-
     def test_typedef_array_dotdotdot(self):
         ffi, lib = self.prepare("""
             typedef int foo_t[...], bar_t[...];
@@ -1932,261 +1838,3 @@ class AppTestRecompiler:
         raises(ffi.error, ffi.sizeof, "vmat_t")
         p = ffi.new("vmat_t", 4)
         assert ffi.sizeof(p[3]) == 8 * ffi.sizeof("int")
-
-    def test_call_with_custom_field_pos(self):
-        ffi, lib = self.prepare("""
-            struct foo { int x; ...; };
-            struct foo f(void);
-            struct foo g(int, ...);
-            """, "test_call_with_custom_field_pos", """
-            struct foo { int y, x; };
-            struct foo f(void) {
-                struct foo s = { 40, 200 };
-                return s;
-            }
-            struct foo g(int a, ...) { }
-        """)
-        assert lib.f().x == 200
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-            'ctype \'struct foo\' not supported as return value.  It is a '
-            'struct declared with "...;", but the C calling convention may '
-            'depend on the missing fields; or, it contains anonymous '
-            'struct/unions.  Such structs are only supported '
-            'as return value if the function is \'API mode\' and non-variadic '
-            '(i.e. declared inside ffibuilder.cdef()+ffibuilder.set_source() '
-            'and not taking a final \'...\' argument)')
-
-    def test_call_with_nested_anonymous_struct(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("needs a GCC extension")
-        ffi, lib = self.prepare("""
-            struct foo { int a; union { int b, c; }; };
-            struct foo f(void);
-            struct foo g(int, ...);
-            """, "test_call_with_nested_anonymous_struct", """
-            struct foo { int a; union { int b, c; }; };
-            struct foo f(void) {
-                struct foo s = { 40 };
-                s.b = 200;
-                return s;
-            }
-            struct foo g(int a, ...) { }
-        """)
-        assert lib.f().b == 200
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-            'ctype \'struct foo\' not supported as return value.  It is a '
-            'struct declared with "...;", but the C calling convention may '
-            'depend on the missing fields; or, it contains anonymous '
-            'struct/unions.  Such structs are only supported '
-            'as return value if the function is \'API mode\' and non-variadic '
-            '(i.e. declared inside ffibuilder.cdef()+ffibuilder.set_source() '
-            'and not taking a final \'...\' argument)')
-
-    def test_call_with_bitfield(self):
-        ffi, lib = self.prepare("""
-            struct foo { int x:5; };
-            struct foo f(void);
-            struct foo g(int, ...);
-            """, "test_call_with_bitfield", """
-            struct foo { int x:5; };
-            struct foo f(void) {
-                struct foo s = { 11 };
-                return s;
-            }
-            struct foo g(int a, ...) { }
-        """)
-        assert lib.f().x == 11
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-            "ctype 'struct foo' not supported as return value.  It is a struct "
-            "with bit fields, which libffi does not support.  Such structs are "
-            "only supported as return value if the function is 'API mode' and "
-            "non-variadic (i.e. declared inside ffibuilder.cdef()+ffibuilder."
-            "set_source() and not taking a final '...' argument)")
-
-    def test_call_with_zero_length_field(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("zero-length field not supported by MSVC")
-        ffi, lib = self.prepare("""
-            struct foo { int a; int x[0]; };
-            struct foo f(void);
-            struct foo g(int, ...);
-            """, "test_call_with_zero_length_field", """
-            struct foo { int a; int x[0]; };
-            struct foo f(void) {
-                struct foo s = { 42 };
-                return s;
-            }
-            struct foo g(int a, ...) { }
-        """)
-        assert lib.f().a == 42
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-           "ctype 'struct foo' not supported as return value.  It is a "
-           "struct with a zero-length array, which libffi does not support.  "
-           "Such structs are only supported as return value if the function is "
-           "'API mode' and non-variadic (i.e. declared inside ffibuilder.cdef()"
-           "+ffibuilder.set_source() and not taking a final '...' argument)")
-
-    def test_call_with_union(self):
-        ffi, lib = self.prepare("""
-            union foo { int a; char b; };
-            union foo f(void);
-            union foo g(int, ...);
-            """, "test_call_with_union", """
-            union foo { int a; char b; };
-            union foo f(void) {
-                union foo s = { 42 };
-                return s;
-            }
-            union foo g(int a, ...) { }
-        """)
-        assert lib.f().a == 42
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-           "ctype 'union foo' not supported as return value by libffi.  "
-           "Unions are only supported as return value if the function is "
-           "'API mode' and non-variadic (i.e. declared inside ffibuilder.cdef()"
-           "+ffibuilder.set_source() and not taking a final '...' argument)")
-
-    def test_call_with_packed_struct(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("needs a GCC extension")
-        ffi, lib = self.prepare("""
-            struct foo { char y; int x; };
-            struct foo f(void);
-            struct foo g(int, ...);
-        """, "test_call_with_packed_struct", """
-            struct foo { char y; int x; } __attribute__((packed));
-            struct foo f(void) {
-                struct foo s = { 40, 200 };
-                return s;
-            }
-            struct foo g(int a, ...) {
-                struct foo s = { 41, 201 };
-                return s;
-            }
-        """, packed=True, min_version=(1, 8, 3))
-        assert ord(lib.f().y) == 40
-        assert lib.f().x == 200
-        e = raises(NotImplementedError, lib.g, 0)
-        assert str(e.value) == (
-           "ctype 'struct foo' not supported as return value.  It is a 'packed'"
-           " structure, with a different layout than expected by libffi.  "
-           "Such structs are only supported as return value if the function is "
-           "'API mode' and non-variadic (i.e. declared inside ffibuilder.cdef()"
-           "+ffibuilder.set_source() and not taking a final '...' argument)")
-
-    def test_gcc_visibility_hidden(self):
-        import sys
-        if sys.platform == 'win32':
-            skip("test for gcc/clang")
-        ffi, lib = self.prepare("""
-        int f(int);
-        """, "test_gcc_visibility_hidden", """
-        int f(int a) { return a + 40; }
-        """, extra_compile_args=['-fvisibility=hidden'])
-        assert lib.f(2) == 42
-
-    def test_override_default_definition(self):
-        ffi, lib = self.prepare("""
-        typedef long int16_t, char16_t;
-        """, "test_override_default_definition", """
-        """)
-        assert ffi.typeof("int16_t") is ffi.typeof("char16_t") is ffi.typeof("long")
-
-    def test_char16_char32_plain_c(self):
-        ffi, lib = self.prepare("""
-            char16_t foo_2bytes(char16_t);
-            char32_t foo_4bytes(char32_t);
-        """, "test_char16_char32_type_nocpp", """
-        #if !defined(__cplusplus) || (!defined(_LIBCPP_VERSION) && __cplusplus < 201103L)
-        typedef uint_least16_t char16_t;
-        typedef uint_least32_t char32_t;
-        #endif
-
-        char16_t foo_2bytes(char16_t a) { return (char16_t)(a + 42); }
-        char32_t foo_4bytes(char32_t a) { return (char32_t)(a + 42); }
-        """, min_version=(1, 11, 0))
-        assert lib.foo_2bytes(u'\u1234') == u'\u125e'
-        assert lib.foo_4bytes(u'\u1234') == u'\u125e'
-        assert lib.foo_4bytes(u'\U00012345') == u'\U0001236f'
-        raises(TypeError, lib.foo_2bytes, u'\U00012345')
-        raises(TypeError, lib.foo_2bytes, 1234)
-        raises(TypeError, lib.foo_4bytes, 1234)
-
-    def test_loader_spec(self):
-        import sys
-        ffi, lib = self.prepare("", "test_loader_spec", "")
-        if sys.version_info < (3,):
-            assert not hasattr(lib, '__loader__')
-            assert not hasattr(lib, '__spec__')
-        else:
-            assert lib.__loader__ is None
-            assert lib.__spec__ is None
-
-    def test_release(self):
-        ffi, lib = self.prepare("", "test_release", "")
-        p = ffi.new("int[]", 123)
-        ffi.release(p)
-        # here, reading p[0] might give garbage or segfault...
-        ffi.release(p)   # no effect
-
-    def test_release_new_allocator(self):
-        ffi, lib = self.prepare("struct ab { int a, b; };",
-                                "test_release_new_allocator",
-                                "struct ab { int a, b; };")
-        seen = []
-        def myalloc(size):
-            seen.append(size)
-            return ffi.new("char[]", b"X" * size)
-        def myfree(raw):
-            seen.append(raw)
-        alloc2 = ffi.new_allocator(alloc=myalloc, free=myfree)
-        p = alloc2("int[]", 15)
-        assert seen == [15 * 4]
-        ffi.release(p)
-        assert seen == [15 * 4, p]
-        ffi.release(p)    # no effect
-        assert seen == [15 * 4, p]
-        #
-        del seen[:]
-        p = alloc2("struct ab *")
-        assert seen == [2 * 4]
-        ffi.release(p)
-        assert seen == [2 * 4, p]
-        ffi.release(p)    # no effect
-        assert seen == [2 * 4, p]
-
-    def test_struct_with_func_with_struct_arg(self):
-        ffi, lib = self.prepare("""struct BinaryTree {
-                int (* CompareKey)(struct BinaryTree tree);
-            };""",
-            "test_struct_with_func_with_struct_arg", """
-            struct BinaryTree {
-                int (* CompareKey)(struct BinaryTree tree);
-            };
-        """)
-        e = raises(RuntimeError, ffi.new, "struct BinaryTree *")
-        # we should check e.value, but untranslated it crashes with a
-        # regular recursion error.  There is a chance it occurs translated
-        # too, but likely the check in the code ">= 1000" usually triggers
-        # before that, and raise a RuntimeError too, but with the more
-        # explicit message.
-
-    def test_call_function_offset_in_bytes(self):
-        from _cffi_backend import _offset_in_bytes
-        ffi, lib = self.prepare("""
-        int foo(char* arg);
-        """, "test_call_function_offset_in_bytes", """
-        int foo(char* arg) {
-            return(arg[0] * 10);
-        }
-        """)
-        assert lib.foo(_offset_in_bytes(b"foo", 0)) == ord(b"f") * 10
-        assert lib.foo(_offset_in_bytes(b"foobxo", 4)) == ord(b"x") * 10
