@@ -1,6 +1,3 @@
-
-from rpython.rlib import rutf8
-
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef
@@ -14,40 +11,37 @@ class MultibyteCodec(W_Root):
         self.name = name
         self.codec = codec
 
-    @unwrap_spec(input='bufferstr', errors="text_or_none")
+    @unwrap_spec(input=str, errors="str_or_None")
     def decode(self, space, input, errors=None):
         if errors is None:
             errors = 'strict'
         state = space.fromcache(CodecState)
         #
         try:
-            utf8_output = c_codecs.decode(self.codec, input, errors,
+            output = c_codecs.decode(self.codec, input, errors,
                                      state.decode_error_handler, self.name)
         except c_codecs.EncodeDecodeError as e:
             raise wrap_unicodedecodeerror(space, e, input, self.name)
         except RuntimeError:
             raise wrap_runtimeerror(space)
-        lgt = rutf8.codepoints_in_utf8(utf8_output)
-        return space.newtuple([space.newutf8(utf8_output, lgt),
-                               space.newint(len(input))])
+        return space.newtuple([space.wrap(output),
+                               space.wrap(len(input))])
 
-    @unwrap_spec(errors="text_or_none")
-    def encode(self, space, w_input, errors=None):
+    @unwrap_spec(input=unicode, errors="str_or_None")
+    def encode(self, space, input, errors=None):
         if errors is None:
             errors = 'strict'
         state = space.fromcache(CodecState)
-        input, length = space.utf8_len_w(w_input)
         #
         try:
-            output = c_codecs.encode(self.codec, input, length, errors,
+            output = c_codecs.encode(self.codec, input, errors,
                                      state.encode_error_handler, self.name)
         except c_codecs.EncodeDecodeError as e:
-            raise wrap_unicodeencodeerror(space, e, input, length,
-                                          self.name)
+            raise wrap_unicodeencodeerror(space, e, input, self.name)
         except RuntimeError:
             raise wrap_runtimeerror(space)
         return space.newtuple([space.newbytes(output),
-                               space.newint(length)])
+                               space.wrap(len(input))])
 
 
 MultibyteCodec.typedef = TypeDef(
@@ -58,35 +52,34 @@ MultibyteCodec.typedef = TypeDef(
 MultibyteCodec.typedef.acceptable_as_base_class = False
 
 
-@unwrap_spec(name='text')
+@unwrap_spec(name=str)
 def getcodec(space, name):
     try:
         codec = c_codecs.getcodec(name)
     except KeyError:
         raise oefmt(space.w_LookupError, "no such codec is supported.")
-    return MultibyteCodec(name, codec)
+    return space.wrap(MultibyteCodec(name, codec))
 
 
 def wrap_unicodedecodeerror(space, e, input, name):
     return OperationError(
         space.w_UnicodeDecodeError,
         space.newtuple([
-            space.newtext(name),
+            space.wrap(name),
             space.newbytes(input),
-            space.newint(e.start),
-            space.newint(e.end),
-            space.newtext(e.reason)]))
+            space.wrap(e.start),
+            space.wrap(e.end),
+            space.wrap(e.reason)]))
 
-def wrap_unicodeencodeerror(space, e, input, inputlen, name):
-    assert inputlen >= 0
+def wrap_unicodeencodeerror(space, e, input, name):
     raise OperationError(
         space.w_UnicodeEncodeError,
         space.newtuple([
-            space.newtext(name),
-            space.newutf8(input, inputlen),
-            space.newint(e.start),
-            space.newint(e.end),
-            space.newtext(e.reason)]))
+            space.wrap(name),
+            space.wrap(input),
+            space.wrap(e.start),
+            space.wrap(e.end),
+            space.wrap(e.reason)]))
 
 def wrap_runtimeerror(space):
     raise oefmt(space.w_RuntimeError, "internal codec error")
