@@ -20,9 +20,9 @@ if os.name == 'nt':
     def _getfunc(space, CDLL, w_name, w_argtypes, w_restype):
         argtypes_w, argtypes, w_restype, restype = unpack_argtypes(
             space, w_argtypes, w_restype)
-        if space.isinstance_w(w_name, space.w_text):
-            # XXX: support LoadLibraryW
-            name = space.text_w(w_name)
+        if (space.isinstance_w(w_name, space.w_str) or
+                space.isinstance_w(w_name, space.w_unicode)):
+            name = space.str_w(w_name)
             try:
                 func = CDLL.cdll.getpointer(name, argtypes, restype,
                                             flags = CDLL.flags)
@@ -52,9 +52,9 @@ if os.name == 'nt':
             raise oefmt(space.w_TypeError,
                         "function name must be a string or integer")
 else:
-    @unwrap_spec(name='text')
+    @unwrap_spec(name=str)
     def _getfunc(space, CDLL, w_name, w_argtypes, w_restype):
-        name = space.text_w(w_name)
+        name = space.str_w(w_name)
         argtypes_w, argtypes, w_restype, restype = unpack_argtypes(
             space, w_argtypes, w_restype)
         try:
@@ -114,7 +114,7 @@ class W_FuncPtr(W_Root):
         try:
             return func_caller.do_and_wrap(self.w_restype)
         except StackCheckError as e:
-            raise OperationError(space.w_ValueError, space.newtext(e.message))
+            raise OperationError(space.w_ValueError, space.wrap(e.message))
         #return self._do_call(space, argchain)
 
     def free_temp_buffers(self, space):
@@ -129,7 +129,7 @@ class W_FuncPtr(W_Root):
         """
         Return the physical address in memory of the function
         """
-        return space.newint(rffi.cast(rffi.SIGNED, self.func.funcsym))
+        return space.wrap(rffi.cast(rffi.LONG, self.func.funcsym))
 
 
 class PushArgumentConverter(FromAppLevelConverter):
@@ -164,13 +164,13 @@ class PushArgumentConverter(FromAppLevelConverter):
     def handle_char_p(self, w_ffitype, w_obj, strval):
         buf = rffi.str2charp(strval)
         self.w_func.to_free.append(rffi.cast(rffi.VOIDP, buf))
-        addr = rffi.cast(lltype.Unsigned, buf)
+        addr = rffi.cast(rffi.ULONG, buf)
         self.argchain.arg(addr)
 
-    def handle_unichar_p(self, w_ffitype, w_obj, utf8val, utf8len):
-        buf = rffi.utf82wcharp(utf8val, utf8len)
+    def handle_unichar_p(self, w_ffitype, w_obj, unicodeval):
+        buf = rffi.unicode2wcharp(unicodeval)
         self.w_func.to_free.append(rffi.cast(rffi.VOIDP, buf))
-        addr = rffi.cast(lltype.Unsigned, buf)
+        addr = rffi.cast(rffi.ULONG, buf)
         self.argchain.arg(addr)
 
     def handle_float(self, w_ffitype, w_obj, floatval):
@@ -212,48 +212,39 @@ class CallFunctionConverter(ToAppLevelConverter):
         # the result buffer may contains garbage in its higher bits.  To get
         # the correct value, and to be sure to handle the signed/unsigned case
         # correctly, we need to cast the result to the correct type.  After
-        # that, we cast it back to SIGNED, because this is what we want to pass
-        # to space.newint in order to get a nice applevel <int>.
+        # that, we cast it back to LONG, because this is what we want to pass
+        # to space.wrap in order to get a nice applevel <int>.
         #
         restype = w_ffitype.get_ffitype()
         call = self.func.call
         if restype is libffi.types.slong:
-            # cast required for 64-bit Windows
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.LONG))
-        elif restype is libffi.types.slonglong:
-            assert libffi.IS_WIN64
-            # for 64-bit Windows, handled with slong above on other systems
-            x = call(self.argchain, rffi.LONGLONG)
+            x = call(self.argchain, rffi.LONG)
         elif restype is libffi.types.sint:
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.INT))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.INT))
         elif restype is libffi.types.sshort:
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.SHORT))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.SHORT))
         elif restype is libffi.types.schar:
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.SIGNEDCHAR))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.SIGNEDCHAR))
         else:
             raise self.error(w_ffitype)
         return x
 
     def get_unsigned(self, w_ffitype):
-        return self.func.call(self.argchain, lltype.Unsigned)
+        return self.func.call(self.argchain, rffi.ULONG)
 
     def get_unsigned_which_fits_into_a_signed(self, w_ffitype):
         # the same comment as get_signed apply
         restype = w_ffitype.get_ffitype()
         call = self.func.call
-        if restype is libffi.types.ulong:
-            assert libffi.IS_WIN64
-            # on win64 this is the same as uint on 32-bit below
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.ULONG))
-        elif restype is libffi.types.uint:
+        if restype is libffi.types.uint:
             assert not libffi.IS_32_BIT
             # on 32bit machines, we should never get here, because it's a case
             # which has already been handled by get_unsigned above.
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.UINT))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.UINT))
         elif restype is libffi.types.ushort:
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.USHORT))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.USHORT))
         elif restype is libffi.types.uchar:
-            x = rffi.cast(rffi.SIGNED, call(self.argchain, rffi.UCHAR))
+            x = rffi.cast(rffi.LONG, call(self.argchain, rffi.UCHAR))
         else:
             raise self.error(w_ffitype)
         return x
@@ -261,7 +252,7 @@ class CallFunctionConverter(ToAppLevelConverter):
 
     def get_pointer(self, w_ffitype):
         ptrres = self.func.call(self.argchain, rffi.VOIDP)
-        return rffi.cast(lltype.Unsigned, ptrres)
+        return rffi.cast(rffi.ULONG, ptrres)
 
     def get_char(self, w_ffitype):
         return self.func.call(self.argchain, rffi.UCHAR)
@@ -276,11 +267,11 @@ class CallFunctionConverter(ToAppLevelConverter):
         return self.func.call(self.argchain, rffi.FLOAT)
 
     def get_struct(self, w_ffitype, w_structdescr):
-        addr = self.func.call(self.argchain, rffi.SIGNED, is_struct=True)
+        addr = self.func.call(self.argchain, rffi.LONG, is_struct=True)
         return w_structdescr.fromaddress(self.space, addr)
 
     def get_struct_rawffi(self, w_ffitype, w_structdescr):
-        uintval = self.func.call(self.argchain, lltype.Unsigned, is_struct=True)
+        uintval = self.func.call(self.argchain, rffi.ULONG, is_struct=True)
         return w_structdescr.fromaddress(self.space, uintval)
 
     def get_void(self, w_ffitype):
@@ -296,7 +287,7 @@ def unpack_argtypes(space, w_argtypes, w_restype):
     restype = unwrap_ffitype(space, w_restype, allow_void=True)
     return argtypes_w, argtypes, w_restype, restype
 
-@unwrap_spec(addr=r_uint, name='text', flags=int)
+@unwrap_spec(addr=r_uint, name=str, flags=int)
 def descr_fromaddr(space, w_cls, addr, name, w_argtypes,
                     w_restype, flags=libffi.FUNCFLAG_CDECL):
     argtypes_w, argtypes, w_restype, restype = unpack_argtypes(space,
@@ -323,7 +314,7 @@ W_FuncPtr.typedef = TypeDef(
 # ========================================================================
 
 class W_CDLL(W_Root):
-    def __init__(self, space, name, mode, handle):
+    def __init__(self, space, name, mode):
         self.flags = libffi.FUNCFLAG_CDECL
         self.space = space
         if name is None:
@@ -331,7 +322,7 @@ class W_CDLL(W_Root):
         else:
             self.name = name
         try:
-            self.cdll = libffi.CDLL(name, mode, handle)
+            self.cdll = libffi.CDLL(name, mode)
         except DLOpenError as e:
             raise wrap_dlopenerror(space, e, self.name)
         except OSError as e:
@@ -340,7 +331,7 @@ class W_CDLL(W_Root):
     def getfunc(self, space, w_name, w_argtypes, w_restype):
         return _getfunc(space, self, w_name, w_argtypes, w_restype)
 
-    @unwrap_spec(name='text')
+    @unwrap_spec(name=str)
     def getaddressindll(self, space, name):
         try:
             address_as_uint = rffi.cast(lltype.Unsigned,
@@ -348,14 +339,14 @@ class W_CDLL(W_Root):
         except KeyError:
             raise oefmt(space.w_ValueError,
                         "No symbol %s found in library %s", name, self.name)
-        return space.newint(address_as_uint)
+        return space.wrap(address_as_uint)
 
     def getidentifier(self, space):
-        return space.newint(self.cdll.getidentifier())
+        return space.wrap(self.cdll.getidentifier())
 
-@unwrap_spec(name='fsencode_or_none', mode=int, handle=int)
-def descr_new_cdll(space, w_type, name, mode=-1, handle=0):
-    return W_CDLL(space, name, mode, handle)
+@unwrap_spec(name='str_or_None', mode=int)
+def descr_new_cdll(space, w_type, name, mode=-1):
+    return space.wrap(W_CDLL(space, name, mode))
 
 
 W_CDLL.typedef = TypeDef(
@@ -368,13 +359,13 @@ W_CDLL.typedef = TypeDef(
     )
 
 class W_WinDLL(W_CDLL):
-    def __init__(self, space, name, mode, handle):
-        W_CDLL.__init__(self, space, name, mode, handle)
+    def __init__(self, space, name, mode):
+        W_CDLL.__init__(self, space, name, mode)
         self.flags = libffi.FUNCFLAG_STDCALL
 
-@unwrap_spec(name='fsencode_or_none', mode=int, handle=int)
-def descr_new_windll(space, w_type, name, mode=-1, handle=0):
-    return W_WinDLL(space, name, mode, handle)
+@unwrap_spec(name='str_or_None', mode=int)
+def descr_new_windll(space, w_type, name, mode=-1):
+    return space.wrap(W_WinDLL(space, name, mode))
 
 
 W_WinDLL.typedef = TypeDef(
@@ -389,4 +380,4 @@ W_WinDLL.typedef = TypeDef(
 # ========================================================================
 
 def get_libc(space):
-    return W_CDLL(space, get_libc_name(), -1, 0)
+    return space.wrap(W_CDLL(space, get_libc_name(), -1))
