@@ -1,53 +1,53 @@
-
-from rpython.rlib import rutf8
-
 from pypy.interpreter.gateway import unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError, oefmt
 
-def create_filter(space, w_category, action, modname):
-    w_modname = space.newtext(modname) if modname is not None else space.w_None
+def create_filter(space, w_category, action):
     return space.newtuple([
-        space.newtext(action), space.w_None, w_category,
-        w_modname, space.newint(0)])
+        space.wrap(action), space.w_None, w_category,
+        space.w_None, space.wrap(0)])
 
 class State:
     def __init__(self, space):
         self.init_filters(space)
         self.w_once_registry = space.newdict()
-        self.w_default_action = space.newtext("default")
-        self.filters_mutated(space)
-
-    def filters_mutated(self, space):
-        self.w_filters_version = space.call_function(space.w_object)
+        self.w_default_action = space.wrap("default")
 
     def init_filters(self, space):
         filters_w = []
 
-        # note: in CPython, all warnings are enabled by default in pydebug mode
+        if (not space.sys.get_flag('py3k_warning') and
+            not space.sys.get_flag('division_warning')):
+            filters_w.append(create_filter(
+                space, space.w_DeprecationWarning, "ignore"))
+
         filters_w.append(create_filter(
-            space, space.w_DeprecationWarning, "default", "__main__"))
+            space, space.w_PendingDeprecationWarning, "ignore"))
         filters_w.append(create_filter(
-            space, space.w_DeprecationWarning, "ignore", None))
+            space, space.w_ImportWarning, "ignore"))
+
+        bytes_warning = space.sys.get_flag('bytes_warning')
+        if bytes_warning > 1:
+            action = "error"
+        elif bytes_warning == 0:
+            action = "ignore"
+        else:
+            action = "default"
         filters_w.append(create_filter(
-            space, space.w_PendingDeprecationWarning, "ignore", None))
-        filters_w.append(create_filter(
-            space, space.w_ImportWarning, "ignore", None))
-        filters_w.append(create_filter(
-            space, space.w_ResourceWarning, "ignore", None))
+            space, space.w_BytesWarning, action))
 
         self.w_filters = space.newlist(filters_w)
 
 def get_warnings_attr(space, name):
     try:
         w_module = space.getitem(space.sys.get('modules'),
-                                 space.newtext('warnings'))
+                                 space.wrap('warnings'))
     except OperationError as e:
         if not e.match(space, space.w_KeyError):
             raise
         return None
 
     try:
-        return space.getattr(w_module, space.newtext(name))
+        return space.getattr(w_module, space.wrap(name))
     except OperationError as e:
         if not e.match(space, space.w_AttributeError):
             raise
@@ -61,88 +61,71 @@ def get_category(space, w_message, w_category):
         w_category = space.w_UserWarning
 
     # Validate category
-    try:
-        if not space.abstract_issubclass_w(w_category, space.w_Warning):
-            raise oefmt(space.w_TypeError,
-                        "category is not a subclass of Warning")
-    except OperationError as e:
-        if e.async(space):
-            raise
-        raise oefmt(space.w_TypeError,
-                    "category must be a Warning subclass, not '%T'",
-                    w_category)
+    if not space.abstract_issubclass_w(w_category, space.w_Warning):
+        raise oefmt(space.w_ValueError,
+                    "category is not a subclass of Warning")
 
     return w_category
 
-def is_internal_frame(space, frame):
-    if frame is None:
-        return False
-    code = frame.getcode()
-    if code is None or code.co_filename is None:
-        return False
-    # XXX XXX HAAAACK copied directly from CPython, which I'm particularly
-    # unhappy about, but I can't do anything more than say "bah"
-    return "importlib" in code.co_filename and "_bootstrap" in code.co_filename
-
-def next_external_frame(space, frame):
-    ec = space.getexecutioncontext()
-    while True:
-        frame = ec.getnextframe_nohidden(frame)
-        if frame is None or not is_internal_frame(space, frame):
-            return frame
-
 def setup_context(space, stacklevel):
     # Setup globals and lineno
     ec = space.getexecutioncontext()
-
-    # Direct copy of CPython's logic, which has grown its own notion of
-    # "internal frames".  xxx not sure I understand this logic.
     frame = ec.gettopframe_nohidden()
-    if stacklevel <= 0 or is_internal_frame(space, frame):
-        while stacklevel > 1 and frame:
-            frame = ec.getnextframe_nohidden(frame)
-            stacklevel -= 1
-    else:
-        while stacklevel > 1 and frame:
-            frame = next_external_frame(space, frame)
-            stacklevel -= 1
-
+    while frame and stacklevel > 1:
+        frame = ec.getnextframe_nohidden(frame)
+        stacklevel -= 1
     if frame:
         w_globals = frame.get_w_globals()
-        w_filename = space.newtext(frame.pycode.co_filename)
         lineno = frame.get_last_lineno()
     else:
         w_globals = space.sys.w_dict
-        w_filename = space.newtext("sys")
         lineno = 1
 
     # setup registry
     try:
-        w_registry = space.getitem(w_globals, space.newtext("__warningregistry__"))
+        w_registry = space.getitem(w_globals, space.wrap("__warningregistry__"))
     except OperationError as e:
         if not e.match(space, space.w_KeyError):
             raise
         w_registry = space.newdict()
-        space.setitem(w_globals, space.newtext("__warningregistry__"), w_registry)
+        space.setitem(w_globals, space.wrap("__warningregistry__"), w_registry)
 
     # setup module
     try:
-        w_module = space.getitem(w_globals, space.newtext("__name__"))
+        w_module = space.getitem(w_globals, space.wrap("__name__"))
     except OperationError as e:
         if not e.match(space, space.w_KeyError):
             raise
-        w_module = space.newtext("<string>")
+        w_module = space.wrap("<string>")
+
+    # setup filename
+    try:
+        w_filename = space.getitem(w_globals, space.wrap("__file__"))
+        filename = space.str_w(w_filename)
+    except OperationError as e:
+        if space.str_w(w_module) == '__main__':
+            w_argv = space.sys.getdictvalue(space, 'argv')
+            if w_argv and space.len_w(w_argv) > 0:
+                w_filename = space.getitem(w_argv, space.wrap(0))
+                if not space.is_true(w_filename):
+                    w_filename = space.wrap('__main__')
+            else:
+                # embedded interpreters don't have sys.argv
+                w_filename = space.wrap('__main__')
+        else:
+            w_filename = w_module
+    else:
+        lc_filename = filename.lower()
+        if (lc_filename.endswith(".pyc") or 
+            lc_filename.endswith(".pyo")):
+            # strip last character
+            w_filename = space.wrap(filename[:-1])
 
     return (w_filename, lineno, w_module, w_registry)
 
 def check_matched(space, w_obj, w_arg):
-    # A 'None' filter always matches
     if space.is_w(w_obj, space.w_None):
         return True
-    # An internal plain text default filter must match exactly
-    if space.is_w(space.type(w_obj), space.w_unicode):
-        return space.eq_w(w_obj, w_arg)
-    # Otherwise assume a regex filter and call its match() method
     return space.is_true(space.call_method(w_obj, "match", w_arg))
 
 def get_filter(space, w_category, w_text, lineno, w_module):
@@ -162,7 +145,7 @@ def get_filter(space, w_category, w_text, lineno, w_module):
             check_matched(space, w_mod, w_module) and
             space.abstract_issubclass_w(w_category, w_cat) and
             (ln == 0 or ln == lineno)):
-            return space.text_w(w_action), w_item
+            return space.str_w(w_action), w_item
 
     action = get_default_action(space)
     if not action:
@@ -172,10 +155,10 @@ def get_filter(space, w_category, w_text, lineno, w_module):
 def get_default_action(space):
     w_action = get_warnings_attr(space, "defaultaction");
     if w_action is None:
-        return space.text_w(space.fromcache(State).w_default_action)
+        return space.str_w(space.fromcache(State).w_default_action)
 
     space.fromcache(State).w_default_action = w_action
-    return space.text_w(w_action)
+    return space.str_w(w_action)
 
 def get_once_registry(space):
     w_registry = get_warnings_attr(space, "onceregistry");
@@ -190,60 +173,54 @@ def update_registry(space, w_registry, w_text, w_category):
     return already_warned(space, w_registry, w_key, should_set=True)
 
 def already_warned(space, w_registry, w_key, should_set=False):
-    w_version_obj = space.finditem_str(w_registry, "version")
-    state = space.fromcache(State)
-    if w_version_obj is not state.w_filters_version:
-        space.call_method(w_registry, "clear")
-        space.setitem_str(w_registry, "version", state.w_filters_version)
+    try:
+        w_warned = space.getitem(w_registry, w_key)
+    except OperationError as e:
+        if not e.match(space, space.w_KeyError):
+            raise
+        if should_set:
+            space.setitem(w_registry, w_key, space.w_True)
+        return False
     else:
-        w_already_warned = space.finditem(w_registry, w_key)
-        if w_already_warned is not None and space.is_true(w_already_warned):
-            return True
-    # This warning wasn't found in the registry, set it.
-    if should_set:
-        space.setitem(w_registry, w_key, space.w_True)
-    return False
+        return space.is_true(w_warned)
 
 def normalize_module(space, w_filename):
-    # XXX: could be more efficient (doesn't necessarily need
-    # fsencoding/redecoding)
-    filename = space.fsencode_w(w_filename)
-    if len(filename) == 0:
-        return space.newtext("<unknown>")
+    if not space.is_true(w_filename):
+        return space.wrap("<unknown>")
+
+    filename = space.str_w(w_filename)
     if filename.endswith(".py"):
         n = len(filename) - 3
         assert n >= 0
         filename = filename[:n]
-        return space.newfilename(filename)
-    return w_filename
+    return space.wrap(filename)
 
 def show_warning(space, w_filename, lineno, w_text, w_category,
                  w_sourceline=None):
-    w_name = space.getattr(w_category, space.newtext("__name__"))
+    w_name = space.getattr(w_category, space.wrap("__name__"))
     w_stderr = space.sys.get("stderr")
 
     # Print "filename:lineno: category: text\n"
-    message = b"%s:%d: %s: %s\n" % (space.utf8_w(w_filename), lineno,
-                                    space.utf8_w(w_name),
-                                    space.utf8_w(w_text))
-    space.call_method(w_stderr, "write", space.newtext(message))
+    message = "%s:%d: %s: %s\n" % (space.str_w(w_filename), lineno,
+                                   space.str_w(w_name), space.str_w(w_text))
+    space.call_method(w_stderr, "write", space.wrap(message))
 
     # Print "  source_line\n"
     if not w_sourceline:
         try:
             # sourceline = linecache.getline(filename, lineno).strip()
-            w_builtins = space.getbuiltinmodule('builtins')
+            w_builtins = space.getbuiltinmodule('__builtin__')
             w_linecachemodule = space.call_method(w_builtins, '__import__',
-                                                  space.newtext("linecache"))
+                                                  space.wrap("linecache"))
             w_sourceline = space.call_method(w_linecachemodule, "getline",
-                                             w_filename, space.newint(lineno))
+                                             w_filename, space.wrap(lineno))
             w_sourceline = space.call_method(w_sourceline, "strip")
         except OperationError:
             w_sourceline = None
 
     if not w_sourceline:
         return
-    line = space.utf8_w(w_sourceline)
+    line = space.str_w(w_sourceline)
     if not line:
         return
 
@@ -253,14 +230,14 @@ def show_warning(space, w_filename, lineno, w_text, w_category,
         if c not in ' \t\014':
             message = "  %s\n" % (line[i:],)
             break
-    space.call_method(w_stderr, "write", space.newtext(message))
+    space.call_method(w_stderr, "write", space.wrap(message))
 
-def do_warn(space, w_message, w_category, stacklevel, w_source=None):
+def do_warn(space, w_message, w_category, stacklevel):
     context_w = setup_context(space, stacklevel)
-    do_warn_explicit(space, w_category, w_message, context_w, w_source=w_source)
+    do_warn_explicit(space, w_category, w_message, context_w)
 
 def do_warn_explicit(space, w_category, w_message, context_w,
-                     w_sourceline=None, w_source=None):
+                     w_sourceline=None):
     w_filename, lineno, w_module, w_registry = context_w
 
     # normalize module
@@ -271,15 +248,15 @@ def do_warn_explicit(space, w_category, w_message, context_w,
     if space.isinstance_w(w_message, space.w_Warning):
         w_text = space.str(w_message)
         w_category = space.type(w_message)
-    elif (not space.isinstance_w(w_message, space.w_unicode) and
-          not space.isinstance_w(w_message, space.w_bytes)):
+    elif (not space.isinstance_w(w_message, space.w_unicode) or
+          not space.isinstance_w(w_message, space.w_str)):
         w_text = space.str(w_message)
         w_message = space.call_function(w_category, w_message)
     else:
         w_text = w_message
         w_message = space.call_function(w_category, w_message)
 
-    w_lineno = space.newint(lineno)
+    w_lineno = space.wrap(lineno)
 
     # create key
     w_key = space.newtuple([w_text, w_category, w_lineno])
@@ -293,9 +270,6 @@ def do_warn_explicit(space, w_category, w_message, context_w,
 
     if action == "error":
         raise OperationError(w_category, w_message)
- 
-    if action == 'ignore':
-        return
 
     # Store in the registry that we've been here, *except* when the action is
     # "always".
@@ -303,8 +277,9 @@ def do_warn_explicit(space, w_category, w_message, context_w,
     if action != 'always':
         if not space.is_w(w_registry, space.w_None):
             space.setitem(w_registry, w_key, space.w_True)
-
-        if action == 'once':
+        if action == 'ignore':
+            return
+        elif action == 'once':
             if space.is_w(w_registry, space.w_None):
                 w_registry = get_once_registry(space)
             warned = update_registry(space, w_registry, w_text, w_category)
@@ -313,7 +288,7 @@ def do_warn_explicit(space, w_category, w_message, context_w,
                 warned = update_registry(space, w_registry, w_text, w_category)
         elif action != 'default':
             try:
-                err = space.text_w(space.str(w_item))
+                err = space.str_w(space.str(w_item))
             except OperationError:
                 err = "???"
             raise oefmt(space.w_RuntimeError,
@@ -323,32 +298,18 @@ def do_warn_explicit(space, w_category, w_message, context_w,
     if warned:
         # Already warned for this module
         return
-
-    w_show_fn = get_warnings_attr(space, "_showwarnmsg")
-    if w_show_fn is None:
+    w_show_fxn = get_warnings_attr(space, "showwarning")
+    if w_show_fxn is None:
         show_warning(space, w_filename, lineno, w_text, w_category,
                      w_sourceline)
-        return
-
-    if not space.is_true(space.callable(w_show_fn)):
-        raise oefmt(space.w_TypeError,
-                    "warnings._showwarnmsg() must be set to a callable")
-    w_message_cls = get_warnings_attr(space, "WarningMessage")
-    if w_message_cls is None:
-        raise oefmt(space.w_RuntimeError,
-                    "unable to get warnings.WarningMessage")
-    w_source = w_source or space.w_None
-    w_msg = space.call_function(
-        w_message_cls, w_message, w_category,
-        w_filename, w_lineno, space.w_None, space.w_None, w_source)
-    space.call_function(w_show_fn, w_msg)
-
+    else:
+        space.call_function(
+            w_show_fxn, w_message, w_category, w_filename, w_lineno)
 
 @unwrap_spec(stacklevel=int)
-def warn(space, w_message, w_category=None, stacklevel=1, w_source=None):
-    "Issue a warning, or maybe ignore it or raise an exception."
+def warn(space, w_message, w_category=None, stacklevel=1):
     w_category = get_category(space, w_message, w_category);
-    do_warn(space, w_message, w_category, stacklevel, w_source)
+    do_warn(space, w_message, w_category, stacklevel)
 
 
 def get_source_line(space, w_globals, lineno):
@@ -357,8 +318,8 @@ def get_source_line(space, w_globals, lineno):
 
     # Check/get the requisite pieces needed for the loader.
     try:
-        w_loader = space.getitem(w_globals, space.newtext("__loader__"))
-        w_module_name = space.getitem(w_globals, space.newtext("__name__"))
+        w_loader = space.getitem(w_globals, space.wrap("__loader__"))
+        w_module_name = space.getitem(w_globals, space.wrap("__name__"))
     except OperationError as e:
         if not e.match(space, space.w_KeyError):
             raise
@@ -366,7 +327,7 @@ def get_source_line(space, w_globals, lineno):
 
     # Make sure the loader implements the optional get_source() method.
     try:
-        w_get_source = space.getattr(w_loader, space.newtext("get_source"))
+        w_get_source = space.getattr(w_loader, space.wrap("get_source"))
     except OperationError as e:
         if not e.match(space, space.w_AttributeError):
             raise
@@ -378,25 +339,20 @@ def get_source_line(space, w_globals, lineno):
         return None
 
     # Split the source into lines.
-    w_source_list = space.call_method(space.w_text, "splitlines", w_source)
+    w_source_list = space.call_method(w_source, "splitlines")
 
     # Get the source line.
-    w_source_line = space.getitem(w_source_list, space.newint(lineno - 1))
+    w_source_line = space.getitem(w_source_list, space.wrap(lineno - 1))
     return w_source_line
 
 @unwrap_spec(lineno=int, w_module = WrappedDefault(None),
              w_registry = WrappedDefault(None),
              w_module_globals = WrappedDefault(None))
 def warn_explicit(space, w_message, w_category, w_filename, lineno,
-                  w_module=None, w_registry=None, w_module_globals=None,
-                  w_source=None):
-    "Low-level inferface to warnings functionality."
+                  w_module=None, w_registry=None, w_module_globals=None):
 
     w_source_line = get_source_line(space, w_module_globals, lineno)
 
     do_warn_explicit(space, w_category, w_message,
                      (w_filename, lineno, w_module, w_registry),
-                     w_source_line, w_source)
-
-def filters_mutated(space):
-    space.fromcache(State).filters_mutated(space)
+                     w_source_line)
