@@ -3,7 +3,7 @@ import pytest
 
 
 class AppTestArray(object):
-    spaceconfig = {'usemodules': ['array', 'struct', 'binascii']}
+    spaceconfig = {'usemodules': ['array', 'struct', '_rawffi', 'binascii']}
 
     def setup_class(cls):
         cls.w_array = cls.space.appexec([], """():
@@ -15,13 +15,14 @@ class AppTestArray(object):
         cls.w_maxint = cls.space.wrap(sys.maxint)
 
     def test_ctor(self):
+        assert len(self.array('c')) == 0
         assert len(self.array('i')) == 0
 
         raises(TypeError, self.array, 'hi')
         raises(TypeError, self.array, 1)
-        raises(ValueError, self.array, 'x')
+        raises(ValueError, self.array, 'q')
 
-        a = self.array('u')
+        a = self.array('c')
         raises(TypeError, a.append, 7)
         raises(TypeError, a.append, 'hi')
         a.append('h')
@@ -29,11 +30,30 @@ class AppTestArray(object):
         assert type(a[0]) is str
         assert len(a) == 1
 
+        a = self.array('u')
+        raises(TypeError, a.append, 7)
+        raises(TypeError, a.append, u'hi')
+        a.append(u'h')
+        assert a[0] == u'h'
+        assert type(a[0]) is unicode
+        assert len(a) == 1
+
+        a = self.array('c', ('a', 'b', 'c'))
+        assert a[0] == 'a'
+        assert a[1] == 'b'
+        assert a[2] == 'c'
+        assert len(a) == 3
+
+        b = self.array('c', a)
+        assert len(b) == 3
+        assert a == b
+        raises(TypeError, self.array, 'i', a)
+
         a = self.array('i', (1, 2, 3))
         b = self.array('h', (1, 2, 3))
         assert a == b
 
-        for tc in 'bhilBHILQqfd':
+        for tc in 'bhilBHILfd':
             assert self.array(tc).typecode == tc
             raises(TypeError, self.array, tc, None)
 
@@ -57,9 +77,9 @@ class AppTestArray(object):
                            ('h', (-32768, 30535, 32767),  int),
                            ('H', (     0, 56783, 65535),  int),
                            ('i', (-32768, 30535, 32767),  int),
-                           ('I', (     0, 56783, 65535), int),
+                           ('I', (     0, 56783, 65535), long),
                            ('l', (-2 ** 32 // 2, 34, 2 ** 32 // 2 - 1),  int),
-                           ('L', (0, 3523532, 2 ** 32 - 1), int),
+                           ('L', (0, 3523532, 2 ** 32 - 1), long),
                            ):
             a = self.array(tc, ok)
             assert len(a) == len(ok)
@@ -69,10 +89,9 @@ class AppTestArray(object):
                 assert a[i] == v
                 assert type(a[i]) is pt or (
                     # A special case: we return ints in Array('I') on 64-bits,
-                    # and in Array('L') on 64-bit Windows,
                     # whereas CPython returns longs.  The difference is
                     # probably acceptable.
-                    (tc == 'I' or tc == 'L' and sys.platform == 'win32') and
+                    tc == 'I' and
                     sys.maxint > 2147483647 and type(a[i]) is int)
             for v in ok:
                 a[1] = v
@@ -89,7 +108,7 @@ class AppTestArray(object):
                 except OverflowError:
                     pass
 
-        for tc in 'BHILQ':
+        for tc in 'BHIL':
             a = self.array(tc)
             itembits = a.itemsize * 8
             vals = [0, 2 ** itembits - 1]
@@ -115,20 +134,8 @@ class AppTestArray(object):
             assert a[2] == 2.5
             assert len(a) == len(values)
 
-    def test_nan(self):
-        for tc in 'fd':
-            a = self.array(tc, [float('nan')])
-            b = self.array(tc, [float('nan')])
-            assert not a == b
-            assert a != b
-            assert not a > b
-            assert not a >= b
-            assert not a < b
-            assert not a <= b
-            assert a.count(float('nan')) == 0
-
     def test_itemsize(self):
-        for t in 'bB':
+        for t in 'cbB':
             assert(self.array(t).itemsize >= 1)
         for t in 'uhHiI':
             assert(self.array(t).itemsize >= 2)
@@ -136,8 +143,6 @@ class AppTestArray(object):
             assert(self.array(t).itemsize >= 4)
         for t in 'd':
             assert(self.array(t).itemsize >= 8)
-        for t in 'Qq':
-            assert(self.array(t).itemsize >= 8)
 
         inttypes = 'bhil'
         for t in inttypes:
@@ -157,52 +162,34 @@ class AppTestArray(object):
             raises(OverflowError, a.append, -1)
             raises(OverflowError, a.append, 2 ** (8 * b))
 
-    def test_errormessage(self):
-        a = self.array("L", [1, 2, 3])
-        excinfo = raises(TypeError, "a[0] = 'abc'")
-        assert str(excinfo.value) == "array item must be integer"
-
     def test_fromstring(self):
-        a = self.array('b')
-        assert not hasattr("a", "fromstring")
-
-    def test_frombytes(self):
         import sys
+
+        a = self.array('c')
+        a.fromstring('Hi!')
+        assert a[0] == 'H' and a[1] == 'i' and a[2] == '!' and len(a) == 3
+        a = self.array('c')
+        a.fromstring(buffer('xyz'))
+        exc = raises(TypeError, a.fromstring, memoryview('xyz'))
+        assert str(exc.value) == "must be string or read-only buffer, not memoryview"
+        assert a[0] == 'x' and a[1] == 'y' and a[2] == 'z' and len(a) == 3
+        a = self.array('c')
+        a.fromstring('')
+        assert not len(a)
+
         for t in 'bBhHiIlLfd':
             a = self.array(t)
-            a.frombytes(b'\x00' * a.itemsize * 2)
+            a.fromstring('\x00' * a.itemsize * 2)
             assert len(a) == 2 and a[0] == 0 and a[1] == 0
             if a.itemsize > 1:
-                raises(ValueError, a.frombytes, b'\x00' * (a.itemsize - 1))
-                raises(ValueError, a.frombytes, b'\x00' * (a.itemsize + 1))
-                raises(ValueError, a.frombytes, b'\x00' * (2 * a.itemsize - 1))
-                raises(ValueError, a.frombytes, b'\x00' * (2 * a.itemsize + 1))
+                raises(ValueError, a.fromstring, '\x00' * (a.itemsize - 1))
+                raises(ValueError, a.fromstring, '\x00' * (a.itemsize + 1))
+                raises(ValueError, a.fromstring, '\x00' * (2 * a.itemsize - 1))
+                raises(ValueError, a.fromstring, '\x00' * (2 * a.itemsize + 1))
             b = self.array(t, b'\x00' * a.itemsize * 2)
             assert len(b) == 2 and b[0] == 0 and b[1] == 0
-            if t in 'bB':
-                old_items = a.tolist()
-                try:
-                    a.frombytes(a)
-                except BufferError:
-                    # CPython behavior:
-                    # "cannot resize an array that is exporting buffers"
-                    # This is the general error we get when we try to
-                    # resize the array while a buffer to that array is
-                    # alive.
-                    assert a.tolist() == old_items
-                else:
-                    # PyPy behavior: we can't reasonably implement that.
-                    # It's harder to crash PyPy in this case, but not
-                    # impossible, because of get_raw_address().  Too
-                    # bad I suppose.
-                    assert a.tolist() == old_items * 2
-            else:
-                if '__pypy__' in sys.modules:
-                    old_items = a.tolist()
-                    a.frombytes(a)
-                    assert a.tolist() == old_items * 2
-                else:
-                    raises(TypeError, a.frombytes, a)
+            if sys.version_info >= (2, 7, 11):
+                raises(ValueError, a.fromstring, a)
 
     def test_fromfile(self):
         def myfile(c, s):
@@ -225,27 +212,10 @@ class AppTestArray(object):
         a.fromfile(myfile(b'\x01', 20), 2)
         assert len(a) == 2 and a[0] == 257 and a[1] == 257
 
-        a = self.array('h')
-        raises(EOFError, a.fromfile, myfile(b'\x01', 2), 2)
-        assert len(a) == 1 and a[0] == 257
-
-        a = self.array('h')
-        raises(ValueError, a.fromfile, myfile(b'\x01', 3), 2)
-        # ValueError: bytes length not a multiple of item size
-        assert len(a) == 0
-
-    def test_fromfile_no_warning(self):
-        import warnings
-        # check that fromfile defers to frombytes, not fromstring
-        class FakeF(object):
-            def read(self, n):
-                return b"a" * n
-        a = self.array('b')
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-            a.fromfile(FakeF(), 4)
-            assert len(w) == 0
+        for i in (0, 1):
+            a = self.array('h')
+            raises(EOFError, a.fromfile, myfile(b'\x01', 2 + i), 2)
+            assert len(a) == 1 and a[0] == 257
 
     def test_fromlist(self):
         a = self.array('b')
@@ -292,12 +262,6 @@ class AppTestArray(object):
         b = self.array('u', u'hi')
         assert len(b) == 2 and b[0] == 'h' and b[1] == 'i'
 
-    def test_setslice_to_extend(self):
-        a = self.array('i')
-        a[0:1] = self.array('i', [9])
-        a[1:5] = self.array('i', [99])
-        assert list(a) == [9, 99]
-
     def test_sequence(self):
         a = self.array('i', [1, 2, 3, 4])
         assert len(a) == 4
@@ -342,6 +306,11 @@ class AppTestArray(object):
         raises(TypeError, "a[1:3] = self.array('I', [5, 6])")
         raises(TypeError, "a[1:3] = [5, 6]")
 
+        a = self.array('i', [1, 2, 3])
+        assert a.__getslice__(1, 2) == a[1:2]
+        a.__setslice__(1, 2, self.array('i', (7,)))
+        assert a[0] == 1 and a[1] == 7 and a[2] == 3
+
     def test_resizingslice(self):
         a = self.array('i', [1, 2, 3])
         a[1:2] = self.array('i', [7, 8, 9])
@@ -374,7 +343,6 @@ class AppTestArray(object):
         assert repr(a[2:1:-1]) == "array('i', [20])"
         assert repr(a[2:-1:-1]) == "array('i')"
         assert repr(a[-1:0:-1]) == "array('i', [20, 21])"
-        del a
 
         for a in range(-4, 5):
             for b in range(-4, 5):
@@ -392,20 +360,6 @@ class AppTestArray(object):
                             assert repr(arr) == repr(self.array('i', lst))
                         except ValueError:
                             assert not ok
-                    del arr
-        # make sure array.__del__ is called before the leak check
-        import gc; gc.collect()
-
-    def test_getslice_large_step(self):
-        import sys
-        a = self.array('b', [1, 2, 3])
-        assert list(a[1::sys.maxsize]) == [2]
-
-    def test_setslice_large_step(self):
-        import sys
-        a = self.array('b', [1, 2, 3])
-        a[1::sys.maxsize] = self.array('b', [42])
-        assert a.tolist() == [1, 42, 3]
 
     def test_toxxx(self):
         a = self.array('i', [1, 2, 3])
@@ -413,22 +367,23 @@ class AppTestArray(object):
         assert type(l) is list and len(l) == 3
         assert a[0] == 1 and a[1] == 2 and a[2] == 3
 
-        b = self.array('i', a.tobytes())
+        b = self.array('i', a.tostring())
         assert len(b) == 3 and b[0] == 1 and b[1] == 2 and b[2] == 3
 
+        assert self.array('c', ('h', 'i')).tostring() == 'hi'
         a = self.array('i', [0, 0, 0])
-        assert a.tobytes() == b'\x00' * 3 * a.itemsize
-        s = self.array('i', [1, 2, 3]).tobytes()
-        assert 0x00 in s
-        assert 0x01 in s
-        assert 0x02 in s
-        assert 0x03 in s
+        assert a.tostring() == '\x00' * 3 * a.itemsize
+        s = self.array('i', [1, 2, 3]).tostring()
+        assert '\x00' in s
+        assert '\x01' in s
+        assert '\x02' in s
+        assert '\x03' in s
         a = self.array('i', s)
         assert a[0] == 1 and a[1] == 2 and a[2] == 3
 
         from struct import unpack
         values = (-129, 128, -128, 127, 0, 255, -1, 256, -32760, 32760)
-        s = self.array('i', values).tobytes()
+        s = self.array('i', values).tostring()
         fmt = 'i' * len(values)
         a = unpack(fmt, s)
         assert a == values
@@ -438,65 +393,45 @@ class AppTestArray(object):
                                ('hilHILfd', (32760, 30123, 3422, 23244))):
             for tc in tcodes:
                 values += ((2 ** self.array(tc).itemsize) // 2 - 1, )
-                s = self.array(tc, values).tobytes()
+                s = self.array(tc, values).tostring()
                 a = unpack(tc * len(values), s)
                 assert a == values
 
-        f = open(self.tempfile, 'wb')
-        self.array('b', (ord('h'), ord('i'))).tofile(f)
+        f = open(self.tempfile, 'w')
+        self.array('c', ('h', 'i')).tofile(f)
         f.close()
-        assert open(self.tempfile, 'rb').readline() == b'hi'
+        assert open(self.tempfile, 'r').readline() == 'hi'
 
-        a = self.array('b')
-        a.fromfile(open(self.tempfile, 'rb'), 2)
-        assert repr(a) == "array('b', [104, 105])"
+        a = self.array('c')
+        a.fromfile(open(self.tempfile, 'r'), 2)
+        assert repr(a) == "array('c', 'hi')"
 
         raises(ValueError, self.array('i').tounicode)
         assert self.array('u', u'hello').tounicode() == u'hello'
 
     def test_empty_tostring(self):
         a = self.array('l')
-        assert not hasattr(a, "tostring")
+        assert a.tostring() == b''
 
     def test_buffer(self):
-        a = self.array('h', b'Hi')
-        buf = memoryview(a)
-        assert buf[0] == 26952
-        raises(IndexError, 'buf[1]')
-        assert buf.tobytes() == b'Hi'
-        assert buf.tolist() == [26952]
-        assert buf.format == 'h'
-        assert buf.itemsize == 2
-        assert buf.shape == (1,)
-        assert buf.ndim == 1
-        assert buf.strides == (2,)
-        assert not buf.readonly
+        a = self.array('h', 'Hi')
+        buf = buffer(a)
+        assert buf[1] == 'i'
 
     def test_buffer_write(self):
-        a = self.array('b', b'hello')
-        buf = memoryview(a)
-        try:
-            buf[3] = b'L'
-        except TypeError:
-            skip("memoryview(array) returns a read-only buffer on CPython")
-        assert a.tobytes() == b'helLo'
+        a = self.array('c', 'hello')
+        buf = buffer(a)
+        exc = raises(TypeError, "buf[3] = 'L'")
+        assert str(exc.value) == "buffer is read-only"
 
     def test_buffer_keepalive(self):
-        import sys
-        if '__pypy__' not in sys.builtin_module_names:
-            skip("CPython: cannot resize an array that is exporting buffers")
-        buf = memoryview(self.array('b', b'text'))
-        assert buf[2] == ord('x')
+        buf = buffer(self.array('c', 'text'))
+        assert buf[2] == 'x'
         #
-        a = self.array('b', b'foobarbaz')
-        buf = memoryview(a)
-        a.frombytes(b'some extra text')
-        assert buf[:] == b'foobarbazsome extra text'
-
-    def test_memview_multi_tobytes(self):
-        a = self.array('i', list(b"abcdef"))
-        m = memoryview(a)
-        assert m.tobytes() == a.tobytes()
+        a = self.array('c', 'foobarbaz')
+        buf = buffer(a)
+        a.fromstring('some extra text')
+        assert buf[:] == 'foobarbazsome extra text'
 
     def test_list_methods(self):
         assert repr(self.array('i')) == "array('i')"
@@ -550,7 +485,8 @@ class AppTestArray(object):
             pass
 
         for v1, v2, tt in (([1, 2, 3], [1, 3, 2], 'bhilBHIL'),
-                         ('abc', 'acb', 'u')):
+                         ('abc', 'acb', 'c'),
+                         (u'abc', u'acb', 'u')):
             for t in tt:
                 a = self.array(t, v1)
                 b = self.array(t, v1)
@@ -609,18 +545,18 @@ class AppTestArray(object):
     def test_reduce(self):
         import pickle
         a = self.array('i', [1, 2, 3])
-        s = pickle.dumps(a)
+        s = pickle.dumps(a, 1)
         b = pickle.loads(s)
         assert a == b
 
         a = self.array('l')
-        s = pickle.dumps(a)
+        s = pickle.dumps(a, 1)
         b = pickle.loads(s)
         assert len(b) == 0 and b.typecode == 'l'
 
         a = self.array('i', [1, 2, 4])
         i = iter(a)
-        #raises(TypeError, pickle.dumps, i)
+        #raises(TypeError, pickle.dumps, i, 1)
 
     def test_copy_swap(self):
         a = self.array('i', [1, 2, 3])
@@ -642,12 +578,6 @@ class AppTestArray(object):
             assert a[1] == 2
             assert a[2] == 3
 
-    def test_deepcopy(self):
-        a = self.array('u', u'\x01\u263a\x00\ufeff')
-        from copy import deepcopy
-        b = deepcopy(a)
-        assert a == b
-
     def test_addmul(self):
         a = self.array('i', [1, 2, 3])
         assert repr(a + a) == "array('i', [1, 2, 3, 1, 2, 3])"
@@ -736,6 +666,9 @@ class AppTestArray(object):
         del a[1:3]
         assert repr(a) == "array('i', [1, 4, 5])"
 
+        a.__delslice__(0, 2)
+        assert repr(a) == "array('i', [5])"
+
         a = self.array('i', [1, 2, 3, 4, 5])
         del a[3:1]
         assert repr(a) == "array('i', [1, 2, 3, 4, 5])"
@@ -762,16 +695,6 @@ class AppTestArray(object):
         assert hasattr(b, '__iter__')
         assert next(b.__iter__()) == 1
 
-    def test_contains(self):
-        a = self.array('i', [1, 6, 3])
-        assert hasattr(a, '__contains__')
-        assert (1 in a) is True
-        assert 6 in a
-        assert 3 in a
-        assert (0 in a) is False
-        assert 12 not in a
-        assert 'a' not in a
-
     def test_lying_iterable(self):
         class lier(object):
             def __init__(self, n):
@@ -780,7 +703,7 @@ class AppTestArray(object):
             def __len__(self):
                 return 3
 
-            def __next__(self):
+            def next(self):
                 self.n -= 1
                 if self.n < 0:
                     raise StopIteration
@@ -800,15 +723,10 @@ class AppTestArray(object):
         assert repr(a) == "array('i', [4, 3, 2, 1, 0])"
 
     def test_type(self):
-        for t in 'bBhHiIlLfduQq':
+        for t in 'bBhHiIlLfdcu':
             assert type(self.array(t)) is self.array
             assert isinstance(self.array(t), self.array)
 
-    def test_iterable(self):
-        import collections
-        for t in 'bBhHiIlLfduQq':
-            assert isinstance(self.array(t), collections.Iterable)
-
     def test_subclass(self):
         assert len(self.array('b')) == 0
 
@@ -861,85 +779,77 @@ class AppTestArray(object):
             def fromlist(self, lst):
                 self.append(7)
 
-            def frombytes(self, lst):
-                self.append(8)
+            def fromstring(self, lst):
+                self.append('8')
 
             def fromunicode(self, lst):
-                self.append('9')
+                self.append(u'9')
 
             def extend(self, lst):
                 self.append(10)
 
-        assert repr(mya('u', 'hi')) == "mya('u', 'hi')"
-        assert repr(mya('i', [1, 2, 3])) == "mya('i', [1, 2, 3])"
-        assert repr(mya('i', (1, 2, 3))) == "mya('i', [1, 2, 3])"
+        assert repr(mya('c', 'hi')) == "array('c', 'hi')"
+        assert repr(mya('u', u'hi')) == "array('u', u'hi')"
+        assert repr(mya('i', [1, 2, 3])) == "array('i', [1, 2, 3])"
+        assert repr(mya('i', (1, 2, 3))) == "array('i', [1, 2, 3])"
 
         a = mya('i')
         a.fromlist([1, 2, 3])
-        assert repr(a) == "mya('i', [7])"
+        assert repr(a) == "array('i', [7])"
 
-        a = mya('b')
-        a.frombytes(b'hi')
-        assert repr(a) == "mya('b', [8])"
+        a = mya('c')
+        a.fromstring('hi')
+        assert repr(a) == "array('c', '8')"
 
         a = mya('u')
-        a.fromunicode('hi')
-        assert repr(a) == "mya('u', '9')"
+        a.fromunicode(u'hi')
+        assert repr(a) == "array('u', u'9')"
 
         a = mya('i')
         a.extend([1, 2, 3])
-        assert repr(a) == "mya('i', [10])"
+        assert repr(a) == "array('i', [10])"
 
     def test_override_to(self):
         class mya(self.array):
             def tolist(self):
                 return 'list'
 
-            def tobytes(self):
+            def tostring(self):
                 return 'str'
 
             def tounicode(self):
                 return 'unicode'
 
         assert mya('i', [1, 2, 3]).tolist() == 'list'
-        assert mya('u', 'hi').tobytes() == 'str'
-        assert mya('u', 'hi').tounicode() == 'unicode'
+        assert mya('c', 'hi').tostring() == 'str'
+        assert mya('u', u'hi').tounicode() == 'unicode'
 
-        assert repr(mya('u', 'hi')) == "mya('u', 'hi')"
-        assert repr(mya('i', [1, 2, 3])) == "mya('i', [1, 2, 3])"
-        assert repr(mya('i', (1, 2, 3))) == "mya('i', [1, 2, 3])"
+        assert repr(mya('c', 'hi')) == "array('c', 'hi')"
+        assert repr(mya('u', u'hi')) == "array('u', u'hi')"
+        assert repr(mya('i', [1, 2, 3])) == "array('i', [1, 2, 3])"
+        assert repr(mya('i', (1, 2, 3))) == "array('i', [1, 2, 3])"
 
     def test_unicode_outofrange(self):
-        input_unicode = u'\x01\u263a\x00\ufeff'
-        a = self.array('u', input_unicode)
-        b = self.array('u', input_unicode)
+        a = self.array('u', u'\x01\u263a\x00\ufeff')
+        b = self.array('u', u'\x01\u263a\x00\ufeff')
         b.byteswap()
-        assert b[2] == u'\u0000'
         assert a != b
-        if b.itemsize == 4:
-            e = raises(ValueError, "b[0]")        # doesn't work
-            assert str(e.value) == (
-                "cannot operate on this array('u') because it contains"
-                " character U+1000000 not in range [U+0000; U+10ffff]"
-                " at index 0")
-            assert str(b) == ("array('u', <character U+1000000 is not in"
-                          " range [U+0000; U+10ffff]>)")
-            raises(ValueError, b.tounicode)   # doesn't work
-        elif b.itemsize == 2:
-            assert b[0] == u'\u0100'
-            byteswaped_unicode = u'\u0100\u3a26\x00\ufffe'
-            assert str(b) == "array('u', %r)" % (byteswaped_unicode,)
-            assert b.tounicode() == byteswaped_unicode
-        assert str(a) == "array('u', %r)" % (input_unicode,)
-        assert a.tounicode() == input_unicode
 
-    def test_unicode_surrogate(self):
-        a = self.array('u', u'\ud800')
-        assert a[0] == u'\ud800'
+    def test_unicode_ord_positive(self):
+        import sys
+        if sys.maxunicode == 0xffff:
+            skip("test for 32-bit unicodes")
+        a = self.array('u', b'\xff\xff\xff\xff')
+        assert len(a) == 1
+        assert repr(a[0]) == "u'\Uffffffff'"
+        if sys.maxint == 2147483647:
+            assert ord(a[0]) == -1
+        else:
+            assert ord(a[0]) == 4294967295
 
     def test_weakref(self):
         import weakref
-        a = self.array('u', 'Hi!')
+        a = self.array('c', 'Hi!')
         r = weakref.ref(a)
         assert r() is a
 
@@ -1021,7 +931,7 @@ class AppTestArray(object):
             raises(TypeError, a.__setitem__, Silly())
             raises(TypeError, a.__setitem__, OldSilly())
 
-        a = array('u', 'hi')
+        a = array('c', 'hi')
         a[0] = 'b'
         assert a[0] == 'b'
 
@@ -1029,18 +939,15 @@ class AppTestArray(object):
         a[0] = u'b'
         assert a[0] == u'b'
 
-    def test_bytearray(self):
-        a = self.array('u', 'hi')
-        b = self.array('u')
-        b.frombytes(bytearray(a.tobytes()))
-        assert a == b
-        assert self.array('u', bytearray(a.tobytes())) == a
 
     def test_buffer_info(self):
-        a = self.array('b', b'Hi!')
+        a = self.array('c', 'Hi!')
         bi = a.buffer_info()
         assert bi[0] != 0
         assert bi[1] == 3
+        import _rawffi
+        data = _rawffi.charp2string(bi[0])
+        assert data[0:3] == 'Hi!'
 
     def test_array_reverse_slice_assign_self(self):
         a = self.array('b', range(4))
@@ -1107,158 +1014,5 @@ class AppTestArray(object):
         raises(TypeError, "a[MyInt(0)]")
         raises(TypeError, "a[MyInt(0):MyInt(5)]")
 
-    def test_fresh_array_buffer_bytes(self):
-        assert bytes(memoryview(self.array('i'))) == b''
-
-    def test_mview_slice_aswritebuf(self):
-        import struct
-        a = self.array('B', b'abcdef')
-        view = memoryview(a)[1:5]
-        struct.pack_into('>H', view, 1, 0x1234)
-        assert a.tobytes() == b'ab\x12\x34ef'
-
-    def test_subclass_repr(self):
-        import array
-        class subclass(self.array):
-            pass
-        assert repr(subclass('i')) == "subclass('i')"
-
-
-class AppTestArrayReconstructor:
-    spaceconfig = dict(usemodules=('array', 'struct'))
-
-    def test_error(self):
-        import array
-        array_reconstructor = array._array_reconstructor
-        UNKNOWN_FORMAT = -1
-        raises(TypeError, array_reconstructor,
-               "", "b", 0, b"")
-        raises(TypeError, array_reconstructor,
-               str, "b", 0, b"")
-        raises(TypeError, array_reconstructor,
-               array.array, "b", '', b"")
-        raises(TypeError, array_reconstructor,
-               array.array, "b", 0, "")
-        raises(ValueError, array_reconstructor,
-               array.array, "?", 0, b"")
-        raises(ValueError, array_reconstructor,
-               array.array, "b", UNKNOWN_FORMAT, b"")
-        raises(ValueError, array_reconstructor,
-               array.array, "b", 22, b"")
-        raises(ValueError, array_reconstructor,
-               array.array, "d", 16, b"a")
-
-    def test_numbers(self):
-        import array, struct
-        array_reconstructor = array._array_reconstructor
-        UNSIGNED_INT8 = 0
-        SIGNED_INT8 = 1
-        UNSIGNED_INT16_LE = 2
-        UNSIGNED_INT16_BE = 3
-        SIGNED_INT16_LE = 4
-        SIGNED_INT16_BE = 5
-        UNSIGNED_INT32_LE = 6
-        UNSIGNED_INT32_BE = 7
-        SIGNED_INT32_LE = 8
-        SIGNED_INT32_BE = 9
-        UNSIGNED_INT64_LE = 10
-        UNSIGNED_INT64_BE = 11
-        SIGNED_INT64_LE = 12
-        SIGNED_INT64_BE = 13
-        IEEE_754_FLOAT_LE = 14
-        IEEE_754_FLOAT_BE = 15
-        IEEE_754_DOUBLE_LE = 16
-        IEEE_754_DOUBLE_BE = 17
-        testcases = (
-            (['B', 'H', 'I', 'L'], UNSIGNED_INT8, '=BBBB',
-             [0x80, 0x7f, 0, 0xff]),
-            (['b', 'h', 'i', 'l'], SIGNED_INT8, '=bbb',
-             [-0x80, 0x7f, 0]),
-            (['H', 'I', 'L'], UNSIGNED_INT16_LE, '<HHHH',
-             [0x8000, 0x7fff, 0, 0xffff]),
-            (['H', 'I', 'L'], UNSIGNED_INT16_BE, '>HHHH',
-             [0x8000, 0x7fff, 0, 0xffff]),
-            (['h', 'i', 'l'], SIGNED_INT16_LE, '<hhh',
-             [-0x8000, 0x7fff, 0]),
-            (['h', 'i', 'l'], SIGNED_INT16_BE, '>hhh',
-             [-0x8000, 0x7fff, 0]),
-            (['I', 'L'], UNSIGNED_INT32_LE, '<IIII',
-             [1<<31, (1<<31)-1, 0, (1<<32)-1]),
-            (['I', 'L'], UNSIGNED_INT32_BE, '>IIII',
-             [1<<31, (1<<31)-1, 0, (1<<32)-1]),
-            (['i', 'l'], SIGNED_INT32_LE, '<iii',
-             [-1<<31, (1<<31)-1, 0]),
-            (['i', 'l'], SIGNED_INT32_BE, '>iii',
-             [-1<<31, (1<<31)-1, 0]),
-            (['L'], UNSIGNED_INT64_LE, '<QQQQ',
-             [1<<31, (1<<31)-1, 0, (1<<32)-1]),
-            (['L'], UNSIGNED_INT64_BE, '>QQQQ',
-             [1<<31, (1<<31)-1, 0, (1<<32)-1]),
-            (['l'], SIGNED_INT64_LE, '<qqq',
-             [-1<<31, (1<<31)-1, 0]),
-            (['l'], SIGNED_INT64_BE, '>qqq',
-             [-1<<31, (1<<31)-1, 0]),
-            # The following tests for INT64 will raise an OverflowError
-            # when run on a 32-bit machine. The tests are simply skipped
-            # in that case.
-            (['L'], UNSIGNED_INT64_LE, '<QQQQ',
-             [1<<63, (1<<63)-1, 0, (1<<64)-1]),
-            (['L'], UNSIGNED_INT64_BE, '>QQQQ',
-             [1<<63, (1<<63)-1, 0, (1<<64)-1]),
-            (['l'], SIGNED_INT64_LE, '<qqq',
-             [-1<<63, (1<<63)-1, 0]),
-            (['l'], SIGNED_INT64_BE, '>qqq',
-             [-1<<63, (1<<63)-1, 0]),
-            (['f'], IEEE_754_FLOAT_LE, '<ffff',
-             [16711938.0, float('inf'), float('-inf'), -0.0]),
-            (['f'], IEEE_754_FLOAT_BE, '>ffff',
-             [16711938.0, float('inf'), float('-inf'), -0.0]),
-            (['d'], IEEE_754_DOUBLE_LE, '<dddd',
-             [9006104071832581.0, float('inf'), float('-inf'), -0.0]),
-            (['d'], IEEE_754_DOUBLE_BE, '>dddd',
-             [9006104071832581.0, float('inf'), float('-inf'), -0.0])
-        )
-        for testcase in testcases:
-            valid_typecodes, mformat_code, struct_fmt, values = testcase
-            arraystr = struct.pack(struct_fmt, *values)
-            for typecode in valid_typecodes:
-                try:
-                    a = array.array(typecode, values)
-                except OverflowError:
-                    continue  # Skip this test case.
-                b = array_reconstructor(
-                    array.array, typecode, mformat_code, arraystr)
-                assert a == b
-
-    def test_unicode(self):
-        import array
-        array_reconstructor = array._array_reconstructor
-        UTF16_LE = 18
-        UTF16_BE = 19
-        UTF32_LE = 20
-        UTF32_BE = 21
-        teststr = "Bonne Journ\xe9e \U0002030a\U00020347"
-        testcases = (
-            (UTF16_LE, "UTF-16-LE"),
-            (UTF16_BE, "UTF-16-BE"),
-            (UTF32_LE, "UTF-32-LE"),
-            (UTF32_BE, "UTF-32-BE")
-        )
-        for testcase in testcases:
-            mformat_code, encoding = testcase
-            a = array.array('u', teststr)
-            b = array_reconstructor(
-                array.array, 'u', mformat_code, teststr.encode(encoding))
-            assert a == b
-
-    def test_iterate_iterator(self):
-        import array
-        it = iter(array.array('b'))
-        assert list(it) == []
-        assert list(iter(it)) == []
-
-    def test_array_cannot_use_str(self):
-        import array
-        e = raises(TypeError, array.array, 'i', 'abcd')
-        assert str(e.value) == ("cannot use a str to initialize an array"
-                                " with typecode 'i'")
+    def test_fresh_array_buffer_str(self):
+        assert str(buffer(self.array('i'))) == ''
