@@ -1,19 +1,16 @@
 from pypy.interpreter.error import oefmt
 from pypy.interpreter.astcompiler import consts
 from rpython.rtyper.lltypesystem import rffi, lltype
-from rpython.rlib.objectmodel import we_are_translated
-from rpython.rlib.rarithmetic import widen
 from pypy.module.cpyext.api import (
     cpython_api, CANNOT_FAIL, CONST_STRING, FILEP, fread, feof, Py_ssize_tP,
-    cpython_struct, ferror)
+    cpython_struct, is_valid_fp)
 from pypy.module.cpyext.pyobject import PyObject
 from pypy.module.cpyext.pyerrors import PyErr_SetFromErrno
-from pypy.module.cpyext.frameobject import PyFrameObject
+from pypy.module.cpyext.funcobject import PyCodeObject
 from pypy.module.__builtin__ import compiling
 
 PyCompilerFlags = cpython_struct(
-    "PyCompilerFlags", (("cf_flags", rffi.INT),
-                        ("cf_feature_version", rffi.INT)))
+    "PyCompilerFlags", (("cf_flags", rffi.INT),))
 PyCompilerFlagsPtr = lltype.Ptr(PyCompilerFlags)
 
 PyCF_MASK = (consts.CO_FUTURE_DIVISION |
@@ -34,7 +31,7 @@ def PyEval_GetBuiltins(space):
     caller = space.getexecutioncontext().gettopframe_nohidden()
     if caller is not None:
         w_globals = caller.get_w_globals()
-        w_builtins = space.getitem(w_globals, space.newtext('__builtins__'))
+        w_builtins = space.getitem(w_globals, space.wrap('__builtins__'))
         if not space.isinstance_w(w_builtins, space.w_dict):
             w_builtins = w_builtins.getdict(space)
     else:
@@ -59,12 +56,7 @@ def PyEval_GetGlobals(space):
         return None
     return caller.get_w_globals()    # borrowed ref
 
-@cpython_api([], PyFrameObject, error=CANNOT_FAIL, result_borrowed=True)
-def PyEval_GetFrame(space):
-    caller = space.getexecutioncontext().gettopframe_nohidden()
-    return caller    # borrowed ref, may be null
-
-@cpython_api([PyObject, PyObject, PyObject], PyObject)
+@cpython_api([PyCodeObject, PyObject, PyObject], PyObject)
 def PyEval_EvalCode(space, w_code, w_globals, w_locals):
     """This is a simplified interface to PyEval_EvalCodeEx(), with just
     the code object, and the dictionaries of global and local variables.
@@ -85,24 +77,6 @@ def PyObject_CallObject(space, w_obj, w_arg):
     callable_object(*args)."""
     return space.call(w_obj, w_arg)
 
-@cpython_api([PyObject], PyObject)
-def PyObject_CallNoArgs(space, w_obj):
-    return space.call_function(w_obj)
-
-@cpython_api([PyObject, PyObject], PyObject)
-def PyObject_CallOneArg(space, w_obj, w_arg):
-    return space.call_function(w_obj, w_arg)
-
-@cpython_api([PyObject, PyObject], PyObject)
-def PyObject_CallMethodNoArgs(space, w_self, w_name):
-    name = space.text_w(w_name)
-    return space.call_method(w_self, name)
-
-@cpython_api([PyObject, PyObject, PyObject], PyObject)
-def PyObject_CallMethodOneArg(space, w_self, w_name, w_arg):
-    name = space.text_w(w_name)
-    return space.call_method(w_self, name, w_arg)
-
 @cpython_api([PyObject, PyObject, PyObject], PyObject)
 def PyObject_Call(space, w_obj, w_args, w_kw):
     """
@@ -114,18 +88,12 @@ def PyObject_Call(space, w_obj, w_args, w_kw):
     apply(callable_object, args, kw) or callable_object(*args, **kw)."""
     return space.call(w_obj, w_args, w_kw)
 
-
-@cpython_api([PyObject, PyObject, PyObject], PyObject)
-def PyCFunction_Call(space, w_obj, w_args, w_kw):
-    return space.call(w_obj, w_args, w_kw)
-
 # These constants are also defined in include/eval.h
 Py_single_input = 256
 Py_file_input = 257
 Py_eval_input = 258
-Py_func_type_input = 345
 
-def compile_string(space, source, filename, start, flags=0, feature_version=-1):
+def compile_string(space, source, filename, start, flags=0):
     w_source = space.newbytes(source)
     start = rffi.cast(lltype.Signed, start)
     if start == Py_file_input:
@@ -134,13 +102,10 @@ def compile_string(space, source, filename, start, flags=0, feature_version=-1):
         mode = 'eval'
     elif start == Py_single_input:
         mode = 'single'
-    elif start == Py_func_type_input:
-        mode = 'func_type'
     else:
         raise oefmt(space.w_ValueError,
                     "invalid mode parameter for compilation")
-    return compiling.compile(space, w_source, filename, mode, flags,
-                             _feature_version=feature_version)
+    return compiling.compile(space, w_source, filename, mode, flags)
 
 def run_string(space, source, filename, start, w_globals, w_locals):
     w_code = compile_string(space, source, filename, start)
@@ -176,12 +141,9 @@ def PyRun_StringFlags(space, source, start, w_globals, w_locals, flagsptr):
     source = rffi.charp2str(source)
     if flagsptr:
         flags = rffi.cast(lltype.Signed, flagsptr.c_cf_flags)
-        feature_version = rffi.cast(lltype.Signed, flagsptr.c_cf_feature_version)
     else:
         flags = 0
-        feature_version = -1
-    w_code = compile_string(space, source, "<string>", start, flags=flags,
-                            feature_version=feature_version)
+    w_code = compile_string(space, source, "<string>", start, flags)
     return compiling.eval(space, w_code, w_globals, w_locals)
 
 @cpython_api([FILEP, CONST_STRING, rffi.INT_real, PyObject, PyObject], PyObject)
@@ -191,22 +153,22 @@ def PyRun_File(space, fp, filename, start, w_globals, w_locals):
     BUF_SIZE = 8192
     source = ""
     filename = rffi.charp2str(filename)
-    with rffi.scoped_alloc_buffer(BUF_SIZE) as buf:
+    buf = lltype.malloc(rffi.CCHARP.TO, BUF_SIZE, flavor='raw')
+    if not is_valid_fp(fp):
+        lltype.free(buf, flavor='raw')
+        PyErr_SetFromErrno(space, space.w_IOError)
+        return None
+    try:
         while True:
-            try:
-                count = fread(buf.raw, 1, BUF_SIZE, fp)
-            except OSError:
-                PyErr_SetFromErrno(space, space.w_IOError)
-                return
+            count = fread(buf, 1, BUF_SIZE, fp)
             count = rffi.cast(lltype.Signed, count)
-            source += rffi.charpsize2str(buf.raw, count)
+            source += rffi.charpsize2str(buf, count)
             if count < BUF_SIZE:
-                if ferror(fp):
-                    PyErr_SetFromErrno(space, space.w_IOError)
-                    return
                 if feof(fp):
                     break
                 PyErr_SetFromErrno(space, space.w_IOError)
+    finally:
+        lltype.free(buf, flavor='raw')
     return run_string(space, source, filename, start, w_globals, w_locals)
 
 # Undocumented function!
@@ -265,51 +227,4 @@ def PyEval_MergeCompilerFlags(space, cf):
     cf.c_cf_flags = rffi.cast(rffi.INT, flags)
     return result
 
-@cpython_api([], rffi.INT_real, error=CANNOT_FAIL)
-def Py_GetRecursionLimit(space):
-    from pypy.module.sys.vm import getrecursionlimit
-    return space.int_w(getrecursionlimit(space))
 
-@cpython_api([rffi.INT_real], lltype.Void, error=CANNOT_FAIL)
-def Py_SetRecursionLimit(space, limit):
-    from pypy.module.sys.vm import setrecursionlimit
-    setrecursionlimit(space, widen(limit))
-
-limit = 0 # for testing
-
-@cpython_api([CONST_STRING], rffi.INT_real, error=1)
-def Py_EnterRecursiveCall(space, where):
-    """Marks a point where a recursive C-level call is about to be performed.
-
-    If USE_STACKCHECK is defined, this function checks if the the OS
-    stack overflowed using PyOS_CheckStack().  In this is the case, it
-    sets a MemoryError and returns a nonzero value.
-
-    The function then checks if the recursion limit is reached.  If this is the
-    case, a RuntimeError is set and a nonzero value is returned.
-    Otherwise, zero is returned.
-
-    where should be a string such as " in instance check" to be
-    concatenated to the RuntimeError message caused by the recursion depth
-    limit."""
-    if not we_are_translated():
-        # XXX hack since the stack checks only work translated
-        global limit
-        limit += 1
-        if limit > 10:
-            raise oefmt(space.w_RecursionError,
-                 "maximum recursion depth exceeded%s", rffi.charp2str(where))
-        return 0
-    from rpython.rlib.rstack import stack_almost_full
-    if stack_almost_full():
-        raise oefmt(space.w_RecursionError,
-                 "maximum recursion depth exceeded%s", rffi.charp2str(where))
-    return 0
-
-@cpython_api([], lltype.Void)
-def Py_LeaveRecursiveCall(space):
-    """Ends a Py_EnterRecursiveCall().  Must be called once for each
-    successful invocation of Py_EnterRecursiveCall()."""
-    # A NOP in PyPy
-    if not we_are_translated():
-        limit = 0
