@@ -1,20 +1,19 @@
-from rpython.rlib.objectmodel import we_are_translated
 from rpython.rtyper.lltypesystem import lltype, rffi
-from rpython.rlib.rbigint import rbigint, InvalidSignednessError
-from rpython.rlib.rarithmetic import maxint, widen
 from pypy.module.cpyext.api import (
-    cpython_api, PyObject, build_type_checkers_flags, Py_ssize_t,
-    CONST_STRING, ADDR, CANNOT_FAIL, INTP_real)
-from pypy.interpreter.error import OperationError, oefmt
-from pypy.interpreter.unicodehelper import wcharpsize2utf8
-from pypy.module.cpyext.pyerrors import PyErr_BadInternalCall
+    cpython_api, PyObject, build_type_checkers, Py_ssize_t,
+    CONST_STRING, ADDR, CANNOT_FAIL)
+from pypy.objspace.std.longobject import W_LongObject
+from pypy.interpreter.error import OperationError
+from pypy.module.cpyext.intobject import PyInt_AsUnsignedLongMask
+from rpython.rlib.rbigint import rbigint
 
-PyLong_Check, PyLong_CheckExact = build_type_checkers_flags("Long")
 
-@cpython_api([rffi.LONG], PyObject)
+PyLong_Check, PyLong_CheckExact = build_type_checkers("Long")
+
+@cpython_api([lltype.Signed], PyObject)
 def PyLong_FromLong(space, val):
     """Return a new PyLongObject object from v, or NULL on failure."""
-    return space.newlong(widen(val))
+    return space.newlong(val)
 
 @cpython_api([Py_ssize_t], PyObject)
 def PyLong_FromSsize_t(space, val):
@@ -48,12 +47,6 @@ def PyLong_FromUnsignedLongLong(space, val):
     or NULL on failure."""
     return space.newlong_from_rarith_int(val)
 
-ULONG_MASK = (2 ** (8 * rffi.sizeof(rffi.ULONG)) -1)
-ULONG_MAX = (2 ** (8 * rffi.sizeof(rffi.ULONG)) -1)
-LONG_MAX = (2 ** (8 * rffi.sizeof(rffi.ULONG) - 1) -1)
-LONG_MIN = (-2 ** (8 * rffi.sizeof(rffi.ULONG) - 1))
-need_to_check = maxint > ULONG_MAX
-
 @cpython_api([PyObject], rffi.ULONG, error=-1)
 def PyLong_AsUnsignedLong(space, w_long):
     """
@@ -61,62 +54,26 @@ def PyLong_AsUnsignedLong(space, w_long):
     If pylong is greater than ULONG_MAX, an OverflowError is
     raised."""
     try:
-        val = space.uint_w(w_long)
+        return rffi.cast(rffi.ULONG, space.uint_w(w_long))
     except OperationError as e:
         if e.match(space, space.w_ValueError):
             e.w_type = space.w_OverflowError
-        if (e.match(space, space.w_OverflowError) and 
-                space.isinstance_w(w_long, space.w_int)):
-            raise oefmt(space.w_OverflowError,
-                "Python int too large to convert to C unsigned long")
-        raise e
-    if need_to_check and val > ULONG_MAX:
-        # On win64 space.uint_w will succeed for 8-byte ints
-        # but long is 4 bytes. So we must check manually
-        raise oefmt(space.w_OverflowError,
-                    "Python int too large to convert to C unsigned long")
-    return rffi.cast(rffi.ULONG, val)
+        raise
 
 @cpython_api([PyObject], rffi.ULONG, error=-1)
 def PyLong_AsUnsignedLongMask(space, w_long):
     """Return a C unsigned long from a Python long integer, without checking
     for overflow.
     """
-    num = space.bigint_w(w_long)
-    val = num.uintmask()
-    if need_to_check and not we_are_translated():
-        # On win64 num.uintmask will succeed for 8-byte ints
-        # but unsigned long is 4 bytes.
-        # The cast below is sufficient when translated, but
-        # we need an extra check when running on CPython.
-        val &= ULONG_MASK
-    return rffi.cast(rffi.ULONG, val)
+    return PyInt_AsUnsignedLongMask(space, w_long)
 
-@cpython_api([PyObject], rffi.LONG, error=-1)
+@cpython_api([PyObject], lltype.Signed, error=-1)
 def PyLong_AsLong(space, w_long):
     """
-    Get a C long int from an int object or any object that has an __int__
-    method.  Return -1 and set an error if overflow occurs.
-    """
-    try:
-        if space.lookup(w_long, '__index__'):
-            val = space.int_w(space.index(w_long))
-        else:
-            val = space.int_w(space.int(w_long))
-    except OperationError as e:
-        if e.match(space, space.w_ValueError):
-            e.w_type = space.w_OverflowError
-        if (e.match(space, space.w_OverflowError) and 
-                space.isinstance_w(w_long, space.w_int)):
-            raise oefmt(space.w_OverflowError,
-                "Python int too large to convert to C long")
-        raise e
-    if need_to_check and (val > LONG_MAX or val < LONG_MIN):
-        # On win64 space.int_w will succeed for 8-byte ints
-        # but long is 4 bytes. So we must check manually
-        raise oefmt(space.w_OverflowError,
-                    "Python int too large to convert to C long")
-    return rffi.cast(rffi.LONG, val)
+    Return a C long representation of the contents of pylong.  If
+    pylong is greater than LONG_MAX, an OverflowError is raised
+    and -1 will be returned."""
+    return space.int_w(w_long)
 
 @cpython_api([PyObject], Py_ssize_t, error=-1)
 def PyLong_AsSsize_t(space, w_long):
@@ -126,15 +83,6 @@ def PyLong_AsSsize_t(space, w_long):
     """
     return space.int_w(w_long)
 
-@cpython_api([PyObject], rffi.SIZE_T, error=-1)
-def PyLong_AsSize_t(space, w_long):
-    """Return a C size_t representation of of pylong.  pylong must be
-    an instance of PyLongObject.
-
-    Raise OverflowError if the value of pylong is out of range for a
-    size_t."""
-    return space.uint_w(w_long)
-
 @cpython_api([PyObject], rffi.LONGLONG, error=-1)
 def PyLong_AsLongLong(space, w_long):
     """
@@ -149,14 +97,10 @@ def PyLong_AsUnsignedLongLong(space, w_long):
     Return a C unsigned long representation of the contents of pylong.
     If pylong is greater than ULONG_MAX, an OverflowError is
     raised."""
-    if not w_long:
-        return PyErr_BadInternalCall(space)
     try:
         return rffi.cast(rffi.ULONGLONG, space.r_ulonglong_w(w_long))
     except OperationError as e:
         if e.match(space, space.w_ValueError):
-            if not w_long:
-                raise
             e.w_type = space.w_OverflowError
         raise
 
@@ -166,12 +110,10 @@ def PyLong_AsUnsignedLongLongMask(space, w_long):
     PyLongObject, if it is not already one, and then return its value as
     unsigned long long, without checking for overflow.
     """
-    if not w_long:
-        return PyErr_BadInternalCall(space)
     num = space.bigint_w(w_long)
     return num.ulonglongmask()
 
-@cpython_api([PyObject, INTP_real], rffi.LONG,
+@cpython_api([PyObject, rffi.CArrayPtr(rffi.INT_real)], lltype.Signed,
              error=-1)
 def PyLong_AsLongAndOverflow(space, w_long, overflow_ptr):
     """
@@ -182,21 +124,17 @@ def PyLong_AsLongAndOverflow(space, w_long, overflow_ptr):
     returned and *overflow will be 0."""
     overflow_ptr[0] = rffi.cast(rffi.INT_real, 0)
     try:
-        val = space.int_w(w_long)
-        if not need_to_check or (val >= LONG_MIN and val <= LONG_MAX):
-            # On win64 space.int_w will succeed for 8-byte ints
-            # but long is 4 bytes. So we must check manually
-            return rffi.cast(rffi.LONG, val)
+        return space.int_w(w_long)
     except OperationError as e:
         if not e.match(space, space.w_OverflowError):
             raise
-    if space.is_true(space.gt(w_long, space.newint(0))):
+    if space.is_true(space.gt(w_long, space.wrap(0))):
         overflow_ptr[0] = rffi.cast(rffi.INT_real, 1)
     else:
         overflow_ptr[0] = rffi.cast(rffi.INT_real, -1)
     return -1
 
-@cpython_api([PyObject, INTP_real], rffi.LONGLONG,
+@cpython_api([PyObject, rffi.CArrayPtr(rffi.INT_real)], rffi.LONGLONG,
              error=-1)
 def PyLong_AsLongLongAndOverflow(space, w_long, overflow_ptr):
     """
@@ -211,7 +149,7 @@ def PyLong_AsLongLongAndOverflow(space, w_long, overflow_ptr):
     except OperationError as e:
         if not e.match(space, space.w_OverflowError):
             raise
-    if space.is_true(space.gt(w_long, space.newint(0))):
+    if space.is_true(space.gt(w_long, space.wrap(0))):
         overflow_ptr[0] = rffi.cast(rffi.INT_real, 1)
     else:
         overflow_ptr[0] = rffi.cast(rffi.INT_real, -1)
@@ -220,7 +158,7 @@ def PyLong_AsLongLongAndOverflow(space, w_long, overflow_ptr):
 @cpython_api([lltype.Float], PyObject)
 def PyLong_FromDouble(space, val):
     """Return a new PyLongObject object from v, or NULL on failure."""
-    return space.long(space.newfloat(val))
+    return space.long(space.wrap(val))
 
 @cpython_api([PyObject], lltype.Float, error=-1.0)
 def PyLong_AsDouble(space, w_long):
@@ -241,8 +179,8 @@ def PyLong_FromString(space, str, pend, base):
     between 2 and 36, inclusive.  Leading spaces are ignored.  If there are
     no digits, ValueError will be raised."""
     s = rffi.charp2str(str)
-    w_str = space.newtext(s)
-    w_base = space.newint(rffi.cast(lltype.Signed, base))
+    w_str = space.wrap(s)
+    w_base = space.wrap(rffi.cast(lltype.Signed, base))
     if pend:
         pend[0] = rffi.ptradd(str, len(s))
     return space.call_function(space.w_long, w_str, w_base)
@@ -254,14 +192,8 @@ def PyLong_FromUnicode(space, u, length, base):
     string, length gives the number of characters, and base is the radix
     for the conversion.  The radix must be in the range [2, 36]; if it is
     out of range, ValueError will be raised."""
-    if length < 0:
-        length = 0
-    w_value = space.newutf8(wcharpsize2utf8(space, u, length), length)
-    return PyLong_FromUnicodeObject(space, w_value, base)
-
-@cpython_api([PyObject, rffi.INT_real], PyObject)
-def PyLong_FromUnicodeObject(space, w_value, base):
-    w_base = space.newint(rffi.cast(lltype.Signed, base))
+    w_value = space.wrap(rffi.wcharpsize2unicode(u, length))
+    w_base = space.wrap(rffi.cast(lltype.Signed, base))
     return space.call_function(space.w_long, w_value, w_base)
 
 @cpython_api([rffi.VOIDP], PyObject)
@@ -273,7 +205,7 @@ def PyLong_FromVoidPtr(space, p):
     value = rffi.cast(ADDR, p)    # signed integer
     if value < 0:
         return space.newlong_from_rarith_int(rffi.cast(lltype.Unsigned, p))
-    return space.newint(value)
+    return space.wrap(value)
 
 @cpython_api([PyObject], rffi.VOIDP, error=lltype.nullptr(rffi.VOIDP.TO))
 def PyLong_AsVoidPtr(space, w_long):
@@ -290,12 +222,11 @@ def _PyLong_NumBits(space, w_long):
 
 @cpython_api([PyObject], rffi.INT_real, error=CANNOT_FAIL)
 def _PyLong_Sign(space, w_long):
-    bigint = space.bigint_w(w_long)
-    return bigint.sign
+    assert isinstance(w_long, W_LongObject)
+    return w_long.num.sign
 
-CONST_UCHARP = lltype.Ptr(lltype.Array(rffi.UCHAR, hints={'nolength': True,
-                                       'render_as_const': True}))
-@cpython_api([CONST_UCHARP, rffi.SIZE_T, rffi.INT_real, rffi.INT_real], PyObject)
+UCHARP = rffi.CArrayPtr(rffi.UCHAR)
+@cpython_api([UCHARP, rffi.SIZE_T, rffi.INT_real, rffi.INT_real], PyObject)
 def _PyLong_FromByteArray(space, bytes, n, little_endian, signed):
     little_endian = rffi.cast(lltype.Signed, little_endian)
     signed = rffi.cast(lltype.Signed, signed)
@@ -307,26 +238,3 @@ def _PyLong_FromByteArray(space, bytes, n, little_endian, signed):
         byteorder = 'big'
     result = rbigint.frombytes(s, byteorder, signed != 0)
     return space.newlong_from_rbigint(result)
-
-@cpython_api([PyObject, rffi.UCHARP, rffi.SIZE_T,
-              rffi.INT_real, rffi.INT_real], rffi.INT_real, error=-1)
-def _PyLong_AsByteArrayO(space, w_v, bytes, n, little_endian, is_signed):
-    n = rffi.cast(lltype.Signed, n)
-    little_endian = rffi.cast(lltype.Signed, little_endian)
-    signed = rffi.cast(lltype.Signed, is_signed) != 0
-    byteorder = 'little' if little_endian else 'big'
-    bigint = space.bigint_w(w_v)
-    try:
-        digits = bigint.tobytes(n, byteorder, signed)
-    except InvalidSignednessError:     # < 0 but not 'signed'
-        # in this case, CPython raises OverflowError even though the C
-        # comments say it should raise TypeError
-        raise oefmt(space.w_OverflowError,
-                    "can't convert negative long to unsigned")
-    except OverflowError:
-        raise oefmt(space.w_OverflowError,
-                    "long too big to convert")
-    assert len(digits) == n
-    for i in range(n):
-        bytes[i] = rffi.cast(rffi.UCHAR, digits[i])
-    return 0
