@@ -1,89 +1,49 @@
 import os
 
 from rpython.rtyper.lltypesystem import rffi, lltype
-from pypy.interpreter.error import OperationError, oefmt, strerror as _strerror
+from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter import pytraceback
 from pypy.module.cpyext.api import cpython_api, CANNOT_FAIL, CONST_STRING
-from pypy.module.cpyext.api import PyObjectFields, cpython_struct
-from pypy.module.cpyext.api import bootstrap_function, slot_function
-from pypy.module.cpyext.pyobject import make_typedescr
 from pypy.module.exceptions.interp_exceptions import W_RuntimeWarning
-from pypy.module.exceptions.interp_exceptions import W_StopIteration
 from pypy.module.cpyext.pyobject import (
-    PyObject, PyObjectP, make_ref, from_ref, decref, get_w_obj_and_decref)
+    PyObject, PyObjectP, make_ref, from_ref, Py_DecRef)
 from pypy.module.cpyext.state import State
 from pypy.module.cpyext.import_ import PyImport_Import
 from rpython.rlib import rposix, jit
 
-PyStopIterationObjectStruct = lltype.ForwardReference()
-PyStopIterationObject = lltype.Ptr(PyStopIterationObjectStruct)
-PyStopIterationObjectFields = PyObjectFields + \
-    (("value", PyObject), )
-cpython_struct("PyStopIterationObject", PyStopIterationObjectFields,
-               PyStopIterationObjectStruct)
-
-@bootstrap_function
-def init_stopiterationobject(space):
-    "Type description of PyStopIterationObject"
-    make_typedescr(W_StopIteration.typedef,
-                   basestruct=PyStopIterationObject.TO,
-                   attach=stopiteration_attach,
-                   dealloc=stopiteration_dealloc)
-
-def stopiteration_attach(space, py_obj, w_obj, w_userdata=None):
-    py_stopiteration = rffi.cast(PyStopIterationObject, py_obj)
-    assert isinstance(w_obj, W_StopIteration)
-    # note: assumes that w_value is read-only; changes on one side won't
-    # be reflected on the other side
-    py_stopiteration.c_value = make_ref(space, w_obj.w_value)
-
-@slot_function([PyObject], lltype.Void)
-def stopiteration_dealloc(space, py_obj):
-    py_stopiteration = rffi.cast(PyStopIterationObject, py_obj)
-    decref(space, py_stopiteration.c_value)
-    from pypy.module.cpyext.object import _dealloc
-    _dealloc(space, py_obj)
-
-
 @cpython_api([PyObject, PyObject], lltype.Void)
 def PyErr_SetObject(space, w_type, w_value):
     """This function is similar to PyErr_SetString() but lets you specify an
     arbitrary Python object for the "value" of the exception."""
     state = space.fromcache(State)
-    operr = OperationError(w_type, w_value)
-    operr.record_context(space, space.getexecutioncontext())
-    state.set_exception(operr)
+    state.set_exception(OperationError(w_type, w_value))
 
 @cpython_api([PyObject, CONST_STRING], lltype.Void)
 def PyErr_SetString(space, w_type, message_ptr):
     message = rffi.charp2str(message_ptr)
-    PyErr_SetObject(space, w_type, space.newtext(message))
+    PyErr_SetObject(space, w_type, space.wrap(message))
 
 @cpython_api([PyObject], lltype.Void, error=CANNOT_FAIL)
 def PyErr_SetNone(space, w_type):
     """This is a shorthand for PyErr_SetObject(type, Py_None)."""
     PyErr_SetObject(space, w_type, space.w_None)
 
-if os.name == 'nt':
-    # For some reason CPython returns a (PyObject*)NULL
-    # This confuses the annotator, so set result_is_ll
-    @cpython_api([rffi.INT_real], PyObject, error=CANNOT_FAIL, result_is_ll=True)
-    def PyErr_SetFromWindowsErr(space, err):
-        PyErr_SetObject(space, space.w_OSError, space.newint(err))
-        return rffi.cast(PyObject, 0)
-
 @cpython_api([], PyObject, result_borrowed=True)
 def PyErr_Occurred(space):
     state = space.fromcache(State)
-    operror = state.get_exception()
-    if operror is None:
+    if state.operror is None:
         return None
-    return operror.w_type     # borrowed ref
+    return state.operror.w_type     # borrowed ref
 
 @cpython_api([], lltype.Void)
 def PyErr_Clear(space):
     state = space.fromcache(State)
     state.clear_exception()
 
+@cpython_api([PyObject], PyObject)
+def PyExceptionInstance_Class(space, w_obj):
+    return space.type(w_obj)
+
 @cpython_api([PyObjectP, PyObjectP, PyObjectP], lltype.Void)
 def PyErr_Fetch(space, ptype, pvalue, ptraceback):
     """Retrieve the error indicator into three variables whose addresses are passed.
@@ -98,14 +58,14 @@ def PyErr_Fetch(space, ptype, pvalue, ptraceback):
     if operror:
         ptype[0] = make_ref(space, operror.w_type)
         pvalue[0] = make_ref(space, operror.get_w_value(space))
-        ptraceback[0] = make_ref(space, operror.get_w_traceback(space))
+        ptraceback[0] = make_ref(space, space.wrap(operror.get_traceback()))
     else:
         ptype[0] = lltype.nullptr(PyObject.TO)
         pvalue[0] = lltype.nullptr(PyObject.TO)
         ptraceback[0] = lltype.nullptr(PyObject.TO)
 
 @cpython_api([PyObject, PyObject, PyObject], lltype.Void)
-def PyErr_Restore(space, py_type, py_value, py_traceback):
+def PyErr_Restore(space, w_type, w_value, w_traceback):
     """Set  the error indicator from the three objects.  If the error indicator is
     already set, it is cleared first.  If the objects are NULL, the error
     indicator is cleared.  Do not pass a NULL type and non-NULL value or
@@ -120,14 +80,13 @@ def PyErr_Restore(space, py_type, py_value, py_traceback):
     error indicator temporarily; use PyErr_Fetch() to save the current
     exception state."""
     state = space.fromcache(State)
-    w_type = get_w_obj_and_decref(space, py_type)
-    w_value = get_w_obj_and_decref(space, py_value)
-    w_traceback = get_w_obj_and_decref(space, py_traceback)
-
     if w_type is None:
         state.clear_exception()
         return
-    state.set_exception(OperationError(w_type, w_value, w_traceback))
+    state.set_exception(OperationError(w_type, w_value))
+    Py_DecRef(space, w_type)
+    Py_DecRef(space, w_value)
+    Py_DecRef(space, w_traceback)
 
 @cpython_api([PyObjectP, PyObjectP, PyObjectP], lltype.Void)
 def PyErr_NormalizeException(space, exc_p, val_p, tb_p):
@@ -136,21 +95,11 @@ def PyErr_NormalizeException(space, exc_p, val_p, tb_p):
     not an instance of the  same class.  This function can be used to instantiate
     the class in that case.  If the values are already normalized, nothing happens.
     The delayed normalization is implemented to improve performance."""
-    if exc_p[0]:
-        w_etype = from_ref(space, exc_p[0])
-    else:
-        # There is no exception, so nothing to do
-        return
-    if val_p[0]:
-        w_evalue = from_ref(space, val_p[0])
-    else:
-        # On CPython, PyErr_SetNone actually sets val to NULL.
-        # Sensible code should probably never trigger this path on PyPy, but...
-        w_evalue = space.w_None
-    operr = OperationError(w_etype, w_evalue)
+    operr = OperationError(from_ref(space, exc_p[0]),
+                           from_ref(space, val_p[0]))
     operr.normalize_exception(space)
-    decref(space, exc_p[0])
-    decref(space, val_p[0])
+    Py_DecRef(space, exc_p[0])
+    Py_DecRef(space, val_p[0])
     exc_p[0] = make_ref(space, operr.w_type)
     val_p[0] = make_ref(space, operr.get_w_value(space))
 
@@ -163,7 +112,7 @@ def PyErr_BadArgument(space):
     error indicator."""
     raise oefmt(space.w_TypeError, "bad argument type for built-in operation")
 
-@cpython_api([], lltype.Void, error=None)
+@cpython_api([], lltype.Void)
 def PyErr_BadInternalCall(space):
     raise oefmt(space.w_SystemError, "Bad internal call!")
 
@@ -192,7 +141,7 @@ def PyErr_SetFromErrno(space, w_type):
     PyErr_SetFromErrnoWithFilename(space, w_type,
                                    lltype.nullptr(rffi.CCHARP.TO))
 
-@cpython_api([PyObject, CONST_STRING], PyObject)
+@cpython_api([PyObject, rffi.CCHARP], PyObject)
 def PyErr_SetFromErrnoWithFilename(space, w_type, llfilename):
     """Similar to PyErr_SetFromErrno(), with the additional behavior that if
     filename is not NULL, it is passed to the constructor of type as a third
@@ -202,7 +151,7 @@ def PyErr_SetFromErrnoWithFilename(space, w_type, llfilename):
     # XXX Doesn't actually do anything with PyErr_CheckSignals.
     if llfilename:
         filename = rffi.charp2str(llfilename)
-        w_filename = space.newfilename(filename)
+        w_filename = space.wrap(filename)
     else:
         w_filename = space.w_None
 
@@ -218,43 +167,16 @@ def PyErr_SetFromErrnoWithFilenameObject(space, w_type, w_value):
     Return value: always NULL."""
     # XXX Doesn't actually do anything with PyErr_CheckSignals.
     errno = rffi.cast(lltype.Signed, rposix._get_errno())
-    msg, lgt = _strerror(errno)
+    msg = os.strerror(errno)
     if w_value:
         w_error = space.call_function(w_type,
-                                      space.newint(errno),
-                                      space.newtext(msg, lgt),
+                                      space.wrap(errno),
+                                      space.wrap(msg),
                                       w_value)
     else:
         w_error = space.call_function(w_type,
-                                      space.newint(errno),
-                                      space.newtext(msg, lgt))
-    raise OperationError(w_type, w_error)
-
-@cpython_api([PyObject, PyObject, PyObject], PyObject)
-@jit.dont_look_inside       # direct use of _get_errno()
-def PyErr_SetFromErrnoWithFilenameObjects(space, w_type, w_value, w_value2):
-    """Similar to PyErr_SetFromErrnoWithFilenameObject(), with the additional
-    behavior that it can take 2 filenames, for failures in rename(), symlink()
-    or copy().
-    Return value: always NULL."""
-    # XXX Doesn't actually do anything with PyErr_CheckSignals.
-    errno = rffi.cast(lltype.Signed, rposix._get_errno())
-    msg, lgt = _strerror(errno)
-    if w_value:
-        if w_value2:
-            w_error = space.call_function(w_type,
-                                          space.newint(errno),
-                                          space.newtext(msg, lgt),
-                                          w_value, None, w_value2)
-        else:
-            w_error = space.call_function(w_type,
-                                          space.newint(errno),
-                                          space.newtext(msg, lgt),
-                                          w_value)
-    else:
-        w_error = space.call_function(w_type,
-                                      space.newint(errno),
-                                      space.newtext(msg, lgt))
+                                      space.wrap(errno),
+                                      space.wrap(msg))
     raise OperationError(w_type, w_error)
 
 @cpython_api([], rffi.INT_real, error=-1)
@@ -277,14 +199,12 @@ def PyErr_GivenExceptionMatches(space, w_given, w_exc):
     exc is a class object, this also returns true when given is an instance
     of a subclass.  If exc is a tuple, all exceptions in the tuple (and
     recursively in subtuples) are searched for a match."""
-    if space.isinstance_w(w_given, space.w_BaseException):
-        w_given_type = space.type(w_given)
+    if (space.isinstance_w(w_given, space.w_BaseException) or
+        space.is_oldstyle_instance(w_given)):
+        w_given_type = space.exception_getclass(w_given)
     else:
         w_given_type = w_given
-    try:
-        return space.exception_match(w_given_type, w_exc)
-    except:
-        return 0
+    return space.exception_match(w_given_type, w_exc)
 
 @cpython_api([PyObject], rffi.INT_real, error=CANNOT_FAIL)
 def PyErr_ExceptionMatches(space, w_exc):
@@ -332,11 +252,11 @@ def PyErr_WarnEx(space, w_category, message_ptr, stacklevel):
     documentation.  There is no C API for warning control."""
     if w_category is None:
         w_category = space.w_None
-    w_message = space.newtext(rffi.charp2str(message_ptr))
-    w_stacklevel = space.newint(rffi.cast(lltype.Signed, stacklevel))
+    w_message = space.wrap(rffi.charp2str(message_ptr))
+    w_stacklevel = space.wrap(rffi.cast(lltype.Signed, stacklevel))
 
-    w_module = PyImport_Import(space, space.newtext("warnings"))
-    w_warn = space.getattr(w_module, space.newtext("warn"))
+    w_module = PyImport_Import(space, space.wrap("warnings"))
+    w_warn = space.getattr(w_module, space.wrap("warn"))
     space.call_function(w_warn, w_message, w_category, w_stacklevel)
     return 0
 
@@ -350,36 +270,6 @@ def PyErr_Warn(space, w_category, message):
     Deprecated; use PyErr_WarnEx() instead."""
     return PyErr_WarnEx(space, w_category, message, 1)
 
-@cpython_api(
-    [PyObject, CONST_STRING, CONST_STRING, rffi.INT_real, CONST_STRING, PyObject],
-    rffi.INT_real, error=-1)
-def PyErr_WarnExplicit(space, w_category, message, filename, lineno, module, w_registry):
-    """Issue a warning message with explicit control over all warning attributes.  This
-    is a straightforward wrapper around the Python function
-    warnings.warn_explicit(), see there for more information.  The module
-    and registry arguments may be set to NULL to get the default effect
-    described there. message and module are UTF-8 encoded strings,
-    filename is decoded from the filesystem encoding
-    (sys.getfilesystemencoding())."""
-    if w_category is None:
-        w_category = space.w_UserWarning
-    w_message = space.newtext(rffi.charp2str(message))
-    # XXX use fsencode
-    w_filename = space.newtext(rffi.charp2str(filename))
-    w_lineno = space.newint(rffi.cast(lltype.Signed, lineno))
-    if module:
-        w_module = space.newtext(rffi.charp2str(module))
-    else:
-        w_module = space.w_None 
-    if w_registry is None:
-        w_registry = space.w_None
-    w_warnings = PyImport_Import(space, space.newtext("warnings"))
-    w_warn = space.getattr(w_warnings, space.newtext("warn_explicit"))
-    space.call_function(w_warn, w_message, w_category, w_filename, w_lineno,
-                        w_module, w_registry)
-    return 0
-
-
 @cpython_api([rffi.INT_real], lltype.Void)
 def PyErr_PrintEx(space, set_sys_last_vars):
     """Print a standard traceback to sys.stderr and clear the error indicator.
@@ -391,12 +281,12 @@ def PyErr_PrintEx(space, set_sys_last_vars):
     type, value and traceback of the printed exception, respectively."""
     if not PyErr_Occurred(space):
         PyErr_BadInternalCall(space)
+    state = space.fromcache(State)
+    operror = state.clear_exception()
 
-    operror = space.fromcache(State).clear_exception()
-    operror.normalize_exception(space)
     w_type = operror.w_type
     w_value = operror.get_w_value(space)
-    w_tb = operror.get_w_traceback(space)
+    w_tb = space.wrap(operror.get_traceback())
 
     if rffi.cast(lltype.Signed, set_sys_last_vars):
         space.sys.setdictvalue(space, "last_type", w_type)
@@ -427,15 +317,15 @@ def PyErr_Display(space, w_type, w_value, tb):
 
 @cpython_api([PyObject, PyObject], rffi.INT_real, error=-1)
 def PyTraceBack_Print(space, w_tb, w_file):
-    space.call_method(w_file, "write", space.newtext(
+    space.call_method(w_file, "write", space.wrap(
         'Traceback (most recent call last):\n'))
     w_traceback = space.call_method(space.builtin, '__import__',
-                                    space.newtext("traceback"))
+                                    space.wrap("traceback"))
     space.call_method(w_traceback, "print_tb", w_tb, space.w_None, w_file)
     return 0
 
 @cpython_api([PyObject], lltype.Void)
-def PyErr_WriteUnraisable(space, where):
+def PyErr_WriteUnraisable(space, w_where):
     """This utility function prints a warning message to sys.stderr when an
     exception has been set but it is impossible for the interpreter to actually
     raise the exception.  It is used, for example, when an exception occurs in
@@ -445,34 +335,10 @@ def PyErr_WriteUnraisable(space, where):
     context in which the unraisable exception occurred. The repr of obj will be
     printed in the warning message."""
 
-    if not where:
-        where = ''
-    else:
-        where = space.text_w(space.repr(from_ref(space, where)))
     state = space.fromcache(State)
     operror = state.clear_exception()
     if operror:
-        operror.write_unraisable(space, where)
-
-@cpython_api([CONST_STRING, PyObject], lltype.Void)
-def _PyErr_WriteUnraisableMsg(space, where, w_obj):
-    """This utility function prints a warning message to sys.stderr when an
-    exception has been set but it is impossible for the interpreter to actually
-    raise the exception.  It is used, for example, when an exception occurs in
-    an __del__() method.
-
-    The function is called with a single argument obj that identifies the
-    context in which the unraisable exception occurred. The repr of obj will be
-    printed in the warning message."""
-
-    if not where:
-        where = ''
-    else:
-        where = rffi.charp2str(where)
-    state = space.fromcache(State)
-    operror = state.clear_exception()
-    if operror:
-        operror.write_unraisable(space, where, w_object=w_obj, with_traceback=True)
+        operror.write_unraisable(space, space.str_w(space.repr(w_where)))
 
 @cpython_api([], lltype.Void)
 def PyErr_SetInterrupt(space):
@@ -486,7 +352,8 @@ def PyErr_SetInterrupt(space):
 
 @cpython_api([PyObjectP, PyObjectP, PyObjectP], lltype.Void)
 def PyErr_GetExcInfo(space, ptype, pvalue, ptraceback):
-    """
+    """---Cython extension---
+
     Retrieve the exception info, as known from ``sys.exc_info()``.  This
     refers to an exception that was already caught, not to an exception
     that was freshly raised.  Returns new references for the three
@@ -506,34 +373,45 @@ def PyErr_GetExcInfo(space, ptype, pvalue, ptraceback):
     if operror:
         ptype[0] = make_ref(space, operror.w_type)
         pvalue[0] = make_ref(space, operror.get_w_value(space))
-        ptraceback[0] = make_ref(space, operror.get_w_traceback(space))
+        ptraceback[0] = make_ref(space, space.wrap(operror.get_traceback()))
     else:
         ptype[0] = lltype.nullptr(PyObject.TO)
         pvalue[0] = lltype.nullptr(PyObject.TO)
         ptraceback[0] = lltype.nullptr(PyObject.TO)
 
 @cpython_api([PyObject, PyObject, PyObject], lltype.Void)
-def PyErr_SetExcInfo(space, py_type, py_value, py_traceback):
-    """
+def PyErr_SetExcInfo(space, w_type, w_value, w_traceback):
+    """---Cython extension---
+
     Set the exception info, as known from ``sys.exc_info()``.  This refers
     to an exception that was already caught, not to an exception that was
     freshly raised.  This function steals the references of the arguments.
     To clear the exception state, pass *NULL* for all three arguments.
     For general rules about the three arguments, see :c:func:`PyErr_Restore`.
-
+ 
     .. note::
-
+ 
        This function is not normally used by code that wants to handle
        exceptions.  Rather, it can be used when code needs to save and
        restore the exception state temporarily.  Use
        :c:func:`PyErr_GetExcInfo` to read the exception state.
     """
-    w_type = get_w_obj_and_decref(space, py_type)
-    w_value = get_w_obj_and_decref(space, py_value)
-    w_traceback = get_w_obj_and_decref(space, py_traceback)
+    if w_value is None or space.is_w(w_value, space.w_None):
+        operror = None
+    else:
+        tb = None
+        if w_traceback is not None:
+            try:
+                tb = pytraceback.check_traceback(space, w_traceback, '?')
+            except OperationError:    # catch and ignore bogus objects
+                pass
+        operror = OperationError(w_type, w_value, tb)
     #
     ec = space.getexecutioncontext()
-    ec.set_sys_exc_info3(w_type, w_value, w_traceback)
+    ec.set_sys_exc_info(operror)
+    Py_DecRef(space, w_type)
+    Py_DecRef(space, w_value)
+    Py_DecRef(space, w_traceback)
 
 @cpython_api([], rffi.INT_real, error=CANNOT_FAIL)
 def PyOS_InterruptOccurred(space):
