@@ -1,6 +1,20 @@
 #include "Python.h"
 #include "structmember.h"
 
+#if PY_MAJOR_VERSION >= 3
+    #define PyInt_FromLong PyLong_FromLong
+    #define PyInt_AsLong PyLong_AsLong
+    #define PyThing_FromStringAndSize PyUnicode_FromStringAndSize
+    #define PyThing_FromString PyUnicode_FromString
+    #define PyThing_Check PyUnicode_Check
+    #define _PyThing_AsString _PyUnicode_AsString
+#else
+    #define PyThing_FromStringAndSize PyString_FromStringAndSize
+    #define PyThing_FromString PyString_FromString
+    #define PyThing_Check PyString_Check
+    #define _PyThing_AsString PyString_AsString
+#endif
+
 typedef struct {
     PyObject_HEAD
     int    foo;        /* the context holder */
@@ -43,15 +57,6 @@ newfooobject(void)
 
 /* foo methods */
 
-static void
-foo_dealloc(fooobject *foop)
-{
-    PyObject_Del(foop);
-}
-
-
-/* foo methods-as-attributes */
-
 static PyObject *
 foo_copy(fooobject *self)
 {
@@ -97,7 +102,7 @@ static PyMethodDef foo_methods[] = {
 static PyObject *
 foo_get_name(PyObject *self, void *closure)
 {
-    return PyString_FromStringAndSize("Foo Example", 11);
+    return PyThing_FromStringAndSize("Foo Example", 11);
 }
 
 static PyObject *
@@ -123,7 +128,7 @@ foo_repr(PyObject *self)
 {
     PyObject *format;
 
-    format = PyString_FromString("<Foo>");
+    format = PyThing_FromString("<Foo>");
     if (format == NULL) return NULL;
     return format;
 }
@@ -139,17 +144,18 @@ static int
 foo_setattro(fooobject *self, PyObject *name, PyObject *value)
 {
     char *name_str;
-    if (!PyString_Check(name)) {
+    if (!PyThing_Check(name)) {
         PyErr_SetObject(PyExc_AttributeError, name);
         return -1;
     }
-    name_str = PyString_AsString(name);
+    name_str = _PyThing_AsString(name);
     if (strcmp(name_str, "set_foo") == 0)
     {
         long v = PyInt_AsLong(value);
         if (v == -1 && PyErr_Occurred())
             return -1;
         self->foo = v;
+        return 0;
     }
     return PyObject_GenericSetAttr((PyObject *)self, name, value);
 }
@@ -181,7 +187,7 @@ static PyMemberDef foo_members[] = {
     {"float_member", T_FLOAT, offsetof(fooobject, foo_float), 0, NULL},
     {"double_member", T_DOUBLE, offsetof(fooobject, foo_double), 0, NULL},
     {"longlong_member", T_LONGLONG, offsetof(fooobject, foo_longlong), 0, NULL},
-    {"ulonglong_member", T_ULONGLONG, offsetof(fooobject, foo_ulonglong), 0, NULL},  
+    {"ulonglong_member", T_ULONGLONG, offsetof(fooobject, foo_ulonglong), 0, NULL},
     {"ssizet_member", T_PYSSIZET, offsetof(fooobject, foo_ssizet), 0, NULL},
     {NULL}  /* Sentinel */
 };
@@ -194,7 +200,7 @@ static PyTypeObject footype = {
     sizeof(fooobject),       /*tp_size*/
     0,                       /*tp_itemsize*/
     /* methods */
-    (destructor)foo_dealloc, /*tp_dealloc*/
+    0,                       /*tp_dealloc*/
     0,                       /*tp_print*/
     0,                       /*tp_getattr*/
     0,                       /*tp_setattr*/
@@ -209,7 +215,7 @@ static PyTypeObject footype = {
     0,                       /*tp_getattro*/
     (setattrofunc)foo_setattro, /*tp_setattro*/
     0,                       /*tp_as_buffer*/
-    Py_TPFLAGS_DEFAULT,      /*tp_flags*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
     foo_doc,                 /*tp_doc*/
     0,                       /*tp_traverse*/
     0,                       /*tp_clear*/
@@ -253,8 +259,7 @@ static PyMethodDef UnicodeSubclass_methods[] = {
 };
 
 PyTypeObject UnicodeSubtype = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.fuu",
     sizeof(UnicodeSubclassObject),
     0,
@@ -312,8 +317,7 @@ PyTypeObject UnicodeSubtype = {
 };
 
 PyTypeObject UnicodeSubtype2 = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.fuu2",
     sizeof(UnicodeSubclassObject),
     0,
@@ -370,14 +374,18 @@ PyTypeObject UnicodeSubtype2 = {
     0           /*tp_weaklist*/
 };
 
+PyTypeObject UnicodeSubtype3 = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "foo.fuu3",
+    sizeof(UnicodeSubclassObject)
+};
 
 /* A Metatype */
 
 PyTypeObject MetaType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Meta",
-    sizeof(PyTypeObject),          /*tp_basicsize*/
+    sizeof(PyHeapTypeObject),/*tp_basicsize*/
     0,          /*tp_itemsize*/
     0,          /*tp_dealloc*/
     0,          /*tp_print*/
@@ -443,7 +451,7 @@ foo_new(PyObject *self, PyObject *args)
     if ((foop = newfooobject()) == NULL) {
         return NULL;
     }
-    
+
     return (PyObject *)foop;
 }
 
@@ -454,11 +462,10 @@ static int initerrtype_init(PyObject *self, PyObject *args, PyObject *kwargs) {
 
 
 PyTypeObject InitErrType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
-    "foo.InitErr",
-    sizeof(PyObject),
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "foo.InitErrType",
+    sizeof(PyObject),/*tp_basicsize*/
+    0,          /*tp_itemsize*/
     0,          /*tp_dealloc*/
     0,          /*tp_print*/
     0,          /*tp_getattr*/
@@ -501,12 +508,12 @@ PyTypeObject InitErrType = {
     0,          /*tp_dictoffset*/
 
     initerrtype_init,          /*tp_init*/
-    0,          /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
+    0,          /*tp_alloc*/
     0,          /*tp_new*/
-    0,          /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
+    0,          /*tp_free*/
+    0,          /*tp_is_gc*/
     0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
+    0,          /*tp_mro*/
     0,          /*tp_cache*/
     0,          /*tp_subclasses*/
     0           /*tp_weaklist*/
@@ -538,8 +545,7 @@ int prop_descr_set(PyObject *self, PyObject *obj, PyObject *value)
 
 
 PyTypeObject SimplePropertyType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Property",
     sizeof(PyObject),
     0,
@@ -607,14 +613,13 @@ static PyTypeObject CustomType;
 static PyObject *newCustom(PyObject *self, PyObject *args)
 {
     PyObject *obj = calloc(1, sizeof(PyObject));
-    obj->ob_type = &CustomType;
+    Py_TYPE(obj) = &CustomType;
     _Py_NewReference(obj);
     return obj;
 }
 
 static PyTypeObject CustomType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Custom",            /*tp_name*/
     sizeof(PyObject),        /*tp_size*/
     0,                       /*tp_itemsize*/
@@ -622,21 +627,84 @@ static PyTypeObject CustomType = {
     (destructor)custom_dealloc, /*tp_dealloc*/
 };
 
+static PyTypeObject TupleLike = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "foo.TupleLike",         /*tp_name*/
+    sizeof(PyObject),        /*tp_size*/
+};
+
+
+static PyObject *size_of_instances(PyObject *self, PyObject *t)
+{
+    return PyInt_FromLong(((PyTypeObject *)t)->tp_basicsize);
+}
+
+
+static PyObject * is_TupleLike(PyObject *self, PyObject * t)
+{
+    int tf = t->ob_type == &TupleLike;
+    if (t->ob_type->tp_itemsize == 0)
+        return PyInt_FromLong(-1);
+    return PyInt_FromLong(tf);
+}
 
 /* List of functions exported by this module */
 
 static PyMethodDef foo_functions[] = {
     {"new",        (PyCFunction)foo_new, METH_NOARGS, NULL},
     {"newCustom",  (PyCFunction)newCustom, METH_NOARGS, NULL},
+    {"size_of_instances", (PyCFunction)size_of_instances, METH_O, NULL},
+    {"is_TupleLike", (PyCFunction)is_TupleLike, METH_O, NULL},
     {NULL,        NULL}    /* Sentinel */
 };
 
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "foo",
+    "Module Doc",
+    -1,
+    foo_functions, 
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+};
+#define INITERROR return NULL
 
 /* Initialize this module. */
+#ifdef __GNUC__
+extern __attribute__((visibility("default")))
+#else
+extern __declspec(dllexport)
+#endif
 
-void initfoo(void)
+PyMODINIT_FUNC
+PyInit_foo(void)
+
+#else
+
+#define INITERROR return
+
+/* Initialize this module. */
+#ifdef __GNUC__
+extern __attribute__((visibility("default")))
+#else
+extern __declspec(dllexport)
+#endif
+
+PyMODINIT_FUNC
+initfoo(void)
+#endif
 {
-    PyObject *m, *d;
+    PyObject *d;
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+#else
+    PyObject *module = Py_InitModule("foo", foo_functions);
+#endif
+    if (module == NULL)
+        INITERROR;
 
     footype.tp_new = PyType_GenericNew;
 
@@ -645,38 +713,60 @@ void initfoo(void)
     MetaType.tp_base = &PyType_Type;
 
     if (PyType_Ready(&footype) < 0)
-        return;
+        INITERROR;
     if (PyType_Ready(&UnicodeSubtype) < 0)
-        return;
+        INITERROR;
     if (PyType_Ready(&UnicodeSubtype2) < 0)
-        return;
+        INITERROR;
     if (PyType_Ready(&MetaType) < 0)
-        return;
+        INITERROR;
     if (PyType_Ready(&InitErrType) < 0)
-        return;
+        INITERROR;
     if (PyType_Ready(&SimplePropertyType) < 0)
-        return;
-    CustomType.ob_type = &MetaType;
+        INITERROR;
+
+    SimplePropertyType.tp_new = PyType_GenericNew;
+    InitErrType.tp_new = PyType_GenericNew;
+
+    Py_TYPE(&CustomType) = &MetaType;
     if (PyType_Ready(&CustomType) < 0)
-        return;
-    m = Py_InitModule("foo", foo_functions);
-    if (m == NULL)
-        return;
-    d = PyModule_GetDict(m);
+        INITERROR;
+
+    UnicodeSubtype3.tp_flags = Py_TPFLAGS_DEFAULT;
+    UnicodeSubtype3.tp_base = &UnicodeSubtype;
+    UnicodeSubtype3.tp_bases = Py_BuildValue("(OO)", &UnicodeSubtype,
+                                                    &CustomType);
+    if (PyType_Ready(&UnicodeSubtype3) < 0)
+        INITERROR;
+
+    TupleLike.tp_flags = Py_TPFLAGS_DEFAULT;
+    TupleLike.tp_base = &PyTuple_Type;
+    if (PyType_Ready(&TupleLike) < 0)
+        INITERROR;
+
+
+    d = PyModule_GetDict(module);
     if (d == NULL)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "fooType", (PyObject *)&footype) < 0)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "UnicodeSubtype", (PyObject *) &UnicodeSubtype) < 0)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "UnicodeSubtype2", (PyObject *) &UnicodeSubtype2) < 0)
-        return;
+        INITERROR;
+    if (PyDict_SetItemString(d, "UnicodeSubtype3", (PyObject *) &UnicodeSubtype3) < 0)
+        INITERROR;
     if (PyDict_SetItemString(d, "MetaType", (PyObject *) &MetaType) < 0)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "InitErrType", (PyObject *) &InitErrType) < 0)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "Property", (PyObject *) &SimplePropertyType) < 0)
-        return;
+        INITERROR;
     if (PyDict_SetItemString(d, "Custom", (PyObject *) &CustomType) < 0)
-        return;
+        INITERROR;
+    if (PyDict_SetItemString(d, "TupleLike", (PyObject *) &TupleLike) < 0)
+        INITERROR;
+#if PY_MAJOR_VERSION >=3
+    return module;
+#endif
 }
