@@ -1,21 +1,20 @@
 import sys
+import weakref
 
 import pytest
 
-from pypy.tool.cpyext.extbuild import SystemCompilationInfo, HERE
+from pypy.tool.cpyext.extbuild import (
+    SystemCompilationInfo, HERE, get_sys_info_app)
 from pypy.interpreter.gateway import unwrap_spec, interp2app
-from pypy.interpreter.error import OperationError
-from rpython.rtyper.lltypesystem import lltype
+from rpython.rtyper.lltypesystem import lltype, ll2ctypes
 from pypy.module.cpyext import api
-from pypy.module.cpyext.api import cts, create_extension_module
-from pypy.module.cpyext.pyobject import from_ref
 from pypy.module.cpyext.state import State
+from pypy.module.cpyext.pyobject import Py_DecRef
+from rpython.tool.identity_dict import identity_dict
 from rpython.tool import leakfinder
 from rpython.rlib import rawrefcount
 from rpython.tool.udir import udir
 
-import pypy.module.cpyext.moduledef  # Make sure all the functions are registered
-
 only_pypy ="config.option.runappdirect and '__pypy__' not in sys.builtin_module_names"
 
 @api.cpython_api([], api.PyObject)
@@ -26,17 +25,10 @@ def PyPy_Crash1(space):
 def PyPy_Crash2(space):
     1/0
 
-@api.cpython_api([api.PyObject], api.PyObject, result_is_ll=True)
-def PyPy_Noop(space, pyobj):
-    return pyobj
-
 class TestApi:
     def test_signature(self):
-        common_functions = api.FUNCTIONS_BY_HEADER[api.pypy_decl]
-        assert 'PyModule_Check' in common_functions
-        assert common_functions['PyModule_Check'].argtypes == [cts.gettype("void *")]
-        assert 'PyModule_GetDict' in common_functions
-        assert common_functions['PyModule_GetDict'].argtypes == [api.PyObject]
+        assert 'PyModule_Check' in api.FUNCTIONS
+        assert api.FUNCTIONS['PyModule_Check'].argtypes == [api.PyObject]
 
 
 class SpaceCompiler(SystemCompilationInfo):
@@ -45,25 +37,11 @@ class SpaceCompiler(SystemCompilationInfo):
         self.space = space
         SystemCompilationInfo.__init__(self, *args, **kwargs)
 
-    def load_module(self, mod, name, use_imp=False):
+    def load_module(self, mod, name):
         space = self.space
-        w_path = space.newtext(mod)
-        w_name = space.newtext(name)
-        if use_imp:
-            # this is VERY slow and should be used only by tests which
-            # actually needs it
-            return space.appexec([w_name, w_path], '''(name, path):
-                import imp
-                return imp.load_dynamic(name, path)''')
-        else:
-            w_spec = space.appexec([w_name, w_path], '''(modname, path):
-                class FakeSpec:
-                    name = modname
-                    origin = path
-                return FakeSpec
-            ''')
-            w_mod = create_extension_module(space, w_spec)
-            return w_mod
+        api.load_extension_module(space, mod, name)
+        return space.getitem(
+            space.sys.get('modules'), space.wrap(name))
 
 
 def get_cpyext_info(space):
@@ -97,108 +75,94 @@ def get_cpyext_info(space):
 
 def freeze_refcnts(self):
     rawrefcount._dont_free_any_more()
-
-def is_interned_string(space, w_obj):
-    try:
-        u = space.utf8_w(w_obj)
-    except OperationError:
-        return False
-    return space.interned_strings.get(u) is not None
-
-def is_allowed_to_leak(space, obj):
-    from pypy.module.cpyext.methodobject import W_PyCFunctionObject
-    try:
-        w_obj = from_ref(space, cts.cast('PyObject*', obj._as_ptr()))
-    except:
-        return False
-    if isinstance(w_obj, W_PyCFunctionObject):
-        return True
-    # It's OK to "leak" some interned strings: if the pyobj is created by
-    # the test, but the w_obj is referred to from elsewhere.
-    return is_interned_string(space, w_obj)
-
-def _get_w_obj(space, c_obj):
-    return from_ref(space, cts.cast('PyObject*', c_obj._as_ptr()))
-
-class CpyextLeak(leakfinder.MallocMismatch):
-    def __str__(self):
-        lines = [leakfinder.MallocMismatch.__str__(self), '']
-        lines.append(
-            "These objects are attached to the following W_Root objects:")
-        for c_obj in self.args[0]:
-            try:
-                lines.append("  %s" % (_get_w_obj(self.args[1], c_obj),))
-            except:
-                pass
-        return '\n'.join(lines)
-
+    return #ZZZ
+    state = self.space.fromcache(RefcountState)
+    self.frozen_refcounts = {}
+    for w_obj, obj in state.py_objects_w2r.iteritems():
+        self.frozen_refcounts[w_obj] = obj.c_ob_refcnt
+    #state.print_refcounts()
+    self.frozen_ll2callocations = set(ll2ctypes.ALLOCATED.values())
 
 class LeakCheckingTest(object):
     """Base class for all cpyext tests."""
-    spaceconfig = {"usemodules" : ['cpyext', 'thread', 'struct', 'array',
+    spaceconfig = dict(usemodules=['cpyext', 'thread', 'struct', 'array',
                                    'itertools', 'time', 'binascii',
-                                   'mmap', 'signal',
-                                   '_cffi_backend',
-                                   ],
-                   "objspace.disable_entrypoints_in_cffi": True}
-    spaceconfig["objspace.std.withspecialisedtuple"] = True
+                                   'micronumpy', 'mmap'
+                                   ])
 
-    @classmethod
-    def preload_builtins(cls, space):
-        """
-        Eagerly create pyobjs for various builtins so they don't look like
-        leaks.
-        """
-        from pypy.module.cpyext.pyobject import make_ref
-        w_to_preload = space.appexec([], """():
-            import sys
-            import mmap
-            #
-            # copied&pasted to avoid importing the whole types.py, which is
-            # expensive on py3k
-            # <types.py>
-            def _f(): pass
-            FunctionType = type(_f)
-            CodeType = type(_f.__code__)
-            try:
-                raise TypeError
-            except TypeError:
-                tb = sys.exc_info()[2]
-                TracebackType = type(tb)
-                FrameType = type(tb.tb_frame)
-                del tb
-            # </types.py>
-            return [
-                #buffer,   ## does not exist on py3k
-                mmap.mmap,
-                FunctionType,
-                CodeType,
-                TracebackType,
-                FrameType,
-                type(str.join),
-            ]
-        """)
-        for w_obj in space.unpackiterable(w_to_preload):
-            make_ref(space, w_obj)
+    enable_leak_checking = True
 
-    def cleanup(self):
-        self.space.getexecutioncontext().cleanup_cpyext_state()
-        for _ in range(5):
-            rawrefcount._collect()
-            self.space.user_del_action._run_finalizers()
-        try:
-            # set check=True to actually enable leakfinder
+    @staticmethod
+    def cleanup_references(space):
+        return #ZZZ
+        state = space.fromcache(RefcountState)
+
+        import gc; gc.collect()
+        # Clear all lifelines, objects won't resurrect
+        for w_obj, obj in state.lifeline_dict._dict.items():
+            if w_obj not in state.py_objects_w2r:
+                state.lifeline_dict.set(w_obj, None)
+            del obj
+        import gc; gc.collect()
+
+        space.getexecutioncontext().cleanup_cpyext_state()
+
+        for w_obj in state.non_heaptypes_w:
+            Py_DecRef(space, w_obj)
+        state.non_heaptypes_w[:] = []
+        state.reset_borrowed_references()
+
+    def check_and_print_leaks(self):
+        rawrefcount._collect()
+        # check for sane refcnts
+        import gc
+
+        if 1:  #ZZZ  not self.enable_leak_checking:
             leakfinder.stop_tracking_allocations(check=False)
-        except leakfinder.MallocMismatch as e:
-            result = e.args[0]
-            filtered_result = {}
-            for obj, value in result.iteritems():
-                if not is_allowed_to_leak(self.space, obj):
-                    filtered_result[obj] = value
-            if filtered_result:
-                raise CpyextLeak(filtered_result, self.space)
-        assert not self.space.finalizer_queue.next_dead()
+            return False
 
+        leaking = False
+        state = self.space.fromcache(RefcountState)
+        gc.collect()
+        lost_objects_w = identity_dict()
+        lost_objects_w.update((key, None) for key in self.frozen_refcounts.keys())
+
+        for w_obj, obj in state.py_objects_w2r.iteritems():
+            base_refcnt = self.frozen_refcounts.get(w_obj)
+            delta = obj.c_ob_refcnt
+            if base_refcnt is not None:
+                delta -= base_refcnt
+                lost_objects_w.pop(w_obj)
+            if delta != 0:
+                leaking = True
+                print >>sys.stderr, "Leaking %r: %i references" % (w_obj, delta)
+                try:
+                    weakref.ref(w_obj)
+                except TypeError:
+                    lifeline = None
+                else:
+                    lifeline = state.lifeline_dict.get(w_obj)
+                if lifeline is not None:
+                    refcnt = lifeline.pyo.c_ob_refcnt
+                    if refcnt > 0:
+                        print >>sys.stderr, "\tThe object also held by C code."
+                    else:
+                        referrers_repr = []
+                        for o in gc.get_referrers(w_obj):
+                            try:
+                                repr_str = repr(o)
+                            except TypeError as e:
+                                repr_str = "%s (type of o is %s)" % (str(e), type(o))
+                            referrers_repr.append(repr_str)
+                        referrers = ", ".join(referrers_repr)
+                        print >>sys.stderr, "\tThe object is referenced by these objects:", \
+                                referrers
+        for w_obj in lost_objects_w:
+            print >>sys.stderr, "Lost object %r" % (w_obj, )
+            leaking = True
+        # the actual low-level leak checking is done by pypy.tool.leakfinder,
+        # enabled automatically by pypy.conftest.
+        return leaking
 
 class AppTestApi(LeakCheckingTest):
     def setup_class(cls):
@@ -215,12 +179,25 @@ class AppTestApi(LeakCheckingTest):
     def teardown_method(self, meth):
         if self.runappdirect:
             return
-        self.cleanup()
+        self.cleanup_references(self.space)
+        # XXX: like AppTestCpythonExtensionBase.teardown_method:
+        # find out how to disable check_and_print_leaks() if the
+        # test failed
+        assert not self.check_and_print_leaks(), (
+            "Test leaks or loses object(s).  You should also check if "
+            "the test actually passed in the first place; if it failed "
+            "it is likely to reach this place.")
 
     @pytest.mark.skipif(only_pypy, reason='pypy only test')
     def test_only_import(self):
         import cpyext
 
+    @pytest.mark.skipif(only_pypy, reason='pypy only test')
+    def test_load_error(self):
+        import cpyext
+        raises(ImportError, cpyext.load_module, "missing.file", "foo")
+        raises(ImportError, cpyext.load_module, self.libc, "invalid.function")
+
     def test_dllhandle(self):
         import sys
         if sys.platform != "win32" or sys.version_info < (2, 6):
@@ -232,13 +209,13 @@ def _unwrap_include_dirs(space, w_include_dirs):
     if w_include_dirs is None:
         return None
     else:
-        return [space.text_w(s) for s in space.listview(w_include_dirs)]
+        return [space.str_w(s) for s in space.listview(w_include_dirs)]
 
 def debug_collect(space):
     rawrefcount._collect()
 
-
 class AppTestCpythonExtensionBase(LeakCheckingTest):
+
     def setup_class(cls):
         space = cls.space
         cls.w_here = space.wrap(str(HERE))
@@ -246,11 +223,16 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
         cls.w_runappdirect = space.wrap(cls.runappdirect)
         if not cls.runappdirect:
             cls.sys_info = get_cpyext_info(space)
+            space.getbuiltinmodule("cpyext")
+            # 'import os' to warm up reference counts
+            w_import = space.builtin.getdictvalue(space, '__import__')
+            space.call_function(w_import, space.wrap("os"))
+            #state = cls.space.fromcache(RefcountState) ZZZ
+            #state.non_heaptypes_w[:] = []
             cls.w_debug_collect = space.wrap(interp2app(debug_collect))
-            cls.preload_builtins(space)
         else:
             def w_import_module(self, name, init=None, body='', filename=None,
-                    include_dirs=None, PY_SSIZE_T_CLEAN=False, use_imp=False):
+                    include_dirs=None, PY_SSIZE_T_CLEAN=False):
                 from extbuild import get_sys_info_app
                 sys_info = get_sys_info_app(self.udir)
                 return sys_info.import_module(
@@ -304,7 +286,7 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
         if self.runappdirect:
             return
 
-        @unwrap_spec(name='text')
+        @unwrap_spec(name=str)
         def compile_module(space, name,
                            w_source_files=None,
                            w_source_strings=None):
@@ -312,7 +294,7 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
             Build an extension module linked against the cpyext api library.
             """
             if not space.is_none(w_source_files):
-                source_files = space.unwrap(w_source_files)
+                source_files = space.listview_bytes(w_source_files)
             else:
                 source_files = None
             if not space.is_none(w_source_strings):
@@ -330,26 +312,24 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
 
             return space.wrap(pydname)
 
-        @unwrap_spec(name='text', init='text_or_none', body='text',
-                     filename='fsencode_or_none', PY_SSIZE_T_CLEAN=bool,
-                     use_imp=bool)
+        @unwrap_spec(name=str, init='str_or_None', body=str,
+                     filename='str_or_None', PY_SSIZE_T_CLEAN=bool)
         def import_module(space, name, init=None, body='',
                           filename=None, w_include_dirs=None,
-                          PY_SSIZE_T_CLEAN=False, use_imp=False):
+                          PY_SSIZE_T_CLEAN=False):
             include_dirs = _unwrap_include_dirs(space, w_include_dirs)
             w_result = self.sys_info.import_module(
-                name, init, body, filename, include_dirs, PY_SSIZE_T_CLEAN,
-                use_imp)
+                name, init, body, filename, include_dirs, PY_SSIZE_T_CLEAN)
             self.record_imported_module(name)
             return w_result
 
 
-        @unwrap_spec(mod='text', name='text')
+        @unwrap_spec(mod=str, name=str)
         def load_module(space, mod, name):
             return self.sys_info.load_module(mod, name)
 
-        @unwrap_spec(modname='text', prologue='text',
-                             more_init='text', PY_SSIZE_T_CLEAN=bool)
+        @unwrap_spec(modname=str, prologue=str,
+                             more_init=str, PY_SSIZE_T_CLEAN=bool)
         def import_extension(space, modname, w_functions, prologue="",
                              w_include_dirs=None, more_init="", PY_SSIZE_T_CLEAN=False):
             functions = space.unwrap(w_functions)
@@ -374,6 +354,7 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
         self.space.call_method(self.space.sys.get("stdout"), "flush")
 
         freeze_refcnts(self)
+        #self.check_and_print_leaks()
 
     def unimport_module(self, name):
         """
@@ -385,29 +366,34 @@ class AppTestCpythonExtensionBase(LeakCheckingTest):
 
     def teardown_method(self, func):
         if self.runappdirect:
-            self.w_debug_collect()
             return
-        debug_collect(self.space)
         for name in self.imported_module_names:
             self.unimport_module(name)
-        self.cleanup()
-        state = self.space.fromcache(State)
-        assert 'operror' not in dir(state)
+        self.cleanup_references(self.space)
+        # XXX: find out how to disable check_and_print_leaks() if the
+        # test failed...
+        assert not self.check_and_print_leaks(), (
+            "Test leaks or loses object(s).  You should also check if "
+            "the test actually passed in the first place; if it failed "
+            "it is likely to reach this place.")
 
 
 class AppTestCpythonExtension(AppTestCpythonExtensionBase):
     def test_createmodule(self):
         import sys
-        self.import_module(name='foo')
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", NULL);
+        """
+        self.import_module(name='foo', init=init)
         assert 'foo' in sys.modules
 
     def test_export_function(self):
         import sys
-        if '__pypy__' in sys.modules:
-            from cpyext import is_cpyext_function
-        else:
-            import inspect
-            is_cpyext_function = inspect.isbuiltin
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         PyObject* foo_pi(PyObject* self, PyObject *args)
         {
@@ -417,58 +403,71 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "return_pi", foo_pi, METH_NOARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "foo",          /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         assert 'foo' in sys.modules
         assert 'return_pi' in dir(module)
         assert module.return_pi is not None
-        assert is_cpyext_function(module.return_pi)
         assert module.return_pi() == 3.14
         assert module.return_pi.__module__ == 'foo'
 
 
     def test_export_docstring(self):
+        import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         PyDoc_STRVAR(foo_pi_doc, "Return pi.");
         PyObject* foo_pi(PyObject* self, PyObject *args)
         {
             return PyFloat_FromDouble(3.14);
         }
-        static PyMethodDef methods[] = {
+        static PyMethodDef methods[] ={
             { "return_pi", foo_pi, METH_NOARGS, foo_pi_doc },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "foo",          /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         doc = module.return_pi.__doc__
         assert doc == "Return pi."
 
-    def test_load_dynamic(self):
-        import sys
+
+    def test_InitModule4(self):
+        init = """
+        PyObject *cookie = PyFloat_FromDouble(3.14);
+        Py_InitModule4("foo", methods, "docstring",
+                       cookie, PYTHON_API_VERSION);
+        Py_DECREF(cookie);
+        """
         body = """
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            NULL,           /* m_methods */
+        PyObject* return_cookie(PyObject* self, PyObject *args)
+        {
+            if (self)
+            {
+                Py_INCREF(self);
+                return self;
+            }
+            else
+                Py_RETURN_FALSE;
+        }
+        static PyMethodDef methods[] = {
+            { "return_cookie", return_cookie, METH_NOARGS },
+            { NULL }
         };
         """
-        foo = self.import_module(name='foo', body=body, use_imp=True)
+        module = self.import_module(name='foo', init=init, body=body)
+        assert module.__doc__ == "docstring"
+        assert module.return_cookie() == 3.14
+
+    def test_load_dynamic(self):
+        import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", NULL);
+        """
+        foo = self.import_module(name='foo', init=init)
         assert 'foo' in sys.modules
         del sys.modules['foo']
         import imp
@@ -533,6 +532,11 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
 
 
     def test_export_function2(self):
+        import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         static PyObject* my_objects[1];
         static PyObject* foo_cached_pi(PyObject* self, PyObject *args)
@@ -562,26 +566,28 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "return_invalid_pointer", foo_retinvalid, METH_NOARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         assert module.return_pi() == 3.14
+        print "A"
         module.drop_pi()
+        print "B"
         module.drop_pi()
+        print "C"
         assert module.return_pi() == 3.14
+        print "D"
         assert module.return_pi() == 3.14
+        print "E"
         module.drop_pi()
         skip("Hmm, how to check for the exception?")
         raises(api.InvalidPointerException, module.return_invalid_pointer)
 
     def test_argument(self):
         import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         PyObject* foo_test(PyObject* self, PyObject *args)
         {
@@ -593,19 +599,16 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "test", foo_test, METH_VARARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         assert module.test(True, True) == True
 
     def test_exception(self):
         import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         static PyObject* foo_pi(PyObject* self, PyObject *args)
         {
@@ -616,23 +619,20 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "raise_exception", foo_pi, METH_NOARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         exc = raises(Exception, module.raise_exception)
         if type(exc.value) is not Exception:
             raise exc.value
 
-        assert str(exc.value) == "moo!"
+        assert exc.value.message == "moo!"
 
     def test_refcount(self):
         import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         static PyObject* foo_pi(PyObject* self, PyObject *args)
         {
@@ -641,12 +641,11 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             Py_ssize_t refcnt_after;
             Py_INCREF(true_obj);
             Py_INCREF(true_obj);
-            if (!PyBool_Check(true_obj))
-                Py_RETURN_NONE;
+            PyBool_Check(true_obj);
             refcnt_after = true_obj->ob_refcnt;
             Py_DECREF(true_obj);
             Py_DECREF(true_obj);
-            fprintf(stderr, "REFCNT %ld %ld\\n", refcnt, refcnt_after);
+            fprintf(stderr, "REFCNT %zd %zd\\n", refcnt, refcnt_after);
             return PyBool_FromLong(refcnt_after == refcnt + 2);
         }
         static PyObject* foo_bar(PyObject* self, PyObject *args)
@@ -662,7 +661,7 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
                 return NULL;
             refcnt_after = true_obj->ob_refcnt;
             Py_DECREF(tup);
-            fprintf(stderr, "REFCNT2 %ld %ld %ld\\n", refcnt, refcnt_after,
+            fprintf(stderr, "REFCNT2 %zd %zd %zd\\n", refcnt, refcnt_after,
                     true_obj->ob_refcnt);
             return PyBool_FromLong(refcnt_after == refcnt + 1 &&
                                    refcnt == true_obj->ob_refcnt);
@@ -673,15 +672,8 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "test_refcount2", foo_bar, METH_NOARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
+        module = self.import_module(name='foo', init=init, body=body)
         assert module.test_refcount()
         assert module.test_refcount2()
 
@@ -690,67 +682,50 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
         import sys
         init = """
             PyErr_SetString(PyExc_Exception, "moo!");
-            return NULL;
         """
-        exc = raises(Exception, self.import_module, name='foo', init=init)
+        exc = raises(Exception, "self.import_module(name='foo', init=init)")
         if type(exc.value) is not Exception:
             raise exc.value
 
-        assert str(exc.value) == "moo!"
+        assert exc.value.message == "moo!"
 
 
     def test_internal_exceptions(self):
         if self.runappdirect:
             skip('cannot import module with undefined functions')
-        import sys
+        init = """
+        if (Py_IsInitialized())
+            Py_InitModule("foo", methods);
+        """
         body = """
         PyAPI_FUNC(PyObject*) PyPy_Crash1(void);
-        PyAPI_FUNC(Py_ssize_t) PyPy_Crash2(void);
-        PyAPI_FUNC(PyObject*) PyPy_Noop(PyObject*);
+        PyAPI_FUNC(long) PyPy_Crash2(void);
         static PyObject* foo_crash1(PyObject* self, PyObject *args)
         {
             return PyPy_Crash1();
         }
         static PyObject* foo_crash2(PyObject* self, PyObject *args)
         {
-            Py_ssize_t a = PyPy_Crash2();
+            int a = PyPy_Crash2();
             if (a == -1)
                 return NULL;
-            return PyFloat_FromDouble((double)a);
+            return PyFloat_FromDouble(a);
         }
         static PyObject* foo_crash3(PyObject* self, PyObject *args)
         {
-            Py_ssize_t a = PyPy_Crash2();
+            int a = PyPy_Crash2();
             if (a == -1)
                 PyErr_Clear();
-            return PyFloat_FromDouble((double)a);
+            return PyFloat_FromDouble(a);
         }
         static PyObject* foo_crash4(PyObject* self, PyObject *args)
         {
-            Py_ssize_t a = PyPy_Crash2();
-            return PyFloat_FromDouble((double)a);
-        }
-        static PyObject* foo_noop(PyObject* self, PyObject* args)
-        {
-            Py_INCREF(args);
-            return PyPy_Noop(args);
-        }
-        static PyObject* foo_set(PyObject* self, PyObject *args)
-        {
-            PyErr_SetString(PyExc_TypeError, "clear called with no error");
-            if (PyLong_Check(args)) {
-                Py_INCREF(args);
-                return args;
-            }
-            return NULL;
+            int a = PyPy_Crash2();
+            return PyFloat_FromDouble(a);
         }
         static PyObject* foo_clear(PyObject* self, PyObject *args)
         {
             PyErr_Clear();
-            if (PyLong_Check(args)) {
-                Py_INCREF(args);
-                return args;
-            }
             return NULL;
         }
         static PyMethodDef methods[] = {
@@ -758,67 +733,26 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
             { "crash2", foo_crash2, METH_NOARGS },
             { "crash3", foo_crash3, METH_NOARGS },
             { "crash4", foo_crash4, METH_NOARGS },
-            { "clear",  foo_clear,  METH_O },
-            { "set",    foo_set,    METH_O },
-            { "noop",   foo_noop,   METH_O },
+            { "clear",  foo_clear, METH_NOARGS },
             { NULL }
         };
-        static struct PyModuleDef moduledef = {
-            PyModuleDef_HEAD_INIT,
-            "%(modname)s",  /* m_name */
-            NULL,           /* m_doc */
-            -1,             /* m_size */
-            methods,        /* m_methods */
-        };
         """
-        module = self.import_module(name='foo', body=body)
-
+        module = self.import_module(name='foo', init=init, body=body)
         # uncaught interplevel exceptions are turned into SystemError
-        expected = "ZeroDivisionError('integer division"
-        exc = raises(SystemError, module.crash1)
-        # Work around difference in err msg btween CPython2 and PyPy2
-        assert exc.value.args[0].startswith(expected)
-
-        exc = raises(SystemError, module.crash2)
-        assert exc.value.args[0].startswith(expected)
-
-        # caught exception, api.cpython_api return value works
+        raises(SystemError, module.crash1)
+        raises(SystemError, module.crash2)
+        # caught exception
         assert module.crash3() == -1
-
-        expected = 'An exception was set, but function returned a value'
-        # PyPy only incompatibility/extension
-        exc = raises(SystemError, module.crash4)
-        assert exc.value.args[0] == expected
-
-        # An exception was set by the previous call, it can pass
-        # cleanly through a call that doesn't check error state
-        assert module.noop(1) == 1
-
-        # clear the exception but return NULL, signalling an error
-        expected = 'Function returned a NULL result without setting an exception'
-        exc = raises(SystemError, module.clear, None)
-        assert exc.value.args[0] == expected
-
-        # Set an exception and return NULL
-        raises(TypeError, module.set, None)
-
-        # clear any exception and return a value
-        assert module.clear(1) == 1
-
-        # Set an exception, but return non-NULL
-        expected = 'An exception was set, but function returned a value'
-        exc = raises(SystemError, module.set, 1)
-        assert exc.value.args[0] == expected
-
-
-        # Clear the exception and return a value, all is OK
-        assert module.clear(1) == 1
+        # An exception was set, but function returned a value
+        raises(SystemError, module.crash4)
+        # No exception set, but NULL returned
+        raises(SystemError, module.clear)
 
     def test_new_exception(self):
         mod = self.import_extension('foo', [
             ('newexc', 'METH_VARARGS',
              '''
-             char *name = _PyUnicode_AsString(PyTuple_GetItem(args, 0));
+             char *name = PyString_AsString(PyTuple_GetItem(args, 0));
              return PyErr_NewException(name, PyTuple_GetItem(args, 1),
                                        PyTuple_GetItem(args, 2));
              '''
@@ -831,7 +765,7 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
         mod = self.import_extension('foo', [
             ('get_hash', 'METH_NOARGS',
              '''
-             return PyLong_FromLong(_Py_HashPointer(Py_None));
+             return PyInt_FromLong(_Py_HashPointer(Py_None));
              '''
              ),
             ])
@@ -847,35 +781,34 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
              '''
              /* XXX in tests, the C type is not correct */
              #define NAME(type) ((PyTypeObject*)&type)->tp_name
-             return Py_BuildValue("ssssss",
+             return Py_BuildValue("sssss",
                                   NAME(PyCell_Type),
                                   NAME(PyModule_Type),
                                   NAME(PyProperty_Type),
                                   NAME(PyStaticMethod_Type),
-                                  NAME(PyClassMethod_Type),
                                   NAME(PyCFunction_Type)
                                   );
              '''
              ),
             ])
         assert mod.get_names() == ('cell', 'module', 'property',
-                                   'staticmethod', 'classmethod',
+                                   'staticmethod',
                                    'builtin_function_or_method')
 
     def test_get_programname(self):
         mod = self.import_extension('foo', [
             ('get_programname', 'METH_NOARGS',
              '''
-             wchar_t* name1 = Py_GetProgramName();
-             wchar_t* name2 = Py_GetProgramName();
+             char* name1 = Py_GetProgramName();
+             char* name2 = Py_GetProgramName();
              if (name1 != name2)
                  Py_RETURN_FALSE;
-             return PyUnicode_FromWideChar(name1, wcslen(name1));
+             return PyString_FromString(name1);
              '''
-             )],
-            prologue='#include <wchar.h>')
+             ),
+            ])
         p = mod.get_programname()
-        print(p)
+        print p
         assert 'py' in p
 
     @pytest.mark.skipif(only_pypy, reason='pypy only test')
@@ -887,35 +820,28 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
              char* name2 = Py_GetVersion();
              if (name1 != name2)
                  Py_RETURN_FALSE;
-             return PyUnicode_FromString(name1);
+             return PyString_FromString(name1);
              '''
              ),
             ])
         p = mod.get_version()
-        print(p)
+        print p
         assert 'PyPy' in p
 
     def test_no_double_imports(self):
         import sys, os
         try:
-            body = """
-            static struct PyModuleDef moduledef = {
-                PyModuleDef_HEAD_INIT,
-                "%(modname)s",  /* m_name */
-                NULL,           /* m_doc */
-                -1,             /* m_size */
-                NULL            /* m_methods */
-            };
-            """
             init = """
             static int _imported_already = 0;
             FILE *f = fopen("_imported_already", "w");
             fprintf(f, "imported_already: %d\\n", _imported_already);
             fclose(f);
             _imported_already = 1;
-            return PyModule_Create(&moduledef);
+            if (Py_IsInitialized()) {
+                Py_InitModule("foo", NULL);
+            }
             """
-            self.import_module(name='foo', init=init, body=body)
+            self.import_module(name='foo', init=init)
             assert 'foo' in sys.modules
 
             f = open('_imported_already')
@@ -953,15 +879,3 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
              '''
              ),
         ])
-
-    def test_consistent_flags(self):
-        import sys
-        mod = self.import_extension('foo', [
-            ('test_optimize', 'METH_NOARGS',
-             '''
-                return PyLong_FromLong(Py_OptimizeFlag);
-             '''),
-        ])
-        # This is intentionally set to -1 by default from missing.c
-        # and should be set to sys.flags.optimize at startup
-        assert mod.test_optimize() == sys.flags.optimize
