@@ -2,6 +2,7 @@ import py
 
 from pypy.module.cpyext.pyobject import PyObject, PyObjectP, make_ref, from_ref
 from pypy.module.cpyext.test.test_api import BaseApiTest
+from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
 from rpython.rtyper.lltypesystem import rffi, lltype
 
 
@@ -40,4 +41,25 @@ class TestTupleObject(BaseApiTest):
         api.PySet_Clear(w_set)
         assert space.len_w(w_set) == 0
 
-    
+    def test_anyset_checkexact(self, space, api):
+        w_set = api.PySet_New(space.wrap([1, 2, 3, 4]))
+        w_frozenset = space.newfrozenset([space.wrap(i) for i in [1, 2, 3, 4]])
+        assert api.PyAnySet_CheckExact(w_set)
+        assert api.PyAnySet_CheckExact(w_frozenset)
+
+class AppTestSetObject(AppTestCpythonExtensionBase):
+    def test_set_macro_cast(self):
+        module = self.import_extension('foo', [
+            ("test_macro_cast", "METH_NOARGS",
+             """
+             PyObject* o = PySet_New(NULL);
+             // no PySetObject
+             char* dumb_pointer = (char*) o;
+
+             PySet_GET_SIZE(o);
+             PySet_GET_SIZE(dumb_pointer);
+
+             return o;
+             """
+            )
+        ])
