@@ -65,10 +65,8 @@ class FakeSpace(ObjSpace):
     w_KeyError = W_TypeObject("KeyError")
     w_SystemExit = W_TypeObject("SystemExit")
     w_KeyboardInterrupt = W_TypeObject("KeyboardInterrupt")
-    w_RuntimeError = W_TypeObject("RuntimeError")
-    w_RecursionError = W_TypeObject("RecursionError")   # py3.5
     w_VisibleDeprecationWarning = W_TypeObject("VisibleDeprecationWarning")
-    w_None = W_Root()
+    w_None = None
 
     w_bool = W_TypeObject("bool")
     w_int = W_TypeObject("int")
@@ -77,15 +75,13 @@ class FakeSpace(ObjSpace):
     w_long = W_TypeObject("long")
     w_tuple = W_TypeObject('tuple')
     w_slice = W_TypeObject("slice")
-    w_bytes = W_TypeObject("str")
-    w_text = w_bytes
+    w_str = W_TypeObject("str")
     w_unicode = W_TypeObject("unicode")
     w_complex = W_TypeObject("complex")
     w_dict = W_TypeObject("dict")
     w_object = W_TypeObject("object")
     w_buffer = W_TypeObject("buffer")
     w_type = W_TypeObject("type")
-    w_frozenset = W_TypeObject("frozenset")
 
     def __init__(self, config=None):
         """NOT_RPYTHON"""
@@ -115,13 +111,10 @@ class FakeSpace(ObjSpace):
         return isinstance(w_obj, ListObject) or isinstance(w_obj, W_NDimArray)
 
     def len(self, w_obj):
-        return self.wrap(self.len_w(w_obj))
-
-    def len_w(self, w_obj):
         if isinstance(w_obj, ListObject):
-            return len(w_obj.items)
+            return self.wrap(len(w_obj.items))
         elif isinstance(w_obj, DictObject):
-            return len(w_obj.items)
+            return self.wrap(len(w_obj.items))
         raise NotImplementedError
 
     def getattr(self, w_obj, w_attr):
@@ -197,13 +190,6 @@ class FakeSpace(ObjSpace):
             return StringObject(obj)
         raise NotImplementedError
 
-    def newtext(self, obj):
-        return StringObject(obj)
-    newbytes = newtext
-
-    def newutf8(self, obj, l):
-        raise NotImplementedError
-
     def newlist(self, items):
         return ListObject(items)
 
@@ -211,7 +197,7 @@ class FakeSpace(ObjSpace):
         return ComplexObject(r, i)
 
     def newfloat(self, f):
-        return FloatObject(f)
+        return self.float(f)
 
     def newslice(self, start, stop, step):
         return SliceObject(self.int_w(start), self.int_w(stop),
@@ -303,16 +289,15 @@ class FakeSpace(ObjSpace):
     def index(self, w_obj):
         return self.wrap(self.int_w(w_obj))
 
-    def bytes_w(self, w_obj):
+    def str_w(self, w_obj):
         if isinstance(w_obj, StringObject):
             return w_obj.v
         raise NotImplementedError
-    text_w = bytes_w
 
-    def utf8_w(self, w_obj):
+    def unicode_w(self, w_obj):
         # XXX
         if isinstance(w_obj, StringObject):
-            return w_obj.v
+            return unicode(w_obj.v)
         raise NotImplementedError
 
     def int(self, w_obj):
@@ -413,18 +398,12 @@ class FakeSpace(ObjSpace):
     def newtuple(self, list_w):
         return ListObject(list_w)
 
-    def newtuple2(self, w_a, w_b):
-        return ListObject([w_a, w_b])
-
     def newdict(self, module=True):
         return DictObject({})
 
-    @specialize.argtype(1)
     def newint(self, i):
         if isinstance(i, IntObject):
             return i
-        if isinstance(i, base_int):
-            return LongObject(i)
         return IntObject(i)
 
     def setitem(self, obj, index, value):
@@ -508,7 +487,7 @@ class SliceObject(W_Root):
         self.step = step
 
 class StringObject(W_Root):
-    tp = FakeSpace.w_bytes
+    tp = FakeSpace.w_str
     def __init__(self, v):
         self.v = v
 
@@ -681,7 +660,7 @@ class RangeConstant(Node):
 
     def execute(self, interp):
         w_list = interp.space.newlist(
-            [interp.space.newfloat(float(i)) for i in range(self.v)]
+            [interp.space.wrap(float(i)) for i in range(self.v)]
         )
         dtype = get_dtype_cache(interp.space).w_float64dtype
         return array(interp.space, w_list, w_dtype=dtype, w_order=None)
@@ -848,7 +827,7 @@ class FunctionCall(Node):
                 w_res = arr.descr_take(interp.space, arg)
             elif self.name == "searchsorted":
                 w_res = arr.descr_searchsorted(interp.space, arg,
-                                               interp.space.newtext('left'))
+                                               interp.space.wrap('left'))
             else:
                 assert False # unreachable code
         elif self.name in THREE_ARG_FUNCTIONS:
