@@ -2,13 +2,12 @@ from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError, oefmt
-from pypy.interpreter.unicodehelper import encode_utf8
 from rpython.rlib import rgc, jit
-from rpython.rlib.objectmodel import specialize
 from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rtyper.tool import rffi_platform
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.translator.platform import platform
+from rpython.rlib.objectmodel import specialize
 
 import sys
 import weakref
@@ -120,6 +119,18 @@ class CConfigure:
         locals()[name] = rffi_platform.ConstantInteger(name)
     for name in xml_model_list:
         locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
+    for name in xml_model_list:
+        locals()[name] = rffi_platform.ConstantInteger(name)
     XML_Parser_SIZE = rffi_platform.SizeOf("XML_Parser")
 
 for k, v in rffi_platform.configure(CConfigure).items():
@@ -245,10 +256,9 @@ for index, (name, params) in enumerate(HANDLERS.items()):
                 'w_arg%d = parser.w_convert_model(space, arg%d)' % (i, i))
             converters.append(
                 'XML_FreeContentModel(parser.itself, arg%d)' % (i,))
-        elif ARG == rffi.INT:
-            converters.append("w_arg%d = space.newint(arg%d)" % (i, i))
         else:
-            assert 0, "missing conversion case"
+            converters.append(
+                'w_arg%d = space.wrap(arg%d)' % (i, i))
         real_params.append(ARG)
     converters = '; '.join(converters)
 
@@ -342,8 +352,6 @@ callback_type = lltype.Ptr(lltype.FuncType(
 XML_SetUnknownEncodingHandler = expat_external(
     'XML_SetUnknownEncodingHandler',
     [XML_Parser, callback_type, rffi.VOIDP], lltype.Void)
-XML_SetEncoding = expat_external(
-    'XML_SetEncoding', [XML_Parser, rffi.CCHARP], rffi.INT)
 
 # Declarations of external functions
 
@@ -400,13 +408,13 @@ XML_ExpatVersion = expat_external(
     'XML_ExpatVersion', [], rffi.CCHARP)
 
 def get_expat_version(space):
-    return space.newtext(rffi.charp2str(XML_ExpatVersion()))
+    return space.wrap(rffi.charp2str(XML_ExpatVersion()))
 
 def get_expat_version_info(space):
     return space.newtuple([
-        space.newint(XML_MAJOR_VERSION),
-        space.newint(XML_MINOR_VERSION),
-        space.newint(XML_MICRO_VERSION)])
+        space.wrap(XML_MAJOR_VERSION),
+        space.wrap(XML_MINOR_VERSION),
+        space.wrap(XML_MICRO_VERSION)])
 
 class Cache:
     def __init__(self, space):
@@ -422,6 +430,7 @@ class W_XMLParserType(W_Root):
 
         self.w_intern = w_intern
 
+        self.returns_unicode = True
         self.ordered_attributes = False
         self.specified_attributes = False
         self.ns_prefixes = False
@@ -476,8 +485,11 @@ getting the advantage of providing document type information to the parser.
     # Handlers management
 
     def w_convert(self, space, s):
-        from pypy.interpreter.unicodehelper import decode_utf8
-        return space.newtext(s)
+        if self.returns_unicode:
+            from pypy.interpreter.unicodehelper import decode_utf8
+            return space.wrap(decode_utf8(space, s))
+        else:
+            return space.wrap(s)
 
     def w_convert_charp(self, space, data):
         if data:
@@ -533,8 +545,8 @@ getting the advantage of providing document type information to the parser.
         children = [self.w_convert_model(space, model.c_children[i])
                     for i in range(model.c_numchildren)]
         return space.newtuple([
-            space.newint(model.c_type),
-            space.newint(model.c_quant),
+            space.wrap(model.c_type),
+            space.wrap(model.c_quant),
             self.w_convert_charp(space, model.c_name),
             space.newtuple(children)])
 
@@ -579,8 +591,8 @@ getting the advantage of providing document type information to the parser.
         # Yes, supports only 8bit encodings
         translationmap = space.unicode_w(
             space.call_method(
-                space.newbytes(self.all_chars), "decode",
-                space.newtext(name), space.newtext("replace")))
+                space.wrap(self.all_chars), "decode",
+                space.wrap(name), space.wrap("replace")))
 
         if len(translationmap) != 256:
             raise oefmt(space.w_ValueError,
@@ -615,7 +627,7 @@ getting the advantage of providing document type information to the parser.
 
 
     def get_namespace_prefixes(self, space):
-        return space.newbool(self.ns_prefixes)
+        return space.wrap(self.ns_prefixes)
 
     def set_namespace_prefixes(self, space, w_value):
         self.ns_prefixes = space.bool_w(w_value)
@@ -623,17 +635,11 @@ getting the advantage of providing document type information to the parser.
 
     # Parse methods
 
-    @unwrap_spec(isfinal=int)
-    def Parse(self, space, w_data, isfinal=False):
+    @unwrap_spec(data=str, isfinal=bool)
+    def Parse(self, space, data, isfinal=False):
         """Parse(data[, isfinal])
 Parse XML data.  `isfinal' should be true at end of input."""
-        if space.isinstance_w(w_data, space.w_unicode):
-            data = encode_utf8(space, w_data.unicode_w(space))
-            # Explicitly set UTF-8 encoding. Return code ignored.
-            XML_SetEncoding(self.itself, "utf-8")
-        else:
-            data = space.charbuf_w(w_data)
-        isfinal = bool(isfinal)
+
         res = XML_Parse(self.itself, data, len(data), isfinal)
         if self._exc_info:
             e = self._exc_info
@@ -643,19 +649,20 @@ Parse XML data.  `isfinal' should be true at end of input."""
             exc = self.set_error(space, XML_GetErrorCode(self.itself))
             raise exc
         self.flush_character_buffer(space)
-        return space.newint(res)
+        return space.wrap(res)
 
     def ParseFile(self, space, w_file):
         """ParseFile(file)
 Parse XML data from file-like object."""
         eof = False
         while not eof:
-            w_data = space.call_method(w_file, 'read', space.newint(2048))
-            eof = space.len_w(w_data) == 0
-            w_res = self.Parse(space, w_data, isfinal=eof)
+            w_data = space.call_method(w_file, 'read', space.wrap(2048))
+            data = space.str_w(w_data)
+            eof = len(data) == 0
+            w_res = self.Parse(space, data, isfinal=eof)
         return w_res
 
-    @unwrap_spec(base='text')
+    @unwrap_spec(base=str)
     def SetBase(self, space, base):
         XML_SetBase(self.itself, base)
 
@@ -666,12 +673,12 @@ information passed to the ExternalEntityRefHandler."""
         if space.is_w(w_context, space.w_None):
             context = None
         else:
-            context = space.text_w(w_context)
+            context = space.str_w(w_context)
 
         if space.is_none(w_encoding):
             encoding = None
         else:
-            encoding = space.text_w(w_encoding)
+            encoding = space.str_w(w_encoding)
 
         xmlparser = XML_ExternalEntityParserCreate(
             self.itself, context, encoding)
@@ -684,13 +691,13 @@ information passed to the ExternalEntityRefHandler."""
         for i in range(NB_HANDLERS):
             parser.handlers[i] = self.handlers[i]
 
-        return parser
+        return space.wrap(parser)
 
     def flush_character_buffer(self, space):
         if not self.buffer:
             return
         w_data = space.call_function(
-            space.getattr(space.newtext(''), space.newtext('join')),
+            space.getattr(space.wrap(''), space.wrap('join')),
             space.newlist(self.buffer))
         self.buffer = []
         self.buffer_used = 0
@@ -706,30 +713,30 @@ information passed to the ExternalEntityRefHandler."""
         colno = XML_GetCurrentColumnNumber(self.itself)
         msg = "%s: line %d, column %d" % (err, lineno, colno)
         w_errorcls = space.fromcache(Cache).w_error
-        w_error = space.call_function(w_errorcls, space.newtext(msg))
-        space.setattr(w_error, space.newtext("code"), space.newint(code))
-        space.setattr(w_error, space.newtext("offset"), space.newint(colno))
-        space.setattr(w_error, space.newtext("lineno"), space.newint(lineno))
+        w_error = space.call_function(w_errorcls, space.wrap(msg))
+        space.setattr(w_error, space.wrap("code"), space.wrap(code))
+        space.setattr(w_error, space.wrap("offset"), space.wrap(colno))
+        space.setattr(w_error, space.wrap("lineno"), space.wrap(lineno))
 
         self.w_error = w_error
         return OperationError(w_errorcls, w_error)
 
     def descr_ErrorCode(self, space):
-        return space.newint(XML_GetErrorCode(self.itself))
+        return space.wrap(XML_GetErrorCode(self.itself))
 
     def descr_ErrorLineNumber(self, space):
-        return space.newint(XML_GetErrorLineNumber(self.itself))
+        return space.wrap(XML_GetErrorLineNumber(self.itself))
 
     def descr_ErrorColumnNumber(self, space):
-        return space.newint(XML_GetErrorColumnNumber(self.itself))
+        return space.wrap(XML_GetErrorColumnNumber(self.itself))
 
     def descr_ErrorByteIndex(self, space):
-        return space.newint(XML_GetErrorByteIndex(self.itself))
+        return space.wrap(XML_GetErrorByteIndex(self.itself))
 
     def get_buffer_size(self, space):
-        return space.newint(self.buffer_size)
+        return space.wrap(self.buffer_size)
     def set_buffer_size(self, space, w_value):
-        value = space.getindex_w(w_value, space.w_OverflowError)
+        value = space.getindex_w(w_value, space.w_TypeError)
         if value <= 0:
             raise oefmt(space.w_ValueError,
                         "buffer_size must be greater than zero")
@@ -737,7 +744,7 @@ information passed to the ExternalEntityRefHandler."""
         self.buffer_size = value
 
     def get_buffer_text(self, space):
-        return space.newbool(self.buffer is not None)
+        return space.wrap(self.buffer is not None)
     def set_buffer_text(self, space, w_value):
         if space.is_true(w_value):
             self.buffer = []
@@ -755,7 +762,7 @@ information passed to the ExternalEntityRefHandler."""
 
 def bool_property(name, cls, doc=None):
     def fget(space, obj):
-        return space.newbool(getattr(obj, name))
+        return space.wrap(getattr(obj, name))
     def fset(space, obj, value):
         setattr(obj, name, space.bool_w(value))
     return GetSetProperty(fget, fset, cls=cls, doc=doc)
@@ -777,6 +784,7 @@ W_XMLParserType.typedef = TypeDef(
     namespace_prefixes = GetSetProperty(W_XMLParserType.get_namespace_prefixes,
                                         W_XMLParserType.set_namespace_prefixes,
                                         cls=W_XMLParserType),
+    returns_unicode = bool_property('returns_unicode', W_XMLParserType),
     ordered_attributes = bool_property('ordered_attributes', W_XMLParserType),
     specified_attributes = bool_property('specified_attributes', W_XMLParserType),
     intern = GetSetProperty(W_XMLParserType.get_intern, cls=W_XMLParserType),
@@ -803,17 +811,17 @@ def ParserCreate(space, w_encoding=None, w_namespace_separator=None,
 Return a new XML parser object."""
     if space.is_none(w_encoding):
         encoding = None
-    elif space.isinstance_w(w_encoding, space.w_text):
-        encoding = space.text_w(w_encoding)
+    elif space.isinstance_w(w_encoding, space.w_str):
+        encoding = space.str_w(w_encoding)
     else:
         raise oefmt(space.w_TypeError,
-                    "ParserCreate() argument 1 must be str or None, not %T",
+                    "ParserCreate() argument 1 must be string or None, not %T",
                     w_encoding)
 
     if space.is_none(w_namespace_separator):
-        namespace_separator = -1
-    elif space.isinstance_w(w_namespace_separator, space.w_text):
-        separator = space.text_w(w_namespace_separator)
+        namespace_separator = 0
+    elif space.isinstance_w(w_namespace_separator, space.w_str):
+        separator = space.str_w(w_namespace_separator)
         if len(separator) == 0:
             namespace_separator = 0
         elif len(separator) == 1:
@@ -824,7 +832,7 @@ Return a new XML parser object."""
                         "omitted, or None")
     else:
         raise oefmt(space.w_TypeError,
-                    "ParserCreate() argument 2 must be str or None, not %T",
+                    "ParserCreate() argument 2 must be string or None, not %T",
                     w_namespace_separator)
 
     # Explicitly passing None means no interning is desired.
@@ -834,7 +842,7 @@ Return a new XML parser object."""
     elif space.is_w(w_intern, space.w_None):
         w_intern = None
 
-    if namespace_separator >= 0:
+    if namespace_separator:
         xmlparser = XML_ParserCreateNS(
             encoding,
             rffi.cast(rffi.CHAR, namespace_separator))
@@ -843,19 +851,19 @@ Return a new XML parser object."""
     # Currently this is just the size of the pointer and some estimated bytes.
     # The struct isn't actually defined in expat.h - it is in xmlparse.c
     # XXX: find a good estimate of the XML_ParserStruct
+    rgc.add_memory_pressure(XML_Parser_SIZE + 300)
     if not xmlparser:
         raise oefmt(space.w_RuntimeError, "XML_ParserCreate failed")
 
     parser = W_XMLParserType(space, xmlparser, w_intern)
-    rgc.add_memory_pressure(XML_Parser_SIZE + 300, parser)
     XML_SetUnknownEncodingHandler(
         parser.itself, UnknownEncodingHandlerData_callback,
         rffi.cast(rffi.VOIDP, parser.id))
-    return parser
+    return space.wrap(parser)
 
 @unwrap_spec(code=int)
 def ErrorString(space, code):
     """ErrorString(errno) -> string
 Returns string error for given number."""
-    return space.newtext(rffi.charp2str(XML_ErrorString(code)))
+    return space.wrap(rffi.charp2str(XML_ErrorString(code)))
 
