@@ -22,7 +22,6 @@ class _AppTestSelect:
         finally:
             readend.close()
             writeend.close()
-        raises(ValueError, select.select, [], [], [], -1)
 
     def test_list_tuple(self):
         import time, select
@@ -275,7 +274,7 @@ class AppTestSelectWithPipes(_AppTestSelect):
         return FileAsSocket(s1), FileAsSocket(s2)
 
     def test_poll_threaded(self):
-        import os, select, _thread as thread, time
+        import os, select, thread, time
         if not hasattr(select, 'poll'):
             skip("no select.poll() on this platform")
         r, w = os.pipe()
@@ -288,7 +287,7 @@ class AppTestSelectWithPipes(_AppTestSelect):
             t = thread.start_new_thread(pollster.poll, ())
             try:
                 time.sleep(0.3)
-                for i in range(100): print(''),  # to release GIL untranslated
+                for i in range(5): print '',  # to release GIL untranslated
                 # trigger ufds array reallocation
                 for fd in rfds:
                     pollster.unregister(fd)
@@ -299,7 +298,7 @@ class AppTestSelectWithPipes(_AppTestSelect):
                 # and make the call to poll() from the thread return
                 os.write(w, b'spam')
                 time.sleep(0.3)
-                for i in range(100): print(''),  # to release GIL untranslated
+                for i in range(5): print '',  # to release GIL untranslated
         finally:
             os.close(r)
             os.close(w)
@@ -319,11 +318,6 @@ class AppTestSelectWithPipes(_AppTestSelect):
         # ^^^ CPython gives 100, PyPy gives 1.  I think both are OK as
         # long as there is no crash.
 
-    def test_PIPE_BUF(self):
-        # no PIPE_BUF on Windows; this test class is skipped on Windows.
-        import select
-        assert isinstance(select.PIPE_BUF, int)
-
 
 class AppTestSelectWithSockets(_AppTestSelect):
     """Same tests with connected sockets.
@@ -335,7 +329,7 @@ class AppTestSelectWithSockets(_AppTestSelect):
     }
 
     import os
-    if hasattr(os, 'uname') and os.uname()[4] == 's390x':
+    if os.uname()[4] == 's390x':
         py.test.skip("build bot for s390x cannot open sockets")
 
     def w_make_server(self):
@@ -358,15 +352,13 @@ class AppTestSelectWithSockets(_AppTestSelect):
     def w_getpair(self):
         """Helper method which returns a pair of connected sockets."""
         import socket
-        import _thread
-
-        self.make_server()
+        import thread
 
         self.make_server()
 
         self.sock.listen(1)
         s2 = socket.socket()
-        _thread.start_new_thread(s2.connect, (self.sockaddress,))
+        thread.start_new_thread(s2.connect, (self.sockaddress,))
         s1, addr2 = self.sock.accept()
 
         # speed up the tests that want to fill the buffers
