@@ -30,7 +30,7 @@ from pypy.interpreter.gateway import unwrap_spec, Arguments
 #   called from the rffi-generated wrapper).  The gc_thread_run()
 #   operation will automatically notice that the current thread id was
 #   not seen before, and (in shadowstack) it will allocate and use a
-#   fresh new stack.
+#   fresh new stack.  Again, this has no effect in asmgcc.
 #
 # * Only then does bootstrap() really run.  The first thing it does
 #   is grab the start-up information (app-level callable and args)
@@ -43,7 +43,7 @@ from pypy.interpreter.gateway import unwrap_spec, Arguments
 #   thread.
 #
 # * Just before a thread finishes, gc_thread_die() is called to free
-#   its shadow stack.
+#   its shadow stack.  This has no effect in asmgcc.
 
 
 class Bootstrapper(object):
@@ -102,7 +102,6 @@ class Bootstrapper(object):
                 os.write(STDERR, "\n")
             except OSError:
                 pass
-        #
         bootstrapper.nbthreads -= 1
         rthread.gc_thread_die()
     bootstrap = staticmethod(bootstrap)
@@ -134,8 +133,7 @@ class Bootstrapper(object):
         except OperationError as e:
             if not e.match(space, space.w_SystemExit):
                 ident = rthread.get_ident()
-                # PyPy adds the thread ident
-                where = 'in thread %d started by ' % ident
+                where = 'thread %d started by ' % ident
                 e.write_unraisable(space, where, w_callable, with_traceback=True)
             e.clear(space)
         # clean up space.threadlocals to remove the ExecutionContext
@@ -160,6 +158,12 @@ def reinit_threads(space):
     bootstrapper.reinit()
     rthread.thread_after_fork()
 
+    # Clean the threading module after a fork()
+    w_modules = space.sys.get('modules')
+    w_threading = space.finditem_str(w_modules, 'threading')
+    if w_threading is not None:
+        space.call_method(w_threading, "_after_fork")
+
 def start_new_thread(space, w_callable, w_args, w_kwargs=None):
     """Start a new thread and return its identifier.  The thread will call the
 function with positional arguments from the tuple args and keyword arguments
@@ -185,7 +189,7 @@ printed unless the exception is SystemExit."""
             raise
     except rthread.error:
         raise wrap_thread_error(space, "can't start new thread")
-    return space.newint(ident)
+    return space.wrap(ident)
 
 
 def get_ident(space):
@@ -197,7 +201,7 @@ allocated consecutive numbers starting at 1, this behavior should not
 be relied upon, and the number should be seen purely as a magic cookie.
 A thread's identity may be reused for another thread after it exits."""
     ident = rthread.get_ident()
-    return space.newint(ident)
+    return space.wrap(ident)
 
 @unwrap_spec(size=int)
 def stack_size(space, size=0):
@@ -227,7 +231,7 @@ the suggested approach in the absence of more specific information)."""
         raise oefmt(space.w_ValueError, "size not valid: %d bytes", size)
     if error == -2:
         raise wrap_thread_error(space, "setting stack size not supported")
-    return space.newint(old_size)
+    return space.wrap(old_size)
 
 def _count(space):
     """_count() -> integer
@@ -238,7 +242,7 @@ yet finished.
 
 This function is meant for internal and specialized purposes only.
 In most applications `threading.enumerate()` should be used instead."""
-    return space.newint(bootstrapper.nbthreads)
+    return space.wrap(bootstrapper.nbthreads)
 
 def exit(space):
     """This is synonymous to ``raise SystemExit''.  It will cause the current
