@@ -1,14 +1,3 @@
-import os
-import stat
-
-from rpython.annotator.model import s_Str0
-from rpython.rlib.objectmodel import enforceargs
-from rpython.rlib.unroll import unrolling_iterable
-from rpython.rlib.rzipfile import RZipFile, BadZipfile
-from rpython.rlib.rzlib import RZlibError
-from rpython.rlib.rstring import assert_str0
-from rpython.rlib.signature import signature, finishsigs
-from rpython.rlib import types
 
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
@@ -17,6 +6,11 @@ from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from pypy.interpreter.module import Module
 from pypy.module.imp import importing
 from pypy.module.zlib.interp_zlib import zlib_error
+from rpython.rlib.unroll import unrolling_iterable
+from rpython.rlib.rzipfile import RZipFile, BadZipfile
+from rpython.rlib.rzlib import RZlibError
+import os
+import stat
 
 ZIPSEP = '/'
 # note that zipfiles always use slash, but for OSes with other
@@ -24,8 +18,10 @@ ZIPSEP = '/'
 
 ENUMERATE_EXTS = unrolling_iterable(
     [(True, True, ZIPSEP + '__init__.pyc'),
+     (True, True, ZIPSEP + '__init__.pyo'),
      (False, True, ZIPSEP + '__init__.py'),
      (True, False, '.pyc'),
+     (True, False, '.pyo'),
      (False, False, '.py')])
 
 class Cache:
@@ -50,58 +46,56 @@ class W_ZipCache(W_Root):
     # I don't care about speed of those, they're obscure anyway
     # THIS IS A TERRIBLE HACK TO BE CPYTHON COMPATIBLE
 
-    def getitem(self, space, w_name):
-        return self._getitem(space, space.text_w(w_name))
-
-    def _getitem(self, space, name):
+    @unwrap_spec(name=str)
+    def getitem(self, space, name):
         try:
             w_zipimporter = self.cache[name]
         except KeyError:
-            raise OperationError(space.w_KeyError, space.newfilename(name))
+            raise OperationError(space.w_KeyError, space.wrap(name))
         assert isinstance(w_zipimporter, W_ZipImporter)
-        w_fs = space.newfilename
+        w = space.wrap
         w_d = space.newdict()
         for key, info in w_zipimporter.zip_file.NameToInfo.iteritems():
             if ZIPSEP != os.path.sep:
                 key = key.replace(ZIPSEP, os.path.sep)
-            space.setitem(w_d, w_fs(key), space.newtuple([
-                w_fs(info.filename), space.newint(info.compress_type), space.newint(info.compress_size),
-                space.newint(info.file_size), space.newint(info.file_offset), space.newint(info.dostime),
-                space.newint(info.dosdate), space.newint(info.CRC)]))
+            space.setitem(w_d, w(key), space.newtuple([
+                w(info.filename), w(info.compress_type), w(info.compress_size),
+                w(info.file_size), w(info.file_offset), w(info.dostime),
+                w(info.dosdate), w(info.CRC)]))
         return w_d
 
     def keys(self, space):
-        return space.newlist([space.newfilename(s)
+        return space.newlist([space.wrap(s)
                               for s in self.cache.keys()])
 
     def values(self, space):
         keys = self.cache.keys()
-        values_w = [self._getitem(space, key) for key in keys]
+        values_w = [self.getitem(space, key) for key in keys]
         return space.newlist(values_w)
 
     def items(self, space):
-        w_fs = space.newfilename
-        items_w = [space.newtuple([w_fs(key), self._getitem(space, key)])
+        w = space.wrap
+        items_w = [space.newtuple([w(key), self.getitem(space, key)])
                    for key in self.cache.keys()]
         return space.newlist(items_w)
 
-    def iteratekeys(self, space):
+    def iterkeys(self, space):
         return space.iter(self.keys(space))
 
-    def iteratevalues(self, space):
+    def itervalues(self, space):
         return space.iter(self.values(space))
 
     def iteritems(self, space):
         return space.iter(self.items(space))
 
-    @unwrap_spec(name='text')
+    @unwrap_spec(name=str)
     def contains(self, space, name):
         return space.newbool(name in self.cache)
 
     def clear(self, space):
         self.cache = {}
 
-    @unwrap_spec(name='text')
+    @unwrap_spec(name=str)
     def delitem(self, space, name):
         del self.cache[name]
 
@@ -109,20 +103,19 @@ W_ZipCache.typedef = TypeDef(
     'zip_dict',
     __getitem__ = interp2app(W_ZipCache.getitem),
     __contains__ = interp2app(W_ZipCache.contains),
-    __iter__ = interp2app(W_ZipCache.iteratekeys),
+    __iter__ = interp2app(W_ZipCache.iterkeys),
     items = interp2app(W_ZipCache.items),
     iteritems = interp2app(W_ZipCache.iteritems),
     keys = interp2app(W_ZipCache.keys),
-    iterkeys = interp2app(W_ZipCache.iteratekeys),
+    iterkeys = interp2app(W_ZipCache.iterkeys),
     values = interp2app(W_ZipCache.values),
-    itervalues = interp2app(W_ZipCache.iteratevalues),
+    itervalues = interp2app(W_ZipCache.itervalues),
     clear = interp2app(W_ZipCache.clear),
     __delitem__ = interp2app(W_ZipCache.delitem),
 )
 
 zip_cache = W_ZipCache()
 
-@finishsigs
 class W_ZipImporter(W_Root):
     def __init__(self, space, name, filename, zip_file, prefix):
         self.space = space
@@ -133,8 +126,8 @@ class W_ZipImporter(W_Root):
 
     def getprefix(self, space):
         if ZIPSEP == os.path.sep:
-            return space.newfilename(self.prefix)
-        return space.newfilename(self.prefix.replace(ZIPSEP, os.path.sep))
+            return space.wrap(self.prefix)
+        return space.wrap(self.prefix.replace(ZIPSEP, os.path.sep))
 
     def _find_relative_path(self, filename):
         if filename.startswith(self.filename):
@@ -145,25 +138,24 @@ class W_ZipImporter(W_Root):
             filename = filename.replace(os.path.sep, ZIPSEP)
         return filename
 
-    @signature(types.self(), types.str0(), returns=types.str0())
     def corr_zname(self, fname):
         if ZIPSEP != os.path.sep:
             return fname.replace(ZIPSEP, os.path.sep)
         else:
             return fname
 
-    @enforceargs(filename=s_Str0, typecheck=False)
     def import_py_file(self, space, modname, filename, buf, pkgpath):
-        w_mod = Module(space, space.newtext(modname))
+        w = space.wrap
+        w_mod = w(Module(space, w(modname)))
         real_name = self.filename + os.path.sep + self.corr_zname(filename)
-        space.setattr(w_mod, space.newtext('__loader__'), self)
+        space.setattr(w_mod, w('__loader__'), space.wrap(self))
         importing._prepare_module(space, w_mod, real_name, pkgpath)
         co_filename = self.make_co_filename(filename)
         code_w = importing.parse_source_module(space, co_filename, buf)
-        importing.exec_code_module(space, w_mod, code_w, co_filename, None)
-        return w_mod
+        return importing.exec_code_module(space, w_mod, code_w, w(modname))
 
     def _parse_mtime(self, space, filename):
+        w = space.wrap
         try:
             info = self.zip_file.NameToInfo[filename]
             t = info.date_time
@@ -171,15 +163,13 @@ class W_ZipImporter(W_Root):
             return 0
         else:
             w_mktime = space.getattr(space.getbuiltinmodule('time'),
-                                     space.newtext('mktime'))
+                                     w('mktime'))
             # XXX this is incredible fishing around module limitations
             #     in order to compare timestamps of .py and .pyc files
             # we need time.mktime support on rpython level
-            w = space.newint
             all = [w(t[0]), w(t[1]), w(t[2]), w(t[3]), w(t[4]),
                    w(t[5]), w(0), w(1), w(-1)]
-            mtime = int(space.float_w(space.call_function(
-                w_mktime, space.newtuple(all))))
+            mtime = int(space.float_w(space.call_function(w_mktime, space.newtuple(all))))
             return mtime
 
     def check_newer_pyfile(self, space, filename, timestamp):
@@ -196,50 +186,28 @@ class W_ZipImporter(W_Root):
             d = -d
         return d > 1    # more than one second => different
 
-    def can_use_pyc(self, space, filename, magic, buf):
+    def can_use_pyc(self, space, filename, magic, timestamp):
         if magic != importing.get_pyc_magic(space):
             return False
-        bitfield = importing._get_long(buf[4:8])
-        if bitfield != 0:
-            # Hash-based pyc. CPython (and we) currently refuse to handle
-            # checked hash-based pycs in zips. We could validate hash-based
-            # pycs against the source, but it seems likely that most people
-            # putting hash-based pycs in a zipfile will use unchecked ones.
-            w_imp = space.getbuiltinmodule("_imp")
-            w_mode = space.getattr(w_imp, space.newtext("check_hash_based_pycs"))
-            mode = space.text_w(w_mode)
-            if mode == "always":
-                return False
-            if mode == "default" and bitfield & 0b10:
-                return False
-            return True
-        timestamp = importing._get_long(buf[8:12])
         if self.check_newer_pyfile(space, filename[:-1], timestamp):
             return False
         return True
 
-    @enforceargs(filename=s_Str0, typecheck=False)
     def import_pyc_file(self, space, modname, filename, buf, pkgpath):
-        # a field are four bytes
-        # | magic | 0b00 | timestamp | size   # traditional timestamp based pyc
-        # | magic | 0b01 | hash1     | hash2  # unchecked
-        # | magic | 0b11 | hash1     | hash2  # checked
-
-        if len(buf) < 16:
-            raise oefmt(get_error(space), "bad pyc data")
+        w = space.wrap
         magic = importing._get_long(buf[:4])
-        if not self.can_use_pyc(space, filename, magic, buf):
+        timestamp = importing._get_long(buf[4:8])
+        if not self.can_use_pyc(space, filename, magic, timestamp):
             return None
-        buf = buf[16:]  # XXX ugly copy, should use sequential read instead
-        w_mod = Module(space, space.newtext(modname))
+        buf = buf[8:] # XXX ugly copy, should use sequential read instead
+        w_mod = w(Module(space, w(modname)))
         real_name = self.filename + os.path.sep + self.corr_zname(filename)
-        space.setattr(w_mod, space.newtext('__loader__'), self)
+        space.setattr(w_mod, w('__loader__'), space.wrap(self))
         importing._prepare_module(space, w_mod, real_name, pkgpath)
-        result = importing.load_compiled_module(
-            space, space.newtext(modname),
-            w_mod, real_name, magic, buf)
-        return result
-
+        w_result = importing.load_compiled_module(space, w(modname), w_mod,
+                                                filename, magic, timestamp,
+                                                buf)
+        return w_result
 
     def have_modulefile(self, space, filename):
         if ZIPSEP != os.path.sep:
@@ -250,22 +218,22 @@ class W_ZipImporter(W_Root):
         except KeyError:
             return False
 
-    @unwrap_spec(fullname='text')
+    @unwrap_spec(fullname=str)
     def find_module(self, space, fullname, w_path=None):
         filename = self.make_filename(fullname)
         for _, _, ext in ENUMERATE_EXTS:
             if self.have_modulefile(space, filename + ext):
-                return self
+                return space.wrap(self)
 
     def make_filename(self, fullname):
-        startpos = fullname.rfind('.') + 1  # 0 when not found
+        startpos = fullname.rfind('.') + 1 # 0 when not found
         assert startpos >= 0
         subname = fullname[startpos:]
         if ZIPSEP == os.path.sep:
             return self.prefix + subname.replace('.', '/')
         else:
-            return (self.prefix.replace(os.path.sep, ZIPSEP) +
-                    subname.replace('.', '/'))
+            return self.prefix.replace(os.path.sep, ZIPSEP) + \
+                    subname.replace('.', '/')
 
     def make_co_filename(self, filename):
         """
@@ -273,20 +241,13 @@ class W_ZipImporter(W_Root):
         gets in code_object.co_filename. Something like
         'myfile.zip/mymodule.py'
         """
-        if ZIPSEP != os.path.sep:
-            filename = filename.replace(ZIPSEP, os.path.sep)
         return self.filename + os.path.sep + filename
 
-    def load_module(self, space, w_fullname):
-        fullname = space.text_w(w_fullname)
+    @unwrap_spec(fullname=str)
+    def load_module(self, space, fullname):
+        w = space.wrap
         filename = self.make_filename(fullname)
         for compiled, is_package, ext in ENUMERATE_EXTS:
-            if '\x00' in filename:
-                # Special case to make the annotator happy:
-                # filenames inside ZIPs shouldn't contain NULs so no module can
-                # possibly be found in this case
-                break
-            filename = assert_str0(filename)
             fname = filename + ext
             try:
                 buf = self.zip_file.read(fname)
@@ -294,7 +255,7 @@ class W_ZipImporter(W_Root):
                 pass
             except RZlibError as e:
                 # in this case, CPython raises the direct exception coming
-                # from the zlib module: let's do the same
+                # from the zlib module: let's to the same
                 raise zlib_error(space, e.msg)
             else:
                 if is_package:
@@ -306,172 +267,101 @@ class W_ZipImporter(W_Root):
                     if compiled:
                         w_result = self.import_pyc_file(space, fullname, fname,
                                                         buf, pkgpath)
-                        if w_result is None:
-                            continue
+                        if w_result is not None:
+                            return w_result
                     else:
-                        w_result = self.import_py_file(space, fullname, fname,
+                        return self.import_py_file(space, fullname, fname,
                                                    buf, pkgpath)
-                    if space.sys.get_flag('verbose') >= 1:
-                        w_stderr = space.sys.get('stderr')
-                        message = "import %s # loaded from Zip %s%s%s\n" % (
-                                fullname, self.filename, os.path.sep, fname)
-                        space.call_method(w_stderr, "write",
-                                          space.newtext(message))
-                    return w_result
                 except:
                     w_mods = space.sys.get('modules')
-                    space.call_method(w_mods, 'pop', w_fullname, space.w_None)
+                    space.call_method(w_mods, 'pop', w(fullname), space.w_None)
                     raise
-        raise oefmt(get_error(space), "can't find module %R", w_fullname)
+        raise oefmt(get_error(space), "can't find module '%s'", fullname)
 
-    @unwrap_spec(filename='text')
+    @unwrap_spec(filename=str)
     def get_data(self, space, filename):
         filename = self._find_relative_path(filename)
+        w = space.wrap
         try:
             data = self.zip_file.read(filename)
-            return space.newbytes(data)
+            return w(data)
         except (KeyError, OSError, BadZipfile):
             raise oefmt(space.w_IOError, "Error reading file")
         except RZlibError as e:
             # in this case, CPython raises the direct exception coming
-            # from the zlib module: let's do the same
+            # from the zlib module: let's to the same
             raise zlib_error(space, e.msg)
 
-    def get_code(self, space, w_fullname):
-        fullname = space.text_w(w_fullname)
+    @unwrap_spec(fullname=str)
+    def get_code(self, space, fullname):
         filename = self.make_filename(fullname)
         for compiled, _, ext in ENUMERATE_EXTS:
-            if '\x00' in filename:
-                # Special case to make the annotator happy:
-                # filenames inside ZIPs shouldn't contain NULs so no module can
-                # possibly be found in this case
-                break
-            filename = assert_str0(filename)
             if self.have_modulefile(space, filename + ext):
                 w_source = self.get_data(space, filename + ext)
-                source = space.bytes_w(w_source)
+                source = space.str_w(w_source)
                 if compiled:
-                    if len(source) < 16:
-                        raise oefmt(get_error(space), "bad pyc data")
                     magic = importing._get_long(source[:4])
-                    if not self.can_use_pyc(space, filename + ext, magic, source):
+                    timestamp = importing._get_long(source[4:8])
+                    if not self.can_use_pyc(space, filename + ext,
+                                            magic, timestamp):
                         continue
-                    # zipimport ignores the size field
-                    w_code = importing.read_compiled_module(
-                        space, filename + ext, source[16:])
+                    code_w = importing.read_compiled_module(
+                        space, filename + ext, source[8:])
                 else:
                     co_filename = self.make_co_filename(filename+ext)
-                    w_code = importing.parse_source_module(
+                    code_w = importing.parse_source_module(
                         space, co_filename, source)
-                return w_code
+                return space.wrap(code_w)
         raise oefmt(get_error(space),
-                    "Cannot find source or code for %R in %R",
-                    w_fullname, space.newfilename(self.name))
+                    "Cannot find source or code for %s in %s",
+                    filename, self.name)
 
-    @unwrap_spec(fullname='text')
+    @unwrap_spec(fullname=str)
     def get_source(self, space, fullname):
         filename = self.make_filename(fullname)
         found = False
         for compiled, _, ext in ENUMERATE_EXTS:
-            if '\x00' in filename:
-                # Special case to make the annotator happy:
-                # filenames inside ZIPs shouldn't contain NULs so no module can
-                # possibly be found in this case
-                break
-            filename = assert_str0(filename)
             fname = filename + ext
             if self.have_modulefile(space, fname):
                 if not compiled:
-                    w_data = self.get_data(space, fname)
-                    # XXX CPython does not handle the coding cookie either.
-                    return space.call_method(w_data, "decode",
-                                             space.newtext("utf-8"))
+                    return self.get_data(space, fname)
                 else:
                     found = True
         if found:
-            # We have the module, but no source.
             return space.w_None
         raise oefmt(get_error(space),
-                    "Cannot find source for %R in %R",
-                    space.newfilename(filename),
-                    space.newfilename(self.name))
+                    "Cannot find source for %s in %s", filename, self.name)
 
-    def get_filename(self, space, w_fullname):
-        fullname = space.text_w(w_fullname)
+    @unwrap_spec(fullname=str)
+    def get_filename(self, space, fullname):
         filename = self.make_filename(fullname)
         for _, is_package, ext in ENUMERATE_EXTS:
-            if '\x00' in filename:
-                # Special case to make the annotator happy:
-                # filenames inside ZIPs shouldn't contain NULs so no module can
-                # possibly be found in this case
-                break
-            filename = assert_str0(filename)
             if self.have_modulefile(space, filename + ext):
-                return space.newfilename(self.filename + os.path.sep +
-                                            self.corr_zname(filename + ext))
+                return space.wrap(self.filename + os.path.sep +
+                                  self.corr_zname(filename + ext))
         raise oefmt(get_error(space),
-                    "Cannot find module %R in %R",
-                    space.newfilename(filename),
-                    space.newfilename(self.name))
+                    "Cannot find module %s in %s", filename, self.name)
 
-    def is_package(self, space, w_fullname):
-        fullname = space.text_w(w_fullname)
+    @unwrap_spec(fullname=str)
+    def is_package(self, space, fullname):
         filename = self.make_filename(fullname)
         for _, is_package, ext in ENUMERATE_EXTS:
-            if '\x00' in filename:
-                # Special case to make the annotator happy:
-                # filenames inside ZIPs shouldn't contain NULs so no module can
-                # possibly be found in this case
-                break
-            filename = assert_str0(filename)
             if self.have_modulefile(space, filename + ext):
-                return space.newbool(is_package)
+                return space.wrap(is_package)
         raise oefmt(get_error(space),
-                    "Cannot find module %R in %R",
-                    space.newfilename(filename),
-                    space.newfilename(self.name))
+                    "Cannot find module %s in %s", filename, self.name)
 
     def getarchive(self, space):
         space = self.space
-        return space.newfilename(self.filename)
+        return space.wrap(self.filename)
 
-    def _find_loader(self, space, fullname):
-        if '\x00' in fullname:
-            # Special case to make the annotator happy:
-            # filenames inside ZIPs shouldn't contain NULs so no module can
-            # possibly be found in this case
-            return False, None
-        fullname = assert_str0(fullname)
-        filename = self.make_filename(fullname)
-        filename = assert_str0(filename)
-        for _, _, ext in ENUMERATE_EXTS:
-            if self.have_modulefile(space, filename + ext):
-                return True, None
-        # See if this is a directory (part of a namespace pkg)
-        dirpath = self.prefix + fullname.split(".")[-1]
-        if self.have_modulefile(space, dirpath + ZIPSEP):
-            return True, self.filename + os.path.sep + self.corr_zname(dirpath)
-        return False, None
-
-    @unwrap_spec(fullname='text')
-    def find_loader(self, space, fullname, w_path=None):
-        found, ns_portion = self._find_loader(space, fullname)
-        if not found:
-            result = [space.w_None, space.newlist([])]
-        elif not ns_portion:
-            result = [self, space.newlist([])]
-        else:
-            result = [space.w_None,
-                      space.newlist([space.newfilename(ns_portion)])]
-        return space.newtuple(result)
-
-def descr_new_zipimporter(space, w_type, w_name):
-    name = space.fsencode_w(w_name)
+@unwrap_spec(name='str0')
+def descr_new_zipimporter(space, w_type, name):
     ok = False
     parts_ends = [i for i in range(0, len(name))
-            if name[i] == os.path.sep or name[i] == ZIPSEP]
+                    if name[i] == os.path.sep or name[i] == ZIPSEP]
     parts_ends.append(len(name))
-    filename = ""  # make annotator happy
+    filename = "" # make annotator happy
     for i in parts_ends:
         filename = name[:i]
         if not filename:
@@ -479,29 +369,28 @@ def descr_new_zipimporter(space, w_type, w_name):
         try:
             s = os.stat(filename)
         except OSError:
-            raise oefmt(get_error(space), "Cannot find name %R", w_name)
+            raise oefmt(get_error(space), "Cannot find name %s", filename)
         if not stat.S_ISDIR(s.st_mode):
             ok = True
             break
     if not ok:
-        raise oefmt(get_error(space), "Did not find %R to be a valid zippath",
-                    w_name)
+        raise oefmt(get_error(space), "Did not find %s to be a valid zippath",
+                    name)
     try:
         w_result = zip_cache.get(filename)
         if w_result is None:
             raise oefmt(get_error(space),
-                        "Cannot import %R from zipfile, recursion detected or"
-                        "already tried and failed", w_name)
+                        "Cannot import %s from zipfile, recursion detected or"
+                        "already tried and failed", name)
     except KeyError:
         zip_cache.cache[filename] = None
     try:
         zip_file = RZipFile(filename, 'r')
     except (BadZipfile, OSError):
-        raise oefmt(get_error(space), "%R seems not to be a zipfile",
-                    space.newfilename(filename))
+        raise oefmt(get_error(space), "%s seems not to be a zipfile", filename)
     except RZlibError as e:
         # in this case, CPython raises the direct exception coming
-        # from the zlib module: let's do the same
+        # from the zlib module: let's to the same
         raise zlib_error(space, e.msg)
 
     prefix = name[len(filename):]
@@ -509,7 +398,7 @@ def descr_new_zipimporter(space, w_type, w_name):
         prefix = prefix[1:]
     if prefix and not prefix.endswith(ZIPSEP) and not prefix.endswith(os.path.sep):
         prefix += ZIPSEP
-    w_result = W_ZipImporter(space, name, filename, zip_file, prefix)
+    w_result = space.wrap(W_ZipImporter(space, name, filename, zip_file, prefix))
     zip_cache.set(filename, w_result)
     return w_result
 
@@ -523,7 +412,6 @@ W_ZipImporter.typedef = TypeDef(
     get_filename = interp2app(W_ZipImporter.get_filename),
     is_package  = interp2app(W_ZipImporter.is_package),
     load_module = interp2app(W_ZipImporter.load_module),
-    find_loader = interp2app(W_ZipImporter.find_loader),
     archive     = GetSetProperty(W_ZipImporter.getarchive),
     prefix      = GetSetProperty(W_ZipImporter.getprefix),
 )
