@@ -1,23 +1,17 @@
 from rpython.annotator.model import SomeInstance, s_None
-from rpython.annotator.listdef import s_list_of_strings
-from rpython.rlib.objectmodel import (instantiate, we_are_translated, specialize,
-    not_rpython)
-from rpython.rlib.nonconst import NonConstant
-from rpython.rlib.rarithmetic import r_uint, r_singlefloat
-from rpython.rlib.debug import make_sure_not_resized
-from rpython.rtyper.extregistry import ExtRegistryEntry
-from rpython.rtyper.lltypesystem import lltype
-from rpython.tool.sourcetools import compile2, func_with_new_name
-from rpython.translator.translator import TranslationContext
-from rpython.translator.c.genc import CStandaloneBuilder
-
-from pypy.tool.option import make_config
 from pypy.interpreter import argument, gateway
 from pypy.interpreter.baseobjspace import W_Root, ObjSpace, SpaceCache
-from pypy.interpreter.buffer import StringBuffer, SimpleView
-from pypy.interpreter.mixedmodule import MixedModule
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from pypy.objspace.std.sliceobject import W_SliceObject
+from rpython.rlib.buffer import StringBuffer
+from rpython.rlib.objectmodel import instantiate, we_are_translated, specialize
+from rpython.rlib.nonconst import NonConstant
+from rpython.rlib.rarithmetic import r_uint, r_singlefloat
+from rpython.rtyper.extregistry import ExtRegistryEntry
+from rpython.rtyper.lltypesystem import lltype
+from pypy.tool.option import make_config
+from rpython.tool.sourcetools import compile2, func_with_new_name
+from rpython.translator.translator import TranslationContext
 
 
 class W_MyObject(W_Root):
@@ -46,14 +40,14 @@ class W_MyObject(W_Root):
         is_root(w_subtype)
 
     def buffer_w(self, space, flags):
-        return SimpleView(StringBuffer("foobar"), w_obj=self)
+        return StringBuffer("foobar")
 
-    def text_w(self, space):
+    def str_w(self, space):
         return NonConstant("foobar")
-    bytes_w = text_w
+    identifier_w = bytes_w = str_w
 
-    def utf8_w(self, space):
-        return NonConstant("foobar")
+    def unicode_w(self, space):
+        return NonConstant(u"foobar")
 
     def int_w(self, space, allow_conversion=True):
         return NonConstant(-42)
@@ -63,42 +57,24 @@ class W_MyObject(W_Root):
 
     def bigint_w(self, space, allow_conversion=True):
         from rpython.rlib.rbigint import rbigint
-        x = 42
-        if we_are_translated():
-            x = NonConstant(x)
-        return rbigint.fromint(x)
+        return rbigint.fromint(NonConstant(42))
 
 class W_MyListObj(W_MyObject):
     def append(self, w_other):
         pass
 
-class W_UnicodeObject(W_MyObject):
-    _length = 21
-    _utf8 = 'foobar'
-
-    def _index_to_byte(self, at):
-        return NonConstant(42)
-
-    def _len(self):
-        return self._length
-
-    def eq_w(self, w_other):
-        return NonConstant(True)
-
-
 class W_MyType(W_MyObject):
     name = "foobar"
     flag_map_or_seq = '?'
-    hasuserdel = False
 
     def __init__(self):
         self.mro_w = [w_some_obj(), w_some_obj()]
         self.dict_w = {'__str__': w_some_obj()}
-        self.hasuserdel = True
 
     def get_module(self):
         return w_some_obj()
 
+
     def getname(self, space):
         return self.name
 
@@ -141,12 +117,9 @@ class Entry(ExtRegistryEntry):
 # ____________________________________________________________
 
 
-BUILTIN_TYPES = ['int', 'float', 'tuple', 'list', 'dict', 'bytes',
-                 'unicode', 'complex', 'slice', 'bool', 'text', 'object',
-                 'set', 'frozenset', 'bytearray', 'memoryview']
-
-INTERP_TYPES = ['function', 'builtin_function', 'module', 'getset_descriptor',
-                'instance', 'classobj']
+BUILTIN_TYPES = ['int', 'str', 'float', 'long', 'tuple', 'list', 'dict',
+                 'unicode', 'complex', 'slice', 'bool', 'basestring', 'object',
+                 'bytearray', 'buffer', 'set', 'frozenset']
 
 class FakeObjSpace(ObjSpace):
     is_fake_objspace = True
@@ -156,35 +129,9 @@ class FakeObjSpace(ObjSpace):
         ObjSpace.__init__(self, config=config)
         self.setup()
 
-        # Be sure to annotate W_SliceObject constructor.
-        # In Python2, this is triggered by W_InstanceObject.__getslice__.
-        def build_slice():
-            self.newslice(self.w_None, self.w_None, self.w_None)
-        def attach_list_strategy():
-            # this is needed for modules which interacts directly with
-            # std.listobject.W_ListObject, e.g. after an isinstance check. For
-            # example, _hpy_universal. We need to attach a couple of attributes
-            # so that the annotator annotates them with the correct types
-            from pypy.objspace.std.listobject import W_ListObject, ObjectListStrategy
-            space = self
-            w_obj = w_some_obj()
-            if isinstance(w_obj, W_ListObject):
-                w_obj.space = space
-                w_obj.strategy = ObjectListStrategy(space)
-                list_w = [w_some_obj(), w_some_obj()]
-                w_obj.lstorage = w_obj.strategy.erase(list_w)
-
-        self._seen_extras.append(build_slice)
-        self._seen_extras.append(attach_list_strategy)
-
     def _freeze_(self):
         return True
 
-    def view_as_kwargs(self, w_obj):
-        return [W_UnicodeObject()] * 3, [W_UnicodeObject()] * 3
-
-    UnicodeObjectCls = W_UnicodeObject
-
     def float_w(self, w_obj, allow_conversion=True):
         is_root(w_obj)
         return NonConstant(42.5)
@@ -193,17 +140,8 @@ class FakeObjSpace(ObjSpace):
         is_root(w_obj)
         return NonConstant(False)
 
-    def hash_w(self, w_obj):
-        return NonConstant(32)
-
-    def len_w(self, w_obj):
-        return NonConstant(37)
-
-    def utf8_len_w(self, space):
-        return NonConstant((NonConstant("utf8len_foobar"), NonConstant(14)))
-
-    @not_rpython
     def unwrap(self, w_obj):
+        "NOT_RPYTHON"
         raise NotImplementedError
 
     def newdict(self, module=False, instance=False, kwargs=False,
@@ -211,25 +149,17 @@ class FakeObjSpace(ObjSpace):
         return w_some_obj()
 
     def newtuple(self, list_w):
-        make_sure_not_resized(list_w)
         for w_x in list_w:
             is_root(w_x)
         return w_some_obj()
 
-    def newtuple2(self, w_a, w_b):
-        return w_some_obj()
-
     def newset(self, list_w=None):
         if list_w is not None:
             for w_x in list_w:
                 is_root(w_x)
         return w_some_obj()
-    newfrozenset = newset
 
     def newlist(self, list_w):
-        # make sure that the annotator thinks that the list is resized
-        list_w.append(W_Root())
-        #
         for w_x in list_w:
             is_root(w_x)
         return W_MyListObj()
@@ -241,17 +171,12 @@ class FakeObjSpace(ObjSpace):
         W_SliceObject(w_start, w_end, w_step)
         return w_some_obj()
 
-    @specialize.argtype(1)
     def newint(self, x):
         return w_some_obj()
 
     def newlong(self, x):
         return w_some_obj()
 
-    @specialize.argtype(1)
-    def newlong_from_rarith_int(self, x):
-        return w_some_obj()
-
     def newfloat(self, x):
         return w_some_obj()
 
@@ -264,44 +189,40 @@ class FakeObjSpace(ObjSpace):
     def newseqiter(self, x):
         return w_some_obj()
 
-    def newmemoryview(self, x):
+    def newbuffer(self, x):
         return w_some_obj()
 
-    @not_rpython
     def marshal_w(self, w_obj):
+        "NOT_RPYTHON"
         raise NotImplementedError
 
     def newbytes(self, x):
         return w_some_obj()
 
-    def newutf8(self, x, l):
-        return W_UnicodeObject()
+    def newunicode(self, x):
+        return w_some_obj()
 
-    def eq_w(self, obj1, obj2):
-        return NonConstant(True)
-
-    @specialize.argtype(1)
-    def newtext(self, x, lgt=-1):
-        return W_UnicodeObject()
-    newtext_or_none = newtext
-    newfilename = newtext
-
-    @not_rpython
     def wrap(self, x):
         if not we_are_translated():
-            if isinstance(x, W_Root):
-                x.spacebind(self)
+            if isinstance(x, gateway.interp2app):
+                self._see_interp2app(x)
+            if isinstance(x, GetSetProperty):
+                self._see_getsetproperty(x)
         if isinstance(x, r_singlefloat):
             self._wrap_not_rpython(x)
         if isinstance(x, list):
             if x == []: # special case: it is used e.g. in sys/__init__.py
                 return w_some_obj()
-            raise NotImplementedError
+            self._wrap_not_rpython(x)
         return w_some_obj()
+    wrap._annspecialcase_ = "specialize:argtype(1)"
+
+    def _wrap_not_rpython(self, x):
+        "NOT_RPYTHON"
+        raise NotImplementedError
 
-    @not_rpython
     def _see_interp2app(self, interp2app):
-        """Called by GatewayCache.build()"""
+        "NOT_RPYTHON"
         activation = interp2app._code.activation
         def check():
             scope_w = [w_some_obj()] * NonConstant(42)
@@ -310,9 +231,8 @@ class FakeObjSpace(ObjSpace):
         check = func_with_new_name(check, 'check__' + interp2app.name)
         self._seen_extras.append(check)
 
-    @not_rpython
     def _see_getsetproperty(self, getsetproperty):
-        """Called by GetSetProperty.spacebind()"""
+        "NOT_RPYTHON"
         space = self
         def checkprop():
             getsetproperty.fget(getsetproperty, space, w_some_obj())
@@ -362,21 +282,9 @@ class FakeObjSpace(ObjSpace):
         see_typedef(self, typedef)
         return w_some_type()
 
-    def getitem(self, w_obj, w_name):
-        is_root(w_obj)
-        is_root(w_name)
-        if isinstance(w_obj, FakeModules):
-            # For reset_lazy_initial_values,
-            # need to pretend we return a MixedModule object
-            return FakeMixedModule()
-        return w_some_type()
-
     def type(self, w_obj):
         return w_some_type()
 
-    def lookup_in_type_where(self, w_type, key):
-        return w_some_obj(), w_some_obj()
-
     def issubtype_w(self, w_sub, w_type):
         is_root(w_sub)
         is_root(w_type)
@@ -397,10 +305,10 @@ class FakeObjSpace(ObjSpace):
         is_root(w_complex)
         return 1.1, 2.2
 
-    @specialize.arg(1)
     def allocate_instance(self, cls, w_subtype):
         is_root(w_subtype)
         return instantiate(cls)
+    allocate_instance._annspecialcase_ = "specialize:arg(1)"
 
     def decode_index(self, w_index_or_slice, seqlength):
         is_root(w_index_or_slice)
@@ -422,32 +330,15 @@ class FakeObjSpace(ObjSpace):
     def unicode_from_object(self, w_obj):
         return w_some_obj()
 
-    def encode_unicode_object(self, w_unicode, encoding, errors):
-        return w_some_obj()
-
     def _try_fetch_pycode(self, w_func):
         return None
 
     def is_generator(self, w_obj):
         return NonConstant(False)
 
-    def is_iterable(self, w_obj):
-        return NonConstant(False)
-
-    def lookup_in_type(self, w_type, name):
-        return w_some_obj()
-
-    def warn(self, w_msg, w_warningcls, stacklevel=2):
-        pass
-
-    def _try_buffer_w(self, w_obj, flags):
-        return w_obj.buffer_w(self, flags)
-
     # ----------
 
-    def translates(self, func=None, argtypes=None, seeobj_w=[],
-                   extra_func=None, c_compile=False,
-                   **kwds):
+    def translates(self, func=None, argtypes=None, seeobj_w=[], **kwds):
         config = make_config(None, **kwds)
         if func is not None:
             if argtypes is None:
@@ -457,14 +348,9 @@ class FakeObjSpace(ObjSpace):
         t = TranslationContext(config=config)
         self.t = t     # for debugging
         ann = t.buildannotator()
-
-        def entry_point(argv):
+        def _do_startup():
             self.threadlocals.enter_thread(self)
-            W_SliceObject(w_some_obj(), w_some_obj(), w_some_obj())
-            if extra_func:
-                extra_func(self)
-            return 0
-        ann.build_types(entry_point, [s_list_of_strings], complete_now=False)
+        ann.build_types(_do_startup, [], complete_now=False)
         if func is not None:
             ann.build_types(func, argtypes, complete_now=False)
         if seeobj_w:
@@ -486,37 +372,16 @@ class FakeObjSpace(ObjSpace):
         #t.viewcg()
         t.buildrtyper().specialize()
         t.checkgraphs()
-        from rpython.translator.backendopt.all import backend_optimizations
-        backend_optimizations(t, replace_we_are_jitted=True)
-        if c_compile:
-            cbuilder = CStandaloneBuilder(t, entry_point, t.config)
-            cbuilder.generate_source(defines=cbuilder.DEBUG_DEFINES)
-            cbuilder.compile()
-            return t, cbuilder
-
 
     def setup(space):
-        from pypy.module.exceptions import interp_exceptions
-        obj_space_exceptions = ObjSpace.ExceptionTable
-        # Add subclasses of the ExceptionTable errors
-        for name, exc in interp_exceptions.__dict__.items():
-            if (isinstance(exc, type) and
-                issubclass(exc, interp_exceptions.W_BaseException)):
-                name = name.replace("W_", "")
-                if name not in obj_space_exceptions:
-                    obj_space_exceptions.append(name)
-
         for name in (ObjSpace.ConstantTable +
-                     obj_space_exceptions +
+                     ObjSpace.ExceptionTable +
                      BUILTIN_TYPES):
-            if name != "str":
-                setattr(space, 'w_' + name, w_some_obj())
-        space.w_bytes = w_some_obj()
-        space.w_text = w_some_obj()
+            setattr(space, 'w_' + name, w_some_obj())
         space.w_type = w_some_type()
         #
         for (name, _, arity, _) in ObjSpace.MethodTable:
-            if name in ('type', 'getitem'):
+            if name == 'type':
                 continue
             args = ['w_%d' % i for i in range(arity)]
             params = args[:]
@@ -542,14 +407,12 @@ class FakeObjSpace(ObjSpace):
 @specialize.memo()
 def see_typedef(space, typedef):
     assert isinstance(typedef, TypeDef)
-    if typedef.name not in BUILTIN_TYPES and typedef.name not in INTERP_TYPES:
-        print
-        print '------ seeing typedef %r ------' % (typedef.name,)
+    if typedef.name not in BUILTIN_TYPES:
         for name, value in typedef.rawdict.items():
             space.wrap(value)
 
 class FakeCompiler(object):
-    def compile(self, code, name, mode, flags, optimize=-1):
+    def compile(self, code, name, mode, flags):
         return FakePyCode()
 FakeObjSpace.default_compiler = FakeCompiler()
 
@@ -557,23 +420,16 @@ class FakePyCode(W_Root):
     def exec_code(self, space, w_globals, w_locals):
         return W_Root()
 
-class FakeMixedModule(MixedModule):
-    def __init__(self):
-        pass
-
-class FakeModules(W_Root):
-    pass
 
 class FakeModule(W_Root):
     def __init__(self):
         self.w_dict = w_some_obj()
     def get(self, name):
         name + "xx"   # check that it's a string
-        return FakeModules()
-    def setmodule(self, w_mod):
-        is_root(w_mod)
+        return w_some_obj()
 FakeObjSpace.sys = FakeModule()
 FakeObjSpace.sys.filesystemencoding = 'foobar'
 FakeObjSpace.sys.defaultencoding = 'ascii'
 FakeObjSpace.sys.dlopenflags = 123
+FakeObjSpace.sys.track_resources = False
 FakeObjSpace.builtin = FakeModule()
