@@ -4,19 +4,15 @@ from rpython.rlib import jit, rcomplex
 from rpython.rlib.rarithmetic import intmask, r_ulonglong
 from rpython.rlib.rbigint import rbigint
 from rpython.rlib.rfloat import (
-    DTSF_STR_PRECISION, formatd, string_to_float)
+    DTSF_STR_PRECISION, copysign, formatd, isinf, isnan, string_to_float)
 from rpython.rlib.rstring import ParseStringError
-from rpython.tool.sourcetools import func_with_new_name
 
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import WrappedDefault, interp2app, unwrap_spec
 from pypy.interpreter.typedef import GetSetProperty, TypeDef
 from pypy.objspace.std import newformat
-from pypy.objspace.std.floatobject import _hash_float, _remove_underscores
-from pypy.objspace.std.unicodeobject import unicode_to_decimal_w
-
-HASH_IMAG = 1000003
+from pypy.objspace.std.floatobject import _hash_float
 
 
 def _split_complex(s):
@@ -115,12 +111,12 @@ def _split_complex(s):
 
 def format_float(x, code, precision):
     # like float2string, except that the ".0" is not necessary
-    if math.isinf(x):
+    if isinf(x):
         if x > 0.0:
             return "inf"
         else:
             return "-inf"
-    elif math.isnan(x):
+    elif isnan(x):
         return "nan"
     else:
         return formatd(x, code, precision)
@@ -132,7 +128,7 @@ def str_format(x):
     return format_float(x, 'g', DTSF_STR_PRECISION)
 
 
-def unpackcomplex(space, w_complex, strict_typing=True, firstarg=True):
+def unpackcomplex(space, w_complex, strict_typing=True):
     """
     convert w_complex into a complex and return the unwrapped (real, imag)
     tuple. If strict_typing==True, we also typecheck the value returned by
@@ -143,57 +139,48 @@ def unpackcomplex(space, w_complex, strict_typing=True, firstarg=True):
         return (w_complex.realval, w_complex.imagval)
     #
     # test for a '__complex__' method, and call it if found.
+    # special case old-style instances, like CPython does.
     w_z = None
-    w_method = space.lookup(w_complex, '__complex__')
-    if w_method is not None:
-        w_z = space.get_and_call_function(w_method, w_complex)
+    if space.is_oldstyle_instance(w_complex):
+        try:
+            w_method = space.getattr(w_complex, space.wrap('__complex__'))
+        except OperationError as e:
+            if not e.match(space, space.w_AttributeError):
+                raise
+        else:
+            w_z = space.call_function(w_method)
+    else:
+        w_method = space.lookup(w_complex, '__complex__')
+        if w_method is not None:
+            w_z = space.get_and_call_function(w_method, w_complex)
     #
     if w_z is not None:
-        # __complex__() must return a complex
+        # __complex__() must return a complex or (float,int,long) object
         # (XXX should not use isinstance here)
-        if isinstance(w_z, W_ComplexObject):
-            if type(w_z) is not W_ComplexObject:
-                space.warn(
-                    space.newtext("__complex__ returned non-complex (type %s).  The ability to return an instance of a strict subclass of complex is deprecated, and may be removed in a future version of Python." % (space.type(w_z).getname(space))),
-                    space.w_DeprecationWarning
-                )
+        if not strict_typing and (space.isinstance_w(w_z, space.w_int) or
+                                  space.isinstance_w(w_z, space.w_long) or
+                                  space.isinstance_w(w_z, space.w_float)):
+            return (space.float_w(w_z), 0.0)
+        elif isinstance(w_z, W_ComplexObject):
             return (w_z.realval, w_z.imagval)
         raise oefmt(space.w_TypeError,
                     "__complex__() must return a complex number")
 
-    # try to see whether it has an __index__
-    if space.lookup(w_complex, '__index__') is not None:
-        result = space.float_w(space.float(space.index(w_complex)))
-        return (result, 0.0)
-
     #
     # no '__complex__' method, so we assume it is a float,
     # unless it is an instance of some subclass of complex.
     if space.isinstance_w(w_complex, space.gettypefor(W_ComplexObject)):
-        real = space.float(space.getattr(w_complex, space.newtext("real")))
-        imag = space.float(space.getattr(w_complex, space.newtext("imag")))
+        real = space.float(space.getattr(w_complex, space.wrap("real")))
+        imag = space.float(space.getattr(w_complex, space.wrap("imag")))
         return (space.float_w(real), space.float_w(imag))
     #
     # Check that it is not a string (on which space.float() would succeed).
-    if (space.isinstance_w(w_complex, space.w_bytes) or
+    if (space.isinstance_w(w_complex, space.w_str) or
         space.isinstance_w(w_complex, space.w_unicode)):
         raise oefmt(space.w_TypeError,
                     "complex number expected, got '%T'", w_complex)
     #
-    try:
-        return (space.float_w(space.float(w_complex)), 0.0)
-    except OperationError as e:
-        if not e.match(space, space.w_TypeError):
-            raise
-        if firstarg:
-            raise oefmt(space.w_TypeError,
-                        "complex() first argument must be a string or a number, not '%T'",
-                         w_complex)
-        else:
-            raise oefmt(space.w_TypeError,
-                        "complex() second argument must be a number, not '%T'",
-                         w_complex)
-
+    return (space.float_w(space.float(w_complex)), 0.0)
 
 
 class W_ComplexObject(W_Root):
@@ -226,6 +213,15 @@ class W_ComplexObject(W_Root):
         rr, ir = rcomplex.c_div(self.as_tuple(), other.as_tuple())
         return W_ComplexObject(rr, ir)
 
+    def divmod(self, space, other):
+        space.warn(space.wrap("complex divmod(), // and % are deprecated"),
+                   space.w_DeprecationWarning)
+        w_div = self.div(other)
+        div = math.floor(w_div.realval)
+        w_mod = self.sub(
+            W_ComplexObject(other.realval * div, other.imagval * div))
+        return (W_ComplexObject(div, 0), w_mod)
+
     def pow(self, other):
         rr, ir = rcomplex.c_pow(self.as_tuple(), other.as_tuple())
         return W_ComplexObject(rr, ir)
@@ -255,10 +251,10 @@ class W_ComplexObject(W_Root):
             return False
         if self.user_overridden_class or w_other.user_overridden_class:
             return self is w_other
-        real1 = space.float_w(space.getattr(self, space.newtext("real")))
-        real2 = space.float_w(space.getattr(w_other, space.newtext("real")))
-        imag1 = space.float_w(space.getattr(self, space.newtext("imag")))
-        imag2 = space.float_w(space.getattr(w_other, space.newtext("imag")))
+        real1 = space.float_w(space.getattr(self, space.wrap("real")))
+        real2 = space.float_w(space.getattr(w_other, space.wrap("real")))
+        imag1 = space.float_w(space.getattr(self, space.wrap("imag")))
+        imag2 = space.float_w(space.getattr(w_other, space.wrap("imag")))
         real1 = float2longlong(real1)
         real2 = float2longlong(real2)
         imag1 = float2longlong(imag1)
@@ -271,22 +267,23 @@ class W_ComplexObject(W_Root):
         from rpython.rlib.longlong2float import float2longlong
         from pypy.objspace.std.util import IDTAG_COMPLEX as tag
         from pypy.objspace.std.util import IDTAG_SHIFT
-        real = space.float_w(space.getattr(self, space.newtext("real")))
-        imag = space.float_w(space.getattr(self, space.newtext("imag")))
+        real = space.float_w(space.getattr(self, space.wrap("real")))
+        imag = space.float_w(space.getattr(self, space.wrap("imag")))
         real_b = rbigint.fromrarith_int(float2longlong(real))
         imag_b = rbigint.fromrarith_int(r_ulonglong(float2longlong(imag)))
         val = real_b.lshift(64).or_(imag_b).lshift(IDTAG_SHIFT).int_or_(tag)
         return space.newlong_from_rbigint(val)
 
-    def descr_int(self, space):
+    def int(self, space):
         raise oefmt(space.w_TypeError, "can't convert complex to int")
 
     def _to_complex(self, space, w_obj):
         if isinstance(w_obj, W_ComplexObject):
             return w_obj
         if space.isinstance_w(w_obj, space.w_int):
-            w_float = space.float_w(w_obj)
-            return W_ComplexObject(w_float, 0.0)
+            return W_ComplexObject(float(space.int_w(w_obj)), 0.0)
+        if space.isinstance_w(w_obj, space.w_long):
+            return W_ComplexObject(space.float_w(w_obj), 0.0)
         if space.isinstance_w(w_obj, space.w_float):
             return W_ComplexObject(space.float_w(w_obj), 0.0)
 
@@ -302,19 +299,14 @@ class W_ComplexObject(W_Root):
             and space.is_w(space.type(w_real), space.w_complex)):
             return w_real
 
-        if space.isinstance_w(w_real, space.w_text):
+        if space.isinstance_w(w_real, space.w_str) or \
+                space.isinstance_w(w_real, space.w_unicode):
             # a string argument
             if not noarg2:
                 raise oefmt(space.w_TypeError, "complex() can't take second"
                                                " arg if first is a string")
-            unistr = unicode_to_decimal_w(space, w_real)
             try:
-                unistr = _remove_underscores(unistr)
-            except ValueError:
-                raise oefmt(space.w_ValueError,
-                            "complex() arg is a malformed string")
-            try:
-                realstr, imagstr = _split_complex(unistr)
+                realstr, imagstr = _split_complex(space.str_w(w_real))
             except ValueError:
                 raise oefmt(space.w_ValueError,
                             "complex() arg is a malformed string")
@@ -327,13 +319,14 @@ class W_ComplexObject(W_Root):
 
         else:
             # non-string arguments
-            realval, imagval = unpackcomplex(space, w_real)
+            realval, imagval = unpackcomplex(space, w_real,
+                                             strict_typing=False)
 
             # now take w_imag into account
             if not noarg2:
                 # complex(x, y) == x+y*j, even if 'y' is already a complex.
                 realval2, imagval2 = unpackcomplex(space, w_imag,
-                                                   firstarg=False)
+                                                   strict_typing=False)
 
                 # try to preserve the signs of zeroes of realval and realval2
                 if imagval2 != 0.0:
@@ -349,29 +342,29 @@ class W_ComplexObject(W_Root):
         return w_obj
 
     def descr___getnewargs__(self, space):
-        return space.newtuple2(space.newfloat(self.realval),
-                               space.newfloat(self.imagval))
+        return space.newtuple([space.newfloat(self.realval),
+                               space.newfloat(self.imagval)])
 
     def descr_repr(self, space):
-        if self.realval == 0 and math.copysign(1., self.realval) == 1.:
-            return space.newtext(repr_format(self.imagval) + 'j')
-        sign = (math.copysign(1., self.imagval) == 1. or
-                math.isnan(self.imagval)) and '+' or ''
-        return space.newtext('(' + repr_format(self.realval)
-                             + sign + repr_format(self.imagval) + 'j)')
+        if self.realval == 0 and copysign(1., self.realval) == 1.:
+            return space.wrap(repr_format(self.imagval) + 'j')
+        sign = (copysign(1., self.imagval) == 1. or
+                isnan(self.imagval)) and '+' or ''
+        return space.wrap('(' + repr_format(self.realval)
+                          + sign + repr_format(self.imagval) + 'j)')
 
     def descr_str(self, space):
-        if self.realval == 0 and math.copysign(1., self.realval) == 1.:
-            return space.newtext(str_format(self.imagval) + 'j')
-        sign = (math.copysign(1., self.imagval) == 1. or
-                math.isnan(self.imagval)) and '+' or ''
-        return space.newtext('(' + str_format(self.realval)
-                             + sign + str_format(self.imagval) + 'j)')
+        if self.realval == 0 and copysign(1., self.realval) == 1.:
+            return space.wrap(str_format(self.imagval) + 'j')
+        sign = (copysign(1., self.imagval) == 1. or
+                isnan(self.imagval)) and '+' or ''
+        return space.wrap('(' + str_format(self.realval)
+                          + sign + str_format(self.imagval) + 'j)')
 
     def descr_hash(self, space):
-        hashreal = _hash_float(self.realval)
-        hashimg = _hash_float(self.imagval)   # 0 if self.imagval == 0
-        h = intmask(hashreal + HASH_IMAG * hashimg)
+        hashreal = _hash_float(space, self.realval)
+        hashimg = _hash_float(space, self.imagval)   # 0 if self.imagval == 0
+        h = intmask(hashreal + 1000003 * hashimg)
         h -= (h == -1)
         return space.newint(h)
 
@@ -379,13 +372,13 @@ class W_ComplexObject(W_Root):
         w_other = self._to_complex(space, w_other)
         if w_other is None:
             return space.w_NotImplemented
-        return space.newtuple2(self, w_other)
+        return space.newtuple([self, w_other])
 
     def descr_format(self, space, w_format_spec):
         return newformat.run_formatter(space, w_format_spec, "format_complex",
                                        self)
 
-    def descr_bool(self, space):
+    def descr_nonzero(self, space):
         return space.newbool((self.realval != 0.0) or (self.imagval != 0.0))
 
     def descr_float(self, space):
@@ -401,13 +394,14 @@ class W_ComplexObject(W_Root):
         try:
             return space.newfloat(math.hypot(self.realval, self.imagval))
         except OverflowError as e:
-            raise OperationError(space.w_OverflowError, space.newtext(str(e)))
+            raise OperationError(space.w_OverflowError, space.wrap(str(e)))
 
     def descr_eq(self, space, w_other):
         if isinstance(w_other, W_ComplexObject):
             return space.newbool((self.realval == w_other.realval) and
                                  (self.imagval == w_other.imagval))
         if (space.isinstance_w(w_other, space.w_int) or
+            space.isinstance_w(w_other, space.w_long) or
             space.isinstance_w(w_other, space.w_float)):
             if self.imagval:
                 return space.w_False
@@ -419,6 +413,7 @@ class W_ComplexObject(W_Root):
             return space.newbool((self.realval != w_other.realval) or
                                  (self.imagval != w_other.imagval))
         if (space.isinstance_w(w_other, space.w_int) or
+            space.isinstance_w(w_other, space.w_long) or
             space.isinstance_w(w_other, space.w_float)):
             if self.imagval:
                 return space.w_True
@@ -426,6 +421,12 @@ class W_ComplexObject(W_Root):
         return space.w_NotImplemented
 
     def _fail_cmp(self, space, w_other):
+        if (isinstance(w_other, W_ComplexObject) or
+            space.isinstance_w(w_other, space.w_int) or
+            space.isinstance_w(w_other, space.w_long) or
+            space.isinstance_w(w_other, space.w_float)):
+            raise oefmt(space.w_TypeError,
+                        "no ordering relation is defined for complex numbers")
         return space.w_NotImplemented
 
     def descr_add(self, space, w_rhs):
@@ -475,7 +476,7 @@ class W_ComplexObject(W_Root):
         try:
             return self.div(w_rhs)
         except ZeroDivisionError as e:
-            raise oefmt(space.w_ZeroDivisionError, "complex division by zero")
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
 
     def descr_rtruediv(self, space, w_lhs):
         w_lhs = self._to_complex(space, w_lhs)
@@ -484,20 +485,65 @@ class W_ComplexObject(W_Root):
         try:
             return w_lhs.div(self)
         except ZeroDivisionError as e:
-            raise oefmt(space.w_ZeroDivisionError, "complex division by zero")
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
 
     def descr_floordiv(self, space, w_rhs):
-        raise oefmt(space.w_TypeError, "can't take floor of complex number.")
-    descr_rfloordiv = func_with_new_name(descr_floordiv, 'descr_rfloordiv')
+        w_rhs = self._to_complex(space, w_rhs)
+        if w_rhs is None:
+            return space.w_NotImplemented
+        # don't care about the slight slowdown you get from using divmod
+        try:
+            return self.divmod(space, w_rhs)[0]
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+
+    def descr_rfloordiv(self, space, w_lhs):
+        w_lhs = self._to_complex(space, w_lhs)
+        if w_lhs is None:
+            return space.w_NotImplemented
+        # don't care about the slight slowdown you get from using divmod
+        try:
+            return w_lhs.divmod(space, self)[0]
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
 
     def descr_mod(self, space, w_rhs):
-        raise oefmt(space.w_TypeError, "can't mod complex numbers.")
-    descr_rmod = func_with_new_name(descr_mod, 'descr_rmod')
+        w_rhs = self._to_complex(space, w_rhs)
+        if w_rhs is None:
+            return space.w_NotImplemented
+        try:
+            return self.divmod(space, w_rhs)[1]
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+
+    def descr_rmod(self, space, w_lhs):
+        w_lhs = self._to_complex(space, w_lhs)
+        if w_lhs is None:
+            return space.w_NotImplemented
+        try:
+            return w_lhs.divmod(space, self)[1]
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
 
     def descr_divmod(self, space, w_rhs):
-        raise oefmt(space.w_TypeError,
-                    "can't take floor or mod of complex number.")
-    descr_rdivmod = func_with_new_name(descr_divmod, 'descr_rdivmod')
+        w_rhs = self._to_complex(space, w_rhs)
+        if w_rhs is None:
+            return space.w_NotImplemented
+        try:
+            div, mod = self.divmod(space, w_rhs)
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+        return space.newtuple([div, mod])
+
+    def descr_rdivmod(self, space, w_lhs):
+        w_lhs = self._to_complex(space, w_lhs)
+        if w_lhs is None:
+            return space.w_NotImplemented
+        try:
+            div, mod = w_lhs.divmod(space, self)
+        except ZeroDivisionError as e:
+            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+        return space.newtuple([div, mod])
 
     @unwrap_spec(w_third_arg=WrappedDefault(None))
     def descr_pow(self, space, w_exponent, w_third_arg):
@@ -535,12 +581,12 @@ class W_ComplexObject(W_Root):
 w_one = W_ComplexObject(1, 0)
 
 
-def complexwprop(name, doc):
+def complexwprop(name):
     def fget(space, w_obj):
         if not isinstance(w_obj, W_ComplexObject):
             raise oefmt(space.w_TypeError, "descriptor is for 'complex'")
         return space.newfloat(getattr(w_obj, name))
-    return GetSetProperty(fget, doc=doc, cls=W_ComplexObject)
+    return GetSetProperty(fget)
 
 W_ComplexObject.typedef = TypeDef("complex",
     __doc__ = """complex(real[, imag]) -> complex number
@@ -549,15 +595,15 @@ Create a complex number from a real part and an optional imaginary part.
 This is equivalent to (real + imag*1j) where imag defaults to 0.""",
     __new__ = interp2app(W_ComplexObject.descr__new__),
     __getnewargs__ = interp2app(W_ComplexObject.descr___getnewargs__),
-    real = complexwprop('realval', doc="the real part of a complex number"),
-    imag = complexwprop('imagval',
-                        doc="the imaginary part of a complex number"),
+    real = complexwprop('realval'),
+    imag = complexwprop('imagval'),
     __repr__ = interp2app(W_ComplexObject.descr_repr),
     __str__ = interp2app(W_ComplexObject.descr_str),
     __hash__ = interp2app(W_ComplexObject.descr_hash),
+    __coerce__ = interp2app(W_ComplexObject.descr_coerce),
     __format__ = interp2app(W_ComplexObject.descr_format),
-    __bool__ = interp2app(W_ComplexObject.descr_bool),
-    __int__ = interp2app(W_ComplexObject.descr_int),
+    __nonzero__ = interp2app(W_ComplexObject.descr_nonzero),
+    __int__ = interp2app(W_ComplexObject.int),
     __float__ = interp2app(W_ComplexObject.descr_float),
     __neg__ = interp2app(W_ComplexObject.descr_neg),
     __pos__ = interp2app(W_ComplexObject.descr_pos),
@@ -576,6 +622,8 @@ This is equivalent to (real + imag*1j) where imag defaults to 0.""",
     __rsub__ = interp2app(W_ComplexObject.descr_rsub),
     __mul__ = interp2app(W_ComplexObject.descr_mul),
     __rmul__ = interp2app(W_ComplexObject.descr_rmul),
+    __div__ = interp2app(W_ComplexObject.descr_truediv),
+    __rdiv__ = interp2app(W_ComplexObject.descr_rtruediv),
     __truediv__ = interp2app(W_ComplexObject.descr_truediv),
     __rtruediv__ = interp2app(W_ComplexObject.descr_rtruediv),
     __floordiv__ = interp2app(W_ComplexObject.descr_floordiv),
