@@ -2,13 +2,13 @@ import math
 import operator
 import sys
 
-from rpython.rlib import rarithmetic, rfloat, jit
+from rpython.rlib import rarithmetic, rfloat
 from rpython.rlib.rarithmetic import LONG_BIT, intmask, ovfcheck_float_to_int
 from rpython.rlib.rarithmetic import int_between
 from rpython.rlib.rbigint import rbigint
 from rpython.rlib.rfloat import (
-    DTSF_ADD_DOT_0, INFINITY, NAN,
-    float_as_rbigint_ratio, formatd, isfinite)
+    DTSF_ADD_DOT_0, DTSF_STR_PRECISION, INFINITY, NAN, copysign,
+    float_as_rbigint_ratio, formatd, isfinite, isinf, isnan)
 from rpython.rlib.rstring import ParseStringError
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rtyper.lltypesystem.module.ll_math import math_fmod
@@ -19,25 +19,15 @@ from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import WrappedDefault, interp2app, unwrap_spec
 from pypy.interpreter.typedef import GetSetProperty, TypeDef
 from pypy.objspace.std import newformat
-from pypy.objspace.std.intobject import HASH_BITS, HASH_MODULUS, W_IntObject
-from pypy.objspace.std.longobject import (
-    W_AbstractLongObject, newlong_from_float)
-from rpython.rlib.rarithmetic import (
-    LONG_BIT, intmask, ovfcheck_float_to_int, r_uint)
+from pypy.objspace.std.longobject import W_LongObject
 from pypy.objspace.std.util import wrap_parsestringerror
 
-HASH_INF  = 314159
-HASH_NAN  = 0
-
-# Here 0.30103 is an upper bound for log10(2)
-NDIGITS_MAX = int((rfloat.DBL_MANT_DIG - rfloat.DBL_MIN_EXP) * 0.30103)
-NDIGITS_MIN = -int((rfloat.DBL_MAX_EXP + 1) * 0.30103)
 
 def float2string(x, code, precision):
     # we special-case explicitly inf and nan here
     if isfinite(x):
         s = formatd(x, code, precision, DTSF_ADD_DOT_0)
-    elif math.isinf(x):
+    elif isinf(x):
         if x > 0.0:
             s = "inf"
         else:
@@ -46,9 +36,6 @@ def float2string(x, code, precision):
         s = "nan"
     return s
 
-def float_repr(x):
-    return float2string(x, 'r', 0)
-
 
 def detect_floatformat():
     from rpython.rtyper.lltypesystem import rffi, lltype
@@ -132,7 +119,7 @@ def make_compare_func(opname):
     def _compare(self, space, w_other):
         if isinstance(w_other, W_FloatObject):
             return space.newbool(op(self.floatval, w_other.floatval))
-        if isinstance(w_other, W_IntObject):
+        if space.isinstance_w(w_other, space.w_int):
             f1 = self.floatval
             i2 = space.int_w(w_other)
             # (double-)floats have always at least 48 bits of precision
@@ -142,21 +129,12 @@ def make_compare_func(opname):
                 f2 = float(i2)
                 res = op(f1, f2)
             return space.newbool(res)
-        if isinstance(w_other, W_AbstractLongObject):
+        if space.isinstance_w(w_other, space.w_long):
             return space.newbool(do_compare_bigint(self.floatval,
                                                    space.bigint_w(w_other)))
         return space.w_NotImplemented
     return func_with_new_name(_compare, 'descr_' + opname)
 
-def newint_from_float(space, floatval):
-    """This is also used from module/math/interp_math.py"""
-    try:
-        value = ovfcheck_float_to_int(floatval)
-    except OverflowError:
-        return newlong_from_float(space, floatval)
-    else:
-        return space.newint(value)
-
 
 class W_FloatObject(W_Root):
     """This is a implementation of the app-level 'float' type.
@@ -214,25 +192,19 @@ class W_FloatObject(W_Root):
 
     @staticmethod
     @unwrap_spec(w_x=WrappedDefault(0.0))
-    def descr__new__(space, w_floattype, w_x, __posonly__):
+    def descr__new__(space, w_floattype, w_x):
+        def _string_to_float(space, w_source, string):
+            try:
+                return rfloat.string_to_float(string)
+            except ParseStringError as e:
+                raise wrap_parsestringerror(space, e, w_source)
+
         w_value = w_x     # 'x' is the keyword argument name in CPython
         if space.lookup(w_value, "__float__") is not None:
             w_obj = space.float(w_value)
-            w_obj_type = space.type(w_obj)
-            if not space.is_w(w_obj_type, space.w_float):
-                space.warn(space.newtext(
-                    "%s.__float__ returned non-float (type %s).  "
-                    "The ability to return an instance of a strict subclass "
-                    "of float is deprecated, and may be removed "
-                    "in a future version of Python." %
-                    (space.type(w_value).name, w_obj_type.name)),
-                    space.w_DeprecationWarning)
-            elif space.is_w(w_floattype, space.w_float):
+            if space.is_w(w_floattype, space.w_float):
                 return w_obj
             value = space.float_w(w_obj)
-        elif space.lookup(w_value, "__index__") is not None:
-            w_obj = space.index(w_value)
-            return space.float(w_obj)
         elif space.isinstance_w(w_value, space.w_unicode):
             from unicodeobject import unicode_to_decimal_w
             value = _string_to_float(space, w_value,
@@ -242,9 +214,9 @@ class W_FloatObject(W_Root):
                 value = space.charbuf_w(w_value)
             except OperationError as e:
                 if e.match(space, space.w_TypeError):
-                    raise oefmt(space.w_TypeError,
-                                "float() argument must be a string or a "
-                                "number, not '%T'", w_value)
+                    raise oefmt(
+                        space.w_TypeError,
+                        "float() argument must be a string or a number")
                 raise
             value = _string_to_float(space, w_value, value)
         w_obj = space.allocate_instance(W_FloatObject, w_floattype)
@@ -252,26 +224,17 @@ class W_FloatObject(W_Root):
         return w_obj
 
     @staticmethod
-    @unwrap_spec(kind='text')
+    @unwrap_spec(kind=str)
     def descr___getformat__(space, w_cls, kind):
         if kind == "float":
-            return space.newtext(_float_format)
+            return space.wrap(_float_format)
         elif kind == "double":
-            return space.newtext(_double_format)
+            return space.wrap(_double_format)
         raise oefmt(space.w_ValueError, "only float and double are valid")
 
     @staticmethod
-    @unwrap_spec(s='text')
+    @unwrap_spec(s=str)
     def descr_fromhex(space, w_cls, s):
-        """float.fromhex(string) -> float
-
-        Create a floating-point number from a hexadecimal string.
-        >>> float.fromhex('0x1.ffffp10')
-        2047.984375
-        >>> float.fromhex('-0x1p-1074')
-        -5e-324
-
-        """
         length = len(s)
         i = 0
         value = 0.0
@@ -390,7 +353,7 @@ class W_FloatObject(W_Root):
                         if digit & half_eps:
                             round_up = False
                             if (digit & (3 * half_eps - 1) or
-                                (half_eps == 8 and key_digit + 1 < float_digits and
+                                (half_eps == 8 and
                                  _hex_digit(s, key_digit + 1, co_end, float_digits) & 1)):
                                 round_up = True
                             else:
@@ -409,31 +372,38 @@ class W_FloatObject(W_Root):
             i += 1
         if i != length:
             raise oefmt(space.w_ValueError, "invalid hex string")
-        w_float = space.newfloat(sign * value)
+        w_float = space.wrap(sign * value)
         return space.call_function(w_cls, w_float)
 
     def _to_float(self, space, w_obj):
         if isinstance(w_obj, W_FloatObject):
             return w_obj
         if space.isinstance_w(w_obj, space.w_int):
+            return W_FloatObject(float(space.int_w(w_obj)))
+        if space.isinstance_w(w_obj, space.w_long):
             return W_FloatObject(space.float_w(w_obj))
 
-    def descr___round__(self, space, w_ndigits=None):
-        return _round_float(space, self, w_ndigits)
-
     def descr_repr(self, space):
-        res = float_repr(self.floatval)
-        return space.newutf8(res, len(res)) # always ascii
-    descr_str = func_with_new_name(descr_repr, 'descr_str')
+        return space.wrap(float2string(self.floatval, 'r', 0))
+
+    def descr_str(self, space):
+        return space.wrap(float2string(self.floatval, 'g', DTSF_STR_PRECISION))
 
     def descr_hash(self, space):
-        h = _hash_float(self.floatval)
-        return space.newint(h)
+        h = _hash_float(space, self.floatval)
+        h -= (h == -1)
+        return space.wrap(h)
 
     def descr_format(self, space, w_spec):
         return newformat.run_formatter(space, w_spec, "format_float", self)
 
-    def descr_bool(self, space):
+    def descr_coerce(self, space, w_other):
+        w_other = self._to_float(space, w_other)
+        if w_other is None:
+            return space.w_NotImplemented
+        return space.newtuple([self, w_other])
+
+    def descr_nonzero(self, space):
         return space.newbool(self.floatval != 0.0)
 
     def descr_float(self, space):
@@ -442,14 +412,23 @@ class W_FloatObject(W_Root):
         a = self.floatval
         return W_FloatObject(a)
 
+    def descr_long(self, space):
+        try:
+            return W_LongObject.fromfloat(space, self.floatval)
+        except OverflowError:
+            raise oefmt(space.w_OverflowError,
+                        "cannot convert float infinity to integer")
+        except ValueError:
+            raise oefmt(space.w_ValueError,
+                        "cannot convert float NaN to integer")
+
     def descr_trunc(self, space):
-        return newint_from_float(space, self.floatval)
-
-    def descr_floor(self, space):
-        return newint_from_float(space, math.floor(self.floatval))
-
-    def descr_ceil(self, space):
-        return newint_from_float(space, math.ceil(self.floatval))
+        try:
+            value = ovfcheck_float_to_int(self.floatval)
+        except OverflowError:
+            return self.descr_long(space)
+        else:
+            return space.newint(value)
 
     def descr_neg(self, space):
         return W_FloatObject(-self.floatval)
@@ -512,7 +491,7 @@ class W_FloatObject(W_Root):
             return space.w_NotImplemented
         rhs = w_rhs.floatval
         if rhs == 0.0:
-            raise oefmt(space.w_ZeroDivisionError, "float division by zero")
+            raise oefmt(space.w_ZeroDivisionError, "float division")
         return W_FloatObject(self.floatval / rhs)
 
     def descr_rdiv(self, space, w_lhs):
@@ -521,7 +500,7 @@ class W_FloatObject(W_Root):
             return space.w_NotImplemented
         selfval = self.floatval
         if selfval == 0.0:
-            raise oefmt(space.w_ZeroDivisionError, "float division by zero")
+            raise oefmt(space.w_ZeroDivisionError, "float division")
         return W_FloatObject(w_lhs.floatval / selfval)
 
     def descr_floordiv(self, space, w_rhs):
@@ -554,7 +533,7 @@ class W_FloatObject(W_Root):
             # fmod returns different results across platforms; ensure
             # it has the same sign as the denominator; we'd like to do
             # "mod = y * 0.0", but that may get optimized away
-            mod = math.copysign(0.0, y)
+            mod = copysign(0.0, y)
 
         return W_FloatObject(mod)
 
@@ -590,10 +569,8 @@ class W_FloatObject(W_Root):
         try:
             result = _pow(space, x, y)
         except PowDomainError:
-            # Negative numbers raised to fractional powers become complex
-            return space.pow(space.newcomplex(x, 0.0),
-                             space.newcomplex(y, 0.0),
-                             w_third_arg)
+            raise oefmt(space.w_ValueError, "negative number cannot be raised "
+                                            "to a fractional power")
         return W_FloatObject(result)
 
     @unwrap_spec(w_third_arg=WrappedDefault(None))
@@ -607,7 +584,7 @@ class W_FloatObject(W_Root):
         return space.float(self)
 
     def descr_get_imag(self, space):
-        return space.newfloat(0.0)
+        return space.wrap(0.0)
 
     def descr_conjugate(self, space):
         return space.float(self)
@@ -616,59 +593,34 @@ class W_FloatObject(W_Root):
         v = self.floatval
         if not rfloat.isfinite(v):
             return space.w_False
-        return space.newbool(math.floor(v) == v)
+        return space.wrap(math.floor(v) == v)
 
     def descr_as_integer_ratio(self, space):
-        """float.as_integer_ratio() -> (int, int)
-
-        Return a pair of integers, whose ratio is exactly equal to the
-        original float and with a positive denominator.  Raise
-        OverflowError on infinities and a ValueError on NaNs.
-
-        >>> (10.0).as_integer_ratio()
-        (10, 1)
-        >>> (0.0).as_integer_ratio()
-        (0, 1)
-        >>> (-.25).as_integer_ratio()
-        (-1, 4)
-
-        """
         value = self.floatval
         try:
             num, den = float_as_rbigint_ratio(value)
         except OverflowError:
             raise oefmt(space.w_OverflowError,
-                        "cannot convert Infinity to integer ratio")
+                        "cannot pass infinity to as_integer_ratio()")
         except ValueError:
             raise oefmt(space.w_ValueError,
-                        "cannot convert NaN to integer ratio")
+                        "cannot pass nan to as_integer_ratio()")
 
         w_num = space.newlong_from_rbigint(num)
         w_den = space.newlong_from_rbigint(den)
         # Try to return int
-        return space.newtuple2(space.int(w_num), space.int(w_den))
+        return space.newtuple([space.int(w_num), space.int(w_den)])
 
     def descr_hex(self, space):
-        """float.hex() -> string
-
-        Return a hexadecimal representation of a floating-point
-        number.
-
-        >>> (-0.1).hex()
-        '-0x1.999999999999ap-4'
-        >>> 3.14159.hex()
-        '0x1.921f9f01b866ep+1'
-
-        """
         TOHEX_NBITS = rfloat.DBL_MANT_DIG + 3 - (rfloat.DBL_MANT_DIG + 2) % 4
         value = self.floatval
         if not isfinite(value):
             return self.descr_str(space)
         if value == 0.0:
-            if math.copysign(1., value) == -1.:
-                return space.newtext("-0x0.0p+0")
+            if copysign(1., value) == -1.:
+                return space.wrap("-0x0.0p+0")
             else:
-                return space.newtext("0x0.0p+0")
+                return space.wrap("0x0.0p+0")
         mant, exp = math.frexp(value)
         shift = 1 - max(rfloat.DBL_MIN_EXP - exp, 0)
         mant = math.ldexp(mant, shift)
@@ -689,9 +641,9 @@ class W_FloatObject(W_Root):
         exp = abs(exp)
         s = ''.join(result)
         if value < 0.0:
-            return space.newtext("-0x%sp%s%d" % (s, sign, exp))
+            return space.wrap("-0x%sp%s%d" % (s, sign, exp))
         else:
-            return space.newtext("0x%sp%s%d" % (s, sign, exp))
+            return space.wrap("0x%sp%s%d" % (s, sign, exp))
 
 
 W_FloatObject.typedef = TypeDef("float",
@@ -700,18 +652,17 @@ W_FloatObject.typedef = TypeDef("float",
 Convert a string or number to a floating point number, if possible.''',
     __new__ = interp2app(W_FloatObject.descr__new__),
     __getformat__ = interp2app(W_FloatObject.descr___getformat__, as_classmethod=True),
-    __round__ = interp2app(W_FloatObject.descr___round__),
     fromhex = interp2app(W_FloatObject.descr_fromhex, as_classmethod=True),
     __repr__ = interp2app(W_FloatObject.descr_repr),
     __str__ = interp2app(W_FloatObject.descr_str),
     __hash__ = interp2app(W_FloatObject.descr_hash),
     __format__ = interp2app(W_FloatObject.descr_format),
-    __bool__ = interp2app(W_FloatObject.descr_bool),
+    __coerce__ = interp2app(W_FloatObject.descr_coerce),
+    __nonzero__ = interp2app(W_FloatObject.descr_nonzero),
     __int__ = interp2app(W_FloatObject.descr_trunc),
     __float__ = interp2app(W_FloatObject.descr_float),
+    __long__ = interp2app(W_FloatObject.descr_long),
     __trunc__ = interp2app(W_FloatObject.descr_trunc),
-    __floor__ = interp2app(W_FloatObject.descr_floor),
-    __ceil__ = interp2app(W_FloatObject.descr_ceil),
     __neg__ = interp2app(W_FloatObject.descr_neg),
     __pos__ = interp2app(W_FloatObject.descr_pos),
     __abs__ = interp2app(W_FloatObject.descr_abs),
@@ -730,6 +681,8 @@ Convert a string or number to a floating point number, if possible.''',
     __rsub__ = interp2app(W_FloatObject.descr_rsub),
     __mul__ = interp2app(W_FloatObject.descr_mul),
     __rmul__ = interp2app(W_FloatObject.descr_rmul),
+    __div__ = interp2app(W_FloatObject.descr_div),
+    __rdiv__ = interp2app(W_FloatObject.descr_rdiv),
     __truediv__ = interp2app(W_FloatObject.descr_div),
     __rtruediv__ = interp2app(W_FloatObject.descr_rdiv),
     __floordiv__ = interp2app(W_FloatObject.descr_floordiv),
@@ -749,73 +702,51 @@ Convert a string or number to a floating point number, if possible.''',
     hex = interp2app(W_FloatObject.descr_hex),
 )
 
-def _remove_underscores(string):
-    i = 0
-    prev = '?'
-    res = []
-    for i in range(len(string)):
-        c = string[i]
-        if c == '_':
-            # undercores can only come after digits
-            if not ord('0') <= ord(prev) <= ord('9'):
-                raise ValueError
-        else:
-            res.append(c)
-            # undercores can only come before digits
-            if prev == '_' and not ord('0') <= ord(c) <= ord('9'):
-                raise ValueError
-        prev = c
-    if prev == "_": # not allowed at end
-        raise ValueError
-    return "".join(res)
 
-def _string_to_float(space, w_source, string):
-    try:
-        if "_" in string:
-            string = _remove_underscores(string)
-    except ValueError:
-        pass
-    else:
+def _hash_float(space, v):
+    if isnan(v):
+        return 0
+
+    # This is designed so that Python numbers of different types
+    # that compare equal hash to the same value; otherwise comparisons
+    # of mapping keys will turn out weird.
+    fractpart, intpart = math.modf(v)
+
+    if fractpart == 0.0:
+        # This must return the same hash as an equal int or long.
         try:
-            return rfloat.string_to_float(string)
-        except ParseStringError as e:
-            pass
-    raise oefmt(space.w_ValueError,
-                "could not convert string to float: %R", w_source)
+            x = ovfcheck_float_to_int(intpart)
+            # Fits in a C long == a Python int, so is its own hash.
+            return x
+        except OverflowError:
+            # Convert to long and use its hash.
+            try:
+                w_lval = W_LongObject.fromfloat(space, v)
+            except (OverflowError, ValueError):
+                # can't convert to long int -- arbitrary
+                if v < 0:
+                    return -271828
+                else:
+                    return 314159
+            return space.int_w(space.hash(w_lval))
 
-@jit.elidable
-def _hash_float(v):
-    if not isfinite(v):
-        if math.isinf(v):
-            return HASH_INF if v > 0 else -HASH_INF
-        return HASH_NAN
+    # The fractional part is non-zero, so we don't have to worry about
+    # making this match the hash of some other type.
+    # Use frexp to get at the bits in the double.
+    # Since the VAX D double format has 56 mantissa bits, which is the
+    # most of any double format in use, each of these parts may have as
+    # many as (but no more than) 56 significant bits.
+    # So, assuming sizeof(long) >= 4, each part can be broken into two
+    # longs; frexp and multiplication are used to do that.
+    # Also, since the Cray double format has 15 exponent bits, which is
+    # the most of any double format in use, shifting the exponent field
+    # left by 15 won't overflow a long (again assuming sizeof(long) >= 4).
 
-    m, e = math.frexp(v)
-
-    sign = 1
-    if m < 0:
-        sign = -1
-        m = -m
-
-    # process 28 bits at a time;  this should work well both for binary
-    # and hexadecimal floating point.
-    x = r_uint(0)
-    while m:
-        x = ((x << 28) & HASH_MODULUS) | x >> (HASH_BITS - 28)
-        m *= 268435456.0  # 2**28
-        e -= 28
-        y = r_uint(m)  # pull out integer part
-        m -= y
-        x += y
-        if x >= HASH_MODULUS:
-            x -= HASH_MODULUS
-
-    # adjust for the exponent;  first reduce it modulo HASH_BITS
-    e = e % HASH_BITS if e >= 0 else HASH_BITS - 1 - ((-1 - e) % HASH_BITS)
-    x = ((x << e) & HASH_MODULUS) | x >> (HASH_BITS - e)
-
-    x = intmask(intmask(x) * sign)
-    x -= (x == -1)
+    v, expo = math.frexp(v)
+    v *= 2147483648.0  # 2**31
+    hipart = int(v)    # take the top 32 bits
+    v = (v - hipart) * 2147483648.0 # get the next 32 bits
+    x = intmask(hipart + int(v) + (expo << 15))
     return x
 
 
@@ -867,16 +798,16 @@ def _pow(space, x, y):
     if y == 0.0:
         # x**0 is 1, even 0**0
         return 1.0
-    if math.isnan(x):
+    if isnan(x):
         # nan**y = nan, unless y == 0
         return x
-    if math.isnan(y):
+    if isnan(y):
         # x**nan = nan, unless x == 1; x**nan = x
         if x == 1.0:
             return 1.0
         else:
             return y
-    if math.isinf(y):
+    if isinf(y):
         # x**inf is: 0.0 if abs(x) < 1; 1.0 if abs(x) == 1; inf if
         # abs(x) > 1 (including case where x infinite)
         #
@@ -889,7 +820,7 @@ def _pow(space, x, y):
             return INFINITY
         else:
             return 0.0
-    if math.isinf(x):
+    if isinf(x):
         # (+-inf)**w is: inf for w positive, 0 for w negative; in oth
         # cases, we need to add the appropriate sign if w is an odd
         # integer.
@@ -901,7 +832,7 @@ def _pow(space, x, y):
                 return abs(x)
         else:
             if y_is_odd:
-                return math.copysign(0.0, x)
+                return copysign(0.0, x)
             else:
                 return 0.0
 
@@ -915,7 +846,7 @@ def _pow(space, x, y):
     # unlike "math.pow(-1.0, bignum)".  See http://mail.python.org/
     # -           pipermail/python-bugs-list/2003-March/016795.html
     if x < 0.0:
-        if math.isnan(y):
+        if isnan(y):
             return NAN
         if math.floor(y) != y:
             raise PowDomainError
@@ -943,38 +874,3 @@ def _pow(space, x, y):
     if negate_result:
         z = -z
     return z
-
-
-def _round_float(space, w_float, w_ndigits=None):
-    # Algorithm copied directly from CPython
-    x = w_float.floatval
-
-    if space.is_none(w_ndigits):
-        # single-argument round: round to nearest integer
-        rounded = rfloat.round_away(x)
-        if math.fabs(x - rounded) == 0.5:
-            # halfway case: round to even
-            rounded = 2.0 * rfloat.round_away(x / 2.0)
-        return newint_from_float(space, rounded)
-
-    # interpret 2nd argument as a Py_ssize_t; clip on overflow
-    ndigits = space.getindex_w(w_ndigits, None)
-
-    # nans and infinities round to themselves
-    if not rfloat.isfinite(x):
-        return space.newfloat(x)
-
-    # Deal with extreme values for ndigits. For ndigits > NDIGITS_MAX, x
-    # always rounds to itself.  For ndigits < NDIGITS_MIN, x always
-    # rounds to +-0.0
-    if ndigits > NDIGITS_MAX:
-        return space.newfloat(x)
-    elif ndigits < NDIGITS_MIN:
-        # return 0.0, but with sign of x
-        return space.newfloat(0.0 * x)
-
-    # finite x, and ndigits is not unreasonably large
-    z = rfloat.round_double(x, ndigits, half_even=True)
-    if math.isinf(z):
-        raise oefmt(space.w_OverflowError, "overflow occurred during round")
-    return space.newfloat(z)
