@@ -1,16 +1,16 @@
 """The unicode/str format() method"""
 
-import math
 import sys
 import string
 
 from pypy.interpreter.error import OperationError, oefmt
-from rpython.rlib import rstring, rlocale, rfloat, jit, rutf8
+from rpython.rlib import rstring, runicode, rlocale, rfloat, jit
 from rpython.rlib.objectmodel import specialize
-from rpython.rlib.rfloat import formatd
+from rpython.rlib.rfloat import copysign, formatd
 from rpython.rlib.rarithmetic import r_uint, intmask
 from pypy.interpreter.signature import Signature
 
+
 @specialize.argtype(1)
 @jit.look_inside_iff(lambda space, s, start, end:
        jit.isconstant(s) and
@@ -48,30 +48,35 @@ def make_template_formatting_class(for_unicode):
     class TemplateFormatter(object):
         is_unicode = for_unicode
 
-        if for_unicode:
-            def wrap(self, u):
-                lgt = rutf8.check_utf8(u, True)
-                return self.space.newutf8(u, lgt)
-        else:
-            def wrap(self, s):
-                return self.space.newbytes(s)
-
         parser_list_w = None
 
         def __init__(self, space, template):
             self.space = space
+            self.empty = u"" if self.is_unicode else ""
             self.template = template
 
-        def build(self, args, w_kwargs):
-            self.args = args
-            self.w_kwargs = w_kwargs
+        def build(self, args):
+            if self.is_unicode:
+                # for unicode, use the slower parse_obj() to get self.w_kwargs
+                # as a wrapped dictionary that may contain full-range unicode
+                # keys.  See test_non_latin1_key
+                space = self.space
+                w_args, w_kwds = args.parse_obj(None, 'format',
+                                                format_signature)
+                self.args = space.listview(w_args)
+                self.w_kwargs = w_kwds
+            else:
+                self.args, self.kwargs = args.unpack()
             self.auto_numbering = 0
             self.auto_numbering_state = ANS_INIT
             return self._build_string(0, len(self.template), 2)
 
         def _build_string(self, start, end, level):
             space = self.space
-            out = rstring.StringBuilder()
+            if self.is_unicode:
+                out = rstring.UnicodeBuilder()
+            else:
+                out = rstring.StringBuilder()
             if not level:
                 raise oefmt(space.w_ValueError, "Recursion depth exceeded")
             level -= 1
@@ -108,7 +113,7 @@ def make_template_formatting_class(for_unicode):
                             assert end_literal > last_literal
                             literal = self.template[last_literal:end_literal]
                             w_entry = space.newtuple([
-                                self.wrap(literal),
+                                space.wrap(literal),
                                 space.w_None, space.w_None, space.w_None])
                             self.parser_list_w.append(w_entry)
                             self.last_end = i
@@ -117,7 +122,6 @@ def make_template_formatting_class(for_unicode):
                     nested = 1
                     field_start = i
                     recursive = False
-                    in_second_part = False
                     while i < end:
                         c = s[i]
                         if c == "{":
@@ -127,13 +131,6 @@ def make_template_formatting_class(for_unicode):
                             nested -= 1
                             if not nested:
                                 break
-                        elif c == "[" and not in_second_part:
-                            i += 1
-                            while i < end and s[i] != "]":
-                                i += 1
-                            continue
-                        elif c == ':' or c == '!':
-                            in_second_part = True
                         i += 1
                     if nested:
                         raise oefmt(space.w_ValueError, "Unmatched '{'")
@@ -172,12 +169,6 @@ def make_template_formatting_class(for_unicode):
                         conversion = None
                         i += 1
                     return s[start:end_name], conversion, i
-                elif c == "[":
-                    while i + 1 < end and s[i + 1] != "]":
-                        i += 1
-                elif c == "{":
-                    raise oefmt(self.space.w_ValueError,
-                                "unexpected '{' in field name")
                 i += 1
             return s[start:end], None, end
 
@@ -220,21 +211,18 @@ def make_template_formatting_class(for_unicode):
             if index == -1:
                 kwarg = name[:i]
                 if self.is_unicode:
-                    w_kwarg = space.newtext(kwarg)
+                    w_arg = space.getitem(self.w_kwargs, space.wrap(kwarg))
                 else:
-                    w_kwarg = space.newbytes(kwarg)
-                w_arg = space.getitem(self.w_kwargs, w_kwarg)
+                    try:
+                        w_arg = self.kwargs[kwarg]
+                    except KeyError:
+                        raise OperationError(space.w_KeyError,
+                                             space.wrap(kwarg))
             else:
-                if self.args is None:
-                    raise oefmt(space.w_ValueError,
-                                "Format string contains positional fields")
                 try:
                     w_arg = self.args[index]
                 except IndexError:
-                    raise oefmt(space.w_IndexError,
-                                "out of range: index %d but only %d argument%s",
-                                index, len(self.args),
-                                "s" if len(self.args) != 1 else "")
+                    raise oefmt(space.w_IndexError, "out of range")
             return self._resolve_lookups(w_arg, name, i, end)
 
         @jit.unroll_safe
@@ -255,7 +243,7 @@ def make_template_formatting_class(for_unicode):
                     if start == i:
                         raise oefmt(space.w_ValueError,
                                     "Empty attribute in format string")
-                    w_attr = self.wrap(name[start:i])
+                    w_attr = space.wrap(name[start:i])
                     if w_obj is not None:
                         w_obj = space.getattr(w_obj, w_attr)
                     else:
@@ -275,9 +263,9 @@ def make_template_formatting_class(for_unicode):
                         raise oefmt(space.w_ValueError, "Missing ']'")
                     index, reached = _parse_int(self.space, name, start, i)
                     if index != -1 and reached == i:
-                        w_item = space.newint(index)
+                        w_item = space.wrap(index)
                     else:
-                        w_item = self.wrap(name[start:i])
+                        w_item = space.wrap(name[start:i])
                     i += 1 # Skip "]"
                     if w_obj is not None:
                         w_obj = space.getitem(w_obj, w_item)
@@ -306,9 +294,9 @@ def make_template_formatting_class(for_unicode):
                 if stop != i:
                     index = -1
             if index >= 0:
-                w_first = space.newint(index)
+                w_first = space.wrap(index)
             else:
-                w_first = self.wrap(name[:i])
+                w_first = space.wrap(name[:i])
             #
             self.parser_list_w = []
             self._resolve_lookups(None, name, i, end)
@@ -325,9 +313,6 @@ def make_template_formatting_class(for_unicode):
                 if self.is_unicode:
                     return space.call_function(space.w_unicode, w_obj)
                 return space.str(w_obj)
-            elif conv == "a":
-                from pypy.objspace.std.unicodeobject import ascii_from_object
-                return ascii_from_object(space, w_obj)
             else:
                 raise oefmt(space.w_ValueError, "invalid conversion")
 
@@ -341,30 +326,24 @@ def make_template_formatting_class(for_unicode):
                     space = self.space
                     startm1 = start - 1
                     assert startm1 >= self.last_end
-                    if conversion is None:
-                        w_conversion = space.w_None
-                    else:
-                        w_conversion = self.wrap(conversion)
                     w_entry = space.newtuple([
-                        self.wrap(self.template[self.last_end:startm1]),
-                        self.wrap(name),
-                        self.wrap(spec),
-                        w_conversion])
+                        space.wrap(self.template[self.last_end:startm1]),
+                        space.wrap(name),
+                        space.wrap(spec),
+                        space.wrap(conversion)])
                     self.parser_list_w.append(w_entry)
                     self.last_end = end + 1
-                return ""
+                return self.empty
             #
             w_obj = self._get_argument(name)
             if conversion is not None:
                 w_obj = self._convert(w_obj, conversion)
             if recursive:
                 spec = self._build_string(spec_start, end, level)
-            w_rendered = self.space.format(w_obj, self.wrap(spec))
-            if self.is_unicode:
-                w_rendered = self.space.unicode_from_object(w_rendered)
-                return self.space.utf8_w(w_rendered)
-            else:
-                return self.space.bytes_w(w_rendered)
+            w_rendered = self.space.format(w_obj, self.space.wrap(spec))
+            unwrapper = "unicode_w" if self.is_unicode else "str_w"
+            to_interp = getattr(self.space, unwrapper)
+            return to_interp(w_rendered)
 
         def formatter_parser(self):
             self.parser_list_w = []
@@ -374,7 +353,7 @@ def make_template_formatting_class(for_unicode):
             space = self.space
             if self.last_end < len(self.template):
                 w_lastentry = space.newtuple([
-                    self.wrap(self.template[self.last_end:]),
+                    space.wrap(self.template[self.last_end:]),
                     space.w_None,
                     space.w_None,
                     space.w_None])
@@ -386,16 +365,14 @@ str_template_formatter = make_template_formatting_class(for_unicode=False)
 unicode_template_formatter = make_template_formatting_class(for_unicode=True)
 
 
-def format_method(space, w_string, args, w_kwargs, is_unicode):
+def format_method(space, w_string, args, is_unicode):
     if is_unicode:
         template = unicode_template_formatter(space,
-                                              space.utf8_w(w_string))
-        r = template.build(args, w_kwargs)
-        lgt = rutf8.check_utf8(r, True)
-        return space.newutf8(r, lgt)
+                                              space.unicode_w(w_string))
+        return space.wrap(template.build(args))
     else:
-        template = str_template_formatter(space, space.bytes_w(w_string))
-        return space.newbytes(template.build(args, w_kwargs))
+        template = str_template_formatter(space, space.str_w(w_string))
+        return space.wrap(template.build(args))
 
 
 class NumberSpec(object):
@@ -426,19 +403,12 @@ def make_formatting_class(for_unicode):
     class Formatter(BaseFormatter):
         """__format__ implementation for builtin types."""
 
-        if for_unicode:
-            def wrap(self, u):
-                lgt = rutf8.check_utf8(u, True)
-                return self.space.newutf8(u, lgt)
-        else:
-            def wrap(self, s):
-                return self.space.newbytes(s)
-
         is_unicode = for_unicode
         _grouped_digits = None
 
         def __init__(self, space, spec):
             self.space = space
+            self.empty = u"" if self.is_unicode else ""
             self.spec = spec
 
         def _is_alignment(self, c):
@@ -458,7 +428,7 @@ def make_formatting_class(for_unicode):
             self._align = default_align
             self._alternate = False
             self._sign = "\0"
-            self._thousands_sep = "\0"
+            self._thousands_sep = False
             self._precision = -1
             the_type = default_type
             spec = self.spec
@@ -468,16 +438,11 @@ def make_formatting_class(for_unicode):
             i = 0
             got_align = True
             got_fill_char = False
-            # The single character could be utf8-encoded unicode
-            if self.is_unicode:
-                after_i = rutf8.next_codepoint_pos(spec, i)
-            else:
-                after_i = i + 1
-            if length - i >= 2 and self._is_alignment(spec[after_i]):
-                self._align = spec[after_i]
-                self._fill_char = spec[i:after_i]
+            if length - i >= 2 and self._is_alignment(spec[i + 1]):
+                self._align = spec[i + 1]
+                self._fill_char = spec[i]
                 got_fill_char = True
-                i = after_i + 1
+                i += 2
             elif length - i >= 1 and self._is_alignment(spec[i]):
                 self._align = spec[i]
                 i += 1
@@ -496,17 +461,8 @@ def make_formatting_class(for_unicode):
                 i += 1
             self._width, i = _parse_int(self.space, spec, i, length)
             if length != i and spec[i] == ",":
-                self._thousands_sep = ","
+                self._thousands_sep = True
                 i += 1
-            if length != i and spec[i] == "_":
-                if self._thousands_sep != "\0":
-                    raise oefmt(
-                        space.w_ValueError, "Cannot specify both ',' and '_'.")
-                self._thousands_sep = "_"
-                i += 1
-                if length != i and spec[i] == ",":
-                    raise oefmt(
-                        space.w_ValueError, "Cannot specify both ',' and '_'.")
             if length != i and spec[i] == ".":
                 i += 1
                 self._precision, i = _parse_int(self.space, spec, i, length)
@@ -518,16 +474,15 @@ def make_formatting_class(for_unicode):
                 presentation_type = spec[i]
                 if self.is_unicode:
                     try:
-                        rutf8.check_utf8(spec[i], True)
-                        the_type = spec[i][0]
-                    except rutf8.CheckError:
+                        the_type = spec[i].encode("ascii")[0]
+                    except UnicodeEncodeError:
                         raise oefmt(space.w_ValueError,
                                     "invalid presentation type")
                 else:
                     the_type = presentation_type
                 i += 1
             self._type = the_type
-            if self._thousands_sep != "\0":
+            if self._thousands_sep:
                 tp = self._type
                 if (tp == "d" or
                     tp == "e" or
@@ -540,16 +495,8 @@ def make_formatting_class(for_unicode):
                     tp == "\0"):
                     # ok
                     pass
-                elif self._thousands_sep == "_" and (
-                        tp == "b" or
-                        tp == "o" or
-                        tp == "x" or
-                        tp == "X"):
-                    pass # ok
                 else:
-                    raise oefmt(space.w_ValueError,
-                                "Cannot specify '%s' with '%s'.", 
-                                self._thousands_sep, tp)
+                    raise oefmt(space.w_ValueError, "invalid type with ','")
             return False
 
         def _calc_padding(self, string, length):
@@ -573,9 +520,8 @@ def make_formatting_class(for_unicode):
             return total
 
         def _lit(self, s):
-            assert len(s) == 1
             if self.is_unicode:
-                return rutf8.unichr_as_utf8(ord(s[0]))
+                return s.decode("latin-1")
             else:
                 return s
 
@@ -588,23 +534,20 @@ def make_formatting_class(for_unicode):
 
         def _builder(self):
             if self.is_unicode:
-                return rutf8.Utf8StringBuilder()
+                return rstring.UnicodeBuilder()
             else:
                 return rstring.StringBuilder()
 
-        def _unknown_presentation(self, w_val):
+        def _unknown_presentation(self, tp):
             raise oefmt(self.space.w_ValueError,
-                        "Unknown format code %s for object of type '%T'", self._type, w_val)
+                        "unknown presentation for %s: '%s'", tp, self._type)
 
-        def format_string(self, w_string):
+        def format_string(self, string):
             space = self.space
-            if not space.is_w(space.type(w_string), space.w_unicode):
-                w_string = space.str(w_string)
-            string = space.utf8_w(w_string)
             if self._parse_spec("s", "<"):
-                return self.wrap(string)
+                return space.wrap(string)
             if self._type != "s":
-                self._unknown_presentation(w_string)
+                self._unknown_presentation("string")
             if self._sign != "\0":
                 raise oefmt(space.w_ValueError,
                             "Sign not allowed in string format specifier")
@@ -616,38 +559,29 @@ def make_formatting_class(for_unicode):
                 raise oefmt(space.w_ValueError,
                             "'=' alignment not allowed in string format "
                             "specifier")
-            length = space.len_w(w_string)
+            length = len(string)
             precision = self._precision
             if precision != -1 and length >= precision:
                 assert precision >= 0
                 length = precision
-                if for_unicode:
-                    w_slice = space.newslice(
-                        space.newint(0), space.newint(precision), space.w_None)
-                    w_string = space.getitem(w_string, w_slice)
-                    string = space.utf8_w(w_string)
-                else:
-                    string = string[:precision]
+                string = string[:precision]
             self._calc_padding(string, length)
-            return self.wrap(self._pad(string))
+            return space.wrap(self._pad(string))
 
         def _get_locale(self, tp):
             if tp == "n":
                 dec, thousands, grouping = rlocale.numeric_formatting()
-            elif self._thousands_sep != "\0":
-                thousands = self._thousands_sep
+            elif self._thousands_sep:
                 dec = "."
+                thousands = ","
                 grouping = "\3"
-                if tp in "boxX":
-                    assert self._thousands_sep == "_"
-                    grouping = "\4"
             else:
                 dec = "."
                 thousands = ""
                 grouping = "\xFF"    # special value to mean 'stop'
             if self.is_unicode:
-                self._loc_dec = rutf8.decode_latin_1(dec)
-                self._loc_thousands = rutf8.decode_latin_1(thousands)
+                self._loc_dec = dec.decode("latin-1")
+                self._loc_thousands = thousands.decode("latin-1")
             else:
                 self._loc_dec = dec
                 self._loc_thousands = thousands
@@ -766,7 +700,7 @@ def make_formatting_class(for_unicode):
                 ts = self._loc_thousands if need_separator else None
                 self._fill_digits(buf, digits, left, n_chars, n_zeros, ts)
             buf.reverse()
-            self._grouped_digits = "".join(buf)
+            self._grouped_digits = self.empty.join(buf)
 
         def _upcase_string(self, s):
             buf = []
@@ -775,16 +709,19 @@ def make_formatting_class(for_unicode):
                 if ord("a") <= index <= ord("z"):
                     c = chr(index - 32)
                 buf.append(c)
-            return "".join(buf)
+            return self.empty.join(buf)
 
 
         def _fill_number(self, spec, num, to_digits, to_prefix, fill_char,
                          to_remainder, upper, grouped_digits=None):
             out = self._builder()
             if spec.n_lpadding:
-                out.append_multiple_char(fill_char, spec.n_lpadding)
+                out.append_multiple_char(fill_char[0], spec.n_lpadding)
             if spec.n_sign:
-                sign = self._lit(spec.sign)
+                if self.is_unicode:
+                    sign = spec.sign.decode("latin-1")
+                else:
+                    sign = spec.sign
                 out.append(sign)
             if spec.n_prefix:
                 pref = num[to_prefix:to_prefix + spec.n_prefix]
@@ -792,7 +729,7 @@ def make_formatting_class(for_unicode):
                     pref = self._upcase_string(pref)
                 out.append(pref)
             if spec.n_spadding:
-                out.append_multiple_char(fill_char, spec.n_spadding)
+                out.append_multiple_char(fill_char[0], spec.n_spadding)
             if spec.n_digits != 0:
                 if self._loc_thousands:
                     if grouped_digits is not None:
@@ -808,11 +745,11 @@ def make_formatting_class(for_unicode):
                     digits = self._upcase_string(digits)
                 out.append(digits)
             if spec.n_decimal:
-                out.append(self._lit(self._loc_dec)[0])
+                out.append(self._lit(".")[0])
             if spec.n_remainder:
                 out.append(num[to_remainder:])
             if spec.n_rpadding:
-                out.append_multiple_char(fill_char, spec.n_rpadding)
+                out.append_multiple_char(fill_char[0], spec.n_rpadding)
             #if complex, need to call twice - just retun the buffer
             return out.build()
 
@@ -827,18 +764,14 @@ def make_formatting_class(for_unicode):
                 if self._sign != "\0":
                     raise oefmt(space.w_ValueError,
                                 "sign not allowed with 'c' presentation type")
-                if self._alternate:
-                    raise oefmt(space.w_ValueError,
-                                "Alternate form (#) not allowed "
-                                "with 'c' presentation type")
                 value = space.int_w(w_num)
-                max_char = 0x10FFFF if self.is_unicode else 0xFF
+                max_char = runicode.MAXUNICODE if self.is_unicode else 0xFF
                 if not (0 <= value <= max_char):
                     raise oefmt(space.w_OverflowError,
                                 "%%c arg not in range(%s)",
                                 hex(max_char))
                 if self.is_unicode:
-                    result = rutf8.unichr_as_utf8(value)
+                    result = runicode.UNICHR(value)
                 else:
                     result = chr(value)
                 n_digits = 1
@@ -881,8 +814,8 @@ def make_formatting_class(for_unicode):
                                         n_remainder, False, result)
             fill = self._fill_char
             upper = self._type == "X"
-            return self.wrap(self._fill_number(spec, result, to_numeric,
-                             to_prefix, fill, to_remainder, upper))
+            return self.space.wrap(self._fill_number(spec, result, to_numeric,
+                                     to_prefix, fill, to_remainder, upper))
 
         def _long_to_base(self, base, value):
             prefix = ""
@@ -894,14 +827,14 @@ def make_formatting_class(for_unicode):
                 prefix = "0x"
             as_str = value.format(LONG_DIGITS[:base], prefix)
             if self.is_unicode:
-                return rutf8.decode_latin_1(as_str)
+                return as_str.decode("latin-1")
             return as_str
 
         def _int_to_base(self, base, value):
             if base == 10:
                 s = str(value)
                 if self.is_unicode:
-                    return rutf8.decode_latin_1(s)
+                    return s.decode("latin-1")
                 return s
             # This part is slow.
             negative = value < 0
@@ -942,7 +875,7 @@ def make_formatting_class(for_unicode):
                 i -= 1
                 buf[i] = "-"
             assert i >= 0
-            return "".join(buf[i:])
+            return self.empty.join(buf[i:])
 
         def format_int_or_long(self, w_num, kind):
             space = self.space
@@ -969,7 +902,7 @@ def make_formatting_class(for_unicode):
                 w_float = space.float(w_num)
                 return self._format_float(w_float)
             else:
-                self._unknown_presentation(w_num)
+                self._unknown_presentation("int" if kind == INT_KIND else "long")
 
         def _parse_number(self, s, i):
             """Determine if s has a decimal point, and the index of the first #
@@ -990,14 +923,14 @@ def make_formatting_class(for_unicode):
             flags = 0
             default_precision = 6
             if self._alternate:
-               flags |= rfloat.DTSF_ALT
-
+                raise oefmt(space.w_ValueError,
+                            "Alternate form (#) not allowed in float formats")
             tp = self._type
             self._get_locale(tp)
             if tp == "\0":
+                tp = "g"
+                default_precision = 12
                 flags |= rfloat.DTSF_ADD_DOT_0
-                tp = "r"
-                default_precision = 0
             elif tp == "n":
                 tp = "g"
             value = space.float_w(w_float)
@@ -1009,8 +942,6 @@ def make_formatting_class(for_unicode):
                 add_pct = False
             if self._precision == -1:
                 self._precision = default_precision
-            elif tp == "r":
-                tp = "g"
             result, special = rfloat.double_to_string(value, tp,
                                                       self._precision, flags)
             if add_pct:
@@ -1026,14 +957,14 @@ def make_formatting_class(for_unicode):
             have_dec_point, to_remainder = self._parse_number(result, to_number)
             n_remainder = len(result) - to_remainder
             if self.is_unicode:
-                digits = rutf8.decode_latin_1(result)
+                digits = result.decode("latin-1")
             else:
                 digits = result
             spec = self._calc_num_width(0, sign, to_number, n_digits,
                                         n_remainder, have_dec_point, digits)
             fill = self._fill_char
-            return self.wrap(self._fill_number(spec, digits, to_number, 0,
-                             fill, to_remainder, False))
+            return self.space.wrap(self._fill_number(spec, digits, to_number, 0,
+                                      fill, to_remainder, False))
 
         def format_float(self, w_float):
             space = self.space
@@ -1052,10 +983,9 @@ def make_formatting_class(for_unicode):
                 tp == "n" or
                 tp == "%"):
                 return self._format_float(w_float)
-            self._unknown_presentation(w_float)
+            self._unknown_presentation("float")
 
         def _format_complex(self, w_complex):
-            flags = 0
             space = self.space
             tp = self._type
             self._get_locale(tp)
@@ -1071,8 +1001,10 @@ def make_formatting_class(for_unicode):
                             "Zero padding is not allowed in complex format "
                             "specifier")
             if self._alternate:
-                flags |= rfloat.DTSF_ALT
-
+                # alternate is invalid
+                raise oefmt(space.w_ValueError,
+                            "Alternate form (#) not allowed in complex format "
+                            "specifier")
             skip_re = 0
             add_parens = 0
             if tp == "\0":
@@ -1081,7 +1013,7 @@ def make_formatting_class(for_unicode):
                 default_precision = 12
                 #test if real part is non-zero
                 if (w_complex.realval == 0 and
-                    math.copysign(1., w_complex.realval) == 1.):
+                    copysign(1., w_complex.realval) == 1.):
                     skip_re = 1
                 else:
                     add_parens = 1
@@ -1094,9 +1026,10 @@ def make_formatting_class(for_unicode):
             if self._precision == -1:
                 self._precision = default_precision
 
+            #might want to switch to double_to_string from formatd
             #in CPython it's named 're' - clashes with re module
-            re_num, special = rfloat.double_to_string(w_complex.realval, tp, self._precision, flags)
-            im_num, special = rfloat.double_to_string(w_complex.imagval, tp, self._precision, flags)
+            re_num = formatd(w_complex.realval, tp, self._precision)
+            im_num = formatd(w_complex.imagval, tp, self._precision)
             n_re_digits = len(re_num)
             n_im_digits = len(im_num)
 
@@ -1130,8 +1063,8 @@ def make_formatting_class(for_unicode):
                                                                to_imag_number)
 
             if self.is_unicode:
-                re_num = rutf8.decode_latin_1(re_num)
-                im_num = rutf8.decode_latin_1(im_num)
+                re_num = re_num.decode("latin-1")
+                im_num = im_num.decode("latin-1")
 
             #set remainder, in CPython _parse_number sets this
             #using n_re_digits causes tests to fail
@@ -1160,7 +1093,7 @@ def make_formatting_class(for_unicode):
             self._fill_char = tmp_fill_char
 
             #compute L and R padding - stored in self._left_pad and self._right_pad
-            self._calc_padding("", re_spec.n_total + im_spec.n_total + 1 +
+            self._calc_padding(self.empty, re_spec.n_total + im_spec.n_total + 1 +
                                            add_parens * 2)
 
             out = self._builder()
@@ -1192,7 +1125,7 @@ def make_formatting_class(for_unicode):
             #add right padding
             out.append_multiple_char(fill, self._right_pad)
 
-            return self.wrap(out.build())
+            return self.space.wrap(out.build())
 
 
         def format_complex(self, w_complex):
@@ -1211,13 +1144,18 @@ def make_formatting_class(for_unicode):
                 tp == "G" or
                 tp == "n"):
                 return self._format_complex(w_complex)
-            self._unknown_presentation(w_complex)
+            self._unknown_presentation("complex")
     return Formatter
 
+str_formatter = make_formatting_class(for_unicode=False)
 unicode_formatter = make_formatting_class(for_unicode=True)
 
 
 @specialize.arg(2)
 def run_formatter(space, w_format_spec, meth, *args):
-    formatter = unicode_formatter(space, space.utf8_w(w_format_spec))
-    return getattr(formatter, meth)(*args)
+    if space.isinstance_w(w_format_spec, space.w_unicode):
+        formatter = unicode_formatter(space, space.unicode_w(w_format_spec))
+        return getattr(formatter, meth)(*args)
+    else:
+        formatter = str_formatter(space, space.str_w(w_format_spec))
+        return getattr(formatter, meth)(*args)
