@@ -1,4 +1,3 @@
-# coding: utf-8
 import py
 from py.test import raises
 from pypy.interpreter.error import OperationError
@@ -12,12 +11,9 @@ class TestW_StdObjSpace:
                           self.space.wrap,
                           self.space.wrap(0))
 
-    def test_utf8(self):
-        assert self.space.isinstance_w(self.space.newtext("abc"), self.space.w_unicode)
-
-    def test_text_w_non_str(self):
-        raises(OperationError,self.space.text_w,self.space.wrap(None))
-        raises(OperationError,self.space.text_w,self.space.wrap(0))
+    def test_str_w_non_str(self):
+        raises(OperationError,self.space.str_w,self.space.wrap(None))
+        raises(OperationError,self.space.str_w,self.space.wrap(0))
 
     def test_int_w_non_int(self):
         raises(OperationError,self.space.int_w,self.space.wrap(None))
@@ -42,25 +38,32 @@ class TestW_StdObjSpace:
 
     def test_fastpath_isinstance(self):
         from pypy.objspace.std.bytesobject import W_BytesObject
-        from pypy.objspace.std.intobject import W_AbstractIntObject
+        from pypy.objspace.std.intobject import W_IntObject
         from pypy.objspace.std.iterobject import W_AbstractSeqIterObject
         from pypy.objspace.std.iterobject import W_SeqIterObject
 
         space = self.space
-        assert space._get_interplevel_cls(space.w_bytes) is W_BytesObject
-        assert space._get_interplevel_cls(space.w_int) is W_AbstractIntObject
+        assert space._get_interplevel_cls(space.w_str) is W_BytesObject
+        assert space._get_interplevel_cls(space.w_int) is W_IntObject
         class X(W_BytesObject):
             def __init__(self):
                 pass
 
             typedef = None
 
-        assert space.isinstance_w(X(), space.w_bytes)
+        assert space.isinstance_w(X(), space.w_str)
 
         w_sequenceiterator = space.gettypefor(W_SeqIterObject)
         cls = space._get_interplevel_cls(w_sequenceiterator)
         assert cls is W_AbstractSeqIterObject
 
+    def test_withstrbuf_fastpath_isinstance(self):
+        from pypy.objspace.std.bytesobject import W_AbstractBytesObject
+
+        space = gettestobjspace(withstrbuf=True)
+        cls = space._get_interplevel_cls(space.w_str)
+        assert cls is W_AbstractBytesObject
+
     def test_wrap_various_unsigned_types(self):
         import sys
         from rpython.rlib.rarithmetic import r_uint
@@ -78,14 +81,3 @@ class TestW_StdObjSpace:
         value = 200
         x = rffi.cast(rffi.UCHAR, r_uint(value))
         assert space.eq_w(space.wrap(value), space.wrap(x))
-
-    def test_wrap_string(self):
-        from pypy.objspace.std.unicodeobject import W_UnicodeObject
-        from pypy.objspace.std.unicodeobject import BadUtf8
-        w_x = self.space.wrap('foo')
-        assert isinstance(w_x, W_UnicodeObject)
-        assert w_x._utf8 == 'foo'
-        #
-        # calling space.wrap() on a byte string which is not utf-8 should
-        # never happen.
-        py.test.raises(BadUtf8, self.space.wrap, 'foo\xF0')
