@@ -1,22 +1,22 @@
 from __future__ import absolute_import
 
 import types
+from collections import defaultdict
 
-from rpython.tool.ansi_print import ansi_log
+from rpython.tool.ansi_print import AnsiLogger
 from rpython.tool.pairtype import pair
 from rpython.tool.error import (format_blocked_annotation_error,
                              gather_error, source_lines)
-from rpython.flowspace.model import (Variable, Constant, FunctionGraph,
-                                      c_last_exception, checkgraph)
+from rpython.flowspace.model import Variable, Constant, checkgraph
 from rpython.translator import simplify, transform
-from rpython.annotator import model as annmodel, signature, unaryop, binaryop
+from rpython.annotator import model as annmodel, signature
+from rpython.annotator.model import (
+    typeof, s_ImpossibleValue, SomeInstance, intersection, difference)
 from rpython.annotator.bookkeeper import Bookkeeper
+from rpython.rtyper.normalizecalls import perform_normalizations
 
-import py
-log = py.log.Producer("annrpython")
-py.log.setconsumer("annrpython", ansi_log)
+log = AnsiLogger("annrpython")
 
-FAIL = object()
 
 class RPythonAnnotator(object):
     """Block annotator for RPython.
@@ -24,7 +24,6 @@ class RPythonAnnotator(object):
 
     def __init__(self, translator=None, policy=None, bookkeeper=None):
         import rpython.rtyper.extfuncregistry # has side effects
-        import rpython.rlib.nonconst # has side effects
 
         if translator is None:
             # interface for tests
@@ -33,7 +32,6 @@ class RPythonAnnotator(object):
             translator.annotator = self
         self.translator = translator
         self.pendingblocks = {}  # map {block: graph-containing-it}
-        self.bindings = {}       # map Variables to SomeValues
         self.annotated = {}      # set of blocks already seen
         self.added_blocks = None # see processblock() below
         self.links_followed = {} # set of links that have ever been followed
@@ -54,7 +52,7 @@ class RPythonAnnotator(object):
         self.bookkeeper = bookkeeper
 
     def __getstate__(self):
-        attrs = """translator pendingblocks bindings annotated links_followed
+        attrs = """translator pendingblocks annotated links_followed
         notify bookkeeper frozen policy added_blocks""".split()
         ret = self.__dict__.copy()
         for key, value in ret.items():
@@ -81,33 +79,22 @@ class RPythonAnnotator(object):
         annmodel.TLS.check_str_without_nul = (
             self.translator.config.translation.check_str_without_nul)
 
-        flowgraph, inputcells = self.get_call_parameters(function, args_s, policy)
-        if not isinstance(flowgraph, FunctionGraph):
-            assert isinstance(flowgraph, annmodel.SomeObject)
-            return flowgraph
+        flowgraph, inputs_s = self.get_call_parameters(function, args_s, policy)
 
         if main_entry_point:
             self.translator.entry_point_graph = flowgraph
-        return self.build_graph_types(flowgraph, inputcells, complete_now=complete_now)
+        return self.build_graph_types(flowgraph, inputs_s, complete_now=complete_now)
 
     def get_call_parameters(self, function, args_s, policy):
         desc = self.bookkeeper.getdesc(function)
-        args = self.bookkeeper.build_args("simple_call", args_s[:])
-        result = []
-        def schedule(graph, inputcells):
-            result.append((graph, inputcells))
-            return annmodel.s_ImpossibleValue
-
         prevpolicy = self.policy
         self.policy = policy
         self.bookkeeper.enter(None)
         try:
-            desc.pycall(schedule, args, annmodel.s_ImpossibleValue)
+            return desc.get_call_parameters(args_s)
         finally:
             self.bookkeeper.leave()
             self.policy = prevpolicy
-        [(graph, inputcells)] = result
-        return graph, inputcells
 
     def annotate_helper(self, function, args_s, policy=None):
         if policy is None:
@@ -143,7 +130,7 @@ class RPythonAnnotator(object):
         # recursively proceed until no more pending block is left
         if complete_now:
             self.complete()
-        return self.binding(flowgraph.getreturnvar(), None)
+        return self.annotation(flowgraph.getreturnvar())
 
     def gettype(self, variable):
         """Return the known type of a control flow graph variable,
@@ -151,9 +138,9 @@ class RPythonAnnotator(object):
         if isinstance(variable, Constant):
             return type(variable.value)
         elif isinstance(variable, Variable):
-            cell = self.bindings.get(variable)
-            if cell:
-                return cell.knowntype
+            s_variable = variable.annotation
+            if s_variable:
+                return s_variable.knowntype
             else:
                 return object
         else:
@@ -177,8 +164,15 @@ class RPythonAnnotator(object):
             # annotations that are passed in, and don't annotate the old
             # graph -- it's already low-level operations!
             for a, s_newarg in zip(block.inputargs, cells):
-                s_oldarg = self.binding(a)
-                assert annmodel.unionof(s_oldarg, s_newarg) == s_oldarg
+                s_oldarg = a.annotation
+                # XXX: Should use s_oldarg.contains(s_newarg) but that breaks
+                # PyPy translation
+                if annmodel.unionof(s_oldarg, s_newarg) != s_oldarg:
+                    raise annmodel.AnnotatorError(
+                        "Late-stage annotation is not allowed to modify the "
+                        "existing annotation for variable %s: %s" %
+                            (a, s_oldarg))
+
         else:
             assert not self.frozen
             if block not in self.annotated:
@@ -221,39 +215,40 @@ class RPythonAnnotator(object):
             raise annmodel.AnnotatorError(text)
         for graph in newgraphs:
             v = graph.getreturnvar()
-            if v not in self.bindings:
-                self.setbinding(v, annmodel.s_ImpossibleValue)
-        # policy-dependent computation
-        self.bookkeeper.compute_at_fixpoint()
+            if v.annotation is None:
+                self.setbinding(v, s_ImpossibleValue)
 
-    def binding(self, arg, default=FAIL):
+    def validate(self):
+        """Check that the annotation results are valid"""
+        self.bookkeeper.check_no_flags_on_instances()
+
+    def annotation(self, arg):
         "Gives the SomeValue corresponding to the given Variable or Constant."
         if isinstance(arg, Variable):
-            try:
-                return self.bindings[arg]
-            except KeyError:
-                if default is not FAIL:
-                    return default
-                else:
-                    raise
+            return arg.annotation
         elif isinstance(arg, Constant):
-            #if arg.value is undefined_value:   # undefined local variables
-            #    return annmodel.s_ImpossibleValue
-            return self.bookkeeper.immutableconstant(arg)
+            return self.bookkeeper.immutablevalue(arg.value)
         else:
             raise TypeError('Variable or Constant expected, got %r' % (arg,))
 
+    def binding(self, arg):
+        "Gives the SomeValue corresponding to the given Variable or Constant."
+        s_arg = self.annotation(arg)
+        if s_arg is None:
+            raise KeyError
+        return s_arg
+
     def typeannotation(self, t):
         return signature.annotation(t, self.bookkeeper)
 
     def setbinding(self, arg, s_value):
-        if arg in self.bindings:
-            assert s_value.contains(self.bindings[arg])
-        self.bindings[arg] = s_value
-
-    def transfer_binding(self, v_target, v_source):
-        assert v_source in self.bindings
-        self.bindings[v_target] = self.bindings[v_source]
+        s_old = arg.annotation
+        if s_old is not None:
+            if not s_value.contains(s_old):
+                log.WARNING("%s does not contain %s" % (s_value, s_old))
+                log.WARNING("%s" % annmodel.unionof(s_value, s_old))
+                assert False
+        arg.annotation = s_value
 
     def warning(self, msg, pos=None):
         if pos is None:
@@ -292,16 +287,28 @@ class RPythonAnnotator(object):
         # get the (current) return value
         v = graph.getreturnvar()
         try:
-            return self.bindings[v]
+            return self.binding(v)
         except KeyError:
             # the function didn't reach any return statement so far.
             # (some functions actually never do, they always raise exceptions)
-            return annmodel.s_ImpossibleValue
+            return s_ImpossibleValue
 
     def reflowfromposition(self, position_key):
         graph, block, index = position_key
         self.reflowpendingblock(graph, block)
 
+    def call_sites(self):
+        newblocks = self.added_blocks
+        if newblocks is None:
+            newblocks = self.annotated  # all of them
+        for block in newblocks:
+            for op in block.operations:
+                if op.opname in ('simple_call', 'call_args'):
+                    yield op
+
+                # some blocks are partially annotated
+                if op.result.annotation is None:
+                    break   # ignore the unannotated part
 
     #___ simplification (should be moved elsewhere?) _______
 
@@ -319,6 +326,9 @@ class RPythonAnnotator(object):
                     graphs[graph] = True
         for graph in graphs:
             simplify.eliminate_empty_blocks(graph)
+        self.bookkeeper.compute_at_fixpoint()
+        if block_subset is None:
+            perform_normalizations(self)
 
 
     #___ flowing annotations in blocks _____________________
@@ -330,7 +340,7 @@ class RPythonAnnotator(object):
         #  * block not in self.annotated:
         #      never seen the block.
         #  * self.annotated[block] == False:
-        #      the input variables of the block are in self.bindings but we
+        #      the input variables of the block have bindings but we
         #      still have to consider all the operations in the block.
         #  * self.annotated[block] == graph-containing-block:
         #      analysis done (at least until we find we must generalize the
@@ -342,10 +352,10 @@ class RPythonAnnotator(object):
             del self.blocked_blocks[block]
         try:
             self.flowin(graph, block)
-        except BlockedInference, e:
+        except BlockedInference as e:
             self.annotated[block] = False   # failed, hopefully temporarily
             self.blocked_blocks[block] = (graph, e.opindex)
-        except Exception, e:
+        except Exception as e:
             # hack for debug tools only
             if not hasattr(e, '__annotator_block'):
                 setattr(e, '__annotator_block', block)
@@ -379,7 +389,7 @@ class RPythonAnnotator(object):
         oldcells = [self.binding(a) for a in block.inputargs]
         try:
             unions = [annmodel.unionof(c1,c2) for c1, c2 in zip(oldcells,inputcells)]
-        except annmodel.UnionError, e:
+        except annmodel.UnionError as e:
             # Add source code to the UnionError
             e.source = '\n'.join(source_lines(graph, block, None, long=True))
             raise
@@ -387,6 +397,34 @@ class RPythonAnnotator(object):
         if unions != oldcells:
             self.bindinputargs(graph, block, unions)
 
+    def apply_renaming(self, s_out, renaming):
+        if hasattr(s_out, 'is_type_of'):
+            renamed_is_type_of = []
+            for v in s_out.is_type_of:
+                renamed_is_type_of += renaming[v]
+            assert s_out.knowntype is type
+            newcell = typeof(renamed_is_type_of)
+            if s_out.is_constant():
+                newcell.const = s_out.const
+            s_out = newcell
+
+        if hasattr(s_out, 'knowntypedata'):
+            renamed_knowntypedata = {}
+            for value, constraints in s_out.knowntypedata.items():
+                renamed_knowntypedata[value] = {}
+                for v, s in constraints.items():
+                    new_vs = renaming.get(v, [])
+                    for new_v in new_vs:
+                        renamed_knowntypedata[value][new_v] = s
+            assert isinstance(s_out, annmodel.SomeBool)
+            newcell = annmodel.SomeBool()
+            if s_out.is_constant():
+                newcell.const = s_out.const
+            s_out = newcell
+            s_out.set_knowntypedata(renamed_knowntypedata)
+        return s_out
+
+
     def whereami(self, position_key):
         graph, block, i = position_key
         blk = ""
@@ -400,18 +438,23 @@ class RPythonAnnotator(object):
         return repr(graph) + blk + opid
 
     def flowin(self, graph, block):
-        #print 'Flowing', block, [self.binding(a) for a in block.inputargs]
         try:
-            for i in range(len(block.operations)):
-                try:
-                    self.bookkeeper.enter((graph, block, i))
-                    self.consider_op(block, i)
-                finally:
-                    self.bookkeeper.leave()
+            i = 0
+            while i < len(block.operations):
+                op = block.operations[i]
+                with self.bookkeeper.at_position((graph, block, i)):
+                    new_ops = op.transform(self)
+                    if new_ops is not None:
+                        block.operations[i:i+1] = new_ops
+                        if not new_ops:
+                            continue
+                        new_ops[-1].result = op.result
+                        op = new_ops[0]
+                    self.consider_op(op)
+                i += 1
 
-        except BlockedInference, e:
-            if (e.op is block.operations[-1] and
-                block.exitswitch == c_last_exception):
+        except BlockedInference as e:
+            if e.op is block.raising_op:
                 # this is the case where the last operation of the block will
                 # always raise an exception which is immediately caught by
                 # an exception handler.  We then only follow the exceptional
@@ -431,144 +474,50 @@ class RPythonAnnotator(object):
                 # other cases are problematic (but will hopefully be solved
                 # later by reflowing).  Throw the BlockedInference up to
                 # processblock().
+                e.opindex = i
                 raise
 
         except annmodel.HarmlesslyBlocked:
             return
 
+        except annmodel.AnnotatorError as e: # note that UnionError is a subclass
+            e.source = gather_error(self, graph, block, i)
+            raise
+
         else:
             # dead code removal: don't follow all exits if the exitswitch
             # is known
             exits = block.exits
             if isinstance(block.exitswitch, Variable):
-                s_exitswitch = self.bindings[block.exitswitch]
+                s_exitswitch = self.binding(block.exitswitch)
                 if s_exitswitch.is_constant():
                     exits = [link for link in exits
                                   if link.exitcase == s_exitswitch.const]
 
-        # mapping (exitcase, variable) -> s_annotation
-        # that can be attached to booleans, exitswitches
-        knowntypedata = getattr(self.bindings.get(block.exitswitch),
-                                "knowntypedata", {})
-
-        # filter out those exceptions which cannot
-        # occour for this specific, typed operation.
-        if block.exitswitch == c_last_exception:
-            op = block.operations[-1]
-            if op.opname in binaryop.BINARY_OPERATIONS:
-                arg1 = self.binding(op.args[0])
-                arg2 = self.binding(op.args[1])
-                binop = getattr(pair(arg1, arg2), op.opname, None)
-                can_only_throw = annmodel.read_can_only_throw(binop, arg1, arg2)
-            elif op.opname in unaryop.UNARY_OPERATIONS:
-                arg1 = self.binding(op.args[0])
-                opname = op.opname
-                if opname == 'contains': opname = 'op_contains'
-                unop = getattr(arg1, opname, None)
-                can_only_throw = annmodel.read_can_only_throw(unop, arg1)
+        if block.canraise:
+            op = block.raising_op
+            s_exception = self.get_exception(op)
+            for link in exits:
+                case = link.exitcase
+                if case is None:
+                    self.follow_link(graph, link, {})
+                    continue
+                if s_exception == s_ImpossibleValue:
+                    break
+                s_case = SomeInstance(self.bookkeeper.getuniqueclassdef(case))
+                s_matching_exc = intersection(s_exception, s_case)
+                if s_matching_exc != s_ImpossibleValue:
+                    self.follow_raise_link(graph, link, s_matching_exc)
+                s_exception = difference(s_exception, s_case)
+        else:
+            if isinstance(block.exitswitch, Variable):
+                knowntypedata = getattr(
+                    block.exitswitch.annotation, "knowntypedata", {})
             else:
-                can_only_throw = None
-
-            if can_only_throw is not None:
-                candidates = can_only_throw
-                candidate_exits = exits
-                exits = []
-                for link in candidate_exits:
-                    case = link.exitcase
-                    if case is None:
-                        exits.append(link)
-                        continue
-                    covered = [c for c in candidates if issubclass(c, case)]
-                    if covered:
-                        exits.append(link)
-                        candidates = [c for c in candidates if c not in covered]
-
-        for link in exits:
-            in_except_block = False
-
-            last_exception_var = link.last_exception # may be None for non-exception link
-            last_exc_value_var = link.last_exc_value # may be None for non-exception link
-
-            if isinstance(link.exitcase, (types.ClassType, type)) \
-                   and issubclass(link.exitcase, py.builtin.BaseException):
-                assert last_exception_var and last_exc_value_var
-                last_exc_value_object = self.bookkeeper.valueoftype(link.exitcase)
-                last_exception_object = annmodel.SomeType()
-                if isinstance(last_exception_var, Constant):
-                    last_exception_object.const = last_exception_var.value
-                last_exception_object.is_type_of = [last_exc_value_var]
-
-                if isinstance(last_exception_var, Variable):
-                    self.setbinding(last_exception_var, last_exception_object)
-                if isinstance(last_exc_value_var, Variable):
-                    self.setbinding(last_exc_value_var, last_exc_value_object)
-
-                last_exception_object = annmodel.SomeType()
-                if isinstance(last_exception_var, Constant):
-                    last_exception_object.const = last_exception_var.value
-                #if link.exitcase is Exception:
-                #    last_exc_value_object = annmodel.SomeObject()
-                #else:
-                last_exc_value_vars = []
-                in_except_block = True
-
-            ignore_link = False
-            cells = []
-            renaming = {}
-            for a,v in zip(link.args,link.target.inputargs):
-                renaming.setdefault(a, []).append(v)
-            for a,v in zip(link.args,link.target.inputargs):
-                if a == last_exception_var:
-                    assert in_except_block
-                    cells.append(last_exception_object)
-                elif a == last_exc_value_var:
-                    assert in_except_block
-                    cells.append(last_exc_value_object)
-                    last_exc_value_vars.append(v)
-                else:
-                    cell = self.binding(a)
-                    if (link.exitcase, a) in knowntypedata:
-                        knownvarvalue = knowntypedata[(link.exitcase, a)]
-                        cell = pair(cell, knownvarvalue).improve()
-                        # ignore links that try to pass impossible values
-                        if cell == annmodel.s_ImpossibleValue:
-                            ignore_link = True
-
-                    if hasattr(cell,'is_type_of'):
-                        renamed_is_type_of = []
-                        for v in cell.is_type_of:
-                            new_vs = renaming.get(v,[])
-                            renamed_is_type_of += new_vs
-                        assert cell.knowntype is type
-                        newcell = annmodel.SomeType()
-                        if cell.is_constant():
-                            newcell.const = cell.const
-                        cell = newcell
-                        cell.is_type_of = renamed_is_type_of
-
-                    if hasattr(cell, 'knowntypedata'):
-                        renamed_knowntypedata = {}
-                        for (value, v), s in cell.knowntypedata.items():
-                            new_vs = renaming.get(v, [])
-                            for new_v in new_vs:
-                                renamed_knowntypedata[value, new_v] = s
-                        assert isinstance(cell, annmodel.SomeBool)
-                        newcell = annmodel.SomeBool()
-                        if cell.is_constant():
-                            newcell.const = cell.const
-                        cell = newcell
-                        cell.set_knowntypedata(renamed_knowntypedata)
-
-                    cells.append(cell)
-
-            if ignore_link:
-                continue
-
-            if in_except_block:
-                last_exception_object.is_type_of = last_exc_value_vars
-
-            self.links_followed[link] = True
-            self.addpendingblock(graph, link.target, cells)
+                knowntypedata = {}
+            for link in exits:
+                constraints = knowntypedata.get(link.exitcase, {})
+                self.follow_link(graph, link, constraints)
 
         if block in self.notify:
             # reflow from certain positions when this block is done
@@ -579,79 +528,98 @@ class RPythonAnnotator(object):
                     callback()
 
 
+    def follow_link(self, graph, link, constraints):
+        assert not (isinstance(link.exitcase, (types.ClassType, type)) and
+                issubclass(link.exitcase, BaseException))
+
+        ignore_link = False
+        inputs_s = []
+        renaming = defaultdict(list)
+        for v_out, v_input in zip(link.args, link.target.inputargs):
+            renaming[v_out].append(v_input)
+
+        for v_out in link.args:
+            s_out = self.annotation(v_out)
+            if v_out in constraints:
+                s_constraint = constraints[v_out]
+                s_out = pair(s_out, s_constraint).improve()
+                # ignore links that try to pass impossible values
+                if s_out == s_ImpossibleValue:
+                    ignore_link = True
+            s_out = self.apply_renaming(s_out, renaming)
+            inputs_s.append(s_out)
+        if ignore_link:
+            return
+
+        self.links_followed[link] = True
+        self.addpendingblock(graph, link.target, inputs_s)
+
+    def follow_raise_link(self, graph, link, s_last_exc_value):
+        v_last_exc_type = link.last_exception
+        v_last_exc_value = link.last_exc_value
+
+        assert (isinstance(link.exitcase, (types.ClassType, type)) and
+                issubclass(link.exitcase, BaseException))
+
+        assert v_last_exc_type and v_last_exc_value
+
+        if isinstance(v_last_exc_value, Variable):
+            self.setbinding(v_last_exc_value, s_last_exc_value)
+
+        if isinstance(v_last_exc_type, Variable):
+            self.setbinding(v_last_exc_type, typeof([v_last_exc_value]))
+
+        inputs_s = []
+        renaming = defaultdict(list)
+        for v_out, v_input in zip(link.args, link.target.inputargs):
+            renaming[v_out].append(v_input)
+
+        for v_out, v_input in zip(link.args, link.target.inputargs):
+            if v_out == v_last_exc_type:
+                s_out = typeof(renaming[v_last_exc_value])
+                if isinstance(v_last_exc_type, Constant):
+                    s_out.const = v_last_exc_type.value
+                elif v_last_exc_type.annotation.is_constant():
+                    s_out.const = v_last_exc_type.annotation.const
+                inputs_s.append(s_out)
+            else:
+                s_out = self.annotation(v_out)
+                s_out = self.apply_renaming(s_out, renaming)
+                inputs_s.append(s_out)
+
+        self.links_followed[link] = True
+        self.addpendingblock(graph, link.target, inputs_s)
+
     #___ creating the annotations based on operations ______
 
-    def consider_op(self, block, opindex):
-        op = block.operations[opindex]
-        argcells = [self.binding(a) for a in op.args]
-        consider_meth = getattr(self,'consider_op_'+op.opname,
-                                None)
-        if not consider_meth:
-            raise Exception,"unknown op: %r" % op
-
+    def consider_op(self, op):
         # let's be careful about avoiding propagated SomeImpossibleValues
         # to enter an op; the latter can result in violations of the
         # more general results invariant: e.g. if SomeImpossibleValue enters is_
         #  is_(SomeImpossibleValue, None) -> SomeBool
         #  is_(SomeInstance(not None), None) -> SomeBool(const=False) ...
         # boom -- in the assert of setbinding()
-        for arg in argcells:
-            if isinstance(arg, annmodel.SomeImpossibleValue):
-                raise BlockedInference(self, op, opindex)
-        try:
-            resultcell = consider_meth(*argcells)
-        except annmodel.AnnotatorError as e: # note that UnionError is a subclass
-            graph = self.bookkeeper.position_key[0]
-            e.source = gather_error(self, graph, block, opindex)
-            raise
+        for arg in op.args:
+            if isinstance(self.annotation(arg), annmodel.SomeImpossibleValue):
+                raise BlockedInference(self, op, -1)
+        resultcell = op.consider(self)
         if resultcell is None:
-            resultcell = self.noreturnvalue(op)
-        elif resultcell == annmodel.s_ImpossibleValue:
-            raise BlockedInference(self, op, opindex) # the operation cannot succeed
+            resultcell = s_ImpossibleValue
+        elif resultcell == s_ImpossibleValue:
+            raise BlockedInference(self, op, -1) # the operation cannot succeed
         assert isinstance(resultcell, annmodel.SomeObject)
         assert isinstance(op.result, Variable)
         self.setbinding(op.result, resultcell)  # bind resultcell to op.result
 
-    def noreturnvalue(self, op):
-        return annmodel.s_ImpossibleValue  # no return value (hook method)
-
-    # XXX "contains" clash with SomeObject method
-    def consider_op_contains(self, seq, elem):
-        self.bookkeeper.count("contains", seq)
-        return seq.op_contains(elem)
-
-    def consider_op_newtuple(self, *args):
-        return annmodel.SomeTuple(items = args)
-
-    def consider_op_newlist(self, *args):
-        return self.bookkeeper.newlist(*args)
-
-    def consider_op_newdict(self):
-        return self.bookkeeper.newdict()
-
-
-    def _registeroperations(cls, unary_ops, binary_ops):
-        # All unary operations
-        d = {}
-        for opname in unary_ops:
-            fnname = 'consider_op_' + opname
-            exec py.code.Source("""
-def consider_op_%s(self, arg, *args):
-    return arg.%s(*args)
-""" % (opname, opname)).compile() in globals(), d
-            setattr(cls, fnname, d[fnname])
-        # All binary operations
-        for opname in binary_ops:
-            fnname = 'consider_op_' + opname
-            exec py.code.Source("""
-def consider_op_%s(self, arg1, arg2, *args):
-    return pair(arg1,arg2).%s(*args)
-""" % (opname, opname)).compile() in globals(), d
-            setattr(cls, fnname, d[fnname])
-    _registeroperations = classmethod(_registeroperations)
-
-# register simple operations handling
-RPythonAnnotator._registeroperations(unaryop.UNARY_OPERATIONS, binaryop.BINARY_OPERATIONS)
+    def get_exception(self, operation):
+        """
+        Return the annotation for all exceptions that `operation` may raise.
+        """
+        can_only_throw = operation.get_can_only_throw(self)
+        if can_only_throw is None:
+            return SomeInstance(self.bookkeeper.getuniqueclassdef(Exception))
+        else:
+            return self.bookkeeper.new_exception(can_only_throw)
 
 
 class BlockedInference(Exception):
