@@ -11,7 +11,7 @@ from rpython.annotator.model import (
     SomeBuiltinMethod, SomeIterator, SomePBC, SomeNone, SomeFloat, s_None,
     SomeByteArray, SomeWeakRef, SomeSingleFloat,
     SomeLongFloat, SomeType, SomeTypeOf, SomeConstantType, unionof, UnionError,
-    union, read_can_only_throw, add_knowntypedata,
+    read_can_only_throw, add_knowntypedata,
     merge_knowntypedata,)
 from rpython.annotator.bookkeeper import immutablevalue, getbookkeeper
 from rpython.flowspace.model import Variable, Constant, const
@@ -381,14 +381,16 @@ class __extend__(pairtype(SomeChar, SomeChar)):
 class __extend__(pairtype(SomeChar, SomeUnicodeCodePoint),
                  pairtype(SomeUnicodeCodePoint, SomeChar)):
     def union((uchr1, uchr2)):
-        no_nul = uchr1.no_nul and uchr2.no_nul
-        return SomeUnicodeCodePoint(no_nul=no_nul)
+        return SomeUnicodeCodePoint()
 
 class __extend__(pairtype(SomeUnicodeCodePoint, SomeUnicodeCodePoint)):
     def union((uchr1, uchr2)):
         no_nul = uchr1.no_nul and uchr2.no_nul
         return SomeUnicodeCodePoint(no_nul=no_nul)
 
+    def add((chr1, chr2)):
+        return SomeUnicodeString()
+
 class __extend__(pairtype(SomeString, SomeUnicodeString),
                  pairtype(SomeUnicodeString, SomeString)):
     def mod((str, unistring)):
@@ -512,13 +514,13 @@ class __extend__(pairtype(SomeTuple, SomeTuple)):
     ne = eq
 
     def lt((tup1, tup2)):
-        raise AnnotatorError("unsupported: (...) < (...)")
+        raise Exception("unsupported: (...) < (...)")
     def le((tup1, tup2)):
-        raise AnnotatorError("unsupported: (...) <= (...)")
+        raise Exception("unsupported: (...) <= (...)")
     def gt((tup1, tup2)):
-        raise AnnotatorError("unsupported: (...) > (...)")
+        raise Exception("unsupported: (...) > (...)")
     def ge((tup1, tup2)):
-        raise AnnotatorError("unsupported: (...) >= (...)")
+        raise Exception("unsupported: (...) >= (...)")
 
 
 class __extend__(pairtype(SomeDict, SomeDict)):
@@ -527,8 +529,6 @@ class __extend__(pairtype(SomeDict, SomeDict)):
         assert dic1.__class__ == dic2.__class__
         return dic1.__class__(dic1.dictdef.union(dic2.dictdef))
 
-    def ne((dic1, dic2)):
-        raise AnnotatorError("dict != dict not implemented")
 
 def _dict_can_only_throw_keyerror(s_dct, *ignore):
     if s_dct.dictdef.dictkey.custom_eq_hash:
@@ -703,13 +703,13 @@ class __extend__(
         pairtype(SomeException, SomeInstance),
         pairtype(SomeException, SomeNone)):
     def union((s_exc, s_inst)):
-        return union(s_exc.as_SomeInstance(), s_inst)
+        return unionof(s_exc.as_SomeInstance(), s_inst)
 
 class __extend__(
         pairtype(SomeInstance, SomeException),
         pairtype(SomeNone, SomeException)):
     def union((s_inst, s_exc)):
-        return union(s_exc.as_SomeInstance(), s_inst)
+        return unionof(s_exc.as_SomeInstance(), s_inst)
 
 class __extend__(pairtype(SomeException, SomeException)):
     def union((s_exc1, s_exc2)):
@@ -727,10 +727,6 @@ def setitem_SomeInstance(annotator, v_ins, v_idx, v_value):
     return [get_setitem,
             op.simple_call(get_setitem.result, v_idx, v_value)]
 
-@op.contains.register_transform(SomeInstance)
-def contains_SomeInstance(annotator, v_ins, v_idx):
-    get_contains = op.getattr(v_ins, const('__contains__'))
-    return [get_contains, op.simple_call(get_contains.result, v_idx)]
 
 class __extend__(pairtype(SomeIterator, SomeIterator)):
 
