@@ -84,8 +84,6 @@ def void(self, op, fcond):
     return []
 
 class ARMRegisterManager(RegisterManager):
-    FORBID_TEMP_BOXES = True
-
     def return_constant(self, v, forbidden_vars=[], selected_reg=None):
         self._check_type(v)
         if isinstance(v, Const):
@@ -96,7 +94,7 @@ class ARMRegisterManager(RegisterManager):
             else:
                 tp = INT
             loc = self.get_scratch_reg(tp,
-                    forbidden_vars,
+                    self.temp_boxes + forbidden_vars,
                     selected_reg=selected_reg)
             immvalue = self.convert_to_imm(v)
             self.assembler.load(loc, immvalue)
@@ -131,9 +129,9 @@ class VFPRegisterManager(ARMRegisterManager):
     def get_scratch_reg(self, type=FLOAT, forbidden_vars=[], selected_reg=None):
         assert type == FLOAT  # for now
         box = TempFloat()
+        self.temp_boxes.append(box)
         reg = self.force_allocate_reg(box, forbidden_vars=forbidden_vars,
                                                     selected_reg=selected_reg)
-        self.temp_boxes.append(box)
         return reg
 
 
@@ -166,9 +164,9 @@ class CoreRegisterManager(ARMRegisterManager):
             box = TempInt()
         else:
             box = TempPtr()
+        self.temp_boxes.append(box)
         reg = self.force_allocate_reg(box, forbidden_vars=forbidden_vars,
                                                     selected_reg=selected_reg)
-        self.temp_boxes.append(box)
         return reg
 
     def get_free_reg(self):
@@ -307,8 +305,9 @@ class Regalloc(BaseRegalloc):
         operations = cpu.gc_ll_descr.rewrite_assembler(cpu, operations,
                                                        allgcrefs)
         # compute longevity of variables
-        longevity = compute_vars_longevity(inputargs, operations)
+        longevity, last_real_usage = compute_vars_longevity(inputargs, operations)
         self.longevity = longevity
+        self.last_real_usage = last_real_usage
         fm = self.frame_manager
         asm = self.assembler
         self.vfprm = VFPRegisterManager(longevity, fm, asm)
@@ -373,10 +372,6 @@ class Regalloc(BaseRegalloc):
             if box.type == REF and self.rm.is_still_alive(box):
                 assert not noregs
                 assert loc.is_core_reg()
-                #val = self.cpu.all_reg_indexes[loc.value]
-                # ^^^ That is the correct way to write it down, but as a
-                #     special case in the arm backend only, this is equivalent
-                #     to just the line below:
                 val = loc.value
                 gcmap[val // WORD // 8] |= r_uint(1) << (val % (WORD * 8))
         for box, loc in self.fm.bindings.iteritems():
@@ -857,6 +852,7 @@ class Regalloc(BaseRegalloc):
         assert boxes[3].getint() == 1    # scale
         ofs = boxes[4].getint()
         size = boxes[5].getint()
+        assert check_imm_arg(ofs)
         return [value_loc, base_loc, index_loc, imm(size), imm(ofs)]
 
     def _prepare_op_gc_load_indexed(self, op, fcond):
@@ -866,6 +862,7 @@ class Regalloc(BaseRegalloc):
         assert boxes[2].getint() == 1    # scale
         ofs = boxes[3].getint()
         nsize = boxes[4].getint()
+        assert check_imm_arg(ofs)
         self.possibly_free_vars_for_op(op)
         self.free_temp_vars()
         res_loc = self.force_allocate_reg(op)
@@ -875,6 +872,8 @@ class Regalloc(BaseRegalloc):
     prepare_op_gc_load_indexed_r = _prepare_op_gc_load_indexed
     prepare_op_gc_load_indexed_f = _prepare_op_gc_load_indexed
 
+    prepare_op_copystrcontent = void
+    prepare_op_copyunicodecontent = void
     prepare_op_zero_array = void
 
     def _prepare_op_same_as(self, op, fcond):
@@ -899,13 +898,6 @@ class Regalloc(BaseRegalloc):
         resloc = self.force_allocate_reg(op)
         return [resloc]
 
-    def prepare_op_load_effective_address(self, op, fcond):
-        args = op.getarglist()
-        arg0 = self.make_sure_var_in_reg(args[0], args)
-        arg1 = self.make_sure_var_in_reg(args[1], args)
-        res = self.force_allocate_reg(op)
-        return [arg0, arg1, res]
-
     def prepare_op_call_malloc_nursery(self, op, fcond):
         size_box = op.getarg(0)
         assert isinstance(size_box, ConstInt)
@@ -1012,6 +1004,7 @@ class Regalloc(BaseRegalloc):
     def prepare_op_cond_call(self, op, fcond):
         # XXX don't force the arguments to be loaded in specific
         # locations before knowing if we can take the fast path
+        # XXX add cond_call_value support
         assert 2 <= op.numargs() <= 4 + 2
         tmpreg = self.get_scratch_reg(INT, selected_reg=r.r4)
         v = op.getarg(1)
@@ -1024,33 +1017,8 @@ class Regalloc(BaseRegalloc):
             arg = op.getarg(i)
             self.make_sure_var_in_reg(arg, args_so_far, selected_reg=reg)
             args_so_far.append(arg)
-
-        if op.type == 'v':
-            # a plain COND_CALL.  Calls the function when args[0] is
-            # true.  Often used just after a comparison operation.
-            self.load_condition_into_cc(op.getarg(0))
-            return [tmpreg]
-        else:
-            # COND_CALL_VALUE_I/R.  Calls the function when args[0]
-            # is equal to 0 or NULL.  Returns the result from the
-            # function call if done, or args[0] if it was not 0/NULL.
-            # Implemented by forcing the result to live in the same
-            # register as args[0], and overwriting it if we really do
-            # the call.
-
-            # Load the register for the result.  Possibly reuse 'args[0]'.
-            # But the old value of args[0], if it survives, is first
-            # spilled away.  We can't overwrite any of op.args[2:] here.
-            args = op.getarglist()
-            resloc = self.rm.force_result_in_reg(op, args[0],
-                                                 forbidden_vars=args[2:])
-            # Test the register for the result.
-            self.assembler.mc.CMP_ri(resloc.value, 0)
-            self.assembler.guard_success_cc = c.EQ
-            return [tmpreg, resloc]
-
-    prepare_op_cond_call_value_i = prepare_op_cond_call
-    prepare_op_cond_call_value_r = prepare_op_cond_call
+        self.load_condition_into_cc(op.getarg(0))
+        return [tmpreg]
 
     def prepare_op_force_token(self, op, fcond):
         # XXX for now we return a regular reg
@@ -1070,7 +1038,7 @@ class Regalloc(BaseRegalloc):
         position = self.rm.position
         for arg in inputargs:
             assert not isinstance(arg, Const)
-            if self.longevity[arg].is_last_real_use_before(position):
+            if self.last_real_usage.get(arg, -1) <= position:
                 self.force_spill_var(arg)
 
         #
