@@ -51,28 +51,28 @@ def _do_call(cpu, metainterp, argboxes, descr, rettype):
     if rettype == INT:
         try:
             result = cpu.bh_call_i(func, args_i, args_r, args_f, descr)
-        except Exception, e:
+        except Exception as e:
             metainterp.execute_raised(e)
             result = 0
         return result
     if rettype == REF:
         try:
             result = cpu.bh_call_r(func, args_i, args_r, args_f, descr)
-        except Exception, e:
+        except Exception as e:
             metainterp.execute_raised(e)
             result = NULL
         return result
     if rettype == FLOAT:
         try:
             result = cpu.bh_call_f(func, args_i, args_r, args_f, descr)
-        except Exception, e:
+        except Exception as e:
             metainterp.execute_raised(e)
             result = longlong.ZEROF
         return result
     if rettype == VOID:
         try:
             cpu.bh_call_v(func, args_i, args_r, args_f, descr)
-        except Exception, e:
+        except Exception as e:
             metainterp.execute_raised(e)
         return None
     raise AssertionError("bad rettype")
@@ -101,6 +101,18 @@ def do_cond_call(cpu, metainterp, argboxes, descr):
     if condbox.getint():
         do_call_n(cpu, metainterp, argboxes[1:], descr)
 
+def do_cond_call_value_i(cpu, metainterp, argboxes, descr):
+    value = argboxes[0].getint()
+    if value == 0:
+        value = do_call_i(cpu, metainterp, argboxes[1:], descr)
+    return value
+
+def do_cond_call_value_r(cpu, metainterp, argboxes, descr):
+    value = argboxes[0].getref_base()
+    if not value:
+        value = do_call_r(cpu, metainterp, argboxes[1:], descr)
+    return value
+
 def do_getarrayitem_gc_i(cpu, _, arraybox, indexbox, arraydescr):
     array = arraybox.getref_base()
     index = indexbox.getint()
@@ -366,6 +378,8 @@ def _make_execute_list():
                          rop.CALL_ASSEMBLER_I,
                          rop.CALL_ASSEMBLER_N,
                          rop.INCREMENT_DEBUG_COUNTER,
+                         rop.COND_CALL_VALUE_R,
+                         rop.COND_CALL_VALUE_I,
                          rop.COND_CALL_GC_WB,
                          rop.COND_CALL_GC_WB_ARRAY,
                          rop.ZERO_ARRAY,
@@ -379,7 +393,7 @@ def _make_execute_list():
                          rop.CALL_RELEASE_GIL_F,
                          rop.CALL_RELEASE_GIL_N,
                          rop.QUASIIMMUT_FIELD,
-                         rop.CALL_MALLOC_GC,
+                         rop.CHECK_MEMORY_ERROR,
                          rop.CALL_MALLOC_NURSERY,
                          rop.CALL_MALLOC_NURSERY_VARSIZE,
                          rop.CALL_MALLOC_NURSERY_VARSIZE_FRAME,
@@ -393,19 +407,13 @@ def _make_execute_list():
                          rop.SAVE_EXC_CLASS,
                          rop.SAVE_EXCEPTION,
                          rop.RESTORE_EXCEPTION,
-                         rop.VEC_RAW_LOAD_I,
-                         rop.VEC_RAW_LOAD_F,
-                         rop.VEC_RAW_STORE,
-                         rop.VEC_GETARRAYITEM_RAW_I,
-                         rop.VEC_GETARRAYITEM_RAW_F,
-                         rop.VEC_SETARRAYITEM_RAW,
-                         rop.VEC_GETARRAYITEM_GC_I,
-                         rop.VEC_GETARRAYITEM_GC_F,
-                         rop.VEC_SETARRAYITEM_GC,
+                         rop.VEC_LOAD_I,
+                         rop.VEC_LOAD_F,
                          rop.GC_LOAD_I,
                          rop.GC_LOAD_R,
                          rop.GC_LOAD_F,
                          rop.GC_LOAD_INDEXED_R,
+                         rop.VEC_STORE,
                          rop.GC_STORE,
                          rop.GC_STORE_INDEXED,
                          rop.LOAD_FROM_GC_TABLE,
