@@ -1,10 +1,9 @@
-from rpython.jit.metainterp.history import Const, ConstInt, ConstPtr
+from rpython.jit.metainterp.history import Const, ConstInt
 from rpython.jit.metainterp.history import FrontendOp, RefFrontendOp
-from rpython.jit.metainterp.history import new_ref_dict
 from rpython.jit.metainterp.resoperation import rop, OpHelpers
 from rpython.jit.metainterp.executor import constant_from_op
 from rpython.rlib.rarithmetic import r_uint32, r_uint
-from rpython.rlib.objectmodel import always_inline, specialize
+from rpython.rlib.objectmodel import always_inline
 
 """ A big note: we don't do heap caches on Consts, because it used
 to be done with the identity of the Const instance. This gives very wonky
@@ -62,19 +61,12 @@ class CacheEntry(object):
         # on writes to the field.
         self.quasiimmut_seen = None
 
-        # set of refs for *constants* that we've seen a quasi-immut field on.
-        self.quasiimmut_seen_refs = None
-
-        self.last_const_box = None
-
     def _clear_cache_on_write(self, seen_allocation_of_target):
         if not seen_allocation_of_target:
             self.cache_seen_allocation.clear()
         self.cache_anything.clear()
         if self.quasiimmut_seen is not None:
             self.quasiimmut_seen.clear()
-        if self.quasiimmut_seen_refs is not None:
-            self.quasiimmut_seen_refs.clear()
 
     def _seen_alloc(self, ref_box):
         if not isinstance(ref_box, RefFrontendOp):
@@ -88,23 +80,11 @@ class CacheEntry(object):
             return self.cache_anything
 
     def do_write_with_aliasing(self, ref_box, fieldbox):
-        ref_box = self._unique_const_heuristic(ref_box)
         seen_alloc = self._seen_alloc(ref_box)
         self._clear_cache_on_write(seen_alloc)
         self._getdict(seen_alloc)[ref_box] = fieldbox
 
-    def _unique_const_heuristic(self, ref_box):
-        # we don't want a dict lookup for every constptr, but comparing against
-        # the last used constptr for the current descr is often enough
-        if isinstance(ref_box, ConstPtr):
-            if self.last_const_box and self.last_const_box.same_constant(ref_box):
-                ref_box = self.last_const_box
-            else:
-                self.last_const_box = ref_box
-        return ref_box
-
     def read(self, ref_box):
-        ref_box = self._unique_const_heuristic(ref_box)
         dict = self._getdict(self._seen_alloc(ref_box))
         try:
             res_box = dict[ref_box]
@@ -113,7 +93,6 @@ class CacheEntry(object):
         return maybe_replace_with_const(res_box)
 
     def read_now_known(self, ref_box, fieldbox):
-        ref_box = self._unique_const_heuristic(ref_box)
         self._getdict(self._seen_alloc(ref_box))[ref_box] = fieldbox
 
     def invalidate_unescaped(self):
@@ -121,8 +100,6 @@ class CacheEntry(object):
         self._invalidate_unescaped(self.cache_seen_allocation)
         if self.quasiimmut_seen is not None:
             self.quasiimmut_seen.clear()
-        if self.quasiimmut_seen_refs is not None:
-            self.quasiimmut_seen_refs.clear()
 
     def _invalidate_unescaped(self, d):
         for ref_box in d.keys():
@@ -185,12 +162,6 @@ class HeapCache(object):
         # heap array cache
         # maps descrs to {index: CacheEntry} dicts
         self.heap_array_cache = {}
-        self.need_guard_not_invalidated = True
-
-        # result of one loop invariant call
-        self.loop_invariant_result = None
-        self.loop_invariant_descr = None
-        self.loop_invariant_arg0int = -1
 
     def reset_keep_likely_virtuals(self):
         # Update only 'head_version', but 'likely_virtual_version' remains
@@ -220,18 +191,9 @@ class HeapCache(object):
             ref_frontend_op._set_heapc_flags(f)
             ref_frontend_op._heapc_deps = None
 
-    def invalidate_caches_varargs(self, opnum, descr, argboxes):
-        self.mark_escaped_varargs(opnum, descr, argboxes)
-        if self.clear_caches_not_necessary(opnum, descr):
-            return
-        self.clear_caches_varargs(opnum, descr, argboxes)
-
-    @specialize.arg(1)
-    def invalidate_caches(self, opnum, descr, *argboxes):
-        self.mark_escaped(opnum, descr, *argboxes)
-        if self.clear_caches_not_necessary(opnum, descr):
-            return
-        self.clear_caches(opnum, descr, *argboxes)
+    def invalidate_caches(self, opnum, descr, argboxes):
+        self.mark_escaped(opnum, descr, argboxes)
+        self.clear_caches(opnum, descr, argboxes)
 
     def _escape_from_write(self, box, fieldbox):
         if self.is_unescaped(box) and self.is_unescaped(fieldbox):
@@ -240,9 +202,7 @@ class HeapCache(object):
         elif fieldbox is not None:
             self._escape_box(fieldbox)
 
-    @specialize.arg(1)
-    def mark_escaped(self, opnum, descr, *argboxes):
-        assert opnum != rop.CALL_N
+    def mark_escaped(self, opnum, descr, argboxes):
         if opnum == rop.SETFIELD_GC:
             assert len(argboxes) == 2
             box, fieldbox = argboxes
@@ -251,37 +211,8 @@ class HeapCache(object):
             assert len(argboxes) == 3
             box, indexbox, fieldbox = argboxes
             self._escape_from_write(box, fieldbox)
-        # these operations don't escape their arguments
-        elif (opnum != rop.GETFIELD_GC_R and
-              opnum != rop.GETFIELD_GC_I and
-              opnum != rop.GETFIELD_GC_F and
-              opnum != rop.PTR_EQ and
-              opnum != rop.PTR_NE and
-              opnum != rop.INSTANCE_PTR_EQ and
-              opnum != rop.INSTANCE_PTR_NE and
-              opnum != rop.ASSERT_NOT_NONE):
-            self._escape_argboxes(*argboxes)
-
-    def _escape_argboxes(self, *argboxes):
-        if len(argboxes) == 0:
-            return
-        self._escape_box(argboxes[0])
-        self._escape_argboxes(*argboxes[1:])
-
-    def mark_escaped_varargs(self, opnum, descr, argboxes):
-        # these opnums should never reach here, they should always end up in
-        # the mark_escaped variant
-        assert (opnum != rop.SETFIELD_GC and
-                opnum != rop.SETARRAYITEM_GC and
-                opnum != rop.GETFIELD_GC_R and
-                opnum != rop.GETFIELD_GC_I and
-                opnum != rop.GETFIELD_GC_F and
-                opnum != rop.PTR_EQ and
-                opnum != rop.PTR_NE and
-                opnum != rop.INSTANCE_PTR_EQ and
-                opnum != rop.INSTANCE_PTR_NE and
-                opnum != rop.ASSERT_NOT_NONE)
-        if (opnum == rop.CALL_N and
+        elif ((opnum == rop.CALL_R or opnum == rop.CALL_I or
+               opnum == rop.CALL_N or opnum == rop.CALL_F) and
               descr.get_extra_info().oopspecindex == descr.get_extra_info().OS_ARRAYCOPY and
               isinstance(argboxes[3], ConstInt) and
               isinstance(argboxes[4], ConstInt) and
@@ -291,16 +222,15 @@ class HeapCache(object):
             # its argument
             # XXX really?
             pass
-        elif (opnum == rop.CALL_N and
-              descr.get_extra_info().oopspecindex == descr.get_extra_info().OS_ARRAYMOVE and
-              isinstance(argboxes[2], ConstInt) and
-              isinstance(argboxes[3], ConstInt) and
-              isinstance(argboxes[4], ConstInt) and
-              descr.get_extra_info().single_write_descr_array is not None):
-            # ARRAYMOVE with constant starts and constant length doesn't escape
-            # its argument
-            pass
-        else:
+        # GETFIELD_GC, PTR_EQ, and PTR_NE don't escape their
+        # arguments
+        elif (opnum != rop.GETFIELD_GC_R and
+              opnum != rop.GETFIELD_GC_I and
+              opnum != rop.GETFIELD_GC_F and
+              opnum != rop.PTR_EQ and
+              opnum != rop.PTR_NE and
+              opnum != rop.INSTANCE_PTR_EQ and
+              opnum != rop.INSTANCE_PTR_NE):
             for box in argboxes:
                 self._escape_box(box)
 
@@ -320,8 +250,7 @@ class HeapCache(object):
                     for i in range(1, len(deps)):
                         self._escape_box(deps[i])
 
-    @specialize.arg_or_var(1)
-    def clear_caches_not_necessary(self, opnum, descr):
+    def clear_caches(self, opnum, descr, argboxes):
         if (opnum == rop.SETFIELD_GC or
             opnum == rop.SETARRAYITEM_GC or
             opnum == rop.SETFIELD_RAW or
@@ -334,24 +263,14 @@ class HeapCache(object):
             opnum == rop.SETFIELD_RAW or
             opnum == rop.SETARRAYITEM_RAW or
             opnum == rop.SETINTERIORFIELD_RAW or
-            opnum == rop.RECORD_EXACT_CLASS or
-            opnum == rop.RAW_STORE or
-            opnum == rop.ASSERT_NOT_NONE):
-            return True
+            opnum == rop.RAW_STORE):
+            return
         if (rop._OVF_FIRST <= opnum <= rop._OVF_LAST or
             rop._NOSIDEEFFECT_FIRST <= opnum <= rop._NOSIDEEFFECT_LAST or
             rop._GUARD_FIRST <= opnum <= rop._GUARD_LAST):
-            return True
-        return False
-
-    def clear_caches(self, opnum, descr, *argboxes):
-        self.clear_caches_varargs(opnum, descr, list(argboxes))
-
-    def clear_caches_varargs(self, opnum, descr, argboxes):
-        self.need_guard_not_invalidated = True # can do better, but good start
+            return
         if (OpHelpers.is_plain_call(opnum) or
             OpHelpers.is_call_loopinvariant(opnum) or
-            OpHelpers.is_cond_call_value(opnum) or
             opnum == rop.COND_CALL):
             effectinfo = descr.get_extra_info()
             ef = effectinfo.extraeffect
@@ -365,9 +284,6 @@ class HeapCache(object):
             elif effectinfo.oopspecindex == effectinfo.OS_ARRAYCOPY:
                 self._clear_caches_arraycopy(opnum, descr, argboxes, effectinfo)
                 return
-            elif effectinfo.oopspecindex == effectinfo.OS_ARRAYMOVE:
-                self._clear_caches_arraymove(opnum, descr, argboxes, effectinfo)
-                return
             else:
                 # Only invalidate things that are escaped
                 # XXX can do better, only do it for the descrs in the effectinfo
@@ -384,54 +300,29 @@ class HeapCache(object):
         # above, but hit an assertion in "pypy test_multiprocessing.py".
         self.reset_keep_likely_virtuals()
 
-    def _clear_caches_arraycopy(self, opnum, descr, argboxes, effectinfo):
-        self._clear_caches_arrayop(argboxes[1], argboxes[2],
-                                   argboxes[3], argboxes[4], argboxes[5],
-                                   effectinfo)
-
-    def _clear_caches_arraymove(self, opnum, descr, argboxes, effectinfo):
-        self._clear_caches_arrayop(argboxes[1], argboxes[1],
-                                   argboxes[2], argboxes[3], argboxes[4],
-                                   effectinfo)
-
-    def _clear_caches_arrayop(self, source_box, dest_box,
-                              source_start_box, dest_start_box, length_box,
-                              effectinfo):
-        seen_allocation_of_target = self._check_flag(dest_box,
-                                                     HF_SEEN_ALLOCATION)
+    def _clear_caches_arraycopy(self, opnum, desrc, argboxes, effectinfo):
+        seen_allocation_of_target = self._check_flag(
+                                            argboxes[2], HF_SEEN_ALLOCATION)
         if (
-            isinstance(source_start_box, ConstInt) and
-            isinstance(dest_start_box, ConstInt) and
-            isinstance(length_box, ConstInt) and
+            isinstance(argboxes[3], ConstInt) and
+            isinstance(argboxes[4], ConstInt) and
+            isinstance(argboxes[5], ConstInt) and
             effectinfo.single_write_descr_array is not None
         ):
             descr = effectinfo.single_write_descr_array
             cache = self.heap_array_cache.get(descr, None)
-            srcstart = source_start_box.getint()
-            dststart = dest_start_box.getint()
-            length = length_box.getint()
-
-            index_current = 0
-            index_delta = +1
-            index_stop = length
-            if srcstart < dststart:   # iterate in reverse order (for ARRAYMOVE)
-                index_current = index_stop - 1
-                index_delta = -1
-                index_stop = -1
-
-            while index_current != index_stop:
-                i = index_current
-                index_current += index_delta
-                assert i >= 0
-
+            srcstart = argboxes[3].getint()
+            dststart = argboxes[4].getint()
+            length = argboxes[5].getint()
+            for i in xrange(length):
                 value = self.getarrayitem(
-                    source_box,
+                    argboxes[1],
                     ConstInt(srcstart + i),
                     descr,
                 )
                 if value is not None:
                     self.setarrayitem(
-                        dest_box,
+                        argboxes[2],
                         ConstInt(dststart + i),
                         value,
                         descr,
@@ -479,7 +370,7 @@ class HeapCache(object):
     def class_now_known(self, box):
         if isinstance(box, Const):
             return
-        self._set_flag(box, HF_KNOWN_CLASS | HF_KNOWN_NULLITY)
+        self._set_flag(box, HF_KNOWN_CLASS)
 
     def is_nullity_known(self, box):
         if isinstance(box, Const):
@@ -491,12 +382,10 @@ class HeapCache(object):
             return
         self._set_flag(box, HF_KNOWN_NULLITY)
 
-    def is_known_nonstandard_virtualizable(self, box):
-        return self._check_flag(box, HF_NONSTD_VABLE) or self._check_flag(box, HF_SEEN_ALLOCATION)
+    def is_nonstandard_virtualizable(self, box):
+        return self._check_flag(box, HF_NONSTD_VABLE)
 
     def nonstandard_virtualizables_now_known(self, box):
-        if isinstance(box, Const):
-            return
         self._set_flag(box, HF_NONSTD_VABLE)
 
     def is_unescaped(self, box):
@@ -511,17 +400,10 @@ class HeapCache(object):
     def new(self, box):
         assert isinstance(box, RefFrontendOp)
         self.update_version(box)
-        add_flags(box, HF_LIKELY_VIRTUAL | HF_SEEN_ALLOCATION | HF_IS_UNESCAPED
-                       | HF_KNOWN_NULLITY)
+        add_flags(box, HF_LIKELY_VIRTUAL | HF_SEEN_ALLOCATION | HF_IS_UNESCAPED)
 
     def new_array(self, box, lengthbox):
-        assert isinstance(box, RefFrontendOp)
-        self.update_version(box)
-        flags = HF_SEEN_ALLOCATION | HF_KNOWN_NULLITY
-        if isinstance(lengthbox, Const):
-            # only constant-length arrays are virtuals
-            flags |= HF_LIKELY_VIRTUAL | HF_IS_UNESCAPED
-        add_flags(box, flags)
+        self.new(box)
         self.arraylen_now_known(box, lengthbox)
 
     def getfield(self, box, descr):
@@ -531,6 +413,8 @@ class HeapCache(object):
         return None
 
     def get_field_updater(self, box, descr):
+        if not isinstance(box, RefFrontendOp):
+            return dummy_field_updater
         cache = self.heap_cache.get(descr, None)
         if cache is None:
             cache = self.heap_cache[descr] = CacheEntry(self)
@@ -611,37 +495,15 @@ class HeapCache(object):
 
     def is_quasi_immut_known(self, fielddescr, box):
         cache = self.heap_cache.get(fielddescr, None)
-        if cache is not None:
-            if isinstance(box, Const):
-                if cache.quasiimmut_seen_refs is not None:
-                    return box.getref_base() in cache.quasiimmut_seen_refs
-            else:
-                if cache.quasiimmut_seen is not None:
-                    return box in cache.quasiimmut_seen
+        if cache is not None and cache.quasiimmut_seen is not None:
+            return box in cache.quasiimmut_seen
         return False
 
     def quasi_immut_now_known(self, fielddescr, box):
         cache = self.heap_cache.get(fielddescr, None)
         if cache is None:
             cache = self.heap_cache[fielddescr] = CacheEntry(self)
-        if isinstance(box, Const):
-            if cache.quasiimmut_seen_refs is None:
-                cache.quasiimmut_seen_refs = new_ref_dict()
-            cache.quasiimmut_seen_refs[box.getref_base()] = None
+        if cache.quasiimmut_seen is not None:
+            cache.quasiimmut_seen[box] = None
         else:
-            if cache.quasiimmut_seen is not None:
-                cache.quasiimmut_seen[box] = None
-            else:
-                cache.quasiimmut_seen = {box: None}
-
-    def call_loopinvariant_known_result(self, allboxes, descr):
-        if self.loop_invariant_descr is not descr:
-            return None
-        if self.loop_invariant_arg0int != allboxes[0].getint():
-            return None
-        return self.loop_invariant_result
-
-    def call_loopinvariant_now_known(self, allboxes, descr, res):
-        self.loop_invariant_descr = descr
-        self.loop_invariant_arg0int = allboxes[0].getint()
-        self.loop_invariant_result = res
+            cache.quasiimmut_seen = {box: None}
