@@ -2,8 +2,7 @@ from rpython.jit.metainterp.optimizeopt.optimizer import (
     Optimization, OptimizationResult, REMOVED)
 from rpython.jit.metainterp.resoperation import rop, OpHelpers, AbstractResOp,\
      ResOperation
-from rpython.jit.metainterp.optimizeopt.util import (
-    make_dispatcher_method, have_dispatcher_method, get_box_replacement)
+from rpython.jit.metainterp.optimizeopt.util import make_dispatcher_method
 from rpython.jit.metainterp.optimizeopt.shortpreamble import PreambleOp
 from rpython.jit.metainterp.optimize import SpeculativeError
 
@@ -34,17 +33,16 @@ class CallPureOptimizationResult(OptimizationResult):
 
 
 class RecentPureOps(object):
-    def __init__(self, limit=16):
-        self.lst = [None] * limit
+    REMEMBER_LIMIT = 16
+
+    def __init__(self):
+        self.lst = [None] * self.REMEMBER_LIMIT
         self.next_index = 0
 
     def add(self, op):
         assert isinstance(op, AbstractResOp)
         next_index = self.next_index
-        new = next_index + 1
-        if new == len(self.lst):
-            new = 0
-        self.next_index = new
+        self.next_index = (next_index + 1) % self.REMEMBER_LIMIT
         self.lst[next_index] = op
 
     def force_preamble_op(self, opt, op, i):
@@ -55,37 +53,37 @@ class RecentPureOps(object):
         return op
 
     def lookup1(self, opt, box0, descr):
-        for i in range(len(self.lst)):
+        for i in range(self.REMEMBER_LIMIT):
             op = self.lst[i]
             if op is None:
                 break
-            if box0.same_box(get_box_replacement(op.getarg(0))) and op.getdescr() is descr:
+            if box0.same_box(opt.get_box_replacement(op.getarg(0))) and op.getdescr() is descr:
                 op = self.force_preamble_op(opt, op, i)
-                return get_box_replacement(op)
+                return opt.get_box_replacement(op)
         return None
 
     def lookup2(self, opt, box0, box1, descr):
-        for i in range(len(self.lst)):
+        for i in range(self.REMEMBER_LIMIT):
             op = self.lst[i]
             if op is None:
                 break
-            if (box0.same_box(get_box_replacement(op.getarg(0))) and
-                box1.same_box(get_box_replacement(op.getarg(1))) and
+            if (box0.same_box(opt.get_box_replacement(op.getarg(0))) and
+                box1.same_box(opt.get_box_replacement(op.getarg(1))) and
                 op.getdescr() is descr):
                 op = self.force_preamble_op(opt, op, i)
-                return get_box_replacement(op)
+                return opt.get_box_replacement(op)
         return None
 
     def lookup(self, optimizer, op):
         numargs = op.numargs()
         if numargs == 1:
             return self.lookup1(optimizer,
-                                get_box_replacement(op.getarg(0)),
+                                optimizer.get_box_replacement(op.getarg(0)),
                                 op.getdescr())
         elif numargs == 2:
             return self.lookup2(optimizer,
-                                get_box_replacement(op.getarg(0)),
-                                get_box_replacement(op.getarg(1)),
+                                optimizer.get_box_replacement(op.getarg(0)),
+                                optimizer.get_box_replacement(op.getarg(1)),
                                 op.getdescr())
         else:
             assert False
@@ -98,14 +96,15 @@ class OptPure(Optimization):
                                           rop._ALWAYS_PURE_FIRST)
         self.call_pure_positions = []
         self.extra_call_pure = []
-        self.known_result_call_pure = []
 
     def propagate_forward(self, op):
         return dispatch_opt(self, op)
 
+    def propagate_postprocess(self, op):
+        dispatch_postprocess(self, op)
+
     def optimize_default(self, op):
         canfold = rop.is_always_pure(op.opnum)
-        ovf = False
         if rop.is_ovf(op.opnum):
             self.postponed_op = op
             return
@@ -114,7 +113,6 @@ class OptPure(Optimization):
             op = self.postponed_op
             self.postponed_op = None
             canfold = nextop.getopnum() == rop.GUARD_NO_OVERFLOW
-            ovf = True
         else:
             nextop = None
 
@@ -132,51 +130,34 @@ class OptPure(Optimization):
                 return
 
             # did we do the exact same operation already?
-            recentops = self.getrecentops(op.getopnum(), create=False)
+            recentops = self.getrecentops(op.getopnum())
             save = True
-            if recentops is not None:
-                oldop = recentops.lookup(self.optimizer, op)
-                if oldop is not None and self._can_reuse_oldop(
-                            recentops, oldop, op, ovf):
-                    self.optimizer.make_equal_to(op, oldop)
-                    return
+            oldop = recentops.lookup(self.optimizer, op)
+            if oldop is not None:
+                self.optimizer.make_equal_to(op, oldop)
+                return
 
         # otherwise, the operation remains
-        if nextop is None and not save and not rop.returns_bool_result(op.getopnum()):
-            # for this case DefaultOptimizationResult would do nothing
-            return self.emit(op)
         return self.emit_result(DefaultOptimizationResult(self, op, save, nextop))
 
-    def _can_reuse_oldop(self, recentops, oldop, op, ovf):
-        if ovf:
-            # careful! this is an ovf operation. so we can only
-            # re-use the result of a prior ovf operation, but not a
-            # regular int_... up, because that might have
-            # overflowed (the other direction is fine). therefore
-            # we need to check that the previous op and the current
-            # op have the same opnum.
-            return isinstance(oldop, AbstractResOp) and oldop.opnum == op.opnum
-        return True
-
-    def getrecentops(self, opnum, create=True):
+    def getrecentops(self, opnum):
         if rop._OVF_FIRST <= opnum <= rop._OVF_LAST:
             opnum = opnum - rop._OVF_FIRST
         else:
             opnum = opnum - rop._ALWAYS_PURE_FIRST
         assert 0 <= opnum < len(self._pure_operations)
         recentops = self._pure_operations[opnum]
-        if recentops is None and create:
-            length = self.optimizer.jitdriver_sd.warmstate.pureop_historylength
-            self._pure_operations[opnum] = recentops = RecentPureOps(length)
+        if recentops is None:
+            self._pure_operations[opnum] = recentops = RecentPureOps()
         return recentops
 
-    def optimize_call_pure(self, op, start_index=0):
+    def optimize_CALL_PURE_I(self, op):
         # Step 1: check if all arguments are constant
-        for i in range(start_index, op.numargs()):
-            self.optimizer.force_box(op.getarg(i))
+        for arg in op.getarglist():
+            self.optimizer.force_box(arg)
             # XXX hack to ensure that virtuals that are
             #     constant are presented that way
-        result = self._can_optimize_call_pure(op, start_index=start_index)
+        result = self._can_optimize_call_pure(op)
         if result is not None:
             # this removes a CALL_PURE with all constant arguments.
             self.make_constant(op, result)
@@ -187,60 +168,33 @@ class OptPure(Optimization):
         # CALL_PURE.
         for pos in self.call_pure_positions:
             old_op = self.optimizer._newoperations[pos]
-            if self.optimize_call_pure_old(op, old_op, start_index):
+            if self.optimize_call_pure(op, old_op):
                 return
         if self.extra_call_pure:
             for i, old_op in enumerate(self.extra_call_pure):
-                if self.optimize_call_pure_old(op, old_op, start_index):
+                if self.optimize_call_pure(op, old_op):
                     if isinstance(old_op, PreambleOp):
                         old_op = self.optimizer.force_op_from_preamble(old_op)
                         self.extra_call_pure[i] = old_op
                     return
-        if self.known_result_call_pure:
-            # needs different logic, since the result does not come from a real
-            # call at all
-            for i, known_result_op in enumerate(self.known_result_call_pure):
-                if op.getdescr() is not known_result_op.getdescr():
-                    continue
-                if self._same_args(known_result_op, op, 1, start_index):
-                    self.make_equal_to(op, known_result_op.getarg(0))
-                    self.last_emitted_operation = REMOVED
-                    return
 
-        # replace CALL_PURE with just CALL (but keep COND_CALL_VALUE)
-        if start_index == 0:
-            opnum = OpHelpers.call_for_descr(op.getdescr())
-            newop = self.optimizer.replace_op_with(op, opnum)
-        else:
-            newop = op
+        # replace CALL_PURE with just CALL
+        opnum = OpHelpers.call_for_descr(op.getdescr())
+        newop = self.optimizer.replace_op_with(op, opnum)
         return self.emit_result(CallPureOptimizationResult(self, newop))
 
-    def optimize_CALL_PURE_I(self, op):
-        return self.optimize_call_pure(op)
     optimize_CALL_PURE_R = optimize_CALL_PURE_I
     optimize_CALL_PURE_F = optimize_CALL_PURE_I
     optimize_CALL_PURE_N = optimize_CALL_PURE_I
 
-    def optimize_COND_CALL_VALUE_I(self, op):
-        return self.optimize_call_pure(op, start_index=1)
-    optimize_COND_CALL_VALUE_R = optimize_COND_CALL_VALUE_I
-
-    def _same_args(self, op1, op2, start_index1, start_index2):
-        j = start_index2
-        for i in range(start_index1, op1.numargs()):
-            box = get_box_replacement(op1.getarg(i))
-            if not get_box_replacement(op2.getarg(j)).same_box(box):
-                return False
-            j += 1
-        return True
-
-    def optimize_call_pure_old(self, op, old_op, start_index):
-        if op.getdescr() is not old_op.getdescr():
+    def optimize_call_pure(self, op, old_op):
+        if (op.numargs() != old_op.numargs() or
+            op.getdescr() is not old_op.getdescr()):
             return False
-        # this will match a call_pure and a cond_call_value with
-        # the same function and arguments
-        old_start_index = OpHelpers.is_cond_call_value(old_op.opnum)
-        if self._same_args(old_op, op, old_start_index, start_index):
+        for i, box in enumerate(old_op.getarglist()):
+            if not self.get_box_replacement(op.getarg(i)).same_box(box):
+                break
+        else:
             # all identical
             # this removes a CALL_PURE that has the same (non-constant)
             # arguments as a previous CALL_PURE.
@@ -259,10 +213,6 @@ class OptPure(Optimization):
             return
         return self.emit(op)
 
-    def optimize_RECORD_KNOWN_RESULT(self, op):
-        # remove the op, but keep the into about the result of the pure call
-        self.known_result_call_pure.append(op)
-
     def flush(self):
         assert self.postponed_op is None
 
@@ -275,15 +225,17 @@ class OptPure(Optimization):
 
     def pure_from_args(self, opnum, args, op, descr=None):
         newop = ResOperation(opnum,
-                             [get_box_replacement(arg) for arg in args],
+                             [self.get_box_replacement(arg) for arg in args],
                              descr=descr)
         newop.set_forwarded(op)
         self.pure(opnum, newop)
 
+    def has_pure_result(self, opnum, args, descr):
+        return False
+    # XXX
+
     def get_pure_result(self, op):
-        recentops = self.getrecentops(op.getopnum(), create=False)
-        if not recentops:
-            return None
+        recentops = self.getrecentops(op.getopnum())
         return recentops.lookup(self.optimizer, op)
 
     def produce_potential_short_preamble_ops(self, sb):
@@ -298,19 +250,11 @@ class OptPure(Optimization):
             # don't move call_pure_with_exception in the short preamble...
             # issue #2015
 
-            # Also, don't move cond_call_value in the short preamble.
-            # The issue there is that it's usually pointless to try to
-            # because the 'value' argument is typically not a loop
-            # invariant, and would really need to be in order to end up
-            # in the short preamble.  Maybe the code works anyway in the
-            # other rare case, but better safe than sorry and don't try.
             effectinfo = op.getdescr().get_extra_info()
             if not effectinfo.check_can_raise(ignore_memoryerror=True):
                 assert rop.is_call(op.opnum)
-                if not OpHelpers.is_cond_call_value(op.opnum):
-                    sb.add_pure_op(op)
+                sb.add_pure_op(op)
 
 dispatch_opt = make_dispatcher_method(OptPure, 'optimize_',
                                       default=OptPure.optimize_default)
-OptPure.propagate_postprocess = make_dispatcher_method(OptPure, 'postprocess_')
-OptPure.have_postprocess_op = have_dispatcher_method(OptPure, 'postprocess_')
+dispatch_postprocess = make_dispatcher_method(OptPure, 'postprocess_')
