@@ -1,18 +1,19 @@
 from rpython.jit.codewriter.effectinfo import EffectInfo
 from rpython.jit.codewriter import longlong
 from rpython.jit.metainterp import compile
-from rpython.jit.metainterp.history import (
-    Const, ConstInt, make_hashable_int, ConstFloat, CONST_NULL)
+from rpython.jit.metainterp.history import (Const, ConstInt, make_hashable_int,
+                                            ConstFloat)
 from rpython.jit.metainterp.optimize import InvalidLoop
+from rpython.jit.metainterp.optimizeopt.intutils import IntBound
 from rpython.jit.metainterp.optimizeopt.optimizer import (
     Optimization, OptimizationResult, REMOVED, CONST_0, CONST_1)
-from rpython.jit.metainterp.optimizeopt.info import (
-    INFO_NONNULL, INFO_NULL, getptrinfo)
-from rpython.jit.metainterp.optimizeopt.util import (
-    _findall, make_dispatcher_method, get_box_replacement)
-from rpython.jit.metainterp.resoperation import (
-    rop, ResOperation, opclasses, OpHelpers)
+from rpython.jit.metainterp.optimizeopt.info import INFO_NONNULL, INFO_NULL
+from rpython.jit.metainterp.optimizeopt.util import _findall, make_dispatcher_method
+from rpython.jit.metainterp.resoperation import rop, ResOperation, opclasses,\
+     OpHelpers
 from rpython.rlib.rarithmetic import highest_bit
+from rpython.rtyper.lltypesystem import llmemory
+from rpython.rtyper import rclass
 import math
 
 
@@ -67,6 +68,7 @@ class OptRewrite(Optimization):
                 return True
         return False
 
+
     def find_rewritable_bool(self, op):
         oldopnum = op.boolinverse
         arg0 = op.getarg(0)
@@ -76,7 +78,7 @@ class OptRewrite(Optimization):
             if self.try_boolinvers(op, top):
                 return True
 
-        oldopnum = op.boolreflex  # FIXME: add INT_ADD, INT_MUL
+        oldopnum = op.boolreflex # FIXME: add INT_ADD, INT_MUL
         if oldopnum != -1:
             top = ResOperation(oldopnum, [arg1, arg0])
             oldop = self.get_pure_result(top)
@@ -102,14 +104,14 @@ class OptRewrite(Optimization):
             return
         elif b2.is_constant():
             val = b2.lower
-            if val == -1 or (b1.bounded() and b1.lower >= 0
-                                          and b1.upper <= val & ~(val + 1)):
+            if val == -1 or b1.lower >= 0 \
+                and b1.upper <= val & ~(val + 1):
                 self.make_equal_to(op, op.getarg(0))
                 return
         elif b1.is_constant():
             val = b1.lower
-            if val == -1 or (b2.bounded() and b2.lower >= 0
-                                          and b2.upper <= val & ~(val + 1)):
+            if val == -1 or b2.lower >= 0 \
+                and b2.upper <= val & ~(val + 1):
                 self.make_equal_to(op, op.getarg(1))
                 return
 
@@ -126,8 +128,8 @@ class OptRewrite(Optimization):
             return self.emit(op)
 
     def optimize_INT_SUB(self, op):
-        arg1 = get_box_replacement(op.getarg(0))
-        arg2 = get_box_replacement(op.getarg(1))
+        arg1 = self.get_box_replacement(op.getarg(0))
+        arg2 = self.get_box_replacement(op.getarg(1))
         b1 = self.getintbound(arg1)
         b2 = self.getintbound(arg2)
         if b2.equal(0):
@@ -160,9 +162,9 @@ class OptRewrite(Optimization):
     def optimize_INT_ADD(self, op):
         if self.is_raw_ptr(op.getarg(0)) or self.is_raw_ptr(op.getarg(1)):
             return self.emit(op)
-        arg1 = get_box_replacement(op.getarg(0))
+        arg1 = self.get_box_replacement(op.getarg(0))
         b1 = self.getintbound(arg1)
-        arg2 = get_box_replacement(op.getarg(1))
+        arg2 = self.get_box_replacement(op.getarg(1))
         b2 = self.getintbound(arg2)
 
         # If one side of the op is 0 the result is the other side.
@@ -202,9 +204,9 @@ class OptRewrite(Optimization):
         self.optimizer.pure_from_args(rop.INT_ADD, [inv_arg0, op], arg1)
 
     def optimize_INT_MUL(self, op):
-        arg1 = get_box_replacement(op.getarg(0))
+        arg1 = self.get_box_replacement(op.getarg(0))
         b1 = self.getintbound(arg1)
-        arg2 = get_box_replacement(op.getarg(1))
+        arg2 = self.get_box_replacement(op.getarg(1))
         b2 = self.getintbound(arg2)
 
         # If one side of the op is 1 the result is the other side.
@@ -238,9 +240,9 @@ class OptRewrite(Optimization):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
 
-        if b2.equal(0):
+        if b2.is_constant() and b2.getint() == 0:
             self.make_equal_to(op, op.getarg(0))
-        elif b1.equal(0):
+        elif b1.is_constant() and b1.getint() == 0:
             self.make_constant_int(op, 0)
         else:
             return self.emit(op)
@@ -249,9 +251,9 @@ class OptRewrite(Optimization):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
 
-        if b2.equal(0):
+        if b2.is_constant() and b2.getint() == 0:
             self.make_equal_to(op, op.getarg(0))
-        elif b1.equal(0):
+        elif b1.is_constant() and b1.getint() == 0:
             self.make_constant_int(op, 0)
         else:
             return self.emit(op)
@@ -267,9 +269,6 @@ class OptRewrite(Optimization):
         else:
             return self.emit(op)
 
-    def postprocess_INT_INVERT(self, op):
-        self.optimizer.pure_from_args(rop.INT_INVERT, [op], op.getarg(0))
-
     def optimize_FLOAT_MUL(self, op):
         arg1 = op.getarg(0)
         arg2 = op.getarg(1)
@@ -277,8 +276,8 @@ class OptRewrite(Optimization):
         # Constant fold f0 * 1.0 and turn f0 * -1.0 into a FLOAT_NEG, these
         # work in all cases, including NaN and inf
         for lhs, rhs in [(arg1, arg2), (arg2, arg1)]:
-            v1 = get_box_replacement(lhs)
-            v2 = get_box_replacement(rhs)
+            v1 = self.get_box_replacement(lhs)
+            v2 = self.get_box_replacement(rhs)
 
             if v1.is_constant():
                 if v1.getfloat() == 1.0:
@@ -296,7 +295,7 @@ class OptRewrite(Optimization):
     def optimize_FLOAT_TRUEDIV(self, op):
         arg1 = op.getarg(0)
         arg2 = op.getarg(1)
-        v2 = get_box_replacement(arg2)
+        v2 = self.get_box_replacement(arg2)
 
         # replace "x / const" by "x * (1/const)" if possible
         newop = op
@@ -332,7 +331,7 @@ class OptRewrite(Optimization):
                                       'was proven to always fail' % r)
                 return
         elif box.type == 'r':
-            box = get_box_replacement(box)
+            box = self.get_box_replacement(box)
             if box.is_constant():
                 if not box.same_constant(constbox):
                     r = self.optimizer.metainterp_sd.logger_ops.repr_of_resop(
@@ -344,7 +343,7 @@ class OptRewrite(Optimization):
         return self.emit(op)
 
     def optimize_GUARD_ISNULL(self, op):
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         if info is not None:
             if info.is_null():
                 return
@@ -355,14 +354,14 @@ class OptRewrite(Optimization):
         return self.emit(op)
 
     def postprocess_GUARD_ISNULL(self, op):
-        self.make_constant(op.getarg(0), CONST_NULL)
+        self.make_constant(op.getarg(0), self.optimizer.cpu.ts.CONST_NULL)
 
     def optimize_GUARD_IS_OBJECT(self, op):
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         if info and info.is_constant():
             if info.is_null():
                 raise InvalidLoop("A GUARD_IS_OBJECT(NULL) found")
-            c = get_box_replacement(op.getarg(0))
+            c = self.get_box_replacement(op.getarg(0))
             if self.optimizer.cpu.check_is_object(c.getref_base()):
                 return
             raise InvalidLoop("A GUARD_IS_OBJECT(not-an-object) found")
@@ -374,9 +373,9 @@ class OptRewrite(Optimization):
         return self.emit(op)
 
     def optimize_GUARD_GC_TYPE(self, op):
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         if info and info.is_constant():
-            c = get_box_replacement(op.getarg(0))
+            c = self.get_box_replacement(op.getarg(0))
             tid = self.optimizer.cpu.get_actual_typeid(c.getref_base())
             if tid != op.getarg(1).getint():
                 raise InvalidLoop("wrong GC type ID found on a constant")
@@ -388,46 +387,28 @@ class OptRewrite(Optimization):
         return self.emit(op)
 
     def optimize_GUARD_SUBCLASS(self, op):
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         optimizer = self.optimizer
-        # must raise 'InvalidLoop' in all cases where 'info' shows the
-        # class cannot possibly match (see test_issue2926)
         if info and info.is_constant():
-            c = get_box_replacement(op.getarg(0))
-            vtable = optimizer.cpu.cls_of_box(c).getint()
+            c = self.get_box_replacement(op.getarg(0))
+            vtable = optimizer.cpu.ts.cls_of_box(c).getint()
             if optimizer._check_subclass(vtable, op.getarg(1).getint()):
                 return
             raise InvalidLoop("GUARD_SUBCLASS(const) proven to always fail")
         if info is not None and info.is_about_object():
             known_class = info.get_known_class(optimizer.cpu)
             if known_class:
-                # Class of 'info' is exactly 'known_class'.
-                # We know statically if the 'guard_subclass' will pass or fail.
                 if optimizer._check_subclass(known_class.getint(),
                                              op.getarg(1).getint()):
                     return
-                else:
-                    raise InvalidLoop(
-                        "GUARD_SUBCLASS(known_class) proven to always fail")
             elif info.get_descr() is not None:
-                # Class of 'info' is either get_descr() or a subclass of it.
-                # We're keeping the 'guard_subclass' at runtime only in the
-                # case where get_descr() is some strict parent class of
-                # the argument to 'guard_subclass'.
-                info_base_descr = info.get_descr().get_vtable()
-                if optimizer._check_subclass(info_base_descr,
+                if optimizer._check_subclass(info.get_descr().get_vtable(),
                                              op.getarg(1).getint()):
-                    return    # guard_subclass always passing
-                elif optimizer._check_subclass(op.getarg(1).getint(),
-                                               info_base_descr):
-                    pass      # don't know, must keep the 'guard_subclass'
-                else:
-                    raise InvalidLoop(
-                        "GUARD_SUBCLASS(base_class) proven to always fail")
+                    return
         return self.emit(op)
 
     def optimize_GUARD_NONNULL(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo is not None:
             if opinfo.is_nonnull():
                 return
@@ -439,12 +420,12 @@ class OptRewrite(Optimization):
 
     def postprocess_GUARD_NONNULL(self, op):
         self.make_nonnull(op.getarg(0))
-        getptrinfo(op.getarg(0)).mark_last_guard(self.optimizer)
+        self.getptrinfo(op.getarg(0)).mark_last_guard(self.optimizer)
 
     def optimize_GUARD_VALUE(self, op):
         arg0 = op.getarg(0)
         if arg0.type == 'r':
-            info = getptrinfo(arg0)
+            info = self.getptrinfo(arg0)
             if info:
                 if info.is_virtual():
                     raise InvalidLoop("promote of a virtual")
@@ -453,7 +434,7 @@ class OptRewrite(Optimization):
                     op = self.replace_old_guard_with_guard_value(op, info,
                                                               old_guard_op)
         elif arg0.type == 'f':
-            arg0 = get_box_replacement(arg0)
+            arg0 = self.get_box_replacement(arg0)
             if arg0.is_constant():
                 return
         constbox = op.getarg(1)
@@ -461,7 +442,7 @@ class OptRewrite(Optimization):
         return self.optimize_guard(op, constbox)
 
     def postprocess_GUARD_VALUE(self, op):
-        box = get_box_replacement(op.getarg(0))
+        box = self.get_box_replacement(op.getarg(0))
         self.make_constant(box, op.getarg(1))
 
     def replace_old_guard_with_guard_value(self, op, info, old_guard_op):
@@ -483,19 +464,17 @@ class OptRewrite(Optimization):
                               'guard that it is not NULL')
         previous_classbox = info.get_known_class(self.optimizer.cpu)
         if previous_classbox is not None:
-            expected_classbox = self.optimizer.cpu.cls_of_box(c_value)
+            expected_classbox = self.optimizer.cpu.ts.cls_of_box(c_value)
             assert expected_classbox is not None
             if not previous_classbox.same_constant(
                     expected_classbox):
                 r = self.optimizer.metainterp_sd.logger_ops.repr_of_resop(op)
                 raise InvalidLoop('A GUARD_VALUE (%s) was proven to '
                                   'always fail' % r)
-        if not self.optimizer.can_replace_guards:
-            return op
         descr = compile.ResumeGuardDescr()
         op = old_guard_op.copy_and_change(rop.GUARD_VALUE,
-                         args=[old_guard_op.getarg(0), op.getarg(1)],
-                         descr=descr)
+                         args = [old_guard_op.getarg(0), op.getarg(1)],
+                         descr = descr)
         # Note: we give explicitly a new descr for 'op'; this is why the
         # old descr must not be ResumeAtPositionDescr (checked above).
         # Better-safe-than-sorry but it should never occur: we should
@@ -510,31 +489,18 @@ class OptRewrite(Optimization):
         return self.optimize_guard(op, CONST_1)
 
     def postprocess_GUARD_TRUE(self, op):
-        box = get_box_replacement(op.getarg(0))
-        box1 = self.optimizer.as_operation(box)
-        if box1 is not None and box1.getopnum() == rop.INT_IS_TRUE:
-            # we can't use the (current) range analysis for this because
-            # "anything but 0" is not a valid range
-            self.pure_from_args(rop.INT_IS_ZERO, [box1.getarg(0)], CONST_0)
+        box = self.get_box_replacement(op.getarg(0))
         self.make_constant(box, CONST_1)
 
     def optimize_GUARD_FALSE(self, op):
         return self.optimize_guard(op, CONST_0)
 
     def postprocess_GUARD_FALSE(self, op):
-        box = get_box_replacement(op.getarg(0))
-        box1 = self.optimizer.as_operation(box)
-        if box1 is not None and box1.getopnum() == rop.INT_IS_ZERO:
-            # we can't use the (current) range analysis for this because
-            # "anything but 0" is not a valid range
-            self.pure_from_args(rop.INT_IS_TRUE, [box1.getarg(0)], CONST_1)
+        box = self.get_box_replacement(op.getarg(0))
         self.make_constant(box, CONST_0)
 
-    def optimize_ASSERT_NOT_NONE(self, op):
-        self.make_nonnull(op.getarg(0))
-
     def optimize_RECORD_EXACT_CLASS(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         expectedclassbox = op.getarg(1)
         assert isinstance(expectedclassbox, Const)
         if opinfo is not None:
@@ -561,13 +527,12 @@ class OptRewrite(Optimization):
                                            compile.ResumeAtPositionDescr):
             # there already has been a guard_nonnull or guard_class or
             # guard_nonnull_class on this value.
-            if (self.optimizer.can_replace_guards and
-                    old_guard_op.getopnum() == rop.GUARD_NONNULL):
+            if old_guard_op.getopnum() == rop.GUARD_NONNULL:
                 # it was a guard_nonnull, which we replace with a
                 # guard_nonnull_class.
                 descr = compile.ResumeGuardDescr()
-                op = old_guard_op.copy_and_change(rop.GUARD_NONNULL_CLASS,
-                            args=[old_guard_op.getarg(0), op.getarg(1)],
+                op = old_guard_op.copy_and_change (rop.GUARD_NONNULL_CLASS,
+                            args = [old_guard_op.getarg(0), op.getarg(1)],
                             descr=descr)
                 # Note: we give explicitly a new descr for 'op'; this is why the
                 # old descr must not be ResumeAtPositionDescr (checked above).
@@ -580,14 +545,14 @@ class OptRewrite(Optimization):
 
     def postprocess_GUARD_CLASS(self, op):
         expectedclassbox = op.getarg(1)
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         old_guard_op = info.get_last_guard(self.optimizer)
         update_last_guard = not old_guard_op or isinstance(
             old_guard_op.getdescr(), compile.ResumeAtPositionDescr)
         self.make_constant_class(op.getarg(0), expectedclassbox, update_last_guard)
 
     def optimize_GUARD_NONNULL_CLASS(self, op):
-        info = getptrinfo(op.getarg(0))
+        info = self.getptrinfo(op.getarg(0))
         if info and info.is_null():
             r = self.optimizer.metainterp_sd.logger_ops.repr_of_resop(op)
             raise InvalidLoop('A GUARD_NONNULL_CLASS (%s) was proven to '
@@ -631,19 +596,6 @@ class OptRewrite(Optimization):
             op = op.copy_and_change(opnum, args=op.getarglist()[1:])
         return self.emit(op)
 
-    def optimize_COND_CALL_VALUE_I(self, op):
-        # look if we know the nullness of the first argument
-        info = self.getnullness(op.getarg(0))
-        if info == INFO_NONNULL:
-            self.make_equal_to(op, op.getarg(0))
-            self.last_emitted_operation = REMOVED
-            return
-        if info == INFO_NULL:
-            opnum = OpHelpers.call_pure_for_type(op.type)
-            op = self.replace_op_with(op, opnum, args=op.getarglist()[1:])
-        return self.emit(op)
-    optimize_COND_CALL_VALUE_R = optimize_COND_CALL_VALUE_I
-
     def _optimize_nullness(self, op, box, expect_nonnull):
         info = self.getnullness(box)
         if info == INFO_NONNULL:
@@ -664,10 +616,10 @@ class OptRewrite(Optimization):
         return self._optimize_nullness(op, op.getarg(0), False)
 
     def _optimize_oois_ooisnot(self, op, expect_isnot, instance):
-        arg0 = get_box_replacement(op.getarg(0))
-        arg1 = get_box_replacement(op.getarg(1))
-        info0 = getptrinfo(arg0)
-        info1 = getptrinfo(arg1)
+        arg0 = self.get_box_replacement(op.getarg(0))
+        arg1 = self.get_box_replacement(op.getarg(1))
+        info0 = self.getptrinfo(arg0)
+        info1 = self.getptrinfo(arg1)
         if info0 and info0.is_virtual():
             if info1 and info1.is_virtual():
                 intres = (info0 is info1) ^ expect_isnot
@@ -707,15 +659,9 @@ class OptRewrite(Optimization):
         return self._optimize_oois_ooisnot(op, True, False)
 
     def optimize_INSTANCE_PTR_EQ(self, op):
-        arg0 = get_box_replacement(op.getarg(0))
-        arg1 = get_box_replacement(op.getarg(1))
-        self.pure_from_args(rop.INSTANCE_PTR_EQ, [arg1, arg0], op)
         return self._optimize_oois_ooisnot(op, False, True)
 
     def optimize_INSTANCE_PTR_NE(self, op):
-        arg0 = get_box_replacement(op.getarg(0))
-        arg1 = get_box_replacement(op.getarg(1))
-        self.pure_from_args(rop.INSTANCE_PTR_NE, [arg1, arg0], op)
         return self._optimize_oois_ooisnot(op, True, True)
 
     def optimize_CALL_N(self, op):
@@ -726,33 +672,17 @@ class OptRewrite(Optimization):
         oopspecindex = effectinfo.oopspecindex
         if oopspecindex == EffectInfo.OS_ARRAYCOPY:
             return self._optimize_CALL_ARRAYCOPY(op)
-        if oopspecindex == EffectInfo.OS_ARRAYMOVE:
-            return self._optimize_CALL_ARRAYMOVE(op)
         return self.emit(op)
 
     def _optimize_CALL_ARRAYCOPY(self, op):
-        if self._optimize_call_arrayop(op, op.getarg(1), op.getarg(2),
-                             op.getarg(3), op.getarg(4), op.getarg(5)):
-            return None
-        return self.emit(op)
-
-    def _optimize_CALL_ARRAYMOVE(self, op):
-        array_box = op.getarg(1)
-        if self._optimize_call_arrayop(op, array_box, array_box,
-                             op.getarg(2), op.getarg(3), op.getarg(4)):
-            return None
-        return self.emit(op)
-
-    def _optimize_call_arrayop(self, op, source_box, dest_box,
-                               source_start_box, dest_start_box, length_box):
-        length = self.get_constant_box(length_box)
+        length = self.get_constant_box(op.getarg(5))
         if length and length.getint() == 0:
-            return True  # 0-length arraycopy or arraymove
+            return None  # 0-length arraycopy
 
-        source_info = getptrinfo(source_box)
-        dest_info = getptrinfo(dest_box)
-        source_start_box = self.get_constant_box(source_start_box)
-        dest_start_box = self.get_constant_box(dest_start_box)
+        source_info = self.getptrinfo(op.getarg(1))
+        dest_info = self.getptrinfo(op.getarg(2))
+        source_start_box = self.get_constant_box(op.getarg(3))
+        dest_start_box = self.get_constant_box(op.getarg(4))
         extrainfo = op.getdescr().get_extra_info()
         if (source_start_box and dest_start_box
             and length and ((dest_info and dest_info.is_virtual()) or
@@ -763,28 +693,16 @@ class OptRewrite(Optimization):
             dest_start = dest_start_box.getint()
             arraydescr = extrainfo.single_write_descr_array
             if arraydescr.is_array_of_structs():
-                return False        # not supported right now
-
-            index_current = 0
-            index_delta = +1
-            index_stop = length.getint()
-            if (source_box is dest_box and        # ARRAYMOVE only
-                    source_start < dest_start):   # iterate in reverse order
-                index_current = index_stop - 1
-                index_delta = -1
-                index_stop = -1
+                return self.emit(op)       # not supported right now
 
             # XXX fish fish fish
-            while index_current != index_stop:
-                index = index_current
-                index_current += index_delta
-                assert index >= 0
+            for index in range(length.getint()):
                 if source_info and source_info.is_virtual():
                     val = source_info.getitem(arraydescr, index + source_start)
                 else:
                     opnum = OpHelpers.getarrayitem_for_descr(arraydescr)
                     newop = ResOperation(opnum,
-                                      [source_box,
+                                      [op.getarg(1),
                                        ConstInt(index + source_start)],
                                        descr=arraydescr)
                     self.optimizer.send_extra_operation(newop)
@@ -793,17 +711,17 @@ class OptRewrite(Optimization):
                     continue
                 if dest_info and dest_info.is_virtual():
                     dest_info.setitem(arraydescr, index + dest_start,
-                                      get_box_replacement(dest_box),
+                                      self.get_box_replacement(op.getarg(2)),
                                       val)
                 else:
                     newop = ResOperation(rop.SETARRAYITEM_GC,
-                                         [dest_box,
+                                         [op.getarg(2),
                                           ConstInt(index + dest_start),
                                           val],
                                          descr=arraydescr)
                     self.optimizer.send_extra_operation(newop)
-            return True
-        return False
+            return None
+        return self.emit(op)
 
     def optimize_CALL_PURE_I(self, op):
         # this removes a CALL_PURE with all constant arguments.
@@ -848,10 +766,12 @@ class OptRewrite(Optimization):
         arg2 = op.getarg(2)
         b2 = self.getintbound(arg2)
 
-        if b1.equal(0):
+        if b1.is_constant() and b1.getint() == 0:
             self.make_constant_int(op, 0)
             self.last_emitted_operation = REMOVED
             return True
+        # This is Python's integer division: 'x // (2**shift)' can always
+        # be replaced with 'x >> shift', even for negative values of x
         if not b2.is_constant():
             return False
         val = b2.getint()
@@ -870,7 +790,7 @@ class OptRewrite(Optimization):
             return True
         else:
             from rpython.jit.metainterp.optimizeopt import intdiv
-            known_nonneg = b1.known_nonnegative()
+            known_nonneg = b1.known_ge(IntBound(0, 0))
             operations = intdiv.division_operations(arg1, val, known_nonneg)
             newop = None
             for newop in operations:
@@ -884,7 +804,7 @@ class OptRewrite(Optimization):
         arg2 = op.getarg(2)
         b2 = self.getintbound(arg2)
 
-        if b1.equal(0):
+        if b1.is_constant() and b1.getint() == 0:
             self.make_constant_int(op, 0)
             self.last_emitted_operation = REMOVED
             return True
@@ -910,7 +830,7 @@ class OptRewrite(Optimization):
             return True
         else:
             from rpython.jit.metainterp.optimizeopt import intdiv
-            known_nonneg = b1.known_nonnegative()
+            known_nonneg = b1.known_ge(IntBound(0, 0))
             operations = intdiv.modulo_operations(arg1, val, known_nonneg)
             newop = None
             for newop in operations:
@@ -926,31 +846,11 @@ class OptRewrite(Optimization):
         self.optimizer.pure_from_args(rop.CAST_PTR_TO_INT, [op], op.getarg(0))
         return self.emit(op)
 
-    def optimize_CONVERT_FLOAT_BYTES_TO_LONGLONG(self, op):
-        self.optimizer.pure_from_args(rop.CONVERT_LONGLONG_BYTES_TO_FLOAT, [op], op.getarg(0))
-        return self.emit(op)
-
-    def optimize_CONVERT_LONGLONG_BYTES_TO_FLOAT(self, op):
-        self.optimizer.pure_from_args(rop.CONVERT_FLOAT_BYTES_TO_LONGLONG, [op], op.getarg(0))
-        return self.emit(op)
-
     def optimize_SAME_AS_I(self, op):
         self.make_equal_to(op, op.getarg(0))
     optimize_SAME_AS_R = optimize_SAME_AS_I
     optimize_SAME_AS_F = optimize_SAME_AS_I
 
-    def serialize_optrewrite(self, available_boxes):
-        res = []
-        for i, box in self.loop_invariant_results.iteritems():
-            box = get_box_replacement(box)
-            if box in available_boxes:
-                res.append((i, box))
-        return res
-
-    def deserialize_optrewrite(self, tups):
-        for i, box in tups:
-            self.loop_invariant_results[i] = box
-
 dispatch_opt = make_dispatcher_method(OptRewrite, 'optimize_',
                                       default=OptRewrite.emit)
 optimize_guards = _findall(OptRewrite, 'optimize_', 'GUARD')
