@@ -10,29 +10,25 @@ from rpython.jit.metainterp.optimizeopt.optimizer import Optimizer,\
 from rpython.jit.metainterp.optimizeopt.vstring import StrPtrInfo
 from rpython.jit.metainterp.optimizeopt.virtualstate import (
     VirtualStateConstructor, VirtualStatesCantMatch)
-from .util import get_box_replacement
-from rpython.jit.metainterp.resoperation import rop, ResOperation, GuardResOp
+from rpython.jit.metainterp.resoperation import rop, ResOperation, GuardResOp,\
+     AbstractResOp
 from rpython.jit.metainterp import compile
 from rpython.rlib.debug import debug_print, debug_start, debug_stop,\
      have_debug_prints
 
-class UnrollOptimizer(Optimizer):
-    def __init__(self, metainterp_sd, jitdriver_sd, optimizations):
-        Optimizer.__init__(self, metainterp_sd, jitdriver_sd, optimizations)
-        self.optunroll = OptUnroll()
-        self.optunroll.optimizer = self
-
+class UnrollableOptimizer(Optimizer):
     def force_op_from_preamble(self, preamble_op):
         if isinstance(preamble_op, PreambleOp):
             if self.optunroll.short_preamble_producer is None:
                 assert False # unreachable code
             op = preamble_op.op
+            self.optimizer.inparg_dict[op] = None # XXX ARGH
             # special hack for int_add(x, accumulator-const) optimization
             self.optunroll.short_preamble_producer.use_box(op,
                                                 preamble_op.preamble_op, self)
             if not preamble_op.op.is_constant():
                 if preamble_op.invented_name:
-                    op = get_box_replacement(op)
+                    op = self.get_box_replacement(op)
                 self.optunroll.potential_extra_ops[op] = preamble_op
             return preamble_op.op
         return preamble_op
@@ -50,7 +46,7 @@ class UnrollOptimizer(Optimizer):
                 # know anything about
 
     def setinfo_from_preamble(self, op, preamble_info, exported_infos):
-        op = get_box_replacement(op)
+        op = self.get_box_replacement(op)
         if op.get_forwarded() is not None:
             return
         if op.is_constant():
@@ -90,166 +86,30 @@ class UnrollOptimizer(Optimizer):
             if preamble_info.is_nonnull():
                 self.make_nonnull(op)
         elif isinstance(preamble_info, intutils.IntBound):
-            loop_info = preamble_info.widen()
-            fix_lo = preamble_info.lower >= MININT/2
-            fix_up = preamble_info.upper <= MAXINT/2
-            intbound = self.getintbound(op)
-            intbound.intersect(loop_info)
+            if preamble_info.lower > MININT/2 or preamble_info.upper < MAXINT/2:
+                intbound = self.getintbound(op)
+                if preamble_info.has_lower and preamble_info.lower > MININT/2:
+                    intbound.has_lower = True
+                    intbound.lower = preamble_info.lower
+                if preamble_info.has_upper and preamble_info.upper < MAXINT/2:
+                    intbound.has_upper = True
+                    intbound.upper = preamble_info.upper
         elif isinstance(preamble_info, info.FloatConstInfo):
             op.set_forwarded(preamble_info._const)
 
-    def optimize_preamble(self, trace, runtime_boxes, call_pure_results, memo):
-        info, newops = self.propagate_all_forward(
-            trace.get_iter(), call_pure_results, flush=False)
-        exported_state = self.optunroll.export_state(info.jump_op.getarglist(),
-                                           info.inputargs,
-                                           runtime_boxes, memo)
-        exported_state.quasi_immutable_deps = info.quasi_immutable_deps
-        # we need to absolutely make sure that we've cleaned up all
-        # the optimization info
-        self._clean_optimization_info(self._newoperations)
-        return exported_state, self._newoperations
 
-    def optimize_peeled_loop(self, trace, celltoken, state, call_pure_results):
-        trace = trace.get_iter()
-        try:
-            label_args = self.optunroll.import_state(trace.inputargs, state)
-        except VirtualStatesCantMatch:
-            raise InvalidLoop("Cannot import state, virtual states don't match")
-        self.optunroll.potential_extra_ops = {}
-        try:
-            info, _ = self.propagate_all_forward(
-                trace, call_pure_results, flush=False)
-        except SpeculativeError:
-            raise InvalidLoop("Speculative heap access would be ill-typed")
-        end_jump = info.jump_op
-        label_op = ResOperation(rop.LABEL, label_args, descr=celltoken)
-        for a in end_jump.getarglist():
-            self.force_box_for_end_of_preamble(get_box_replacement(a))
-        current_vs = self.optunroll.get_virtual_state(end_jump.getarglist())
-        # pick the vs we want to jump to
-        assert isinstance(celltoken, JitCellToken)
-
-        target_virtual_state = self.optunroll.pick_virtual_state(
-            current_vs, state.virtual_state, celltoken.target_tokens)
-        # force the boxes for virtual state to match
-        try:
-            args = target_virtual_state.make_inputargs(
-               [get_box_replacement(x) for x in end_jump.getarglist()],
-               self, force_boxes=True)
-            for arg in args:
-                if arg is not None:
-                    self.force_box(arg)
-        except VirtualStatesCantMatch:
-            raise InvalidLoop("Virtual states did not match "
-                              "after picking the virtual state, when forcing"
-                              " boxes")
-        extra_same_as = self.optunroll.short_preamble_producer.extra_same_as[:]
-        target_token = self.optunroll.finalize_short_preamble(label_op,
-                                                    state.virtual_state)
-        label_op.setdescr(target_token)
-
-        try:
-            new_virtual_state = self.optunroll.jump_to_existing_trace(
-                end_jump, label_op, state.runtime_boxes, force_boxes=False)
-        except InvalidLoop:
-            # inlining short preamble failed, jump to preamble
-            self.jump_to_preamble(celltoken, end_jump)
-            return (UnrollInfo(target_token, label_op, extra_same_as,
-                               self.quasi_immutable_deps),
-                    self._newoperations)
-
-        if new_virtual_state is not None:
-            # Attempt to force virtual boxes in order to avoid jumping
-            # to the preamble.
-            try:
-                new_virtual_state = self.optunroll.jump_to_existing_trace(
-                    end_jump, label_op, state.runtime_boxes, force_boxes=True)
-            except InvalidLoop:
-                pass
-
-        if new_virtual_state is not None:
-            self.jump_to_preamble(celltoken, end_jump)
-            return (UnrollInfo(target_token, label_op, extra_same_as,
-                               self.quasi_immutable_deps),
-                    self._newoperations)
-
-        self.optunroll.disable_retracing_if_max_retrace_guards(
-            self._newoperations, target_token)
-
-        return (UnrollInfo(target_token, label_op, extra_same_as,
-                           self.quasi_immutable_deps),
-                self._newoperations)
-
-    def optimize_bridge(self, trace, runtime_boxes, call_pure_results,
-                        inline_short_preamble, box_names_memo, resumestorage):
-        from rpython.jit.metainterp.optimizeopt.bridgeopt import deserialize_optimizer_knowledge
-        frontend_inputargs = trace.inputargs
-        trace = trace.get_iter()
-        self.optunroll._check_no_forwarding([trace.inputargs])
-        if resumestorage:
-            deserialize_optimizer_knowledge(self,
-                                            resumestorage, frontend_inputargs,
-                                            trace.inputargs)
-        info, ops = self.propagate_all_forward(trace,
-            call_pure_results, False)
-        jump_op = info.jump_op
-        cell_token = jump_op.getdescr()
-        assert isinstance(cell_token, JitCellToken)
-        if not inline_short_preamble or len(cell_token.target_tokens) == 1:
-            self.jump_to_preamble(cell_token, jump_op)
-            return info, self._newoperations[:]
-        # force all the information that does not go to the short
-        # preamble at all
-        self.flush()
-        for a in jump_op.getarglist():
-            self.force_box_for_end_of_preamble(a)
-        try:
-            vs = self.optunroll.jump_to_existing_trace(jump_op, None, runtime_boxes,
-                                             force_boxes=False)
-        except InvalidLoop:
-            self.jump_to_preamble(cell_token, jump_op)
-            return info, self._newoperations[:]
-        if vs is None:
-            return info, self._newoperations[:]
-        warmrunnerdescr = self.metainterp_sd.warmrunnerdesc
-        limit = warmrunnerdescr.memory_manager.retrace_limit
-        if cell_token.retraced_count < limit:
-            cell_token.retraced_count += 1
-            debug_print('Retracing (%d/%d)' % (cell_token.retraced_count, limit))
-        else:
-            # Try forcing boxes to avoid jumping to the preamble
-            try:
-                vs = self.optunroll.jump_to_existing_trace(jump_op, None, runtime_boxes,
-                                                 force_boxes=True)
-            except InvalidLoop:
-                pass
-            if vs is None:
-                return info, self._newoperations[:]
-            debug_print("Retrace count reached, jumping to preamble")
-            self.jump_to_preamble(cell_token, jump_op)
-            return info, self._newoperations[:]
-        exported_state = self.optunroll.export_state(info.jump_op.getarglist(),
-                                           info.inputargs, runtime_boxes,
-                                           box_names_memo)
-        exported_state.quasi_immutable_deps = self.quasi_immutable_deps
-        self._clean_optimization_info(self._newoperations)
-        return exported_state, self._newoperations
-
-    def jump_to_preamble(self, cell_token, jump_op):
-        assert cell_token.target_tokens[0].virtual_state is None
-        jump_op = jump_op.copy_and_change(
-            rop.JUMP, descr=cell_token.target_tokens[0])
-        self.send_extra_operation(jump_op)
-
-
-class OptUnroll(Optimization):
+class UnrollOptimizer(Optimization):
     """Unroll the loop into two iterations. The first one will
     become the preamble or entry bridge (don't think there is a
     distinction anymore)"""
 
     short_preamble_producer = None
 
+    def __init__(self, metainterp_sd, jitdriver_sd, optimizations):
+        self.optimizer = UnrollableOptimizer(metainterp_sd, jitdriver_sd,
+                                             optimizations)
+        self.optimizer.optunroll = self
+
     def get_virtual_state(self, args):
         modifier = VirtualStateConstructor(self.optimizer)
         return modifier.get_virtual_state(args)
@@ -262,6 +122,100 @@ class OptUnroll(Optimization):
             assert not self.optimizer._newoperations
 
 
+    def optimize_preamble(self, trace, runtime_boxes, call_pure_results, memo):
+        info, newops = self.optimizer.propagate_all_forward(
+            trace.get_iter(), call_pure_results, flush=False)
+        exported_state = self.export_state(info.jump_op.getarglist(),
+                                           info.inputargs,
+                                           runtime_boxes, memo)
+        exported_state.quasi_immutable_deps = info.quasi_immutable_deps
+        # we need to absolutely make sure that we've cleaned up all
+        # the optimization info
+        self.optimizer._clean_optimization_info(self.optimizer._newoperations)
+        return exported_state, self.optimizer._newoperations
+
+    def optimize_peeled_loop(self, trace, celltoken, state,
+                             call_pure_results, inline_short_preamble=True):
+        trace = trace.get_iter()
+        try:
+            label_args = self.import_state(trace.inputargs, state)
+        except VirtualStatesCantMatch:
+            raise InvalidLoop("Cannot import state, virtual states don't match")
+        self.potential_extra_ops = {}
+        self.optimizer.init_inparg_dict_from(label_args)
+        try:
+            info, _ = self.optimizer.propagate_all_forward(
+                trace, call_pure_results, flush=False)
+        except SpeculativeError:
+            raise InvalidLoop("Speculative heap access would be ill-typed")
+        end_jump = info.jump_op
+        label_op = ResOperation(rop.LABEL, label_args,
+                                descr=celltoken)
+        for a in end_jump.getarglist():
+            self.optimizer.force_box_for_end_of_preamble(
+                self.optimizer.get_box_replacement(a))
+        current_vs = self.get_virtual_state(end_jump.getarglist())
+        # pick the vs we want to jump to
+        assert isinstance(celltoken, JitCellToken)
+
+        target_virtual_state = self.pick_virtual_state(current_vs,
+                                                       state.virtual_state,
+                                                celltoken.target_tokens)
+        # force the boxes for virtual state to match
+        try:
+            args = target_virtual_state.make_inputargs(
+               [self.get_box_replacement(x) for x in end_jump.getarglist()],
+               self.optimizer, force_boxes=True)
+            for arg in args:
+                if arg is not None:
+                    self.optimizer.force_box(arg)
+        except VirtualStatesCantMatch:
+            raise InvalidLoop("Virtual states did not match "
+                              "after picking the virtual state, when forcing"
+                              " boxes")
+        extra_same_as = self.short_preamble_producer.extra_same_as[:]
+        target_token = self.finalize_short_preamble(label_op,
+                                                    state.virtual_state)
+        label_op.setdescr(target_token)
+
+        if not inline_short_preamble:
+            self.jump_to_preamble(celltoken, end_jump, info)
+            return (UnrollInfo(target_token, label_op, extra_same_as,
+                               self.optimizer.quasi_immutable_deps),
+                    self.optimizer._newoperations)
+
+        try:
+            new_virtual_state = self.jump_to_existing_trace(
+                    end_jump, label_op, state.runtime_boxes, force_boxes=False)
+        except InvalidLoop:
+            # inlining short preamble failed, jump to preamble
+            self.jump_to_preamble(celltoken, end_jump, info)
+            return (UnrollInfo(target_token, label_op, extra_same_as,
+                               self.optimizer.quasi_immutable_deps),
+                    self.optimizer._newoperations)
+
+        if new_virtual_state is not None:
+            # Attempt to force virtual boxes in order to avoid jumping
+            # to the preamble.
+            try:
+                new_virtual_state = self.jump_to_existing_trace(
+                        end_jump, label_op, state.runtime_boxes, force_boxes=True)
+            except InvalidLoop:
+                pass
+
+        if new_virtual_state is not None:
+            self.jump_to_preamble(celltoken, end_jump, info)
+            return (UnrollInfo(target_token, label_op, extra_same_as,
+                               self.optimizer.quasi_immutable_deps),
+                    self.optimizer._newoperations)
+
+        self.disable_retracing_if_max_retrace_guards(
+            self.optimizer._newoperations, target_token)
+
+        return (UnrollInfo(target_token, label_op, extra_same_as,
+                           self.optimizer.quasi_immutable_deps),
+                self.optimizer._newoperations)
+
     def disable_retracing_if_max_retrace_guards(self, ops, target_token):
         maxguards = self.optimizer.metainterp_sd.warmrunnerdesc.memory_manager.max_retrace_guards
         count = 0
@@ -282,6 +236,52 @@ class OptUnroll(Optimization):
                 return token.virtual_state
         return label_vs
 
+    def optimize_bridge(self, trace, runtime_boxes, call_pure_results,
+                        inline_short_preamble, box_names_memo):
+        trace = trace.get_iter()
+        self._check_no_forwarding([trace.inputargs])
+        info, ops = self.optimizer.propagate_all_forward(trace,
+            call_pure_results, False)
+        jump_op = info.jump_op
+        cell_token = jump_op.getdescr()
+        assert isinstance(cell_token, JitCellToken)
+        if not inline_short_preamble or len(cell_token.target_tokens) == 1:
+            return self.jump_to_preamble(cell_token, jump_op, info)
+        # force all the information that does not go to the short
+        # preamble at all
+        self.optimizer.flush()
+        for a in jump_op.getarglist():
+            self.optimizer.force_box_for_end_of_preamble(a)
+        try:
+            vs = self.jump_to_existing_trace(jump_op, None, runtime_boxes,
+                                             force_boxes=False)
+        except InvalidLoop:
+            return self.jump_to_preamble(cell_token, jump_op, info)
+        if vs is None:
+            return info, self.optimizer._newoperations[:]
+        warmrunnerdescr = self.optimizer.metainterp_sd.warmrunnerdesc
+        limit = warmrunnerdescr.memory_manager.retrace_limit
+        if cell_token.retraced_count < limit:
+            cell_token.retraced_count += 1
+            debug_print('Retracing (%d/%d)' % (cell_token.retraced_count, limit))
+        else:
+            # Try forcing boxes to avoid jumping to the preamble
+            try:
+                vs = self.jump_to_existing_trace(jump_op, None, runtime_boxes,
+                                                 force_boxes=True)
+            except InvalidLoop:
+                pass
+            if vs is None:
+                return info, self.optimizer._newoperations[:]
+            debug_print("Retrace count reached, jumping to preamble")
+            return self.jump_to_preamble(cell_token, jump_op, info)
+        exported_state = self.export_state(info.jump_op.getarglist(),
+                                           info.inputargs, runtime_boxes,
+                                           box_names_memo)
+        exported_state.quasi_immutable_deps = self.optimizer.quasi_immutable_deps
+        self.optimizer._clean_optimization_info(self.optimizer._newoperations)
+        return exported_state, self.optimizer._newoperations
+
     def finalize_short_preamble(self, label_op, virtual_state):
         sb = self.short_preamble_producer
         self.optimizer._clean_optimization_info(sb.short_inputargs)
@@ -301,28 +301,19 @@ class OptUnroll(Optimization):
         label_op.initarglist(label_op.getarglist() + sb.used_boxes)
         return target_token
 
+    def jump_to_preamble(self, cell_token, jump_op, info):
+        assert cell_token.target_tokens[0].virtual_state is None
+        jump_op = jump_op.copy_and_change(rop.JUMP,
+                                          descr=cell_token.target_tokens[0])
+        self.optimizer.send_extra_operation(jump_op)
+        return info, self.optimizer._newoperations[:]
+
 
     def jump_to_existing_trace(self, jump_op, label_op, runtime_boxes, force_boxes=False):
-        # there is a big conceptual problem here: it's not safe at all to catch
-        # InvalidLoop in the callers of _jump_to_existing_trace and then
-        # continue trying to jump to some other label, because inlining the
-        # short preamble could have worked partly, leaving some unwanted new
-        # ops at the end of the trace. Here's at least a stopgap to stop
-        # terrible things from happening: we *must not* move any of those bogus
-        # guards earlier into the trace. see
-        # test_unroll_shortpreamble_mutates_bug in test_loop, and issue #3598
-
-        # leaving the bogus operations at the end of the trace is not great,
-        # but should be safe: at worst, they just always do a bit of stuff and
-        # then fail
-        with self.optimizer.cant_replace_guards():
-            return self._jump_to_existing_trace(jump_op, label_op, runtime_boxes, force_boxes)
-
-    def _jump_to_existing_trace(self, jump_op, label_op, runtime_boxes, force_boxes=False):
         jitcelltoken = jump_op.getdescr()
         assert isinstance(jitcelltoken, JitCellToken)
         virtual_state = self.get_virtual_state(jump_op.getarglist())
-        args = [get_box_replacement(op) for op in jump_op.getarglist()]
+        args = [self.get_box_replacement(op) for op in jump_op.getarglist()]
         for target_token in jitcelltoken.target_tokens:
             target_virtual_state = target_token.virtual_state
             if target_virtual_state is None:
@@ -336,7 +327,7 @@ class OptUnroll(Optimization):
                     if isinstance(guard, GuardResOp):
                         guard.rd_resume_position = patchguardop.rd_resume_position
                         guard.setdescr(compile.ResumeAtPositionDescr())
-                    self.optimizer.send_extra_operation(guard)
+                    self.send_extra_operation(guard)
             except VirtualStatesCantMatch:
                 continue
 
@@ -352,14 +343,20 @@ class OptUnroll(Optimization):
                 continue
 
             short_preamble = target_token.short_preamble
-            extra = self.inline_short_preamble(args + virtuals, args,
-                                short_preamble, self.optimizer.patchguardop,
-                                target_token, label_op)
-            self.optimizer.send_extra_operation(jump_op.copy_and_change(rop.JUMP,
+            try:
+                extra = self.inline_short_preamble(args + virtuals, args,
+                                    short_preamble, self.optimizer.patchguardop,
+                                    target_token, label_op)
+            except KeyError:
+                # SHOULD NOT OCCUR BUT DOES: WHY??  issue #2185
+                self.optimizer.metainterp_sd.logger_ops.log_short_preamble([],
+                    short_preamble, {})
+                raise
+
+            self.send_extra_operation(jump_op.copy_and_change(rop.JUMP,
                                       args=args + extra,
                                       descr=target_token))
             return None # explicit because the return can be non-None
-
         return virtual_state
 
     def _map_args(self, mapping, arglist):
@@ -414,21 +411,19 @@ class OptUnroll(Optimization):
                 i += 1
                 self.optimizer.send_extra_operation(op)
             # force all of them except the virtuals
-            for arg in (args_no_virtuals +
-                        self._map_args(mapping, short_jump_args)):
-                self.optimizer.force_box(get_box_replacement(arg))
+            for arg in args_no_virtuals + short_jump_args:
+                self.optimizer.force_box(self.get_box_replacement(arg))
             self.optimizer.flush()
             # done unless "short" has grown again
             if i == len(short) - 1:
                 break
 
-        return [get_box_replacement(box)
+        return [self.get_box_replacement(box)
                 for box in self._map_args(mapping, short_jump_args)]
 
     def _expand_info(self, arg, infos):
-        arg1 = self.optimizer.as_operation(arg)
-        if arg1 is not None and rop.is_same_as(arg1.opnum):
-            info = self.optimizer.getinfo(arg1.getarg(0))
+        if isinstance(arg, AbstractResOp) and rop.is_same_as(arg.opnum):
+            info = self.optimizer.getinfo(arg.getarg(0))
         else:
             info = self.optimizer.getinfo(arg)
         if arg in infos:
@@ -451,7 +446,7 @@ class OptUnroll(Optimization):
                     for a in original_label_args]
         self.optimizer.flush()
         virtual_state = self.get_virtual_state(end_args)
-        end_args = [get_box_replacement(arg) for arg in end_args]
+        end_args = [self.get_box_replacement(arg) for arg in end_args]
         infos = {}
         for arg in end_args:
             self._expand_info(arg, infos)
@@ -475,7 +470,8 @@ class OptUnroll(Optimization):
     def import_state(self, targetargs, exported_state):
         # the mapping between input args (from old label) and what we need
         # to actually emit. Update the info
-        assert len(exported_state.next_iteration_args) == len(targetargs)
+        assert (len(exported_state.next_iteration_args) ==
+                len(targetargs))
         for i, target in enumerate(exported_state.next_iteration_args):
             source = targetargs[i]
             assert source is not target
