@@ -1,13 +1,14 @@
 from rpython.jit.codewriter.effectinfo import EffectInfo
 from rpython.jit.metainterp.history import (Const, ConstInt, ConstPtr,
     get_const_ptr_for_string, get_const_ptr_for_unicode, REF, INT,
-    DONT_CHANGE, CONST_NULL)
-from rpython.jit.metainterp.optimizeopt.optimizer import (
-    CONST_0, CONST_1, REMOVED, Optimization)
-from rpython.jit.metainterp.optimizeopt.util import (
-    make_dispatcher_method, have_dispatcher_method, get_box_replacement)
-from rpython.jit.metainterp.resoperation import rop, ResOperation
-from .info import AbstractVirtualPtrInfo, getptrinfo
+    DONT_CHANGE)
+from rpython.jit.metainterp.optimizeopt import optimizer
+from rpython.jit.metainterp.optimizeopt.optimizer import CONST_0, CONST_1
+from rpython.jit.metainterp.optimizeopt.optimizer import llhelper, REMOVED
+from rpython.jit.metainterp.optimizeopt.util import make_dispatcher_method
+from rpython.jit.metainterp.resoperation import rop, ResOperation,\
+     AbstractResOp
+from rpython.jit.metainterp.optimizeopt import info
 from rpython.rlib.objectmodel import specialize, we_are_translated
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rtyper import annlowlevel
@@ -47,7 +48,8 @@ mode_unicode = StrOrUnicode(rstr.UNICODE, annlowlevel.hlunicode, u'', unichr,
 # ____________________________________________________________
 
 
-class StrPtrInfo(AbstractVirtualPtrInfo):
+
+class StrPtrInfo(info.AbstractVirtualPtrInfo):
     #_attrs_ = ('length', 'lenbound', 'lgtop', 'mode', '_cached_vinfo', '_is_virtual')
 
     lenbound = None
@@ -58,6 +60,7 @@ class StrPtrInfo(AbstractVirtualPtrInfo):
         self.length = length
         self._is_virtual = is_virtual
         self.mode = mode
+        self.length = length
 
     def getlenbound(self, mode):
         from rpython.jit.metainterp.optimizeopt import intutils
@@ -70,8 +73,8 @@ class StrPtrInfo(AbstractVirtualPtrInfo):
         return self.lenbound
 
     @specialize.arg(2)
-    def get_constant_string_spec(self, optstring, mode):
-        return None  # can't be constant
+    def get_constant_string_spec(self, string_optimizer, mode):
+        return None # can't be constant
 
     def force_box(self, op, optforce):
         if not self.is_virtual():
@@ -80,13 +83,13 @@ class StrPtrInfo(AbstractVirtualPtrInfo):
             s = self.get_constant_string_spec(optforce, mode_string)
             if s is not None:
                 c_s = get_const_ptr_for_string(s)
-                get_box_replacement(op).set_forwarded(c_s)
+                optforce.get_box_replacement(op).set_forwarded(c_s)
                 return c_s
         else:
             s = self.get_constant_string_spec(optforce, mode_unicode)
             if s is not None:
                 c_s = get_const_ptr_for_unicode(s)
-                get_box_replacement(op).set_forwarded(c_s)
+                optforce.get_box_replacement(op).set_forwarded(c_s)
                 return c_s
         self._is_virtual = False
         lengthbox = self.getstrlen(op, optforce.optimizer.optstring, self.mode)
@@ -94,32 +97,33 @@ class StrPtrInfo(AbstractVirtualPtrInfo):
         if not we_are_translated():
             newop.name = 'FORCE'
         optforce.emit_extra(newop)
-        newop = optforce.optimizer.getlastop()
+        newop = optforce.getlastop()
         newop.set_forwarded(self)
-        op = get_box_replacement(op)
+        op = optforce.get_box_replacement(op)
         op.set_forwarded(newop)
         optstring = optforce.optimizer.optstring
         self.initialize_forced_string(op, optstring, op, CONST_0, self.mode)
         return newop
 
-    def initialize_forced_string(self, op, optstring, targetbox,
+    def initialize_forced_string(self, op, string_optimizer, targetbox,
                                  offsetbox, mode):
-        return self.string_copy_parts(op, optstring, targetbox,
+        return self.string_copy_parts(op, string_optimizer, targetbox,
                                       offsetbox, mode)
 
-    def getstrlen(self, op, optstring, mode):
-        assert op is not None
+    def getstrlen(self, op, string_optimizer, mode, create_ops=True):
         if self.lgtop is not None:
             return self.lgtop
         assert not self.is_virtual()
+        if not create_ops:
+            return None
         lengthop = ResOperation(mode.STRLEN, [op])
         lengthop.set_forwarded(self.getlenbound(mode))
         self.lgtop = lengthop
-        optstring.emit_extra(lengthop)
+        string_optimizer.emit_extra(lengthop)
         return lengthop
 
     def make_guards(self, op, short, optimizer):
-        AbstractVirtualPtrInfo.make_guards(self, op, short, optimizer)
+        info.AbstractVirtualPtrInfo.make_guards(self, op, short, optimizer)
         if self.lenbound and self.lenbound.lower >= 1:
             if self.mode is mode_string:
                 lenop = ResOperation(rop.STRLEN, [op])
@@ -129,21 +133,21 @@ class StrPtrInfo(AbstractVirtualPtrInfo):
             short.append(lenop)
             self.lenbound.make_guards(lenop, short, optimizer)
 
-    def string_copy_parts(self, op, optstring, targetbox, offsetbox,
+    def string_copy_parts(self, op, string_optimizer, targetbox, offsetbox,
                           mode):
         # Copies the pointer-to-string 'self' into the target string
         # given by 'targetbox', at the specified offset.  Returns the offset
         # at the end of the copy.
-        lengthbox = self.getstrlen(op, optstring, mode)
-        srcbox = self.force_box(op, optstring)
-        return copy_str_content(optstring, srcbox, targetbox,
+        lengthbox = self.getstrlen(op, string_optimizer, mode)
+        srcbox = self.force_box(op, string_optimizer)
+        return copy_str_content(string_optimizer, srcbox, targetbox,
                                 CONST_0, offsetbox, lengthbox, mode)
-
+     
 class VStringPlainInfo(StrPtrInfo):
     #_attrs_ = ('mode', '_is_virtual')
 
     _chars = None
-
+    
     def __init__(self, mode, is_virtual, length):
         if length != -1:
             self._chars = [None] * length
@@ -168,8 +172,7 @@ class VStringPlainInfo(StrPtrInfo):
     def is_virtual(self):
         return self._is_virtual
 
-    def getstrlen(self, op, optstring, mode):
-        assert op is not None
+    def getstrlen(self, op, string_optimizer, mode, create_ops=True):
         if self.lgtop is None:
             self.lgtop = ConstInt(len(self._chars))
         return self.lgtop
@@ -182,29 +185,29 @@ class VStringPlainInfo(StrPtrInfo):
         return mode.emptystr.join([mode.chr(c.getint())
                                    for c in self._chars])
 
-    def string_copy_parts(self, op, optstring, targetbox, offsetbox,
+    def string_copy_parts(self, op, string_optimizer, targetbox, offsetbox,
                           mode):
         if not self.is_virtual():
-            return StrPtrInfo.string_copy_parts(self, op, optstring,
+            return StrPtrInfo.string_copy_parts(self, op, string_optimizer,
                                                 targetbox, offsetbox, mode)
         else:
-            return self.initialize_forced_string(op, optstring,
+            return self.initialize_forced_string(op, string_optimizer,
                                                  targetbox, offsetbox, mode)
 
-    def initialize_forced_string(self, op, optstring, targetbox,
+    def initialize_forced_string(self, op, string_optimizer, targetbox,
                                  offsetbox, mode):
         for i in range(len(self._chars)):
-            assert not isinstance(targetbox, Const)  # ConstPtr never makes sense
-            charbox = self.strgetitem(i)  # can't be virtual
+            assert not isinstance(targetbox, Const) # ConstPtr never makes sense
+            charbox = self.strgetitem(i) # can't be virtual
             if charbox is not None:
                 op = ResOperation(mode.STRSETITEM, [targetbox,
                                                     offsetbox,
                                                     charbox])
-                optstring.emit_extra(op)
-            offsetbox = _int_add(optstring, offsetbox, CONST_1)
+                string_optimizer.emit_extra(op)
+            offsetbox = _int_add(string_optimizer, offsetbox, CONST_1)
         return offsetbox
 
-    def _visitor_walk_recursive(self, instbox, visitor):
+    def _visitor_walk_recursive(self, instbox, visitor, optimizer):
         visitor.register_virtual_fields(instbox, self._chars)
 
     @specialize.argtype(1)
@@ -216,7 +219,7 @@ class VStringSliceInfo(StrPtrInfo):
     start = None
     lgtop = None
     s = None
-
+    
     def __init__(self, s, start, length, mode):
         self.s = s
         self.start = start
@@ -227,37 +230,36 @@ class VStringSliceInfo(StrPtrInfo):
     def is_virtual(self):
         return self._is_virtual
 
-    def string_copy_parts(self, op, optstring, targetbox, offsetbox,
+    def string_copy_parts(self, op, string_optimizer, targetbox, offsetbox,
                           mode):
-        return copy_str_content(optstring, self.s, targetbox,
+        return copy_str_content(string_optimizer, self.s, targetbox,
                                 self.start, offsetbox, self.lgtop, mode)
 
     @specialize.arg(2)
-    def get_constant_string_spec(self, optstring, mode):
-        vstart = optstring.getintbound(self.start)
-        vlength = optstring.getintbound(self.lgtop)
+    def get_constant_string_spec(self, string_optimizer, mode):
+        vstart = string_optimizer.getintbound(self.start)
+        vlength = string_optimizer.getintbound(self.lgtop)
         if vstart.is_constant() and vlength.is_constant():
-            vstr = getptrinfo(self.s)
-            s1 = vstr.get_constant_string_spec(optstring, mode)
+            vstr = string_optimizer.getptrinfo(self.s)
+            s1 = vstr.get_constant_string_spec(string_optimizer, mode)
             if s1 is None:
                 return None
-            start = vstart.get_constant_int()
-            length = vlength.get_constant_int()
+            start = vstart.getint()
+            length = vlength.getint()
             assert start >= 0
             assert length >= 0
-            return s1[start: start + length]
+            return s1[start : start + length]
         return None
 
-    def getstrlen(self, op, optstring, mode):
-        assert op is not None
+    def getstrlen(self, op, string_optimizer, mode, create_ops=True):
         return self.lgtop
 
-    def _visitor_walk_recursive(self, instbox, visitor):
+    def _visitor_walk_recursive(self, instbox, visitor, optimizer):
         boxes = [self.s, self.start, self.lgtop]
         visitor.register_virtual_fields(instbox, boxes)
-        opinfo = getptrinfo(self.s)
+        opinfo = optimizer.getptrinfo(self.s)
         if opinfo and opinfo.is_virtual():
-            opinfo.visitor_walk_recursive(self.s, visitor)
+            opinfo.visitor_walk_recursive(self.s, visitor, optimizer)
 
     @specialize.argtype(1)
     def visitor_dispatch_virtual_type(self, visitor):
@@ -269,7 +271,7 @@ class VStringConcatInfo(StrPtrInfo):
     vleft = None
     vright = None
     _is_virtual = False
-
+    
     def __init__(self, mode, vleft, vright, is_virtual):
         self.vleft = vleft
         self.vright = vright
@@ -278,97 +280,98 @@ class VStringConcatInfo(StrPtrInfo):
     def is_virtual(self):
         return self._is_virtual
 
-    def getstrlen(self, op, optstring, mode):
-        assert op is not None
+    def getstrlen(self, op, string_optimizer, mode, create_ops=True):
         if self.lgtop is not None:
             return self.lgtop
-        lefti = getptrinfo(self.vleft)
-        len1box = lefti.getstrlen(self.vleft, optstring, mode)
+        lefti = string_optimizer.getptrinfo(self.vleft)
+        len1box = lefti.getstrlen(self.vleft, string_optimizer, mode,
+                                  create_ops)
         if len1box is None:
             return None
-        righti = getptrinfo(self.vright)
-        len2box = righti.getstrlen(self.vright, optstring, mode)
+        righti = string_optimizer.getptrinfo(self.vright)
+        len2box = righti.getstrlen(self.vright, string_optimizer, mode,
+                                   create_ops)
         if len2box is None:
             return None
-        self.lgtop = _int_add(optstring, len1box, len2box)
-            # ^^^ may still be None, if optstring is None
+        self.lgtop = _int_add(string_optimizer, len1box, len2box, create_ops)
+            # ^^^ may still be None, if string_optimizer is None
         return self.lgtop
 
     @specialize.arg(2)
-    def get_constant_string_spec(self, optstring, mode):
-        ileft = getptrinfo(self.vleft)
-        s1 = ileft.get_constant_string_spec(optstring, mode)
+    def get_constant_string_spec(self, string_optimizer, mode):
+        ileft = string_optimizer.getptrinfo(self.vleft)
+        s1 = ileft.get_constant_string_spec(string_optimizer, mode)
         if s1 is None:
             return None
-        iright = getptrinfo(self.vright)
-        s2 = iright.get_constant_string_spec(optstring, mode)
+        iright = string_optimizer.getptrinfo(self.vright)
+        s2 = iright.get_constant_string_spec(string_optimizer, mode)
         if s2 is None:
             return None
         return s1 + s2
 
-    def string_copy_parts(self, op, optstring, targetbox, offsetbox,
+    def string_copy_parts(self, op, string_optimizer, targetbox, offsetbox,
                           mode):
-        lefti = getptrinfo(self.vleft)
-        offsetbox = lefti.string_copy_parts(self.vleft, optstring,
+        lefti = string_optimizer.getptrinfo(self.vleft)
+        offsetbox = lefti.string_copy_parts(self.vleft, string_optimizer,
                                             targetbox, offsetbox, mode)
-        righti = getptrinfo(self.vright)
-        offsetbox = righti.string_copy_parts(self.vright, optstring,
+        righti = string_optimizer.getptrinfo(self.vright)
+        offsetbox = righti.string_copy_parts(self.vright, string_optimizer,
                                              targetbox, offsetbox, mode)
         return offsetbox
 
-    def _visitor_walk_recursive(self, instbox, visitor):
+    def _visitor_walk_recursive(self, instbox, visitor, optimizer):
         # we don't store the lengthvalue in guards, because the
         # guard-failed code starts with a regular STR_CONCAT again
         leftbox = self.vleft
         rightbox = self.vright
         visitor.register_virtual_fields(instbox, [leftbox, rightbox])
-        leftinfo = getptrinfo(leftbox)
-        rightinfo = getptrinfo(rightbox)
+        leftinfo = optimizer.getptrinfo(leftbox)
+        rightinfo = optimizer.getptrinfo(rightbox)
         if leftinfo and leftinfo.is_virtual():
-            leftinfo.visitor_walk_recursive(leftbox, visitor)
+            leftinfo.visitor_walk_recursive(leftbox, visitor, optimizer)
         if rightinfo and rightinfo.is_virtual():
-            rightinfo.visitor_walk_recursive(rightbox, visitor)
+            rightinfo.visitor_walk_recursive(rightbox, visitor, optimizer)
 
     @specialize.argtype(1)
     def visitor_dispatch_virtual_type(self, visitor):
         return visitor.visit_vstrconcat(self.mode is mode_unicode)
 
 
-def copy_str_content(optstring, srcbox, targetbox,
+def copy_str_content(string_optimizer, srcbox, targetbox,
                      srcoffsetbox, offsetbox, lengthbox, mode,
                      need_next_offset=True):
-    srcbox = get_box_replacement(srcbox)
-    srcoffset = optstring.getintbound(srcoffsetbox)
-    lgt = optstring.getintbound(lengthbox)
+    srcbox = string_optimizer.get_box_replacement(srcbox)
+    srcoffset = string_optimizer.getintbound(srcoffsetbox)
+    lgt = string_optimizer.getintbound(lengthbox)
     if isinstance(srcbox, ConstPtr) and srcoffset.is_constant():
         M = 5
     else:
         M = 2
-    if lgt.is_constant() and lgt.get_constant_int() <= M:
+    if lgt.is_constant() and lgt.getint() <= M:
         # up to M characters are done "inline", i.e. with STRGETITEM/STRSETITEM
         # instead of just a COPYSTRCONTENT.
-        for i in range(lgt.get_constant_int()):
-            charbox = optstring.strgetitem(None, srcbox, srcoffsetbox,
+        for i in range(lgt.getint()):
+            charbox = string_optimizer.strgetitem(None, srcbox, srcoffsetbox,
                                                   mode)
-            srcoffsetbox = _int_add(optstring, srcoffsetbox, CONST_1)
-            assert not isinstance(targetbox, Const)  # ConstPtr never makes sense
-            optstring.emit_extra(ResOperation(mode.STRSETITEM,
+            srcoffsetbox = _int_add(string_optimizer, srcoffsetbox, CONST_1)
+            assert not isinstance(targetbox, Const)# ConstPtr never makes sense
+            string_optimizer.emit_extra(ResOperation(mode.STRSETITEM,
                     [targetbox, offsetbox, charbox]))
-            offsetbox = _int_add(optstring, offsetbox, CONST_1)
+            offsetbox = _int_add(string_optimizer, offsetbox, CONST_1)
     else:
         if need_next_offset:
-            nextoffsetbox = _int_add(optstring, offsetbox, lengthbox)
+            nextoffsetbox = _int_add(string_optimizer, offsetbox, lengthbox)
         else:
             nextoffsetbox = None
         assert not isinstance(targetbox, Const)   # ConstPtr never makes sense
         op = ResOperation(mode.COPYSTRCONTENT, [srcbox, targetbox,
                                                 srcoffsetbox, offsetbox,
                                                 lengthbox])
-        optstring.emit_extra(op)
+        string_optimizer.emit_extra(op)
         offsetbox = nextoffsetbox
     return offsetbox
 
-def _int_add(optstring, box1, box2):
+def _int_add(string_optimizer, box1, box2, create_ops=True):
     if isinstance(box1, ConstInt):
         if box1.value == 0:
             return box2
@@ -376,21 +379,23 @@ def _int_add(optstring, box1, box2):
             return ConstInt(box1.value + box2.value)
     elif isinstance(box2, ConstInt) and box2.value == 0:
         return box1
+    if not create_ops:
+        return None
     op = ResOperation(rop.INT_ADD, [box1, box2])
-    optstring.optimizer.send_extra_operation(op)
+    string_optimizer.send_extra_operation(op)
     return op
 
-def _int_sub(optstring, box1, box2):
+def _int_sub(string_optimizer, box1, box2):
     if isinstance(box2, ConstInt):
         if box2.value == 0:
             return box1
         if isinstance(box1, ConstInt):
             return ConstInt(box1.value - box2.value)
     op = ResOperation(rop.INT_SUB, [box1, box2])
-    optstring.optimizer.send_extra_operation(op)
+    string_optimizer.send_extra_operation(op)
     return op
 
-def _strgetitem(optstring, strbox, indexbox, mode, resbox=None):
+def _strgetitem(string_optimizer, strbox, indexbox, mode, resbox=None):
     if isinstance(strbox, ConstPtr) and isinstance(indexbox, ConstInt):
         if mode is mode_string:
             s = strbox.getref(lltype.Ptr(rstr.STR))
@@ -399,18 +404,18 @@ def _strgetitem(optstring, strbox, indexbox, mode, resbox=None):
             s = strbox.getref(lltype.Ptr(rstr.UNICODE))
             resnewbox = ConstInt(ord(s.chars[indexbox.getint()]))
         if resbox is not None:
-            optstring.make_equal_to(resbox, resnewbox)
+            string_optimizer.make_equal_to(resbox, resnewbox)
         return resnewbox
     if resbox is None:
         resbox = ResOperation(mode.STRGETITEM, [strbox, indexbox])
     else:
-        resbox = optstring.replace_op_with(resbox, mode.STRGETITEM,
+        resbox = string_optimizer.replace_op_with(resbox, mode.STRGETITEM,
                                                   [strbox, indexbox])
-    optstring.emit_extra(resbox)
+    string_optimizer.emit_extra(resbox)
     return resbox
 
 
-class OptString(Optimization):
+class OptString(optimizer.Optimization):
     "Handling of strings and unicodes."
 
     def setup(self):
@@ -419,6 +424,9 @@ class OptString(Optimization):
     def propagate_forward(self, op):
         return dispatch_opt(self, op)
 
+    def propagate_postprocess(self, op):
+        return dispatch_postprocess(self, op)
+
     def make_vstring_plain(self, op, mode, length):
         vvalue = VStringPlainInfo(mode, True, length)
         op = self.replace_op_with(op, op.getopnum())
@@ -439,7 +447,6 @@ class OptString(Optimization):
 
     def optimize_NEWSTR(self, op):
         return self._optimize_NEWSTR(op, mode_string)
-
     def optimize_NEWUNICODE(self, op):
         return self._optimize_NEWSTR(op, mode_unicode)
 
@@ -459,7 +466,7 @@ class OptString(Optimization):
         self.pure_from_args(mode_unicode.STRLEN, [op], op.getarg(0))
 
     def optimize_STRSETITEM(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo:
             assert not opinfo.is_constant()
             # strsetitem(ConstPtr) never makes sense
@@ -467,7 +474,7 @@ class OptString(Optimization):
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 opinfo.strsetitem(indexbox.getint(),
-                                  get_box_replacement(op.getarg(2)))
+                                  self.get_box_replacement(op.getarg(2)))
                 return
         self.make_nonnull(op.getarg(0))
         return self.emit(op)
@@ -476,7 +483,6 @@ class OptString(Optimization):
 
     def optimize_STRGETITEM(self, op):
         return self._optimize_STRGETITEM(op, mode_string)
-
     def optimize_UNICODEGETITEM(self, op):
         return self._optimize_STRGETITEM(op, mode_unicode)
 
@@ -485,28 +491,29 @@ class OptString(Optimization):
 
     def strgetitem(self, op, s, index, mode):
         self.make_nonnull_str(s, mode)
-        sinfo = getptrinfo(s)
+        sinfo = self.getptrinfo(s)
         #
         if isinstance(sinfo, VStringSliceInfo) and sinfo.is_virtual(): # slice
             index = _int_add(self.optimizer, sinfo.start, index)
             s = sinfo.s
-            sinfo = getptrinfo(sinfo.s)
+            sinfo = self.getptrinfo(sinfo.s)
         #
-        vindex = self.getintbound(index)
         if isinstance(sinfo, VStringPlainInfo):
             # even if no longer virtual
+            vindex = self.getintbound(index)
             if vindex.is_constant():
-                result = sinfo.strgetitem(vindex.get_constant_int())
+                result = sinfo.strgetitem(vindex.getint())
                 if result is not None:
                     if op is not None:
                         self.make_equal_to(op, result)
                     return result
         #
+        vindex = self.getintbound(index)
         if isinstance(sinfo, VStringConcatInfo) and vindex.is_constant():
-            leftinfo = getptrinfo(sinfo.vleft)
+            leftinfo = self.getptrinfo(sinfo.vleft)
             len1box = leftinfo.getstrlen(sinfo.vleft, self, mode)
             if isinstance(len1box, ConstInt):
-                raw_index = vindex.get_constant_int()
+                raw_index = vindex.getint()
                 len1 = len1box.getint()
                 if raw_index < len1:
                     return self.strgetitem(op, sinfo.vleft, index, mode)
@@ -518,30 +525,13 @@ class OptString(Optimization):
 
     def optimize_STRLEN(self, op):
         return self._optimize_STRLEN(op, mode_string)
-
     def optimize_UNICODELEN(self, op):
         return self._optimize_STRLEN(op, mode_unicode)
 
     def _optimize_STRLEN(self, op, mode):
-        arg1 = get_box_replacement(op.getarg(0))
-        opinfo = getptrinfo(arg1)
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo:
-            lgtop = opinfo.getstrlen(arg1, self, mode)
-            if lgtop is not None:
-                self.make_equal_to(op, lgtop)
-                return
-        return self.emit(op)
-
-    def optimize_STRHASH(self, op):
-        return self._optimize_STRHASH(op, mode_string)
-
-    def optimize_UNICODEHASH(self, op):
-        return self._optimize_STRHASH(op, mode_unicode)
-
-    def _optimize_STRHASH(self, op, mode):
-        opinfo = getptrinfo(op.getarg(0))
-        if opinfo:
-            lgtop = opinfo.getstrhash(op, mode)
+            lgtop = opinfo.getstrlen(op, self, mode, False)
             if lgtop is not None:
                 self.make_equal_to(op, lgtop)
                 return
@@ -560,22 +550,22 @@ class OptString(Optimization):
         assert op.getarg(2).type == INT
         assert op.getarg(3).type == INT
         assert op.getarg(4).type == INT
-        src = getptrinfo(op.getarg(0))
-        dst = getptrinfo(op.getarg(1))
+        src = self.getptrinfo(op.getarg(0))
+        dst = self.getptrinfo(op.getarg(1))
         srcstart = self.getintbound(op.getarg(2))
         dststart = self.getintbound(op.getarg(3))
         length = self.getintbound(op.getarg(4))
         dst_virtual = (isinstance(dst, VStringPlainInfo) and dst.is_virtual())
 
-        if length.is_constant() and length.get_constant_int() == 0:
+        if length.is_constant() and length.getint() == 0:
             return
         elif ((str and (src.is_virtual() or src.is_constant())) and
               srcstart.is_constant() and dststart.is_constant() and
               length.is_constant() and
-              (length.get_constant_int() < 20 or ((src.is_virtual() or src.is_constant()) and dst_virtual))):
-            src_start = srcstart.get_constant_int()
-            dst_start = dststart.get_constant_int()
-            actual_length = length.get_constant_int()
+              (length.getint() < 20 or ((src.is_virtual() or src.is_constant()) and dst_virtual))):
+            src_start = srcstart.getint()
+            dst_start = dststart.getint()
+            actual_length = length.getint()
             for index in range(actual_length):
                 vresult = self.strgetitem(None, op.getarg(0),
                                           ConstInt(index + src_start), mode)
@@ -636,7 +626,7 @@ class OptString(Optimization):
         # More generally, supporting non-constant but virtual cases is
         # not obvious, because of the exception UnicodeDecodeError that
         # can be raised by ll_str2unicode()
-        varg = getptrinfo(op.getarg(1))
+        varg = self.getptrinfo(op.getarg(1))
         s = None
         if varg:
             s = varg.get_constant_string_spec(self, mode_string)
@@ -654,14 +644,14 @@ class OptString(Optimization):
         self.make_nonnull_str(op.getarg(1), mode)
         self.make_nonnull_str(op.getarg(2), mode)
         self.make_vstring_concat(op, mode,
-                                 get_box_replacement(op.getarg(1)),
-                                 get_box_replacement(op.getarg(2)))
+                                 self.get_box_replacement(op.getarg(1)),
+                                 self.get_box_replacement(op.getarg(2)))
         self.last_emitted_operation = REMOVED
         return True, None
 
     def opt_call_stroruni_STR_SLICE(self, op, mode):
         self.make_nonnull_str(op.getarg(1), mode)
-        vstr = getptrinfo(op.getarg(1))
+        vstr = self.getptrinfo(op.getarg(1))
         vstart = self.getintbound(op.getarg(2))
         vstop = self.getintbound(op.getarg(3))
         #
@@ -671,8 +661,8 @@ class OptString(Optimization):
         #if (isinstance(vstr, VStringPlainInfo) and vstart.is_constant()
         #    and vstop.is_constant()):
         #    value = self.make_vstring_plain(op, mode, -1)
-        #    value.setup_slice(vstr._chars, vstart.get_constant_int(),
-        #                      vstop.get_constant_int())
+        #    value.setup_slice(vstr._chars, vstart.getint(),
+        #                      vstop.getint())
         #    return True, None
         #
         startbox = op.getarg(2)
@@ -690,17 +680,17 @@ class OptString(Optimization):
 
     @specialize.arg(2)
     def opt_call_stroruni_STR_EQUAL(self, op, mode):
-        arg1 = get_box_replacement(op.getarg(1))
-        arg2 = get_box_replacement(op.getarg(2))
-        i1 = getptrinfo(arg1)
-        i2 = getptrinfo(arg2)
+        arg1 = self.get_box_replacement(op.getarg(1))
+        arg2 = self.get_box_replacement(op.getarg(2))
+        i1 = self.getptrinfo(arg1)
+        i2 = self.getptrinfo(arg2)
         #
         if i1:
-            l1box = i1.getstrlen(arg1, self, mode)
+            l1box = i1.getstrlen(arg1, self, mode, create_ops=False)
         else:
             l1box = None
         if i2:
-            l2box = i2.getstrlen(arg2, self, mode)
+            l2box = i2.getstrlen(arg2, self, mode, create_ops=False)
         else:
             l2box = None
         if (l1box is not None and l2box is not None and
@@ -733,17 +723,17 @@ class OptString(Optimization):
         return False, None
 
     def handle_str_equal_level1(self, arg1, arg2, resultop, mode):
-        i1 = getptrinfo(arg1)
-        i2 = getptrinfo(arg2)
+        i1 = self.getptrinfo(arg1)
+        i2 = self.getptrinfo(arg2)
         l2box = None
         l1box = None
         if i2:
-            l2box = i2.getstrlen(arg2, self, mode)
+            l2box = i2.getstrlen(arg2, self, mode, create_ops=False)
         if isinstance(l2box, ConstInt):
             if l2box.value == 0:
                 if i1 and i1.is_nonnull():
                     self.make_nonnull_str(arg1, mode)
-                    i1 = getptrinfo(arg1)
+                    i1 = self.getptrinfo(arg1)
                     lengthbox = i1.getstrlen(arg1, self, mode)
                 else:
                     lengthbox = None
@@ -756,18 +746,21 @@ class OptString(Optimization):
                     return True, None
             if l2box.value == 1:
                 if i1:
-                    l1box = i1.getstrlen(arg1, self, mode)
+                    l1box = i1.getstrlen(arg1, self, mode, False)
                 if isinstance(l1box, ConstInt) and l1box.value == 1:
                     # comparing two single chars
-                    vchar1 = self.strgetitem(None, arg1, CONST_0, mode)
-                    vchar2 = self.strgetitem(None, arg2, CONST_0, mode)
+                    vchar1 = self.strgetitem(None, arg1, optimizer.CONST_0,
+                                             mode)
+                    vchar2 = self.strgetitem(None, arg2, optimizer.CONST_0,
+                                             mode)
                     seo = self.optimizer.send_extra_operation
                     op = self.optimizer.replace_op_with(resultop, rop.INT_EQ,
                                 [vchar1, vchar2], descr=DONT_CHANGE)
                     seo(op)
                     return True, None
                 if isinstance(i1, VStringSliceInfo):
-                    vchar = self.strgetitem(None, arg2, CONST_0, mode)
+                    vchar = self.strgetitem(None, arg2, optimizer.CONST_0,
+                                            mode)
                     do = EffectInfo.OS_STREQ_SLICE_CHAR
                     return True, self.generate_modified_call(do, [i1.s, i1.start,
                                                                   i1.lgtop, vchar],
@@ -780,23 +773,24 @@ class OptString(Optimization):
             if i1 and i1.is_null():
                 self.make_constant(resultop, CONST_1)
                 return True, None
-            op = self.optimizer.replace_op_with(
-                resultop, rop.PTR_EQ, [arg1, CONST_NULL], descr=DONT_CHANGE)
+            op = self.optimizer.replace_op_with(resultop, rop.PTR_EQ,
+                                                [arg1, llhelper.CONST_NULL],
+                                                descr=DONT_CHANGE)
             return True, self.emit(op)
         #
         return False, None
 
     def handle_str_equal_level2(self, arg1, arg2, resultbox, mode):
-        i1 = getptrinfo(arg1)
-        i2 = getptrinfo(arg2)
+        i1 = self.getptrinfo(arg1)
+        i2 = self.getptrinfo(arg2)
         l2box = None
         if i2:
-            l2box = i2.getstrlen(arg1, self, mode)
+            l2box = i2.getstrlen(arg1, self, mode, create_ops=False)
         if l2box:
             l2info = self.getintbound(l2box)
             if l2info.is_constant():
-                if l2info.get_constant_int() == 1:
-                    vchar = self.strgetitem(None, arg2, CONST_0, mode)
+                if l2info.getint() == 1:
+                    vchar = self.strgetitem(None, arg2, optimizer.CONST_0, mode)
                     if i1 and i1.is_nonnull():
                         do = EffectInfo.OS_STREQ_NONNULL_CHAR
                     else:
@@ -814,21 +808,19 @@ class OptString(Optimization):
         return False, None
 
     def opt_call_stroruni_STR_CMP(self, op, mode):
-        arg1 = get_box_replacement(op.getarg(1))
-        arg2 = get_box_replacement(op.getarg(2))
-        i1 = getptrinfo(arg1)
-        i2 = getptrinfo(arg2)
+        i1 = self.getptrinfo(op.getarg(1))
+        i2 = self.getptrinfo(op.getarg(2))
         if not i1 or not i2:
             return False, None
-        l1box = i1.getstrlen(arg1, self, mode)
-        l2box = i2.getstrlen(arg2, self, mode)
+        l1box = i1.getstrlen(None, self, mode, False)
+        l2box = i2.getstrlen(None, self, mode, False)
         if (l1box is not None and l2box is not None and
             isinstance(l1box, ConstInt) and
             isinstance(l2box, ConstInt) and
             l1box.getint() == l2box.getint() == 1):
             # comparing two single chars
-            char1 = self.strgetitem(None, op.getarg(1), CONST_0, mode)
-            char2 = self.strgetitem(None, op.getarg(2), CONST_0, mode)
+            char1 = self.strgetitem(None, op.getarg(1), optimizer.CONST_0, mode)
+            char2 = self.strgetitem(None, op.getarg(2), optimizer.CONST_0, mode)
             seo = self.optimizer.send_extra_operation
             op = self.replace_op_with(op, rop.INT_SUB, [char1, char2],
                                       descr=DONT_CHANGE)
@@ -837,13 +829,13 @@ class OptString(Optimization):
         return False, None
 
     def opt_call_SHRINK_ARRAY(self, op):
-        i1 = getptrinfo(op.getarg(1))
+        i1 = self.getptrinfo(op.getarg(1))
         i2 = self.getintbound(op.getarg(2))
         # If the index is constant, if the argument is virtual (we only support
         # VStringPlainValue for now) we can optimize away the call.
         if (i2 and i2.is_constant() and i1 and i1.is_virtual() and
             isinstance(i1, VStringPlainInfo)):
-            length = i2.get_constant_int()
+            length = i2.getint()
             i1.shrink(length)
             self.last_emitted_operation = REMOVED
             self.make_equal_to(op, op.getarg(1))
@@ -862,8 +854,7 @@ class OptString(Optimization):
 
 dispatch_opt = make_dispatcher_method(OptString, 'optimize_',
                                       default=OptString.emit)
-OptString.propagate_postprocess = make_dispatcher_method(OptString, 'postprocess_')
-OptString.have_postprocess_op = have_dispatcher_method(OptString, 'postprocess_')
+dispatch_postprocess = make_dispatcher_method(OptString, 'postprocess_')
 
 
 def _findall_call_oopspec():
