@@ -3,10 +3,10 @@ import sys, py
 from rpython.tool.sourcetools import func_with_new_name
 from rpython.rtyper.lltypesystem import lltype, llmemory
 from rpython.rtyper.annlowlevel import (llhelper, MixLevelHelperAnnotator,
-    hlstr, cast_instance_to_gcref, cast_gcref_to_instance)
+    cast_base_ptr_to_instance, hlstr, cast_instance_to_gcref)
 from rpython.rtyper.llannotation import lltype_to_annotation
-from rpython.rtyper.rclass import OBJECTPTR
 from rpython.annotator import model as annmodel
+from rpython.annotator.dictdef import DictDef
 from rpython.rtyper.llinterp import LLException
 from rpython.rtyper.test.test_llinterp import get_interpreter, clear_tcache
 from rpython.flowspace.model import SpaceOperation, Variable, Constant
@@ -52,19 +52,12 @@ def apply_jit(translator, backend_name="auto", inline=False,
         jd.jitdriver.is_recursive = True
     else:
         count_recursive = 0
-        invalid = 0
         for jd in warmrunnerdesc.jitdrivers_sd:
             count_recursive += jd.jitdriver.is_recursive
-            invalid += (jd.jitdriver.has_unique_id and
-                           not jd.jitdriver.is_recursive)
         if count_recursive == 0:
             raise Exception("if you have more than one jitdriver, at least"
                 " one of them has to be marked with is_recursive=True,"
                 " none found")
-        if invalid > 0:
-            raise Exception("found %d jitdriver(s) with 'get_unique_id=...' "
-                            "specified but without 'is_recursive=True'" %
-                            (invalid,))
     for jd in warmrunnerdesc.jitdrivers_sd:
         jd.warmstate.set_param_inlining(inline)
         jd.warmstate.set_param_vec(vec)
@@ -87,7 +80,7 @@ def ll_meta_interp(function, args, backendopt=False,
     return jittify_and_run(interp, graph, args, backendopt=backendopt, **kwds)
 
 def jittify_and_run(interp, graph, args, repeat=1, graph_and_interp_only=False,
-                    backendopt=False, trace_limit=2**14, inline=False,
+                    backendopt=False, trace_limit=sys.maxint, inline=False,
                     loop_longevity=0, retrace_limit=5, function_threshold=4,
                     disable_unrolling=sys.maxint,
                     enable_opts=ALL_OPTS_NAMES, max_retrace_guards=15,
@@ -219,15 +212,6 @@ def debug_checks():
     stats.maybe_view()
     stats.check_consistency()
 
-# ____________________________________________________________
-# always disabled hooks interface
-
-from rpython.rlib.jit import JitHookInterface
-
-class NoHooksInterface(JitHookInterface):
-    def are_hooks_enabled(self):
-        return False
-
 # ____________________________________________________________
 
 class WarmRunnerDesc(object):
@@ -268,7 +252,7 @@ class WarmRunnerDesc(object):
         else:
             self.jitcounter = counter.DeterministicJitCounter()
         #
-        self.make_hooks(policy.jithookiface)
+        self.hooks = policy.jithookiface
         self.make_virtualizable_infos()
         self.make_driverhook_graphs()
         self.make_enter_functions()
@@ -425,9 +409,9 @@ class WarmRunnerDesc(object):
         graph = copygraph(graph)
         [jmpp] = find_jit_merge_points([graph])
         graph.startblock = support.split_before_jit_merge_point(*jmpp)
-        # XXX this is incredibly obscure, but this is sometimes necessary
+        # XXX this is incredibly obscure, but this is sometiems necessary
         #     so we don't explode in checkgraph. for reasons unknown this
-        #     is not contained within simplify_graph
+        #     is not contanied within simplify_graph
         removenoops.remove_same_as(graph)
         # a crash in the following checkgraph() means that you forgot
         # to list some variable in greens=[] or reds=[] in JitDriver,
@@ -473,7 +457,7 @@ class WarmRunnerDesc(object):
         auto_inline_graphs(self.translator, graphs, 0.01)
 
     def build_cpu(self, CPUClass, translate_support_code=False,
-                  no_stats=False, no_stats_history=False, supports_floats=True,
+                  no_stats=False, supports_floats=True,
                   supports_longlong=True, supports_singlefloats=True,
                   **kwds):
         assert CPUClass is not None
@@ -482,10 +466,6 @@ class WarmRunnerDesc(object):
             stats = history.NoStats()
         else:
             stats = history.Stats(None)
-            if no_stats_history:
-                stats.set_history = lambda history: None
-                # ^^^ for test_jitiface.test_memmgr_release_all.  otherwise,
-                # stats.history attribute keeps the most recent loop alive
         self.stats = stats
         if translate_support_code:
             self.annhelper = MixLevelHelperAnnotator(self.translator.rtyper)
@@ -511,12 +491,6 @@ class WarmRunnerDesc(object):
             self.metainterp_sd.opencoder_model = Model
         self.stats.metainterp_sd = self.metainterp_sd
 
-    def make_hooks(self, hooks):
-        if hooks is None:
-            # interface not overridden, use a special one that is never enabled
-            hooks = NoHooksInterface()
-        self.hooks = hooks
-
     def make_virtualizable_infos(self):
         vinfos = {}
         for jd in self.jitdrivers_sd:
@@ -669,10 +643,10 @@ class WarmRunnerDesc(object):
         jd.num_green_args = len(jd._green_args_spec)
         jd.num_red_args = len(jd.red_args_types)
         RESTYPE = graph.getreturnvar().concretetype
-        jd._JIT_ENTER_FUNCTYPE = lltype.FuncType(ALLARGS, lltype.Void)
-        jd._PTR_JIT_ENTER_FUNCTYPE = lltype.Ptr(jd._JIT_ENTER_FUNCTYPE)
-        jd._PORTAL_FUNCTYPE = lltype.FuncType(ALLARGS, RESTYPE)
-        jd._PTR_PORTAL_FUNCTYPE = lltype.Ptr(jd._PORTAL_FUNCTYPE)
+        (jd._JIT_ENTER_FUNCTYPE,
+         jd._PTR_JIT_ENTER_FUNCTYPE) = self.cpu.ts.get_FuncType(ALLARGS, lltype.Void)
+        (jd._PORTAL_FUNCTYPE,
+         jd._PTR_PORTAL_FUNCTYPE) = self.cpu.ts.get_FuncType(ALLARGS, RESTYPE)
         #
         if jd.result_type == 'v':
             ASMRESTYPE = lltype.Void
@@ -684,8 +658,8 @@ class WarmRunnerDesc(object):
             ASMRESTYPE = lltype.Float
         else:
             assert False
-        jd._PTR_ASSEMBLER_HELPER_FUNCTYPE = lltype.Ptr(lltype.FuncType(
-            [llmemory.GCREF, llmemory.GCREF], ASMRESTYPE))
+        (_, jd._PTR_ASSEMBLER_HELPER_FUNCTYPE) = self.cpu.ts.get_FuncType(
+            [llmemory.GCREF, llmemory.GCREF], ASMRESTYPE)
 
     def rewrite_jitcell_accesses(self):
         jitdrivers_by_name = {}
@@ -695,7 +669,7 @@ class WarmRunnerDesc(object):
                 jitdrivers_by_name[name] = jd
         m = _find_jit_markers(self.translator.graphs,
                               ('get_jitcell_at_key', 'trace_next_iteration',
-                               'dont_trace_here', 'trace_next_iteration_hash', 'mark_as_being_traced'))
+                               'dont_trace_here', 'trace_next_iteration_hash'))
         accessors = {}
 
         def get_accessor(name, jitdriver_name, function, ARGS, green_arg_spec):
@@ -747,8 +721,6 @@ class WarmRunnerDesc(object):
                 func = JitCell.get_jitcell
             elif op.args[0].value == 'dont_trace_here':
                 func = JitCell.dont_trace_here
-            elif op.args[0].value == 'mark_as_being_traced':
-                func = JitCell.mark_as_being_traced
             elif op.args[0].value == 'trace_next_iteration_hash':
                 func = JitCell.trace_next_iteration_hash
             else:
@@ -856,19 +828,19 @@ class WarmRunnerDesc(object):
         # make sure we make a copy of function so it no longer belongs
         # to extregistry
         func = op.args[1].value
-        if func.__name__.startswith('stats_'):
+        if func.func_name.startswith('stats_'):
             # get special treatment since we rewrite it to a call that accepts
             # jit driver
             assert len(op.args) >= 3, ("%r must have a first argument "
                                        "(which is None)" % (func,))
-            func = func_with_new_name(func, func.__name__ + '_compiled')
+            func = func_with_new_name(func, func.func_name + '_compiled')
 
             def new_func(ignored, *args):
                 return func(self, *args)
             ARGS = [lltype.Void] + [arg.concretetype for arg in op.args[3:]]
         else:
             ARGS = [arg.concretetype for arg in op.args[2:]]
-            new_func = func_with_new_name(func, func.__name__ + '_compiled')
+            new_func = func_with_new_name(func, func.func_name + '_compiled')
         RESULT = op.result.concretetype
         FUNCPTR = lltype.Ptr(lltype.FuncType(ARGS, RESULT))
         ptr = self.helper_func(FUNCPTR, new_func)
@@ -915,8 +887,8 @@ class WarmRunnerDesc(object):
         #
         from rpython.jit.metainterp.warmstate import specialize_value
         from rpython.jit.metainterp.warmstate import unspecialize_value
-        portal_ptr = lltype.functionptr(
-            PORTALFUNC, 'portal', graph=portalgraph)
+        portal_ptr = self.cpu.ts.functionptr(PORTALFUNC, 'portal',
+                                         graph=portalgraph)
         jd._portal_ptr = portal_ptr
         #
         portalfunc_ARGS = []
@@ -938,6 +910,7 @@ class WarmRunnerDesc(object):
         RESULT = PORTALFUNC.RESULT
         result_kind = history.getkind(RESULT)
         assert result_kind.startswith(jd.result_type)
+        ts = self.cpu.ts
         state = jd.warmstate
         maybe_compile_and_run = jd._maybe_compile_and_run_fn
         EnterJitAssembler = jd._EnterJitAssembler
@@ -1000,11 +973,11 @@ class WarmRunnerDesc(object):
                         return e.result
                 #
                 if isinstance(e, jitexc.ExitFrameWithExceptionRef):
+                    value = ts.cast_to_baseclass(e.value)
                     if not we_are_translated():
-                        value = lltype.cast_opaque_ptr(OBJECTPTR, e.value)
-                        raise LLException(value.typeptr, value)
+                        raise LLException(ts.get_typeptr(value), value)
                     else:
-                        value = cast_gcref_to_instance(Exception, e.value)
+                        value = cast_base_ptr_to_instance(Exception, value)
                         assert value is not None
                         raise value
                 #
@@ -1092,10 +1065,10 @@ class WarmRunnerDesc(object):
 
         closures = {}
         graphs = self.translator.graphs
-        SET_PARAM_FUNC = lltype.Ptr(lltype.FuncType(
-            [lltype.Signed], lltype.Void))
-        SET_PARAM_STR_FUNC = lltype.Ptr(lltype.FuncType(
-            [lltype.Ptr(STR)], lltype.Void))
+        _, PTR_SET_PARAM_FUNCTYPE = self.cpu.ts.get_FuncType([lltype.Signed],
+                                                             lltype.Void)
+        _, PTR_SET_PARAM_STR_FUNCTYPE = self.cpu.ts.get_FuncType(
+            [lltype.Ptr(STR)], lltype.Void)
         def make_closure(jd, fullfuncname, is_string):
             if jd is None:
                 def closure(i):
@@ -1110,9 +1083,9 @@ class WarmRunnerDesc(object):
                         i = hlstr(i)
                     getattr(state, fullfuncname)(i)
             if is_string:
-                TP = SET_PARAM_STR_FUNC
+                TP = PTR_SET_PARAM_STR_FUNCTYPE
             else:
-                TP = SET_PARAM_FUNC
+                TP = PTR_SET_PARAM_FUNCTYPE
             funcptr = self.helper_func(TP, closure)
             return Constant(funcptr, TP)
         #
