@@ -97,7 +97,6 @@ class BaseGCTransformer(object):
         self.inline = inline
         if translator and inline:
             self.lltype_to_classdef = translator.rtyper.lltype_to_classdef_mapping()
-            self.raise_analyzer = RaiseAnalyzer(translator)
         self.graphs_to_inline = {}
         self.graph_dependencies = {}
         self.ll_finalizers_ptrs = []
@@ -114,36 +113,28 @@ class BaseGCTransformer(object):
         self.seen_graphs.add(graph)
         self.minimal_transform.add(graph)
 
-    def inline_helpers_into(self, graph):
+    def inline_helpers(self, graphs):
         from rpython.translator.backendopt.inline import iter_callsites
-        to_enum = []
-        for called, block, i in iter_callsites(graph, None):
-            if called in self.graphs_to_inline:
-                to_enum.append(called)
-        any_inlining = False
-        for inline_graph in to_enum:
-            try:
-                inline.inline_function(self.translator, inline_graph, graph,
-                                       self.lltype_to_classdef,
-                                       self.raise_analyzer,
-                                       cleanup=False)
-                any_inlining = True
-            except inline.CannotInline as e:
-                print 'CANNOT INLINE:', e
-                print '\t%s into %s' % (inline_graph, graph)
-                raise      # for now, make it a fatal error
-        cleanup_graph(graph)
-        if any_inlining:
-            constant_fold_graph(graph)
-        return any_inlining
-
-    def inline_helpers_and_postprocess(self, graphs):
+        raise_analyzer = RaiseAnalyzer(self.translator)
         for graph in graphs:
-            any_inlining = self.inline and self.inline_helpers_into(graph)
-            self.postprocess_graph(graph, any_inlining)
-
-    def postprocess_graph(self, graph, any_inlining):
-        pass
+            to_enum = []
+            for called, block, i in iter_callsites(graph, None):
+                if called in self.graphs_to_inline:
+                    to_enum.append(called)
+            must_constfold = False
+            for inline_graph in to_enum:
+                try:
+                    inline.inline_function(self.translator, inline_graph, graph,
+                                           self.lltype_to_classdef,
+                                           raise_analyzer,
+                                           cleanup=False)
+                    must_constfold = True
+                except inline.CannotInline as e:
+                    print 'CANNOT INLINE:', e
+                    print '\t%s into %s' % (inline_graph, graph)
+            cleanup_graph(graph)
+            if must_constfold:
+                constant_fold_graph(graph)
 
     def compute_borrowed_vars(self, graph):
         # the input args are borrowed, and stay borrowed for as long as they
@@ -210,9 +201,6 @@ class BaseGCTransformer(object):
         self.var_last_needed_in = None
         self.curr_block = None
 
-    def start_transforming_graph(self, graph):
-        pass    # for asmgcc.py
-
     def transform_graph(self, graph):
         if graph in self.minimal_transform:
             if self.minimalgctransformer:
@@ -222,7 +210,6 @@ class BaseGCTransformer(object):
         if graph in self.seen_graphs:
             return
         self.seen_graphs.add(graph)
-        self.start_transforming_graph(graph)
 
         self.links_to_split = {} # link -> vars to pop_alive across the link
 
@@ -387,6 +374,9 @@ class BaseGCTransformer(object):
         return hop.cast_result(rmodel.inputconst(lltype.Ptr(ARRAY_TYPEID_MAP),
                                         lltype.nullptr(ARRAY_TYPEID_MAP)))
 
+    def get_prebuilt_hash(self, obj):
+        return None
+
 
 class MinimalGCTransformer(BaseGCTransformer):
     def __init__(self, parenttransformer):
@@ -424,7 +414,7 @@ MinimalGCTransformer.MinimalGCTransformer = None
 
 # ________________________________________________________________
 
-def mallocHelpers(gckind):
+def mallocHelpers():
     class _MallocHelpers(object):
         def _freeze_(self):
             return True
@@ -437,13 +427,6 @@ def mallocHelpers(gckind):
         return result
     mh._ll_malloc_fixedsize = _ll_malloc_fixedsize
 
-    def _ll_malloc_fixedsize_zero(size):
-        result = mh.allocate(size, zero=True)
-        if not result:
-            raise MemoryError()
-        return result
-    mh._ll_malloc_fixedsize_zero = _ll_malloc_fixedsize_zero
-
     def _ll_compute_size(length, size, itemsize):
         try:
             varsize = ovfcheck(itemsize * length)
@@ -462,25 +445,18 @@ def mallocHelpers(gckind):
     mh._ll_malloc_varsize_no_length = _ll_malloc_varsize_no_length
     mh.ll_malloc_varsize_no_length = _ll_malloc_varsize_no_length
 
-    if gckind == 'raw':
-        llopstore = llop.raw_store
-    elif gckind == 'gc':
-        llopstore = llop.gc_store
-    else:
-        raise AssertionError(gckind)
-
-
     def ll_malloc_varsize(length, size, itemsize, lengthoffset):
         result = mh.ll_malloc_varsize_no_length(length, size, itemsize)
-        llopstore(lltype.Void, result, lengthoffset, length)
+        (result + lengthoffset).signed[0] = length
         return result
     mh.ll_malloc_varsize = ll_malloc_varsize
 
     def _ll_malloc_varsize_no_length_zero(length, size, itemsize):
         tot_size = _ll_compute_size(length, size, itemsize)
-        result = mh.allocate(tot_size, zero=True)
+        result = mh.allocate(tot_size)
         if not result:
             raise MemoryError()
+        llmemory.raw_memclear(result, tot_size)
         return result
     mh.ll_malloc_varsize_no_length_zero = _ll_malloc_varsize_no_length_zero
 
@@ -491,19 +467,20 @@ class GCTransformer(BaseGCTransformer):
     def __init__(self, translator, inline=False):
         super(GCTransformer, self).__init__(translator, inline=inline)
 
-        mh = mallocHelpers(gckind='raw')
+        mh = mallocHelpers()
         mh.allocate = llmemory.raw_malloc
         ll_raw_malloc_fixedsize = mh._ll_malloc_fixedsize
-        ll_raw_malloc_fixedsize_zero = mh._ll_malloc_fixedsize_zero
         ll_raw_malloc_varsize_no_length = mh.ll_malloc_varsize_no_length
         ll_raw_malloc_varsize = mh.ll_malloc_varsize
         ll_raw_malloc_varsize_no_length_zero  = mh.ll_malloc_varsize_no_length_zero
 
+        stack_mh = mallocHelpers()
+        stack_mh.allocate = lambda size: llop.stack_malloc(llmemory.Address, size)
+        ll_stack_malloc_fixedsize = stack_mh._ll_malloc_fixedsize
+
         if self.translator:
             self.raw_malloc_fixedsize_ptr = self.inittime_helper(
                 ll_raw_malloc_fixedsize, [lltype.Signed], llmemory.Address)
-            self.raw_malloc_fixedsize_zero_ptr = self.inittime_helper(
-                ll_raw_malloc_fixedsize_zero, [lltype.Signed], llmemory.Address)
             self.raw_malloc_varsize_no_length_ptr = self.inittime_helper(
                 ll_raw_malloc_varsize_no_length, [lltype.Signed]*3, llmemory.Address, inline=False)
             self.raw_malloc_varsize_ptr = self.inittime_helper(
@@ -511,6 +488,9 @@ class GCTransformer(BaseGCTransformer):
             self.raw_malloc_varsize_no_length_zero_ptr = self.inittime_helper(
                 ll_raw_malloc_varsize_no_length_zero, [lltype.Signed]*3, llmemory.Address, inline=False)
 
+            self.stack_malloc_fixedsize_ptr = self.inittime_helper(
+                ll_stack_malloc_fixedsize, [lltype.Signed], llmemory.Address)
+
     def gct_malloc(self, hop, add_flags=None):
         TYPE = hop.spaceop.result.concretetype.TO
         assert not TYPE._is_varsize()
@@ -523,16 +503,21 @@ class GCTransformer(BaseGCTransformer):
         hop.cast_result(v_raw)
 
     def gct_fv_raw_malloc(self, hop, flags, TYPE, c_size):
-        if flags.get('zero'):
-            ll_func = self.raw_malloc_fixedsize_zero_ptr
-        else:
-            ll_func = self.raw_malloc_fixedsize_ptr
-        v_raw = hop.genop("direct_call", [ll_func, c_size],
+        v_raw = hop.genop("direct_call", [self.raw_malloc_fixedsize_ptr, c_size],
                           resulttype=llmemory.Address)
+        if flags.get('zero'):
+            hop.genop("raw_memclear", [v_raw, c_size])
         if flags.get('track_allocation', True):
             hop.genop("track_alloc_start", [v_raw])
         return v_raw
 
+    def gct_fv_stack_malloc(self, hop, flags, TYPE, c_size):
+        v_raw = hop.genop("direct_call", [self.stack_malloc_fixedsize_ptr, c_size],
+                          resulttype=llmemory.Address)
+        if flags.get('zero'):
+            hop.genop("raw_memclear", [v_raw, c_size])
+        return v_raw
+
     def gct_malloc_varsize(self, hop, add_flags=None):
         flags = hop.spaceop.args[1].value
         if add_flags:
@@ -543,7 +528,12 @@ class GCTransformer(BaseGCTransformer):
         return self.varsize_malloc_helper(hop, flags, meth, [])
 
     def gct_gc_add_memory_pressure(self, hop):
-        pass
+        if hasattr(self, 'raw_malloc_memory_pressure_ptr'):
+            op = hop.spaceop
+            size = op.args[0]
+            return hop.genop("direct_call",
+                          [self.raw_malloc_memory_pressure_ptr,
+                           size])
 
     def varsize_malloc_helper(self, hop, flags, meth, extraargs):
         def intconst(c): return rmodel.inputconst(lltype.Signed, c)
@@ -577,10 +567,9 @@ class GCTransformer(BaseGCTransformer):
                                                                     c_offset_to_length):
         if flags.get('add_memory_pressure', False):
             if hasattr(self, 'raw_malloc_memory_pressure_varsize_ptr'):
-                v_adr = rmodel.inputconst(llmemory.Address, llmemory.NULL)
                 hop.genop("direct_call",
                           [self.raw_malloc_memory_pressure_varsize_ptr,
-                           v_length, c_item_size, v_adr])
+                           v_length, c_item_size])
         if c_offset_to_length is None:
             if flags.get('zero'):
                 fnptr = self.raw_malloc_varsize_no_length_zero_ptr
