@@ -3,7 +3,7 @@ import sys
 import py
 
 from rpython.rlib.nonconst import NonConstant
-from rpython.rlib.objectmodel import CDefinedIntSymbolic, keepalive_until_here, specialize, not_rpython, we_are_translated
+from rpython.rlib.objectmodel import CDefinedIntSymbolic, keepalive_until_here, specialize
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rtyper.extregistry import ExtRegistryEntry
 from rpython.tool.sourcetools import rpython_wrapper
@@ -84,7 +84,6 @@ def hint(x, **kwds):
 
     * promote - promote the argument from a variable into a constant
     * promote_string - same, but promote string by *value*
-    * promote_unicode - same, but promote unicode string by *value*
     * access_directly - directly access a virtualizable, as a structure
                         and don't treat it as a virtualizable
     * fresh_virtualizable - means that virtualizable was just allocated.
@@ -127,9 +126,6 @@ def promote(x):
 def promote_string(x):
     return hint(x, promote_string=True)
 
-def promote_unicode(x):
-    return hint(x, promote_unicode=True)
-
 def dont_look_inside(func):
     """ Make sure the JIT does not trace inside decorated function
     (it becomes a call instead)
@@ -155,7 +151,7 @@ def unroll_safe(func):
     if getattr(func, '_elidable_function_', False):
         raise TypeError("it does not make sense for %s to be both elidable and unroll_safe" % func)
     if not getattr(func, '_jit_look_inside_', True):
-        raise TypeError("it does not make sense for %s to be both unroll_safe and dont_look_inside" % func)
+        raise TypeError("it does not make sense for %s to be both elidable and dont_look_inside" % func)
     func._jit_unroll_safe_ = True
     return func
 
@@ -196,7 +192,7 @@ def elidable_promote(promote_args='all'):
         d = {"_orig_func_unlikely_name": func, "hint": hint}
         exec py.code.Source("\n".join(code)).compile() in d
         result = d["f"]
-        result.__name__ = func.__name__ + "_promote"
+        result.func_name = func.func_name + "_promote"
         return result
     return decorator
 
@@ -248,10 +244,6 @@ def look_inside_iff(predicate):
     return inner
 
 def oopspec(spec):
-    """ The JIT compiler won't look inside this decorated function,
-        but instead during translation, rewrites it according to the handler in
-        rpython/jit/codewriter/jtransform.py.
-    """
     def decorator(func):
         func.oopspec = spec
         return func
@@ -265,6 +257,27 @@ def not_in_trace(func):
     func.oopspec = "jit.not_in_trace()"   # note that 'func' may take arguments
     return func
 
+def call_shortcut(func):
+    """A decorator to ensure that a function has a fast-path.
+    DOES NOT RELIABLY WORK ON METHODS, USE ONLY ON FUNCTIONS!
+
+    Only useful on functions that the JIT doesn't normally look inside.
+    It still replaces residual calls to that function with inline code
+    that checks for a fast path, and only does the call if not.  For
+    now, graphs made by the following kinds of functions are detected:
+
+           def func(x, y, z):         def func(x, y, z):
+               if y.field:                 r = y.field
+                   return y.field          if r is None:
+               ...                             ...
+                                           return r
+
+    Fast-path detection is always on, but this decorator makes the
+    codewriter complain if it cannot find the promized fast-path.
+    """
+    func._call_shortcut_ = True
+    return func
+
 
 @oopspec("jit.isconstant(value)")
 @specialize.call_location()
@@ -427,7 +440,7 @@ def jit_callback(name):
         jitdriver = JitDriver(get_printable_location=get_printable_location,
                               greens=[], reds='auto', name=name)
         #
-        args = ','.join(['a%d' % i for i in range(func.__code__.co_argcount)])
+        args = ','.join(['a%d' % i for i in range(func.func_code.co_argcount)])
         source = """def callback_with_jitdriver(%(args)s):
                         jitdriver.jit_merge_point()
                         return real_callback(%(args)s)""" % locals()
@@ -601,14 +614,14 @@ class JitDriver(object):
     inline_jit_merge_point = False
     _store_last_enter_jit = None
 
-    @not_rpython
     def __init__(self, greens=None, reds=None, virtualizables=None,
                  get_jitcell_at=None, set_jitcell_at=None,
                  get_printable_location=None, confirm_enter_jit=None,
                  can_never_inline=None, should_unroll_one_iteration=None,
                  name='jitdriver', check_untranslated=True, vectorize=False,
                  get_unique_id=None, is_recursive=False, get_location=None):
-        """get_location:
+        """ NOT_RPYTHON
+            get_location:
               The return value is designed to provide enough information to express the
               state of an interpreter when invoking jit_merge_point.
               For a bytecode interperter such as PyPy this includes, filename, line number,
@@ -645,6 +658,8 @@ class JitDriver(object):
             raise AttributeError("no 'greens' or 'reds' supplied")
         if virtualizables is not None:
             self.virtualizables = virtualizables
+        if get_unique_id is not None:
+            assert is_recursive, "get_unique_id and is_recursive must be specified at the same time"
         for v in self.virtualizables:
             assert v in self.reds
         # if reds are automatic, they won't be passed to jit_merge_point, so
@@ -657,12 +672,8 @@ class JitDriver(object):
         self._make_extregistryentries()
         assert get_jitcell_at is None, "get_jitcell_at no longer used"
         assert set_jitcell_at is None, "set_jitcell_at no longer used"
-        for green in self.greens:
-            if "." in green:
-                raise ValueError("green fields are buggy! if you need them fixed, please talk to us")
         self.get_printable_location = get_printable_location
         self.get_location = get_location
-        self.has_unique_id = (get_unique_id is not None)
         if get_unique_id is None:
             get_unique_id = lambda *args: 0
         self.get_unique_id = get_unique_id
@@ -910,7 +921,7 @@ class ExtEnterLeaveMarker(ExtRegistryEntry):
             return
         bk = self.bookkeeper
         s_func = bk.immutablevalue(func)
-        uniquekey = 'jitdriver.%s' % func.__name__
+        uniquekey = 'jitdriver.%s' % func.func_name
         args_s = args_s[:]
         for name in variables:
             if '.' not in name:
@@ -1091,8 +1102,7 @@ class JitHookInterface(object):
     """ This is the main connector between the JIT and the interpreter.
     Several methods on this class will be invoked at various stages
     of JIT running like JIT loops compiled, aborts etc.
-    An instance of this class has to be passed into the JitPolicy constructor
-    (and will then be available as policy.jithookiface).
+    An instance of this class will be available as policy.jithookiface.
     """
     # WARNING: You should make a single prebuilt instance of a subclass
     # of this class.  You can, before translation, initialize some
@@ -1102,13 +1112,6 @@ class JitHookInterface(object):
     # of the program!  A line like ``pypy_hooks.foo = ...`` must not
     # appear inside your interpreter's RPython code.
 
-    def are_hooks_enabled(self):
-        """ A hook that is called to check whether the interpreter's hooks are
-        enabled at all. Only if this function returns True, are the other hooks
-        called. Otherwise, nothing happens. This is done because constructing
-        some of the hooks' arguments is expensive, so we'd rather not do it."""
-        return True
-
     def on_abort(self, reason, jitdriver, greenkey, greenkey_repr, logops, operations):
         """ A hook called each time a loop is aborted with jitdriver and
         greenkey where it started, reason is a string why it got aborted
@@ -1159,9 +1162,6 @@ def record_exact_class(value, cls):
     """
     Assure the JIT that value is an instance of cls. This is a precise
     class check, like a guard_class.
-
-    See also debug.ll_assert_not_none(x), which asserts that x is not None
-    and also assures the JIT that it is the case.
     """
     assert type(value) is cls
 
@@ -1194,103 +1194,32 @@ class Entry(ExtRegistryEntry):
         return hop.gendirectcall(ll_record_exact_class, v_inst, v_cls)
 
 def _jit_conditional_call(condition, function, *args):
-    pass           # special-cased below
+    pass
 
 @specialize.call_location()
 def conditional_call(condition, function, *args):
-    """Does the same as:
-
-         if condition:
-             function(*args)
-
-    but is better for the JIT, in case the condition is often false
-    but could be true occasionally.  It allows the JIT to always produce
-    bridge-free code.  The function is never looked inside.
-    """
     if we_are_jitted():
         _jit_conditional_call(condition, function, *args)
     else:
         if condition:
             function(*args)
-conditional_call._always_inline_ = 'try'
-
-def _jit_conditional_call_value(value, function, *args):
-    return value    # special-cased below
-
-@specialize.call_location()
-def conditional_call_elidable(value, function, *args):
-    """Does the same as:
-
-        if value == <0 or None or NULL>:
-            value = function(*args)
-        return value
-
-    For the JIT.  Allows one branch which doesn't create a bridge,
-    typically used for caching.  The value and the function's return
-    type must match and cannot be a float: they must be either regular
-    'int', or something that turns into a pointer.
-
-    Even if the function is not marked @elidable, it is still treated
-    mostly like one.  The only difference is that (in heapcache.py)
-    we don't assume this function won't change anything observable.
-    This is useful for caches, as you can write:
-
-        def _compute_and_cache(...):
-            self.cache = ...compute...
-            return self.cache
-
-        x = jit.conditional_call_elidable(self.cache, _compute_and_cache, ...)
-
-    """
-    if we_are_translated() and we_are_jitted():
-        #^^^ the occasional test patches we_are_jitted() to True
-        return _jit_conditional_call_value(value, function, *args)
-    else:
-        if isinstance(value, int):
-            if value == 0:
-                value = function(*args)
-                assert isinstance(value, int)
-        else:
-            if not isinstance(value, list) and not value:
-                value = function(*args)
-                assert not isinstance(value, int)
-        return value
-conditional_call_elidable._always_inline_ = 'try'
+conditional_call._always_inline_ = True
 
 class ConditionalCallEntry(ExtRegistryEntry):
-    _about_ = _jit_conditional_call, _jit_conditional_call_value
+    _about_ = _jit_conditional_call
 
     def compute_result_annotation(self, *args_s):
-        s_res = self.bookkeeper.emulate_pbc_call(self.bookkeeper.position_key,
-                                                 args_s[1], args_s[2:])
-        if self.instance == _jit_conditional_call_value:
-            from rpython.annotator import model as annmodel
-            # the result is either s_res, i.e. the function result, or
-            # it is args_s[0]-but-not-none.  The "not-none" part is
-            # only useful for pointer-like types, but it means that
-            # args_s[0] could be NULL without the result of the whole
-            # conditional_call_elidable() necessarily returning a result
-            # that can be NULL.
-            return annmodel.unionof(s_res, args_s[0].nonnoneify())
+        self.bookkeeper.emulate_pbc_call(self.bookkeeper.position_key,
+                                         args_s[1], args_s[2:])
 
     def specialize_call(self, hop):
         from rpython.rtyper.lltypesystem import lltype
 
-        if self.instance == _jit_conditional_call:
-            opname = 'jit_conditional_call'
-            COND = lltype.Bool
-            resulttype = None
-        elif self.instance == _jit_conditional_call_value:
-            opname = 'jit_conditional_call_value'
-            COND = hop.r_result
-            resulttype = hop.r_result.lowleveltype
-        else:
-            assert False
-        args_v = hop.inputargs(COND, lltype.Void, *hop.args_r[2:])
+        args_v = hop.inputargs(lltype.Bool, lltype.Void, *hop.args_r[2:])
         args_v[1] = hop.args_r[1].get_concrete_llfn(hop.args_s[1],
                                                     hop.args_s[2:], hop.spaceop)
         hop.exception_is_here()
-        return hop.genop(opname, args_v, resulttype=resulttype)
+        return hop.genop('jit_conditional_call', args_v)
 
 def enter_portal_frame(unique_id):
     """call this when starting to interpret a function. calling this is not
@@ -1315,7 +1244,6 @@ class Counters(object):
     TRACING
     BACKEND
     OPS
-    HEAPCACHED_OPS
     RECORDED_OPS
     GUARDS
     OPT_OPS
@@ -1329,7 +1257,6 @@ class Counters(object):
     ABORT_BAD_LOOP
     ABORT_ESCAPE
     ABORT_FORCE_QUASIIMMUT
-    ABORT_SEGMENTED_TRACE
     NVIRTUALS
     NVHOLES
     NVREUSED
