@@ -120,7 +120,7 @@ def enforceargs(*types_, **kwds):
     """ Decorate a function with forcing of RPython-level types on arguments.
     None means no enforcing.
 
-    When not translated, the type of the actual arguments is checked against
+    When not translated, the type of the actual arguments are checked against
     the enforced types every time the function is called. You can disable the
     typechecking by passing ``typecheck=False`` to @enforceargs.
     """
@@ -137,29 +137,33 @@ def enforceargs(*types_, **kwds):
     def decorator(f):
         def get_annotation(t):
             from rpython.annotator.signature import annotation
-            from rpython.annotator.model import SomeObject
+            from rpython.annotator.model import SomeObject, SomeString, SomeUnicodeString
             if isinstance(t, SomeObject):
                 return t
-            return annotation(t)
-
+            s_result = annotation(t)
+            if (isinstance(s_result, SomeString) or
+                isinstance(s_result, SomeUnicodeString)):
+                return s_result.__class__(can_be_None=True)
+            return s_result
         def get_type_descr_of_argument(arg):
             # we don't want to check *all* the items in list/dict: we assume
             # they are already homogeneous, so we only check the first
             # item. The case of empty list/dict is handled inside typecheck()
             if isinstance(arg, list):
-                return [get_type_descr_of_argument(arg[0])]
+                item = arg[0]
+                return [get_type_descr_of_argument(item)]
             elif isinstance(arg, dict):
                 key, value = next(arg.iteritems())
                 return {get_type_descr_of_argument(key): get_type_descr_of_argument(value)}
             else:
                 return type(arg)
-
         def typecheck(*args):
             from rpython.annotator.model import SomeList, SomeDict, SomeChar,\
                  SomeInteger
-            for i, (s_expected, arg) in enumerate(zip(s_types, args)):
-                if s_expected is None:
+            for i, (expected_type, arg) in enumerate(zip(types, args)):
+                if expected_type is None:
                     continue
+                s_expected = get_annotation(expected_type)
                 # special case: if we expect a list or dict and the argument
                 # is an empty list/dict, the typecheck always pass
                 if isinstance(s_expected, SomeList) and arg == []:
@@ -176,7 +180,7 @@ def enforceargs(*types_, **kwds):
                 s_argtype = get_annotation(get_type_descr_of_argument(arg))
                 if not s_expected.contains(s_argtype):
                     msg = "%s argument %r must be of type %s" % (
-                        f.__name__, srcargs[i], types[i])
+                        f.func_name, srcargs[i], expected_type)
                     raise TypeError(msg)
         #
         template = """
@@ -197,10 +201,6 @@ def enforceargs(*types_, **kwds):
         assert len(srcargs) == len(types), (
             'not enough types provided: expected %d, got %d' %
             (len(types), len(srcargs)))
-
-        s_types = [None if typ is None else
-                   NOT_CONSTANT if typ is NOT_CONSTANT else
-                   get_annotation(typ) for typ in types]
         result._annenforceargs_ = types
         return result
     return decorator
@@ -223,29 +223,6 @@ def try_inline(func):
     func._always_inline_ = 'try'
     return func
 
-def not_rpython(func):
-    """ mark a function as not rpython. the translation process will raise an
-    error if it encounters the function. """
-    # test is in annotator/test/test_annrpython.py
-    func._not_rpython_ = True
-    return func
-
-def llhelper_error_value(error_value):
-    """
-    This decorator has two effects:
-
-      1. declare that this llhelper can raise RPython exceptions, which should
-         be correctly propagated
-
-      2. specify the error_value to return in case of exception.
-    """
-    # relevant tests:
-    #     - test_ll2ctypes.test_llhelper_error_value
-    #     - test_exceptiontransform.test_custom_error_value
-    def decorate(func):
-        func._llhelper_error_value_ = error_value
-        return func
-    return decorate
 
 # ____________________________________________________________
 
@@ -334,8 +311,6 @@ def sc_we_are_translated(ctx):
 def register_replacement_for(replaced_function, sandboxed_name=None):
     def wrap(func):
         from rpython.rtyper.extregistry import ExtRegistryEntry
-        # to support calling func directly
-        func._sandbox_external_name = sandboxed_name
         class ExtRegistry(ExtRegistryEntry):
             _about_ = replaced_function
             def compute_annotation(self):
@@ -393,37 +368,6 @@ class Entry(ExtRegistryEntry):
         hop.exception_cannot_occur()
         return hop.inputconst(lltype.Void, translator.config)
 
-
-def revdb_flag_io_disabled():
-    if not revdb_enabled():
-        return False
-    return _revdb_flag_io_disabled()
-
-def _revdb_flag_io_disabled():
-    # moved in its own function for the import statement
-    from rpython.rlib import revdb
-    return revdb.flag_io_disabled()
-
-@not_rpython
-def revdb_enabled():
-    return False
-
-class Entry(ExtRegistryEntry):
-    _about_ = revdb_enabled
-
-    def compute_result_annotation(self):
-        from rpython.annotator import model as annmodel
-        config = self.bookkeeper.annotator.translator.config
-        if config.translation.reverse_debugger:
-            return annmodel.s_True
-        else:
-            return annmodel.s_False
-
-    def specialize_call(self, hop):
-        from rpython.rtyper.lltypesystem import lltype
-        hop.exception_cannot_occur()
-        return hop.inputconst(lltype.Bool, hop.s_result.const)
-
 # ____________________________________________________________
 
 class FREED_OBJECT(object):
@@ -493,37 +437,6 @@ class Entry(ExtRegistryEntry):
         hop.exception_is_here()
         hop.gendirectcall(r_list.LIST._ll_resize_hint, v_list, v_sizehint)
 
-def list_get_physical_size(l):
-    """ try to get the physical size of an overallocated RPython list. will
-    return the regular length untranslated. """
-    return len(l)
-
-def ll_physical_size(l):
-    return len(l.items)
-
-class Entry(ExtRegistryEntry):
-    _about_ = list_get_physical_size
-
-    def compute_result_annotation(self, s_l):
-        from rpython.annotator import model as annmodel
-        if annmodel.s_None.contains(s_l):
-            pass # first argument is only None so far, but we
-                 # expect a generalization later
-        elif not isinstance(s_l, annmodel.SomeList):
-            raise annmodel.AnnotatorError("First argument must be a list")
-        return annmodel.SomeInteger(nonneg=True)
-
-    def specialize_call(self, hop):
-        from rpython.rtyper.lltypesystem.rlist import FixedSizeListRepr, ListRepr
-        v_list, = hop.inputargs(*hop.args_r)
-        if isinstance(hop.args_r[0], ListRepr):
-            ll_func = ll_physical_size
-        else:
-            ll_func = v_list.concretetype.TO.ll_length
-        hop.exception_cannot_occur()
-        return hop.gendirectcall(ll_func, v_list)
-
-
 # ____________________________________________________________
 #
 # id-like functions.  The idea is that calling hash() or id() is not
@@ -538,14 +451,8 @@ def compute_hash(x):
 
     Note that this can return 0 or -1 too.
 
-    NOTE: It returns a different number before and after translation!
-    Dictionaries will be rehashed when the translated program starts.
-    Be careful about other places that store or depend on a hash value:
-    if such a place can exist before translation, you should add for
-    example a _cleanup_() method to clear this cache during translation.
-
-    (Nowadays we could completely remove compute_hash() and decide that
-    hash(x) is valid RPython instead, at least for the types listed here.)
+    It returns the same number, both before and after translation.
+    Dictionaries don't need to be rehashed after translation.
     """
     if isinstance(x, (str, unicode)):
         return _hash_string(x)
@@ -563,11 +470,17 @@ def compute_identity_hash(x):
     """RPython equivalent of object.__hash__(x).  This returns the
     so-called 'identity hash', which is the non-overridable default hash
     of Python.  Can be called for any RPython-level object that turns
-    into a GC object, but not NULL.  The value will be different before
-    and after translation (WARNING: this is a change with older RPythons!)
+    into a GC object, but not NULL.  The value is not guaranteed to be the
+    same before and after translation, except for RPython instances on the
+    lltypesystem.
     """
     assert x is not None
-    return object.__hash__(x)
+    result = object.__hash__(x)
+    try:
+        x.__dict__['__precomputed_identity_hash'] = result
+    except (TypeError, AttributeError):
+        pass
+    return result
 
 def compute_unique_id(x):
     """RPython equivalent of id(x).  The 'x' must be an RPython-level
@@ -592,20 +505,11 @@ def current_object_addr_as_int(x):
 
 # ----------
 
-@specialize.ll()
+HASH_ALGORITHM = "rpython"  # XXX Is there a better name?
+
 def _hash_string(s):
-    """The default algorithm behind compute_hash() for a string or a unicode.
-    This is a modified Fowler-Noll-Vo (FNV) hash.  According to Wikipedia,
-    FNV needs carefully-computed constants called FNV primes and FNV offset
-    basis, which are absent from the present algorithm.  Nevertheless,
-    this matches CPython 2.7 without -R, which has proven a good hash in
-    practice (even if not crypographical nor randomizable).
-
-    There is a mechanism to use another one in programs after translation.
-    See rsiphash.py, which implements the algorithm of CPython >= 3.4.
-    """
+    """The algorithm behind compute_hash() for a string or a unicode."""
     from rpython.rlib.rarithmetic import intmask
-
     length = len(s)
     if length == 0:
         return -1
@@ -617,9 +521,6 @@ def _hash_string(s):
     x ^= length
     return intmask(x)
 
-def ll_hash_string(ll_s):
-    return _hash_string(ll_s.chars)
-
 def _hash_float(f):
     """The algorithm behind compute_hash() for a float.
     This implementation is identical to the CPython implementation,
@@ -627,9 +528,9 @@ def _hash_float(f):
     In RPython, floats cannot be used with ints in dicts, anyway.
     """
     from rpython.rlib.rarithmetic import intmask
-    from rpython.rlib.rfloat import isfinite
+    from rpython.rlib.rfloat import isfinite, isinf
     if not isfinite(f):
-        if math.isinf(f):
+        if isinf(f):
             if f < 0.0:
                 return -271828
             else:
@@ -644,9 +545,8 @@ def _hash_float(f):
     return intmask(x)
 TAKE_NEXT = float(2**31)
 
-@not_rpython
 def _hash_tuple(t):
-    """The algorithm behind compute_hash() for a tuple.
+    """NOT_RPYTHON.  The algorithm behind compute_hash() for a tuple.
     It is modelled after the old algorithm of Python 2.3, which is
     a bit faster than the one introduced by Python 2.4.  We assume
     that nested tuples are very uncommon in RPython, making the bad
@@ -675,21 +575,6 @@ class Entry(ExtRegistryEntry):
         hop.exception_is_here()
         return hop.gendirectcall(ll_fn, v_obj)
 
-class Entry(ExtRegistryEntry):
-    _about_ = ll_hash_string
-    # this is only used when annotating the code in rstr.py, and so
-    # it always occurs after the RPython program signalled its intent
-    # to use a different hash.  The code below overwrites the use of
-    # ll_hash_string() to make the annotator think a possibly different
-    # function was called.
-
-    def compute_annotation(self):
-        from rpython.annotator import model as annmodel
-        bk = self.bookkeeper
-        translator = bk.annotator.translator
-        fn = getattr(translator, 'll_hash_string', ll_hash_string)
-        return annmodel.SomePBC([bk.getdesc(fn)])
-
 class Entry(ExtRegistryEntry):
     _about_ = compute_identity_hash
 
@@ -751,6 +636,13 @@ class Entry(ExtRegistryEntry):
 def hlinvoke(repr, llcallable, *args):
     raise TypeError("hlinvoke is meant to be rtyped and not called direclty")
 
+def is_in_callback():
+    """Returns True if we're currently in a callback *or* if there are
+    multiple threads around.
+    """
+    from rpython.rtyper.lltypesystem import rffi
+    return rffi.stackcounter.stacks_counter > 1
+
 
 class UnboxedValue(object):
     """A mixin class to use for classes that have exactly one field which
@@ -819,19 +711,11 @@ class r_dict(object):
     def _newdict(self):
         return {}
 
-    def __init__(self, key_eq, key_hash, force_non_null=False, simple_hash_eq=False):
-        """ force_non_null=True means that the key can never be None (even if
-        the annotator things it could be)
-
-        simple_hash_eq=True means that the hash function is very fast, meaning it's
-        efficient enough that the dict does not have to store the hash per key.
-        It also implies that neither the hash nor the eq function will mutate
-        the dictionary. """
+    def __init__(self, key_eq, key_hash, force_non_null=False):
         self._dict = self._newdict()
         self.key_eq = key_eq
         self.key_hash = key_hash
         self.force_non_null = force_non_null
-        self.simple_hash_eq = simple_hash_eq
 
     def __getitem__(self, key):
         return self._dict[_r_dictkey(self, key)]
@@ -858,9 +742,6 @@ class r_dict(object):
     def setdefault(self, key, default):
         return self._dict.setdefault(_r_dictkey(self, key), default)
 
-    def pop(self, key, *default):
-        return self._dict.pop(_r_dictkey(self, key), *default)
-
     def popitem(self):
         dk, value = self._dict.popitem()
         return dk.key, value
@@ -1012,83 +893,6 @@ def delitem_with_hash(d, key, h):
         return
     d.delitem_with_hash(key, h)
 
-@specialize.call_location()
-def delitem_if_value_is(d, key, value):
-    """Same as 'if d.get(key) is value: del d[key]'.  It is safe even in
-    case 'd' is an r_dict and the lookup involves callbacks that might
-    release the GIL."""
-    if not we_are_translated():
-        try:
-            if d[key] is value:
-                del d[key]
-        except KeyError:
-            pass
-        return
-    d.delitem_if_value_is(key, value)
-
-def _untranslated_move_to_end(d, key, last):
-    "NOT_RPYTHON"
-    value = d.pop(key)
-    if last:
-        d[key] = value
-    else:
-        items = d.items()
-        d.clear()
-        d[key] = value
-        # r_dict.update does not support list of tuples, do it manually
-        for key, value in items:
-            d[key] = value
-
-@specialize.call_location()
-def move_to_end(d, key, last=True):
-    if not we_are_translated():
-        _untranslated_move_to_end(d, key, last)
-        return
-    d.move_to_end(key, last)
-
-@not_rpython
-def dict_to_switch(d, inline=True):
-    """Convert dictionary with integer or char keys to a switch statement."""
-    from rpython.rlib.unroll import unrolling_iterable
-
-    assert len(d) >= 1
-    firstkey = next(iter(d))
-    if isinstance(firstkey, int):
-        precheck = always_inline(lambda key: key)
-        for key in d:
-            assert isinstance(key, int)
-    else:
-        assert isinstance(firstkey, str) # only int and char allowed for now
-        for key in d:
-            assert isinstance(key, str) and len(key) == 1
-
-        @always_inline
-        def precheck(key):
-            # make sure it's a char switch
-            if len(key) > 1:
-                raise KeyError
-            return key[0]
-
-    cached_size = len(d)
-    unrolling_iteritems = unrolling_iterable(d.iteritems())
-
-    def lookup(query):
-        if we_are_translated():
-            query = precheck(query)
-            for key, value in unrolling_iteritems:
-                if key == query:
-                    return value
-            else:
-                raise KeyError
-        else:
-            assert len(d) == cached_size, "dictionary size changed!"
-            return d[query]
-
-
-    if inline:
-        lookup = always_inline(lookup)
-    return lookup
-
 # ____________________________________________________________
 
 def import_from_mixin(M, special_methods=['__init__', '__del__']):
@@ -1145,13 +949,3 @@ def import_from_mixin(M, special_methods=['__init__', '__del__']):
         target[key] = value
     if immutable_fields:
         target['_immutable_fields_'] = target.get('_immutable_fields_', []) + immutable_fields
-
-def never_allocate(cls):
-    """
-    Class decorator to ensure that a class is NEVER instantiated at runtime.
-
-    Useful e.g for context manager which are expected to be constant-folded
-    away.
-    """
-    cls._rpython_never_allocate_ = True
-    return cls
