@@ -5,7 +5,7 @@ import types
 
 from rpython.rlib import jit
 from rpython.rlib.objectmodel import we_are_translated, enforceargs, specialize
-from rpython.rlib.objectmodel import CDefinedIntSymbolic, not_rpython
+from rpython.rlib.objectmodel import CDefinedIntSymbolic
 from rpython.rtyper.extregistry import ExtRegistryEntry
 from rpython.rtyper.lltypesystem import lltype, llmemory
 
@@ -13,63 +13,12 @@ from rpython.rtyper.lltypesystem import lltype, llmemory
 # General GC features
 
 collect = gc.collect
-enable = gc.enable
-disable = gc.disable
-isenabled = gc.isenabled
-
-def collect_step():
-    """
-    If the GC is incremental, run a single gc-collect-step.
-
-    Return an integer which encodes the starting and ending GC state. Use
-    rgc.{old_state,new_state,is_done} to decode it.
-
-    If the GC is not incremental, do a full collection and return a value on
-    which rgc.is_done() return True.
-    """
-    gc.collect()
-    return _encode_states(1, 0)
-
-def _encode_states(oldstate, newstate):
-    return oldstate << 8 | newstate
-
-def old_state(states):
-    return (states & 0xFF00) >> 8
-
-def new_state(states):
-    return states & 0xFF
-
-def is_done(states):
-    """
-    Return True if the return value of collect_step signals the end of a major
-    collection
-    """
-    old = old_state(states)
-    new = new_state(states)
-    return is_done__states(old, new)
-
-def is_done__states(oldstate, newstate):
-    "Like is_done, but takes oldstate and newstate explicitly"
-    # a collection is considered done when it ends up in the starting state
-    # (which is usually represented as 0). This logic works for incminimark,
-    # which is currently the only gc actually used and for which collect_step
-    # is implemented. In case we add more GC in the future, we might want to
-    # delegate this logic to the GC itself, but for now it is MUCH simpler to
-    # just write it in plain RPython.
-    return oldstate != 0 and newstate == 0
 
 def set_max_heap_size(nbytes):
     """Limit the heap size to n bytes.
     """
     pass
 
-def must_split_gc_address_space():
-    """Returns True if we have a "split GC address space", i.e. if
-    we are translating with an option that doesn't support taking raw
-    addresses inside GC objects and "hacking" at them.  This is
-    notably the case with --revdb."""
-    return False
-
 # for test purposes we allow objects to be pinned and use
 # the following list to keep track of the pinned objects
 _pinned_objects = []
@@ -175,44 +124,6 @@ class CollectEntry(ExtRegistryEntry):
             args_v = hop.inputargs(lltype.Signed)
         return hop.genop('gc__collect', args_v, resulttype=hop.r_result)
 
-
-class EnableDisableEntry(ExtRegistryEntry):
-    _about_ = (gc.enable, gc.disable)
-
-    def compute_result_annotation(self):
-        from rpython.annotator import model as annmodel
-        return annmodel.s_None
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        opname = self.instance.__name__
-        return hop.genop('gc__%s' % opname, hop.args_v, resulttype=hop.r_result)
-
-
-class IsEnabledEntry(ExtRegistryEntry):
-    _about_ = gc.isenabled
-
-    def compute_result_annotation(self):
-        from rpython.annotator import model as annmodel
-        return annmodel.s_Bool
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        return hop.genop('gc__isenabled', hop.args_v, resulttype=hop.r_result)
-
-
-class CollectStepEntry(ExtRegistryEntry):
-    _about_ = collect_step
-
-    def compute_result_annotation(self):
-        from rpython.annotator import model as annmodel
-        return annmodel.SomeInteger()
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        return hop.genop('gc__collect_step', hop.args_v, resulttype=hop.r_result)
-
-
 class SetMaxHeapSizeEntry(ExtRegistryEntry):
     _about_ = set_max_heap_size
 
@@ -236,18 +147,6 @@ def can_move(p):
     """
     return True
 
-class SplitAddrSpaceEntry(ExtRegistryEntry):
-    _about_ = must_split_gc_address_space
- 
-    def compute_result_annotation(self):
-        config = self.bookkeeper.annotator.translator.config
-        result = config.translation.split_gc_address_space
-        return self.bookkeeper.immutablevalue(result)
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        return hop.inputconst(lltype.Bool, hop.s_result.const)
-
 class CanMoveEntry(ExtRegistryEntry):
     _about_ = can_move
 
@@ -381,25 +280,18 @@ def ll_arraycopy(source, dest, source_start, dest_start, length):
 
     TP = lltype.typeOf(source).TO
     assert TP == lltype.typeOf(dest).TO
-
-    slowpath = False
-    if must_split_gc_address_space():
-        slowpath = True
-    elif _contains_gcptr(TP.OF):
+    if _contains_gcptr(TP.OF):
         # perform a write barrier that copies necessary flags from
         # source to dest
         if not llop.gc_writebarrier_before_copy(lltype.Bool, source, dest,
                                                 source_start, dest_start,
                                                 length):
-            slowpath = True
-    if slowpath:
-        # if the write barrier is not supported, or if we translate with
-        # the option 'split_gc_address_space', then copy by hand
-        i = 0
-        while i < length:
-            copy_item(source, dest, i + source_start, i + dest_start)
-            i += 1
-        return
+            # if the write barrier is not supported, copy by hand
+            i = 0
+            while i < length:
+                copy_item(source, dest, i + source_start, i + dest_start)
+                i += 1
+            return
     source_addr = llmemory.cast_ptr_to_adr(source)
     dest_addr   = llmemory.cast_ptr_to_adr(dest)
     cp_source_addr = (source_addr + llmemory.itemoffsetof(TP, 0) +
@@ -433,14 +325,6 @@ def ll_shrink_array(p, smallerlength):
     field = getattr(p, TP._names[0])
     setattr(newp, TP._names[0], field)
 
-    if must_split_gc_address_space():
-        # do the copying element by element
-        i = 0
-        while i < smallerlength:
-            newp.chars[i] = p.chars[i]
-            i += 1
-        return newp
-
     ARRAY = getattr(TP, TP._arrayfld)
     offset = (llmemory.offsetof(TP, TP._arrayfld) +
               llmemory.itemoffsetof(ARRAY, 0))
@@ -461,18 +345,9 @@ def ll_arrayclear(p):
 
     length = len(p)
     ARRAY = lltype.typeOf(p).TO
-    if must_split_gc_address_space():
-        # do the clearing element by element
-        from rpython.rtyper.lltypesystem import rffi
-        ZERO = rffi.cast(ARRAY.OF, 0)
-        i = 0
-        while i < length:
-            p[i] = ZERO
-            i += 1
-    else:
-        offset = llmemory.itemoffsetof(ARRAY, 0)
-        dest_addr = llmemory.cast_ptr_to_adr(p) + offset
-        llmemory.raw_memclear(dest_addr, llmemory.sizeof(ARRAY.OF) * length)
+    offset = llmemory.itemoffsetof(ARRAY, 0)
+    dest_addr = llmemory.cast_ptr_to_adr(p) + offset
+    llmemory.raw_memclear(dest_addr, llmemory.sizeof(ARRAY.OF) * length)
     keepalive_until_here(p)
 
 
@@ -503,12 +378,13 @@ def must_be_light_finalizer(func):
 
 class FinalizerQueue(object):
     """A finalizer queue.  See pypy/doc/discussion/finalizer-order.rst.
+    Note: only works with the framework GCs (like minimark).  It is
+    ignored with Boehm or with refcounting (used by tests).
     """
     # Must be subclassed, and the subclass needs these attributes:
     #
     #    Class:
     #        the class (or base class) of finalized objects
-    #        --or-- None to handle low-level GCREFs directly
     #
     #    def finalizer_trigger(self):
     #        called to notify that new items have been put in the queue
@@ -521,13 +397,11 @@ class FinalizerQueue(object):
     def next_dead(self):
         if we_are_translated():
             from rpython.rtyper.lltypesystem.lloperation import llop
-            from rpython.rtyper.lltypesystem.llmemory import GCREF
-            from rpython.rtyper.annlowlevel import cast_gcref_to_instance
+            from rpython.rtyper.rclass import OBJECTPTR
+            from rpython.rtyper.annlowlevel import cast_base_ptr_to_instance
             tag = FinalizerQueue._get_tag(self)
-            ptr = llop.gc_fq_next_dead(GCREF, tag)
-            if self.Class is not None:
-                ptr = cast_gcref_to_instance(self.Class, ptr)
-            return ptr
+            ptr = llop.gc_fq_next_dead(OBJECTPTR, tag)
+            return cast_base_ptr_to_instance(self.Class, ptr)
         try:
             return self._queue.popleft()
         except (AttributeError, IndexError):
@@ -536,25 +410,20 @@ class FinalizerQueue(object):
     @specialize.arg(0)
     @jit.dont_look_inside
     def register_finalizer(self, obj):
-        from rpython.rtyper.lltypesystem.llmemory import GCREF
-        if self.Class is None:
-            assert lltype.typeOf(obj) == GCREF
-        else:
-            assert isinstance(obj, self.Class)
+        assert isinstance(obj, self.Class)
         if we_are_translated():
             from rpython.rtyper.lltypesystem.lloperation import llop
-            from rpython.rtyper.annlowlevel import cast_instance_to_gcref
+            from rpython.rtyper.rclass import OBJECTPTR
+            from rpython.rtyper.annlowlevel import cast_instance_to_base_ptr
             tag = FinalizerQueue._get_tag(self)
-            if self.Class is not None:
-                obj = cast_instance_to_gcref(obj)
-            llop.gc_fq_register(lltype.Void, tag, obj)
+            ptr = cast_instance_to_base_ptr(obj)
+            llop.gc_fq_register(lltype.Void, tag, ptr)
             return
         else:
             self._untranslated_register_finalizer(obj)
 
-    @not_rpython
     def _get_tag(self):
-        "special-cased below"
+        "NOT_RPYTHON: special-cased below"
 
     def _reset(self):
         import collections
@@ -657,50 +526,18 @@ class FqTagEntry(ExtRegistryEntry):
         hop.exception_cannot_occur()
         return hop.inputconst(lltype.Signed, hop.s_result.const)
 
-@jit.dont_look_inside
-@specialize.argtype(0)
-def may_ignore_finalizer(obj):
-    """Optimization hint: says that it is valid for any finalizer
-    for 'obj' to be ignored, depending on the GC."""
-    from rpython.rtyper.lltypesystem.lloperation import llop
-    llop.gc_ignore_finalizer(lltype.Void, obj)
-
-@jit.dont_look_inside
-def move_out_of_nursery(obj):
-    """ Returns another object which is a copy of obj; but at any point
-        (either now or in the future) the returned object might suddenly
-        become identical to the one returned.
-
-        NOTE: Only use for immutable objects!
-
-        NOTE: Might fail on some GCs!  You have to check again
-        can_move() afterwards.  It should always work with the default
-        GC.  With Boehm, can_move() is always False so
-        move_out_of_nursery() should never be called in the first place.
-    """
-    return obj
-
-class MoveOutOfNurseryEntry(ExtRegistryEntry):
-    _about_ = move_out_of_nursery
-
-    def compute_result_annotation(self, s_obj):
-        return s_obj
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        return hop.genop('gc_move_out_of_nursery', hop.args_v, resulttype=hop.r_result)
 
 # ____________________________________________________________
 
 
-@not_rpython
 def get_rpy_roots():
+    "NOT_RPYTHON"
     # Return the 'roots' from the GC.
     # The gc typically returns a list that ends with a few NULL_GCREFs.
     return [_GcRef(x) for x in gc.get_objects()]
 
-@not_rpython
 def get_rpy_referents(gcref):
+    "NOT_RPYTHON"
     x = gcref._x
     if isinstance(x, list):
         d = x
@@ -728,36 +565,26 @@ def _keep_object(x):
         return False
     return type(x).__module__ != '__builtin__'   # keep non-builtins
 
-def add_memory_pressure(estimate, object=None):
+def add_memory_pressure(estimate):
     """Add memory pressure for OpaquePtrs."""
     pass
 
 class AddMemoryPressureEntry(ExtRegistryEntry):
     _about_ = add_memory_pressure
 
-    def compute_result_annotation(self, s_nbytes, s_object=None):
+    def compute_result_annotation(self, s_nbytes):
         from rpython.annotator import model as annmodel
-        if s_object is not None:
-            if not isinstance(s_object, annmodel.SomeInstance):
-                raise Exception("Wrong kind of object passed to "
-                                "add memory pressure")
-            self.bookkeeper.memory_pressure_types.add(s_object.classdef)
         return annmodel.s_None
 
     def specialize_call(self, hop):
-        v_size = hop.inputarg(lltype.Signed, 0)
-        if len(hop.args_v) == 2:
-            v_obj = hop.inputarg(hop.args_r[1], 1)
-            args = [v_size, v_obj]
-        else:
-            args = [v_size]
+        [v_size] = hop.inputargs(lltype.Signed)
         hop.exception_cannot_occur()
-        return hop.genop('gc_add_memory_pressure', args,
+        return hop.genop('gc_add_memory_pressure', [v_size],
                          resulttype=lltype.Void)
 
 
-@not_rpython
 def get_rpy_memory_usage(gcref):
+    "NOT_RPYTHON"
     # approximate implementation using CPython's type info
     Class = type(gcref._x)
     size = Class.__basicsize__
@@ -765,14 +592,11 @@ def get_rpy_memory_usage(gcref):
         size += Class.__itemsize__ * len(gcref._x)
     return size
 
-@not_rpython
 def get_rpy_type_index(gcref):
+    "NOT_RPYTHON"
     from rpython.rlib.rarithmetic import intmask
     Class = gcref._x.__class__
-    i = intmask(id(Class))
-    if i < 0:
-        i = ~i    # always return a positive number, at least
-    return i
+    return intmask(id(Class))
 
 def cast_gcref_to_int(gcref):
     # This is meant to be used on cast_instance_to_gcref results.
@@ -783,44 +607,33 @@ def cast_gcref_to_int(gcref):
     else:
         return id(gcref._x)
 
-(TOTAL_MEMORY, TOTAL_ALLOCATED_MEMORY, TOTAL_MEMORY_PRESSURE,
- PEAK_MEMORY, PEAK_ALLOCATED_MEMORY, TOTAL_ARENA_MEMORY,
- TOTAL_RAWMALLOCED_MEMORY, PEAK_ARENA_MEMORY, PEAK_RAWMALLOCED_MEMORY,
- NURSERY_SIZE, TOTAL_GC_TIME) = range(11)
-
-@not_rpython
-def get_stats(stat_no):
-    """ Long docstring goes here
-    """
-    raise NotImplementedError
-
-@not_rpython
 def dump_rpy_heap(fd):
+    "NOT_RPYTHON"
     raise NotImplementedError
 
-@not_rpython
 def get_typeids_z():
+    "NOT_RPYTHON"
     raise NotImplementedError
 
-@not_rpython
 def get_typeids_list():
+    "NOT_RPYTHON"
     raise NotImplementedError
 
-@not_rpython
 def has_gcflag_extra():
+    "NOT_RPYTHON"
     return True
 has_gcflag_extra._subopnum = 1
 
 _gcflag_extras = set()
 
-@not_rpython
 def get_gcflag_extra(gcref):
+    "NOT_RPYTHON"
     assert gcref   # not NULL!
     return gcref in _gcflag_extras
 get_gcflag_extra._subopnum = 2
 
-@not_rpython
 def toggle_gcflag_extra(gcref):
+    "NOT_RPYTHON"
     assert gcref   # not NULL!
     try:
         _gcflag_extras.remove(gcref)
@@ -988,24 +801,12 @@ class Entry(ExtRegistryEntry):
         return hop.genop('gc_get_rpy_type_index', vlist,
                          resulttype = hop.r_result)
 
-class Entry(ExtRegistryEntry):
-    _about_ = get_stats
-    def compute_result_annotation(self, s_no):
-        from rpython.annotator.model import SomeInteger
-        if not isinstance(s_no, SomeInteger):
-            raise Exception("expecting an integer")
-        return SomeInteger()
-    def specialize_call(self, hop):
-        args = hop.inputargs(lltype.Signed)
-        hop.exception_cannot_occur()
-        return hop.genop('gc_get_stats', args, resulttype=lltype.Signed)
-
-@not_rpython
 def _is_rpy_instance(gcref):
+    "NOT_RPYTHON"
     raise NotImplementedError
 
-@not_rpython
 def _get_llcls_from_cls(Class):
+    "NOT_RPYTHON"
     raise NotImplementedError
 
 class Entry(ExtRegistryEntry):
@@ -1217,17 +1018,17 @@ class _ResizableListSupportingRawPtr(list):
     rgc.nonmoving_raw_ptr_for_resizable_list() might be
     used if needed.  For now, only supports lists of chars.
     """
-    __slots__ = ('_ll_list',)   # either None or a struct of TYPE=LIST_OF(Char)
+    __slots__ = ('_raw_items',)   # either None or a rffi.CCHARP
 
     def __init__(self, lst):
-        self._ll_list = None
+        self._raw_items = None
         self.__from_list(lst)
 
     def __resize(self):
         """Called before an operation changes the size of the list"""
-        if self._ll_list is not None:
+        if self._raw_items is not None:
             list.__init__(self, self.__as_list())
-            self._ll_list = None
+            self._raw_items = None
 
     def __from_list(self, lst):
         """Initialize the list from a copy of the list 'lst'."""
@@ -1238,46 +1039,46 @@ class _ResizableListSupportingRawPtr(list):
             return
         if len(self) != len(lst):
             self.__resize()
-        if self._ll_list is None:
+        if self._raw_items is None:
             list.__init__(self, lst)
         else:
-            assert len(self) == self._ll_list.length == len(lst)
+            assert len(self) == self._raw_items._obj.getlength() == len(lst)
             for i in range(len(self)):
-                self._ll_list.items[i] = lst[i]
+                self._raw_items[i] = lst[i]
 
     def __as_list(self):
         """Return a list (the same or a different one) which contains the
         items in the regular way."""
-        if self._ll_list is None:
+        if self._raw_items is None:
             return self
-        length = self._ll_list.length
+        length = self._raw_items._obj.getlength()
         assert length == len(self)
-        return [self._ll_list.items[i] for i in range(length)]
+        return [self._raw_items[i] for i in range(length)]
 
     def __getitem__(self, index):
-        if self._ll_list is None:
+        if self._raw_items is None:
             return list.__getitem__(self, index)
         if index < 0:
             index += len(self)
         if not (0 <= index < len(self)):
             raise IndexError
-        return self._ll_list.items[index]
+        return self._raw_items[index]
 
     def __setitem__(self, index, new):
-        if self._ll_list is None:
+        if self._raw_items is None:
             return list.__setitem__(self, index, new)
         if index < 0:
             index += len(self)
         if not (0 <= index < len(self)):
             raise IndexError
-        self._ll_list.items[index] = new
+        self._raw_items[index] = new
 
     def __delitem__(self, index):
         self.__resize()
         list.__delitem__(self, index)
 
     def __getslice__(self, i, j):
-        return self.__class__(list.__getslice__(self.__as_list(), i, j))
+        return list.__getslice__(self.__as_list(), i, j)
 
     def __setslice__(self, i, j, new):
         lst = self.__as_list()
@@ -1384,23 +1185,15 @@ class _ResizableListSupportingRawPtr(list):
         list.sort(lst, *args, **kwds)
         self.__from_list(lst)
 
-    def _get_ll_list(self):
-        from rpython.rtyper.lltypesystem import rffi
-        from rpython.rtyper.lltypesystem.rlist import LIST_OF
-        if self._ll_list is None:
-            LIST = LIST_OF(lltype.Char)
-            existing_items = list(self)
-            n = len(self)
-            self._ll_list = lltype.malloc(LIST, immortal=True)
-            self._ll_list.length = n
-            self._ll_list.items = lltype.malloc(LIST.items.TO, n)
-            self.__from_list(existing_items)
-            assert self._ll_list is not None
-        return self._ll_list
-
     def _nonmoving_raw_ptr_for_resizable_list(self):
-        ll_list = self._get_ll_list()
-        return ll_nonmovable_raw_ptr_for_resizable_list(ll_list)
+        if self._raw_items is None:
+            existing_items = list(self)
+            from rpython.rtyper.lltypesystem import lltype, rffi
+            self._raw_items = lltype.malloc(rffi.CCHARP.TO, len(self),
+                                           flavor='raw', immortal=True)
+            self.__from_list(existing_items)
+            assert self._raw_items is not None
+        return self._raw_items
 
 def resizable_list_supporting_raw_ptr(lst):
     return _ResizableListSupportingRawPtr(lst)
@@ -1409,18 +1202,6 @@ def nonmoving_raw_ptr_for_resizable_list(lst):
     assert isinstance(lst, _ResizableListSupportingRawPtr)
     return lst._nonmoving_raw_ptr_for_resizable_list()
 
-def ll_for_resizable_list(lst):
-    """
-    This is the equivalent of llstr(), but for lists. It can be called only if
-    the list has been created by calling resizable_list_supporting_raw_ptr().
-
-    In theory, all the operations on lst are immediately visible also on
-    ll_list. However, support for that is incomplete in
-    _ResizableListSupportingRawPtr and as such, the pointer becomes invalid as
-    soon as you call a resizing operation on lst.
-    """
-    assert isinstance(lst, _ResizableListSupportingRawPtr)
-    return lst._get_ll_list()
 
 def _check_resizable_list_of_chars(s_list):
     from rpython.annotator import model as annmodel
@@ -1442,10 +1223,6 @@ class Entry(ExtRegistryEntry):
         return s_list
 
     def specialize_call(self, hop):
-        from rpython.rtyper.lltypesystem.rlist import LIST_OF
-        if hop.args_r[0].LIST != LIST_OF(lltype.Char):
-            raise ValueError('Resizable list of chars does not have the '
-                             'expected low-level type')
         hop.exception_cannot_occur()
         return hop.inputarg(hop.args_r[0], 0)
 
@@ -1464,24 +1241,6 @@ class Entry(ExtRegistryEntry):
         return hop.gendirectcall(ll_nonmovable_raw_ptr_for_resizable_list,
                                  v_list)
 
-class Entry(ExtRegistryEntry):
-    _about_ = ll_for_resizable_list
-
-    def compute_result_annotation(self, s_list):
-        from rpython.rtyper.lltypesystem.rlist import LIST_OF
-        from rpython.rtyper.llannotation import lltype_to_annotation
-        _check_resizable_list_of_chars(s_list)
-        LIST = LIST_OF(lltype.Char)
-        return lltype_to_annotation(lltype.Ptr(LIST))
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        assert hop.args_r[0].lowleveltype == hop.r_result.lowleveltype
-        v_ll_list, = hop.inputargs(*hop.args_r)
-        return hop.genop('same_as', [v_ll_list],
-                         resulttype = hop.r_result.lowleveltype)
-
-
 @jit.dont_look_inside
 def ll_nonmovable_raw_ptr_for_resizable_list(ll_list):
     """
