@@ -23,8 +23,6 @@ _POSIX = os.name == "posix"
 _MS_WINDOWS = os.name == "nt"
 _64BIT = "64bit" in platform.architecture()[0]
 _CYGWIN = "cygwin" == sys.platform
-_DARWIN = "darwin" == sys.platform
-_ARM64 = "arm64" == platform.machine()
 
 class RMMapError(Exception):
     def __init__(self, message):
@@ -61,8 +59,6 @@ if _POSIX:
     constant_names = ['MAP_SHARED', 'MAP_PRIVATE', 'MAP_FIXED',
                       'PROT_READ', 'PROT_WRITE',
                       'MS_SYNC']
-    if _DARWIN and _ARM64:
-        constant_names.append('MAP_JIT')
     opt_constant_names = ['MAP_ANON', 'MAP_ANONYMOUS', 'MAP_NORESERVE',
                           'PROT_EXEC',
                           'MAP_DENYWRITE', 'MAP_EXECUTABLE']
@@ -82,36 +78,6 @@ if _POSIX:
     CConfig.MADV_FREE = (
         rffi_platform.DefinedConstantInteger('MADV_FREE'))
 
-    madv_constant_names = [
-        "MADV_NORMAL",
-        "MADV_RANDOM",
-        "MADV_SEQUENTIAL",
-        "MADV_WILLNEED",
-
-        #Linux-specific
-        "MADV_REMOVE",
-        "MADV_DONTFORK",
-        "MADV_DOFORK",
-        "MADV_HWPOISON",
-        "MADV_MERGEABLE",
-        "MADV_UNMERGEABLE",
-        "MADV_SOFT_OFFLINE",
-        "MADV_HUGEPAGE",
-        "MADV_NOHUGEPAGE",
-        "MADV_DONTDUMP",
-        "MADV_DODUMP",
-
-        # FreeBSD-specific
-        "MADV_NOSYNC",
-        "MADV_AUTOSYNC",
-        "MADV_NOCORE",
-        "MADV_CORE",
-        "MADV_PROTECT",
-    ]
-    for name in madv_constant_names:
-        setattr(CConfig, name, rffi_platform.DefinedConstantInteger(name))
-
-
 elif _MS_WINDOWS:
     constant_names = ['PAGE_READONLY', 'PAGE_READWRITE', 'PAGE_WRITECOPY',
                       'FILE_MAP_READ', 'FILE_MAP_WRITE', 'FILE_MAP_COPY',
@@ -205,10 +171,6 @@ if _POSIX:
                                                        includes=includes)
     _get_allocation_granularity = _get_page_size = lambda: _pagesize
 
-    if _DARWIN and _ARM64:
-        _, c_pthread_jit_write_protect_np = external('pthread_jit_write_protect_np',
-            [rffi.INT], lltype.Void)
-
 elif _MS_WINDOWS:
 
     class ComplexCConfig:
@@ -324,7 +286,6 @@ elif _MS_WINDOWS:
             lltype.free(high_ref, flavor='raw')
 
     INVALID_HANDLE = INVALID_HANDLE_VALUE
-    has_madvise = False
 
 PAGESIZE = _get_page_size()
 ALLOCATIONGRANULARITY = _get_allocation_granularity()
@@ -531,7 +492,6 @@ class MMap(object):
 
         self.setslice(start, data)
         self.pos = start + data_len
-        return data_len
 
     def write_byte(self, byte):
         if len(byte) != 1:
@@ -547,8 +507,6 @@ class MMap(object):
         return rffi.ptradd(self.data, offset)
 
     def getslice(self, start, length):
-        if length < 0:
-            return ''
         return rffi.charpsize2str(self.getptr(start), length)
 
     def setslice(self, start, newdata):
@@ -591,9 +549,8 @@ class MMap(object):
             if not has_mremap:
                 raise RValueError("mmap: resizing not available--no mremap()")
 
-            # resize the underlying file first, if there is one
-            if self.fd >= 0:
-                os.ftruncate(self.fd, self.offset + newsize)
+            # resize the underlying file first
+            os.ftruncate(self.fd, self.offset + newsize)
 
             # now resize the mmap
             newdata = c_mremap(self.getptr(0), self.size, newsize,
@@ -663,45 +620,10 @@ class MMap(object):
             index += self.size
         self.data[index] = value[0]
 
-    if has_madvise:
-        def madvise(self, flags, start, end):
-            res = c_madvise_safe(rffi.cast(PTR, rffi.ptradd(self.data, + start)),
-                                 rffi.cast(size_t, end),
-                                 rffi.cast(rffi.INT, flags))
-            if rffi.cast(lltype.Signed, res) == 0:
-                return
-            errno = rposix.get_saved_errno()
-            raise OSError(errno, os.strerror(errno))
-
 def _check_map_size(size):
     if size < 0:
         raise RTypeError("memory mapped size must be positive")
 
-if _DARWIN and _ARM64:
-
-    class Nester(object):
-        def __init__(self):
-            self.counter = 0
-
-    nester = Nester()
-
-    def enter_assembler_writing():
-        if nester.counter == 0:
-            c_pthread_jit_write_protect_np(0)
-        nester.counter += 1
-
-    def leave_assembler_writing():
-        nester.counter -= 1
-        if nester.counter == 0:
-            c_pthread_jit_write_protect_np(1)
-
-else:
-    def enter_assembler_writing():
-        pass
-
-    def leave_assembler_writing():
-        pass
-
 if _POSIX:
     def mmap(fileno, length, flags=MAP_SHARED,
              prot=PROT_WRITE | PROT_READ, access=_ACCESS_DEFAULT, offset=0):
@@ -785,8 +707,6 @@ if _POSIX:
 
     def alloc_hinted(hintp, map_size):
         flags = MAP_PRIVATE | MAP_ANONYMOUS
-        if _DARWIN and _ARM64:
-            flags |= MAP_JIT
         prot = PROT_EXEC | PROT_READ | PROT_WRITE
         if we_are_translated():
             flags = NonConstant(flags)
@@ -911,7 +831,7 @@ elif _MS_WINDOWS:
         # assume -1 and 0 both mean invalid file descriptor
         # to 'anonymously' map memory.
         if fileno != -1 and fileno != 0:
-            fh = rffi.cast(HANDLE, rwin32.get_osfhandle(fileno))
+            fh = rwin32.get_osfhandle(fileno)
             # Win9x appears to need us seeked to zero
             # SEEK_SET = 0
             # libc._lseek(fileno, 0, SEEK_SET)
