@@ -3,12 +3,10 @@ from rpython.rlib.objectmodel import specialize, we_are_translated, enforceargs
 from rpython.rlib.rstring import StringBuilder, UnicodeBuilder
 from rpython.rlib.rarithmetic import r_uint, intmask, widen
 from rpython.rlib.unicodedata import unicodedb
-from rpython.tool.sourcetools import func_with_new_name
 from rpython.rtyper.lltypesystem import lltype, rffi
-from rpython.rlib import jit, nonconst
+from rpython.rlib import jit
 
 
-# We always use MAXUNICODE = 0x10ffff when unicode objects use utf8
 if rffi.sizeof(lltype.UniChar) == 4:
     MAXUNICODE = 0x10ffff
     allow_surrogate_by_default = False
@@ -17,8 +15,6 @@ else:
     allow_surrogate_by_default = True
 
 BYTEORDER = sys.byteorder
-BYTEORDER2 = BYTEORDER[0] + 'e'      # either "le" or "be"
-assert BYTEORDER2 in ('le', 'be')
 
 # python 2.7 has a preview of py3k behavior, so those functions
 # are used either when we're testing wide pypy on narrow cpython
@@ -95,7 +91,6 @@ def is_narrow_host():
 
 def default_unicode_error_decode(errors, encoding, msg, s,
                                  startingpos, endingpos):
-    assert endingpos >= 0
     if errors == 'replace':
         return u'\ufffd', endingpos
     if errors == 'ignore':
@@ -104,7 +99,6 @@ def default_unicode_error_decode(errors, encoding, msg, s,
 
 def default_unicode_error_encode(errors, encoding, msg, u,
                                  startingpos, endingpos):
-    assert endingpos >= 0
     if errors == 'replace':
         return u'?', None, endingpos
     if errors == 'ignore':
@@ -114,7 +108,15 @@ def default_unicode_error_encode(errors, encoding, msg, u,
 # ____________________________________________________________
 # utf-8
 
-_utf8_code_length = ''.join(map(chr, [
+utf8_code_length = [
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, # 00-0F
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, # 70-7F
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, # 80-8F
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -123,17 +125,17 @@ _utf8_code_length = ''.join(map(chr, [
     2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, # D0-DF
     3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, # E0-EF
     4, 4, 4, 4, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  # F0-F4 - F5-FF
-]))
+]
 
-# if you can't use the @elidable version, call str_decode_utf_8_impl()
-# directly
-@jit.elidable
 def str_decode_utf_8(s, size, errors, final=False,
                      errorhandler=None, allow_surrogates=allow_surrogate_by_default):
     if errorhandler is None:
         errorhandler = default_unicode_error_decode
-    return str_decode_utf_8_elidable(s, size, errors, final, errorhandler,
-                                     allow_surrogates=allow_surrogates)
+    result = UnicodeBuilder(size)
+    pos = str_decode_utf_8_impl(s, size, errors, final, errorhandler,
+                                 allow_surrogates=allow_surrogates,
+                                 result=result)
+    return result.build(), pos
 
 def _invalid_cont_byte(ordch):
     return ordch>>6 != 0x2    # 0b10
@@ -155,15 +157,14 @@ def _invalid_byte_2_of_4(ordch1, ordch2):
             (ordch1 == 0xf0 and ordch2 < 0x90) or
             (ordch1 == 0xf4 and ordch2 > 0x8f))
 
-# NOTE: this is a slightly fixed algorithm when compared with
-# CPython2's.  It is closer to CPython3's.  See comments in
-# test_invalid_cb_for_3bytes_seq().
+# note: this specialize() is here for rtyper/rstr.py, which calls this
+# function too but with its own fixed errorhandler
+@specialize.arg_or_var(4)
 def str_decode_utf_8_impl(s, size, errors, final, errorhandler,
-                          allow_surrogates):
+                          allow_surrogates, result):
     if size == 0:
-        return u'', 0
+        return 0
 
-    result = UnicodeBuilder(size)
     pos = 0
     while pos < size:
         ordch1 = ord(s[pos])
@@ -174,7 +175,7 @@ def str_decode_utf_8_impl(s, size, errors, final, errorhandler,
             pos += 1
             continue
 
-        n = ord(_utf8_code_length[ordch1 - 0x80])
+        n = utf8_code_length[ordch1]
         if pos + n > size:
             if not final:
                 break
@@ -315,9 +316,7 @@ def str_decode_utf_8_impl(s, size, errors, final, errorhandler,
                 result.append(unichr(0xDC00 + (c & 0x03FF)))
             pos += 4
 
-    return result.build(), pos
-str_decode_utf_8_elidable = jit.elidable(
-    func_with_new_name(str_decode_utf_8_impl, "str_decode_utf_8_elidable"))
+    return pos
 
 def _encodeUCS4(result, ch):
     # Encode UCS4 Unicode ordinals
@@ -326,26 +325,16 @@ def _encodeUCS4(result, ch):
     result.append((chr((0x80 | ((ch >> 6) & 0x3f)))))
     result.append((chr((0x80 | (ch & 0x3f)))))
 
-# if you can't use the @elidable version, call unicode_encode_utf_8_impl()
-# directly
-@jit.elidable
 def unicode_encode_utf_8(s, size, errors, errorhandler=None,
                          allow_surrogates=allow_surrogate_by_default):
-    # In this function, allow_surrogates can be:
-    #
-    #  * True:  surrogates are always allowed.  A valid surrogate pair
-    #           is replaced with the non-BMP unicode char it stands for,
-    #           which is then encoded as 4 bytes.
-    #
-    #  * False: surrogates are always forbidden.
-    #
-    # See also unicode_encode_utf8sp().
-    #
     if errorhandler is None:
         errorhandler = default_unicode_error_encode
-    return unicode_encode_utf_8_elidable(s, size, errors, errorhandler,
-                                         allow_surrogates=allow_surrogates)
+    return unicode_encode_utf_8_impl(s, size, errors, errorhandler,
+                                     allow_surrogates=allow_surrogates)
 
+# note: this specialize() is here for rtyper/rstr.py, which calls this
+# function too but with its own fixed errorhandler
+@specialize.arg_or_var(3)
 def unicode_encode_utf_8_impl(s, size, errors, errorhandler,
                               allow_surrogates=False):
     assert(size >= 0)
@@ -378,12 +367,7 @@ def unicode_encode_utf_8_impl(s, size, errors, errorhandler,
                             pos += 1
                             _encodeUCS4(result, ch3)
                             continue
-                    # note: if the program only ever calls this with
-                    # allow_surrogates=True, then we'll never annotate
-                    # the following block of code, and errorhandler()
-                    # will never be called.  This causes RPython
-                    # problems.  Avoid it with the nonconst hack.
-                    if not allow_surrogates or nonconst.NonConstant(False):
+                    if not allow_surrogates:
                         ru, rs, pos = errorhandler(errors, 'utf8',
                                                    'surrogates not allowed',
                                                    s, pos-1, pos)
@@ -406,70 +390,6 @@ def unicode_encode_utf_8_impl(s, size, errors, errorhandler,
             else:
                 _encodeUCS4(result, ch)
     return result.build()
-unicode_encode_utf_8_elidable = jit.elidable(
-    enforceargs(s=unicode, allow_surrogates=bool)(
-    func_with_new_name(unicode_encode_utf_8_impl,
-                       "unicode_encode_utf_8_elidable")))
-
-def unicode_encode_utf8sp(s, size):
-    # Surrogate-preserving utf-8 encoding.  Any surrogate character
-    # turns into its 3-bytes encoding, whether it is paired or not.
-    # This should always be reversible, and the reverse is the regular
-    # str_decode_utf_8() with allow_surrogates=True.
-    assert(size >= 0)
-    result = StringBuilder(size)
-    pos = 0
-    while pos < size:
-        ch = ord(s[pos])
-        pos += 1
-        if ch < 0x80:
-            # Encode ASCII
-            result.append(chr(ch))
-        elif ch < 0x0800:
-            # Encode Latin-1
-            result.append(chr((0xc0 | (ch >> 6))))
-            result.append(chr((0x80 | (ch & 0x3f))))
-        elif ch < 0x10000:
-            # Encode UCS2 Unicode ordinals, and surrogates
-            result.append((chr((0xe0 | (ch >> 12)))))
-            result.append((chr((0x80 | ((ch >> 6) & 0x3f)))))
-            result.append((chr((0x80 | (ch & 0x3f)))))
-        else:
-            _encodeUCS4(result, ch)
-    return result.build()
-
-class SurrogateError(Exception):
-    def __init__(self, char, index):
-        self.char = char
-        self.index = index
-
-def unicode_encode_utf8_forbid_surrogates(s, size):
-    # Strict surrogate-forbidding utf-8 encoding.  Any surrogate character
-    # raises an interp-level SurrogateError, even on 16-bit hosts.
-    # --- XXX check in detail what occurs on 16-bit hosts in PyPy 3 ---
-    assert(size >= 0)
-    result = StringBuilder(size)
-    pos = 0
-    while pos < size:
-        ch = ord(s[pos])
-        pos += 1
-        if ch < 0x80:
-            # Encode ASCII
-            result.append(chr(ch))
-        elif ch < 0x0800:
-            # Encode Latin-1
-            result.append(chr((0xc0 | (ch >> 6))))
-            result.append(chr((0x80 | (ch & 0x3f))))
-        elif ch < 0x10000:
-            if 0xD800 <= ch <= 0xDFFF:
-                raise SurrogateError(ch, pos)
-            # Encode UCS2 Unicode ordinals
-            result.append((chr((0xe0 | (ch >> 12)))))
-            result.append((chr((0x80 | ((ch >> 6) & 0x3f)))))
-            result.append((chr((0x80 | (ch & 0x3f)))))
-        else:
-            _encodeUCS4(result, ch)
-    return result.build()
 
 # ____________________________________________________________
 # utf-16
@@ -492,31 +412,9 @@ def str_decode_utf_16_le(s, size, errors, final=True,
                                                          errorhandler, "little")
     return result, length
 
-def py3k_str_decode_utf_16(s, size, errors, final=True,
-                           errorhandler=None):
-    result, length, byteorder = str_decode_utf_16_helper(s, size, errors, final,
-                                                         errorhandler, "native",
-                                                         'utf-16-' + BYTEORDER2)
-    return result, length
-
-def py3k_str_decode_utf_16_be(s, size, errors, final=True,
-                              errorhandler=None):
-    result, length, byteorder = str_decode_utf_16_helper(s, size, errors, final,
-                                                         errorhandler, "big",
-                                                         'utf-16-be')
-    return result, length
-
-def py3k_str_decode_utf_16_le(s, size, errors, final=True,
-                              errorhandler=None):
-    result, length, byteorder = str_decode_utf_16_helper(s, size, errors, final,
-                                                         errorhandler, "little",
-                                                         'utf-16-le')
-    return result, length
-
 def str_decode_utf_16_helper(s, size, errors, final=True,
                              errorhandler=None,
-                             byteorder="native",
-                             public_encoding_name='utf16'):
+                             byteorder="native"):
     if errorhandler is None:
         errorhandler = default_unicode_error_decode
     bo = 0
@@ -574,8 +472,7 @@ def str_decode_utf_16_helper(s, size, errors, final=True,
         if len(s) - pos < 2:
             if not final:
                 break
-            r, pos = errorhandler(errors, public_encoding_name,
-                                  "truncated data",
+            r, pos = errorhandler(errors, 'utf16', "truncated data",
                                   s, pos, len(s))
             result.append(r)
             if len(s) - pos < 2:
@@ -591,8 +488,7 @@ def str_decode_utf_16_helper(s, size, errors, final=True,
             if not final:
                 break
             errmsg = "unexpected end of data"
-            r, pos = errorhandler(errors, public_encoding_name,
-                                  errmsg, s, pos, len(s))
+            r, pos = errorhandler(errors, 'utf16', errmsg, s, pos, len(s))
             result.append(r)
             if len(s) - pos < 2:
                 break
@@ -608,12 +504,12 @@ def str_decode_utf_16_helper(s, size, errors, final=True,
                                            (ch2 & 0x3FF)) + 0x10000))
                 continue
             else:
-                r, pos = errorhandler(errors, public_encoding_name,
+                r, pos = errorhandler(errors, 'utf16',
                                       "illegal UTF-16 surrogate",
                                       s, pos - 4, pos - 2)
                 result.append(r)
         else:
-            r, pos = errorhandler(errors, public_encoding_name,
+            r, pos = errorhandler(errors, 'utf16',
                                   "illegal encoding",
                                   s, pos - 2, pos)
             result.append(r)
@@ -621,11 +517,7 @@ def str_decode_utf_16_helper(s, size, errors, final=True,
 
 def unicode_encode_utf_16_helper(s, size, errors,
                                  errorhandler=None,
-                                 allow_surrogates=True,
-                                 byteorder='little',
-                                 public_encoding_name='utf16'):
-    if errorhandler is None:
-        errorhandler = default_unicode_error_encode
+                                 byteorder='little'):
     if size == 0:
         if byteorder == 'native':
             result = StringBuilder(2)
@@ -638,121 +530,55 @@ def unicode_encode_utf_16_helper(s, size, errors,
         _STORECHAR(result, 0xFEFF, BYTEORDER)
         byteorder = BYTEORDER
 
-    pos = 0
-    while pos < size:
-        ch = ord(s[pos])
-        pos += 1
+    i = 0
+    while i < size:
+        ch = ord(s[i])
+        i += 1
+        ch2 = 0
+        if ch >= 0x10000:
+            ch2 = 0xDC00 | ((ch-0x10000) & 0x3FF)
+            ch  = 0xD800 | ((ch-0x10000) >> 10)
 
-        if ch < 0xD800:
-            _STORECHAR(result, ch, byteorder)
-        elif ch >= 0x10000:
-            _STORECHAR(result, 0xD800 | ((ch-0x10000) >> 10), byteorder)
-            _STORECHAR(result, 0xDC00 | ((ch-0x10000) & 0x3FF), byteorder)
-        elif ch >= 0xE000 or allow_surrogates:
-            _STORECHAR(result, ch, byteorder)
-        else:
-            ru, rs, pos = errorhandler(errors, public_encoding_name,
-                                       'surrogates not allowed',
-                                       s, pos-1, pos)
-            if rs is not None:
-                # py3k only
-                if len(rs) % 2 != 0:
-                    errorhandler('strict', public_encoding_name,
-                                 'surrogates not allowed',
-                                 s, pos-1, pos)
-                result.append(rs)
-                continue
-            for ch in ru:
-                if ord(ch) < 0xD800:
-                    _STORECHAR(result, ord(ch), byteorder)
-                else:
-                    errorhandler('strict', public_encoding_name,
-                                 'surrogates not allowed',
-                                 s, pos-1, pos)
-            continue
+        _STORECHAR(result, ch, byteorder)
+        if ch2:
+            _STORECHAR(result, ch2, byteorder)
 
     return result.build()
 
 def unicode_encode_utf_16(s, size, errors,
-                          errorhandler=None,
-                          allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "native")
+                          errorhandler=None):
+    return unicode_encode_utf_16_helper(s, size, errors, errorhandler, "native")
+
 
 def unicode_encode_utf_16_be(s, size, errors,
-                             errorhandler=None,
-                             allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "big")
+                             errorhandler=None):
+    return unicode_encode_utf_16_helper(s, size, errors, errorhandler, "big")
+
 
 def unicode_encode_utf_16_le(s, size, errors,
-                             errorhandler=None,
-                             allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "little")
-
-def py3k_unicode_encode_utf_16(s, size, errors,
-                          errorhandler=None,
-                          allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "native",
-                                        'utf-16-' + BYTEORDER2)
-
-def py3k_unicode_encode_utf_16_be(s, size, errors,
-                             errorhandler=None,
-                             allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "big",
-                                        'utf-16-be')
-
-def py3k_unicode_encode_utf_16_le(s, size, errors,
-                             errorhandler=None,
-                             allow_surrogates=True):
-    return unicode_encode_utf_16_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "little",
-                                        'utf-16-le')
+                             errorhandler=None):
+    return unicode_encode_utf_16_helper(s, size, errors, errorhandler, "little")
 
 
 # ____________________________________________________________
-# utf-32 (not used in PyPy any more)
+# utf-32
 
 def str_decode_utf_32(s, size, errors, final=True,
                       errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "native")
+    result, length, byteorder = str_decode_utf_32_helper(s, size, errors, final,
+                                                         errorhandler, "native")
     return result, length
 
 def str_decode_utf_32_be(s, size, errors, final=True,
                          errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "big")
+    result, length, byteorder = str_decode_utf_32_helper(s, size, errors, final,
+                                                         errorhandler, "big")
     return result, length
 
 def str_decode_utf_32_le(s, size, errors, final=True,
                          errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "little")
-    return result, length
-
-def py3k_str_decode_utf_32(s, size, errors, final=True,
-                           errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "native",
-        'utf-32-' + BYTEORDER2, allow_surrogates=False)
-    return result, length
-
-def py3k_str_decode_utf_32_be(s, size, errors, final=True,
-                              errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "big",
-        'utf-32-be', allow_surrogates=False)
-    return result, length
-
-def py3k_str_decode_utf_32_le(s, size, errors, final=True,
-                              errorhandler=None):
-    result, length, byteorder = str_decode_utf_32_helper(
-        s, size, errors, final, errorhandler, "little",
-        'utf-32-le', allow_surrogates=False)
+    result, length, byteorder = str_decode_utf_32_helper(s, size, errors, final,
+                                                         errorhandler, "little")
     return result, length
 
 BOM32_DIRECT  = intmask(0x0000FEFF)
@@ -760,9 +586,7 @@ BOM32_REVERSE = intmask(0xFFFE0000)
 
 def str_decode_utf_32_helper(s, size, errors, final=True,
                              errorhandler=None,
-                             byteorder="native",
-                             public_encoding_name='utf32',
-                             allow_surrogates=True):
+                             byteorder="native"):
     if errorhandler is None:
         errorhandler = default_unicode_error_decode
     bo = 0
@@ -817,8 +641,7 @@ def str_decode_utf_32_helper(s, size, errors, final=True,
         if len(s) - pos < 4:
             if not final:
                 break
-            r, pos = errorhandler(errors, public_encoding_name,
-                                  "truncated data",
+            r, pos = errorhandler(errors, 'utf32', "truncated data",
                                   s, pos, len(s))
             result.append(r)
             if len(s) - pos < 4:
@@ -826,15 +649,8 @@ def str_decode_utf_32_helper(s, size, errors, final=True,
             continue
         ch = ((ord(s[pos + iorder[3]]) << 24) | (ord(s[pos + iorder[2]]) << 16) |
               (ord(s[pos + iorder[1]]) << 8)  | ord(s[pos + iorder[0]]))
-        if not allow_surrogates and 0xD800 <= ch <= 0xDFFFF:
-            r, pos = errorhandler(errors, public_encoding_name,
-                                  "code point in surrogate code point "
-                                  "range(0xd800, 0xe000)",
-                                  s, pos, len(s))
-            result.append(r)
-        elif ch >= 0x110000:
-            r, pos = errorhandler(errors, public_encoding_name,
-                                  "codepoint not in range(0x110000)",
+        if ch >= 0x110000:
+            r, pos = errorhandler(errors, 'utf32', "codepoint not in range(0x110000)",
                                   s, pos, len(s))
             result.append(r)
             continue
@@ -866,11 +682,7 @@ def _STORECHAR32(result, CH, byteorder):
 
 def unicode_encode_utf_32_helper(s, size, errors,
                                  errorhandler=None,
-                                 allow_surrogates=True,
-                                 byteorder='little',
-                                 public_encoding_name='utf32'):
-    if errorhandler is None:
-        errorhandler = default_unicode_error_encode
+                                 byteorder='little'):
     if size == 0:
         if byteorder == 'native':
             result = StringBuilder(4)
@@ -883,72 +695,33 @@ def unicode_encode_utf_32_helper(s, size, errors,
         _STORECHAR32(result, 0xFEFF, BYTEORDER)
         byteorder = BYTEORDER
 
-    pos = 0
-    while pos < size:
-        ch = ord(s[pos])
-        pos += 1
+    i = 0
+    while i < size:
+        ch = ord(s[i])
+        i += 1
         ch2 = 0
-        if not allow_surrogates and 0xD800 <= ch < 0xE000:
-            ru, rs, pos = errorhandler(errors, public_encoding_name,
-                                        'surrogates not allowed',
-                                        s, pos-1, pos)
-            if rs is not None:
-                # py3k only
-                if len(rs) % 4 != 0:
-                    errorhandler('strict', public_encoding_name,
-                                    'surrogates not allowed',
-                                    s, pos-1, pos)
-                result.append(rs)
-                continue
-            for ch in ru:
-                if ord(ch) < 0xD800:
-                    _STORECHAR32(result, ord(ch), byteorder)
-                else:
-                    errorhandler('strict', public_encoding_name,
-                                    'surrogates not allowed',
-                                    s, pos-1, pos)
-            continue
-        if 0xD800 <= ch < 0xDC00 and MAXUNICODE < 65536 and pos < size:
-            ch2 = ord(s[pos])
-            if 0xDC00 <= ch2 < 0xE000:
+        if MAXUNICODE < 65536 and 0xD800 <= ch <= 0xDBFF and i < size:
+            ch2 = ord(s[i])
+            if 0xDC00 <= ch2 <= 0xDFFF:
                 ch = (((ch & 0x3FF)<<10) | (ch2 & 0x3FF)) + 0x10000;
-                pos += 1
+                i += 1
         _STORECHAR32(result, ch, byteorder)
 
     return result.build()
 
 def unicode_encode_utf_32(s, size, errors,
-                          errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "native")
+                          errorhandler=None):
+    return unicode_encode_utf_32_helper(s, size, errors, errorhandler, "native")
+
 
 def unicode_encode_utf_32_be(s, size, errors,
-                             errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "big")
+                             errorhandler=None):
+    return unicode_encode_utf_32_helper(s, size, errors, errorhandler, "big")
+
 
 def unicode_encode_utf_32_le(s, size, errors,
-                             errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "little")
-
-def py3k_unicode_encode_utf_32(s, size, errors,
-                               errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "native",
-                                        'utf-32-' + BYTEORDER2)
-
-def py3k_unicode_encode_utf_32_be(s, size, errors,
-                                  errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "big",
-                                        'utf-32-be')
-
-def py3k_unicode_encode_utf_32_le(s, size, errors,
-                                  errorhandler=None, allow_surrogates=True):
-    return unicode_encode_utf_32_helper(s, size, errors, errorhandler,
-                                        allow_surrogates, "little",
-                                        'utf-32-le')
+                             errorhandler=None):
+    return unicode_encode_utf_32_helper(s, size, errors, errorhandler, "little")
 
 
 # ____________________________________________________________
@@ -1568,12 +1341,7 @@ def make_unicode_escape_function(pass_printable=False, unicode_output=False,
         CHR = chr
 
     def unicode_escape(s, size, errors, errorhandler=None):
-        # errors and errorhandler are not used: this function cannot cause
-        # Unicode errors
-        return _unicode_escape(s, size)
-
-    @jit.elidable
-    def _unicode_escape(s, size):
+        # errorhandler is not used: this function cannot cause Unicode errors
         result = STRING_BUILDER(size)
 
         if quotes:
@@ -1664,8 +1432,6 @@ def make_unicode_escape_function(pass_printable=False, unicode_output=False,
 # This function is also used by _codecs/interp_codecs.py
 (unicode_encode_unicode_escape, raw_unicode_escape_helper
  ) = make_unicode_escape_function()
-(_, raw_unicode_escape_helper_unicode
-) = make_unicode_escape_function(unicode_output=True)
 
 # ____________________________________________________________
 # Raw unicode escape
@@ -1779,6 +1545,8 @@ def str_decode_unicode_internal(s, size, errors, final=False,
                                     "truncated input",
                                     s, pos, size)
             result.append(res)
+            if pos > size - unicode_bytes:
+                break
             continue
         t = r_uint(0)
         h = 0
@@ -1903,9 +1671,7 @@ if sys.platform == 'win32':
                 if MultiByteToWideChar(CP_ACP, flags,
                                        dataptr, size, buf.raw, usize) == 0:
                     _decode_mbcs_error(s, errorhandler)
-                ret = buf.str(usize)
-                assert ret is not None
-                return ret, size
+                return buf.str(usize), size
 
     def unicode_encode_mbcs(s, size, errors, errorhandler=None,
                             force_replace=True):
