@@ -11,7 +11,7 @@ from rpython.tool.udir import udir
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.translator.platform import CompilationError
 from rpython.rtyper.lltypesystem import lltype, rffi
-from rpython.rlib.rarithmetic import intmask, r_longlong, widen
+from rpython.rlib.rarithmetic import intmask
 from rpython.rlib import jit
 
 # This module can be imported on any platform,
@@ -20,7 +20,7 @@ WIN32 = os.name == "nt"
 
 if WIN32:
     eci = ExternalCompilationInfo(
-        includes = ['windows.h', 'stdio.h', 'stdlib.h', 'io.h'],
+        includes = ['windows.h', 'stdio.h', 'stdlib.h'],
         libraries = ['kernel32'],
         )
 else:
@@ -30,8 +30,8 @@ class CConfig:
     _compilation_info_ = eci
 
     if WIN32:
-        DWORD_PTR = rffi_platform.SimpleType("DWORD_PTR", rffi.UNSIGNED)
-        WORD = rffi_platform.SimpleType("WORD", rffi.USHORT)
+        DWORD_PTR = rffi_platform.SimpleType("DWORD_PTR", rffi.LONG)
+        WORD = rffi_platform.SimpleType("WORD", rffi.UINT)
         DWORD = rffi_platform.SimpleType("DWORD", rffi.UINT)
         BOOL = rffi_platform.SimpleType("BOOL", rffi.LONG)
         BYTE = rffi_platform.SimpleType("BYTE", rffi.UCHAR)
@@ -46,11 +46,9 @@ class CConfig:
         LPWSTR = rffi_platform.SimpleType("LPWSTR", rffi.CWCHARP)
         LPCWSTR = rffi_platform.SimpleType("LPCWSTR", rffi.CWCHARP)
         LPDWORD = rffi_platform.SimpleType("LPDWORD", rffi.UINTP)
-        LPWORD = rffi_platform.SimpleType("LPWORD", rffi.USHORTP)
         LPBOOL = rffi_platform.SimpleType("LPBOOL", rffi.LONGP)
-        LPBYTE = rffi_platform.SimpleType("LPBYTE", rffi.UCHARP)
         SIZE_T = rffi_platform.SimpleType("SIZE_T", rffi.SIZE_T)
-        ULONG_PTR = rffi_platform.SimpleType("ULONG_PTR", rffi.UNSIGNED)
+        ULONG_PTR = rffi_platform.SimpleType("ULONG_PTR", rffi.ULONG)
 
         HRESULT = rffi_platform.SimpleType("HRESULT", rffi.LONG)
         HLOCAL = rffi_platform.SimpleType("HLOCAL", rffi.VOIDP)
@@ -115,13 +113,11 @@ class CConfig:
                        MB_ERR_INVALID_CHARS ERROR_NO_UNICODE_TRANSLATION
                        WC_NO_BEST_FIT_CHARS STD_INPUT_HANDLE STD_OUTPUT_HANDLE
                        STD_ERROR_HANDLE HANDLE_FLAG_INHERIT FILE_TYPE_CHAR
-                       LOAD_WITH_ALTERED_SEARCH_PATH CT_CTYPE3 C3_HIGHSURROGATE
-                       CP_ACP CP_UTF8 CP_UTF7 CP_OEMCP MB_ERR_INVALID_CHARS
                     """
         from rpython.translator.platform import host_factory
         static_platform = host_factory()
         if static_platform.name == 'msvc':
-            defines += ' PROCESS_QUERY_LIMITED_INFORMATION'
+            defines += ' PROCESS_QUERY_LIMITED_INFORMATION' 
         for name in defines.split():
             locals()[name] = rffi_platform.ConstantInteger(name)
 
@@ -140,15 +136,6 @@ if WIN32:
     HMODULE = HANDLE
     NULL_HANDLE = rffi.cast(HANDLE, 0)
     INVALID_HANDLE_VALUE = rffi.cast(HANDLE, -1)
-    GENERIC_READ     = rffi.cast(DWORD, r_longlong(0x80000000))
-    GENERIC_WRITE    = rffi.cast(DWORD, r_longlong(0x40000000))
-    GENERIC_EXECUTE  = rffi.cast(DWORD, r_longlong(0x20000000))
-    GENERIC_ALL      = rffi.cast(DWORD, r_longlong(0x10000000))
-    FILE_SHARE_READ  = rffi.cast(DWORD, r_longlong(0x00000001))
-    FILE_SHARE_WRITE = rffi.cast(DWORD, r_longlong(0x00000002))
-    ALL_READ_WRITE   = rffi.cast(DWORD, r_longlong(0xC0000003))
-    SHARE_READ_WRITE = rffi.cast(DWORD, r_longlong(0x00000003))
-
     PFILETIME = rffi.CArrayPtr(FILETIME)
 
     _GetLastError = winexternal('GetLastError', [], DWORD,
@@ -167,9 +154,7 @@ if WIN32:
         (into the same "saved LastError" variable).
         """
         from rpython.rlib import rthread
-        # extra cast to LONG to match CPython behaviour
-        lasterror = rffi.cast(rffi.LONG, rthread.tlfield_rpy_lasterror.getraw())
-        return rffi.cast(lltype.Signed, lasterror)
+        return rffi.cast(lltype.Signed, rthread.tlfield_rpy_lasterror.getraw())
 
     def SetLastError_saved(err):
         """Set the value of the saved LastError.  This value will be used in
@@ -186,15 +171,13 @@ if WIN32:
         function, if that C function was declared with the flag
         llexternal(..., save_err=RFFI_SAVE_LASTERROR | RFFI_ALT_ERRNO).
         Functions without that flag don't change the saved LastError.
-        Alternatively, if the function was declared
+        Alternatively, if the function was declared 
         RFFI_SAVE_WSALASTERROR | RFFI_ALT_ERRNO,
         then the value of the C-level WSAGetLastError() is saved instead
         (into the same "saved alt LastError" variable).
         """
         from rpython.rlib import rthread
-        # extra cast to LONG to match CPython behaviour
-        lasterror = rffi.cast(rffi.LONG, rthread.tlfield_alt_lasterror.getraw())
-        return rffi.cast(lltype.Signed, lasterror)
+        return rffi.cast(lltype.Signed, rthread.tlfield_alt_lasterror.getraw())
 
     def SetLastError_alt_saved(err):
         """Set the value of the saved alt LastError.  This value will be used in
@@ -212,28 +195,12 @@ if WIN32:
     GetModuleHandle = winexternal('GetModuleHandleA', [rffi.CCHARP], HMODULE)
     LoadLibrary = winexternal('LoadLibraryA', [rffi.CCHARP], HMODULE,
                               save_err=rffi.RFFI_SAVE_LASTERROR)
-    def wrap_loadlibraryex(func):
-        def loadlibrary(name, flags=LOAD_WITH_ALTERED_SEARCH_PATH):
-            # Requires a full path name with '/' -> '\\'
-            return func(name, NULL_HANDLE, flags)
-        return loadlibrary
-
-    _LoadLibraryExA = winexternal('LoadLibraryExA',
-                                [rffi.CCHARP, HANDLE, DWORD], HMODULE,
-                                save_err=rffi.RFFI_SAVE_LASTERROR)
-    LoadLibraryExA = wrap_loadlibraryex(_LoadLibraryExA)
-    LoadLibraryW = winexternal('LoadLibraryW', [rffi.CWCHARP], HMODULE,
-                              save_err=rffi.RFFI_SAVE_LASTERROR)
-    _LoadLibraryExW = winexternal('LoadLibraryExW',
-                                [rffi.CWCHARP, HANDLE, DWORD], HMODULE,
-                                save_err=rffi.RFFI_SAVE_LASTERROR)
-    LoadLibraryExW = wrap_loadlibraryex(_LoadLibraryExW)
     GetProcAddress = winexternal('GetProcAddress',
                                  [HMODULE, rffi.CCHARP],
                                  rffi.VOIDP)
     FreeLibrary = winexternal('FreeLibrary', [HMODULE], BOOL, releasegil=False)
 
-    LocalFree = winexternal('LocalFree', [HLOCAL], HLOCAL)
+    LocalFree = winexternal('LocalFree', [HLOCAL], DWORD)
     CloseHandle = winexternal('CloseHandle', [HANDLE], BOOL, releasegil=False,
                               save_err=rffi.RFFI_SAVE_LASTERROR)
     CloseHandle_no_err = winexternal('CloseHandle', [HANDLE], BOOL,
@@ -243,42 +210,62 @@ if WIN32:
         'FormatMessageA',
         [DWORD, rffi.VOIDP, DWORD, DWORD, rffi.CCHARP, DWORD, rffi.VOIDP],
         DWORD)
-    FormatMessageW = winexternal(
-        'FormatMessageW',
-        [DWORD, rffi.VOIDP, DWORD, DWORD, rffi.CWCHARP, DWORD, rffi.VOIDP],
-        DWORD)
 
-    _get_osfhandle = rffi.llexternal('_get_osfhandle', [rffi.INT], rffi.INTP)
+    _get_osfhandle = rffi.llexternal('_get_osfhandle', [rffi.INT], HANDLE)
 
     def get_osfhandle(fd):
-        from rpython.rlib.rposix import FdValidator
-        with FdValidator(fd):
-            handle = rffi.cast(HANDLE, _get_osfhandle(fd))
+        from rpython.rlib.rposix import validate_fd
+        validate_fd(fd)
+        handle = _get_osfhandle(fd)
         if handle == INVALID_HANDLE_VALUE:
             raise WindowsError(ERROR_INVALID_HANDLE, "Invalid file handle")
         return handle
 
-    _open_osfhandle = rffi.llexternal('_open_osfhandle', [rffi.INTP, rffi.INT], rffi.INT)
-
-    def open_osfhandle(handle, flags):
-        from rpython.rlib.rposix import FdValidator
-        fd = _open_osfhandle(handle, flags)
-        with FdValidator(fd):
-            return fd
-    
-    wcsncpy_s = rffi.llexternal('wcsncpy_s', 
-                    [rffi.CWCHARP, rffi.SIZE_T, rffi.CWCHARP, rffi.SIZE_T], rffi.INT)
-
     def build_winerror_to_errno():
         """Build a dictionary mapping windows error numbers to POSIX errno.
         The function returns the dict, and the default value for codes not
         in the dict."""
         # Prior to Visual Studio 8, the MSVCRT dll doesn't export the
         # _dosmaperr() function, which is available only when compiled
-        # against the static CRT library. After Visual Studio 9, this
-        # private function seems to be gone, so use a static map, from
-        # CPython PC/errmap.h
-        errors = {
+        # against the static CRT library.
+        from rpython.translator.platform import host_factory
+        static_platform = host_factory()
+        if static_platform.name == 'msvc':
+            static_platform.cflags = ['/MT']  # static CRT
+            static_platform.version = 0       # no manifest
+        cfile = udir.join('dosmaperr.c')
+        cfile.write(r'''
+                #include <errno.h>
+                #include <WinError.h>
+                #include <stdio.h>
+                #ifdef __GNUC__
+                #define _dosmaperr mingw_dosmaperr
+                #endif
+                int main()
+                {
+                    int i;
+                    for(i=1; i < 65000; i++) {
+                        _dosmaperr(i);
+                        if (errno == EINVAL) {
+                            /* CPython issue #12802 */
+                            if (i == ERROR_DIRECTORY)
+                                errno = ENOTDIR;
+                            else
+                                continue;
+                        }
+                        printf("%d\t%d\n", i, errno);
+                    }
+                    return 0;
+                }''')
+        try:
+            exename = static_platform.compile(
+                [cfile], ExternalCompilationInfo(),
+                outputfilename = "dosmaperr",
+                standalone=True)
+        except (CompilationError, WindowsError):
+            # Fallback for the mingw32 compiler
+            assert static_platform.name == 'mingw32'
+            errors = {
                 2: 2, 3: 2, 4: 24, 5: 13, 6: 9, 7: 12, 8: 12, 9: 12, 10: 7,
                 11: 8, 15: 2, 16: 13, 17: 18, 18: 2, 19: 13, 20: 13, 21: 13,
                 22: 13, 23: 13, 24: 13, 25: 13, 26: 13, 27: 13, 28: 13,
@@ -288,18 +275,17 @@ if WIN32:
                 132: 13, 145: 41, 158: 13, 161: 2, 164: 11, 167: 13, 183: 17,
                 188: 8, 189: 8, 190: 8, 191: 8, 192: 8, 193: 8, 194: 8,
                 195: 8, 196: 8, 197: 8, 198: 8, 199: 8, 200: 8, 201: 8,
-                202: 8, 206: 2, 215: 11, 232: 32, 267: 20, 1816: 12,
+                202: 8, 206: 2, 215: 11, 267: 20, 1816: 12,
                 }
+        else:
+            output = os.popen(str(exename))
+            errors = dict(map(int, line.split())
+                          for line in output)
         return errors, errno.EINVAL
 
     # A bit like strerror...
     def FormatError(code):
         return llimpl_FormatError(code)
-    def FormatErrorW(code):
-        """
-        returns utf8, n_codepoints
-        """
-        return llimpl_FormatErrorW(code)
 
     def llimpl_FormatError(code):
         "Return a message corresponding to the given Windows error code."
@@ -307,7 +293,7 @@ if WIN32:
         buf[0] = lltype.nullptr(rffi.CCHARP.TO)
         try:
             msglen = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
-                                   FORMAT_MESSAGE_FROM_SYSTEM |
+                                   FORMAT_MESSAGE_FROM_SYSTEM | 
                                    FORMAT_MESSAGE_IGNORE_INSERTS,
                                    None,
                                    rffi.cast(DWORD, code),
@@ -332,47 +318,12 @@ if WIN32:
 
         return result
 
-    def llimpl_FormatErrorW(code):
-        "Return a utf8-encoded msg and its length"
-        buf = lltype.malloc(rffi.CWCHARPP.TO, 1, flavor='raw')
-        buf[0] = lltype.nullptr(rffi.CWCHARP.TO)
-        try:
-            msglen = FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER |
-                                    FORMAT_MESSAGE_FROM_SYSTEM |
-                                    FORMAT_MESSAGE_IGNORE_INSERTS,
-                                    None,
-                                    rffi.cast(DWORD, code),
-                                    DEFAULT_LANGUAGE,
-                                    rffi.cast(rffi.CWCHARP, buf),
-                                    0, None)
-            buflen = intmask(msglen)
-
-            # remove trailing cr/lf and dots
-            s_buf = buf[0]
-            while buflen > 0 and (ord(s_buf[buflen - 1]) <= ord(' ') or
-                                  s_buf[buflen - 1] == u'.'):
-                buflen -= 1
-
-            if buflen <= 0:
-                msg = 'Windows Error %d' % (code,)
-                result = msg, len(msg)
-            else:
-                result = rffi.wcharpsize2utf8(s_buf, buflen), buflen
-        finally:
-            LocalFree(rffi.cast(rffi.VOIDP, buf[0]))
-            lltype.free(buf, flavor='raw')
-
-        return result
-
     def lastSavedWindowsError(context="Windows Error"):
         code = GetLastError_saved()
         return WindowsError(code, context)
 
     def FAILED(hr):
-        # XXX convert to int before checking result
-        #     because 32-bit arithmetic is unimplemented on win64
-        #     this is fine since HRESULT is defined as (signed) LONG
-        return int(rffi.cast(HRESULT, hr)) < 0
+        return rffi.cast(HRESULT, hr) < 0
 
     _GetModuleFileName = winexternal('GetModuleFileNameA',
                                      [HMODULE, rffi.CCHARP, DWORD],
@@ -551,7 +502,7 @@ if WIN32:
         try:
             if not _GetHandleInformation(handle, pflags):
                 raise lastSavedWindowsError("GetHandleInformation")
-            flags = widen(pflags[0])
+            flags = pflags[0]
         finally:
             lltype.free(pflags, flavor='raw')
         return (flags & HANDLE_FLAG_INHERIT) != 0
@@ -583,34 +534,3 @@ if WIN32:
                 os.close(fd2)
                 raise
         return res
-    
-    GetConsoleMode = winexternal(
-        'GetConsoleMode', [HANDLE, LPDWORD], BOOL)
-        
-    GetNumberOfConsoleInputEvents = winexternal(
-        'GetNumberOfConsoleInputEvents', [HANDLE, LPDWORD], BOOL)
-
-    ERROR_INSUFFICIENT_BUFFER = 122
-    ERROR_OPERATION_ABORTED   = 995
-    CP_UTF8 = 65001 
-    
-    ReadConsoleW = winexternal(
-        'ReadConsoleW', [HANDLE, LPWSTR, DWORD, LPDWORD, LPVOID], BOOL,
-        save_err=rffi.RFFI_SAVE_LASTERROR)
-        
-    WriteConsoleW = winexternal(
-        'WriteConsoleW', [HANDLE, LPVOID, DWORD, LPDWORD, LPVOID], BOOL,
-        save_err=rffi.RFFI_SAVE_LASTERROR)
-
-    GetStringTypeW = winexternal(
-        'GetStringTypeW', [DWORD, rffi.CWCHARP, rffi.INT, LPWORD], BOOL, 
-        save_err=rffi.RFFI_SAVE_LASTERROR)
-
-    _SetEnvironmentVariableW = winexternal(
-        'SetEnvironmentVariableW', [LPWSTR, LPWSTR], BOOL,
-        save_err=rffi.RFFI_SAVE_LASTERROR)
-
-    def SetEnvironmentVariableW(name, value):
-        with rffi.scoped_unicode2wcharp(name) as nameWbuf:
-            with rffi.scoped_unicode2wcharp(value) as valueWbuf:
-                return _SetEnvironmentVariableW(nameWbuf, valueWbuf) 
