@@ -1,6 +1,3 @@
-from hypothesis import given, strategies as st, assume
-import pytest
-
 from rpython.rtyper.test.test_llinterp import interpret
 from rpython.translator.c.test.test_genc import compile
 from rpython.tool.pytest.expecttest import ExpectTest
@@ -11,10 +8,10 @@ import errno
 import py
 
 def rposix_requires(funcname):
-    return pytest.mark.skipif(not hasattr(rposix, funcname),
+    return py.test.mark.skipif(not hasattr(rposix, funcname),
         reason="Requires rposix.%s()" % funcname)
 
-win_only = pytest.mark.skipif("os.name != 'nt'")
+win_only = py.test.mark.skipif("os.name != 'nt'")
 
 class TestPosixFunction:
     def test_access(self):
@@ -62,7 +59,6 @@ class TestPosixFunction:
         compile(f, (str, float))(str(fname), t1)
         assert t1 == os.stat(str(fname)).st_mtime
 
-    @py.test.mark.skipif("sys.platform == 'darwin'")
     def test_utime_negative_fraction(self):
         def f(fname, t1):
             os.utime(fname, (t1, t1))
@@ -84,14 +80,6 @@ class TestPosixFunction:
         # the most intriguing failure of ntpath.py should not repeat, here:
         assert not data.endswith(stuff)
 
-    @win_only
-    def test__getfullpathname_long(self):
-        stuff = "C:" + "\\abcd" * 100
-        py.test.raises(WindowsError, rposix.getfullpathname, stuff)
-        ustuff = u"C:" + u"\\abcd" * 100
-        res = rposix.getfullpathname(ustuff)
-        assert res == ustuff
-
     def test_getcwd(self):
         assert rposix.getcwd() == os.getcwd()
 
@@ -150,8 +138,7 @@ class TestPosixFunction:
         arg = '%s -c "print 1+1" > %s' % (sys.executable, filename)
         data = rposix.system(arg)
         assert data == 0
-        with file(filename) as f:
-            assert f.read().strip() == '2'
+        assert file(filename).read().strip() == '2'
         os.unlink(filename)
 
 
@@ -294,20 +281,6 @@ class TestPosixFunction:
     def test_isatty(self):
         assert rposix.isatty(-1) is False
 
-    @py.test.mark.skipif("not hasattr(rposix, 'makedev')")
-    def test_makedev(self):
-        dev = rposix.makedev(24, 7)
-        assert rposix.major(dev) == 24
-        assert rposix.minor(dev) == 7
-
-    @py.test.mark.skipif("not hasattr(rposix, 'memfd_create')")
-    def test_memfd_create(self):
-        fd = rposix.memfd_create("abc", rposix.MFD_CLOEXEC)
-        try:
-            s = "defghi?"
-            os.write(fd, s)
-        finally:
-            os.close(fd)
 
 @py.test.mark.skipif("not hasattr(os, 'ttyname')")
 class TestOsExpect(ExpectTest):
@@ -487,6 +460,15 @@ class BasePosixUnicodeOrAscii:
             except Exception:
                 pass
 
+    @win_only
+    def test_is_valid_fd(self):
+        assert rposix.is_valid_fd(0) == 1
+        fid = open(str(udir.join('validate_test.txt')), 'w')
+        fd = fid.fileno()
+        assert rposix.is_valid_fd(fd) == 1
+        fid.close()
+        assert rposix.is_valid_fd(fd) == 0
+
     def test_putenv(self):
         from rpython.rlib import rposix_environ
 
@@ -504,7 +486,7 @@ class BasePosixUnicodeOrAscii:
 
 class TestPosixAscii(BasePosixUnicodeOrAscii):
     def _get_filename(self):
-        return unicode(udir.join('test_open_ascii'))
+        return str(udir.join('test_open_ascii'))
 
     @rposix_requires('openat')
     def test_openat(self):
@@ -681,291 +663,3 @@ def test_set_status_flags():
     finally:
         os.close(fd1)
         os.close(fd2)
-
-@rposix_requires('getpriority')
-def test_getpriority():
-    # a "don't crash" kind of test only
-    prio = rposix.getpriority(rposix.PRIO_PROCESS, 0)
-    rposix.setpriority(rposix.PRIO_PROCESS, 0, prio)
-    py.test.raises(OSError, rposix.getpriority, rposix.PRIO_PGRP, 123456789)
-
-if sys.platform != 'win32':
-    def test_sendfile():
-        from rpython.rlib import rsocket
-        s1, s2 = rsocket.socketpair()
-        relpath = 'test_sendfile'
-        filename = str(udir.join(relpath))
-        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
-        os.write(fd, 'abcdefghij')
-        res = rposix.sendfile(s1.fd, fd, 3, 5)
-        assert res == 5
-        data = os.read(s2.fd, 10)
-        assert data == 'defgh'
-        os.close(fd)
-        s2.close()
-        s1.close()
-
-    def test_sendfile_invalid_offset():
-        from rpython.rlib import rsocket
-        s1, s2 = rsocket.socketpair()
-        relpath = 'test_sendfile_invalid_offset'
-        filename = str(udir.join(relpath))
-        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
-        os.write(fd, 'abcdefghij')
-        with py.test.raises(OSError) as excinfo:
-            rposix.sendfile(s1.fd, fd, -1, 5)
-        assert excinfo.value.errno == errno.EINVAL
-        os.close(fd)
-        s2.close()
-        s1.close()
-
-if sys.platform.startswith('linux'):
-    def test_sendfile_no_offset():
-        from rpython.rlib import rsocket
-        s1, s2 = rsocket.socketpair()
-        relpath = 'test_sendfile'
-        filename = str(udir.join(relpath))
-        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
-        os.write(fd, 'abcdefghij')
-        os.lseek(fd, 3, 0)
-        res = rposix.sendfile_no_offset(s1.fd, fd, 5)
-        assert res == 5
-        data = os.read(s2.fd, 10)
-        assert data == 'defgh'
-        os.close(fd)
-        s2.close()
-        s1.close()
-
-@rposix_requires('pread')
-def test_pread():
-    fname = str(udir.join('os_test.txt'))
-    fd = os.open(fname, os.O_RDWR | os.O_CREAT)
-    try:
-        assert fd >= 0
-        os.write(fd, b'Hello world')
-        os.lseek(fd, 0, 0)
-        assert rposix.pread(fd, 2, 1) == b'el'
-    finally:
-        os.close(fd)
-    py.test.raises(OSError, rposix.pread, fd, 2, 1)
-
-@rposix_requires('pwrite')
-def test_pwrite():
-    fname = str(udir.join('os_test.txt'))
-    fd = os.open(fname, os.O_RDWR | os.O_CREAT, 0777)
-    try:
-        assert fd >= 0
-        os.write(fd, b'Hello world')
-        os.lseek(fd, 0, 0)
-        rposix.pwrite(fd, b'ea', 1)
-        assert os.read(fd, 4) == b'Heal'
-    finally:
-        os.close(fd)
-    py.test.raises(OSError, rposix.pwrite, fd, b'ea', 1)
-
-@rposix_requires('posix_fadvise')
-def test_posix_fadvise():
-    if sys.maxint <= 2**32:
-        py.test.skip("ll2ctypes run of posix_fadvise() on 32-bit "
-                     "gets confused by the size of OFF_T")
-    fname = str(udir.join('test_os_posix_fadvise'))
-    fd = os.open(fname, os.O_CREAT | os.O_RDWR)
-    try:
-        os.write(fd, b"foobar")
-        assert rposix.posix_fadvise(fd, 0, 1, rposix.POSIX_FADV_WILLNEED) is None
-        assert rposix.posix_fadvise(fd, 1, 1, rposix.POSIX_FADV_NORMAL) is None
-        assert rposix.posix_fadvise(fd, 2, 1, rposix.POSIX_FADV_SEQUENTIAL) is None
-        assert rposix.posix_fadvise(fd, 3, 1, rposix.POSIX_FADV_RANDOM) is None
-        assert rposix.posix_fadvise(fd, 4, 1, rposix.POSIX_FADV_NOREUSE) is None
-        assert rposix.posix_fadvise(fd, 5, 1, rposix.POSIX_FADV_DONTNEED) is None
-        py.test.raises(OSError, rposix.posix_fadvise, fd, 6, 1, 1234567)
-    finally:
-        os.close(fd)
-
-@rposix_requires('posix_fallocate')
-def test_posix_fallocate():
-    if sys.maxint <= 2**32:
-        py.test.skip("ll2ctypes run of posix_fallocate() on 32-bit "
-                     "gets confused by the size of OFF_T")
-    fname = str(udir.join('os_test.txt'))
-    fd = os.open(fname, os.O_WRONLY | os.O_CREAT, 0777)
-    try:
-        assert rposix.posix_fallocate(fd, 0, 10) == 0
-    except OSError as inst:
-        """ ZFS seems not to support fallocate.
-        so skipping solaris-based since it is likely to come with ZFS
-        """
-        if inst.errno != errno.EINVAL or not sys.platform.startswith("sunos"):
-            raise
-    finally:
-        os.close(fd)
-
-@rposix_requires('sched_get_priority_max')
-def test_sched_get_priority_max():
-    assert rposix.sched_get_priority_max(rposix.SCHED_FIFO) != -1
-    assert rposix.sched_get_priority_max(rposix.SCHED_RR) != -1
-    assert rposix.sched_get_priority_max(rposix.SCHED_OTHER) != -1
-    if sys.platform != 'darwin':
-        assert rposix.sched_get_priority_max(rposix.SCHED_BATCH) != -1
-
-@rposix_requires('sched_get_priority_min')
-def test_sched_get_priority_min():
-    assert rposix.sched_get_priority_min(rposix.SCHED_FIFO) != -1
-    assert rposix.sched_get_priority_min(rposix.SCHED_RR) != -1
-    assert rposix.sched_get_priority_min(rposix.SCHED_OTHER) != -1
-    if sys.platform != 'darwin':
-        assert rposix.sched_get_priority_min(rposix.SCHED_BATCH) != -1
-
-@rposix_requires('sched_get_priority_min')
-def test_os_sched_priority_max_greater_than_min():
-    policy = rposix.SCHED_RR
-    low = rposix.sched_get_priority_min(policy)
-    high = rposix.sched_get_priority_max(policy)
-    assert isinstance(low, int) == True
-    assert isinstance(high, int) == True
-    assert  high > low
-
-@rposix_requires('sched_yield')
-def test_sched_yield():
-    if sys.platform != 'win32':
-        rposix.sched_yield()
-
-@rposix_requires('lockf')
-def test_os_lockf():
-    fname = str(udir.join('os_test.txt'))
-    fd = os.open(fname, os.O_WRONLY | os.O_CREAT, 0777)
-    try:
-        os.write(fd, b'test')
-        os.lseek(fd, 0, 0)
-        rposix.lockf(fd, rposix.F_LOCK, 4)
-        rposix.lockf(fd, rposix.F_ULOCK, 4)
-    finally:
-        os.close(fd)
-
-def check_working_xattr():
-    fname = str(udir.join('xattr_test0.txt'))
-    with open(fname, 'wb'):
-        pass
-    try:
-        rposix.setxattr(fname, 'user.foo', '')
-    except OSError:
-        return False
-    else:
-        return True
-
-@pytest.mark.skipif(not (hasattr(rposix, 'getxattr') and check_working_xattr()),
-    reason="Requires working rposix.getxattr()")
-@given(
-    name=st.text(
-        alphabet=st.characters(min_codepoint=1), min_size=1, max_size=10),
-    value=st.binary(max_size=10),
-    follow_symlinks=st.booleans(), use_fd=st.booleans())
-def test_xattr(name, value, follow_symlinks, use_fd):
-    assume(follow_symlinks or not use_fd)
-    name = 'user.' + name.encode('utf-8')
-    fname = str(udir.join('xattr_test.txt'))
-    try:
-        os.unlink(fname)
-    except OSError:
-        pass
-    with open(fname, 'wb'):
-        pass
-    if use_fd:
-        file_id = os.open(fname, os.O_CREAT, 0777)
-        read, write, delete = rposix.fgetxattr, rposix.fsetxattr, rposix.fremovexattr
-        all_names = rposix.flistxattr
-    else:
-        file_id = fname
-        if follow_symlinks:
-            read, write, delete = rposix.getxattr, rposix.setxattr, rposix.removexattr
-            all_names = rposix.listxattr
-        else:
-            read = lambda *args, **kwargs: rposix.getxattr(*args, follow_symlinks=False, **kwargs)
-            write = lambda *args, **kwargs: rposix.setxattr(*args, follow_symlinks=False, **kwargs)
-            delete = lambda *args, **kwargs: rposix.removexattr(*args, follow_symlinks=False, **kwargs)
-            all_names = lambda *args, **kwargs: rposix.listxattr(*args, follow_symlinks=False, **kwargs)
-    try:
-        init_names = all_names(file_id)
-        with pytest.raises(OSError):
-            read(file_id, name)
-        write(file_id, name, value)
-        assert read(file_id, name) == value
-        assert set(all_names(file_id)) == set(init_names + [name])
-        assert '' not in all_names(file_id)
-        delete(file_id, name)
-        with pytest.raises(OSError):
-            read(file_id, name)
-        assert set(all_names(file_id)) == set(init_names)
-    finally:
-        if use_fd:
-            os.close(file_id)
-
-@pytest.mark.skipif(not hasattr(rposix, 'getgrouplist'),
-    reason="Requires working rposix.getgrouplist()")
-@rposix_requires('getgrouplist')
-def test_getgrouplist():
-    import pwd
-    user = pwd.getpwuid(os.getuid())[0]
-    group = pwd.getpwuid(os.getuid())[3]
-    assert group in rposix.getgrouplist(user, group)
-
-@pytest.mark.skipif(not hasattr(rposix, 'sched_rr_get_interval'),
-    reason="Requires working rposix.sched_rr_get_interval()")
-def test_sched_rr_get_interval():
-    try:
-        interval = rposix.sched_rr_get_interval(0)
-    except OSError as e:
-        # This likely means that sched_rr_get_interval is only valid for
-        # processes with the SCHED_RR scheduler in effect.
-        if e.errno != errno.EINVAL:
-                raise
-        pytest.mark.skip("only works on SCHED_RR processes")
-    assert isinstance(interval, float)
-    # Reasonable constraints, I think.
-    assert interval > 0
-    assert interval < 1.
-
-@pytest.mark.skipif(not hasattr(rposix, 'sched_getscheduler'),
-    reason="Requires working rposix.sched_getscheduler()")
-def test_get_and_set_scheduler_and_param():
-    possible_schedulers = [sched for name, sched in rposix.__dict__.items()
-                                   if name.startswith("SCHED_")]
-    mine = rposix.sched_getscheduler(0)
-    assert mine in possible_schedulers
-    try:
-        parent = rposix.sched_getscheduler(os.getppid())
-    except OSError as e:
-        if e.errno != errno.EPERM:
-            raise
-    else:
-        assert parent in possible_schedulers
-    with pytest.raises(OSError):
-        rposix.sched_getscheduler(-1)
-    with pytest.raises(OSError):
-        rposix.sched_getparam(-1)
-    param = rposix.sched_getparam(0)
-    assert param == 0
-
-    # POSIX states that calling sched_setparam() or sched_setscheduler() on
-    # a process with a scheduling policy other than SCHED_FIFO or SCHED_RR
-    # is implementation-defined: NetBSD and FreeBSD can return EINVAL.
-    if not sys.platform.startswith(('freebsd', 'netbsd')):
-        try:
-            rposix.sched_setscheduler(0, mine, param)
-            rposix.sched_setparam(0, param)
-        except OSError as e:
-            if e.errno != errno.EPERM:
-                raise
-        with pytest.raises(OSError):
-            rposix.sched_setparam(-1, param)
-    with pytest.raises(OSError):
-        rposix.sched_setscheduler(-1, mine, param)
-
-    large = 214748364700
-    if large < sys.maxint:
-        param = large # rposix.sched_param(large)
-        with pytest.raises(OSError):
-            rposix.sched_setparam(0, param)
-        # param = rposix.sched_param(sched_priority=-large)
-        # with pytest.raises(OverflowError):
-        #    rposix.sched_setparam(0, param)
