@@ -7,6 +7,8 @@ from rpython.annotator import model as annmodel
 from rpython.annotator.policy import AnnotatorPolicy
 from rpython.annotator.signature import Sig
 from rpython.annotator.specialize import flatten_star_args
+from rpython.rtyper.llannotation import (
+    SomePtr, annotation_to_lltype, lltype_to_annotation)
 from rpython.rtyper.normalizecalls import perform_normalizations
 from rpython.rtyper.lltypesystem import lltype, llmemory
 from rpython.flowspace.model import Constant
@@ -39,9 +41,10 @@ class KeyComp(object):
     __repr__ = __str__
 
 class LowLevelAnnotatorPolicy(AnnotatorPolicy):
-    def __init__(pol, rtyper=None):
-        pol.rtyper = rtyper
+    def __init__(self, rtyper=None):
+        self.rtyper = rtyper
 
+    @staticmethod
     def lowlevelspecialize(funcdesc, args_s, key_for_args):
         args_s, key1, builder = flatten_star_args(funcdesc, args_s)
         key = []
@@ -54,10 +57,13 @@ class LowLevelAnnotatorPolicy(AnnotatorPolicy):
                 assert s_obj.is_constant(), "ambiguous low-level helper specialization"
                 key.append(KeyComp(s_obj.const))
                 new_args_s.append(s_obj)
+            elif isinstance(s_obj, annmodel.SomeNone):
+                key.append(KeyComp(None))
+                new_args_s.append(s_obj)
             else:
                 new_args_s.append(annmodel.not_const(s_obj))
                 try:
-                    key.append(annmodel.annotation_to_lltype(s_obj))
+                    key.append(annotation_to_lltype(s_obj))
                 except ValueError:
                     # passing non-low-level types to a ll_* function is allowed
                     # for module/ll_*
@@ -68,28 +74,20 @@ class LowLevelAnnotatorPolicy(AnnotatorPolicy):
         flowgraph = funcdesc.cachedgraph(key, builder=builder)
         args_s[:] = new_args_s
         return flowgraph
-    lowlevelspecialize = staticmethod(lowlevelspecialize)
 
+    @staticmethod
     def default_specialize(funcdesc, args_s):
         return LowLevelAnnotatorPolicy.lowlevelspecialize(funcdesc, args_s, {})
-    default_specialize = staticmethod(default_specialize)
-
-    def specialize__semierased(funcdesc, args_s):
-        a2l = annmodel.annotation_to_lltype
-        l2a = annmodel.lltype_to_annotation
-        args_s[:] = [l2a(a2l(s)) for s in args_s]
-        return LowLevelAnnotatorPolicy.default_specialize(funcdesc, args_s)
-    specialize__semierased = staticmethod(specialize__semierased)
 
     specialize__ll = default_specialize
 
+    @staticmethod
     def specialize__ll_and_arg(funcdesc, args_s, *argindices):
         keys = {}
         for i in argindices:
             keys[i] = args_s[i].const
         return LowLevelAnnotatorPolicy.lowlevelspecialize(funcdesc, args_s,
                                                           keys)
-    specialize__ll_and_arg = staticmethod(specialize__ll_and_arg)
 
 def annotate_lowlevel_helper(annotator, ll_function, args_s, policy=None):
     if policy is None:
@@ -101,27 +99,26 @@ def annotate_lowlevel_helper(annotator, ll_function, args_s, policy=None):
 
 class MixLevelAnnotatorPolicy(LowLevelAnnotatorPolicy):
 
-    def __init__(pol, annhelper):
-        pol.annhelper = annhelper
-        pol.rtyper = annhelper.rtyper
+    def __init__(self, annhelper):
+        self.rtyper = annhelper.rtyper
 
-    def default_specialize(pol, funcdesc, args_s):
+    def default_specialize(self, funcdesc, args_s):
         name = funcdesc.name
         if name.startswith('ll_') or name.startswith('_ll_'): # xxx can we do better?
-            return super(MixLevelAnnotatorPolicy, pol).default_specialize(
+            return super(MixLevelAnnotatorPolicy, self).default_specialize(
                 funcdesc, args_s)
         else:
             return AnnotatorPolicy.default_specialize(funcdesc, args_s)
 
-    def specialize__arglltype(pol, funcdesc, args_s, i):
-        key = pol.rtyper.getrepr(args_s[i]).lowleveltype
+    def specialize__arglltype(self, funcdesc, args_s, i):
+        key = self.rtyper.getrepr(args_s[i]).lowleveltype
         alt_name = funcdesc.name+"__for_%sLlT" % key._short_name()
         return funcdesc.cachedgraph(key, alt_name=valid_identifier(alt_name))
 
-    def specialize__genconst(pol, funcdesc, args_s, i):
+    def specialize__genconst(self, funcdesc, args_s, i):
         # XXX this is specific to the JIT
-        TYPE = annmodel.annotation_to_lltype(args_s[i], 'genconst')
-        args_s[i] = annmodel.lltype_to_annotation(TYPE)
+        TYPE = annotation_to_lltype(args_s[i], 'genconst')
+        args_s[i] = lltype_to_annotation(TYPE)
         alt_name = funcdesc.name + "__%s" % (TYPE._short_name(),)
         return funcdesc.cachedgraph(TYPE, alt_name=valid_identifier(alt_name))
 
@@ -132,10 +129,10 @@ class MixLevelHelperAnnotator(object):
         self.rtyper = rtyper
         self.policy = MixLevelAnnotatorPolicy(self)
         self.pending = []     # list of (ll_function, graph, args_s, s_result)
-        self.delayedreprs = {}
+        self.delayedreprs = set()
         self.delayedconsts = []
         self.delayedfuncs = []
-        self.newgraphs = {}
+        self.newgraphs = set()
 
     def getgraph(self, ll_function, args_s, s_result):
         # get the graph of the mix-level helper ll_function and prepare it for
@@ -193,7 +190,7 @@ class MixLevelHelperAnnotator(object):
         else:
             delayed = r.set_setup_maybe_delayed()
         if delayed:
-            self.delayedreprs[r] = True
+            self.delayedreprs.add(r)
         return r
 
     def s_r_instanceof(self, cls, can_be_None=True, check_never_seen=True):
@@ -235,7 +232,7 @@ class MixLevelHelperAnnotator(object):
             ann.annotated[graph.returnblock] = graph
             s_function = bk.immutablevalue(ll_function)
             bk.emulate_pbc_call(graph, s_function, args_s)
-            self.newgraphs[graph] = True
+            self.newgraphs.add(graph)
         ann.complete_helpers(self.policy)
         for ll_function, graph, args_s, s_result in self.pending:
             s_real_result = ann.binding(graph.getreturnvar())
@@ -246,13 +243,13 @@ class MixLevelHelperAnnotator(object):
                                 (graph, s_result, s_real_result))
         del self.pending[:]
         for graph in translator.graphs[original_graph_count:]:
-            self.newgraphs[graph] = True
+            self.newgraphs.add(graph)
 
     def finish_rtype(self):
         rtyper = self.rtyper
         translator = rtyper.annotator.translator
         original_graph_count = len(translator.graphs)
-        perform_normalizations(rtyper)
+        perform_normalizations(rtyper.annotator)
         for r in self.delayedreprs:
             r.set_setup_delayed(False)
         rtyper.call_all_setups()
@@ -260,7 +257,7 @@ class MixLevelHelperAnnotator(object):
             p._become(repr.convert_const(obj))
         rtyper.call_all_setups()
         for p, graph in self.delayedfuncs:
-            self.newgraphs[graph] = True
+            self.newgraphs.add(graph)
             real_p = rtyper.getcallable(graph)
             REAL = lltype.typeOf(real_p).TO
             FUNCTYPE = lltype.typeOf(p).TO
@@ -273,13 +270,13 @@ class MixLevelHelperAnnotator(object):
         del self.delayedconsts[:]
         del self.delayedfuncs[:]
         for graph in translator.graphs[original_graph_count:]:
-            self.newgraphs[graph] = True
+            self.newgraphs.add(graph)
 
     def backend_optimize(self, **flags):
         # only optimize the newly created graphs
         from rpython.translator.backendopt.all import backend_optimizations
         translator = self.rtyper.annotator.translator
-        newgraphs = self.newgraphs.keys()
+        newgraphs = list(self.newgraphs)
         backend_optimizations(translator, newgraphs, secondary=True,
                               inline_graph_from_anywhere=True, **flags)
         self.newgraphs.clear()
@@ -351,19 +348,30 @@ class LLHelperEntry(extregistry.ExtRegistryEntry):
     _about_ = llhelper
 
     def compute_result_annotation(self, s_F, s_callable):
+        from rpython.annotator.description import FunctionDesc
         assert s_F.is_constant()
-        assert s_callable.is_constant()
+        assert isinstance(s_callable, annmodel.SomePBC)
         F = s_F.const
         FUNC = F.TO
-        args_s = [annmodel.lltype_to_annotation(T) for T in FUNC.ARGS]
-        key = (llhelper, s_callable.const)
-        s_res = self.bookkeeper.emulate_pbc_call(key, s_callable, args_s)
-        assert annmodel.lltype_to_annotation(FUNC.RESULT).contains(s_res)
-        return annmodel.SomePtr(F)
+        args_s = [lltype_to_annotation(T) for T in FUNC.ARGS]
+        for desc in s_callable.descriptions:
+            assert isinstance(desc, FunctionDesc)
+            assert desc.pyobj is not None
+            if s_callable.is_constant():
+                assert s_callable.const is desc.pyobj
+            key = (llhelper, desc.pyobj)
+            s_res = self.bookkeeper.emulate_pbc_call(key, s_callable, args_s)
+            assert lltype_to_annotation(FUNC.RESULT).contains(s_res)
+        return SomePtr(F)
 
     def specialize_call(self, hop):
         hop.exception_cannot_occur()
-        return hop.args_r[1].get_unique_llfn()
+        if hop.args_s[1].is_constant():
+            return hop.args_r[1].get_unique_llfn()
+        else:
+            F = hop.args_s[0].const
+            assert hop.args_r[1].lowleveltype == F
+            return hop.inputarg(hop.args_r[1], 1)
 
 # ____________________________________________________________
 
@@ -418,14 +426,18 @@ def make_string_entries(strtype):
         def compute_result_annotation(self, s_str):
             from rpython.rtyper.lltypesystem.rstr import STR, UNICODE
             if strtype is str:
-                return annmodel.lltype_to_annotation(lltype.Ptr(STR))
+                return lltype_to_annotation(lltype.Ptr(STR))
             else:
-                return annmodel.lltype_to_annotation(lltype.Ptr(UNICODE))
+                return lltype_to_annotation(lltype.Ptr(UNICODE))
 
         def specialize_call(self, hop):
+            from rpython.rtyper.lltypesystem.rstr import (string_repr,
+                                                          unicode_repr)
             hop.exception_cannot_occur()
-            assert hop.args_r[0].lowleveltype == hop.r_result.lowleveltype
-            v_ll_str, = hop.inputargs(*hop.args_r)
+            if strtype is str:
+                v_ll_str = hop.inputarg(string_repr, 0)
+            else:
+                v_ll_str = hop.inputarg(unicode_repr, 0)
             return hop.genop('same_as', [v_ll_str],
                              resulttype = hop.r_result.lowleveltype)
 
@@ -462,7 +474,7 @@ def cast_object_to_ptr(PTR, object):
 
 @specialize.argtype(0)
 def cast_instance_to_base_ptr(instance):
-    from rpython.rtyper.lltypesystem.rclass import OBJECTPTR
+    from rpython.rtyper.rclass import OBJECTPTR
     return cast_object_to_ptr(OBJECTPTR, instance)
 
 @specialize.argtype(0)
@@ -470,18 +482,27 @@ def cast_instance_to_gcref(instance):
     return lltype.cast_opaque_ptr(llmemory.GCREF,
                                   cast_instance_to_base_ptr(instance))
 
+@specialize.argtype(0)
+def cast_nongc_instance_to_base_ptr(instance):
+    from rpython.rtyper.rclass import NONGCOBJECTPTR
+    return cast_object_to_ptr(NONGCOBJECTPTR, instance)
+
+@specialize.argtype(0)
+def cast_nongc_instance_to_adr(instance):
+    return llmemory.cast_ptr_to_adr(cast_nongc_instance_to_base_ptr(instance))
+
 class CastObjectToPtrEntry(extregistry.ExtRegistryEntry):
     _about_ = cast_object_to_ptr
 
     def compute_result_annotation(self, s_PTR, s_object):
         assert s_PTR.is_constant()
         if isinstance(s_PTR.const, lltype.Ptr):
-            return annmodel.SomePtr(s_PTR.const)
+            return SomePtr(s_PTR.const)
         else:
             assert False
 
     def specialize_call(self, hop):
-        from rpython.rtyper import rpbc
+        from rpython.rtyper.rnone import NoneRepr
         PTR = hop.r_result.lowleveltype
         if isinstance(PTR, lltype.Ptr):
             T = lltype.Ptr
@@ -491,7 +512,7 @@ class CastObjectToPtrEntry(extregistry.ExtRegistryEntry):
             assert False
 
         hop.exception_cannot_occur()
-        if isinstance(hop.args_r[1], rpbc.NoneFrozenPBCRepr):
+        if isinstance(hop.args_r[1], NoneRepr):
             return hop.inputconst(PTR, null)
         v_arg = hop.inputarg(hop.args_r[1], arg=1)
         assert isinstance(v_arg.concretetype, T)
@@ -511,6 +532,21 @@ def cast_base_ptr_to_instance(Class, ptr):
                                   % (ptr, Class))
     return ptr
 
+cast_base_ptr_to_nongc_instance = cast_base_ptr_to_instance
+
+@specialize.arg(0)
+def cast_gcref_to_instance(Class, ptr):
+    """Reverse the hacking done in cast_instance_to_gcref()."""
+    from rpython.rtyper.rclass import OBJECTPTR
+    ptr = lltype.cast_opaque_ptr(OBJECTPTR, ptr)
+    return cast_base_ptr_to_instance(Class, ptr)
+
+@specialize.arg(0)
+def cast_adr_to_nongc_instance(Class, ptr):
+    from rpython.rtyper.rclass import NONGCOBJECTPTR
+    ptr = llmemory.cast_adr_to_ptr(ptr, NONGCOBJECTPTR)
+    return cast_base_ptr_to_nongc_instance(Class, ptr)
+
 class CastBasePtrToInstanceEntry(extregistry.ExtRegistryEntry):
     _about_ = cast_base_ptr_to_instance
 
@@ -535,14 +571,14 @@ class CastBasePtrToInstanceEntry(extregistry.ExtRegistryEntry):
 def placeholder_sigarg(s):
     if s == "self":
         def expand(s_self, *args_s):
-            assert isinstance(s_self, annmodel.SomePtr)
+            assert isinstance(s_self, SomePtr)
             return s_self
     elif s == "SELF":
         raise NotImplementedError
     else:
         assert s.islower()
         def expand(s_self, *args_s):
-            assert isinstance(s_self, annmodel.SomePtr)
+            assert isinstance(s_self, SomePtr)
             return getattr(s_self.ll_ptrtype.TO, s.upper())
     return expand
 
