@@ -1,9 +1,9 @@
 from weakref import WeakValueDictionary
 
 from rpython.annotator import model as annmodel
-from rpython.rlib import jit, types, objectmodel, rgc
+from rpython.rlib import jit, types
 from rpython.rlib.objectmodel import (malloc_zero_filled, we_are_translated,
-    ll_hash_string, keepalive_until_here, specialize, enforceargs, dont_inline)
+    _hash_string, keepalive_until_here, specialize, enforceargs)
 from rpython.rlib.signature import signature
 from rpython.rlib.rarithmetic import ovfcheck
 from rpython.rtyper.error import TyperError
@@ -12,7 +12,6 @@ from rpython.rtyper.lltypesystem import ll_str, llmemory
 from rpython.rtyper.lltypesystem.lltype import (GcStruct, Signed, Array, Char,
     UniChar, Ptr, malloc, Bool, Void, GcArray, nullptr, cast_primitive,
     typeOf, staticAdtMethod, GcForwardReference)
-from rpython.rtyper.rbool import BoolRepr
 from rpython.rtyper.rmodel import inputconst, Repr
 from rpython.rtyper.rint import IntegerRepr
 from rpython.rtyper.rstr import (AbstractStringRepr, AbstractCharRepr,
@@ -45,13 +44,11 @@ def new_malloc(TP, name):
 mallocstr = new_malloc(STR, 'mallocstr')
 mallocunicode = new_malloc(UNICODE, 'mallocunicode')
 
-@specialize.memo()
 def emptystrfun():
-    return string_repr.convert_const("")
+    return emptystr
 
-@specialize.memo()
 def emptyunicodefun():
-    return unicode_repr.convert_const(u'')
+    return emptyunicode
 
 def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     @specialize.arg(0)
@@ -91,17 +88,6 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
         ll_assert(srcstart + length <= len(src.chars), "copystrc: src ovf")
         ll_assert(dststart >= 0, "copystrc: negative dststart")
         ll_assert(dststart + length <= len(dst.chars), "copystrc: dst ovf")
-        #
-        # If the 'split_gc_address_space' option is set, we must copy
-        # manually, character-by-character
-        if rgc.must_split_gc_address_space():
-            i = 0
-            while i < length:
-                dst.chars[dststart + i] = src.chars[srcstart + i]
-                i += 1
-            return
-        #  
-        #
         # from here, no GC operations can happen
         asrc = _get_raw_buf(SRC_TP, src, srcstart)
         adst = _get_raw_buf(DST_TP, dst, dststart)
@@ -122,16 +108,6 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
         """
         # xxx Warning: same note as above apply: don't do this at home
         assert length >= 0
-        #
-        # If the 'split_gc_address_space' option is set, we must copy
-        # manually, character-by-character
-        if rgc.must_split_gc_address_space():
-            i = 0
-            while i < length:
-                ptrdst[i] = src.chars[srcstart + i]
-                i += 1
-            return
-        #
         # from here, no GC operations can happen
         asrc = _get_raw_buf(SRC_TP, src, srcstart)
         adst = llmemory.cast_ptr_to_adr(ptrdst)
@@ -148,16 +124,6 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     def copy_raw_to_string(ptrsrc, dst, dststart, length):
         # xxx Warning: same note as above apply: don't do this at home
         assert length >= 0
-        #
-        # If the 'split_gc_address_space' option is set, we must copy
-        # manually, character-by-character
-        if rgc.must_split_gc_address_space():
-            i = 0
-            while i < length:
-                dst.chars[dststart + i] = ptrsrc[i]
-                i += 1
-            return
-        #
         # from here, no GC operations can happen
         adst = _get_raw_buf(SRC_TP, dst, dststart)
         asrc = llmemory.cast_ptr_to_adr(ptrsrc)
@@ -170,19 +136,15 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     copy_raw_to_string = func_with_new_name(copy_raw_to_string,
                                               'copy_raw_to_%s' % name)
 
-    return (copy_string_to_raw, copy_raw_to_string, copy_string_contents,
-            _get_raw_buf)
+    return copy_string_to_raw, copy_raw_to_string, copy_string_contents
 
 (copy_string_to_raw,
  copy_raw_to_string,
- copy_string_contents,
- _get_raw_buf_string) = _new_copy_contents_fun(STR, STR, Char, 'string')
+ copy_string_contents) = _new_copy_contents_fun(STR, STR, Char, 'string')
 
 (copy_unicode_to_raw,
  copy_raw_to_unicode,
- copy_unicode_contents,
- _get_raw_buf_unicode) = _new_copy_contents_fun(UNICODE, UNICODE, UniChar,
-                                                'unicode')
+ copy_unicode_contents) = _new_copy_contents_fun(UNICODE, UNICODE, UniChar, 'unicode')
 
 CONST_STR_CACHE = WeakValueDictionary()
 CONST_UNICODE_CACHE = WeakValueDictionary()
@@ -303,6 +265,21 @@ class UniCharRepr(AbstractUniCharRepr, UnicodeRepr):
 #  get flowed and annotated, mostly with SomePtr.
 #
 
+FAST_COUNT = 0
+FAST_FIND = 1
+FAST_RFIND = 2
+
+
+from rpython.rlib.rarithmetic import LONG_BIT as BLOOM_WIDTH
+
+
+def bloom_add(mask, c):
+    return mask | (1 << (ord(c) & (BLOOM_WIDTH - 1)))
+
+
+def bloom(mask, c):
+    return mask & (1 << (ord(c) & (BLOOM_WIDTH - 1)))
+
 
 class LLHelpers(AbstractLLHelpers):
     from rpython.rtyper.annlowlevel import llstr, llunicode
@@ -394,23 +371,23 @@ class LLHelpers(AbstractLLHelpers):
     @staticmethod
     def ll_strhash(s):
         if s:
-            return jit.conditional_call_elidable(s.hash,
-                                                 LLHelpers._ll_strhash, s)
+            return LLHelpers._ll_strhash(s)
         else:
             return 0
 
     @staticmethod
-    @dont_inline
-    @jit.dont_look_inside
+    @jit.elidable
+    @jit.call_shortcut
     def _ll_strhash(s):
         # unlike CPython, there is no reason to avoid to return -1
         # but our malloc initializes the memory to zero, so we use zero as the
-        # special non-computed-yet value.  Also, jit.conditional_call_elidable
-        # always checks for zero, for now.
-        x = ll_hash_string(s)
+        # special non-computed-yet value.
+        x = s.hash
         if x == 0:
-            x = 29872897
-        s.hash = x
+            x = _hash_string(s.chars)
+            if x == 0:
+                x = 29872897
+            s.hash = x
         return x
 
     @staticmethod
@@ -419,7 +396,6 @@ class LLHelpers(AbstractLLHelpers):
 
     @staticmethod
     def ll_strfasthash(s):
-        ll_assert(s.hash != 0, "ll_strfasthash: hash==0")
         return s.hash     # assumes that the hash is already computed
 
     @staticmethod
@@ -452,10 +428,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos <= rpos and s.chars[lpos] == ch:
+            while lpos < rpos and s.chars[lpos] == ch:
                 lpos += 1
         if right:
-            while lpos <= rpos and s.chars[rpos] == ch:
+            while lpos < rpos + 1 and s.chars[rpos] == ch:
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -473,10 +449,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos <= rpos and s.chars[lpos].isspace():
+            while lpos < rpos and s.chars[lpos].isspace():
                 lpos += 1
         if right:
-            while lpos <= rpos and s.chars[rpos].isspace():
+            while lpos < rpos + 1 and s.chars[rpos].isspace():
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -494,10 +470,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos <= rpos and LLHelpers.ll_contains(s2, s.chars[lpos]):
+            while lpos < rpos and LLHelpers.ll_contains(s2, s.chars[lpos]):
                 lpos += 1
         if right:
-            while lpos <= rpos and LLHelpers.ll_contains(s2, s.chars[rpos]):
+            while lpos < rpos + 1 and LLHelpers.ll_contains(s2, s.chars[rpos]):
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -705,31 +681,6 @@ class LLHelpers(AbstractLLHelpers):
     @staticmethod
     @signature(types.any(), types.any(), types.int(), types.int(), returns=types.int())
     def ll_find(s1, s2, start, end):
-        from rpython.rlib.rstring import SEARCH_FIND
-        if start < 0:
-            start = 0
-        n = len(s1.chars)
-        if end > n:
-            end = n
-        if end - start < 0:
-            return -1
-
-        m = len(s2.chars)
-        if m <= 1:
-            if m == 0:
-                return start
-            res = LLHelpers.ll_find_char(s1, s2.chars[0], start, end)
-            jit.record_exact_value(res < end, True)
-            return res
-
-        res = LLHelpers.ll_search(s1, s2, start, end, SEARCH_FIND)
-        jit.record_exact_value(res < end, True)
-        return res
-
-    @staticmethod
-    @signature(types.any(), types.any(), types.int(), types.int(), returns=types.int())
-    def ll_rfind(s1, s2, start, end):
-        from rpython.rlib.rstring import SEARCH_RFIND
         if start < 0:
             start = 0
         if end > len(s1.chars):
@@ -738,20 +689,29 @@ class LLHelpers(AbstractLLHelpers):
             return -1
 
         m = len(s2.chars)
-        if m <= 1:
-            if m == 0:
-                return end
-            res = LLHelpers.ll_rfind_char(s1, s2.chars[0], start, end)
-            jit.record_exact_value(res < end, True)
-            return res
+        if m == 1:
+            return LLHelpers.ll_find_char(s1, s2.chars[0], start, end)
 
-        res = LLHelpers.ll_search(s1, s2, start, end, SEARCH_RFIND)
-        jit.record_exact_value(res < end, True)
-        return res
+        return LLHelpers.ll_search(s1, s2, start, end, FAST_FIND)
+
+    @staticmethod
+    @signature(types.any(), types.any(), types.int(), types.int(), returns=types.int())
+    def ll_rfind(s1, s2, start, end):
+        if start < 0:
+            start = 0
+        if end > len(s1.chars):
+            end = len(s1.chars)
+        if end - start < 0:
+            return -1
+
+        m = len(s2.chars)
+        if m == 1:
+            return LLHelpers.ll_rfind_char(s1, s2.chars[0], start, end)
+
+        return LLHelpers.ll_search(s1, s2, start, end, FAST_RFIND)
 
     @classmethod
     def ll_count(cls, s1, s2, start, end):
-        from rpython.rlib.rstring import SEARCH_COUNT
         if start < 0:
             start = 0
         if end > len(s1.chars):
@@ -763,19 +723,99 @@ class LLHelpers(AbstractLLHelpers):
         if m == 1:
             return cls.ll_count_char(s1, s2.chars[0], start, end)
 
-        res = cls.ll_search(s1, s2, start, end, SEARCH_COUNT)
+        res = cls.ll_search(s1, s2, start, end, FAST_COUNT)
         assert res >= 0
         return res
 
     @staticmethod
+    @jit.elidable
     def ll_search(s1, s2, start, end, mode):
-        from rpython.rtyper.annlowlevel import hlstr, hlunicode
-        from rpython.rlib import rstring
-        tp = typeOf(s1)
-        if tp == string_repr.lowleveltype or tp == Char:
-            return rstring._search(hlstr(s1), hlstr(s2), start, end, mode)
+        count = 0
+        n = end - start
+        m = len(s2.chars)
+
+        if m == 0:
+            if mode == FAST_COUNT:
+                return end - start + 1
+            elif mode == FAST_RFIND:
+                return end
+            else:
+                return start
+
+        w = n - m
+
+        if w < 0:
+            if mode == FAST_COUNT:
+                return 0
+            return -1
+
+        mlast = m - 1
+        skip = mlast - 1
+        mask = 0
+
+        if mode != FAST_RFIND:
+            for i in range(mlast):
+                mask = bloom_add(mask, s2.chars[i])
+                if s2.chars[i] == s2.chars[mlast]:
+                    skip = mlast - i - 1
+            mask = bloom_add(mask, s2.chars[mlast])
+
+            i = start - 1
+            while i + 1 <= start + w:
+                i += 1
+                if s1.chars[i + m - 1] == s2.chars[m - 1]:
+                    for j in range(mlast):
+                        if s1.chars[i + j] != s2.chars[j]:
+                            break
+                    else:
+                        if mode != FAST_COUNT:
+                            return i
+                        count += 1
+                        i += mlast
+                        continue
+
+                    if i + m < len(s1.chars):
+                        c = s1.chars[i + m]
+                    else:
+                        c = '\0'
+                    if not bloom(mask, c):
+                        i += m
+                    else:
+                        i += skip
+                else:
+                    if i + m < len(s1.chars):
+                        c = s1.chars[i + m]
+                    else:
+                        c = '\0'
+                    if not bloom(mask, c):
+                        i += m
         else:
-            return rstring._search(hlunicode(s1), hlunicode(s2), start, end, mode)
+            mask = bloom_add(mask, s2.chars[0])
+            for i in range(mlast, 0, -1):
+                mask = bloom_add(mask, s2.chars[i])
+                if s2.chars[i] == s2.chars[0]:
+                    skip = i - 1
+
+            i = start + w + 1
+            while i - 1 >= start:
+                i -= 1
+                if s1.chars[i] == s2.chars[0]:
+                    for j in xrange(mlast, 0, -1):
+                        if s1.chars[i + j] != s2.chars[j]:
+                            break
+                    else:
+                        return i
+                    if i - 1 >= 0 and not bloom(mask, s1.chars[i - 1]):
+                        i -= m
+                    else:
+                        i -= skip
+                else:
+                    if i - 1 >= 0 and not bloom(mask, s1.chars[i - 1]):
+                        i -= m
+
+        if mode != FAST_COUNT:
+            return -1
+        return count
 
     @staticmethod
     @signature(types.int(), types.any(), returns=types.any())
@@ -1042,19 +1082,19 @@ class LLHelpers(AbstractLLHelpers):
         strlen = len(chars)
         i = 0
         #XXX: only space is allowed as white space for now
-        while i < strlen and ord(chars[i]) == ord(' '):
+        while i < strlen and chars[i] == ' ':
             i += 1
         if not i < strlen:
             raise ValueError
         #check sign
         sign = 1
-        if ord(chars[i]) == ord('-'):
+        if chars[i] == '-':
             sign = -1
             i += 1
-        elif ord(chars[i]) == ord('+'):
+        elif chars[i] == '+':
             i += 1
         # skip whitespaces between sign and digits
-        while i < strlen and ord(chars[i]) == ord(' '):
+        while i < strlen and chars[i] == ' ':
             i += 1
         #now get digits
         val = 0
@@ -1076,12 +1116,32 @@ class LLHelpers(AbstractLLHelpers):
         if i == oldpos:
             raise ValueError # catch strings like '+' and '+  '
         #skip trailing whitespace
-        while i < strlen and ord(chars[i]) == ord(' '):
+        while i < strlen and chars[i] == ' ':
             i += 1
         if not i == strlen:
             raise ValueError
         return sign * val
 
+    # interface to build strings:
+    #   x = ll_build_start(n)
+    #   ll_build_push(x, next_string, 0)
+    #   ll_build_push(x, next_string, 1)
+    #   ...
+    #   ll_build_push(x, next_string, n-1)
+    #   s = ll_build_finish(x)
+
+    @staticmethod
+    def ll_build_start(parts_count):
+        return malloc(TEMP, parts_count)
+
+    @staticmethod
+    def ll_build_push(builder, next_string, index):
+        builder[index] = next_string
+
+    @staticmethod
+    def ll_build_finish(builder):
+        return LLHelpers.ll_join_strs(len(builder), builder)
+
     @staticmethod
     @specialize.memo()
     def ll_constant(s):
@@ -1128,8 +1188,6 @@ class LLHelpers(AbstractLLHelpers):
                     vchunk = hop.gendirectcall(r_arg.ll_str, vitem)
                 elif code == 'd':
                     assert isinstance(r_arg, IntegerRepr)
-                    # fail early for ints too small, not when specializing target
-                    assert isinstance(r_arg, BoolRepr) or r_arg.opprefix is not None
                     #vchunk = hop.gendirectcall(r_arg.ll_str, vitem)
                     vchunk = hop.gendirectcall(ll_str.ll_int2dec, vitem)
                 elif code == 'f':
@@ -1137,14 +1195,10 @@ class LLHelpers(AbstractLLHelpers):
                     vchunk = hop.gendirectcall(r_arg.ll_str, vitem)
                 elif code == 'x':
                     assert isinstance(r_arg, IntegerRepr)
-                    # fail early for ints too small, not when specializing target
-                    assert isinstance(r_arg, BoolRepr) or r_arg.opprefix is not None
                     vchunk = hop.gendirectcall(ll_str.ll_int2hex, vitem,
                                                inputconst(Bool, False))
                 elif code == 'o':
                     assert isinstance(r_arg, IntegerRepr)
-                    # fail early for ints too small, not when specializing target
-                    assert isinstance(r_arg, BoolRepr) or r_arg.opprefix is not None
                     vchunk = hop.gendirectcall(ll_str.ll_int2oct, vitem,
                                                inputconst(Bool, False))
                 else:
@@ -1174,16 +1228,6 @@ class LLHelpers(AbstractLLHelpers):
         SRC = typeOf(src).TO     # STR or UNICODE
         DST = typeOf(dst).TO     # GcArray
         assert DST.OF is SRC.chars.OF
-        #
-        # If the 'split_gc_address_space' option is set, we must copy
-        # manually, character-by-character
-        if rgc.must_split_gc_address_space():
-            i = 0
-            while i < length:
-                dst[i] = src.chars[i]
-                i += 1
-            return lst
-        #
         # from here, no GC operations can happen
         asrc = llmemory.cast_ptr_to_adr(src) + (
             llmemory.offsetof(SRC, 'chars') +
@@ -1210,8 +1254,7 @@ STR.become(GcStruct('rpy_string', ('hash',  Signed),
                               'gethash': LLHelpers.ll_strhash,
                               'length': LLHelpers.ll_length,
                               'find': LLHelpers.ll_find,
-                              'rfind': LLHelpers.ll_rfind},
-                    hints={'remove_hash': True}))
+                              'rfind': LLHelpers.ll_rfind}))
 UNICODE.become(GcStruct('rpy_unicode', ('hash', Signed),
                         ('chars', Array(UniChar, hints={'immutable': True})),
                         adtmeths={'malloc' : staticAdtMethod(mallocunicode),
@@ -1219,8 +1262,8 @@ UNICODE.become(GcStruct('rpy_unicode', ('hash', Signed),
                                   'copy_contents' : staticAdtMethod(copy_unicode_contents),
                                   'copy_contents_from_str' : staticAdtMethod(copy_unicode_contents),
                                   'gethash': LLHelpers.ll_strhash,
-                                  'length': LLHelpers.ll_length},
-                    hints={'remove_hash': True}))
+                                  'length': LLHelpers.ll_length}
+                        ))
 
 
 # TODO: make the public interface of the rstr module cleaner
@@ -1235,6 +1278,8 @@ unichar_repr = UniCharRepr()
 char_repr.ll = LLHelpers
 unichar_repr.ll = LLHelpers
 unicode_repr = UnicodeRepr()
+emptystr = string_repr.convert_const("")
+emptyunicode = unicode_repr.convert_const(u'')
 
 StringRepr.repr = string_repr
 UnicodeRepr.repr = unicode_repr
@@ -1293,6 +1338,14 @@ def ll_getnextindex(iter):
 string_repr.iterator_repr = StringIteratorRepr()
 unicode_repr.iterator_repr = UnicodeIteratorRepr()
 
-@specialize.memo()
-def conststr(s):
-    return string_repr.convert_const(s)
+# these should be in rclass, but circular imports prevent (also it's
+# not that insane that a string constant is built in this file).
+
+instance_str_prefix = string_repr.convert_const("<")
+instance_str_infix  = string_repr.convert_const(" object at 0x")
+instance_str_suffix = string_repr.convert_const(">")
+
+null_str = string_repr.convert_const("NULL")
+
+unboxed_instance_str_prefix = string_repr.convert_const("<unboxed ")
+unboxed_instance_str_suffix = string_repr.convert_const(">")
