@@ -8,101 +8,37 @@ from rpython.rtyper.rmodel import IteratorRepr, inputconst, Repr
 from rpython.rtyper.rint import IntegerRepr
 from rpython.rtyper.rfloat import FloatRepr
 from rpython.tool.pairtype import pairtype, pair
-
-def str_decode_utf8(s):
-    from rpython.rlib.rstring import UnicodeBuilder
-    from rpython.rlib import runicode
-
-    size = len(s)
-    if size == 0:
-        return u''
-
-    result = UnicodeBuilder(size)
-    pos = 0
-    while pos < size:
-        ordch1 = ord(s[pos])
-        # fast path for ASCII
-        # XXX maybe use a while loop here
-        if ordch1 < 0x80:
-            result.append(unichr(ordch1))
-            pos += 1
-            continue
-
-        n = ord(runicode._utf8_code_length[ordch1 - 0x80])
-        if pos + n > size:
-            raise UnicodeDecodeError('utf8', s, pos, pos + 1,
-                                      'whatever')
-        if n == 0:
-            raise UnicodeDecodeError('utf8', s, pos, pos + 1,
-                                     'whatever')
-        elif n == 1:
-            assert 0, "ascii should have gone through the fast path"
-
-        elif n == 2:
-            ordch2 = ord(s[pos+1])
-            if runicode._invalid_byte_2_of_2(ordch2):
-
-                raise UnicodeDecodeError('utf8', s, pos, pos + 1,
-                                         'whatever')
-            # 110yyyyy 10zzzzzz -> 00000000 00000yyy yyzzzzzz
-            result.append(unichr(((ordch1 & 0x1F) << 6) +    # 0b00011111
-                                 (ordch2 & 0x3F)))           # 0b00111111
-            pos += 2
-
-        elif n == 3:
-            ordch2 = ord(s[pos+1])
-            ordch3 = ord(s[pos+2])
-            if (runicode._invalid_byte_2_of_3(ordch1, ordch2, True) or
-                runicode._invalid_byte_3_of_3(ordch3)):
-                raise UnicodeDecodeError('utf8', s, pos, pos + 1,
-                                         'whatever')
-            # 1110xxxx 10yyyyyy 10zzzzzz -> 00000000 xxxxyyyy yyzzzzzz
-            result.append(unichr(((ordch1 & 0x0F) << 12) +     # 0b00001111
-                                 ((ordch2 & 0x3F) << 6) +      # 0b00111111
-                                 (ordch3 & 0x3F)))             # 0b00111111
-            pos += 3
-
-        elif n == 4:
-            ordch2 = ord(s[pos+1])
-            ordch3 = ord(s[pos+2])
-            ordch4 = ord(s[pos+3])
-            if (runicode._invalid_byte_2_of_4(ordch1, ordch2) or
-                runicode._invalid_byte_3_of_4(ordch3) or
-                runicode._invalid_byte_4_of_4(ordch4)):
-
-                raise UnicodeDecodeError('utf8', s, pos, pos + 1,
-                                         'whatever')
-            # 11110www 10xxxxxx 10yyyyyy 10zzzzzz -> 000wwwxx xxxxyyyy yyzzzzzz
-            c = (((ordch1 & 0x07) << 18) +      # 0b00000111
-                 ((ordch2 & 0x3F) << 12) +      # 0b00111111
-                 ((ordch3 & 0x3F) << 6) +       # 0b00111111
-                 (ordch4 & 0x3F))               # 0b00111111
-            if c <= runicode.MAXUNICODE:
-                result.append(runicode.UNICHR(c))
-            else:
-                # compute and append the two surrogates:
-                # translate from 10000..10FFFF to 0..FFFF
-                c -= 0x10000
-                # high surrogate = top 10 bits added to D800
-                result.append(unichr(0xD800 + (c >> 10)))
-                # low surrogate = bottom 10 bits added to DC00
-                result.append(unichr(0xDC00 + (c & 0x03FF)))
-            pos += 4
-
-    return result.build()
+from rpython.tool.sourcetools import func_with_new_name
+from rpython.rlib.rstring import UnicodeBuilder
 
 
 class AbstractStringRepr(Repr):
 
+    def __init__(self, *args):
+        Repr.__init__(self, *args)
+        self.rstr_decode_utf_8 = None
+
+    def ensure_ll_decode_utf8(self):
+        from rpython.rlib.runicode import str_decode_utf_8_impl
+        self.rstr_decode_utf_8 = func_with_new_name(str_decode_utf_8_impl,
+                                                    'rstr_decode_utf_8_impl')
+
     @jit.elidable
     def ll_decode_utf8(self, llvalue):
         from rpython.rtyper.annlowlevel import hlstr
         value = hlstr(llvalue)
         assert value is not None
-        # NB. keep the arguments in sync with annotator/unaryop.py
-        u = str_decode_utf8(value)
-        # XXX maybe the whole ''.decode('utf-8') should be not RPython.
-        return self.ll.llunicode(u)
+        result = UnicodeBuilder(len(value))
+        self.rstr_decode_utf_8(
+            value, len(value), 'strict', final=True,
+            errorhandler=self.ll_raise_unicode_exception_decode,
+            allow_surrogates=False, result=result)
+        return self.ll.llunicode(result.build())
+
+    @staticmethod
+    def ll_raise_unicode_exception_decode(errors, encoding, msg, s,
+                                       startingpos, endingpos):
+        raise UnicodeDecodeError(encoding, s, startingpos, endingpos, msg)
 
     def _str_reprs(self, hop):
         return hop.args_r[0].repr, hop.args_r[1].repr
@@ -410,7 +346,8 @@ class AbstractStringRepr(Repr):
             return hop.gendirectcall(self.ll.ll_str2unicode, v_self)
         elif encoding == 'latin-1':
             return hop.gendirectcall(self.ll_decode_latin1, v_self)
-        elif encoding == 'utf-8' or encoding == 'utf8':
+        elif encoding == 'utf-8':
+            self.ensure_ll_decode_utf8()
             return hop.gendirectcall(self.ll_decode_utf8, v_self)
         else:
             raise TyperError("encoding %s not implemented" % (encoding, ))
@@ -449,6 +386,15 @@ class AbstractStringRepr(Repr):
 
 class AbstractUnicodeRepr(AbstractStringRepr):
 
+    def __init__(self, *args):
+        AbstractStringRepr.__init__(self, *args)
+        self.runicode_encode_utf_8 = None
+
+    def ensure_ll_encode_utf8(self):
+        from rpython.rlib.runicode import unicode_encode_utf_8_impl
+        self.runicode_encode_utf_8 = func_with_new_name(
+            unicode_encode_utf_8_impl, 'runicode_encode_utf_8')
+
     def rtype_method_upper(self, hop):
         raise TyperError("Cannot do toupper on unicode string")
 
@@ -458,15 +404,19 @@ class AbstractUnicodeRepr(AbstractStringRepr):
     @jit.elidable
     def ll_encode_utf8(self, ll_s):
         from rpython.rtyper.annlowlevel import hlunicode
-        from rpython.rlib import runicode
         s = hlunicode(ll_s)
         assert s is not None
-        errorhandler = runicode.default_unicode_error_encode
-        # NB. keep the arguments in sync with annotator/unaryop.py
-        bytes = runicode.unicode_encode_utf_8_elidable(
-            s, len(s), 'strict', errorhandler, True)
+        bytes = self.runicode_encode_utf_8(
+            s, len(s), 'strict',
+            errorhandler=self.ll_raise_unicode_exception_encode,
+            allow_surrogates=False)
         return self.ll.llstr(bytes)
 
+    @staticmethod
+    def ll_raise_unicode_exception_encode(errors, encoding, msg, u,
+                                          startingpos, endingpos):
+        raise UnicodeEncodeError(encoding, u, startingpos, endingpos, msg)
+
     def rtype_method_encode(self, hop):
         if not hop.args_s[1].is_constant():
             raise TyperError("encoding must be constant")
@@ -481,7 +431,8 @@ class AbstractUnicodeRepr(AbstractStringRepr):
             return hop.gendirectcall(self.ll_str, v_self)
         elif encoding == "latin-1":
             return hop.gendirectcall(self.ll_encode_latin1, v_self)
-        elif encoding == 'utf-8' or encoding == 'utf8':
+        elif encoding == 'utf-8':
+            self.ensure_ll_encode_utf8()
             return hop.gendirectcall(self.ll_encode_utf8, v_self)
         else:
             raise TyperError("encoding %s not implemented" % (encoding, ))
@@ -766,7 +717,9 @@ class __extend__(AbstractUniCharRepr):
         return hop.genop('cast_unichar_to_int', vlist, resulttype=Signed)
 
 
-class __extend__(pairtype(AbstractUniCharRepr, AbstractUniCharRepr)):
+class __extend__(pairtype(AbstractUniCharRepr, AbstractUniCharRepr),
+                 pairtype(AbstractCharRepr, AbstractUniCharRepr),
+                 pairtype(AbstractUniCharRepr, AbstractCharRepr)):
     def rtype_eq(_, hop): return _rtype_unchr_compare_template(hop, 'eq')
     def rtype_ne(_, hop): return _rtype_unchr_compare_template(hop, 'ne')
     def rtype_lt(_, hop): return _rtype_unchr_compare_template_ord(hop, 'lt')
@@ -786,7 +739,12 @@ def _rtype_unchr_compare_template_ord(hop, func):
     vlist = hop.inputargs(*hop.args_r)
     vlist2 = []
     for v in vlist:
-        v = hop.genop('cast_unichar_to_int', [v], resulttype=lltype.Signed)
+        if v.concretetype == lltype.Char:
+            v = hop.genop('cast_char_to_int', [v], resulttype=lltype.Signed)
+        elif v.concretetype == lltype.UniChar:
+            v = hop.genop('cast_unichar_to_int', [v], resulttype=lltype.Signed)
+        else:
+            assert 0, v.concretetype
         vlist2.append(v)
     return hop.genop('int_' + func, vlist2, resulttype=Bool)
 
@@ -811,6 +769,11 @@ class __extend__(pairtype(AbstractStringRepr, AbstractCharRepr)):
             return llops.gendirectcall(r_from.ll.ll_stritem_nonneg, v, c_zero)
         return NotImplemented
 
+class __extend__(pairtype(AbstractCharRepr, AbstractUniCharRepr)):
+    def convert_from_to((r_from, r_to), v, llops):
+        v2 = llops.genop('cast_char_to_int', [v], resulttype=Signed)
+        return llops.genop('cast_int_to_unichar', [v2], resulttype=UniChar)
+
 # ____________________________________________________________
 #
 #  Iteration.
