@@ -2,7 +2,7 @@ from __future__ import with_statement
 import py
 import sys
 from rpython.rtyper.lltypesystem.lltype import typeOf, Void, malloc, free
-from rpython.rtyper.llinterp import LLInterpreter, LLException
+from rpython.rtyper.llinterp import LLInterpreter, LLException, log
 from rpython.rtyper.rmodel import inputconst
 from rpython.rtyper.annlowlevel import hlstr, llhelper
 from rpython.rtyper.exceptiondata import UnknownException
@@ -16,13 +16,10 @@ from rpython.conftest import option
 from rpython.rtyper.rtyper import llinterp_backend
 
 # switch on logging of interp to show more info on failing tests
-
 def setup_module(mod):
-    mod.logstate = py.log._getstate()
-    py.log.setconsumer("llinterp", py.log.STDOUT)
-
+    log.output_disabled = False
 def teardown_module(mod):
-    py.log._setstate(mod.logstate)
+    log.output_disabled = True
 
 
 def gengraph(func, argtypes=[], viewbefore='auto', policy=None,
