@@ -459,8 +459,6 @@ class AbstractTestRstr(BaseRtypingTest):
             return const('    ').strip(' ')
         def left2():
             return const('a  ').strip(' ')
-        def leftall():
-            return const('!!').lstrip(const('!'))
         res = self.interpret(both, [])
         assert self.ll_to_string(res) == const('ab')
         res = self.interpret(left, [])
@@ -471,8 +469,6 @@ class AbstractTestRstr(BaseRtypingTest):
         assert self.ll_to_string(res) == const('')
         res = self.interpret(left2, [])
         assert self.ll_to_string(res) == const('a')
-        res = self.interpret(leftall, [])
-        assert self.ll_to_string(res) == const('')
 
     def test_strip_multiple_chars(self):
         const = self.const
@@ -486,8 +482,6 @@ class AbstractTestRstr(BaseRtypingTest):
             return const(' \t\t   ').strip('\t ')
         def left2():
             return const('a  ').strip(' \t')
-        def leftall():
-            return const('!ab!').lstrip(const('!ab'))
         res = self.interpret(both, [])
         assert self.ll_to_string(res) == const('b')
         res = self.interpret(left, [])
@@ -498,8 +492,6 @@ class AbstractTestRstr(BaseRtypingTest):
         assert self.ll_to_string(res) == const('')
         res = self.interpret(left2, [])
         assert self.ll_to_string(res) == const('a')
-        res = self.interpret(leftall, [])
-        assert self.ll_to_string(res) == const('')
 
     def test_upper(self):
         const = self.const
@@ -1059,7 +1051,7 @@ class AbstractTestRstr(BaseRtypingTest):
                 l = [const('a'), const('b')]
             else:
                 l = [const('a')]
-            l += y     # list += string
+            l += y
             return const('').join(l)
 
         assert self.ll_to_string(self.interpret(f, [1,
