@@ -8,7 +8,7 @@ from rpython.translator.c.gcc.instruction import InsnFunctionStart, InsnStop
 from rpython.translator.c.gcc.instruction import InsnSetLocal, InsnCopyLocal
 from rpython.translator.c.gcc.instruction import InsnPrologue, InsnEpilogue
 from rpython.translator.c.gcc.instruction import InsnGCROOT, InsnCondJump
-from rpython.translator.c.gcc.instruction import InsnStackAdjust
+from rpython.translator.c.gcc.instruction import InsnStackAdjust, InsnPushed
 from rpython.translator.c.gcc.instruction import InsnCannotFollowEsp
 from rpython.translator.c.gcc.instruction import LocalVar, somenewvalue
 from rpython.translator.c.gcc.instruction import frameloc_esp, frameloc_ebp
@@ -31,7 +31,7 @@ class FunctionGcRootTracker(object):
         cls.r_binaryinsn    = re.compile(r"\t[a-z]\w*\s+(?P<source>"+cls.OPERAND+"),\s*(?P<target>"+cls.OPERAND+")\s*$")
 
         cls.r_jump          = re.compile(r"\tj\w+\s+"+cls.LABEL+"\s*" + cls.COMMENT + "$")
-        cls.r_jmp_switch    = re.compile(r"\tjmp\t[*]"+cls.LABEL+"[(]")
+        cls.r_jmp_switch    = re.compile(r"\tjmp\t[*]")
         cls.r_jmp_source    = re.compile(r"\d*[(](%[\w]+)[,)]")
 
     def __init__(self, funcname, lines, filetag=0):
@@ -149,6 +149,14 @@ class FunctionGcRootTracker(object):
             else:
                 del self.insns[i]
 
+        # the remaining instructions must have their 'previous_insns' list
+        # trimmed of dead previous instructions
+        all_remaining_insns = set(self.insns)
+        assert self.insns[0].previous_insns == ()
+        for insn in self.insns[1:]:
+            insn.previous_insns = [previnsn for previnsn in insn.previous_insns
+                                            if previnsn in all_remaining_insns]
+
     def find_noncollecting_calls(self):
         cannot_collect = {}
         for line in self.lines:
@@ -286,6 +294,18 @@ class FunctionGcRootTracker(object):
                     else:
                         insn1.framesize = size_at_insn1
 
+        # trim: instructions with no framesize are removed from self.insns,
+        # and from the 'previous_insns' lists
+        if 0:    # <- XXX disabled because it seems bogus, investigate more
+          assert hasattr(self.insns[0], 'framesize')
+          old = self.insns[1:]
+          del self.insns[1:]
+          for insn in old:
+            if hasattr(insn, 'framesize'):
+                self.insns.append(insn)
+                insn.previous_insns = [previnsn for previnsn in insn.previous_insns
+                                                if hasattr(previnsn, 'framesize')]
+
     def fixlocalvars(self):
         def fixvar(localvar):
             if localvar is None:
@@ -403,6 +423,7 @@ class FunctionGcRootTracker(object):
                 # the original value for gcmaptable.s.  That's a hack.
                 self.lines.insert(call.lineno+1, '%s=.+%d\n' % (label,
                                                                 self.OFFSET_LABELS))
+                self.lines.insert(call.lineno+1, '\t.hidden\t%s\n' % (label,))
                 self.lines.insert(call.lineno+1, '\t.globl\t%s\n' % (label,))
         call.global_label = label
 
@@ -502,7 +523,13 @@ class FunctionGcRootTracker(object):
         # raw data, not GC pointers
         'movnt', 'mfence', 'lfence', 'sfence',
         # bit manipulations
-        'bextr',
+        'andn', 'bextr', 'blsi', 'blsmask', 'blsr', 'tzcnt', 'lzcnt',
+        # uh, this can occur with a 'call' on the following line...
+        'rex64',
+        # movbe, converts from big-endian, so most probably not GC pointers
+        'movbe',
+        # xchgb, byte-sized, so not GC pointers
+        'xchgb',
     ])
 
     # a partial list is hopefully good enough for now; it's all to support
@@ -566,13 +593,6 @@ class FunctionGcRootTracker(object):
         else:
             return []
 
-    # The various cmov* operations
-    for name in '''
-        e ne g ge l le a ae b be p np s ns o no
-        '''.split():
-        locals()['visit_cmov' + name] = binary_insn
-        locals()['visit_cmov' + name + 'l'] = binary_insn
-
     def _visit_and(self, line):
         match = self.r_binaryinsn.match(line)
         target = match.group("target")
@@ -645,14 +665,22 @@ class FunctionGcRootTracker(object):
         match = self.r_unaryinsn.match(line)
         source = match.group(1)
         return self.insns_for_copy(source, self.TOP_OF_STACK_MINUS_WORD) + \
-               [InsnStackAdjust(-self.WORD)]
+               [InsnPushed(-self.WORD)]
 
     def _visit_pop(self, target):
         return [InsnStackAdjust(+self.WORD)] + \
                self.insns_for_copy(self.TOP_OF_STACK_MINUS_WORD, target)
 
     def _visit_prologue(self):
-        # for the prologue of functions that use %ebp as frame pointer
+        # For the prologue of functions that use %ebp as frame pointer.
+        # First, find the latest InsnStackAdjust; if it's not a PUSH,
+        # then consider that this 'mov %rsp, %rbp' is actually unrelated
+        i = -1
+        while not isinstance(self.insns[i], InsnStackAdjust):
+            i -= 1
+        if not isinstance(self.insns[i], InsnPushed):
+            return []
+        #
         self.uses_frame_pointer = True
         self.r_localvar = self.r_localvarfp
         return [InsnPrologue(self.WORD)]
@@ -674,14 +702,29 @@ class FunctionGcRootTracker(object):
             return self.visit_ret(line)
         return []
 
+    def visit_ud2(self, line):
+        return InsnStop("ud2")    # unreachable instruction
+
     def visit_jmp(self, line):
         tablelabels = []
         match = self.r_jmp_switch.match(line)
         if match:
-            # this is a jmp *Label(%index), used for table-based switches.
-            # Assume that the table is just a list of lines looking like
-            # .long LABEL or .long 0, ending in a .text or .section .text.hot.
-            tablelabels.append(match.group(1))
+            # this is a jmp *Label(%index) or jmp *%addr, used for
+            # table-based switches.  Assume that the table is coming
+            # after a .section .rodata and a label, and is a list of
+            # lines looking like .long LABEL or .long 0 or .long L2-L1,
+            # ending in a .text or .section .text.hot.
+            lineno = self.currentlineno + 1
+            if '.section' not in self.lines[lineno]:
+                pass  # bah, probably a tail-optimized indirect call...
+            else:
+                assert '.rodata' in self.lines[lineno]
+                lineno += 1
+                while '.align' in self.lines[lineno]:
+                    lineno += 1
+                match = self.r_label.match(self.lines[lineno])
+                assert match, repr(self.lines[lineno])
+                tablelabels.append(match.group(1))
         elif self.r_unaryinsn_star.match(line):
             # maybe a jmp similar to the above, but stored in a
             # registry:
@@ -783,22 +826,18 @@ class FunctionGcRootTracker(object):
         return prefix + [InsnCondJump(label)] + postfix
 
     visit_jmpl = visit_jmp
-    visit_jg = conditional_jump
-    visit_jge = conditional_jump
-    visit_jl = conditional_jump
-    visit_jle = conditional_jump
-    visit_ja = conditional_jump
-    visit_jae = conditional_jump
-    visit_jb = conditional_jump
-    visit_jbe = conditional_jump
-    visit_jp = conditional_jump
-    visit_jnp = conditional_jump
-    visit_js = conditional_jump
-    visit_jns = conditional_jump
-    visit_jo = conditional_jump
-    visit_jno = conditional_jump
-    visit_jc = conditional_jump
-    visit_jnc = conditional_jump
+
+    # The various conditional jumps and cmov* operations
+    for name in '''
+        e g ge l le a ae b be p s o c
+        '''.split():
+        # NB. visit_je() and visit_jne() are overridden below
+        locals()['visit_j' + name] = conditional_jump
+        locals()['visit_jn' + name] = conditional_jump
+        locals()['visit_cmov' + name] = binary_insn
+        locals()['visit_cmov' + name + 'l'] = binary_insn
+        locals()['visit_cmovn' + name] = binary_insn
+        locals()['visit_cmovn' + name + 'l'] = binary_insn
 
     def visit_je(self, line):
         return self.conditional_jump(line, je=True)
@@ -815,13 +854,17 @@ class FunctionGcRootTracker(object):
         return []
 
     def _visit_xchg(self, line):
-        # only support the format used in VALGRIND_DISCARD_TRANSLATIONS
+        # support the format used in VALGRIND_DISCARD_TRANSLATIONS
         # which is to use a marker no-op "xchgl %ebx, %ebx"
         match = self.r_binaryinsn.match(line)
         source = match.group("source")
         target = match.group("target")
         if source == target:
             return []
+        # ignore the 'rpy_fastgil' atomic exchange, or any locked
+        # atomic exchange at all (involving memory)
+        if not source.startswith('%'):
+            return []
         raise UnrecognizedOperation(line)
 
     def visit_call(self, line):
@@ -1031,6 +1074,7 @@ class FunctionGcRootTracker64(FunctionGcRootTracker):
     visit_leaq = FunctionGcRootTracker._visit_lea
 
     visit_xorq = FunctionGcRootTracker.binary_insn
+    visit_xchgl = FunctionGcRootTracker._visit_xchg
     visit_xchgq = FunctionGcRootTracker._visit_xchg
     visit_testq = FunctionGcRootTracker._visit_test
 
@@ -1079,7 +1123,7 @@ class ElfFunctionGcRootTracker32(FunctionGcRootTracker32):
     REG2LOC = dict((_reg, LOC_REG | ((_i+1)<<2))
                    for _i, _reg in enumerate(CALLEE_SAVE_REGISTERS))
     OPERAND = r'(?:[-\w$%+.:@"]+(?:[(][\w%,]+[)])?|[(][\w%,]+[)])'
-    LABEL   = r'([a-zA-Z_$.][a-zA-Z0-9_$@.]*)'
+    LABEL   = r'([a-zA-Z_$.][a-zA-Z0-9_$.]*)(?:@[@a-zA-Z0-9_$.]*)?'
     OFFSET_LABELS   = 2**30
     TOP_OF_STACK_MINUS_WORD = '-4(%esp)'
 
@@ -1141,7 +1185,7 @@ class ElfFunctionGcRootTracker64(FunctionGcRootTracker64):
     REG2LOC = dict((_reg, LOC_REG | ((_i+1)<<2))
                    for _i, _reg in enumerate(CALLEE_SAVE_REGISTERS))
     OPERAND = r'(?:[-\w$%+.:@"]+(?:[(][\w%,]+[)])?|[(][\w%,]+[)])'
-    LABEL   = r'([a-zA-Z_$.][a-zA-Z0-9_$@.]*)'
+    LABEL   = r'([a-zA-Z_$.][a-zA-Z0-9_$.]*)(?:@[@a-zA-Z0-9_$.]*)?'
     OFFSET_LABELS   = 2**30
     TOP_OF_STACK_MINUS_WORD = '-8(%rsp)'
 
@@ -1463,7 +1507,8 @@ class ElfAssemblerParser(AssemblerParser):
         functionlines = []
         in_function = False
         for line in iterlines:
-            if self.FunctionGcRootTracker.r_functionstart.match(line):
+            match = self.FunctionGcRootTracker.r_functionstart.match(line)
+            if match and not match.group(1).startswith('__x86.get_pc_thunk.'):
                 assert not in_function, (
                     "missed the end of the previous function")
                 yield False, functionlines
@@ -1476,6 +1521,9 @@ class ElfAssemblerParser(AssemblerParser):
                 yield True, functionlines
                 in_function = False
                 functionlines = []
+        if in_function and ".get_pc_thunk.bx" in functionlines[0]:
+            in_function = False     # xxx? ignore this rare unclosed stub at
+                                    # the end of the file
         assert not in_function, (
             "missed the end of the previous function")
         yield False, functionlines
