@@ -27,7 +27,7 @@ int pypy_main_function(int argc, char *argv[]) __attribute__((__noinline__));
 #  include "forwarddecl.h"
 # endif
 
-#if defined(MS_WINDOWS) && defined(RPY_SANDBOXED)
+#if defined(MS_WINDOWS)
 #  include <stdio.h>
 #  include <fcntl.h>
 #  include <io.h>
@@ -62,11 +62,11 @@ int pypy_main_function(int argc, char *argv[])
 {
     char *errmsg;
     int i, exitcode;
-    RPyListOfString *list;
 
-#if defined(MS_WINDOWS) && defined(RPY_SANDBOXED)
+#if defined(MS_WINDOWS)
     _setmode(0, _O_BINARY);
     _setmode(1, _O_BINARY);
+    _setmode(2, _O_BINARY);
 #endif
 
 #ifdef RPY_WITH_GIL
@@ -94,15 +94,7 @@ int pypy_main_function(int argc, char *argv[])
 
     RPython_StartupCode();
 
-    list = _RPyListOfString_New(argc);
-    if (RPyExceptionOccurred()) goto memory_out;
-    for (i=0; i<argc; i++) {
-        RPyString *s = RPyString_FromString(argv[i]);
-        if (RPyExceptionOccurred()) goto memory_out;
-        _RPyListOfString_SetItem(list, i, s);
-    }
-
-    exitcode = STANDALONE_ENTRY_POINT(list);
+    exitcode = STANDALONE_ENTRY_POINT(argc, argv);
 
     pypy_debug_alloc_results();
 
