@@ -1,6 +1,5 @@
 import py
 import sys, os, re
-import textwrap
 
 from rpython.config.translationoption import get_combined_translation_config
 from rpython.config.translationoption import SUPPORT__THREAD
@@ -40,8 +39,7 @@ class StandaloneTests(object):
     config = None
 
     def compile(self, entry_point, debug=True, shared=False,
-                stackcheck=False, entrypoints=None, local_icon=None,
-                exe_name=None):
+                stackcheck=False, entrypoints=None, local_icon=None):
         t = TranslationContext(self.config)
         ann = t.buildannotator()
         ann.build_types(entry_point, [s_list_of_strings])
@@ -68,8 +66,7 @@ class StandaloneTests(object):
             kwds = {}
         cbuilder = CStandaloneBuilder(t, entry_point, t.config, **kwds)
         if debug:
-            cbuilder.generate_source(defines=cbuilder.DEBUG_DEFINES,
-                                     exe_name=exe_name)
+            cbuilder.generate_source(defines=cbuilder.DEBUG_DEFINES)
         else:
             cbuilder.generate_source()
         cbuilder.compile()
@@ -184,10 +181,7 @@ class TestStandalone(StandaloneTests):
         f.close()
 
         import struct
-        fmt = "LLL"
-        if sys.platform == 'win32' and sys.maxint > 2**31:
-            fmt = "QQQ"
-        counters = struct.unpack(fmt, counters_data)
+        counters = struct.unpack("LLL", counters_data)
 
         assert counters == (0,3,2)
 
@@ -247,22 +241,20 @@ class TestStandalone(StandaloneTests):
             os.write(1, str(tot))
             return 0
         from rpython.translator.interactive import Translation
-        t = Translation(entry_point, backend='c', profopt=True, profoptargs="10", shared=True)
+        # XXX this is mostly a "does not crash option"
+        t = Translation(entry_point, backend='c', profopt="100")
+        # no counters
         t.backendopt()
         exe = t.compile()
-        assert (os.path.isfile("%s" % exe))
-
-        t = Translation(entry_point, backend='c', profopt=True, profoptargs="10", shared=False)
+        out = py.process.cmdexec("%s 500" % exe)
+        assert int(out) == 500*501/2
+        t = Translation(entry_point, backend='c', profopt="100",
+                        noprofopt=True)
+        # no counters
         t.backendopt()
         exe = t.compile()
-        assert (os.path.isfile("%s" % exe))
-
-        import rpython.translator.goal.targetrpystonedalone as rpy
-        t = Translation(rpy.entry_point, backend='c', profopt=True, profoptargs='1000', shared=False)
-        t.backendopt()
-        exe = t.compile()
-        assert (os.path.isfile("%s" % exe))
-
+        out = py.process.cmdexec("%s 500" % exe)
+        assert int(out) == 500*501/2
 
     if hasattr(os, 'setpgrp'):
         def test_os_setpgrp(self):
@@ -287,12 +279,13 @@ class TestStandalone(StandaloneTests):
             return 0
         from rpython.translator.interactive import Translation
         # XXX this is mostly a "does not crash option"
-        t = Translation(entry_point, backend='c', profopt=True, profoptargs='10', shared=True)
+        t = Translation(entry_point, backend='c', profopt="")
         # no counters
         t.backendopt()
         exe = t.compile()
         #py.process.cmdexec(exe)
-        t = Translation(entry_point, backend='c', profopt=True, profoptargs='10', shared=True)
+        t = Translation(entry_point, backend='c', profopt="",
+                        noprofopt=True)
         # no counters
         t.backendopt()
         exe = t.compile()
@@ -525,50 +518,6 @@ class TestStandalone(StandaloneTests):
         assert not err
         assert path.check(file=0)
 
-    def test_debug_start_stop_timestamp(self):
-        from rpython.rlib.rtimer import read_timestamp
-        def entry_point(argv):
-            timestamp = bool(int(argv[1]))
-            ts1 = debug_start("foo", timestamp=timestamp)
-            ts2 = read_timestamp()
-            ts3 = debug_stop("foo", timestamp=timestamp)
-            print ts1
-            print ts2
-            print ts3
-            return 0
-        t, cbuilder = self.compile(entry_point)
-
-        def parse_out(out):
-            lines = out.strip().splitlines()
-            ts1, ts2, ts3 = lines
-            return int(ts1), int(ts2), int(ts3)
-
-        # check with PYPYLOG :-
-        out, err = cbuilder.cmdexec("1", err=True, env={'PYPYLOG': ':-'})
-        ts1, ts2, ts3 = parse_out(out)
-        assert ts3 > ts2 > ts1
-        expected = ('[%x] {foo\n' % ts1 +
-                    '[%x] foo}\n' % ts3)
-        assert err == expected
-
-        # check with PYPYLOG profiling only
-        out, err = cbuilder.cmdexec("1", err=True, env={'PYPYLOG': '-'})
-        ts1, ts2, ts3 = parse_out(out)
-        assert ts3 > ts2 > ts1
-        expected = ('[%x] {foo\n' % ts1 +
-                    '[%x] foo}\n' % ts3)
-        assert err == expected
-
-        # check with PYPYLOG undefined
-        out, err = cbuilder.cmdexec("1", err=True, env={})
-        ts1, ts2, ts3 = parse_out(out)
-        assert ts3 > ts2 > ts1
-
-        # check with PYPYLOG undefined and timestamp=False
-        out, err = cbuilder.cmdexec("0", err=True, env={})
-        ts1, ts2, ts3 = parse_out(out)
-        assert ts1 == ts3 == 42;
-
     def test_debug_print_start_stop_nonconst(self):
         def entry_point(argv):
             debug_start(argv[1])
@@ -904,45 +853,26 @@ class TestStandalone(StandaloneTests):
         # The traceback stops at f() because it's the first function that
         # captures the AssertionError, which makes the program abort.
 
-    def test_shared1(self, monkeypatch):
+    def test_shared(self, monkeypatch):
         def f(argv):
             print len(argv)
         def entry_point(argv):
             f(argv)
             return 0
-        # Make sure the '.' in exe_name is propagated
-        t, cbuilder = self.compile(entry_point, shared=True, exe_name='pypy3.9')
-        assert 'pypy3.9' in str(cbuilder.executable_name)
+        t, cbuilder = self.compile(entry_point, shared=True)
         assert cbuilder.shared_library_name is not None
         assert cbuilder.shared_library_name != cbuilder.executable_name
-        assert 'exe' not in str(cbuilder.shared_library_name)
-        # it must be something with a '.basename' to make the driver.py happy
-        assert not isinstance(cbuilder.shared_library_name, str)
-        assert 'pypy3.9' in str(cbuilder.shared_library_name)
         #Do not set LD_LIBRARY_PATH, make sure $ORIGIN flag is working
         out, err = cbuilder.cmdexec("a b")
         assert out == "3"
         if sys.platform == 'win32':
-            assert 'pypy3.9.exe' not in str(cbuilder.shared_library_name)
-            assert 'pypy3.9.exe' in str(cbuilder.executable_name)
-            # Make sure we have a pypy3.9w.exe
+            # Make sure we have a test_1w.exe
             # Since stdout, stderr are piped, we will get output
             exe = cbuilder.executable_name
             wexe = exe.new(purebasename=exe.purebasename + 'w')
             out, err = cbuilder.cmdexec("a b", exe = wexe)
             assert out == "3"
 
-    def test_shared2(self, monkeypatch):
-        def f(argv):
-            print len(argv)
-        def entry_point(argv):
-            f(argv)
-            return 0
-        # Make sure the '.exe' in exe_name is not propagated
-        t, cbuilder = self.compile(entry_point, shared=True, exe_name='pypy')
-        assert 'pypy' == cbuilder.executable_name.purebasename
-        assert 'libpypy' == cbuilder.shared_library_name.purebasename
-
     def test_gcc_options(self):
         # check that the env var CC is correctly interpreted, even if
         # it contains the compiler name followed by some options.
@@ -1132,93 +1062,22 @@ class TestStandalone(StandaloneTests):
         out = cbuilder.cmdexec('')
         assert out.strip() == expected
 
-    def test_call_at_startup(self):
-        from rpython.rtyper.extregistry import ExtRegistryEntry
 
-        class State:
-            seen = 0
-        state = State()
-        def startup():
-            state.seen += 1
-        def enablestartup():
-            "NOT_RPYTHON"
-        def entry_point(argv):
-            state.seen += 100
-            assert state.seen == 101
-            print 'ok'
-            enablestartup()
-            return 0
+class TestMaemo(TestStandalone):
+    def setup_class(cls):
+        py.test.skip("TestMaemo: tests skipped for now")
+        from rpython.translator.platform.maemo import check_scratchbox
+        check_scratchbox()
+        config = get_combined_translation_config(translating=True)
+        config.translation.platform = 'maemo'
+        cls.config = config
 
-        class Entry(ExtRegistryEntry):
-            _about_ = enablestartup
+    def test_profopt(self):
+        py.test.skip("Unsupported")
 
-            def compute_result_annotation(self):
-                bk = self.bookkeeper
-                s_callable = bk.immutablevalue(startup)
-                key = (enablestartup,)
-                bk.emulate_pbc_call(key, s_callable, [])
+    def test_prof_inline(self):
+        py.test.skip("Unsupported")
 
-            def specialize_call(self, hop):
-                hop.exception_cannot_occur()
-                bk = hop.rtyper.annotator.bookkeeper
-                s_callable = bk.immutablevalue(startup)
-                r_callable = hop.rtyper.getrepr(s_callable)
-                ll_init = r_callable.get_unique_llfn().value
-                bk.annotator.translator._call_at_startup.append(ll_init)
-
-        t, cbuilder = self.compile(entry_point)
-        out = cbuilder.cmdexec('')
-        assert out.strip() == 'ok'
-
-    def test_gcc_precompiled_header(self):
-        if sys.platform == 'win32':
-            py.test.skip("no win")
-        def entry_point(argv):
-            os.write(1, "hello world\n")
-            argv = argv[1:]
-            os.write(1, "argument count: " + str(len(argv)) + "\n")
-            for s in argv:
-                os.write(1, "   '" + str(s) + "'\n")
-            return 0
-
-        t, cbuilder = self.compile(entry_point)
-        if "gcc" not in t.platform.cc:
-            py.test.skip("gcc only")
-        data = cbuilder.cmdexec('hi there')
-        assert data.startswith('''hello world\nargument count: 2\n   'hi'\n   'there'\n''')
-
-        # check that the precompiled header was generated
-        assert cbuilder.targetdir.join("singleheader.h.gch").check()
-
-    def test_field_access_stats(self):
-        t = TranslationContext(self.config)
-        t.config.translation.countfieldaccess = True
-        class A:
-            pass
-        def entry_point(argv):
-            l = []
-            for i in range(100):
-                a = A()
-                a.x = i
-                l.append(a)
-            res = 0
-            for a in l + l:
-                res += a.x
-            print(res)
-            return 0
-        entry_point([])
-
-        t.buildannotator().build_types(entry_point, [s_list_of_strings])
-        t.buildrtyper().specialize()
-        #
-        cbuilder = CStandaloneBuilder(t, entry_point, t.config)
-        cbuilder.generate_source(defines=cbuilder.DEBUG_DEFINES)
-        cbuilder.compile()
-        fn = udir.join("fieldstats")
-        data = cbuilder.cmdexec(env={"PYPYLOG": "stats-fields:%s" % fn})
-        stats = fn.read()
-        assert "write a_inst_x 100" in stats
-        assert "read a_inst_x 200" in stats
 
 class TestThread(object):
     gcrootfinder = 'shadowstack'
@@ -1226,7 +1085,7 @@ class TestThread(object):
 
     def compile(self, entry_point, no__thread=True):
         t = TranslationContext(self.config)
-        t.config.translation.gc = "incminimark"
+        t.config.translation.gc = "semispace"
         t.config.translation.gcrootfinder = self.gcrootfinder
         t.config.translation.thread = True
         t.config.translation.no__thread = no__thread
