@@ -10,60 +10,45 @@ import rpython
 rpydir = str(py.path.local(rpython.__file__).join('..'))
 
 def _get_compiler_type(cc, x64_flag):
+    import subprocess
     if not cc:
         cc = os.environ.get('CC','')
     if not cc:
         return MsvcPlatform(x64=x64_flag)
     elif cc.startswith('mingw') or cc == 'gcc':
         return MingwPlatform(cc)
-    return MsvcPlatform(cc=cc, x64=x64_flag)
-
-def _get_vcver0():
-    # try to get the compiler which served to compile python
-    msc_pos = sys.version.find('MSC v.')
-    if msc_pos != -1:
-        msc_ver = int(sys.version[msc_pos+6:msc_pos+10])
-        # 1500 -> 90, 1900 -> 140
-        vsver = (msc_ver / 10) - 60
-        return vsver
-    return None
+    else:
+        return MsvcPlatform(cc=cc, x64=x64_flag)
+    try:
+        subprocess.check_output([cc, '--version'])
+    except:
+        raise ValueError("Could not find compiler specified by cc option '%s',"
+                         " it must be a valid exe file on your path" % cc)
+    return MingwPlatform(cc)
 
 def Windows(cc=None):
     return _get_compiler_type(cc, False)
 
-def Windows_x64(cc=None, ver0=None):
-    #raise Exception("Win64 is not supported.  You must either build for Win32"
-    #                " or contribute the missing support in PyPy.")
+def Windows_x64(cc=None):
+    raise Exception("Win64 is not supported.  You must either build for Win32"
+                    " or contribute the missing support in PyPy.")
     return _get_compiler_type(cc, True)
 
-def _find_vcvarsall(version, x64flag):
-    import rpython.tool.setuptools_msvc as msvc
-    if x64flag:
-        arch = 'x64'
-    else:
-        arch = 'x86'
-    if version >= 140:
-        return msvc.msvc14_get_vc_env(arch)
-    else:
-        return msvc.msvc9_query_vcvarsall(version / 10.0, arch)
-
 def _get_msvc_env(vsver, x64flag):
-    vcdict = None
-    toolsdir = None
+    vcvars = None
     try:
         toolsdir = os.environ['VS%sCOMNTOOLS' % vsver]
     except KeyError:
-        # use setuptools from python3 to find tools
+        # try to import from the "Microsoft Visual C++ Compiler for Python 2.7"
         try:
-            vcdict = _find_vcvarsall(vsver, x64flag)
-        except ImportError as e:
-            if 'setuptools' in str(e):
-                log.error('is setuptools installed (perhaps try %s -mensurepip)?' % sys.executable)
-            log.error('looking for compiler %s raised exception "%s' % (vsver, str(e)))
-        except Exception as e:
-            log.error('looking for compiler %s raised exception "%s' % (vsver, str(e)))
+            import setuptools
+        except ImportError:
             return None
-    else:
+        # XXX works for 90 but is it generalizable?
+        toolsdir = ''
+        vcvars = setuptools.msvc.msvc9_find_vcvarsall(vsver/10)
+
+    if not vcvars:
         if x64flag:
             vsinstalldir = os.path.abspath(os.path.join(toolsdir, '..', '..'))
             vcinstalldir = os.path.join(vsinstalldir, 'VC')
@@ -71,96 +56,51 @@ def _get_msvc_env(vsver, x64flag):
             vcvars = os.path.join(vcbindir, 'amd64', 'vcvarsamd64.bat')
         else:
             vcvars = os.path.join(toolsdir, 'vsvars32.bat')
-            if not os.path.exists(vcvars):
-                # even msdn does not know which to run
-                # see https://msdn.microsoft.com/en-us/library/1700bbwd(v=vs.90).aspx
-                # which names both
-                vcvars = os.path.join(toolsdir, 'vcvars32.bat')
 
-        import subprocess
-        try:
-            popen = subprocess.Popen('"%s" & set' % (vcvars,),
-                                 stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
+    import subprocess
+    try:
+        popen = subprocess.Popen('"%s" & set' % (vcvars,),
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
 
-            stdout, stderr = popen.communicate()
-            if popen.wait() != 0 or stdout[:5].lower() == 'error':
-                log.msg('Running "%s" errored: \n\nstdout:\n%s\n\nstderr:\n%s' % (
-                    vcvars, stdout.split()[0], stderr))
-                return None
-            else:
-                log.msg('Running "%s" succeeded' %(vcvars,))
-        except Exception as e:
-            log.msg('Running "%s" failed: "%s"' % (vcvars, str(e)))
+        stdout, stderr = popen.communicate()
+        if popen.wait() != 0:
             return None
-
-        stdout = stdout.replace("\r\n", "\n")
-        vcdict = {}
-        for line in stdout.split("\n"):
-            if '=' not in line:
-                continue
-            key, value = line.split('=', 1)
-            vcdict[key] = value
+    except:
+        return None
     env = {}
-    for key, value in vcdict.items():
+
+    stdout = stdout.replace("\r\n", "\n")
+    for line in stdout.split("\n"):
+        if '=' not in line:
+            continue
+        key, value = line.split('=', 1)
         if key.upper() in ['PATH', 'INCLUDE', 'LIB']:
-            if sys.version_info[0] < 3:
-                env[key.upper()] = value.encode('utf-8')
-            else:
-                env[key.upper()] = value
-    if 'PATH' not in env:
-        log.msg('Did not find "PATH" in stdout\n%s' %(stdout))
-    if not _find_executable('mt.exe', env['PATH']):
-        # For some reason the sdk bin path is missing?
-        # put it together from some other env variables that happened to exist
-        # on the buildbot where this occurred
-        if 'WindowsSDKVersion' in vcdict and 'WindowsSdkDir' in vcdict:
-            binpath = vcdict['WindowsSdkDir'] + '\\bin\\' + vcdict['WindowsSDKVersion'] + 'x86'
-            env['PATH'] += ';' + binpath
-        if not _find_executable('mt.exe', env['PATH']):
-            log.msg('Could not find mt.exe on path=%s' % env['PATH'])
-            log.msg('Running vsver %s set this env' % vsver)
-            for key, value in vcdict.items():
-                log.msg('%s=%s' %(key, value))
-    log.msg("Updated environment with vsver %d, using x64 %s" % (vsver, x64flag,))
+            env[key.upper()] = value
+    ## log.msg("Updated environment with %s" % (vcvars,))
     return env
 
-def find_msvc_env(x64flag=False, ver0=None):
-    vcvers = [160, 150, 141, 140, 100, 90]
-    if ver0 in vcvers:
-        vcvers.insert(0, ver0)
-    errs = []
-    for vsver in vcvers:
+def find_msvc_env(x64flag=False):
+    # First, try to get the compiler which served to compile python
+    msc_pos = sys.version.find('MSC v.')
+    if msc_pos != -1:
+        msc_ver = int(sys.version[msc_pos+6:msc_pos+10])
+        # 1300 -> 70, 1310 -> 71, 1400 -> 80, 1500 -> 90
+        vsver = (msc_ver / 10) - 60
         env = _get_msvc_env(vsver, x64flag)
+
         if env is not None:
-            return env, vsver
+            return env
+
+    # Then, try any other version
+    for vsver in (100, 90, 80, 71, 70): # All the versions I know
+        env = _get_msvc_env(vsver, x64flag)
+
+        if env is not None:
+            return env
     log.error("Could not find a Microsoft Compiler")
     # Assume that the compiler is already part of the environment
 
-# copied from distutils.spawn
-def _find_executable(executable, path=None):
-    """Tries to find 'executable' in the directories listed in 'path'.
-
-    A string listing directories separated by 'os.pathsep'; defaults to
-    os.environ['PATH'].  Returns the complete filename or None if not found.
-    """
-    if path is None:
-        path = os.environ['PATH']
-    paths = path.split(os.pathsep)
-
-    for ext in '.exe', '':
-        newexe = executable + ext
-
-        if os.path.isfile(newexe):
-            return newexe
-        else:
-            for p in paths:
-                f = os.path.join(p, newexe)
-                if os.path.isfile(f):
-                    # the file exists, we have a shot at spawn working
-                    return f
-    return None
-
 class MsvcPlatform(Platform):
     name = "msvc"
     so_ext = 'dll'
@@ -170,41 +110,36 @@ class MsvcPlatform(Platform):
 
     cc = 'cl.exe'
     link = 'link.exe'
-    make = 'nmake'
-    if _find_executable('jom.exe'):
-        make = 'jom.exe'
 
-    cflags = ('/MD', '/O2', '/FS', '/Zi')
-    # allow >2GB address space, set stack to 3MB (1MB is too small)
-    link_flags = ('/nologo', '/debug','/LARGEADDRESSAWARE',
-                  '/STACK:3145728', '/MANIFEST:EMBED')
+    cflags = ('/MD', '/O2', '/Zi')
+    link_flags = ('/debug',)
     standalone_only = ()
     shared_only = ()
     environ = None
 
-    def __init__(self, cc=None, x64=False, ver0=None):
+    def __init__(self, cc=None, x64=False):
         self.x64 = x64
-        patch_os_env(self.externals)
-        self.c_environ = os.environ.copy()
         if cc is None:
-            msvc_compiler_environ, self.vsver = find_msvc_env(x64, ver0=ver0)
+            msvc_compiler_environ = find_msvc_env(x64)
             Platform.__init__(self, 'cl.exe')
             if msvc_compiler_environ:
+                self.c_environ = os.environ.copy()
                 self.c_environ.update(msvc_compiler_environ)
-                self.version = "MSVC %s" % str(self.vsver)
-                if self.vsver > 90:
-                    tag = '14x'
-                else:
-                    tag = '%d' % self.vsver
-                if x64:
-                    self.externals_branch = 'win64_%s' % tag
-                else:
-                    self.externals_branch = 'win32_%s' % tag
         else:
             self.cc = cc
 
+        # detect version of current compiler
+        returncode, stdout, stderr = _run_subprocess(self.cc, '',
+                                                     env=self.c_environ)
+        r = re.search(r'Microsoft.+C/C\+\+.+\s([0-9]+)\.([0-9]+).*', stderr)
+        if r is not None:
+            self.version = int(''.join(r.groups())) / 10 - 60
+        else:
+            # Probably not a msvc compiler...
+            self.version = 0
+
         # Try to find a masm assembler
-        returncode, stdout, stderr = _run_subprocess('ml.exe' if not x64 else 'ml64.exe', [],
+        returncode, stdout, stderr = _run_subprocess('ml.exe', '',
                                                      env=self.c_environ)
         r = re.search('Macro Assembler', stderr)
         if r is None and os.path.exists('c:/masm32/bin/ml.exe'):
@@ -247,7 +182,7 @@ class MsvcPlatform(Platform):
     def _linkfiles(self, link_files):
         return list(link_files)
 
-    def _args_for_shared(self, args, **kwds):
+    def _args_for_shared(self, args):
         return ['/dll'] + args
 
     def check___thread(self):
@@ -281,12 +216,24 @@ class MsvcPlatform(Platform):
         if not standalone:
             args = self._args_for_shared(args)
 
-        # Tell the linker to embed a manifest with the default
-        # UAC level asInvoker (Visual Studio 2008 +)
-        args += ["/MANIFEST:EMBED"]
+        if self.version >= 80:
+            # Tell the linker to generate a manifest file
+            temp_manifest = exe_name.dirpath().join(
+                exe_name.purebasename + '.manifest')
+            args += ["/MANIFEST", "/MANIFESTFILE:%s" % (temp_manifest,)]
 
         self._execute_c_compiler(self.link, args, exe_name)
 
+        if self.version >= 80:
+            # Now, embed the manifest into the program
+            if standalone:
+                mfid = 1
+            else:
+                mfid = 2
+            out_arg = '-outputresource:%s;%s' % (exe_name, mfid)
+            args = ['-nologo', '-manifest', str(temp_manifest), out_arg]
+            self._execute_c_compiler('mt.exe', args, exe_name)
+
         return exe_name
 
     def _handle_error(self, returncode, stdout, stderr, outname):
@@ -306,7 +253,7 @@ class MsvcPlatform(Platform):
 
     def gen_makefile(self, cfiles, eci, exe_name=None, path=None,
                      shared=False, headers_to_precompile=[],
-                     no_precompile_cfiles = [], profopt=False, config=None):
+                     no_precompile_cfiles = [], icon=None):
         cfiles = self._all_cfiles(cfiles, eci)
 
         if path is None:
@@ -382,23 +329,12 @@ class MsvcPlatform(Platform):
         if self.x64:
             definitions.append(('_WIN64', '1'))
 
-        if shared and self.make == 'jom.exe':
-            # Add `.SYNC` for jom.exe and get it to create
-            # main.c, wmain.c in a separate step before trying to compile them
-            rules = [('all',
-                      'main.c wmain.c .SYNC $(DEFAULT_TARGET) .SYNC $(WTARGET)',
-                      []),
-                    ]
-        else:
-            rules = [('all', '$(DEFAULT_TARGET) $(WTARGET)', []),]
-        rules += [
+        rules = [
+            ('all', '$(DEFAULT_TARGET) $(WTARGET)', []),
             ('.asm.obj', '', '$(MASM) /nologo /Fo$@ /c $< $(INCLUDEDIRS)'),
             ]
 
         if len(headers_to_precompile)>0:
-            if shared:
-                no_precompile_cfiles += [m.makefile_dir / 'main.c',
-                                         m.makefile_dir / 'wmain.c']
             stdafx_h = path.join('stdafx.h')
             txt  = '#ifndef PYPY_STDAFX_H\n'
             txt += '#define PYPY_STDAFX_H\n'
@@ -425,9 +361,9 @@ class MsvcPlatform(Platform):
             no_precompile = []
             for f in list(no_precompile_cfiles):
                 f = m.pathrel(py.path.local(f))
-                if f not in no_precompile and (f.endswith('.c') or f.endswith('.cpp')):
+                if f not in no_precompile and f.endswith('.c'):
                     no_precompile.append(f)
-                    target = f[:f.rfind('.')] + '.obj'
+                    target = f[:-1] + 'obj'
                     rules.append((target, f,
                         '$(CC) /nologo $(CFLAGS) $(CFLAGSEXTRA) '
                         '/Fo%s /c %s $(INCLUDEDIRS)' %(target, f)))
@@ -436,23 +372,13 @@ class MsvcPlatform(Platform):
             rules.append(('.c.obj', '',
                           '$(CC) /nologo $(CFLAGS) $(CFLAGSEXTRA) '
                           '/Fo$@ /c $< $(INCLUDEDIRS)'))
-            if shared:
-                rules.append(('main.obj', 'main.c',
-                              '$(CC) /nologo $(CFLAGS) $(CFLAGSEXTRA) '
-                              '/Fo$@ /c main.c $(INCLUDEDIRS)'))
-                rules.append(('wmain.obj', 'wmain.c',
-                              '$(CC) /nologo $(CFLAGS) $(CFLAGSEXTRA) '
-                              '/Fo$@ /c wmain.c $(INCLUDEDIRS)'))
 
-        icon = config.translation.icon if config else None
-        manifest = config.translation.manifest if config else None
+
         if icon:
             shutil.copyfile(icon, str(path.join('icon.ico')))
             rc_file = path.join('icon.rc')
             rc_file.write('IDI_ICON1 ICON DISCARDABLE "icon.ico"')
             rules.append(('icon.res', 'icon.rc', 'rc icon.rc'))
-        if manifest:
-            shutil.copyfile(manifest, str(path.join('pypy.manifest')))
 
 
         for args in definitions:
@@ -461,32 +387,32 @@ class MsvcPlatform(Platform):
         for rule in rules:
             m.rule(*rule)
 
-        if len(headers_to_precompile) > 0:
+        if len(headers_to_precompile)>0 and self.version >= 80:
             # at least from VS2013 onwards we need to include PCH
             # objects in the final link command
-            linkobjs = 'stdafx.obj '
+            linkobjs = 'stdafx.obj @<<\n$(OBJECTS)\n<<'
         else:
-            linkobjs = ''
-        if len(' '.join(rel_ofiles)) > 2048:
-            # command line is limited in length, use a response file
-            linkobjs += '@<<\n$(OBJECTS)\n<<'
-        else:
-            linkobjs += '$(OBJECTS)'
+            linkobjs = '@<<\n$(OBJECTS)\n<<'
+
         extra_deps = []
         if icon and not shared:
             extra_deps.append('icon.res')
             linkobjs = 'icon.res ' + linkobjs
-        if manifest and not shared:
-            linkflags.append('/MANIFESTINPUT:pypy.manifest')
-        m.rule('$(TARGET)', ['$(OBJECTS)'] + extra_deps,
-                [ '$(CC_LINK) $(LDFLAGS) $(LDFLAGSEXTRA)' + 
-                  ' $(LINKFILES) /out:$@ $(LIBDIRS) $(LIBS) ' + 
-                  linkobjs,
-                ])
+        if self.version < 80:
+            m.rule('$(TARGET)', ['$(OBJECTS)'] + extra_deps,
+                    [ '$(CC_LINK) /nologo $(LDFLAGS) $(LDFLAGSEXTRA) /out:$@' +\
+                      ' $(LIBDIRS) $(LIBS) ' + linkobjs,
+                   ])
+        else:
+            m.rule('$(TARGET)', ['$(OBJECTS)'] + extra_deps,
+                    [ '$(CC_LINK) /nologo $(LDFLAGS) $(LDFLAGSEXTRA)' + \
+                      ' $(LINKFILES) /out:$@ $(LIBDIRS) $(LIBS) /MANIFEST' + \
+                      ' /MANIFESTFILE:$*.manifest ' + linkobjs,
+                    'mt.exe -nologo -manifest $*.manifest -outputresource:$@;1',
+                    ])
         m.rule('debugmode_$(TARGET)', ['$(OBJECTS)'] + extra_deps,
-                [ '$(CC_LINK) /DEBUG $(LDFLAGS) $(LDFLAGSEXTRA)' + 
-                  ' $(LINKFILES) /out:$@ $(LIBDIRS) $(LIBS) ' + 
-                  linkobjs,
+                [ '$(CC_LINK) /nologo /DEBUG $(LDFLAGS) $(LDFLAGSEXTRA)' + \
+                  ' $(LINKFILES) /out:$@ $(LIBDIRS) $(LIBS) ' + linkobjs,
                 ])
 
         if shared:
@@ -499,41 +425,36 @@ class MsvcPlatform(Platform):
                    '{ return $(PYPY_MAIN_FUNCTION)(argc, argv); } > $@')
             deps = ['main.obj']
             m.rule('wmain.c', '',
-                   ['echo #define WIN32_LEAN_AND_MEAN > $@.tmp',
-                   'echo #include "stdlib.h" >> $@.tmp',
-                   'echo #include "windows.h" >> $@.tmp',
-                   'echo int $(PYPY_MAIN_FUNCTION)(int, char*[]); >> $@.tmp',
-                   'echo int WINAPI WinMain( >> $@.tmp',
-                   'echo     HINSTANCE hInstance,      /* handle to current instance */ >> $@.tmp',
-                   'echo     HINSTANCE hPrevInstance,  /* handle to previous instance */ >> $@.tmp',
-                   'echo     LPSTR lpCmdLine,          /* pointer to command line */ >> $@.tmp',
-                   'echo     int nCmdShow              /* show state of window */ >> $@.tmp',
-                   'echo ) >> $@.tmp',
-                   'echo    { return $(PYPY_MAIN_FUNCTION)(__argc, __argv); } >> $@.tmp',
-                   'move $@.tmp $@',
-                   ])
+                   ['echo #define WIN32_LEAN_AND_MEAN > $@',
+                   'echo #include "stdlib.h" >> $@',
+                   'echo #include "windows.h" >> $@',
+                   'echo int $(PYPY_MAIN_FUNCTION)(int, char*[]); >> $@',
+                   'echo int WINAPI WinMain( >> $@',
+                   'echo     HINSTANCE hInstance,      /* handle to current instance */ >> $@',
+                   'echo     HINSTANCE hPrevInstance,  /* handle to previous instance */ >> $@',
+                   'echo     LPSTR lpCmdLine,          /* pointer to command line */ >> $@',
+                   'echo     int nCmdShow              /* show state of window */ >> $@',
+                   'echo ) >> $@',
+                   'echo    { return $(PYPY_MAIN_FUNCTION)(__argc, __argv); } >> $@'])
             wdeps = ['wmain.obj']
             if icon:
                 deps.append('icon.res')
                 wdeps.append('icon.res')
-            manifest_args = '/MANIFEST:EMBED'
-            if manifest:
-                manifest_args += ' /MANIFESTINPUT:pypy.manifest'
             m.rule('$(DEFAULT_TARGET)', ['$(TARGET)'] + deps,
-                   ['$(CC_LINK) /DEBUG /LARGEADDRESSAWARE /STACK:3145728 ' +
-                    ' '.join(deps) + ' $(SHARED_IMPORT_LIB) ' +
-                    manifest_args + ' /out:$@ '
+                   ['$(CC_LINK) /nologo /debug %s ' % (' '.join(deps),) + \
+                    '$(SHARED_IMPORT_LIB) /out:$@ ' + \
+                    '/MANIFEST /MANIFESTFILE:$*.manifest',
+                    'mt.exe -nologo -manifest $*.manifest -outputresource:$@;1',
                     ])
             m.rule('$(WTARGET)', ['$(TARGET)'] + wdeps,
-                   ['$(CC_LINK) /DEBUG /LARGEADDRESSAWARE /STACK:3145728 ' +
-                    '/SUBSYSTEM:WINDOWS '  +
-                    ' '.join(wdeps) + ' $(SHARED_IMPORT_LIB) ' +
-                    manifest_args + ' /out:$@ '
+                   ['$(CC_LINK) /nologo /debug /SUBSYSTEM:WINDOWS %s ' % (' '.join(wdeps),) + \
+                    '$(SHARED_IMPORT_LIB) /out:$@ ' + \
+                    '/MANIFEST /MANIFESTFILE:$*.manifest',
+                    'mt.exe -nologo -manifest $*.manifest -outputresource:$@;1',
                     ])
             m.rule('debugmode_$(DEFAULT_TARGET)', ['debugmode_$(TARGET)']+deps,
-                   ['$(CC_LINK) /DEBUG /LARGEADDRESSAWARE /STACK:3145728 ' +
-                    ' '.join(deps) + ' debugmode_$(SHARED_IMPORT_LIB) ' +
-                    manifest_args + ' /out:$@ '
+                   ['$(CC_LINK) /nologo /DEBUG %s ' % (' '.join(deps),) + \
+                    'debugmode_$(SHARED_IMPORT_LIB) /out:$@',
                     ])
 
         return m
@@ -543,11 +464,11 @@ class MsvcPlatform(Platform):
             path = path_to_makefile.makefile_dir
         else:
             path = path_to_makefile
-        log.execute('%s %s in %s' % (self.make, " ".join(extra_opts), path))
+        log.execute('make %s in %s' % (" ".join(extra_opts), path))
         oldcwd = path.chdir()
         try:
             returncode, stdout, stderr = _run_subprocess(
-                self.make,
+                'nmake',
                 ['/nologo', '/f', str(path.join('Makefile'))] + extra_opts,
                 env = self.c_environ)
         finally:
@@ -555,22 +476,6 @@ class MsvcPlatform(Platform):
 
         self._handle_error(returncode, stdout, stderr, path.join('make'))
 
-# These are the external libraries, created and maintained by get_externals.py
-# The buildbot runs get_externals before building
-def patch_os_env(externals = Platform.externals):
-    #print 'adding %s to PATH, INCLUDE, LIB' % basepath
-    binpath = externals + r'\bin'
-    path = os.environ['PATH']
-    if binpath not in path:
-        path = binpath + ';' + path
-        # make sure externals is in current path for tests and translating
-        os.environ['PATH'] = path
-    if externals not in os.environ.get('INCLUDE', ''):
-        os.environ['INCLUDE'] = externals + r'\include;' + os.environ.get('INCLUDE', '')
-    if externals not in os.environ.get('LIB', ''):
-        os.environ['LIB'] = externals + r'\lib;' + os.environ.get('LIB', '')
-    return None
-
 class WinDefinition(posix.Definition):
     def write(self, f):
         def write_list(prefix, lst):
@@ -631,7 +536,7 @@ class MingwPlatform(posix.BasePosix):
             cc = 'gcc'
         Platform.__init__(self, cc)
 
-    def _args_for_shared(self, args, **kwds):
+    def _args_for_shared(self, args):
         return ['-shared'] + args
 
     def _include_dirs_for_libffi(self):
